@@ -1,10 +1,12 @@
 #!/bin/sh
-#		$Id: job26.sh,v 1.3 2007/03/30 02:57:36 pwessel Exp $
+#		$Id: job26.sh,v 1.6 2008/02/22 21:10:43 remko Exp $
 #		GMT EXAMPLE 26
 #
 # Purpose:	Demonstrate general vertical perspective projection
 # GMT progs:	pscoast
-# Unix progs:	none
+# Unix progs:	rm
+#
+ps=example_26.ps
 
 # first do an overhead of the east coast from 160 km altitude point straight down
 
@@ -19,10 +21,11 @@ Height=0.0
 
 PROJ=-JG${longitude}/${latitude}/${altitude}/${azimuth}/${tilt}/${twist}/${Width}/${Height}/4i
 
-pscoast -Rg $PROJ -X1i -B5g5/5g5 -Glightbrown -Slightblue -W0.25p -Dl -N1/1p,red -N2,0.5p -P -K -Y5i > example_26.ps
+pscoast -Rg $PROJ -X1i -B5g5/5g5 -Glightbrown -Slightblue -W0.25p -Dl -N1/1p,red -N2,0.5p -P -K \
+	-Y5i > $ps
 
-# now point from an altitude of 160 km with a specific tilt and azimuth and with a wider restricted view
-# and a boresight twist of 45 degrees
+# now point from an altitude of 160 km with a specific tilt and azimuth and with a wider restricted
+# view and a boresight twist of 45 degrees
 
 tilt=55
 azimuth=210
@@ -32,4 +35,6 @@ Height=30.0
 
 PROJ=-JG${longitude}/${latitude}/${altitude}/${azimuth}/${tilt}/${twist}/${Width}/${Height}/5i
 
-pscoast -R $PROJ -B5g5/5g5 -Glightbrown -Slightblue -W0.25p -Ia -Dl -Na -O -X1i -Y-4i -U/-1.75i/-0.75i/"Example 26 in Cookbook" >> example_26.ps
+pscoast -R $PROJ -B5g5/5g5 -Glightbrown -Slightblue -W0.25p -Ia/blue -Di -Na -O -X1i -Y-4i \
+	-U/-1.75i/-0.75i/"Example 26 in Cookbook" >> $ps
+rm -f .gmt*
