@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmean.c,v 1.50 2007/03/12 19:52:26 remko Exp $
+ *    $Id: blockmean.c,v 1.58 2008/04/25 04:00:40 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -72,9 +72,11 @@ int main (int argc, char **argv)
 	double	weight, weighted_z, *in, wesn[4], out[7], iw;
 
 	int	i, j, n_expected_fields, n_fields, n_req, n_out, w_col;
-	int	n_files = 0, fno, n_args, *np = NULL;
+	int	n_files = 0, fno, n_args;
 	
-	size_t n_blocks, ij, n_cells_filled, n_read, n_lost, n_pitched;
+	GMT_LONG	n_cells_filled, n_read, n_lost, n_pitched, *np = NULL;
+	
+	size_t ij, n_blocks;
 
 	char	modifier, line[BUFSIZ], format[BUFSIZ];
 	
@@ -122,8 +124,8 @@ int main (int argc, char **argv)
 					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
-					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
-					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					GMT_io.in_col_type[GMT_X] = GMT_io.out_col_type[GMT_X] = GMT_IS_LON;
+					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
@@ -231,45 +233,41 @@ int main (int argc, char **argv)
 	
 	GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
 
-	n_blocks = ((size_t)h.nx) * ((size_t)h.ny);
+	n_blocks = ((GMT_LONG)h.nx) * ((GMT_LONG)h.ny);
 	
 	if ((zw = (struct BLK_PAIR *) GMT_memory (VNULL, n_blocks, sizeof (struct BLK_PAIR), GMT_program)) == NULL) {
-		fprintf (stderr, "%s: ERROR: Unable to allocate memory for ", GMT_program);
-		PRINT_SIZE_T (stderr, n_blocks);
-		fprintf (stderr, " blocks [%d bytes for zw].\n", (int)sizeof (struct BLK_PAIR));
+		fprintf (stderr, "%s: ERROR: Unable to allocate memory for %ld blocks [%ld bytes for zw].\n",
+			GMT_program, (GMT_LONG)n_blocks, (GMT_LONG)sizeof (struct BLK_PAIR));
 		exit (EXIT_FAILURE);
 	}
 	if (!Ctrl->C.active && (xy = (struct BLK_PAIR *) GMT_memory (VNULL, n_blocks, sizeof (struct BLK_PAIR), GMT_program)) == NULL) {
-		fprintf (stderr, "%s: ERROR: Unable to allocate memory for ", GMT_program);
-		PRINT_SIZE_T (stderr, n_blocks);
-		fprintf (stderr, " blocks [%d bytes for xy].\n", (int)sizeof (struct BLK_PAIR));
+		fprintf (stderr, "%s: ERROR: Unable to allocate memory for %ld blocks [%ld bytes for xy].\n",
+			GMT_program, (GMT_LONG)n_blocks, (GMT_LONG)sizeof (struct BLK_PAIR));
 		exit (EXIT_FAILURE);
 	}
 	if (Ctrl->E.active && (slh = (struct BLK_SLH *) GMT_memory (VNULL, n_blocks, sizeof (struct BLK_SLH), GMT_program)) == NULL) {
-		fprintf (stderr, "%s: ERROR: Unable to allocate memory for ", GMT_program);
-		PRINT_SIZE_T (stderr, n_blocks);
-		fprintf (stderr, " blocks [%d bytes for slh].\n", (int)sizeof (struct BLK_SLH));
+		fprintf (stderr, "%s: ERROR: Unable to allocate memory for %ld blocks [%ld bytes for slh].\n",
+			GMT_program, (GMT_LONG)n_blocks, (GMT_LONG)sizeof (struct BLK_SLH));
 		exit (EXIT_FAILURE);
 	}
-	if ((Ctrl->W.weighted[GMT_IN] && Ctrl->E.active) && (np = (int *) GMT_memory (VNULL, n_blocks, sizeof (int), GMT_program)) == NULL) {
-		fprintf (stderr, "%s: ERROR: Unable to allocate memory for ", GMT_program);
-		PRINT_SIZE_T (stderr, n_blocks);
-		fprintf (stderr, " blocks [%d bytes for counter].\n", (int)sizeof (int));
+	if ((Ctrl->W.weighted[GMT_IN] && Ctrl->E.active) && (np = (GMT_LONG *) GMT_memory (VNULL, n_blocks, sizeof (GMT_LONG), GMT_program)) == NULL) {
+		fprintf (stderr, "%s: ERROR: Unable to allocate memory for %ld blocks [%ld bytes for counter].\n",
+			GMT_program, (GMT_LONG)n_blocks, (GMT_LONG)sizeof (GMT_LONG));
 		exit (EXIT_FAILURE);
 	}
 	if (gmtdefs.verbose) {
 		double mem;
 		char unit = 'M';
-		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, h.x_min, h.x_max, h.y_min, h.y_max, h.nx, h.ny);
 		mem = (double)sizeof (struct BLK_PAIR);
 		if (!Ctrl->C.active) mem += (double)sizeof (struct BLK_PAIR);
-		if (Ctrl->E.active) mem += (double)sizeof (struct BLK_SLH);
-		if (Ctrl->W.weighted[GMT_IN] && Ctrl->E.active) mem += (double)sizeof (int);
+		if (Ctrl->E.active)  mem += (double)sizeof (struct BLK_SLH);
+		if (Ctrl->W.weighted[GMT_IN] && Ctrl->E.active) mem += (double)sizeof (GMT_LONG);
 		mem *= (double)n_blocks;
 		mem /= (1024.0 * 1024.0);	/* Report Mbytes */
-		if (mem > 1000.0) {	/* Report Gbytes */
-			mem /= 1024;
+		if (mem > 1000.0) {		/* Report Gbytes */
+			mem /= 1024.0;
 			unit = 'G';
 		}
 		fprintf (stderr, "%s: Using a total of %.3g %cb for all arrays.\n", GMT_program, mem, unit);
@@ -328,24 +326,24 @@ int main (int argc, char **argv)
 
 			n_read++;
 
-			if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
-			if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+			if (GMT_y_is_outside (in[GMT_Y],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
 			/* Get i and j indices of this block */
 			
-			i = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			i = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
 			if ( i < 0 || i >= h.nx ) continue;
-			j = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			j = GMT_y_to_j (in[GMT_Y], h.y_min, h.y_inc, h.xy_off, h.ny);
 			if ( j < 0 || j >= h.ny ) continue;
 			
 			/* OK, this point is inside and will be used */
 			
 			weight = (Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0;
 			weighted_z = in[2] * weight;
-			ij = ((size_t)j) * ((size_t)h.nx) + ((size_t)i);		/* 64-bit safe 1-D index */
+			ij = GMT_IJ (j, i, h.nx);		/* 64-bit safe 1-D index */
 			if (use_xy) {
-				xy[ij].a[BLK_X] += (in[0]*weight);
-				xy[ij].a[BLK_Y] += (in[1]*weight);
+				xy[ij].a[BLK_X] += (in[GMT_X]*weight);
+				xy[ij].a[BLK_Y] += (in[GMT_Y]*weight);
 			}
 			if (Ctrl->E.active) {	/* Add up sum(w*z^2) and n for weighted stdev and keep track of min,max */
 				slh[ij].a[BLK_S] += (weighted_z * in[2]);
@@ -379,14 +377,14 @@ int main (int argc, char **argv)
 		if (Ctrl->W.weighted[GMT_OUT]) out[w_col] = zw[ij].a[BLK_W];
 		iw = 1.0 / zw[ij].a[BLK_W];
 		if (use_xy) {
-			out[0] = xy[ij].a[BLK_X] * iw;
-			out[1] = xy[ij].a[BLK_Y] * iw;
+			out[GMT_X] = xy[ij].a[BLK_X] * iw;
+			out[GMT_Y] = xy[ij].a[BLK_Y] * iw;
 		}
 		else {	/* Use block center */
-			i = ij % h.nx;
-			j = ij / h.nx;
-			out[0] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
-			out[1] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+			i = ij % ((GMT_LONG)h.nx);
+			j = ij / ((GMT_LONG)h.nx);
+			out[GMT_X] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+			out[GMT_Y] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 		}
 		if (Ctrl->S.mode) {
 			out[2] = (Ctrl->S.mode == 2) ? zw[ij].a[BLK_W] : zw[ij].a[BLK_Z];
@@ -416,18 +414,13 @@ int main (int argc, char **argv)
 	}
 	
 	n_lost = n_read - n_pitched;
-	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: N read: ", GMT_program);	PRINT_SIZE_T (stderr, n_read);
-		fprintf(stderr," N used: ");	PRINT_SIZE_T (stderr, n_pitched);
-		fprintf(stderr," N outside_area: ");	PRINT_SIZE_T (stderr, n_lost);
-		fprintf(stderr," N cells filled: ");	PRINT_SIZE_T (stderr, n_cells_filled);
-		fprintf(stderr,"\n");
-	}
-
-        GMT_end (argc, argv);
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld N cells filled: %ld\n",
+		GMT_program, n_read, n_pitched, n_lost, n_cells_filled);
 
 	Free_Blockmean_Ctrl (Ctrl);	/* Deallocate control structure */
 
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 #include "block_subs.c"
