@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmedian.c,v 1.44 2007/04/02 15:25:57 remko Exp $
+ *    $Id: blockmedian.c,v 1.52 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,8 +56,10 @@ int main (int argc, char **argv)
 	int	i, ix, iy, fno, n_files = 0, n_args, n_req, w_col;
 	int	n_expected_fields, n_fields, n_out, go_quickly = 0;
 
-	size_t	nz, n_read, n_pitched, n_cells_filled, n_alloc, n_lost, index;
-	size_t	first_in_cell, first_in_new_cell, nz_alloc;
+	GMT_LONG	nz, n_read, n_pitched, n_cells_filled, n_lost, index;
+	GMT_LONG	first_in_cell, first_in_new_cell;
+	
+	size_t	n_alloc, nz_alloc;
 	
 	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
 
@@ -66,7 +68,7 @@ int main (int argc, char **argv)
 	struct BLK_DATA *data;
 	struct BLOCKMEDIAN_CTRL *Ctrl;
 	
-	void median_output (struct GRD_HEADER *h, size_t first_in_cell, size_t first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data);
+	void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data);
 
 	argc = GMT_begin (argc, argv);
 
@@ -106,8 +108,8 @@ int main (int argc, char **argv)
 					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
-					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
-					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					GMT_io.in_col_type[GMT_X] = GMT_io.out_col_type[GMT_X] = GMT_IS_LON;
+					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
@@ -221,10 +223,10 @@ int main (int argc, char **argv)
 
 	go_quickly = (Ctrl->Q.active) ? 1 : 0;	
 	if (Ctrl->C.active) go_quickly = 2;	/* Flag used in output calculation */
-	h.xy_off = 0.5 * h.node_offset;	/* Use to calculate mean location of block */
+	h.xy_off = 0.5 * h.node_offset;		/* Use to calculate mean location of block */
 
 	if (gmtdefs.verbose) {
-		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, h.x_min, h.x_max, h.y_min, h.y_max, h.nx, h.ny);
 	}
 
@@ -283,27 +285,27 @@ int main (int argc, char **argv)
 
 			n_read++;
 
-			if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
-			if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+			if (GMT_y_is_outside (in[GMT_Y],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
-			ix = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			ix = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
 			if ( ix < 0 || ix >= h.nx ) continue;
-			iy = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			iy = GMT_y_to_j (in[GMT_Y], h.y_min, h.y_inc, h.xy_off, h.ny);
 			if ( iy < 0 || iy >= h.ny ) continue;
 
-			index = ((size_t)iy) * ((size_t)h.nx) + ((size_t)ix);		/* 64-bit safe 1-D index */
+			index = GMT_IJ (iy, ix, h.nx);		/* 64-bit safe 1-D index */
 
 			data[n_pitched].i = index;
 			data[n_pitched].a[BLK_W] = ((Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0);
 			if (!Ctrl->C.active) {
-				data[n_pitched].a[BLK_X] = in[0];
-				data[n_pitched].a[BLK_Y] = in[1];
+				data[n_pitched].a[BLK_X] = in[GMT_X];
+				data[n_pitched].a[BLK_Y] = in[GMT_Y];
 			}
 			data[n_pitched].a[BLK_Z] = in[2];
 
 			n_pitched++;
-			if (n_pitched == n_alloc) {
-				n_alloc += GMT_CHUNK;
+			if (n_pitched == (GMT_LONG)n_alloc) {
+				n_alloc <<= 1;
 				data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof (struct BLK_DATA), GMT_program);
 			}
 		}
@@ -311,15 +313,11 @@ int main (int argc, char **argv)
 
 	}
 
-	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_pitched, sizeof (struct BLK_DATA), GMT_program);
+	n_alloc = (size_t)n_pitched;
+	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof (struct BLK_DATA), GMT_program);
 	
 	n_lost = n_read - n_pitched;
-	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: N read: ", GMT_program);	PRINT_SIZE_T (stderr, n_read);
-		fprintf(stderr," N used: ");	PRINT_SIZE_T (stderr, n_pitched);
-		fprintf(stderr," N outside_area: ");	PRINT_SIZE_T (stderr, n_lost);
-		fprintf(stderr,"\n");
-	}
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, n_pitched, n_lost);
 
 	/* Ready to go. */
 
@@ -331,7 +329,7 @@ int main (int argc, char **argv)
 
 	/* Sort on index and Z value */
 
-	qsort((void *)data, n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
+	qsort((void *)data, (size_t)n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
 
 	/* Find n_in_cell and write appropriate output  */
 
@@ -347,21 +345,21 @@ int main (int argc, char **argv)
 			if (Ctrl->E.active) {	/* Must get a temporary copy of the sorted z array */
 				z_tmp[nz] = data[first_in_new_cell].a[BLK_Z];
 				nz++;
-				if (nz == nz_alloc) {
-					nz_alloc += GMT_CHUNK;
+				if (nz == (GMT_LONG)nz_alloc) {
+					nz_alloc <<= 1;
 					z_tmp = (double *) GMT_memory ((void *)z_tmp, nz_alloc, sizeof (double), GMT_program);
 				}
 			}
 			first_in_new_cell++;
 		}
-		median_output(&h, first_in_cell, first_in_new_cell, weight, &out[0], &out[1], &out[2], go_quickly, Ctrl->T.quartile, data);
+		median_output(&h, first_in_cell, first_in_new_cell, weight, &out[GMT_X], &out[GMT_Y], &out[2], go_quickly, Ctrl->T.quartile, data);
 		if (Ctrl->E.active) {
 			out[4] = z_tmp[0];	/* Low value */
 			out[5] = z_tmp[nz-1];	/* High value */
 			/* Turn z_tmp into absolute deviations from the median (out[2]) */
 			if (nz > 1) {
 				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[2]);
-				qsort ((void *)z_tmp, nz, sizeof (double), GMT_comp_double_asc);
+				qsort ((void *)z_tmp, (size_t)nz, sizeof (double), GMT_comp_double_asc);
 				out[3] = (nz%2) ? z_tmp[nz/2] : 0.5 * (z_tmp[(nz-1)/2] + z_tmp[nz/2]);
 				out[3] *= 1.4826;	/* This will be L1 MAD-based scale */
 			}
@@ -376,26 +374,22 @@ int main (int argc, char **argv)
 		first_in_cell = first_in_new_cell;
 	}
 
-	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: N_cells_filled: ", GMT_program);
-		PRINT_SIZE_T (stderr, n_cells_filled);
-		fprintf(stderr,"\n");
-	}
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %ld\n", GMT_program, n_cells_filled);
 
 	GMT_free ((void *)data);
 	if (Ctrl->E.active) GMT_free ((void *)z_tmp);
 
-        GMT_end (argc, argv);
-
 	Free_Blockmedian_Ctrl (Ctrl);	/* Deallocate control structure */
 
+    GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
-void median_output (struct GRD_HEADER *h, size_t first_in_cell, size_t first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data)
+void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data)
 {
 	double	weight_half, weight_count;
-	size_t	index, n_in_cell, index1;
+	GMT_LONG index, n_in_cell, index1;
 
 	weight_half  = quartile * weight_sum;	/* Normally, quartile will be 0.5 hence the name of the variable */
 	n_in_cell = first_in_new_cell - first_in_cell;
@@ -428,9 +422,9 @@ void median_output (struct GRD_HEADER *h, size_t first_in_cell, size_t first_in_
 	if (go_quickly == 1) return;
 
 	if (go_quickly == 2) {	/* Get center of block */
-		int i, j;
-		j = data[index].i / h->nx;
-		i = data[index].i % h->nx;
+		GMT_LONG i, j;
+		j = data[index].i / ((GMT_LONG)h->nx);
+		i = data[index].i % ((GMT_LONG)h->nx);
 		*xx = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 		*yy = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 		return;
@@ -438,7 +432,7 @@ void median_output (struct GRD_HEADER *h, size_t first_in_cell, size_t first_in_
 	
 	/* We get here when we need median x,y locations */
 	
-	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
+	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
 	index = first_in_cell;
 	weight_count = data[first_in_cell].a[BLK_W];
 	while (weight_count < weight_half) {
@@ -450,7 +444,7 @@ void median_output (struct GRD_HEADER *h, size_t first_in_cell, size_t first_in_
 	else
 		*xx = data[index].a[BLK_X];
 
-	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
+	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
 	index = first_in_cell;
 	weight_count = data[first_in_cell].a[BLK_W];
 	while (weight_count < weight_half) {
