@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmode.c,v 1.45 2007/04/02 15:25:57 remko Exp $
+ *    $Id: blockmode.c,v 1.53 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -53,8 +53,10 @@ int main (int argc, char **argv)
 	int	i, j, ix, iy, fno, n_files = 0, n_args, n_req;
 	int	n_expected_fields, n_fields, n_out, w_col;
 	
-	size_t index, first_in_cell, first_in_new_cell, n_lost, n_read;
-	size_t n_pitched, n_cells_filled, n_alloc, n_in_cell, nz, nz_alloc;
+	GMT_LONG index, first_in_cell, first_in_new_cell, n_lost, n_read;
+	GMT_LONG n_pitched, n_cells_filled, n_in_cell, nz;
+
+	size_t n_alloc, nz_alloc;
 
 	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
 
@@ -63,7 +65,7 @@ int main (int argc, char **argv)
 	struct BLK_DATA *data;
 	struct BLOCKMODE_CTRL *Ctrl;
 	
-	double weighted_mode (struct BLK_DATA *d, double wsum, size_t n, int k);
+	double weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, int k);
 	
 	argc = GMT_begin (argc, argv);
 
@@ -103,8 +105,8 @@ int main (int argc, char **argv)
 					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
-					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
-					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					GMT_io.in_col_type[GMT_X] = GMT_io.out_col_type[GMT_X] = GMT_IS_LON;
+					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
@@ -209,7 +211,7 @@ int main (int argc, char **argv)
 	h.xy_off = 0.5 * h.node_offset;	/* Use to calculate mean location of block */
 
 	if (gmtdefs.verbose) {
-		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, h.x_min, h.x_max, h.y_min, h.y_max, h.nx, h.ny);
 	}
 
@@ -268,27 +270,27 @@ int main (int argc, char **argv)
 
 			n_read++;
 
-			if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
-			if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+			if (GMT_y_is_outside (in[GMT_Y],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
-			ix = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			ix = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
 			if ( ix < 0 || ix >= h.nx ) continue;
-			iy = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			iy = GMT_y_to_j (in[GMT_Y], h.y_min, h.y_inc, h.xy_off, h.ny);
 			if ( iy < 0 || iy >= h.ny ) continue;
 
-			index = ((size_t)iy) * ((size_t)h.nx) + ((size_t)ix);		/* 64-bit safe 1-D index */
+			index = GMT_IJ (iy, ix, h.nx);		/* 64-bit safe 1-D index */
 
 			data[n_pitched].i = index;
 			if (mode_xy) {
-				data[n_pitched].a[BLK_X] = in[0];
-				data[n_pitched].a[BLK_Y] = in[1];
+				data[n_pitched].a[BLK_X] = in[GMT_X];
+				data[n_pitched].a[BLK_Y] = in[GMT_Y];
 			}
 			data[n_pitched].a[BLK_Z] = in[2];
 			data[n_pitched].a[BLK_W] = (Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0;
 
 			n_pitched++;
-			if (n_pitched == n_alloc) {
-				n_alloc += GMT_CHUNK;
+			if (n_pitched == (GMT_LONG)n_alloc) {
+				n_alloc <<= 1;
 				data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof(struct BLK_DATA), GMT_program);
 			}
 		}
@@ -296,14 +298,11 @@ int main (int argc, char **argv)
 
 	}
 
-	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_pitched, sizeof(struct BLK_DATA), GMT_program);
+	n_alloc = (size_t)n_pitched;
+	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof(struct BLK_DATA), GMT_program);
 	n_lost = n_read - n_pitched;
-	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: N read: ", GMT_program);	PRINT_SIZE_T (stderr, n_read);
-		fprintf(stderr," N used: ");	PRINT_SIZE_T (stderr, n_pitched);
-		fprintf(stderr," N outside_area: ");	PRINT_SIZE_T (stderr, n_lost);
-		fprintf(stderr,"\n");
-	}
+	
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, n_pitched, n_lost);
 
 	/* Ready to go. */
 
@@ -315,7 +314,7 @@ int main (int argc, char **argv)
 
 	/* Sort on index and Z value */
 
-	qsort((void *)data, n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
+	qsort((void *)data, (size_t)n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
 
 	/* Find n_in_cell and write appropriate output  */
 
@@ -326,27 +325,27 @@ int main (int argc, char **argv)
 		if (Ctrl->E.active) z_tmp[0] = data[first_in_cell].a[BLK_Z];
 		nz = 1;
 		if (Ctrl->C.active) {
-			j = data[first_in_cell].i / h.nx;
-			i = data[first_in_cell].i % h.nx;
-			out[0] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
-			out[1] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+			j = data[first_in_cell].i / ((GMT_LONG)h.nx);
+			i = data[first_in_cell].i % ((GMT_LONG)h.nx);
+			out[GMT_X] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+			out[GMT_Y] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 		}
 		else {
-			out[0] = data[first_in_cell].a[BLK_X];
-			out[1] = data[first_in_cell].a[BLK_Y];
+			out[GMT_X] = data[first_in_cell].a[BLK_X];
+			out[GMT_Y] = data[first_in_cell].a[BLK_Y];
 		}
 		first_in_new_cell = first_in_cell + 1;
 		while ( (first_in_new_cell < n_pitched) && (data[first_in_new_cell].i == data[first_in_cell].i) ) {
 			weight += data[first_in_new_cell].a[BLK_W];	/* Summing up weights */
 			if (mode_xy) {
-				out[0] += data[first_in_new_cell].a[BLK_X];
-				out[1] += data[first_in_new_cell].a[BLK_Y];
+				out[GMT_X] += data[first_in_new_cell].a[BLK_X];
+				out[GMT_Y] += data[first_in_new_cell].a[BLK_Y];
 			}
 			if (Ctrl->E.active) {	/* Must get a temporary copy of the sorted z array */
 				z_tmp[nz] = data[first_in_new_cell].a[BLK_Z];
 				nz++;
-				if (nz == nz_alloc) {
-					nz_alloc += GMT_CHUNK;
+				if (nz == (GMT_LONG)nz_alloc) {
+					nz_alloc <<= 1;
 					z_tmp = (double *) GMT_memory ((void *)z_tmp, nz_alloc, sizeof (double), GMT_program);
 				}
 			}
@@ -357,44 +356,44 @@ int main (int argc, char **argv)
 			out[2] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 2);
 			if (Ctrl->Q.active) {
 				i_n_in_cell = 1.0 / n_in_cell;
-				out[0] *= i_n_in_cell;
-				out[1] *= i_n_in_cell;
+				out[GMT_X] *= i_n_in_cell;
+				out[GMT_Y] *= i_n_in_cell;
 			}
 			else if (mode_xy) {
-				qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
-				out[0] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 0);
+				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
+				out[GMT_X] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 0);
 
-				qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
-				out[1] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 1);
+				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
+				out[GMT_Y] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 1);
 			}
 		}
 		else if (n_in_cell == 2) {
 			if (data[first_in_cell].a[BLK_W] > data[first_in_cell+1].a[BLK_W]) {
 				out[2] = data[first_in_cell].a[BLK_Z];
 				if (Ctrl->Q.active) {
-					out[0] *= 0.5;
-					out[1] *= 0.5;
+					out[GMT_X] *= 0.5;
+					out[GMT_Y] *= 0.5;
 				}
 				else if (mode_xy) {
-					out[0] = data[first_in_cell].a[BLK_X];
-					out[1] = data[first_in_cell].a[BLK_Y];
+					out[GMT_X] = data[first_in_cell].a[BLK_X];
+					out[GMT_Y] = data[first_in_cell].a[BLK_Y];
 				}
 			}
 			else if (data[first_in_cell].a[BLK_W] < data[first_in_cell+1].a[BLK_W]) {
 				out[2] = data[first_in_cell+1].a[BLK_Z];
 				if (Ctrl->Q.active) {
-					out[0] *= 0.5;
-					out[1] *= 0.5;
+					out[GMT_X] *= 0.5;
+					out[GMT_Y] *= 0.5;
 				}
 				else if (mode_xy) {
-					out[0] = data[first_in_cell+1].a[BLK_X];
-					out[1] = data[first_in_cell+1].a[BLK_Y];
+					out[GMT_X] = data[first_in_cell+1].a[BLK_X];
+					out[GMT_Y] = data[first_in_cell+1].a[BLK_Y];
 				}
 			}
 			else {
 				if (mode_xy) {	/* Need average location */
-					out[0] *= 0.5;
-					out[1] *= 0.5;
+					out[GMT_X] *= 0.5;
+					out[GMT_Y] *= 0.5;
 				}
 				out[2] = 0.5 * (data[first_in_cell].a[BLK_Z] + data[first_in_cell+1].a[BLK_Z]);
 			}
@@ -408,7 +407,7 @@ int main (int argc, char **argv)
 			/* Turn z_tmp into absolute deviations from the mode (out[2]) */
 			if (nz > 1) {
 				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[2]);
-				qsort ((void *)z_tmp, nz, sizeof (double), GMT_comp_double_asc);
+				qsort ((void *)z_tmp, (size_t)nz, sizeof (double), GMT_comp_double_asc);
 				out[3] = (nz%2) ? z_tmp[nz/2] : 0.5 * (z_tmp[(nz-1)/2] + z_tmp[nz/2]);
 				out[3] *= 1.4826;	/* This will be LMS MAD-based scale */
 			}
@@ -423,23 +422,19 @@ int main (int argc, char **argv)
 		first_in_cell = first_in_new_cell;
 	}
 
-	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: N_cells_filled: ", GMT_program);
-		PRINT_SIZE_T (stderr, n_cells_filled);
-		fprintf(stderr,"\n");
-	}
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %ld\n", GMT_program, n_cells_filled);
 
 	GMT_free ((void *)data);
 	if (Ctrl->E.active) GMT_free ((void *)z_tmp);
 
-        GMT_end (argc, argv);
-
 	Free_Blockmode_Ctrl (Ctrl);	/* Deallocate control structure */
 
+    GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
-double	weighted_mode (struct BLK_DATA *d, double wsum, size_t n, int k)
+double	weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, int k)
 {
 	/* Estimate mode by finding a maximum in the estimated
 	   pdf of weighted data.  Estimate the pdf as the finite 
@@ -461,7 +456,7 @@ double	weighted_mode (struct BLK_DATA *d, double wsum, size_t n, int k)
 	   I assumed n > 2 when I wrote this.  */
 	   
 	double	top, topj, topi, bottomj, bottomi, pj, pi;
-	size_t	i, j, nh;
+	GMT_LONG	i, j, nh;
 
 	i = 0;
 	j = n - 1;
