@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: filter1d.c,v 1.40 2007/03/31 18:06:32 pwessel Exp $
+ *    $Id: filter1d.c,v 1.50 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -103,13 +103,13 @@ struct FILTER1D_INFO {	/* Control structure for all aspects of the filter setup
 	int	*n_this_col;		/* Pointer to array of counters [one per column]  */
 	int	*n_left;		/* Pointer to array of counters [one per column]  */
 	int	*n_right;		/* Pointer to array of counters [one per column]  */
-	int	n_rows;			/* Number of rows of input  */
+	GMT_LONG	n_rows;			/* Number of rows of input  */
 	int	n_cols;			/* Number of columns of input  */
 	int	t_col;			/* Column of time abscissae (independent variable)  */
 	int	n_f_wts;		/* Number of filter weights  */
 	int	half_n_f_wts;		/* Half the number of filter weights  */
-	int	n_row_alloc;		/* Number of rows of data to allocate  */
-	int	n_work_alloc;		/* Number of rows of workspace to allocate  */
+	GMT_LONG	n_row_alloc;		/* Number of rows of data to allocate  */
+	GMT_LONG	n_work_alloc;		/* Number of rows of workspace to allocate  */
 	int	filter_type;		/* Flag indicating desired filter type  */
 	int	kind;				/* -1 skip +ve, +1 skip -ve, else use all  [for the l|L|u|U filter] */
 	int	way;			/* -1 find minimum, +1 find maximum  [for the l|L|u|U filter] */
@@ -208,6 +208,7 @@ int main (int argc, char **argv)
 								break;
 							case 'F':
 							case 'f':
+								Ctrl->F.width = DBL_MAX;	/* To avoid range test errors before reading coefficients */
 								if (argv[i][3] && !GMT_access (&argv[i][3], R_OK))
 									Ctrl->F.file = strdup (&argv[i][3]);
 								else
@@ -482,7 +483,7 @@ int main (int argc, char **argv)
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, F.n_rows);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, F.n_rows);
 			exit (EXIT_FAILURE);
 		}
 
@@ -503,7 +504,7 @@ int main (int argc, char **argv)
 			new_time = F.data[F.t_col][F.n_rows];
 			F.n_rows++;
 			if (new_time < last_time) {
-				fprintf (stderr, "%s: Error! Time decreases at line # %d\n", GMT_program, F.n_rows);
+				fprintf (stderr, "%s: Error! Time decreases at line # %ld\n", GMT_program, F.n_rows);
 				fprintf (stderr, "\tUse UNIX utility sort and then try again.\n");
 				return(-1);
 			}
@@ -511,7 +512,7 @@ int main (int argc, char **argv)
 		}
 
 		if (F.n_rows == F.n_row_alloc) {	/* Need more memory */
-			F.n_row_alloc += GMT_CHUNK;
+			F.n_row_alloc <<= 1;
 			allocate_more_data_space (&F);
 		}
 	}
@@ -519,7 +520,7 @@ int main (int argc, char **argv)
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
 
 	for (i = 0; i < F.n_cols; i++) F.data[i] = (double *) GMT_memory ((void *)F.data[i], (size_t)F.n_rows, sizeof (double), GMT_program);
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d data lines\n", GMT_program, F.n_rows);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld data lines\n", GMT_program, F.n_rows);
 
 	/* Initialize scale parameters and last_loc based on min and max of data  */
 
@@ -576,6 +577,7 @@ int set_up_filter (struct FILTER1D_INFO *F)
 		F->n_f_wts = 0;
 		w_sum = 0.0;
 		while (GMT_fgets (buffer, BUFSIZ, F->fp_wt)) {
+			if (GMT_is_a_blank_line (buffer)) continue;	/* Skip blank lines or # comments */
 			if (sscanf (buffer, "%lf", &F->f_wt[F->n_f_wts]) != 1) {
 				fprintf (stderr, "%s: Error decoding filter weights from file\n", GMT_program);
 				exit (EXIT_FAILURE);
@@ -583,7 +585,7 @@ int set_up_filter (struct FILTER1D_INFO *F)
 			w_sum += F->f_wt[F->n_f_wts];
 			F->n_f_wts++;
 			if (F->n_f_wts == F->n_work_alloc) {	/* Need more memory */
-				F->n_work_alloc += GMT_CHUNK;
+				F->n_work_alloc <<= 1;
 				allocate_more_work_space (F);
 			}
 		}
@@ -653,18 +655,19 @@ int set_up_filter (struct FILTER1D_INFO *F)
 
 int do_the_filter (struct FILTER1D_INFO *F)
 {
-	int	i_row, i_col, left, right, n_l, n_r, iq, i_f_wt;
-	int	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
+	GMT_LONG	i_row, left, right, n_l, n_r, iq, i_f_wt;
+	GMT_LONG	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
+	int i_col;
 	double	time, delta_time, *outval, wt, val, med, scl, small;
 	BOOLEAN	*good_one;		/* Pointer to array of logicals [one per column]  */
 	double	*wt_sum;		/* Pointer for array of weight sums [each column]  */
 	double	*data_sum;		/* Pointer for array of data * weight sums [columns]  */
 
-	BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right);	/* Tests for lack of data condition  (optional)  */
-	void	get_robust_estimates (struct FILTER1D_INFO *F, int j, int n, int both);	/* Does just what it says  */
+	BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, GMT_LONG left, GMT_LONG right);	/* Tests for lack of data condition  (optional)  */
+	void	get_robust_estimates (struct FILTER1D_INFO *F, int j, GMT_LONG n, int both);	/* Does just what it says  */
 
 	outval = (double *)GMT_memory (VNULL, (size_t)F->n_cols, sizeof (double), GMT_program);
-	good_one = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	good_one = (BOOLEAN *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(BOOLEAN), GMT_program);
 	wt_sum = (double *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double), GMT_program);
 	data_sum = (double *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double), GMT_program);
 
@@ -737,7 +740,11 @@ int do_the_filter (struct FILTER1D_INFO *F)
 					if (!(good_one[i_col])) continue;
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
+#ifdef __LP64__
+					if ((((double)labs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
+#else
 					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
+#endif
 				}
 			}
 			if ( (F->filter_type > FILTER1D_CONVOLVE) && F->check_q) {
@@ -779,7 +786,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 
 			for (i_row = left; i_row < right; i_row++) {
 				delta_time = time - F->data[F->t_col][i_row];
-				i_f_wt = F->half_n_f_wts + (int)floor(0.5 + delta_time/F->dt);
+				i_f_wt = F->half_n_f_wts + (GMT_LONG)floor(0.5 + delta_time/F->dt);
 				if ( (i_f_wt < 0) || (i_f_wt >= F->n_f_wts) ) continue;
 
 				for(i_col = 0; i_col < F->n_cols; i_col++) {
@@ -812,7 +819,11 @@ int do_the_filter (struct FILTER1D_INFO *F)
 				if (F->check_asym && !(F->robust) ) {
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
+#ifdef __LP64__
+					if ((((double)labs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
+#else
 					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
+#endif
 						good_one[i_col] = FALSE;
 						continue;
 					}
@@ -899,11 +910,16 @@ void allocate_more_work_space (struct FILTER1D_INFO *F)
 
 void free_space (struct FILTER1D_INFO *F)
 {
+	int i;
+	if (F->robust || (F->filter_type > FILTER1D_CONVOLVE) ) {	
+		for (i = 0; i < F->n_cols; i++)	GMT_free ((void *)F->work[i]);
+		GMT_free ((void *)F->work);
+	}
+	for (i = 0; i < F->n_cols; i++)	GMT_free ((void *)F->data[i]);
 	GMT_free ((void *)F->data);
 	GMT_free ((void *)F->n_this_col);
-	GMT_free ((void *)F->n_left);
-	GMT_free ((void *)F->n_right);
-	GMT_free ((void *)F->work);
+	if (F->check_asym) GMT_free ((void *)F->n_left);
+	if (F->check_asym) GMT_free ((void *)F->n_right);
 	GMT_free ((void *)F->min_loc);
 	GMT_free ((void *)F->max_loc);
 	GMT_free ((void *)F->last_loc);
@@ -912,11 +928,12 @@ void free_space (struct FILTER1D_INFO *F)
 	GMT_free ((void *)F->max_scl);
 	GMT_free ((void *)F->last_scl);
 	GMT_free ((void *)F->this_scl);
+	if (F->n_f_wts) GMT_free ((void *)F->f_wt);
 }
 
-BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right)
+BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, GMT_LONG left, GMT_LONG right)
 {
-	int	last_row, this_row;
+	GMT_LONG	last_row, this_row;
 	BOOLEAN	lacking = FALSE;
 	double	last_t;
 
@@ -939,9 +956,10 @@ BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, int left, int right)
 	return (lacking);
 }
 
-void get_robust_estimates (struct FILTER1D_INFO *F, int j, int n, int both)
+void get_robust_estimates (struct FILTER1D_INFO *F, int j, GMT_LONG n, int both)
 {
-	int i, n_smooth, sort_me = TRUE;
+	GMT_LONG i, n_smooth;
+	BOOLEAN sort_me = TRUE;
 	double	low, high, last, temp;
 
 	if (F->filter_type > FILTER1D_MODE) {
@@ -1033,7 +1051,7 @@ void load_parameters (struct FILTER1D_INFO *F, struct FILTER1D_CTRL *Ctrl)
 void *New_Filter1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct FILTER1D_CTRL *C;
 
-	C = (struct FILTER1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct FILTER1D_CTRL), "New_Filter1d_Ctrl");
+	C = (struct FILTER1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct FILTER1D_CTRL), "New_Filter1d_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.ncols = 2;						/* Expected # of columns */
@@ -1042,6 +1060,6 @@ void *New_Filter1d_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Filter1d_Ctrl (struct FILTER1D_CTRL *C) {	/* Deallocate control structure */
-	if (C->F.file) GMT_free ((void *)C->F.file);	
+	if (C->F.file) free ((void *)C->F.file);	
 	GMT_free ((void *)C);	
 }
