@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2rgb.c,v 1.44 2007/03/24 01:42:06 pwessel Exp $
+ *	$Id: gmt2rgb.c,v 1.51 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *--------------------------------------------------------------------*/
 /*
  * gmt2rgb reads either (1) an 8, 24, or 32 bit Sun rasterfile and writes out the
- * red, green, and blue components in three separate grd files, or (2) a z grd
+ * red, green, and blue components in three separate grid files, or (2) a z grd
  * file and a cpt file and compute r, g, b and write these out instead.
  *
  * Author:	Paul Wessel
@@ -149,7 +149,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"%s %s - Write r/g/b gridfiles from a gridfile, a raw RGB file, or SUN rasterfile\n\n", GMT_program, GMT_VERSION);
+		fprintf (stderr,"%s %s - Write r/g/b grid files from a grid file, a raw RGB file, or SUN rasterfile\n\n", GMT_program, GMT_VERSION);
 		fprintf (stderr,"usage: %s <rasterfile|rgbfile||grdfile> [-C<cptfile>] [-F] [-G<nametemplate>] [%s]\n", GMT_program, GMT_Id_OPT);
 		fprintf (stderr,"\t[-L<layer>] [%s] [-V] [-W<width/height>[/<n_bytes>]]\n\n", GMT_Rgeo_OPT);
 
@@ -158,17 +158,17 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t<infile> can be one of three different intput files:\n");
 		fprintf (stderr,"\t  (1) An 8, 24, or 32-bit Sun rasterfile.  Use -I, -R, and -F to change the\n");
 		fprintf (stderr,"\t      the default values of dx = dy = 1 and region 1/ncols/1/nrows.\n");
-		fprintf (stderr,"\t  (2) A regular z grdfile.  Use -C to provide a cpt file with which\n");
+		fprintf (stderr,"\t  (2) A regular z grid file.  Use -C to provide a cpt file with which\n");
 		fprintf (stderr,"\t      to convert z to r/g/b triplets. -R, -I, and -F are ignored.\n");
 		fprintf (stderr,"\t  (3) A RGB or RGBA raw rasterfile. Since raw rastefiles have no header, you have to\n");
 		fprintf (stderr,"\t      give the image dimensions via -W\n");
 		fprintf (stderr,"\t      However, you may take the chance of letting the program try to\n");
 		fprintf (stderr,"\t      guess the image dimensions (slow, via FFT spectrum).\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
-		fprintf (stderr,"\t-C color palette file to convert z to rgb.  If given, we assume a z grdfile is provided,\n");
+		fprintf (stderr,"\t-C color palette file to convert z to rgb.  If given, we assume a z grid file is provided,\n");
 		fprintf (stderr,"\t   else we will try to read a Sun rasterfile.\n");
 		fprintf (stderr, "\t-F will force pixel registration [Default is grid registration]\n");
-		fprintf (stderr,"\t-G Give outputfile name template for the three red, green, blue gridfiles.\n");
+		fprintf (stderr,"\t-G Give outputfile name template for the three red, green, blue grid files.\n");
 		fprintf (stderr,"\t   The template MUST contain the format code %%c which will be replaced with r, g, and b.\n");
 		fprintf (stderr,"\t   [Default is gmt2rgb_%%c.grd]\n");
 		fprintf (stderr, "\t-I specifies grid size(s).  Append m (or c) to <dx> and/or <dy> for minutes (or seconds)\n");
@@ -216,7 +216,7 @@ int main (int argc, char **argv)
 	}
 	else {
 		if (!file) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input z grd file\n", GMT_program);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input z grid file\n", GMT_program);
 			error++;
 		}
 		if (!Ctrl->C.file) {
@@ -351,7 +351,7 @@ int main (int argc, char **argv)
 unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel, int nx, int ny) {
 	/* loadraw reads a raw binary grb or rgba rasterfile of depth 24, or 32 into memory */
 
-	int j, i, nm;
+	GMT_LONG j, i, nm;
 	unsigned char *buffer;
 
 	FILE *fp;
@@ -366,7 +366,7 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 	header->depth = 24;
 	header->width = nx;
 	header->height = ny;
-	nm = nx * ny * byte_per_pixel;
+	nm = ((size_t)nx) * ((size_t)ny) * ((size_t)byte_per_pixel);
 	header->length = nm;
 
 	buffer = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (unsigned char), GMT_program);
@@ -393,7 +393,8 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 	unsigned char *buffer;
 	float	*work, *datac, *img_pow, pow_max = -FLT_MAX, pm;
-	int	narray, img_size, k = 0, j, rgb[3], inc, n_pix, i, l, even;
+	int	k = 0, j, rgb[3], inc, i, l, even;
+	GMT_LONG narray, img_size, n_pix;
 	FILE *fp;
 
 	if ((fp = GMT_fopen (file, "rb")) == NULL) {
@@ -462,9 +463,9 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		fprintf (stderr, "%s WARNING: first test based on FFT failed to guess image dimensions.\n\tI'll do now a second try\t", GMT_program);
 		k = 1;		pm = 1;		l = 1;
 		while (k < 41) {
-			i = *raw_ny + (int)copysign((float)l,pm);
+			i = *raw_ny + (int)irint (copysign((double)l, (double)pm));
 			pm *= -1.;
-			j = (*raw_nx) + (int)copysign((float)l,pm);
+			j = (*raw_nx) + (int)irint (copysign((double)l, (double)pm));
 			if (i*j == n_pix) {	/* Got a good candidate */
 				*raw_ny = i;	*raw_nx = j;
 				fprintf (stderr, "... SUCESS (W = %d, H = %d)\n", *raw_nx, *raw_ny);
@@ -494,7 +495,7 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 void *New_Gmt2rgb_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GMT2RGB_CTRL *C;
 	
-	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMT2RGB_CTRL), "New_Gmt2rgb_Ctrl");
+	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT2RGB_CTRL), "New_Gmt2rgb_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -505,7 +506,7 @@ void *New_Gmt2rgb_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	if (C->G.name) GMT_free ((void *)C->G.name);	
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->G.name) free ((void *)C->G.name);	
 	GMT_free ((void *)C);	
 }
