@@ -1,4 +1,4 @@
-/*      $Id: gmt_agc_io.c,v 1.17 2007/03/24 01:42:06 pwessel Exp $
+/*      $Id: gmt_agc_io.c,v 1.22 2008/05/01 03:00:38 guru Exp $
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -25,14 +25,13 @@
 
 # define ZBLOCKWIDTH 	40
 # define ZBLOCKHEIGHT 	40
-# define RECORDLENGTH 	1614
-# define FIRSTZ  	12
 # define PREHEADSIZE	12
 # define POSTHEADSIZE	2
-# define BUFFHEADSIZE	(6 + POSTHEADSIZE)
+# define HEADINDSIZE	6
+# define BUFFHEADSIZE	(HEADINDSIZE + POSTHEADSIZE)
+# define RECORDLENGTH 	(ZBLOCKWIDTH*ZBLOCKHEIGHT + PREHEADSIZE + POSTHEADSIZE)
 
 # define AGCHEADINDICATOR	"agchd:"
-# define HEADINDSIZE		6
 # define PARAMSIZE		(int)((GRD_REMARK_LEN - HEADINDSIZE) / BUFFHEADSIZE)
 
 int GMT_is_agc_grid (char *file)
@@ -45,7 +44,7 @@ int GMT_is_agc_grid (char *file)
 	if (!strcmp(file, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
 	if (STAT (file, &buf)) return (GMT_GRDIO_STAT_FAILED);	/* Inquiry about file failed somehow */
 	if ((fp = GMT_fopen(file, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
-	if (GMT_fread ((void *)recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) return (GMT_GRDIO_READ_FAILED);
+	if (GMT_fread ((void *)recdata, sizeof(float), (size_t)RECORDLENGTH, fp) < RECORDLENGTH) return (GMT_GRDIO_READ_FAILED);
 	
 	y_min = recdata[0];
 	y_max = recdata[1];
@@ -82,7 +81,7 @@ int GMT_agc_read_grd_info (struct GRD_HEADER *header)
 	else if ((fp = GMT_fopen (header->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	if (GMT_fread ((void *)recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) return (GMT_GRDIO_READ_FAILED);
+	if (GMT_fread ((void *)recdata, sizeof(float), (size_t)RECORDLENGTH, fp) < RECORDLENGTH) return (GMT_GRDIO_READ_FAILED);
 	
 	header->node_offset = 0;	/* Hardwired since no info about this in the header */
 	header->y_min = recdata[0];
@@ -93,10 +92,9 @@ int GMT_agc_read_grd_info (struct GRD_HEADER *header)
 	header->x_inc = recdata[5];
 	header->nx = GMT_get_n (header->x_min, header->x_max, header->x_inc, header->node_offset);
 	header->ny = GMT_get_n (header->y_min, header->y_max, header->y_inc, header->node_offset);
-	header->y_order = irint (ceil ((header->y_max - header->y_min) / (ZBLOCKHEIGHT * header->y_inc)));
 	header->z_scale_factor = 1.0;
 	header->z_add_offset = 0.0;
-	for (i = 6; i < FIRSTZ; i++) agchead[i-6] = recdata[i];
+	for (i = 6; i < PREHEADSIZE; i++) agchead[i-6] = recdata[i];
 	agchead[BUFFHEADSIZE-2] = recdata[RECORDLENGTH-2];
 	agchead[BUFFHEADSIZE-1] = recdata[RECORDLENGTH-1];
 	SaveAGCHeader (header->remark, agchead);
@@ -123,7 +121,7 @@ int GMT_agc_write_grd_info (struct GRD_HEADER *header)
 	
 	packAGCheader (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	if (GMT_fwrite ((void *)prez, sizeof(float), PREHEADSIZE, fp) < PREHEADSIZE) return (GMT_GRDIO_WRITE_FAILED);
+	if (GMT_fwrite ((void *)prez, sizeof(float), (size_t)PREHEADSIZE, fp) < PREHEADSIZE) return (GMT_GRDIO_WRITE_FAILED);
 
 	if (fp != GMT_stdout) GMT_fclose (fp);
 
@@ -139,20 +137,20 @@ int GMT_agc_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_in;			/* Number of columns in subregion */
-	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int i, j, ij, j_gmt, i_0_out;	/* Misc. counters */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
+	GMT_LONG height_in;			/* Number of columns in subregion */
+	GMT_LONG inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
+	GMT_LONG i, j, j_gmt, i_0_out;	/* Misc. counters */
 	int *k;				/* Array with indices */
-	int size;			/* Length of data type */
-	int datablockcol, datablockrow, n_read = 0, rowstart, rowend, colstart, colend, row, col;
+	GMT_LONG block, n_blocks, n_blocks_x, n_blocks_y;	/* Misc. counters */
+	GMT_LONG datablockcol, datablockrow, rowstart, rowend, colstart, colend, row, col;
+	GMT_LONG ij;
 	FILE *fp;			/* File pointer to data or pipe */
-	BOOLEAN check = FALSE;		/* TRUE if nan-proxies are used to signify NaN (for non-floating point types) */
 	float z[ZBLOCKWIDTH][ZBLOCKHEIGHT];
-	int ReadRecord (FILE *fpi, int recnum, float *z);
+	int ReadRecord (FILE *fpi, GMT_LONG recnum, float *z);
 	
 	if (!strcmp (header->name, "=")) {
 #ifdef SET_IO_MODE
@@ -163,9 +161,6 @@ int GMT_agc_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	else if ((fp = GMT_fopen (header->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	size = GMT_grd_data_size (header->type, &header->nan_value);
-	check = !GMT_is_dnan (header->nan_value);
-
 	GMT_err_pass (GMT_grd_prep_io (header, &w, &e, &s, &n, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), header->name);
 
 	width_out = width_in;		/* Width of output array */
@@ -187,10 +182,12 @@ int GMT_agc_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	header->z_min = +DBL_MAX;
 	header->z_max = -DBL_MAX;
 	
+	n_blocks_y = irint (ceil ((double)header->ny / (double)ZBLOCKHEIGHT));
+	n_blocks_x = irint (ceil ((double)header->nx / (double)ZBLOCKWIDTH));
+	n_blocks = n_blocks_x * n_blocks_y;
 	datablockcol = datablockrow = 0;
-	while ( !feof(fp) ) {
-		if (ReadRecord(fp, n_read, (float *)z)) return (GMT_GRDIO_READ_FAILED);
-  		n_read++;
+	for (block = 0; block < n_blocks; block++) {
+		if (ReadRecord(fp, block, (float *)z)) return (GMT_GRDIO_READ_FAILED);
 		rowstart = datablockrow * ZBLOCKHEIGHT;
 		rowend = MIN(rowstart + ZBLOCKHEIGHT, header->ny);
 		for (i = 0, row = rowstart; row < rowend; i++, row++) {
@@ -205,7 +202,7 @@ int GMT_agc_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 			}
 		}
 
-		if (++datablockrow >= header->y_order) {
+		if (++datablockrow >= n_blocks_y) {
 			datablockrow = 0;
 			datablockcol++;
 		}
@@ -246,20 +243,20 @@ int GMT_agc_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	/*		for imaginary parts when processed by grdfft etc. */
 
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_out;			/* Number of columns in subregion */
-	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int i, j, i2, j2, ij;		/* Misc. counters */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
+	GMT_LONG height_out;			/* Number of columns in subregion */
+	GMT_LONG inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
+	GMT_LONG i, j, i2, j2;		/* Misc. counters */
 	int *k;				/* Array with indices */
-	int size;			/* Length of data type */
+	GMT_LONG block, n_blocks, n_blocks_x, n_blocks_y;	/* Misc. counters */
+	GMT_LONG ij;
 	FILE *fp;			/* File pointer to data or pipe */
-	BOOLEAN check = FALSE;		/* TRUE if nan-proxies are used to signify NaN (for non-floating point types) */
 	float outz[ZBLOCKWIDTH][ZBLOCKHEIGHT];
-	int rowstart, rowend, colstart, colend = 0, datablockcol, datablockrow;
-	int j_gmt, row, col;
+	GMT_LONG rowstart, rowend, colstart, colend = 0, datablockcol, datablockrow;
+	GMT_LONG j_gmt, row, col;
 	float prez[PREHEADSIZE], postz[POSTHEADSIZE];
 	int WriteRecord (FILE *file, float *rec, float *prerec, float *postrec);
 	void packAGCheader (float *prez, float *postz, struct GRD_HEADER *header);
@@ -273,9 +270,6 @@ int GMT_agc_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	else if ((fp = GMT_fopen (header->name, "rb+")) == NULL && (fp = fopen (header->name, "wb")) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 	
-	size = GMT_grd_data_size (header->type, &header->nan_value);
-	check = !GMT_is_dnan (header->nan_value);
-
 	GMT_err_pass (GMT_grd_prep_io (header, &w, &e, &s, &n, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &k), header->name);
 
 	width_in = width_out;		/* Physical width of input array */
@@ -315,9 +309,11 @@ int GMT_agc_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	
 	packAGCheader (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	header->y_order = irint (ceil ((header->y_max - header->y_min) / (ZBLOCKHEIGHT * header->y_inc)));
+	n_blocks_y = irint (ceil ((double)header->ny / (double)ZBLOCKHEIGHT));
+	n_blocks_x = irint (ceil ((double)header->nx / (double)ZBLOCKWIDTH));
+	n_blocks = n_blocks_x * n_blocks_y;
 	datablockcol = datablockrow = 0;
-	do {
+	for (block = 0; block < n_blocks; block++) {
 		rowstart = datablockrow * ZBLOCKHEIGHT;
 		rowend = MIN(rowstart + ZBLOCKHEIGHT, header->ny);
 		for (i = 0, row = rowstart; row < rowend; i++, row++) {
@@ -334,11 +330,11 @@ int GMT_agc_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 
 		if (WriteRecord (fp, (float*)outz, prez, postz)) return (GMT_GRDIO_WRITE_FAILED);
 
-		if (++datablockrow >= header->y_order) {
+		if (++datablockrow >= n_blocks_y) {
 			datablockrow = 0;
 			datablockcol++;
 		}
-	} while (rowend != header->ny || colend != header->nx);
+	}
 
 	if (fp != GMT_stdout) GMT_fclose (fp);
 
@@ -375,23 +371,23 @@ void SaveAGCHeader (char *remark, float *agchead)
 	}
 }
 
-int ReadRecord (FILE *fpi, int recnum, float *z)
+int ReadRecord (FILE *fpi, GMT_LONG recnum, float *z)
 { 
 	int nitems;
-	float garbage[FIRSTZ];
+	float garbage[PREHEADSIZE];
 
-	if (GMT_fread ((void *)garbage, sizeof(float), FIRSTZ, fpi) < FIRSTZ) return (GMT_GRDIO_READ_FAILED);
-	nitems = GMT_fread ((void *)z, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, fpi);
+	if (GMT_fread ((void *)garbage, sizeof(float), (size_t)PREHEADSIZE, fpi) < (size_t)PREHEADSIZE) return (GMT_GRDIO_READ_FAILED);
+	nitems = GMT_fread ((void *)z, sizeof(float), (size_t)(ZBLOCKWIDTH * ZBLOCKHEIGHT), fpi);
 
 	if (nitems != (ZBLOCKWIDTH * ZBLOCKHEIGHT) && !feof(fpi)) return (GMT_GRDIO_READ_FAILED);	/* Bad stuff */
-	if (GMT_fread ((void *)garbage, sizeof(float), RECORDLENGTH - (ZBLOCKWIDTH * ZBLOCKHEIGHT + FIRSTZ), fpi) < (RECORDLENGTH - (ZBLOCKWIDTH * ZBLOCKHEIGHT + FIRSTZ))) return (GMT_GRDIO_READ_FAILED);
+	if (GMT_fread ((void *)garbage, sizeof(float), (size_t)POSTHEADSIZE, fpi) < (size_t)POSTHEADSIZE) return (GMT_GRDIO_READ_FAILED);
 	return (GMT_NOERROR);
 }
 
 int WriteRecord (FILE *file, float *rec, float *prerec, float *postrec)
 {
-	if (GMT_fwrite ((void *)prerec, sizeof(float), PREHEADSIZE, file) < PREHEADSIZE) return (GMT_GRDIO_WRITE_FAILED);
-	if (GMT_fwrite ((void *)rec, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, file) < (ZBLOCKWIDTH * ZBLOCKHEIGHT)) return (GMT_GRDIO_WRITE_FAILED);
-	if (GMT_fwrite( (void *)postrec, sizeof(float), POSTHEADSIZE, file) < POSTHEADSIZE)  return (GMT_GRDIO_WRITE_FAILED); 
+	if (GMT_fwrite ((void *)prerec, sizeof(float), (size_t)PREHEADSIZE, file) < (size_t)PREHEADSIZE) return (GMT_GRDIO_WRITE_FAILED);
+	if (GMT_fwrite ((void *)rec, sizeof(float), (size_t)(ZBLOCKWIDTH * ZBLOCKHEIGHT), file) < (size_t)(ZBLOCKWIDTH * ZBLOCKHEIGHT)) return (GMT_GRDIO_WRITE_FAILED);
+	if (GMT_fwrite( (void *)postrec, sizeof(float), (size_t)POSTHEADSIZE, file) < (size_t)POSTHEADSIZE)  return (GMT_GRDIO_WRITE_FAILED); 
 	return (GMT_NOERROR);
 }
