@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_boundcond.h,v 1.9 2007/01/30 20:37:08 pwessel Exp $
+ *	$Id: gmt_boundcond.h,v 1.13 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,10 @@
 
 struct GMT_EDGEINFO {
 	/* Description below is the final outcome after parse and verify */
-	int	nxp;	/* if X periodic, nxp > 0 is the period in pixels  */
-	int	nyp;	/* if Y periodic, nxp > 0 is the period in pixels  */
+	GMT_LONG	nxp;	/* if X periodic, nxp > 0 is the period in pixels  */
+	GMT_LONG	nyp;	/* if Y periodic, nxp > 0 is the period in pixels  */
 	BOOLEAN	gn;	/* TRUE if top    edge will be set as N pole  */
-	BOOLEAN	gs;	/* TRUE if bottom edge will be set as N pole  */
+	BOOLEAN	gs;	/* TRUE if bottom edge will be set as S pole  */
 };
 
 
