@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_calclock.c,v 1.54 2007/03/08 02:24:20 pwessel Exp $
+ *	$Id: gmt_calclock.c,v 1.61 2008/03/24 08:58:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,35 +38,28 @@
 BOOLEAN want_iso = FALSE;	/* Temporary to test compilation.  Delete this.  */
 
 /* Functions to assemble/disassemble a continuous
-variable (GMT_dtime) and a calendar day (GMT_cal_rd)
+variable (double) and a calendar day (GMT_cal_rd)
 and time of day (double secs):  
 */
 
-GMT_dtime GMT_rdc2dt (GMT_cal_rd rd, double secs) {
-#ifndef OLDCAL
+double GMT_rdc2dt (GMT_cal_rd rd, double secs) {
 /*	Given rata die rd and double seconds, return 
 	time in TIME_UNIT relative to chosen TIME_EPOCH  */
 	double f_days;
-	f_days = (rd - GMT_time_system[gmtdefs.time_system].rata_die - GMT_time_system[gmtdefs.time_system].epoch_t0);
-	if (gmtdefs.time_system == 6)  f_days += GMT_GCAL_EPOCH;
-	return ((f_days * GMT_DAY2SEC_F  + secs) * GMT_time_system[gmtdefs.time_system].i_scale);
-#else
-/*	Given rata die rd and double seconds, return 
-	GMT_dtime value of time  */
-	return ((GMT_dtime)(GMT_DAY2SEC_F*(rd-GMT_GCAL_EPOCH) + secs));
-#endif
+	f_days = (rd - gmtdefs.time_system.rata_die - gmtdefs.time_system.epoch_t0);
+	return ((f_days * GMT_DAY2SEC_F  + secs) * gmtdefs.time_system.i_scale);
 }
 
-int	splitinteger(double value, int epsilon, double *doublepart) {
+GMT_LONG	splitinteger(double value, int epsilon, double *doublepart) {
 	/* Split value into integer and and floating part for date usage.
 	   While the integer can be negative, doublepart is always >= 0
 	   When doublepart is close to 0 or close to epsilon, doublepart is set
 	   to zero and in the latter case, splitinteger is raised by one.
 	   This makes value "snap" to multiples of epsilon.
 	*/
-	int i;
+	GMT_LONG i;
 
-	i = (int) floor(value/(double)epsilon);
+	i = (GMT_LONG) floor(value/(double)epsilon);
 	*doublepart = value - ((double)i)*((double)epsilon);
 	if ((*doublepart) < GMT_SMALL)
 		*doublepart = 0.0;	/* Snap to the lower integer */
@@ -77,24 +70,16 @@ int	splitinteger(double value, int epsilon, double *doublepart) {
 	return i;
 }
 
-void	GMT_dt2rdc (GMT_dtime t, GMT_cal_rd *rd, double *s) {
+void	GMT_dt2rdc (double t, GMT_cal_rd *rd, double *s) {
 
-	int i;
+	GMT_LONG i;
 
-#ifndef OLDCAL
 /*	Given time in TIME_UNIT relative to TIME_EPOCH, load rata die of this day
 	in rd and the seconds since the start of that day in s.  */
 	double t_sec;
-	t_sec = (t * GMT_time_system[gmtdefs.time_system].scale + GMT_time_system[gmtdefs.time_system].epoch_t0 * GMT_DAY2SEC_F);
-	i = splitinteger(t_sec, 86400, s) + GMT_time_system[gmtdefs.time_system].rata_die;
-	if (gmtdefs.time_system == 6) i -= GMT_GCAL_EPOCH;
+	t_sec = (t * gmtdefs.time_system.scale + gmtdefs.time_system.epoch_t0 * GMT_DAY2SEC_F);
+	i = splitinteger(t_sec, 86400, s) + gmtdefs.time_system.rata_die;
 	*rd = (GMT_cal_rd)(i);
-#else
-/*	Given GMT_dtime value t, load calendar data of that day
-	in rd and the seconds since the start of that day in s.  */
-	i = splitinteger(t, 86400, s);
-	*rd = (GMT_cal_rd)(i+GMT_GCAL_EPOCH);
-#endif
 }
 
 
@@ -131,7 +116,7 @@ GMT_cal_rd GMT_kday_on_or_before (GMT_cal_rd date, int kday) {
 /*	Given date and kday, return the date of the nearest kday
 	on or before the given date.
 */
-	return ((GMT_cal_rd)(date - GMT_cal_imod((date-kday), 7)));
+	return ((GMT_cal_rd)(date - GMT_cal_imod((int)(date-kday), 7)));
 }
 
 GMT_cal_rd GMT_kday_after (GMT_cal_rd date, int kday) {
@@ -187,7 +172,7 @@ GMT_cal_rd GMT_rd_from_gymd (int gy, int gm, int gd) {
 	return the rata die integer day number.  */
 	
 	double s;
-	int	day_offset, yearm1, rd;
+	GMT_LONG	day_offset, yearm1, rd;
 	
 	
 	if (gm <= 2) {
@@ -203,15 +188,10 @@ GMT_cal_rd GMT_rd_from_gymd (int gy, int gm, int gd) {
 	}
 	
 	yearm1 = gy - 1;
-#ifndef OLDCAL
-	rd = day_offset + gd;
-#else
-	rd = GMT_GCAL_EPOCH - 1 + day_offset + gd;
-#endif
-	rd += 365 * yearm1;
+	rd = day_offset + gd + 365 * yearm1;
 	s = floor(yearm1/4.0) - floor(yearm1/100.0) + floor(yearm1/400.0);
 	s += floor( (367 * gm - 362)/12.0);
-	rd += (int)s;
+	rd += (GMT_LONG)irint(s);
 	return ((GMT_cal_rd)rd);
 }
 
@@ -220,11 +200,7 @@ int	GMT_gyear_from_rd (GMT_cal_rd date) {
 
 	int d0, d1, d2, d3, n400, n100, n4, n1, year;
 	
-#ifndef OLDCAL
 	d0 = date - 1;
-#else
-	d0 = date - GMT_GCAL_EPOCH;
-#endif
 	n400 = (int) floor (d0 / 146097.0);
 	d1 = GMT_cal_imod (d0, 146097);
 	n100 = (int) floor (d1 / 36524.0);
@@ -267,7 +243,7 @@ void GMT_gcal_from_rd (GMT_cal_rd date, struct GMT_gcal *gcal) {
 	
 	/* Day of the week in 0 thru 6:  */
 	
-	gcal->day_w = GMT_cal_imod (date, 7);
+	gcal->day_w = GMT_cal_imod ((int)date, 7);
 	
 	/* proleptic Gregorian operations:  */
 
@@ -306,39 +282,6 @@ void GMT_gcal_from_rd (GMT_cal_rd date, struct GMT_gcal *gcal) {
 	gcal->iso_d = (gcal->day_w) ? gcal->day_w : 7;
 }
 
-double	GMT_usert_from_dt (GMT_dtime t) {
-#ifndef OLDCAL
-	return (t);
-#else
-	double x;
-
-	x = t - GMT_time_system[gmtdefs.time_system].epoch_t0;
-	if (GMT_time_system[gmtdefs.time_system].i_scale != 1.0) x *= GMT_time_system[gmtdefs.time_system].i_scale;
-	return (x);
-#endif
-}
-
-GMT_dtime	GMT_dt_from_usert (double x) {
-
-	/* If we are going to allow the user to have irregular times
-		(months, years) then we have to do something complicated.
-		
-		For now, just scale and offset ?
-	*/
-	
-#ifndef OLDCAL
-	return (x);
-#else
-	if (GMT_time_system[gmtdefs.time_system].scale == 1.0) {
-		return ( (GMT_dtime) (x + GMT_time_system[gmtdefs.time_system].epoch_t0) );
-	}
-	else {
-		return ( (GMT_dtime) (x*GMT_time_system[gmtdefs.time_system].scale 
-			+ GMT_time_system[gmtdefs.time_system].epoch_t0) );
-	}
-#endif
-}
-
 int	GMT_y2_to_y4_yearfix (int y2) {
 
 	/* Convert 2-digit year to 4-digit year, using 
@@ -424,7 +367,7 @@ BOOLEAN	GMT_hms_is_bad (int h, int m, double s) {
 	return (FALSE);
 }
 
-void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
+void	GMT_gcal_from_dt (double t, struct GMT_gcal *cal) {
 
 	/* Given time in internal units, load cal and clock info
 		in cal.
@@ -434,7 +377,7 @@ void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
 	
 	GMT_cal_rd rd;
 	double	x;
-	int i;
+	GMT_LONG i;
 
 	GMT_dt2rdc (t, &rd, &x);
 	GMT_gcal_from_rd (rd, cal);
@@ -867,7 +810,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 	if (step_secs == GMT_DAY2SEC_I) {
 		/* Special case of a 1-day step.  */
 		if (p->sd[0] != 0.0) {	/* Floor it to start of day.  */
-			p->dt[0] -= (p->sd[0] * GMT_time_system[gmtdefs.time_system].i_scale);
+			p->dt[0] -= (p->sd[0] * gmtdefs.time_system.i_scale);
 			p->sd[0] = 0.0;
 		}
 		/* Now we step to next day in rd first, and set dt from there.
@@ -881,7 +824,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 		if (init) {
 			x = step_secs * floor (p->sd[0] / step_secs);
 			if (x != p->sd[0]) {
-				p->dt[0] -= ((p->sd[0] - x) * GMT_time_system[gmtdefs.time_system].i_scale);
+				p->dt[0] -= ((p->sd[0] - x) * gmtdefs.time_system.i_scale);
 				x = p->sd[0];
 			}
 		}
@@ -900,7 +843,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 		}
 		else {
 			p->sd[1] = x;
-			p->dt[1] = p->dt[0] + step_secs * GMT_time_system[gmtdefs.time_system].i_scale;
+			p->dt[1] = p->dt[0] + step_secs * gmtdefs.time_system.i_scale;
 			/* No call here to reset cc[1] struct, as rd hasn't changed.
 				Later, if it is desired to reset struct for clock
 				changes on same day, add a call here.  */
@@ -934,7 +877,7 @@ int	GMT_gmonth_length (int year,  int month) {
 	return (k);
 }
 
-void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct GMT_CLOCK_IO *C, BOOLEAN upper, int kind, GMT_dtime dt)
+void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct GMT_CLOCK_IO *C, BOOLEAN upper, int kind, double dt)
 {	/* Given the internal time representation dt and the formatting information
 	 * in the D and C structure, write the calendar representation to strings date and clock,
 	 * but skip either string if it is a NULL pointer */
@@ -942,9 +885,9 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 	struct GMT_gcal calendar;
 	int i_sec, m_sec, ap, ival[3];
 	char text[GMT_CALSTRING_LENGTH];
-	GMT_dtime step;
+	double step;
 
-	step = 0.5 / C->f_sec_to_int / GMT_time_system[gmtdefs.time_system].scale;	/* Precision desired in time units */
+	step = 0.5 / C->f_sec_to_int / gmtdefs.time_system.scale;	/* Precision desired in time units */
 
 	GMT_gcal_from_dt (dt + step, &calendar);			/* Convert dt to a complete calendar structure */
 	
@@ -965,7 +908,7 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 			if (D->item_pos[1] != -1) ival[D->item_pos[1]] = calendar.month;
 			if (D->item_pos[2] != -1) ival[D->item_pos[2]] = calendar.day_m;
 		}
-		memset ((void *)date, 0, GMT_CALSTRING_LENGTH);			/* To set all to zero */
+		memset ((void *)date, 0, (size_t)GMT_CALSTRING_LENGTH);			/* To set all to zero */
 		if (D->mw_text)	{						/* Must write month or week name */
 			if (D->iso_calendar)
 				strcpy (text, GMT_time_language.week_name[kind]);
@@ -985,7 +928,7 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 
 	if (!clock) return;	/* Do not want a formatted clock string - return here */
 	
-	memset ((void *)clock, 0, GMT_CALSTRING_LENGTH);			/* To set all to zero */
+	memset ((void *)clock, 0,  (size_t)GMT_CALSTRING_LENGTH);			/* To set all to zero */
 	i_sec = (int) floor (calendar.sec);
 	m_sec = (int) floor (C->f_sec_to_int * (calendar.sec - i_sec));
 	
@@ -1016,7 +959,7 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 	return;
 }
 
-void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_PLOT_AXIS_ITEM *T, GMT_dtime t)
+void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_PLOT_AXIS_ITEM *T, double t)
 {	/* Assemble the annotation label given the formatting options presented */
 	struct GMT_gcal calendar;
 
@@ -1027,7 +970,7 @@ void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_P
 			(P->date.compact) ? sprintf (string, "%d", calendar.year) : sprintf (string, "%4.4d", calendar.year);
 			break;
 		case 'y':	/* 2-digit integer year */
-			/* (P->date.compact) ? sprintf (string, "%d", calendar.year % 100) : sprintf (string, "%2.2d", calendar.year % 100); */
+			/* (P->date.compact) ? sprintf (string, "%ld", calendar.year % 100) : sprintf (string, "%2.2ld", calendar.year % 100); */
 			sprintf (string, "%2.2d", calendar.year % 100);
 			break;
 		case 'O':	/* Plot via date format */
@@ -1075,7 +1018,7 @@ void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_P
 			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'c':	/* 2-digit seconds */
-			(P->date.compact) ? sprintf (string, "%d", irint(calendar.sec)) : sprintf (string, "%2.2d", irint(calendar.sec));
+			(P->date.compact) ? sprintf (string, "%d", (int)irint(calendar.sec)) : sprintf (string, "%2.2d", (int)irint(calendar.sec));
 			break;
 		default:
 			fprintf (stderr, "%s: INTERNAL ERROR: wrong unit passed to GMT_get_time_label\n", GMT_program);
@@ -1103,7 +1046,7 @@ int	GMT_read_clock (char *s, double *t) {
 */
 
 	double	dsec;
-	int	j, k;
+	GMT_LONG	j, k;
 	char	*cm, *cs;
 	
 	/* Get hours:  */
@@ -1114,7 +1057,7 @@ int	GMT_read_clock (char *s, double *t) {
 		s[k-j] = '\0';
 		cm = &cm[1];	/* Move beyond colon  */
 	}
-	if ( (sscanf(s, "%d", &k)) != 1) return (-1);
+	if ( (sscanf(s, "%ld", &k)) != 1) return (-1);
 	if (k < 0 || k > 24) return (-1);
 	*t = GMT_HR2SEC_I * k;		/* t now has hours (in secs) */
 	
@@ -1127,7 +1070,7 @@ int	GMT_read_clock (char *s, double *t) {
 		cm[k-j] = '\0';
 		cs = &cs[1];
 	}
-	if ( (sscanf(cm, "%d", &k)) != 1) return (-1);
+	if ( (sscanf(cm, "%ld", &k)) != 1) return (-1);
 	if (k < 0 || k > 59) return (-1);
 	(*t) += GMT_MIN2SEC_I * k;	/* Now add in minutes (in secs) */
 	
@@ -1151,7 +1094,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 	present then <year>, <j>, <k> are (proleptic
 	gregorian) year, month, day of month.  */
 
-	int	i, j, k, itemp1, itemp2=1, itemp3=1, is_iso=0;
+	GMT_LONG	i, j, k, itemp1, itemp2=1, itemp3=1, is_iso=0;
 	char	*cj, *ck = CNULL;
 	
 	/* A period or comma would be wrong for this format:  */
@@ -1172,7 +1115,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 		s[k-(i+j)] = '\0';
 	}
 	
-	if ( (sscanf (s, "%d", &itemp1) ) != 1) return (-1);
+	if ( (sscanf (s, "%ld", &itemp1) ) != 1) return (-1);
 	
 	/* A null value of cj is not an error.  We might have
 		had "1985T" passed to a function and it might
@@ -1195,7 +1138,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 			k = strlen (cj);
 			cj[k-j] = '\0';
 		}
-		if ( (sscanf (cj, "%d", &itemp2) ) != 1) return (-1);
+		if ( (sscanf (cj, "%ld", &itemp2) ) != 1) return (-1);
 		if (itemp2 < 1) return (-1);
 		if (is_iso) {
 			if (itemp2 > 53) return (-1);
@@ -1205,7 +1148,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 		}
 		if (ck) {
 			/* Read it, skipping the leading hyphen.  */
-			if ( (sscanf (&ck[1], "%d", &itemp3) ) != 1) return (-1);
+			if ( (sscanf (&ck[1], "%ld", &itemp3) ) != 1) return (-1);
 			if (itemp3 < 1) return (-1);
 			if (is_iso) {
 				if (itemp3 > 7) return (-1);
