@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_colors.h,v 1.23 2007/01/30 20:37:08 pwessel Exp $
+ *	$Id: gmt_colors.h,v 1.31 2008/04/30 01:32:56 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,10 +43,8 @@
 #define GMT_FGD 1
 #define GMT_NAN 2
 
-#define GMT_N_COLOR_NAMES 663
-
 /* How B/W TV's convert RGB to Gray */
-#define GMT_YIQ(rgb) irint (0.299 * (rgb[0]) + 0.587 * (rgb[1]) + 0.114 * (rgb[2]))
+#define GMT_YIQ(rgb) ((int)irint (0.299 * (rgb[0]) + 0.587 * (rgb[1]) + 0.114 * (rgb[2])))
 
 /* Determine if a RGB combination is grayshade */
 #define GMT_is_gray(r,g,b) ((r) == (g) && (r) == (b))
@@ -54,6 +52,9 @@
 /* Determine if a R(=G=B) gray combination is in fact B/W */
 #define GMT_is_bw(r) ((r) == 0 || (r) == 255)
 
+/* Determine if there is any fill at all */
+#define GMT_is_nofill(F) (F->rgb[0] == -1 && !F->use_pattern)
+
 /* Here is the definition of the GMT_lut structure that is used in programs
  * that deals with coloring of grids.
  */
@@ -71,7 +72,7 @@ struct GMT_LUT {
 struct GMT_BFN_COLOR {	/* For back-, fore-, and nan-colors */
 	int rgb[3];
 	double hsv[3];
-	int skip;
+	BOOLEAN skip;
 	struct GMT_FILL *fill;
 };
 
@@ -86,7 +87,9 @@ EXTERN_MSC BOOLEAN GMT_b_and_w;		/* TRUE if only black OR white is used */
 EXTERN_MSC BOOLEAN GMT_continuous;	/* TRUE if colors change continuously within slice */
 EXTERN_MSC BOOLEAN GMT_cpt_pattern;	/* TRUE if cpt file contains any patterns */
 EXTERN_MSC BOOLEAN GMT_cpt_skip;	/* TRUE if current z-slice is to be skipped */
-
+#ifdef GMT_CPT2	
+EXTERN_MSC BOOLEAN GMT_categorical;	/* TRUE if the CPT applies to categorical data */
+#endif
 EXTERN_MSC void GMT_sample_cpt (double z[], int nz, BOOLEAN continuous, BOOLEAN reverse, int log_mode);
 
 #endif /* _GMT_COLORS_H */
