@@ -1,14 +1,14 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_customio.c,v 1.61 2007/03/24 01:42:06 pwessel Exp $
+ *	$Id: gmt_customio.c,v 1.69 2008/04/02 15:46:41 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
  *	the Free Software Foundation; version 2 of the License.
  *
- *	This program is distributed in the hope that it will be useful,
+ *	This program is distributed in the hope that it wi1552ll be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
@@ -30,7 +30,7 @@
  *	1. Write the five required routines (see below).
  *	2. increment parameter GMT_N_GRD_FORMATS in file gmt_grdio.h
  *	3. Append another entry in the gmt_customio.h file.
- *	4. Provide another entry in the $GMT_SHAREDIR/gmtformats.d file
+ *	4. Provide another entry in the $GMT_SHAREDIR/conf/gmt_formats.conf file
  *
  * Author:	Paul Wessel
  * Date:	9-SEP-1992
@@ -304,16 +304,17 @@ void GMT_grdio_init (void) {
 
 #define	RAS_MAGIC	0x59a66a95
 
-int GMT_is_ras_grid (char *file)
+int GMT_is_ras_grid (struct GRD_HEADER *header)
 {	/* Determine if file is a Sun rasterfile */
 	FILE *fp;
 	struct rasterfile h;
-	if (!strcmp(file, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if ((fp = GMT_fopen (file, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
+	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
+	if ((fp = GMT_fopen (header->name, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
 	if (GMT_read_rasheader (fp, &h)) return (GMT_GRDIO_READ_FAILED);
 	if (h.magic != RAS_MAGIC) return (GMT_GRDIO_NOT_RAS);
 	if (h.type != 1 || h.depth != 8) return (GMT_GRDIO_NOT_8BIT_RAS);
-	return (GMT_grd_format_decoder ("rb"));
+	header->type = GMT_grd_format_decoder ("rb");
+	return (header->type);
 }
 
 int GMT_ras_read_grd_info (struct GRD_HEADER *header)
@@ -393,8 +394,9 @@ int GMT_ras_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col, first_row, last_row, kk, inc = 1;
-	int i, j, j2, ij, width_in, width_out, height_in, i_0_out, n2;
+	GMT_LONG first_col, last_col, first_row, last_row, inc = 1;
+	GMT_LONG i, j, j2, width_in, width_out, height_in, i_0_out, n2;
+	GMT_LONG kk, ij;
 	FILE *fp;
 	BOOLEAN piping = FALSE, check;
 	unsigned char *tmp;
@@ -415,7 +417,7 @@ int GMT_ras_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	else
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	n2 = (int) ceil (header->nx / 2.0) * 2;	/* Sun 8-bit rasters are stored using 16-bit words */
+	n2 = (GMT_LONG) ceil (header->nx / 2.0) * 2;	/* Sun 8-bit rasters are stored using 16-bit words */
 	tmp = (unsigned char *) GMT_memory (VNULL, (size_t)n2, sizeof (unsigned char), "GMT_ras_read_grd");
 
 	header->z_min = DBL_MAX;	header->z_max = -DBL_MAX;
@@ -479,10 +481,11 @@ int GMT_ras_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	/* padding:	# of empty rows/columns to remove on w, e, s, n of grid, respectively */
 	/* complex:	Must be FALSE for rasterfiles.    If 64 is added we write no header */
 
-	int i, i2, kk, inc = 1;
-	int j, ij, j2, width_in, width_out, height_out, n2;
-	int first_col, last_col, first_row, last_row;
+	GMT_LONG i, i2, inc = 1;
+	GMT_LONG j, j2, width_in, width_out, height_out, n2;
+	GMT_LONG first_col, last_col, first_row, last_row;
 	int *k;
+	GMT_LONG kk, ij;
 
 	BOOLEAN check, do_header = TRUE;
 
@@ -510,7 +513,7 @@ int GMT_ras_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	h.maptype = 0;
 	h.maplength = 0;
 
-	n2 = (int) ceil (header->nx / 2.0) * 2;
+	n2 = (GMT_LONG) ceil (header->nx / 2.0) * 2;
 	tmp = (unsigned char *) GMT_memory (VNULL, (size_t)n2, sizeof (unsigned char), "GMT_ras_write_grd");
 
 	check = !GMT_is_dnan (header->nan_value);
@@ -567,13 +570,13 @@ int GMT_read_rasheader (FILE *fp, struct rasterfile *h)
 	 */
 
 	unsigned char byte[4];
-	int i, j, value, in[4];
+	GMT_LONG i, j, value, in[4];
 
 	for (i = 0; i < 8; i++) {
 
 		if (GMT_fread ((void *)byte, sizeof (unsigned char), (size_t)4, fp) != 4) return (GMT_GRDIO_READ_FAILED);
 
-		for (j = 0; j < 4; j++) in[j] = (int)byte[j];
+		for (j = 0; j < 4; j++) in[j] = (GMT_LONG)byte[j];
 
 		value = (in[0] << 24) + (in[1] << 16) + (in[2] << 8) + in[3];
 
@@ -618,7 +621,7 @@ int GMT_write_rasheader (FILE *fp, struct rasterfile *h)
 	 */
 
 	unsigned char byte[4];
-	int i, value;
+	GMT_LONG i, value;
 
 	if (h->type == RT_OLD && h->length == 0) {
 		h->length = 2 * irint (ceil (h->width * h->depth / 16.0)) * h->height;
@@ -682,54 +685,54 @@ int GMT_write_rasheader (FILE *fp, struct rasterfile *h)
  * Functions :	GMT_bit_read_grd, GMT_bit_write_grd
  *-----------------------------------------------------------*/
 
-int GMT_is_native_grid (char *file)
+int GMT_is_native_grid (struct GRD_HEADER *header)
 {
 	struct STAT buf;
-	int nm, mx, status, size;
+	GMT_LONG nm, mx, status, size;
 	double item_size;
-	struct GRD_HEADER header;
+	struct GRD_HEADER t_head;
 	
-	if (!strcmp(file, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if (STAT (file, &buf)) return (GMT_GRDIO_STAT_FAILED);		/* Inquiry about file failed somehow */
-	strcpy (header.name, file);
-	if ((status = GMT_native_read_grd_info (&header))) return (GMT_GRDIO_READ_FAILED);	/* Failed to read header */
-	if (header.nx <= 0 || header.ny <= 0) return (GMT_GRDIO_BAD_VAL);		/* Garbage for nx or ny */
-	nm = header.nx * header.ny;
-	if (nm <= 0) return (GMT_GRDIO_BAD_VAL);		/* Overflow for nx * ny */
+	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
+	if (STAT (header->name, &buf)) return (GMT_GRDIO_STAT_FAILED);		/* Inquiry about file failed somehow */
+	strcpy (t_head.name, header->name);
+	if ((status = GMT_native_read_grd_info (&t_head))) return (GMT_GRDIO_READ_FAILED);	/* Failed to read header */
+	if (t_head.nx <= 0 || t_head.ny <= 0) return (GMT_GRDIO_BAD_VAL);		/* Garbage for nx or ny */
+	nm = ((GMT_LONG)t_head.nx) * ((GMT_LONG)t_head.ny);
+	if (nm <= 0) return (GMT_GRDIO_BAD_VAL);		/* Overflow for nx * ny? */
 	item_size = (buf.st_size - GRD_HEADER_SIZE) / nm;	/* Estimate size of elements */
 	size = irint (item_size);
-	if (fabs(item_size - (double)size) > GMT_CONV_LIMIT) return (GMT_GRDIO_BAD_VAL);	/* Size not an integer */
+	if (!GMT_IS_ZERO(item_size - (double)size)) return (GMT_GRDIO_BAD_VAL);	/* Size not an integer */
 	
 	switch (size) {
 		case 0:	/* Possibly bit map; check some more */
-			mx = (int) ceil (header.nx / 32.0);
-			nm = mx * header.ny;
+			mx = (GMT_LONG) ceil (t_head.nx / 32.0);
+			nm = mx * ((GMT_LONG)t_head.ny);
 			if ((buf.st_size - GRD_HEADER_SIZE) == nm)	/* yes it was a bit mask file */
-				return (GMT_grd_format_decoder ("bm"));
+				header->type = GMT_grd_format_decoder ("bm");
 			else	/* No, junk data */
 				return (GMT_GRDIO_BAD_VAL);
 			break;
 		case 1:	/* 1-byte elements */
-			return (GMT_grd_format_decoder ("bb"));
+			header->type = GMT_grd_format_decoder ("bb");
 			break;
 		case 2:	/* 2-byte short int elements */
-			return (GMT_grd_format_decoder ("bs"));
+			header->type = GMT_grd_format_decoder ("bs");
 			break;
 		case 4:	/* 4-byte elements - could be int or float */
 			/* See if we can decide it is a float grid */
-			if ((header.z_scale_factor == 1.0 && header.z_add_offset == 0.0) || 
-				fabs((header.z_min/header.z_scale_factor) - rint(header.z_min/header.z_scale_factor)) > GMT_CONV_LIMIT || fabs((header.z_max/header.z_scale_factor) - rint(header.z_max/header.z_scale_factor) > GMT_CONV_LIMIT))
-				return (GMT_grd_format_decoder ("bf"));
+			if ((t_head.z_scale_factor == 1.0 && t_head.z_add_offset == 0.0) || fabs((t_head.z_min/t_head.z_scale_factor) - rint(t_head.z_min/t_head.z_scale_factor)) > GMT_CONV_LIMIT || fabs((t_head.z_max/t_head.z_scale_factor) - rint(t_head.z_max/t_head.z_scale_factor)) > GMT_CONV_LIMIT)
+				header->type = GMT_grd_format_decoder ("bf");
 			else
-				return (GMT_grd_format_decoder ("bi"));
+				header->type = GMT_grd_format_decoder ("bi");
 			break;
 		case 8:	/* 8-byte elements */
-			return (GMT_grd_format_decoder ("bd"));
+			header->type = GMT_grd_format_decoder ("bd");
 			break;
 		default:	/* Garbage */
 			return (GMT_GRDIO_BAD_VAL);
 			break;
 	}
+	return (header->type);
 }
 
 int GMT_bit_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
@@ -741,9 +744,10 @@ int GMT_bit_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col, first_row, last_row, kk, word, bit, err;
-	int i, j, j2, ij, width_in, width_out, height_in, i_0_out, inc = 1, mx;
+	GMT_LONG first_col, last_col, first_row, last_row, word, bit, err;
+	GMT_LONG i, j, j2, width_in, width_out, height_in, i_0_out, inc = 1, mx;
 	int *k;
+	GMT_LONG kk, ij;
 	FILE *fp;
 	BOOLEAN piping = FALSE, check = FALSE;
 	unsigned int *tmp, ival;
@@ -762,7 +766,7 @@ int GMT_bit_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 		return (GMT_GRDIO_OPEN_FAILED);
 
 	check = !GMT_is_dnan (header->nan_value);
-	mx = (int) ceil (header->nx / 32.0);
+	mx = (GMT_LONG) ceil (header->nx / 32.0);
 
 	GMT_err_pass (GMT_grd_prep_io (header, &w, &e, &s, &n, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), header->name);
 
@@ -832,9 +836,11 @@ int GMT_bit_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc.   If 64 is added we write no header*/
 
-	int i, i2, kk, *k;
-	int j, ij, j2, width_in, width_out, height_out, mx, word, bit, err, inc = 1;
-	int first_col, last_col, first_row, last_row;
+	int *k;
+	GMT_LONG i, i2;
+	GMT_LONG j, j2, width_in, width_out, height_out, mx, word, bit, err, inc = 1;
+	GMT_LONG first_col, last_col, first_row, last_row;
+	GMT_LONG kk, ij;
 	BOOLEAN check = FALSE, do_header = TRUE;
 	unsigned int *tmp, ival;
 
@@ -892,7 +898,7 @@ int GMT_bit_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 		GMT_err_trap (GMT_native_write_grd_header (fp, header));
 	}
 
-	mx = (int) ceil (width_out / 32.0);
+	mx = (GMT_LONG) ceil (width_out / 32.0);
 	tmp = (unsigned int *) GMT_memory (VNULL, (size_t)mx, sizeof (unsigned int), "GMT_bit_write_grd");
 
 	i2 = first_col + pad[0];
@@ -1037,16 +1043,17 @@ int GMT_native_read_grd (struct GRD_HEADER *header, float *grid, double w, doubl
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_in;			/* Number of columns in subregion */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG height_in;			/* Number of columns in subregion */
 	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int kk, err, i, j, j2, ij, i_0_out;	/* Misc. counters */
+	int err, i, j, j2, i_0_out;	/* Misc. counters */
 	int *k;				/* Array with indices */
 	int type;			/* Data type */
-	int size;			/* Length of data type */
+	size_t size;			/* Length of data type */
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
+	GMT_LONG kk, ij;
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
 	FILE *fp;			/* File pointer to data or pipe */
 	BOOLEAN piping = FALSE;		/* TRUE if we read input pipe instead of from file */
 	BOOLEAN check = FALSE;		/* TRUE if nan-proxies are used to signify NaN (for non-floating point types) */
@@ -1144,16 +1151,17 @@ int GMT_native_write_grd (struct GRD_HEADER *header, float *grid, double w, doub
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc.  If 64 is added we write no header */
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_out;			/* Number of columns in subregion */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
+	GMT_LONG height_out;			/* Number of columns in subregion */
 	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int i, j, i2, j2, ij,err;	/* Misc. counters */
+	int i, j, i2, j2, err;	/* Misc. counters */
 	int *k;				/* Array with indices */
 	int type;			/* Data type */
-	int size;			/* Length of data type */
+	size_t size;			/* Length of data type */
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
+	GMT_LONG ij;
 	FILE *fp;			/* File pointer to data or pipe */
 	BOOLEAN check = FALSE;		/* TRUE if nan-proxies are used to signify NaN (for non-floating point types) */
 	BOOLEAN do_header = TRUE;	/* TRUE if we should write the header first */
@@ -1362,17 +1370,21 @@ struct srf_header7 {	/* Surfer 7 file header structure */
 	int len_d;		/* Length in bytes of the DATA section */
 };
 
-int GMT_is_srf_grid (char *file)
+int GMT_is_srf_grid (struct GRD_HEADER *header)
 {
 	FILE *fp;
 	char id[5];
-	if (!strcmp(file, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if ((fp = GMT_fopen (file, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
+	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
+	if ((fp = GMT_fopen (header->name, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
 	if (GMT_fread (id, sizeof (char), (size_t)4, fp) < (size_t)4) return (GMT_GRDIO_READ_FAILED);
 	GMT_fclose (fp); 
-	if (!strncmp (id, "DSBB",4)) return (GMT_grd_format_decoder ("sf"));
-	if (!strncmp (id, "DSRB",4)) return (GMT_grd_format_decoder ("sd"));
-	return (GMT_GRDIO_BAD_VAL);	/* Neither */
+	if (!strncmp (id, "DSBB", (size_t)4))
+		header->type = GMT_grd_format_decoder ("sf");
+	else if (!strncmp (id, "DSRB", (size_t)4))
+		header->type = GMT_grd_format_decoder ("sd");
+	else
+		return (GMT_GRDIO_BAD_VAL);	/* Neither */
+	return (header->type);
 }
 
 int GMT_srf_read_grd_info (struct GRD_HEADER *header)
@@ -1395,9 +1407,9 @@ int GMT_srf_read_grd_info (struct GRD_HEADER *header)
 
 	if (GMT_fread (id, sizeof (char), (size_t)4, fp) < (size_t)4) return (GMT_GRDIO_READ_FAILED); 
 	if (GMT_fseek(fp, 0L, SEEK_SET)) return (GMT_GRDIO_SEEK_FAILED);
-	if (strncmp (id, "DSBB",4) && strncmp (id, "DSRB",4)) return (GMT_GRDIO_NOT_SURFER);
+	if (strncmp (id, "DSBB", (size_t)4) && strncmp (id, "DSRB", (size_t)4)) return (GMT_GRDIO_NOT_SURFER);
 
-	if (!strncmp (id, "DSBB",4)) {		/* Version 6 format */
+	if (!strncmp (id, "DSBB", (size_t)4)) {		/* Version 6 format */
 		if (GMT_read_srfheader6 (fp, &h6)) return (GMT_GRDIO_READ_FAILED);
 		header->type = 6;
 	}
@@ -1500,16 +1512,17 @@ int GMT_srf_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_in;			/* Number of columns in subregion */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
+	GMT_LONG height_in;			/* Number of columns in subregion */
 	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int kk, i, j, j2, ij, i_0_out; 	/* Misc. counters */
+	int i, j, j2, i_0_out; 	/* Misc. counters */
 	int *k;				/* Array with indices */
 	int type;			/* Data type */
-	int size;			/* Length of data type */
+	size_t size;			/* Length of data type */
+	GMT_LONG kk, ij;
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
 	FILE *fp;			/* File pointer to data or pipe */
 	BOOLEAN piping = FALSE;		/* TRUE if we read input pipe instead of from file */
 	void *tmp;			/* Array pointer for reading in rows of data */
@@ -1549,6 +1562,12 @@ int GMT_srf_read_grd (struct GRD_HEADER *header, float *grid, double w, double e
 		i_0_out *= 2;
 		inc = 2;
 	}
+	if ( (last_row - first_row + 1) != header->ny) {    /* We have a sub-region */
+		/* Surfer grids are stored starting from Lower Left, which is contrary to
+		   the rest of GMT grids that start at Top Left. So we must do a shift here */
+		first_row = header->ny - height_in;
+		last_row = first_row + height_in - 1;
+	}
 
 	/* Allocate memory for one row of data (for reading purposes) */
 
@@ -1612,16 +1631,17 @@ int GMT_srf_write_grd (struct GRD_HEADER *header, float *grid, double w, double
 	/*		Note: The file has only real values, we simply allow space in the array */
 	/*		for imaginary parts when processed by grdfft etc. */
 
-	int first_col, last_col;	/* First and last column to deal with */
-	int first_row, last_row;	/* First and last row to deal with */
-	int width_in;			/* Number of items in one row of the subregion */
-	int width_out;			/* Width of row as return (may include padding) */
-	int height_out;			/* Number of columns in subregion */
+	GMT_LONG first_col, last_col;	/* First and last column to deal with */
+	GMT_LONG first_row, last_row;	/* First and last row to deal with */
+	GMT_LONG width_out;			/* Width of row as return (may include padding) */
+	GMT_LONG height_out;			/* Number of columns in subregion */
 	int inc = 1;			/* Step in array: 1 for ordinary data, 2 for complex (skipping imaginary) */
-	int i, j, i2, j2, ij;		/* Misc. counters */
+	int i, j, i2, j2;		/* Misc. counters */
 	int *k;				/* Array with indices */
 	int type;			/* Data type */
-	int size;			/* Length of data type */
+	size_t size;			/* Length of data type */
+	GMT_LONG ij;
+	GMT_LONG width_in;			/* Number of items in one row of the subregion */
 	FILE *fp;			/* File pointer to data or pipe */
 	void *tmp;			/* Array pointer for writing in rows of data */
 	struct srf_header6 h;
