@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_globals.h,v 1.14 2007/01/31 21:51:03 remko Exp $
+ *	$Id: gmt_globals.h,v 1.33 2008/04/15 15:55:29 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -26,12 +26,114 @@
  *
  */
 
-#define GMT_N_UNIQUE 59		/* Number of unique options */
+#ifdef DEBUG
+struct MEMORY_TRACKER *GMT_mem_keeper;
+#endif
 
-struct GMT_COMMON *GMT;			/* Pointer to struct with common option settings */
+struct GMT_CTRL *GMT;			/* Pointer to struct with common option settings */
 
 struct GMT_DEFAULTS gmtdefs = {		/* Initial default values */
-#include "gmt_defaults.h"
+	20.0,			/* ANNOT_MIN_ANGLE */
+	0.0,			/* ANNOT_MIN_SPACING */
+	{0, 0},			/* ANNOT_FONT_PRIMARY, ANNOT_FONT_SECONDARY*/
+	{14, 16},		/* ANNOT_FONT_SIZE_PRIMARY, ANNOT_FONT_SIZE_SECONDARY */
+	{0.075,	0.075},		/* ANNOT_OFFSET_PRIMARY, ANNOT_OFFSET_SECONDARY */
+	"WESN",			/* BASEMAP_AXES */
+	{0, 0, 0},		/* BASEMAP_FRAME_RGB */
+	0,			/* BASEMAP_TYPE */
+	{0, 0, 0},		/* COLOR_BACKGROUND */
+	{255, 255, 255},	/* COLOR_FOREGROUND */
+	{128, 128, 128},	/* COLOR_NAN */
+	0,			/* COLOR_IMAGE */
+	3,			/* COLOR_MODEL */
+	"%g",			/* D_FORMAT */
+	0,			/* DEGREE_FORMAT */
+	300,			/* DOTS_PR_INCH */
+	0,			/* ELLIPSOID */
+	{1.25, 0.0, {0, 0, 0}, ""},	/* FRAME_PEN */
+	0.075,			/* FRAME_WIDTH */
+	1.0,			/* GLOBAL_X_SCALE */
+	1.0,			/* GLOBAL_Y_SCALE */
+	{0.0, 0.0},		/* GRID_CROSS_SIZE P/S */
+	"nf",			/* GRID_FORMAT */
+	{
+	  {0.25, 0.0, {0, 0, 0}, ""},	/* GRID_PEN_PRIMARY */
+	  {0.5,  0.0, {0, 0, 0}, ""}	/* GRID_PEN_SECONDARY */
+	},
+	FALSE,			/* GRIDFILE_SHORTHAND */
+	0,			/* HEADER_FONT */
+	36,			/* HEADER_FONT_SIZE */
+	0.1875,			/* HEADER_OFFSET */
+	1.0,			/* HSV_MIN_SATURATION */
+	0.1,			/* HSV_MAX_SATURATION */
+	0.3,			/* HSV_MIN_VALUE */
+	1.0,			/* HSV_MAX_VALUE */
+	1,			/* INTERPOLANT */
+	{ FALSE, FALSE },	/* IO_HEADER [0|1] */
+	1,			/* N_HEADER_RECS (if -H is set) */
+	0,			/* LABEL_FONT */
+	24,			/* LABEL_FONT_SIZE */
+	0.1125,			/* LABEL_OFFSET */
+	0.01,
+	-1.0,			/* MAP_SCALE_FACTOR */
+	0.075,			/* MAP_SCALE_HEIGHT */
+	1,			/* MEASURE_UNIT set to inch */
+	25,			/* MEDIA set to Letter */
+	1,			/* N_COPIES */
+	1,			/* OBLIQUE_ANNOTATION */
+	{255, 255, 255},	/* PAGE_COLOR */
+	FALSE,			/* PORTRAIT */
+	{612, 792},		/* PAPER_MEDIA (US Letter) */
+	{85.0, 90.0},		/* POLAR_CAP (controls gridlines near poles) */
+	0,			/* PS_COLOR (2 = HSV, 1 = CMYK, 0 = RGB) */
+	0,			/* PS_IMAGE_COMPRESS (0 = NONE, 1 = RLE, 2 = LZW) */
+	TRUE,			/* PS_IMAGE_FORMAT (TRUE = HEX, FALSE = BIN) */
+	0,			/* PS_LINE_CAP (0 = butt, 1 = round, 2 = square) */
+	0,			/* PS_LINE_JOIN (0 = miter, 1 = arc, 2 = bevel) */
+	0,			/* PS_MITER_LIMIT (0 = Default, or 1-180) */
+	FALSE,			/* PS_VERBOSE (TRUE = write comments, FALSE = no comments) */
+	0.075,			/* TICK_LENGTH */
+	{0.5, 0.0, {0, 0, 0}, ""},	/* TICK_PEN */
+	FALSE,			/* UNIX_TIME */
+	1, {-0.75, -0.75},	/* UNIX_TIME_POS */
+	"%Y %b %d %H:%M:%S",	/* UNIX_TIME_FORMAT */
+	0.0,			/* VECTOR_SHAPE */
+	FALSE,			/* VERBOSE */
+	FALSE,			/* WANT_EURO_FONT */
+	9.0,			/* X_AXIS_LENGTH */
+	6.0,			/* Y_AXIS_LENGTH */
+	1.0,			/* X_ORIGIN */
+	1.0,			/* Y_ORIGIN */
+	{FALSE,	FALSE},		/* XY_TOGGLE */
+	0,			/* Y_AXIS_TYPE */
+	{	/* Ellipsoid structure and its members */
+#include "gmt_ellipsoids.h"	/* This is created by GNUmakefile - do not edit it manually */
+	},
+	{	/* Datum structure and its members */
+#include "gmt_datums.h"		/* This is created by GNUmakefile - do not edit it manually */
+	},
+	"hh:mm:ss",		/* INPUT_CLOCK_FORMAT */
+	"yyyy-mm-dd",		/* INPUT_DATE_FORMAT */
+	"hh:mm:ss",		/* OUTPUT_CLOCK_FORMAT */
+	"yyyy-mm-dd",		/* OUTPUT_DATE_FORMAT */
+	"+D",			/* OUTPUT_DEGREE_FORMAT */
+	"hh:mm:ss",		/* PLOT_CLOCK_FORMAT */
+	"yyyy-mm-dd",		/* PLOT_DATE_FORMAT */
+	"+ddd:mm:ss",		/* PLOT_DEGREE_FORMAT */
+	{ "full", "full" },	/* TIME_FORMAT_PRIMARY, TIME_FORMAT_SECONDARY */
+	FALSE,			/* TIME_IS_INTERVAL */
+	0.5,			/* TIME_INTERVAL_FRACTION */
+	FALSE,			/* WANT_LEAP_SECONDS */
+	{ "2000-01-01T12:00:00", 'd',		/* TIME_EPOCH, TIME_UNIT (J2000) */
+	730120, 0.5, 86400.0, 1.0/86400.0 },	/* rata_die, epoch_t0, scale, i_scale based on the above */
+	0,			/* TIME_WEEK_START */
+	"us",			/* TIME_LANGUAGE */
+	1950,			/* Y2K_OFFSET_YEAR */
+	"\t",			/* FIELD_DELIMITER */
+	gmt_ring,		/* DEGREE_SYMBOL */
+	{ "Standard", 		/* CHAR_ENCODING */
+	  { 32, 32, 32, 32, 32 } }, /* PostScript codes for degree, ring, colon, singlequote, and doublequote [Initialized to space] */
+	TRUE,			/* HISTORY */
 };
 
 struct GMT_FONT *GMT_font;		/* Name and height of fonts recognized by GMT */
@@ -41,11 +143,6 @@ struct GMT_HASH GMT_rgb_hashnode[GMT_N_COLOR_NAMES];/* Used to translate colorna
 
 struct GMT_HASH GMT_month_hashnode[12];		/* Used to translate months to 1-12 */
 
-#define GMT_N_SYSTEMS 8		/* Number of time systems in gmt_time_systems.h */
-struct GMT_TIME_SYSTEM GMT_time_system[GMT_N_SYSTEMS] = {	/* List of all used time systems */
-#include "gmt_time_systems.h"
-};
-
 /*--------------------------------------------------------------------*/
 /*	For plotting purposes */
 /*--------------------------------------------------------------------*/
@@ -57,8 +154,8 @@ struct GMT_TRUNCATE_TIME GMT_truncate_time;
 double *GMT_x_plot = 0;				/* Holds the x/y (inches) of a line to be plotted */
 double *GMT_y_plot = 0;
 int *GMT_pen = 0;				/* Pen (3 = up, 2 = down) for these points */
-int GMT_n_plot = 0;				/* Number of such points */
-int GMT_n_alloc = 0;				/* Size of allocated plot arrays */
+GMT_LONG GMT_n_plot = 0;				/* Number of such points */
+GMT_LONG GMT_n_alloc = 0;				/* Size of allocated plot arrays */
 int GMT_x_status_new;				/* Tells us what quadrant old and new points are in */
 int GMT_y_status_new;
 int GMT_x_status_old;
@@ -88,8 +185,8 @@ BOOLEAN GMT_meridian_straight = FALSE;		/* TRUE if meridians plot as straight li
 BOOLEAN GMT_parallel_straight = FALSE;		/* TRUE if parallels plot as straight lines */
 int GMT_3D_mode = 3;				/* Determines if we draw fore and/or back 3-D box lines [Default is both] */
 char *GMT_plot_format[3][2];			/* Keeps the 6 formats for dd:mm:ss plot output */
-int GMT_n_lon_nodes = 360;			/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
-int GMT_n_lat_nodes = 180;			/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
+GMT_LONG GMT_n_lon_nodes = 360;			/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
+GMT_LONG GMT_n_lat_nodes = 180;			/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
 double GMT_dlon = 0.0;				/* Steps taken in longitude along gridlines (gets reset in gmt_init.c) */
 double GMT_dlat = 0.0;				/* Steps taken in latitude along gridlines (gets reset in gmt_init.c) */
 /*--------------------------------------------------------------------*/
@@ -105,7 +202,9 @@ BOOLEAN GMT_b_and_w;			/* TRUE if only black and white are needed */
 BOOLEAN GMT_continuous;			/* TRUE if continuous color tables have been given */
 BOOLEAN GMT_cpt_pattern = FALSE;	/* TRUE if cpt file contains any patterns */
 BOOLEAN GMT_cpt_skip = FALSE;		/* TRUE if current z-slice is to be skipped */
-
+#ifdef GMT_CPT2	
+BOOLEAN GMT_categorical = FALSE;	/* TRUE if CPT applies to categorical data */
+#endif
 /*--------------------------------------------------------------------*/
 /*	For projection purposes */
 /*--------------------------------------------------------------------*/
@@ -134,10 +233,11 @@ struct GMT_IO GMT_io = {
 	{ FALSE, FALSE },
 	{ FALSE, FALSE },
 	{ FALSE, FALSE },
+	{ FALSE, FALSE },
 	FALSE,
 	FALSE,
-	{ 0, 0 },
 	0,
+	{ 0, 0 },
 	-1,
 	-1,
 	0,
@@ -147,12 +247,21 @@ struct GMT_IO GMT_io = {
 	{ '>', '>'},
 	"",
 	"",
+	"",
 	"r",
 	"w",
 	"a+",
 	(BOOLEAN *)NULL,
 	(int *)NULL,
 	(int *)NULL,
+	0,
+	0,
+	0,
+	0,
+	(int *)NULL,
+	(double *)NULL,
+	(double *)NULL,
+	(double *)NULL,
 	{ {-1, -1, -1, -1}, {-1, -1, -1, -1}, FALSE, FALSE, "", FALSE, FALSE, FALSE, FALSE, { "", ""} },
 	{ {-1, -1, -1, -1}, {-1, -1, -1, -1}, FALSE, FALSE, "", FALSE, FALSE, FALSE, FALSE, { "", ""} },
 	{ {-1, -1, -1}, 0, 0.0, FALSE, FALSE, { "am", "pm" }, "", { "", ""} },
@@ -184,6 +293,7 @@ char *GMT_USERDIR = CNULL;
 char *GMT_DATADIR = CNULL;
 char *GMT_GRIDDIR = CNULL;
 char *GMT_IMGDIR = CNULL;
+char *GMT_TMPDIR = CNULL;
 char *GMT_program;	/* Name of current GMT program */
 
 float GMT_f_NaN;
