@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_grdio.c,v 1.97 2007/03/24 01:42:06 pwessel Exp $
+ *	$Id: gmt_grdio.c,v 1.116 2008/04/07 18:48:05 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
  * Date:	9-SEP-1992
  * Modified:	06-DEC-2001
  * Version:	4
+ * 64-bit Compliant: Yes
  *
  * Functions include:
  *
@@ -66,15 +67,15 @@ int GMT_grdformats[GMT_N_GRD_FORMATS][2] = {
 #include "gmt_grdformats.h"
 };
 
-void GMT_grd_do_scaling (float *grid, int nm, double scale, double offset);
+void GMT_grd_do_scaling (float *grid, GMT_LONG nm, double scale, double offset);
 void GMT_grd_get_units (struct GRD_HEADER *header);
 void GMT_grd_set_units (struct GRD_HEADER *header);
-int GMT_is_nc_grid (char *file);
-int GMT_is_native_grid (char *file);
-int GMT_is_ras_grid (char *file);
-int GMT_is_srf_grid (char *file);
-int GMT_is_mgg2_grid (char *file);
-int GMT_is_agc_grid (char *file);
+int GMT_is_nc_grid (struct GRD_HEADER *header);
+int GMT_is_native_grid (struct GRD_HEADER *header);
+int GMT_is_ras_grid (struct GRD_HEADER *header);
+int GMT_is_srf_grid (struct GRD_HEADER *header);
+int GMT_is_mgg2_grid (struct GRD_HEADER *header);
+int GMT_is_agc_grid (struct GRD_HEADER *header);
 
 /* GENERIC I/O FUNCTIONS FOR GRIDDED DATA FILES */
 
@@ -151,10 +152,12 @@ int GMT_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 	 */
 
 	int err;
+	GMT_LONG nm;
 
 	GMT_err_trap ((*GMT_io_readgrd[header->type]) (header, grid, w, e, s, n, pad, complex));
 	if (header->z_scale_factor == 0.0) fprintf (stderr, "GMT Warning: scale_factor should not be 0.\n");
-	GMT_grd_do_scaling (grid, ((header->nx + pad[0] + pad[1]) * (header->ny + pad[2] + pad[3])), header->z_scale_factor, header->z_add_offset);
+	nm = ((GMT_LONG)(header->nx + pad[0] + pad[1])) * ((GMT_LONG)(header->ny + pad[2] + pad[3]));
+	GMT_grd_do_scaling (grid, nm, header->z_scale_factor, header->z_add_offset);
 	header->z_min = header->z_min * header->z_scale_factor + header->z_add_offset;
 	header->z_max = header->z_max * header->z_scale_factor + header->z_add_offset;
 	header->xy_off = 0.5 * header->node_offset;
@@ -173,6 +176,7 @@ int GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 	 */
 
 	int err;
+	GMT_LONG nm;
 
 	GMT_err_trap (GMT_grd_get_format (file, header, FALSE));
 	if (GMT_is_dnan(header->z_scale_factor))
@@ -182,7 +186,8 @@ int GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 		fprintf (stderr, "GMT Warning: scale_factor should not be 0. Reset to 1.\n");
 	}
 	GMT_grd_set_units (header);
-	GMT_grd_do_scaling (grid, (header->nx * header->ny), 1.0/header->z_scale_factor, -header->z_add_offset/header->z_scale_factor);
+	nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
+	GMT_grd_do_scaling (grid, nm, 1.0/header->z_scale_factor, -header->z_add_offset/header->z_scale_factor);
 	return ((*GMT_io_writegrd[header->type]) (header, grid, w, e, s, n, pad, complex));
 }
 
@@ -199,7 +204,7 @@ void GMT_expand_filename (char *file, char *fname)
 			start = f_length - length;
 			found = (start < 0) ? FALSE : !strncmp (&file[start], GMT_file_suffix[i], (size_t)length);
 		}
-		if (found) {
+		if (found) {	/* file ended in a recognized shorthand extension */
 			i--;
 			sprintf (fname, "%s=%d/%g/%g/%g", file, GMT_file_id[i], GMT_file_scale[i], GMT_file_offset[i], GMT_file_nan[i]);
 		}
@@ -212,17 +217,31 @@ void GMT_expand_filename (char *file, char *fname)
 
 int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 {
+	/* This functions does a couple of things:
+	 * 1. It tries to determine what kind of grid file this is. If a file is openeed for
+	 *    reading we see if (a) a particular format has been specified with
+	 *    the =<code> suffix, or (b) we are able to guess the format based on known
+	 *    characteristics of various formats, or (c) assume the default grid format.
+	 *    If a file is opened for writing, only option (a) and (c) apply.
+	 *    If we cannot obtain the format we return an error.
+	 * 2. We strip the suffix off. The relevant info is stored in the header struct.
+	 * 3. In case of netCDF grids, the optional ?<varname> is stripped off as well.
+	 *    The info is stored in header->varname.
+	 * 4. If a file is open for reading, we set header->name to the full path of the file
+	 *    by seaching in current dir and the various GMT_*DIR paths.
+	 */
+	
 	int i = 0, val, j;
-	char code[GMT_TEXT_LEN];
+	char code[GMT_TEXT_LEN], tmp[BUFSIZ];
 
-	GMT_expand_filename (file, header->name);
+	GMT_expand_filename (file, header->name);	/* May append a suffix to header->name */
 
 	/* Set default values */
 	header->z_scale_factor = GMT_d_NaN, header->z_add_offset = 0.0, header->nan_value = GMT_d_NaN;
 
 	while (header->name[i] && header->name[i] != '=') i++;
 
-	if (header->name[i]) {	/* Get format type, scale, offset and missing value from suffix */
+	if (header->name[i]) {	/* Reading or writing when =suffix is present: get format type, scale, offset and missing value */
 		i++;
 		sscanf (&header->name[i], "%[^/]/%lf/%lf/%lf", code, &header->z_scale_factor, &header->z_add_offset, &header->nan_value);
 		val = GMT_grd_format_decoder (code);
@@ -230,30 +249,40 @@ int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 		header->type = val;
 		j = (i == 1) ? i : i - 1;
 		header->name[j] = 0;
+		sscanf (header->name, "%[^?]?%s", tmp, header->varname);    /* Strip off variable name */
+		if (magic) {	/* Reading: possibly prepend a path from GMT_[GRID|DATA|IMG]DIR */
+			if (!GMT_getdatapath (tmp, header->name)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Reading: possily prepended a path from GMT_[GRID|DATA|IMG]DIR */
+		}
+		else		/* Writing: store truncated pathname */
+			strcpy (header->name, tmp);
 	}
-	else if (magic) {	/* Determine file format automatically based on grid content */
+	else if (magic) {	/* Reading: determine file format automatically based on grid content */
+		sscanf (header->name, "%[^?]?%s", tmp, header->varname);    /* Strip off variable name */
+		if (!GMT_getdatapath (tmp, header->name)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Possily prepended a path from GMT_[GRID|DATA|IMG]DIR */
 		/* First check if we have a netCDF grid. This MUST be first, because ?var needs to be stripped off. */
-		if ((header->type = GMT_is_nc_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if ((val = GMT_is_nc_grid (header)) >= 0) return (GMT_NOERROR);
 		/* Continue only when file was a pipe or when nc_open didn't like the file. */
-		if (header->type != GMT_GRDIO_NC_NO_PIPE && header->type != GMT_GRDIO_OPEN_FAILED) return (header->type);
+		if (val != GMT_GRDIO_NC_NO_PIPE && val != GMT_GRDIO_OPEN_FAILED) return (val);
 		/* Then check for native GMT grid */
-		if ((header->type = GMT_is_native_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if (GMT_is_native_grid (header) >= 0) return (GMT_NOERROR);
 		/* Next check for Sun raster GMT grid */
-		if ((header->type = GMT_is_ras_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if (GMT_is_ras_grid (header) >= 0) return (GMT_NOERROR);
 		/* Then check for Golden Software surfer grid */
-		if ((header->type = GMT_is_srf_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if (GMT_is_srf_grid (header) >= 0) return (GMT_NOERROR);
 		/* Then check for NGDC GRD98 GMT grid */
-		if ((header->type = GMT_is_mgg2_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if (GMT_is_mgg2_grid (header) >= 0) return (GMT_NOERROR);
 		/* Then check for AGC GMT grid */
-		if ((header->type = GMT_is_agc_grid (header->name)) >= 0) return (GMT_NOERROR);
+		if (GMT_is_agc_grid (header) >= 0) return (GMT_NOERROR);
 		return (GMT_GRDIO_UNKNOWN_FORMAT);	/* No supported format found */
 	}
-	else {			/* Get format type, scale, offset and missing value from gmtdefs.grid_format */
+	else {			/* Writing: get format type, scale, offset and missing value from gmtdefs.grid_format */
+		if (sscanf (header->name, "%[^?]?%s", tmp, header->varname) > 1) strcpy (header->name, tmp);    /* Strip off variable name */
 		sscanf (gmtdefs.grid_format, "%[^/]/%lf/%lf/%lf", code, &header->z_scale_factor, &header->z_add_offset, &header->nan_value);
 		val = GMT_grd_format_decoder (code);
 		if (val < 0) return (val);
 		header->type = val;
 	}
+	if (header->type == GMT_grd_format_decoder ("af")) header->nan_value = 0.0;	/* 0 is NaN in the AGC format */
 	return (GMT_NOERROR);
 }
 
@@ -282,7 +311,7 @@ int GMT_grd_data_size (int format, double *nan_value)
 			return (sizeof(double));
 			break;
 		default:
-			fprintf (stderr, "Unknown grid data type: %c\n", GMT_grdformats[format][1]);
+			fprintf (stderr, "Unknown grid data type: %c\n", (int)GMT_grdformats[format][1]);
 			return (GMT_GRDIO_UNKNOWN_TYPE);
 	}
 }
@@ -312,10 +341,10 @@ int GMT_grd_format_decoder (const char *code)
 	return (id);
 }
 
-void GMT_grd_do_scaling (float *grid, int nm, double scale, double offset)
+void GMT_grd_do_scaling (float *grid, GMT_LONG nm, double scale, double offset)
 {
 	/* Routine that scales and offsets the data if specified */
-	int i;
+	GMT_LONG i;
 
 	if (scale == 1.0 && offset == 0.0) return;
 
@@ -378,7 +407,7 @@ int GMT_grd_RI_verify (struct GRD_HEADER *h, int mode)
 	return (GMT_NOERROR);
 }
 
-int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s, double *n, int *width, int *height, int *first_col, int *last_col, int *first_row, int *last_row, int **index)
+int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s, double *n, GMT_LONG *width, GMT_LONG *height, GMT_LONG *first_col, GMT_LONG *last_col, GMT_LONG *first_row, GMT_LONG *last_row, int **index)
 {
 	/* Determines which rows and columns to extract, and if it is
 	 * a grid that is periodic and wraps around and returns indices. */
@@ -419,10 +448,10 @@ int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s,
 
 		/* Get first and last row and column numbers */
 
-		*first_col = (int)floor ((*w - header->x_min) / header->x_inc + small);
-		*last_col  = (int)ceil  ((*e - header->x_min) / header->x_inc - small) - 1 + one_or_zero;
-		*first_row = (int)floor ((header->y_max - *n) / header->y_inc + small);
-		*last_row  = (int)ceil  ((header->y_max - *s) / header->y_inc - small) - 1 + one_or_zero;
+		*first_col = (GMT_LONG)floor ((*w - header->x_min) / header->x_inc + small);
+		*last_col  = (GMT_LONG)ceil  ((*e - header->x_min) / header->x_inc - small) - 1 + one_or_zero;
+		*first_row = (GMT_LONG)floor ((header->y_max - *n) / header->y_inc + small);
+		*last_row  = (GMT_LONG)ceil  ((header->y_max - *s) / header->y_inc - small) - 1 + one_or_zero;
 
 		if ((*last_col - *first_col + 1) > *width) (*last_col)--;
 		if ((*last_row - *first_row + 1) > *height) (*last_row)--;
@@ -461,35 +490,36 @@ void GMT_decode_grd_h_info (char *input, struct GRD_HEADER *h) {
 	This routine is usually called if -D<input> was given by user,
 	and after GMT_grd_init() has been called.
 */
-	char	ptr[BUFSIZ], sep = '/';
-	int	entry = 0, pos = 0;
+	char ptr[BUFSIZ], sep[] = "/";
+	int entry = 0, pos = 0;
 
 	if (input[0] != input[strlen(input)-1]) {}
+	else if (input[0] == '=') {}
 	else if (input[0] >= 'A' && input[0] <= 'Z') {}
-	else if (input[0] >= 'a' && input[0] <= 'a') {}
+	else if (input[0] >= 'a' && input[0] <= 'z') {}
 	else if (input[0] >= '0' && input[0] <= '9') {}
 	else {
-		sep = input[0];
+		sep[0] = input[0];
 		pos = 1;
 	}
 
-	while ((GMT_strtok (input, &sep, &pos, ptr))) {
+	while ((GMT_strtok (input, sep, &pos, ptr))) {
 		if (ptr[0] != '=') {
 			switch (entry) {
 				case 0:
 					memset ( (void *)h->x_units, 0, (size_t)GRD_UNIT_LEN);
 					if (strlen(ptr) >= GRD_UNIT_LEN) fprintf (stderr, "%s: GMT WARNING: X unit string exceeds upper length of %d characters (truncated)\n", GMT_program, GRD_UNIT_LEN);
-					strncpy (h->x_units, ptr, GRD_UNIT_LEN);
+					strncpy (h->x_units, ptr, (size_t)GRD_UNIT_LEN);
 					break;
 				case 1:
 					memset ( (void *)h->y_units, 0, (size_t)GRD_UNIT_LEN);
 					if (strlen(ptr) >= GRD_UNIT_LEN) fprintf (stderr, "%s: GMT WARNING: Y unit string exceeds upper length of %d characters (truncated)\n", GMT_program, GRD_UNIT_LEN);
-					strncpy (h->y_units, ptr, GRD_UNIT_LEN);
+					strncpy (h->y_units, ptr, (size_t)GRD_UNIT_LEN);
 					break;
 				case 2:
 					memset ( (void *)h->z_units, 0, (size_t)GRD_UNIT_LEN);
 					if (strlen(ptr) >= GRD_UNIT_LEN) fprintf (stderr, "%s: GMT WARNING: Z unit string exceeds upper length of %d characters (truncated)\n", GMT_program, GRD_UNIT_LEN);
-					strncpy (h->z_units, ptr, GRD_UNIT_LEN);
+					strncpy (h->z_units, ptr, (size_t)GRD_UNIT_LEN);
 					break;
 				case 3:
 					h->z_scale_factor = atof (ptr);
@@ -499,11 +529,11 @@ void GMT_decode_grd_h_info (char *input, struct GRD_HEADER *h) {
 					break;
 				case 5:
 					if (strlen(ptr) >= GRD_TITLE_LEN) fprintf (stderr, "%s: GMT WARNING: Title string exceeds upper length of %d characters (truncated)\n", GMT_program, GRD_TITLE_LEN);
-					strncpy (h->title, ptr, GRD_TITLE_LEN);
+					strncpy (h->title, ptr, (size_t)GRD_TITLE_LEN);
 					break;
 				case 6:
 					if (strlen(ptr) >= GRD_REMARK_LEN) fprintf (stderr, "%s: GMT WARNING: Remark string exceeds upper length of %d characters (truncated)\n", GMT_program, GRD_REMARK_LEN);
-					strncpy (h->remark, ptr, GRD_REMARK_LEN);
+					strncpy (h->remark, ptr, (size_t)GRD_REMARK_LEN);
 					break;
 				default:
 					break;
@@ -579,7 +609,7 @@ int GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
 	else	/* All other */
 		G->n_byte = G->header.nx * G->size;
 
-	G->v_row = (void *) GMT_memory (VNULL, G->n_byte, 1, GMT_program);
+	G->v_row = (void *) GMT_memory (VNULL, (size_t)G->n_byte, (size_t)1, GMT_program);
 
 	G->row = 0;
 	G->auto_advance = TRUE;	/* Default is to read sequential rows */
@@ -588,6 +618,7 @@ int GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
 
 void GMT_close_grd (struct GMT_GRDFILE *G)
 {
+	GMT_free ((void *)G->v_row);
 	if (GMT_grdformats[G->header.type][0] == 'c' || GMT_grdformats[G->header.type][0] == 'n')
 		nc_close (G->fid);
 	else
@@ -628,13 +659,13 @@ int GMT_read_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
 		}
 		if (!G->auto_advance && GMT_fseek (G->fp, (long)(GRD_HEADER_SIZE + G->row * G->n_byte), SEEK_SET)) return (GMT_GRDIO_SEEK_FAILED);
 
-		if (GMT_fread (G->v_row, G->size, (size_t)G->header.nx, G->fp) != (size_t)G->header.nx)  return (GMT_GRDIO_READ_FAILED);	/* Get one row */
+		if (GMT_fread (G->v_row, (size_t)G->size, (size_t)G->header.nx, G->fp) != (size_t)G->header.nx)  return (GMT_GRDIO_READ_FAILED);	/* Get one row */
 		for (i = 0; i < G->header.nx; i++) {
 			row[i] = GMT_decode (G->v_row, i, GMT_grdformats[G->header.type][1]);	/* Convert whatever to float */
 			if (G->check && row[i] == G->header.nan_value) row[i] = GMT_f_NaN;
 		}
 	}
-	GMT_grd_do_scaling (row, G->header.nx, G->scale, G->offset);
+	GMT_grd_do_scaling (row, (GMT_LONG)G->header.nx, G->scale, G->offset);
 	G->row++;
 	return (GMT_NOERROR);
 }
@@ -647,9 +678,9 @@ int GMT_write_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
 
 	size = GMT_grd_data_size (G->header.type, &G->header.nan_value);
 
-	tmp = (void *) GMT_memory (VNULL, (size_t)G->header.nx, size, "GMT_write_grd_row");
+	tmp = (void *) GMT_memory (VNULL, (size_t)G->header.nx, (size_t)size, "GMT_write_grd_row");
 
-	GMT_grd_do_scaling (row, G->header.nx, G->scale, G->offset);
+	GMT_grd_do_scaling (row, (GMT_LONG)G->header.nx, G->scale, G->offset);
 	for (i = 0; i < G->header.nx; i++) if (GMT_is_fnan (row[i]) && G->check) row[i] = (float)G->header.nan_value;
 
 	switch (GMT_grdformats[G->header.type][0]) {
@@ -670,16 +701,30 @@ int GMT_write_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
 	return (GMT_NOERROR);
 }
 
-
 void GMT_grd_init (struct GRD_HEADER *header, int argc, char **argv, BOOLEAN update)
 {	/* GMT_grd_init initializes a grd header to default values and copies the
 	 * command line to the header variable command.
 	 * update = TRUE if we only want to update command line */
 	int i, len;
 
+	if (update)	/* Only clean the command history */
+		memset ((void *)header->command, 0, (size_t)GRD_COMMAND_LEN);
+	else {		/* Wipe the slate clean */
+		memset ((void *)header, 0, (size_t)sizeof(struct GRD_HEADER));
+
+		/* Set the variables that are not initialized to 0/FALSE/NULL */
+		header->z_scale_factor		= 1.0;
+		header->type			= -1;
+		header->y_order			= 1;
+		header->z_id			= -1;
+		header->nan_value		= GMT_d_NaN;
+		strcpy (header->x_units, "x");
+		strcpy (header->y_units, "y");
+		strcpy (header->z_units, "z");
+	}
+
 	/* Always update command line history */
 
-	memset ((void *)header->command, 0, (size_t)GRD_COMMAND_LEN);
 	if (argc > 0) {
 		strcpy (header->command, argv[0]);
 		len = strlen (header->command);
@@ -691,35 +736,6 @@ void GMT_grd_init (struct GRD_HEADER *header, int argc, char **argv, BOOLEAN upd
 		}
 		header->command[len] = 0;
 	}
-
-	if (update) return;	/* Leave other variables unchanged */
-
-	/* Here we initialize the variables to default settings */
-
-	header->x_min = header->x_max	= 0.0;
-	header->y_min = header->y_max	= 0.0;
-	header->z_min = header->z_max	= 0.0;
-	header->x_inc = header->y_inc	= 0.0;
-	header->z_scale_factor		= 1.0;
-	header->z_add_offset		= 0.0;
-	header->nx = header->ny		= 0;
-	header->node_offset		= 0;
-	header->type			= -1;
-	header->y_order			= 1;
-	header->z_id			= -1;
-	header->nan_value		= GMT_d_NaN;
-	header->xy_off			= 0.0;
-
-	memset ((void *)header->name, 0, (size_t)GMT_LONG_TEXT);
-
-	memset ((void *)header->x_units, 0, (size_t)GRD_UNIT_LEN);
-	memset ((void *)header->y_units, 0, (size_t)GRD_UNIT_LEN);
-	memset ((void *)header->z_units, 0, (size_t)GRD_UNIT_LEN);
-	strcpy (header->x_units, "x");
-	strcpy (header->y_units, "y");
-	strcpy (header->z_units, "z");
-	memset ((void *)header->title, 0, (size_t)GRD_TITLE_LEN);
-	memset ((void *)header->remark, 0, (size_t)GRD_REMARK_LEN);
 }
 
 void GMT_grd_shift (struct GRD_HEADER *header, float *grd, double shift)
@@ -727,7 +743,7 @@ void GMT_grd_shift (struct GRD_HEADER *header, float *grd, double shift)
 	/* Rotate geographical, global grid in e-w direction
 	 * This function will shift a grid by shift degrees */
 
-	int i, j, k, ij, nc, nx1, n_shift, width, n_warn = 0;
+	GMT_LONG i, j, k, ij, nc, nx1, n_shift, width, n_warn = 0;
 	float *tmp;
 
 	tmp = (float *) GMT_memory (VNULL, (size_t)header->nx, sizeof (float), "GMT_grd_shift");
@@ -761,7 +777,7 @@ void GMT_grd_shift (struct GRD_HEADER *header, float *grd, double shift)
 		header->x_max -= 360.0;
 	}
 
-	if (n_warn) fprintf (stderr, "%s: Gridline-registered global grid has inconsistant values at repeated node for %d rows\n", GMT_program, n_warn);
+	if (n_warn) fprintf (stderr, "%s: Gridline-registered global grid has inconsistant values at repeated node for %ld rows\n", GMT_program, n_warn);
 
 	GMT_free ((void *) tmp);
 }
@@ -923,9 +939,9 @@ int GMT_adjust_loose_wesn (double *w, double *e, double *s, double *n, struct GR
 			break;
 	}
 	global = GMT_grd_is_global (header);
-	if (GMT_io.in_col_type[0] != GMT_IS_LON || fabs (fabs (*e - *w) - 360.0) > GMT_CONV_LIMIT) {	/* Do this unless a 360 longitude wrap */
+	if (GMT_io.in_col_type[0] != GMT_IS_LON || GMT_360_RANGE (*w, *e)) {	/* Do this unless a 360 longitude wrap */
 		small = GMT_SMALL * header->x_inc;
-		start = (GMT_io.in_col_type[0] == GMT_IS_LON && fabs (fabs (header->x_min - *w) - 360.0) < GMT_CONV_LIMIT) ? *w : header->x_min;
+		start = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (*w, header->x_min)) ? *w : header->x_min;
 
 		i_d = (*w - header->x_min) / header->x_inc;
 		if (i_d < 0 && !global) i_d = 0;
@@ -1021,14 +1037,14 @@ void GMT_grd_set_units (struct GRD_HEADER *header)
 	for (i = 0; i < 3; i++) {
 		switch (GMT_io.out_col_type[i]) {
 		case GMT_IS_LON:
-			strcpy (string[i], "Longitude [degrees_east]"); break;
+			strcpy (string[i], "longitude [degrees_east]"); break;
 		case GMT_IS_LAT:
-			strcpy (string[i], "Latitude [degrees_north]"); break;
+			strcpy (string[i], "latitude [degrees_north]"); break;
 		case GMT_IS_ABSTIME:
 		case GMT_IS_RELTIME:
 		case GMT_IS_RATIME:
 			/* Determine time unit */
-			switch (GMT_time_system[gmtdefs.time_system].unit) {
+			switch (gmtdefs.time_system.unit) {
 			case 'y':
 				strcpy (unit, "years"); break;
 			case 'o':
@@ -1043,7 +1059,7 @@ void GMT_grd_set_units (struct GRD_HEADER *header)
 				strcpy (unit, "seconds"); break;
 			}
 			GMT_format_calendar (date, clock, &GMT_io.date_output, &GMT_io.clock_output, FALSE, 1, 0.0);
-			sprintf (string[i], "Time [%s since %s %s]", unit, date, clock);
+			sprintf (string[i], "time [%s since %s %s]", unit, date, clock);
 			/* Warning for non-double grids */
 			if (i == 2 && GMT_grdformats[header->type][1] != 'd') fprintf (stderr, "%s: Warning: Use double precision output grid to avoid loss of significance of time coordinate.\n", GMT_program);
 			break;
@@ -1058,8 +1074,9 @@ void GMT_grd_get_units (struct GRD_HEADER *header)
 	   When "Time": transform the data scale and offset to match the current time system.
 	*/
 	int i;
-	char string[3][GRD_UNIT_LEN], *cal, *l;
+	char string[3][GRD_UNIT_LEN], *units;
 	double scale = 1.0, offset = 0.0;
+	struct GMT_TIME_SYSTEM time_system;
 
 	/* Copy unit strings */
 	strcpy (string[0], header->x_units);
@@ -1081,43 +1098,32 @@ void GMT_grd_get_units (struct GRD_HEADER *header)
 		/* Change name of variable and unit to lower case for comparison */
 		GMT_str_tolower (string[i]);
 
-		if (!strncmp (string[i], "lon", 3) || strstr (string[i], "degrees_e")) {
+		if (!strncmp (string[i], "longitude", (size_t)9) || strstr (string[i], "degrees_e")) {
 			/* Input data type is longitude */
 			GMT_io.in_col_type[i] = GMT_IS_LON;
 			project_info.degree[i] = TRUE;
 		}
-		else if (!strncmp (string[i], "lat", 3) || strstr (string[i], "degrees_n")) {
+		else if (!strncmp (string[i], "latitude", (size_t)8) || strstr (string[i], "degrees_n")) {
 			/* Input data type is latitude */
 			GMT_io.in_col_type[i] = GMT_IS_LAT;
 			project_info.degree[i] = TRUE;
 		}
-		else if (!strncmp (string[i], "time", 4)) {
+		else if (!strcmp (string[i], "time") || !strncmp (string[i], "time [", (size_t)6)) {
 			/* Input data type is time */
 			GMT_io.in_col_type[i] = GMT_IS_RELTIME;
 			project_info.xyz_projection[i] = GMT_TIME;
-			/* Determine relative time units */
-			if (strstr (string[i], "years"))
-				scale = GMT_YR2SEC_F / GMT_time_system[gmtdefs.time_system].scale;
-			else if (strstr (string[i], "months"))
-				scale = GMT_MON2SEC_F / GMT_time_system[gmtdefs.time_system].scale;
-			else if (strstr (string[i], "days"))
-				scale = GMT_DAY2SEC_F / GMT_time_system[gmtdefs.time_system].scale;
-			else if (strstr (string[i], "hours"))
-				scale = GMT_HR2SEC_F / GMT_time_system[gmtdefs.time_system].scale;
-			else if (strstr (string[i], "minutes"))
-				scale = GMT_MIN2SEC_F / GMT_time_system[gmtdefs.time_system].scale;
-			else if (strstr (string[i], "seconds"))
-				scale = 1.0 / GMT_time_system[gmtdefs.time_system].scale;
-			else
-				fprintf (stderr, "%s: Warning: Time unit in grid not recognised; assumed %c.\n", GMT_program, GMT_time_system[gmtdefs.time_system].unit);
-			/* Determine relative time epoch */
-			if ((l = strstr (string[i], "since"))) {
-				cal = l + 6;
-				if ((l = strchr (cal, ' '))) *l = 'T';
-				if (GMT_scanf (cal, GMT_IS_ABSTIME, &offset) == GMT_IS_NAN) fprintf (stderr, "%s: Warning: Epoch in grid not recognised; assumed %s.\n", GMT_program, GMT_time_system[gmtdefs.time_system].epoch);
-			}
-			else
-				fprintf (stderr, "%s: Warning: No epoch for time in grid specified; assumed %s.\n", GMT_program, GMT_time_system[gmtdefs.time_system].epoch);
+
+			/* Determine coordinates epoch and units (default is internal system) */
+			memcpy ((void *)&time_system, (void *)&gmtdefs.time_system, sizeof (struct GMT_TIME_SYSTEM));
+			units = strchr (string[i], '[') + 1;
+			if (!units || GMT_get_time_system (units, &time_system) || GMT_init_time_system_structure (&time_system))
+				fprintf (stderr, "%s: Warning: Time units [%s] in grid not recognised, defaulting to gmtdefaults.\n", GMT_program, units);
+
+			/* Determine scale between grid and internal time system, as well as the offset (in internal units) */
+			scale = time_system.scale * gmtdefs.time_system.i_scale;
+			offset = (time_system.rata_die - gmtdefs.time_system.rata_die) + (time_system.epoch_t0 - gmtdefs.time_system.epoch_t0);
+			offset *= GMT_DAY2SEC_F * gmtdefs.time_system.i_scale;
+
 			/* Scale data scale and extremes based on scale and offset */
 			if (i == 0) {
 				header->x_min = header->x_min * scale + offset;
@@ -1158,7 +1164,7 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 	 * if we are dealing with standard 72 or 80 img latitude; else it must be specified.
 	 */
 
-	int min, i, j, k, ij, mx, my, first_i, n_skip, n_cols;
+	GMT_LONG min, i, j, k, ij, mx, my, first_i, n_skip, n_cols;
 	short int *i2;
 	char file[BUFSIZ];
 	struct STAT buf;
@@ -1183,7 +1189,7 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 		default:
 			if (lat == 0.0) return (GMT_GRDIO_BAD_IMG_LAT);
 			min = (buf.st_size > GMT_IMG_NLON_2M*GMT_IMG_NLAT_2M_80*GMT_IMG_ITEMSIZE) ? 1 : 2;
-			fprintf (stderr, "%s: img file %s has unusual size - grid increment defaults to %d min\n", GMT_program, file, min);
+			fprintf (stderr, "%s: img file %s has unusual size - grid increment defaults to %ld min\n", GMT_program, file, min);
 			break;
 	}
 
@@ -1200,8 +1206,9 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 		/* Select plain Mercator on a sphere with -Jm1 -R0/360/-lat/+lat */
 		gmtdefs.ellipsoid = GMT_N_ELLIPSOIDS - 1;
 		project_info.units_pr_degree = TRUE;
-		project_info.m_got_parallel = FALSE;
-		project_info.pars[0] = 1.0;
+		project_info.pars[0] = 180.0;
+		project_info.pars[1] = 0.0;
+		project_info.pars[2] = 1.0;
 		project_info.projection = GMT_MERCATOR;
 		project_info.degree[0] = project_info.degree[1] = TRUE;
 
@@ -1228,15 +1235,15 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 	grd->xy_off = 0.5;
 
 	n_cols = (min == 1) ? GMT_IMG_NLON_1M : GMT_IMG_NLON_2M;		/* Number of columns (10800 or 21600) */
-	first_i = (int)floor (grd->x_min / grd->x_inc);				/* first tile partly or fully inside region */
+	first_i = (GMT_LONG)floor (grd->x_min / grd->x_inc);				/* first tile partly or fully inside region */
 	if (first_i < 0) first_i += n_cols;
-	n_skip = (int)floor ((project_info.ymax - grd->y_max) / grd->y_inc);	/* Number of rows clearly above y_max */
+	n_skip = (GMT_LONG)floor ((project_info.ymax - grd->y_max) / grd->y_inc);	/* Number of rows clearly above y_max */
 	if (GMT_fseek (fp, (long)(n_skip * n_cols * GMT_IMG_ITEMSIZE), SEEK_SET)) return (GMT_GRDIO_SEEK_FAILED);
 
 	i2 = (short int *) GMT_memory (VNULL, (size_t)n_cols, sizeof (short int), GMT_program);
 	for (j = 0; j < grd->ny; j++) {	/* Read all the rows, offset by 2 boundary rows and cols */
 		ij = (j + GMT_pad[3]) * mx + GMT_pad[0];
-		GMT_fread ((void *)i2, sizeof (short int), n_cols, fp);
+		GMT_fread ((void *)i2, sizeof (short int), (size_t)n_cols, fp);
 		for (i = 0, k = first_i; i < grd->nx; i++) {	/* Process this row's values */
 			switch (mode) {
 				case 0:	/* No encoded track flags, do nothing */
@@ -1265,8 +1272,8 @@ struct GMT_GRID *GMT_create_grid (char *arg)
 {	/* Allocates space for a new grid container.  No space allocated for the float grid itself */
 	struct GMT_GRID * G;
 
-	G = (struct GMT_GRID *) GMT_memory (VNULL, 1, sizeof (struct GMT_GRID), arg);
-	G->header = (struct GRD_HEADER *) GMT_memory (VNULL, 1, sizeof (struct GRD_HEADER), arg);
+	G = (struct GMT_GRID *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_GRID), arg);
+	G->header = (struct GRD_HEADER *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD_HEADER), arg);
 
 	return (G);
 }
