@@ -1,42 +1,15 @@
-/*	$Id: gmt_mgg_header2.c,v 1.13 2006/10/23 03:35:57 pwessel Exp $
+/*	$Id: gmt_mgg_header2.c,v 1.29 2008/05/06 20:41:18 guru Exp $
  *
  *	Code donated by David Divens, NOAA/NGDC
- *	This is the README file:
- ------------------------------------------------------------------------
- Adding MGG file format to GMT
------------------------------
-1) Modify gmt_grdio.h
-   - Increment GMT_N_GRD_FORMATS.
-     This number is one based, the formats are zero based so this
-     value will be one greater than the grid number you are creating.
-     
-2) Modify gmt_customio.c
-   - Make sure both gmt_mgg_header2.h and gmt_mgg_header2.c are in the same
-     directory as gmt_customio.c
+ *	Distributed under the GNU Public License (see COPYING for details)
+ *--------------------------------------------------------------------*/
 
-   - Scroll down to the comment
-     FORMAT # N:
-
-   - Add the following lines:
-    id = N;
-    GMT_io_readinfo[id]  = (PFI) mgg2_read_grd_info;
-    GMT_io_writeinfo[id] = (PFI) mgg2_write_grd_info;
-    GMT_io_readgrd[id]   = (PFI) mgg2_read_grd;
-    GMT_io_writegrd[id]  = (PFI) mgg2_write_grd;
-
-   - Jump down to the end of the file and add this line:
-    #include "gmt_mgg_header2.c"
-
-2) Modify gmt_customio.h
-   - At the end of this file add:
-    #include "gmt_mgg_header2."
-
-Build GMT with new format by typing:
-> make all
-
-Test format by checking a grid:
-> grdinfo gnaXXXXX.g03=N
---------------------------------------------------------------------*/
+/* Comments added by P. Wessel:
+ *
+ * 1) GRD98 can now support pixel grids since 11/24/2006.
+ * 2) Learned that 1/x_inc and 1/y_inc are stored as integers.  This means
+ *    GRD98 imposes restrictions on x_inc & y_inc.
+ */
 
 #include "gmt_mgg_header2.h"
 
@@ -70,20 +43,26 @@ static void degrees2dms(double degrees, int *deg, int *min, int *sec)
 	*sec = (int)(degrees * SEC_PER_MIN);
 }
 
-static void GMT2MGG2(struct GRD_HEADER *gmt, MGG_GRID_HEADER_2 *mgg)
+int GMT2MGG2(struct GRD_HEADER *gmt, MGG_GRID_HEADER_2 *mgg)
 {
+	double f;
 	memset(mgg, 0, sizeof(MGG_GRID_HEADER_2));
 	
-	mgg->version     = MGG_MAGIC_NUM + VERSION;
+	mgg->version     = MGG_MAGIC_NUM + MGG_VERSION;
 	mgg->length      = sizeof(MGG_GRID_HEADER_2);
-    mgg->dataType    = 1;
+	mgg->dataType    = 1;
 	
+	mgg->cellRegistration = gmt->node_offset;
 	mgg->lonNumCells = gmt->nx;
-	mgg->lonSpacing  = (int)rint(gmt->x_inc * SEC_PER_DEG);
+	f  = gmt->x_inc * SEC_PER_DEG;
+	mgg->lonSpacing  = (int)rint(f);
+	if (fabs (f - (double)mgg->lonSpacing) > GMT_CONV_LIMIT) return (GMT_GRDIO_GRD98_XINC);
 	degrees2dms(gmt->x_min, &mgg->lonDeg, &mgg->lonMin, &mgg->lonSec);
 	
 	mgg->latNumCells = gmt->ny;
+	f  = gmt->y_inc * SEC_PER_DEG;
 	mgg->latSpacing  = (int)rint(gmt->y_inc * SEC_PER_DEG);
+	if (fabs (f - (double)mgg->latSpacing) > GMT_CONV_LIMIT) return (GMT_GRDIO_GRD98_YINC);
 	degrees2dms(gmt->y_max, &mgg->latDeg, &mgg->latMin, &mgg->latSec);
 
 	/* Default values */
@@ -109,23 +88,28 @@ static void GMT2MGG2(struct GRD_HEADER *gmt, MGG_GRID_HEADER_2 *mgg)
 		mgg->maxValue  = (int)gmt->z_max;
 	}
 #endif
+	return (GMT_NOERROR);
 }
 
 static void MGG2_2GMT(MGG_GRID_HEADER_2 *mgg, struct GRD_HEADER *gmt)
 {
-	memset(gmt, 0, sizeof(struct GRD_HEADER));
-
+	int one_or_zero;
+	
+	/* Do not memset the gmt header since it has the file name set */
+	
+	gmt->type = GMT_grd_format_decoder ("rf");
+	gmt->node_offset    = mgg->cellRegistration;
+	one_or_zero	    = 1 - gmt->node_offset;
 	gmt->nx             = mgg->lonNumCells;
 	gmt->x_min          = dms2degrees(mgg->lonDeg, mgg->lonMin, mgg->lonSec);
 	gmt->x_inc          = dms2degrees(0, 0, mgg->lonSpacing);
-	gmt->x_max          = gmt->x_min + (gmt->x_inc * (gmt->nx - 1));
+	gmt->x_max          = gmt->x_min + (gmt->x_inc * (gmt->nx - one_or_zero));
 
 	gmt->ny             = mgg->latNumCells;
 	gmt->y_max          = dms2degrees(mgg->latDeg, mgg->latMin, mgg->latSec);
 	gmt->y_inc          = dms2degrees(0, 0, mgg->latSpacing);
-	gmt->y_min          = gmt->y_max - (gmt->y_inc * (gmt->ny - 1));
+	gmt->y_min          = gmt->y_max - (gmt->y_inc * (gmt->ny - one_or_zero));
  
-	gmt->node_offset    = 0;
 	gmt->z_min          = (double)mgg->minValue / (double)mgg->precision;
 	gmt->z_max          = (double)mgg->maxValue / (double)mgg->precision;
 	gmt->z_scale_factor = 1.0;
@@ -158,6 +142,7 @@ static void swap_long(void *ptr)
 
 static void swap_header(MGG_GRID_HEADER_2 *header)
 {
+   int i;
    swap_long(&header->version);
    swap_long(&header->length);
    swap_long(&header->dataType);
@@ -179,45 +164,28 @@ static void swap_header(MGG_GRID_HEADER_2 *header)
    swap_long(&header->numType);
    swap_long(&header->waterDatum);
    swap_long(&header->dataLimit);
-   swap_long(&header->unused1);
-   swap_long(&header->unused2);
-   swap_long(&header->unused3);
-   swap_long(&header->unused4);
-   swap_long(&header->unused5);
-   swap_long(&header->unused6);
-   swap_long(&header->unused7);
-   swap_long(&header->unused8);
-   swap_long(&header->unused9);
-   swap_long(&header->unused10);
-   swap_long(&header->unused11);
+   swap_long(&header->cellRegistration);
+   for (i = 0; i < GRD98_N_UNUSED; i++) swap_long(&header->unused[i]);
 }
 
-int GMT_is_mgg2_grid (char *file)
-{	/* Determine if file is a Sun rasterfile */
+int GMT_is_mgg2_grid (struct GRD_HEADER *header)
+{	/* Determine if file is a GRD98 file */
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
 
-	if (!strcmp(file, "=")) {	/* Cannot check on pipes */
-		fprintf (stderr, "GMT Fatal Error: Cannot guess grid format type if grid is passed via pipe!\n");
-		exit (EXIT_FAILURE);
-	}
-	if ((fp = GMT_fopen(file, GMT_io.r_mode)) == NULL) {
-		fprintf(stderr, "GMT Fatal Error: Could not open file %s!\n", file);
-		exit (-1);
-	}
+	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
+	if ((fp = GMT_fopen(header->name, GMT_io.r_mode)) == NULL) return (GMT_GRDIO_OPEN_FAILED);
 
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
-	if (fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) {
-		fprintf(stderr, "GMT Fatal Error: Error reading file %s!\n", file);
-		exit (-1);
-	}
+	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
 	/* Swap header bytes if necessary */
 	swap_header(&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + VERSION) return (-1);	/* Not this kind of file */
-	return (GMT_grd_format_decoder ("rf"));
+	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) return (-1);	/* Not this kind of file */
+	header->type = GMT_grd_format_decoder ("rf");
+	return (header->type);
 }
 
 int mgg2_read_grd_info (struct GRD_HEADER *header)
@@ -226,64 +194,55 @@ int mgg2_read_grd_info (struct GRD_HEADER *header)
 	MGG_GRID_HEADER_2	mggHeader;
 
 	if (!strcmp(header->name, "=")) {
-		fp = stdin;
-	} else if ((fp = GMT_fopen(header->name, GMT_io.r_mode)) == NULL) {
-		fprintf(stderr, "GMT Fatal Error: Could not open file %s!\n", header->name);
-		exit (-1);
-	}
+		fp = GMT_stdin;
+	} else if ((fp = GMT_fopen(header->name, GMT_io.r_mode)) == NULL)
+		return (GMT_GRDIO_OPEN_FAILED);
 
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
-	if (fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) {
-		fprintf(stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-		exit (-1);
-	}
+	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
-    /* Swap header bytes if necessary */
-    swap_header(&mggHeader);
+	/* Swap header bytes if necessary */
+	swap_header(&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + VERSION) {
-		fprintf(stderr, "GMT Fatal Error: Unrecognized header, expected 0x%04X saw 0x%04X\n", MGG_MAGIC_NUM + VERSION, mggHeader.version);
-		exit (-1);
+	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) {
+		fprintf(stderr, "GMT Fatal Error: Unrecognized header, expected 0x%04X saw 0x%04X\n", MGG_MAGIC_NUM + MGG_VERSION, mggHeader.version);
+		return (GMT_GRDIO_GRD98_BADMAGIC);
 	}
 
 	if (mggHeader.length != sizeof(MGG_GRID_HEADER_2)) {
 		fprintf(stderr, "GMT Fatal Error: Invalid grid header size, expected %d, found %d\n", (int)sizeof(MGG_GRID_HEADER_2), mggHeader.length);
-		exit (-1);
+		return (GMT_GRDIO_GRD98_BADLENGTH);
 	}
 
-	if (fp != stdin) fclose(fp);
+	if (fp != GMT_stdin) GMT_fclose(fp);
 	
 	MGG2_2GMT(&mggHeader, header);
 	
-	return FALSE;
+	return (GMT_NOERROR);
 }
 
 int mgg2_write_grd_info (struct GRD_HEADER *header)
 {
 	FILE			*fp;
 	MGG_GRID_HEADER_2	mggHeader;
+	int err;
 	
 	if (!strcmp(header->name, "=")) {
-		fp = stdout;
-	} else if ((fp = GMT_fopen(header->name, GMT_io.w_mode)) == NULL) {
-		fprintf(stderr, "GMT Fatal Error: Could not create file %s!\n", header->name);
-		exit (-1);
-	}
+		fp = GMT_stdout;
+	} else if ((fp = GMT_fopen(header->name, GMT_io.w_mode)) == NULL)
+		return (GMT_GRDIO_CREATE_FAILED);
 	
-	GMT2MGG2(header, &mggHeader);
+	if ((err = GMT2MGG2(header, &mggHeader))) return (err);
 
-    /* Swap header bytes if necessary */
-    swap_header(&mggHeader);
+	/* Swap header bytes if necessary */
+	swap_header(&mggHeader);
 
-	if (fwrite (&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) {
-		fprintf (stderr, "GMT Fatal Error: Error writing file %s!\n", header->name);
-		exit (-1);
-	}
+	if (GMT_fwrite (&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 
-	if (fp != stdout) fclose (fp);
+	if (fp != GMT_stdout) GMT_fclose (fp);
 	
-	return FALSE;
+	return (GMT_NOERROR);
 }
 
 int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int pad[], BOOLEAN complex)
@@ -293,43 +252,36 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	int  *tLong  = NULL;
 	short *tShort = NULL;
 	char  *tChar  = NULL;
-	int first_col, last_col, first_row, last_row, nm, kk, one_or_zero;
-	int i, j, j2, ij, width_in, width_out, height_in, i_0_out, inc = 1;
+	GMT_LONG first_col, last_col, first_row, last_row, one_or_zero;
+	GMT_LONG j, j2, width_in, height_in, i_0_out, inc = 1;
+	GMT_LONG i, kk, ij, nm, width_out;
 	BOOLEAN piping = FALSE, geo = FALSE;
 	double half_or_zero, x, small;
 	long long_offset;	/* For fseek only */
 	
-	if (complex) {
-		fprintf (stderr, "GMT Fatal Error: MGG grdfile %s cannot hold complex data!\n", header->name);
-		exit (-1);
-	}
+	if (complex) return (GMT_GRDIO_GRD98_COMPLEX);
 
 	if (!strcmp (header->name, "=")) {
-		fp = stdin;
+		fp = GMT_stdin;
 		piping = TRUE;
 	}
-/*	else if ((fp = fopen (header->name, "rb")) != NULL) {  */
 	else if ((fp = GMT_fopen (header->name, GMT_io.r_mode)) != NULL) {
-		fread(&mggHeader, 1, sizeof(MGG_GRID_HEADER_2), fp);
-        swap_header(&mggHeader);
+		if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
+		swap_header(&mggHeader);
 		if (mggHeader.numType == 0) mggHeader.numType = sizeof(int);
 	}
 
 	else {
-		fprintf (stderr, "GMT Fatal Error: Could not open file! (%s)", header->name);
-		exit (-1);
+		return (GMT_GRDIO_OPEN_FAILED);
 	}
 	
 	if (w == 0.0 && e == 0.0) {	/* Get entire file and return */
-		nm = header->nx * header->ny;
-		tLong  = (int *) GMT_memory (CNULL, nm, sizeof (int), "mgg_read_grd");
+		nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
+		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
 
-		if (fread ((void *)tLong, mggHeader.numType, nm, fp) != (size_t)nm) {
-			fprintf (stderr, "GMT Fatal Error: Error reading file! (%s)", header->name);
-			exit (-1);
-		}
+		if (GMT_fread ((void *)tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_READ_FAILED);
 		for (i = 0; i < nm; i++) {
 			/* 4-byte values */
 			if (mggHeader.numType == sizeof(int)) {
@@ -353,13 +305,12 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 			}
 
 			else {
-				fprintf(stderr, "GMT Fatal Error: Unknown datalen %d\n", mggHeader.numType);
-				exit(-1);
+				return (GMT_GRDIO_UNKNOWN_TYPE);
 			}
 		}
 		GMT_free ((void *)tLong);
-		fclose(fp);
-		return (FALSE);
+		GMT_fclose(fp);
+		return (GMT_NOERROR);
 	}
 
 	/* Must deal with a subregion */
@@ -376,18 +327,18 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	/* Get first and last row and column numbers */
 	
 	small = 0.1 * header->x_inc;
-	first_col = (int)floor ((w - header->x_min + small) / header->x_inc);
-	last_col  = (int)ceil ((e - header->x_min - small) / header->x_inc) - 1 + one_or_zero;
+	first_col = (GMT_LONG)floor ((w - header->x_min + small) / header->x_inc);
+	last_col  = (GMT_LONG)ceil ((e - header->x_min - small) / header->x_inc) - 1 + one_or_zero;
 	small = 0.1 * header->y_inc;
-	first_row = (int)floor ((header->y_max - n + small) / header->y_inc);
-	last_row  = (int)ceil ((header->y_max - s - small) / header->y_inc) - 1 + one_or_zero;
+	first_row = (GMT_LONG)floor ((header->y_max - n + small) / header->y_inc);
+	last_row  = (GMT_LONG)ceil ((header->y_max - s - small) / header->y_inc) - 1 + one_or_zero;
 
 	if ((last_col - first_col + 1) > width_in) last_col--;
 	if ((last_row - first_row + 1) > height_in) last_row--;
 	if ((last_col - first_col + 1) > width_in) first_col++;
 	if ((last_row - first_row + 1) > height_in) first_row++;
 	
-	width_out = width_in;		/* Width of output array */
+	width_out = (GMT_LONG)width_in;		/* Width of output array */
 	if (pad[0] > 0) width_out += pad[0];
 	if (pad[1] > 0) width_out += pad[1];
 	
@@ -395,11 +346,11 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	
 	if (geo) {	/* Must rollover in longitudes */
 		int *k;
-		tLong  = (int *) GMT_memory (CNULL, header->nx, sizeof (int), "mgg_read_grd");
+		tLong  = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *)tLong;
 		tChar  = (char *)tLong;
 
-		k = (int *) GMT_memory (CNULL, width_in, sizeof (int), "mgg_read_grd");
+		k = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_read_grd");
 		
 		half_or_zero = (header->node_offset) ? 0.5 : 0.0;
 		small = 0.1 * header->x_inc;	/* Anything smaller than 0.5 dx will do */
@@ -413,21 +364,15 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) {
-					fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-					exit (-1);
-				}
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else { /* Simply seek by it */
 			long_offset = (long)(first_row * header->nx * mggHeader.numType);
-			fseek (fp, long_offset, 1);
+			if (GMT_fseek (fp, long_offset, 1)) return (GMT_GRDIO_SEEK_FAILED);
 		}
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
-			if (fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) {
-				fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-				exit (-1);
-			}
+			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			ij = (j2 + pad[3]) * width_out + i_0_out;
 			for (i = 0; i < width_in; i++) {
@@ -450,45 +395,35 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 				}
 
 				else {
-					fprintf(stderr, "GMT Fatal Error: Unknown datalen %d\n", mggHeader.numType);
-					exit(-1);
+					return (GMT_GRDIO_UNKNOWN_TYPE);
 				}
 			}
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) {
-					fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-					exit (-1);
-				}
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 			
 		GMT_free ((void *)k);
 	}
 	else {	/* A bit easier here */
-		tLong = (int *) GMT_memory (CNULL, header->nx, sizeof (int), "mgg_read_grd");
+		tLong = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
 
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) {
-					fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-					exit (-1);
-				}
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else {/* Simply seek by it */
 			long_offset = (long) (first_row * header->nx * mggHeader.numType);
-			fseek (fp, long_offset, 1);
+			if (GMT_fseek (fp, long_offset, 1)) return (GMT_GRDIO_SEEK_FAILED);
 		}
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
 			ij = (j2 + pad[3]) * width_out + i_0_out;
-			if (fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) {
-				fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-				exit (-1);
-			}
+			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			for (i = 0; i < width_in; i++) {
 				kk = ij+i;
@@ -513,18 +448,14 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 				}
 
 				else {
-					fprintf(stderr, "GMT Fatal Error: Unknown datalen %d\n", mggHeader.numType);
-					exit(-1);
+					return (GMT_GRDIO_UNKNOWN_TYPE);
 				}
 			}
 		}
 
 		if (piping)	{/* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (fread ((void *) tLong, mggHeader.numType, header->nx, fp)) {
-					fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", header->name);
-					exit (-1);
-				}
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) < (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 	}
@@ -540,7 +471,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	for (j = 0; j < header->ny; j++) {
 		for (i = 0; i < header->nx; i++) {
 			ij = (j + pad[3]) * width_out + i + pad[0];
-			if (GMT_is_fnan ((double) grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
@@ -548,16 +479,18 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	if (fp != stdin) fclose (fp);
 	
 	GMT_free ((void *)tLong);
-	return (FALSE);
+	return (GMT_NOERROR);
 }
 
 	
 int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
 {
 	MGG_GRID_HEADER_2	mggHeader;
-	int i, i2, nm, kk, *k;
-	int j, ij, j2, width_in, width_out, height_out, one_or_zero;
-	int first_col, last_col, first_row, last_row;
+	int *k;
+	GMT_LONG i2, kk, err;
+	GMT_LONG j, j2, width_out, height_out, one_or_zero;
+	GMT_LONG first_col, last_col, first_row, last_row;
+	GMT_LONG i, nm, ij, width_in;
 	
 	BOOLEAN geo = FALSE;
 	double half_or_zero, small, x;
@@ -567,44 +500,35 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	char  *tChar;
 	FILE *fp;
 	
-	if (complex) {
-		fprintf (stderr, "GMT Fatal Error: MGG grdfile %s cannot hold complex data!\n", header->name);
-		exit (-1);
-	}
+	if (complex) return (GMT_GRDIO_GRD98_COMPLEX);
 	if (!strcmp (header->name, "=")) {
-		fp = stdout;
-	}
-/*	else if ((fp = fopen (header->name, "wb")) == NULL) {      */
-	else if ((fp = GMT_fopen (header->name, GMT_io.w_mode)) == NULL) {
-		fprintf (stderr, "GMT Fatal Error: Could not create file %s!\n", header->name);
-		exit (-1);
+		fp = GMT_stdout;
 	}
+	else if ((fp = GMT_fopen (header->name, GMT_io.w_mode)) == NULL)
+		return (GMT_GRDIO_CREATE_FAILED);
 	
 	if (w == 0.0 && e == 0.0) {	/* Write entire file and return */
 		/* Find min/max of data */
 		
-		nm = header->nx * header->ny;
+		nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
 		header->z_min = DBL_MAX;	header->z_max = -DBL_MAX;
 		for (i = 0; i < nm; i++) {
 			ij = (complex) ? 2 * i : i;
-			if (GMT_is_fnan ((double)grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
-		GMT2MGG2(header, &mggHeader);
+		if ((err = GMT2MGG2(header, &mggHeader))) return (err);
 		swap_header(&mggHeader);
-		if (fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), 1, fp) != 1) {
-			fprintf (stderr, "GMT Fatal Error: Error writing file %s!\n", header->name);
-			exit (-1);
-		}
+		if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 		swap_header(&mggHeader);
 
-		tLong  = (int *) GMT_memory (CNULL, nm, sizeof (int), "mgg_write_grd");
+		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_write_grd");
 		tShort = (short*)tLong;
 		tChar  = (char*)tLong;
 		
 		for (i = 0; i < nm; i++) {
-			if (GMT_is_fnan ((double)grid[i])) {
+			if (GMT_is_fnan (grid[i])) {
 				if (mggHeader.numType == sizeof(int)) {
 			       tLong[i]  = mggHeader.nanValue;
 			       swap_long(&tLong[i]);
@@ -614,8 +538,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				} else if (mggHeader.numType == sizeof(char)) {
 					tChar[i]  = (char)mggHeader.nanValue;
 				} else {
-					fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-					exit(-1);
+					return (GMT_GRDIO_UNKNOWN_TYPE);
 				}
 			} else {
 				if (grid[i] > -0.1 && grid[i] < 0.0) grid[i] = (float)(-0.1);
@@ -635,17 +558,13 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				}
 
 				else {
-					fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-					exit(-1);
+					return (GMT_GRDIO_UNKNOWN_TYPE);
 				}
 			}
 		}
-		if (fwrite (tLong, mggHeader.numType, nm, fp) != (size_t)nm) {
-			fprintf (stderr, "GMT Fatal Error: Error writing file %s!\n", header->name);
-			exit (-1);
-		}
+		if (GMT_fwrite (tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_WRITE_FAILED);
 		GMT_free ((void *)tLong);
-		return (FALSE);
+		return (GMT_NOERROR);
 	}
 
 	if (w < header->x_min || e > header->x_max) geo = TRUE;	/* Dealing with periodic grid */
@@ -660,18 +579,18 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	/* Get first and last row and column numbers */
 	
 	small     = 0.1 * header->x_inc;
-	first_col = (int)floor ((w - header->x_min + small) / header->x_inc);
-	last_col  = (int)ceil ((e - header->x_min - small) / header->x_inc) -1 + one_or_zero;
+	first_col = (GMT_LONG)floor ((w - header->x_min + small) / header->x_inc);
+	last_col  = (GMT_LONG)ceil ((e - header->x_min - small) / header->x_inc) -1 + one_or_zero;
 	small     = 0.1 * header->y_inc;
-	first_row = (int)floor ((header->y_max - n + small) / header->y_inc);
-	last_row  = (int)ceil ((header->y_max - s - small) / header->y_inc) -1 + one_or_zero;
+	first_row = (GMT_LONG)floor ((header->y_max - n + small) / header->y_inc);
+	last_row  = (GMT_LONG)ceil ((header->y_max - s - small) / header->y_inc) -1 + one_or_zero;
 	
 	if ((last_col - first_col + 1) > width_out) last_col--;
 	if ((last_row - first_row + 1) > height_out) last_row--;
 	if ((last_col - first_col + 1) > width_out) first_col++;
 	if ((last_row - first_row + 1) > height_out) first_row++;
 
-	width_in = width_out;		/* Physical width of input array */
+	width_in = (GMT_LONG)width_out;		/* Physical width of input array */
 	if (pad[0] > 0) width_in += pad[0];
 	if (pad[1] > 0) width_in += pad[1];
 	
@@ -686,27 +605,24 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	for (j = first_row, j2 = pad[3]; j <= last_row; j++, j2++) {
 		for (i = first_col, i2 = pad[0]; i <= last_col; i++, i2++) {
 			ij = j2 * width_in + i2;
-			if (GMT_is_fnan ((double)grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
 	}
 	
 	/* store header information and array */
-	GMT2MGG2(header, &mggHeader);
+	if ((err = GMT2MGG2(header, &mggHeader))) return (err);;
 	swap_header(&mggHeader);
-	if (fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), 1, fp) != 1) {
-		fprintf (stderr, "GMT Fatal Error: Error writing file %s!\n", header->name);
-		exit (-1);
-	}
+	if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 	swap_header(&mggHeader);
 
-	tLong  = (int *) GMT_memory (CNULL, width_in, sizeof (int), "mgg_write_grd");
+	tLong  = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_write_grd");
 	tShort = (short *) tLong;
 	tChar  = (char *)  tLong;
 	
 	if (geo) {
-		k = (int *) GMT_memory (CNULL, width_out, sizeof (int), "mgg_write_grd");
+		k = (int *) GMT_memory (CNULL, (size_t)width_out, sizeof (int), "mgg_write_grd");
 		
 		half_or_zero = (header->node_offset) ? 0.5 : 0.0;
 		small = 0.1 * header->x_inc;	/* Anything smaller than 0.5 dx will do */
@@ -723,13 +639,12 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 			ij = j2 * width_in + i2;
 			for (i = 0; i < width_out; i++) {
 				kk = ij+k[i];
-				if (GMT_is_fnan ((double)grid[kk])) {
+				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
 					else {
-						fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-						exit(-1);
+						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				} else {
 					if (grid[kk] > -0.1 && grid[kk] < 0) grid[kk] = (float)(-0.1);
@@ -747,14 +662,13 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 					}
 					
 					else {
-						fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-						exit(-1);
+						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				}
 				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
 				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			fwrite ((void *)tLong, mggHeader.numType, width_out, fp);
+			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
 		}
 		GMT_free ((void *)k);
 	}
@@ -764,13 +678,12 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 			ij = j2 * width_in + i2;
 			for (i = 0; i < width_out; i++) {
 				kk = ij+i;
-				if (GMT_is_fnan ((double)grid[kk])) {
+				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
 					else {
-						fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-						exit(-1);
+						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				} else {
 					if (grid[kk] > -0.1 && grid[kk] < 0) grid[kk] = (float)(-0.1);
@@ -787,19 +700,18 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 					}
 
 					else {
-						fprintf(stderr, "GMT Fatal Error: Unknown numType %d\n", mggHeader.numType);
-						exit(-1);
+						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				}
 				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
 				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			fwrite ((void *)tLong, mggHeader.numType, width_out, fp);
+			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
 		}
 	}
-	if (fp != stdout) fclose (fp);
+	if (fp != GMT_stdout) GMT_fclose (fp);
 	
 	GMT_free ((void *)tLong);
 	
-	return (FALSE);
+	return (GMT_NOERROR);
 }
