@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_notunix.h,v 1.15 2007/01/30 20:37:08 pwessel Exp $
+ *	$Id: gmt_notunix.h,v 1.19 2008/05/02 00:44:42 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -61,7 +61,11 @@
 /* This section will override those in gmt_notposix.h which cannot
  * automatically be generated under Windows.
  */
- 
+
+/* Turn off some annoying "security" warnings in Vis Studio */
+
+#pragma warning( disable : 4996 )
+
 #define SET_IN_NOTUNIX	/* This forces the following not to be reset in gmt_notposix.h */
 
 /* These functions are available under Windows with MSVC compilers */
@@ -105,6 +109,11 @@
 #define strdup(s) _strdup(s)
 #define STAT _stat
 
+typedef int mode_t;		/* mdoe_t not defined under Windows; assumed a signed 4-byte integer */
+#ifdef _WIN64
+typedef __int64 GMT_LONG;		/* A signed 8-byte integer; otherwise set in gmt.h */
+#endif
+
 /* WIN32 versus _WIN32:
  *
  * In GMT, the WIN32 flag is predefined by the MicroSoft C compiler.
