@@ -7,9 +7,9 @@ struct Z {	/* -Z[<flags>] */
 	int skip;
 };
 
-int GMT_parse_z_io (char *txt, struct Z *z)
+GMT_LONG GMT_parse_z_io (char *txt, struct Z *z)
 {
-	int i, k = 0;
+	GMT_LONG i, k = 0;
 
 	/* BOOLEAN input:  currently unused */
 
