@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_shore.c,v 1.28 2007/03/12 19:52:26 remko Exp $
+ *	$Id: gmt_shore.c,v 1.36 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -102,8 +102,9 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
 	
 	sprintf (stem, "binned_GSHHS_%c", res);
 	
-        if (!GMT_shore_getpathname (stem, path)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Failed to find file */
-        
+	if (!GMT_shore_getpathname (stem, path)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Failed to find file */
+	memset ((void *)c, 0, sizeof (struct GMT_SHORE));
+		
 	GMT_err_trap (nc_open (path, NC_NOWRITE,&c->cdfid));
                 
 	/* Get all id tags */
@@ -198,8 +199,8 @@ int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_leve
 /* max_level: Polygons with higher levels are ignored */
 {
 	size_t start[1], count[1];
-	int cut_area, *seg_area, *seg_info, *seg_start;
-	int s, i, err;
+	int *seg_area, *seg_info, *seg_start;
+	int s, i, err, cut_area;
 	double w, e, dx;
 	
 	c->node_level[0] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 9) & 7, max_level);
@@ -403,9 +404,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
         GMT_err_trap (nc_get_vara_short (c->cdfid, c->seg_level_id, start, count, seg_level));
         GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
 
-
-	c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
-	
+	c->seg = NULL;
 	for (s = i = 0; i < c->ns; i++) {
 		if (n_levels == 0)
 			skip = FALSE;
@@ -414,7 +413,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 				if (seg_level[i] == level[k]) skip = FALSE;
 		}
 		if (skip) continue;
-		
+		if (!c->seg) c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
 		c->seg[s].n = seg_n[i];
 		c->seg[s].level = seg_level[i];
 		c->seg[s].dx = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_br_bin");
@@ -567,7 +566,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].level = (dir == 1) ? 2 * ((low_level - 1) / 2) + 1: 2 * (low_level/2);
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 		
@@ -585,7 +584,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].level = c->seg[id].level;
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 	}
@@ -641,6 +640,7 @@ void GMT_free_br (struct GMT_BR *c)
 		GMT_free ((void *)c->seg[i].dy);
 	}
 	if (c->ns) GMT_free ((void *)c->seg);
+	
 }
 		
 void GMT_shore_cleanup (struct GMT_SHORE *c)
@@ -661,7 +661,7 @@ void GMT_br_cleanup (struct GMT_BR *c)
 	
 }
 
-int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sample, double step, int anti_bin)
+int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, double step, int anti_bin)
 {
 	/* This function will go through each of the polygons and determine
 	 * if the polygon is clipped by the map boundary, and if so if it
@@ -671,14 +671,13 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich,
 	 * Upon return the polygons are in x,y inches, not degrees.
 	 *
 	 * *p is the array of np polygons
-	 * greenwich is TRUE if area crosses Greenwich
 	 * sample is TRUE if we need to resample the polygons to reduce point spacing
 	 * step is the new maximum point separation in degrees
 	 * anti_bin, if >= 0, indicates a possible problem bin at the antipole using -JE only
 	 * We also explicitly close all polygons if they are not so already.
 	 */
 
-	int k, np_new, n_use, n, start, n_alloc;
+	GMT_LONG k, np_new, n_use, n, start, n_alloc;
 	BOOLEAN close;
 	double *xtmp, *ytmp;
 	struct GMT_GSHHS_POL *p;
@@ -689,12 +688,12 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich,
 		
 	for (k = 0; k < np; k++) {
 			
-		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, p[k].n, greenwich, step);
+		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, (GMT_LONG)p[k].n, step, 0);
 		
 		/* Clip polygon against map boundary if necessary and return plot x,y in inches */
 				
-		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
-			p[k].n = 0;
+		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, (GMT_LONG)p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
+			p[k].n = 0;	/* Note the memory in lon, lat not freed yet */
 			continue;
 		}
 			
@@ -964,7 +963,7 @@ char *GMT_shore_getpathname (char *stem, char *path) {
 	 
 	/* 1. First check for coastline.conf */
 	
-	if (GMT_getsharepath ("coast", "coastline", ".conf", path)) {
+	if (GMT_getsharepath ("conf", "coastline", ".conf", path) || GMT_getsharepath ("coast", "coastline", ".conf", path)) {
 
 		/* We get here if coastline.conf exists - search among its directories for the named file */
 
