@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_shore.h,v 1.15 2007/03/12 19:52:26 remko Exp $
+ *	$Id: gmt_shore.h,v 1.19 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -69,8 +69,8 @@ struct GMT_SHORE {
 	int n_pt;		/* Number of points present in the data set */
 	
 	int *bin_firstseg;	/* Array with ids of first segment per bin */
-	short *bin_info;	/* Array with levels of all 4 nodes per bin */
-	short *bin_nseg;	/* Array with number of segments per bin */
+	short int *bin_info;	/* Array with levels of all 4 nodes per bin */
+	short int *bin_nseg;	/* Array with number of segments per bin */
 	
 	char units[80];		/* Units of lon/lat */
 	char title[80];		/* Title of data set */
@@ -104,13 +104,13 @@ struct GMT_SHORE_SEGMENT {
 	unsigned char entry;	/* Side (0-3) the segment starts on, or 4 for closed segments */
 	unsigned char exit;	/* Side (0-3) the segment ends on, or 4 for closed segments */
 	unsigned short n;	/* Number of points in segment */
-	short *dx;		/* Array of scaled longitudes relative to SW corner */
-	short *dy;		/* Array of scaled latitudes relative to SW corner */
+	short int *dx;		/* Array of scaled longitudes relative to SW corner */
+	short int *dy;		/* Array of scaled latitudes relative to SW corner */
 };
 
 struct GSHHS_SIDE {
 	unsigned short pos;	/* Position along side in 0-65535 range */
-	short id;	/* Local segment id */
+	short int id;	/* Local segment id */
 };
 
 struct GMT_BR {	/* Structure for Borders and Rivers */
@@ -139,7 +139,7 @@ struct GMT_BR {	/* Structure for Borders and Rivers */
 	int n_pt;		/* Number of points present in the data set */
 	
 	int *bin_firstseg;	/* Array with ids of first segment per bin */
-	short *bin_nseg;	/* Array with number of segments per bin */
+	short int *bin_nseg;	/* Array with number of segments per bin */
 	
 	char units[80];		/* Units of lon/lat */
 	char title[80];		/* Title of data set */
@@ -170,13 +170,13 @@ struct GMT_BR {	/* Structure for Borders and Rivers */
 struct GMT_BR_SEGMENT {
 	unsigned short n;	/* Number of points in segment */
 	unsigned short level;	/* Hierarchical level of segment */
-	short *dx;		/* Array of scaled longitudes relative to SW corner */
-	short *dy;		/* Array of scaled latitudes relative to SW corner */
+	short int *dx;		/* Array of scaled longitudes relative to SW corner */
+	short int *dy;		/* Array of scaled latitudes relative to SW corner */
 };
 
 struct GMT_GSHHS_POL {
-	int n;
-	int interior;	/* TRUE if polygon is inside bin */
+	GMT_LONG n;
+	BOOLEAN interior;	/* TRUE if polygon is inside bin */
 	int level;
 	double *lon;
 	double *lat;
@@ -195,7 +195,7 @@ EXTERN_MSC int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e
 EXTERN_MSC int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, double s, double n);
 EXTERN_MSC int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct GMT_GSHHS_POL **pol);
 EXTERN_MSC int GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct GMT_GSHHS_POL **pol);
-EXTERN_MSC int GMT_prep_polygons (struct GMT_GSHHS_POL **p, int np, BOOLEAN greenwich, BOOLEAN sample, double step, int anti_bin);
+EXTERN_MSC int GMT_prep_polygons (struct GMT_GSHHS_POL **p, int np, BOOLEAN sample, double step, int anti_bin);
 EXTERN_MSC int GMT_set_resolution (char *res, char opt);
 
 #endif /* _GMT_SHORE_H */
