@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_stat.c,v 1.47 2007/04/02 22:25:29 remko Exp $
+ *	$Id: gmt_stat.c,v 1.65 2008/04/04 20:47:24 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -45,10 +45,12 @@
  *					platforms without native library support.
  *		09-MAY-2000 P. Wessel: Added GMT_chi2 and GMT_cumpoisson
  *		18-AUG-2000 P. Wessel: Moved GMT_mode and GMT_median from gmt_support to here.
- *					Added float versions of these two functions for grd data.
+ *					Added float versions of GMT_mode for grd data.
  *		27-JUL-2005 P. Wessel: Added Chebyshev polynomials Tn(x)
  *		07-SEP-2005 P. Wessel: Added GMT_corrcoeff (x,y)
  *		24-MAR-2006 P. Wessel: Added GMT_zdist (x)
+ *		06-JUL-2007 P. Wessel: Added GMT_psi () and GMT_PvQv()
+ *		28-SEP-2007 R. Scharroo: Added GMT_plm_bar and made GMT_factorial public
  *
  * PUBLIC functions:
  *
@@ -79,6 +81,9 @@
  *	GMT_rand:	Uniformly distributed random numbers 0 < x < 1
  *	GMT_nrand:	Normally distributed random numbers from N(0,1)
  *	GMT_corrcoeff:	Correlation coefficient.
+ *	GMT_psi:	Digamma (psi) function.
+ *	GMT_PvQv:	Legendre functions Pv and Qv for imaginary v and real x (-1/+1).
+ *	GMT_factorial:	Factorials.
  */
 
 #define GMT_WITH_NO_PS
@@ -86,8 +91,8 @@
 
 int GMT_inc_beta (double a, double b, double x, double *ibeta);
 int GMT_ln_gamma_r (double x, double *lngam);
-int GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob, int iside);
-int GMT_student_t_a (double t, int n, double *prob);
+int GMT_f_test_new (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob, int iside);
+int GMT_student_t_a (double t, GMT_LONG n, double *prob);
 double GMT_ln_gamma (double xx);
 double GMT_cf_beta (double a, double b, double x);
 double GMT_bei (double x);
@@ -95,7 +100,7 @@ double GMT_kei (double x);
 double GMT_ber (double x);
 double GMT_ker (double x);
 double GMT_plm (int l, int m, double x);
-double GMT_factorial (int n);
+double GMT_factorial (GMT_LONG n);
 double GMT_i0 (double x);
 double GMT_i1 (double x);
 double GMT_in (int n, double x);
@@ -133,12 +138,12 @@ double GMT_yn (int n, double x);
 void sincos (double a, double *s, double *c);
 #endif
 
-int	GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob, int iside)
+int	GMT_f_test_new (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob, int iside)
 {
 	/* Given chisq1 and chisq2, random variables distributed as chi-square
 		with nu1 and nu2 degrees of freedom, respectively, except that
 		chisq1 is scaled by var1, and chisq2 is scaled by var2, let
-		the null hypothesis, H0, be that var1 = var2.  This routine 
+		the null hypothesis, H0, be that var1 = var2.  This routine
 		assigns prob, the probability that we can reject H0 in favor
 		of a new hypothesis, H1, according to iside:
 			iside=+1 means H1 is that var1 > var2
@@ -156,7 +161,7 @@ int	GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob
 	*/
 
 	double	q;	/* The probability from GMT_f_q(), which is the prob
-				that H0 should be retained even though 
+				that H0 should be retained even though
 				chisq1/nu1 > chisq2/nu2.  */
 
 	if (chisq1 <= 0.0 || chisq2 <= 0.0 || nu1 < 1 || nu2 < 1) {
@@ -193,7 +198,7 @@ int	GMT_f_test (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 		chisq2 and nu2.  If these are independent
 		and we form the ratio
 		F = max(chisq1,chisq2)/min(chisq1,chisq2)
-		then we can ask what is the probability 
+		then we can ask what is the probability
 		that an F greater than this would occur
 		by chance.  It is this probability that
 		is returned in prob.  When prob is small,
@@ -236,7 +241,7 @@ int	GMT_f_test (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 	return(0);
 }
 
-int	GMT_sig_f (double chi1, int n1, double chi2, int n2, double level, double *prob)
+int	GMT_sig_f (double chi1, GMT_LONG n1, double chi2, GMT_LONG n2, double level, double *prob)
 {
 	/* Returns TRUE if chi1/n1 significantly less than chi2/n2
 		at the level level.  Returns FALSE if:
@@ -416,12 +421,12 @@ double	GMT_cf_beta (double a, double b, double x)
 void	GMT_gamma_ser (double *gamser, double a, double x, double *gln) {
 	/* Returns the incomplete gamma function P(a,x) by series rep.
 	 * Press et al, gser() */
-	 
+
 	int n;
 	double sum, del, ap;
-	 
+
 	GMT_ln_gamma_r (a, gln);
-	 
+
 	if (x < 0.0) {
 		fprintf (stderr, "GMT DOMAIN ERROR:  x < 0 in GMT_gamma_ser(x)\n");
 		*gamser = GMT_d_NaN;
@@ -496,12 +501,12 @@ double GMT_gammq (double a, double x) {
 
 /*
  * Kelvin-Bessel functions, ber, bei, ker, kei.  ber(x) and bei(x) are even;
- * ker(x) and kei(x) are defined only for x > 0 and x >=0, respectively.  
+ * ker(x) and kei(x) are defined only for x > 0 and x >=0, respectively.
  * For x <= 8 we use polynomial approximations in Abramowitz & Stegun.
  * For x > 8 we use asymptotic series of Russell, quoted in Watson (Theory
  * of Bessel Functions).
  */
- 
+
 double GMT_ber (double x)
 {
 	double t, rxsq, alpha, beta;
@@ -604,7 +609,7 @@ double GMT_ker (double x)
 		t *= t;
 		t *= t;  /* t = pow(x/8, 4)  */
 		return (-log(0.5 * x) * GMT_ber(x) + 0.25 * M_PI * GMT_bei(x)
-			-0.57721566  + t * (
+			-M_EULER  + t * (
 			-59.05819744 + t * (
 			171.36272133 + t * (
 			-60.60977451 + t * (
@@ -737,7 +742,7 @@ double GMT_in (int n, double x)
 	tox = 2.0 / fabs (x);
 	bip = res = 0.0;
 	bi = 1.0;
-	m = 2 * (n + irint (sqrt (IACC * n)));
+	m = 2 * (n + irint (sqrt ((double)(IACC * n))));
 	for (j = m; j >= 1; j--) {
 		bim = bip + ((double)j) * tox * bi;
 		bip = bi;
@@ -765,7 +770,7 @@ double GMT_k0 (double x)
 	if (x <= 2.0) {
 		y = 0.25 * x * x;
 		z = x * x / 14.0625;
-		res = (-log(0.5*x) * (1.0 + z * (3.5156229 + z * (3.0899424 + z * (1.2067492 + z * (0.2659732 + z * (0.360768e-1 + z * 0.45813e-2))))))) + (-0.5772156 + y * (0.42278420 + y * (0.23069756 + y * (0.3488590e-1 + y * (0.262698e-2 + y * (0.10750e-3 + y * 0.74e-5))))));
+		res = (-log(0.5*x) * (1.0 + z * (3.5156229 + z * (3.0899424 + z * (1.2067492 + z * (0.2659732 + z * (0.360768e-1 + z * 0.45813e-2))))))) + (-M_EULER + y * (0.42278420 + y * (0.23069756 + y * (0.3488590e-1 + y * (0.262698e-2 + y * (0.10750e-3 + y * 0.74e-5))))));
 	}
 	else {
 		y = 2.0 / x;
@@ -816,12 +821,19 @@ double GMT_kn (int n, double x)
 
 double GMT_plm (int l, int m, double x)
 {
+	/* Unnormalized associated Legendre polynomial of degree l and order m, including
+	 * Condon-Shortley phase (-1)^m */
 	double fact, pll = 0, pmm, pmmp1, somx2;
 	int i, ll;
 
 	/* x is cosine of colatitude and must be -1 <= x <= +1 */
 	if (fabs(x) > 1.0) {
-		fprintf (stderr, "GMT DOMAIN ERROR:  fabs(x) > 1.0 in GMT_plm(x)\n");
+		fprintf (stderr, "%s: Error: |x| > 1.0 in GMT_plm_bar\n", GMT_program);
+		return (GMT_d_NaN);
+	}
+
+	if (m < 0 || m > l) {
+		fprintf (stderr, "%s: Error: GMT_plm requires 0 <= m <= l.\n", GMT_program);
 		return (GMT_d_NaN);
 	}
 
@@ -848,6 +860,115 @@ double GMT_plm (int l, int m, double x)
 	return (pll);
 }
 
+double GMT_plm_bar (int l, int m, double x, BOOLEAN ortho)
+{
+	/* This function computes the normalized associated Legendre function of x for degree
+	 * l and order m. u must be in the range [-1;1] and 0 <= |m| <= l.
+	 * The routine is largely based on the second modified forward column method described in
+	 * Holmes and Featherstone (2002). It is stable up to very high degree and order
+	 * (at least 3000). This is achieved by individually computing the sectorials to P_m,m
+	 * and then iterate up to P_l,m divided by P_m,m and the scale factor 1e280.
+	 * Eventually, the result is multiplied again with these to terms.
+	 *
+	 * When ortho=FALSE, normalization is according to the geophysical convention.
+	 * - The Condon-Shortley phase (-1)^m is NOT included.
+	 * - The normalization factor is sqrt(k*(2l+1)*(l-m)!/(l+m)!) with k=2 except for m=0: k=1.
+	 * - The integral of Plm**2 over [-1;1] is 2*k.
+	 * - The integrals of (Plm*cos(m*lon))**2 and (Plm*sin(m*lon))**2 over a sphere are 4 pi.
+	 *
+	 * When ortho=TRUE, the results are orthonormalized.
+	 * - The Condon-Shortley phase (-1)^m is NOT included.
+	 * - The normalization factor is sqrt((2l+1)*(l-m)!/(l+m)!/(4*pi)).
+	 * - The integral of Plm**2 over [-1;1] is 1/(2*pi).
+	 * - The integral of (Plm*exp(i*m*lon))**2 over a sphere is 1.
+	 *
+	 * When called with -m, the Condon-Shortley phase will be included.
+	 *
+	 * Note that the orthonormalized form produces an integral of 1 when using imaginary terms
+	 * for longitude. In geophysics, the imaginary components are split into cosine and sine terms
+	 * EACH of which have an average power of 1 over the sphere (i.e. an integral of 4 pi).
+	 *
+	 * This routine could be further expanded to produce an array of P_m,m through P_l,m. This
+	 * would be practical if Legendre functions of various degrees are to be computed at the same
+	 * latitude. Also, differentials could be added.
+	 *
+	 * Reference:
+	 * S. A. Holmes and W. E. Featherstone. A unified approach to the Clenshaw summation and the
+	 * recursive computation of very high degree and order normalised associated Legendre functions.
+	 * Journal of Geodesy, 76, 279-299, 2002. doi:10.1007/s00190-002-0216-2.
+	 */
+	int i;
+	BOOLEAN csphase = FALSE;
+	double scalef=1.0e280, u, r, pmm, pmm0, pmm1, pmm2;
+
+	/* x is cosine of colatitude (sine of latitude) and must be -1 <= x <= +1 */
+
+	if (fabs(x) > 1.0) {
+		fprintf (stderr, "%s: Error: |x| > 1.0 in GMT_plm_bar\n", GMT_program);
+		return (GMT_d_NaN);
+	}
+
+	/* If m is negative, include Condon-Shortley phase */
+
+	if (m < 0) {
+		csphase = TRUE;
+		m = -m;
+	}
+
+	if (m > l) {
+		fprintf (stderr, "%s: Error: GMT_plm_bar requires 0 <= m <= l.\n", GMT_program);
+		return (GMT_d_NaN);
+	}
+
+	/* u is sine of colatitude (cosine of latitude) so that 0 <= s <= 1 */
+
+	u = d_sqrt ((1.0 - x)*(1.0 + x));
+
+	/* Initialize P_00=1 and compute up to P_mm using recurrence relation.
+	   The result is a normalisation factor: sqrt((2l+1)(l-m)!/(l+m)!) */
+
+	pmm = 1.0;
+	for (i = 1; i <= m; i++) pmm = d_sqrt (1.0 + 0.5/i) * u * pmm;
+
+	/* If orthonormalization is requested: multiply by sqrt(1/4pi)
+	   In case of geophysical convertion : multiply by sqrt(2-delta_0m) */
+
+	if (ortho) {
+		pmm *= 0.5 / d_sqrt(M_PI);
+	}
+	else if (m != 0)
+		pmm *= d_sqrt(2.0);
+
+	/* If C-S phase is requested, apply it now */
+
+	if ((m & 1) && csphase) pmm = -pmm;
+
+	/* If l==m, we are done */
+
+	if (l == m) return (pmm);
+
+	/* In the next section all P_l,m are divided by P_m,m and scaled by factor scalef.
+	   First compute P_m+1,m / P_m,m */
+
+	pmm0 = 1.0/scalef;
+	pmm1 = pmm0 * x * d_sqrt ((double)(2*m + 3));
+
+	/* Use second modified column forward recurrence relation to compute P_m+2,m / P_m,m */
+
+	for (i = m+2; i <= l; i++) {
+		r = (2*i+1.0) / (i+m) / (i-m);
+		pmm2 = x * pmm1 * d_sqrt(r*(2*i-1)) - pmm0 * d_sqrt(r*(i-m-1)*(i+m-1)/(2*i-3));
+		pmm0 = pmm1;
+		pmm1 = pmm2;
+	}
+
+	/* Return P_l,m */
+
+	pmm1 *= pmm;
+	pmm1 *= scalef;
+	return (pmm1);
+}
+
 /* GMT_sinc (x) calculates the sinc function */
 
 double GMT_sinc (double x)
@@ -859,9 +980,9 @@ double GMT_sinc (double x)
 
 /* GMT_factorial (n) calculates the factorial n! */
 
-double GMT_factorial (int n)
+double GMT_factorial (GMT_LONG n)
 {
-	int i;
+	GMT_LONG i;
 	double val = 1.0;
 
 	if (n < 0) {
@@ -890,8 +1011,8 @@ double GMT_dilog (double x)
 	   between two points on a sphere such that their Gram
 	   matrix expression (see Parker) was different from 1; this
 	   is smallest epsilon such that dilog(epsilon) - dilog(0)
-	   != 0.0.  This turned out to be very much smaller than I 
-	   would have guessed from the apparent e-15 accuracy of 
+	   != 0.0.  This turned out to be very much smaller than I
+	   would have guessed from the apparent e-15 accuracy of
 	   Parker's polynomial.  So I think this is very accurate.
 
 	   If this is called with x < 0, it returns dilog(0) and
@@ -912,23 +1033,23 @@ double GMT_dilog (double x)
 	if (x < 0.5) {
 		y = -log (1.0 - x);
 		ysq = y * y;
-		z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 + 
-			ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 + 
+		z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 +
+			ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 +
 			ysq * (-9.1764954e-08 + 1.798670e-09 * ysq))))));
 		return (pisqon6 - z + y * log (x));
 	}
 	if (x < 2.0) {
 		y = -log (x);
 		ysq = y * y;
-		z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 + 
-			ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 + 
+		z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 +
+			ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 +
 			ysq * (-9.1764954e-08 + 1.798670e-09 * ysq))))));
 		return (z);
 	}
 	y = log (x);
 	ysq = y * y;
-	z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 + 
-		ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 + 
+	z = y * (1.0 + y * (-0.25 + y * (0.027777777777213 +
+		ysq * (-2.7777776990e-04 + ysq * (4.724071696e-06 +
 		ysq * (-9.1764954e-08 + 1.798670e-09 * ysq))))));
 	return (-z - 0.5 * ysq);
 }
@@ -952,7 +1073,7 @@ double GMT_erfinv (double y)
 
 	if (fy > 1.0) return (GMT_d_NaN);	/* Outside domain */
 
-	if (fabs (1.0 - fy) < GMT_CONV_LIMIT) {	/* Close to +- Inf */
+	if (GMT_IS_ZERO (1.0 - fy)) {	/* Close to +- Inf */
 		return (copysign (DBL_MAX, y));
 	}
 
@@ -1147,7 +1268,7 @@ double	GMT_strtod (const char *s, char **ends) {
 	routine.  If the library routine exists,
 	it should be used, as this one will probably
 	be slower.  Also, the library routine has
-	ways of dealing with LOCALE info on the 
+	ways of dealing with LOCALE info on the
 	radix character, and error setting for
 	over and underflows.  Here, I rely on atof()
 	to do that.
@@ -1225,7 +1346,7 @@ double	GMT_strtod (const char *s, char **ends) {
 }
 #endif
 
-int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
+int     GMT_f_q (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob)
 {
 	/* Routine to compute Q(F, nu1, nu2) = 1 - P(F, nu1, nu2), where nu1
 		and nu2 are positive integers, chisq1 and chisq2 are random
@@ -1282,7 +1403,7 @@ int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 	return (0);
 }
 
-int	GMT_student_t_a(double t, int n, double *prob)
+int	GMT_student_t_a(double t, GMT_LONG n, double *prob)
 {
 	/* Probability integral called A(t,n) by Abramowitz &
 	Stegun for the student's t distribution with n degrees
@@ -1300,14 +1421,14 @@ int	GMT_student_t_a(double t, int n, double *prob)
 	if t < 0.  Otherwise it sets *prob = the probability
 	fabs(tau) <= t and returns (0).
 
-	As n -> oo, we can replace this function with 
+	As n -> oo, we can replace this function with
 	erf (t / M_SQRT2).  However, it isn't clear how large
 	n has to be to make this a good approximation.  I
-	consulted six books; one of them suggested this 
+	consulted six books; one of them suggested this
 	approximation for n >= 30, but all the others did not
 	say when to use this approximation (A&S, in particular,
-	does not say).  I tried some numerical experiments 
-	which suggested that the relative error in this 
+	does not say).  I tried some numerical experiments
+	which suggested that the relative error in this
 	approximation would be < 0.01 for n > 30, all t, but
 	I also found that the expression here is stable to
 	large n and large t, so I decided to leave it as is.
@@ -1316,7 +1437,7 @@ int	GMT_student_t_a(double t, int n, double *prob)
 */
 
 	double	theta, s, c, csq, term, sum;
-	int	k, kstop, kt, kb;
+	GMT_LONG	k, kstop, kt, kb;
 
 	if (t < 0.0 || n <= 0) {
 		fprintf (stderr, "GMT_student_t_a:  Bad argument(s).\n");
@@ -1411,7 +1532,7 @@ double GMT_zcrit (double alpha)
  *
  * Function to invert the cumulative normal probability
  * function.  If z is a standardized normal random deviate,
- * and Q(z) = p is the cumulative Gaussian probability 
+ * and Q(z) = p is the cumulative Gaussian probability
  * function, then z = qsnorm(p).
  *
  * Note that 0.0 < p < 1.0.  Data values outside this range
@@ -1465,7 +1586,7 @@ double GMT_tcrit (double alpha, double nu)
 {
 	/* Critical values for Student t-distribution */
 
-	int NU;
+	GMT_LONG NU;
 	BOOLEAN done;
 	double t_low, t_high, t_mid, p_high, p_mid, p, sign;
 
@@ -1479,7 +1600,7 @@ double GMT_tcrit (double alpha, double nu)
 	}
 	t_low = GMT_zcrit (alpha);
 	t_high = 5.0;
-	NU = irint(nu);
+	NU = (GMT_LONG)irint(nu);
 	GMT_student_t_a (t_high, NU, &p_high);
 	while (p_high < p) {	/* Must pick higher starting point */
 		t_high *= 2.0;
@@ -1492,7 +1613,7 @@ double GMT_tcrit (double alpha, double nu)
 	while (!done) {
 		t_mid = 0.5 * (t_low + t_high);
 		GMT_student_t_a (t_mid, NU, &p_mid);
-		if (fabs (p_mid - p) < GMT_CONV_LIMIT) {
+		if (GMT_IS_ZERO (p_mid - p)) {
 			done = TRUE;
 		}
 		else if (p_mid > p) {	/* Too high */
@@ -1527,7 +1648,7 @@ double GMT_chi2crit (double alpha, double nu)
 	while (!done) {
 		chi2_mid = 0.5 * (chi2_low + chi2_high);
 		GMT_chi2 (chi2_mid, nu, &p_mid);
-		if (fabs (p_mid - p) < GMT_CONV_LIMIT) {
+		if (GMT_IS_ZERO (p_mid - p)) {
 			done = TRUE;
 		}
 		else if (p_mid < p) {	/* Too high */
@@ -1544,7 +1665,7 @@ double GMT_Fcrit (double alpha, double nu1, double nu2)
 {
 	/* Critical values for F-distribution */
 
-	int NU1, NU2;
+	GMT_LONG NU1, NU2;
 	BOOLEAN done;
 	double F_low, F_high, F_mid, p_high, p_mid, p, chisq1, chisq2;
 	void F_to_ch1_ch2 (double F, double nu1, double nu2, double *chisq1, double *chisq2);
@@ -1553,8 +1674,8 @@ double GMT_Fcrit (double alpha, double nu1, double nu2)
 	p = 1.0 - alpha;
 	F_low = 0.0;
 	F_to_ch1_ch2 (F_high, nu1, nu2, &chisq1, &chisq2);
-	NU1 = irint (nu1);
-	NU2 = irint (nu2);
+	NU1 = (GMT_LONG)irint (nu1);
+	NU2 = (GMT_LONG)irint (nu2);
 	GMT_f_q (chisq1, NU1, chisq2, NU2, &p_high);
 	while (p_high > p) {	/* Must pick higher starting point */
 		F_high *= 2.0;
@@ -1569,7 +1690,7 @@ double GMT_Fcrit (double alpha, double nu1, double nu2)
 		F_mid = 0.5 * (F_low + F_high);
 		F_to_ch1_ch2 (F_mid, nu1, nu2, &chisq1, &chisq2);
 		GMT_f_q (chisq1, NU1, chisq2, NU2, &p_mid);
-		if (fabs (p_mid - p) < GMT_CONV_LIMIT) {
+		if (GMT_IS_ZERO (p_mid - p)) {
 			done = TRUE;
 		}
 		else if (p_mid < p) {	/* Too high */
@@ -1664,7 +1785,7 @@ double GMT_jn (int n, double x)
 	if (n == 1) return (GMT_j1 (x));
 
 	ax = fabs (x);
-	if (ax <= GMT_CONV_LIMIT) return (0.0);
+	if (GMT_IS_ZERO (ax)) return (0.0);
 
 	if (ax > (double)n) {	/* Upwards recurrence */
 		tox = 2.0 / ax;
@@ -1860,7 +1981,7 @@ double GMT_nrand (void) {
 	 * Press et al, Numerical Recipes, 2nd edition.  Will
 	 * return values that have zero mean and unit variance.
 	 */
-	 
+
 	static int iset = 0;
 	static double gset;
 	double fac, r, v1, v2;
@@ -1876,7 +1997,7 @@ double GMT_nrand (void) {
 
 		/* Now make Box-Muller transformation to get two normal deviates.  Return
 		 * one and save the other for the next time GMT_nrand is called */
-	 
+
 		gset = v1 * fac;
 		iset = 1;	/* Set flag for next time */
 		return (v2 * fac);
@@ -1891,7 +2012,7 @@ double GMT_lrand (void) {
 	/* Laplace random number generator.  As nrand, it will
 	 * return values that have zero mean and unit variance.
 	 */
-	 
+
 	double rand_0_to_1;
 
 	rand_0_to_1 = GMT_rand ();	/* Gives uniformly distributed random values in 0-1 range */
@@ -1901,7 +2022,7 @@ double GMT_lrand (void) {
 void GMT_chi2 (double chi2, double nu, double *prob) {
 	/* Evaluate probability that chi2 will exceed the
 	 * theoretical chi2 by chance. */
- 
+
  	*prob = GMT_gammq (0.5 * nu, 0.5 * chi2);
 }
 
@@ -1985,7 +2106,7 @@ double	GMT_log1p (double x) {
 
 	u = 1.0 + x;
 
-	if (u == 1.0) 
+	if (u == 1.0)
 		return (x);
 	else
 		return ( log(u) * (x/(u - 1.0) ) );
@@ -2007,23 +2128,23 @@ double	GMT_atanh (double x) {
 
 #endif
 
-int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial, double *med)
+int GMT_median (double *x, GMT_LONG n, double xmin, double xmax, double m_initial, double *med)
 {
 	double	lower_bound, upper_bound, m_guess, t_0, t_1, t_middle;
 	double	lub, glb, xx, temp;
-	size_t	i, n_above, n_below, n_equal, n_lub, n_glb;
-	int	finished = FALSE;
-	int	iteration = 0;
+	GMT_LONG	i, n_above, n_below, n_equal, n_lub, n_glb, one;	/* These must be signed integers */
+	GMT_LONG	iteration = 0;
+	BOOLEAN	finished = FALSE;
 
-	if (n == 0) {
+	if (n == (GMT_LONG)0) {
 		*med = m_initial;
 		return (1);
 	}
-	if (n == 1) {
+	if (n == (GMT_LONG)1) {
 		*med = x[0];
 		return (1);
 	}
-	if (n == 2) {
+	if (n == (GMT_LONG)2) {
 		*med = 0.5 * (x[0] + x[1]);
 		return (1);
 	}
@@ -2031,13 +2152,14 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 	m_guess = m_initial;
 	lower_bound = xmin;
 	upper_bound = xmax;
-	t_0 = 0;
-	t_1 = n - 1;
-	t_middle = 0.5 * (n - 1);
+	one = (GMT_LONG)1;
+	t_0 = 0.0;
+	t_1 = (double)(n - one);
+	t_middle = 0.5 * t_1;
 
 	do {
 
-		n_above = n_below = n_equal = n_lub = n_glb = 0;
+		n_above = n_below = n_equal = n_lub = n_glb = (GMT_LONG)0;
 		lub = xmax;
 		glb = xmin;
 
@@ -2051,7 +2173,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 				n_above++;
 				if (xx < lub) {
 					lub = xx;
-					n_lub = 1;
+					n_lub = one;
 				}
 				else if (xx == lub) {
 					n_lub++;
@@ -2061,7 +2183,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 				n_below++;
 				if (xx > glb) {
 					glb = xx;
-					n_glb = 1;
+					n_glb = one;
 				}
 				else if (xx == glb) {
 					n_glb++;
@@ -2075,12 +2197,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 
 		if ((labs((long)(n_above - n_below))) <= (long)n_equal) {
 
-			if (n_equal) {
-				*med = m_guess;
-			}
-			else {
-				*med = 0.5 * (lub + glb);
-			}
+			*med = (n_equal) ? m_guess : 0.5 * (lub + glb);
 			finished = TRUE;
 		}
 		else if ((labs ((long)((n_above - n_lub) - (n_below + n_equal)))) < (long)n_lub) {
@@ -2098,14 +2215,14 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 		else if ( n_above > (n_below + n_equal) ) {  /* Guess is too low  */
 
 			lower_bound = m_guess;
-			t_0 = n_below + n_equal - 1;
+			t_0 = (double)(n_below + n_equal - one);
 			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
 			m_guess = (temp > lub) ? temp : lub;	/* Move guess at least to lub  */
 		}
 		else if ( n_below > (n_above + n_equal) ) {  /* Guess is too high  */
 
 			upper_bound = m_guess;
-			t_1 = n_below + n_equal - 1;
+			t_1 = (double)(n_below + n_equal - one);
 			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
 			m_guess = (temp < glb) ? temp : glb;	/* Move guess at least to glb  */
 		}
@@ -2120,9 +2237,9 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 	return (iteration);
 }
 
-int GMT_mode (double *x, size_t n, size_t j, int sort, int mode_selection, int *n_multiples, double *mode_est)
+int GMT_mode (double *x, GMT_LONG n, GMT_LONG j, int sort, int mode_selection, int *n_multiples, double *mode_est)
 {
-	size_t	i, istop;
+	GMT_LONG	i, istop;
 	int multiplicity;
 	double	mid_point_sum = 0.0, length, short_length = DBL_MAX, this_mode;
 
@@ -2175,10 +2292,10 @@ int GMT_mode (double *x, size_t n, size_t j, int sort, int mode_selection, int *
 	return (0);
 }
 
-int GMT_mode_f (float *x, size_t n, size_t j, int sort, int mode_selection, int *n_multiples, double *mode_est)
+int GMT_mode_f (float *x, GMT_LONG n, GMT_LONG j, int sort, int mode_selection, int *n_multiples, double *mode_est)
 {
-	size_t	i, istop;
-	int multiplicity;
+	GMT_LONG	i, istop;
+	GMT_LONG multiplicity;
 	double	mid_point_sum = 0.0, length, short_length = FLT_MAX, this_mode;
 
 	if (n == 0) return (0);
@@ -2231,12 +2348,12 @@ int GMT_mode_f (float *x, size_t n, size_t j, int sort, int mode_selection, int
 
 /* Replacement slower functions until we figure out the problem with the algorithm */
 
-void GMT_getmad (double *x, size_t n, double location, double *scale)
+void GMT_getmad (double *x, GMT_LONG n, double location, double *scale)
 {
-	size_t i;
+	GMT_LONG i;
 	double *dev, med;
 
-	dev = (double *) GMT_memory (VNULL, n, sizeof(double), GMT_program);
+	dev = (double *) GMT_memory (VNULL, (size_t)n, sizeof(double), GMT_program);
 	for (i = 0; i < n; i++) dev[i] = fabs (x[i] - location);
 	qsort ((void *)dev, (size_t)n, sizeof (double), GMT_comp_double_asc);
 	for (i = n; GMT_is_dnan (dev[i-1]) && i > 1; i--);
@@ -2248,13 +2365,13 @@ void GMT_getmad (double *x, size_t n, double location, double *scale)
 	*scale = 1.4826 * med;
 }
 
-void GMT_getmad_f (float *x, size_t n, double location, double *scale)
+void GMT_getmad_f (float *x, GMT_LONG n, double location, double *scale)
 {
-	size_t i;
+	GMT_LONG i;
 	float *dev;
 	double med;
 
-	dev = (float *) GMT_memory (VNULL, n, sizeof(float), GMT_program);
+	dev = (float *) GMT_memory (VNULL, (size_t)n, sizeof(float), GMT_program);
 	for (i = 0; i < n; i++) dev[i] = (float) fabs ((double)(x[i] - location));
 	qsort ((void *)dev, (size_t)n, sizeof (float), GMT_comp_float_asc);
 	for (i = n; GMT_is_fnan (dev[i-1]) && i > 1; i--);
@@ -2266,12 +2383,12 @@ void GMT_getmad_f (float *x, size_t n, double location, double *scale)
 	*scale = 1.4826 * med;
 }
 
-void GMT_getmad_BROKEN (double *x, int n, double location, double *scale)
+void GMT_getmad_BROKEN (double *x, GMT_LONG n, double location, double *scale)
 {
 	/* Compute MAD (Median Absolute Deviation) for a double data set */
 
 	double	e_low, e_high, error, last_error;
-	int	i_low, i_high, n_dev, n_dev_stop;
+	GMT_LONG	i_low, i_high, n_dev, n_dev_stop;
 
 	i_low = 0;
 	while (i_low < n && x[i_low] <= location) i_low++;
@@ -2329,12 +2446,12 @@ void GMT_getmad_BROKEN (double *x, int n, double location, double *scale)
 	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
 }
 
-void GMT_getmad_f_BROKEN (float *x, int n, double location, double *scale)
+void GMT_getmad_f_BROKEN (float *x, GMT_LONG n, double location, double *scale)
 {
 	/* Compute MAD (Median Absolute Deviation) for a float data set */
 
 	double	e_low, e_high, error, last_error;
-	int	i_low, i_high, n_dev, n_dev_stop;
+	GMT_LONG	i_low, i_high, n_dev, n_dev_stop;
 
 	i_low = 0;
 	while (i_low < n && x[i_low] <= location) i_low++;
@@ -2393,7 +2510,7 @@ void GMT_getmad_f_BROKEN (float *x, int n, double location, double *scale)
 	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
 }
 
-double GMT_extreme (double x[], size_t n, double x_default, int kind, int way)
+double GMT_extreme (double x[], GMT_LONG n, double x_default, int kind, int way)
 {
 	/* Returns the extreme value in the x array according to:
 	*  kind: -1 means only consider negative values.
@@ -2405,7 +2522,7 @@ double GMT_extreme (double x[], size_t n, double x_default, int kind, int way)
 	* If kind == 0 we expect x_default to be set so that x[0] will reset x_select.
 	*/
 
-	size_t i, k;
+	GMT_LONG i, k;
 	double x_select = GMT_d_NaN;
 
 	for (i = k = 0; i < n; i++) {
@@ -2425,10 +2542,10 @@ int GMT_chebyshev (double x, int n, double *t)
 	/* Calculates the n'th Chebyshev polynomial at x */
 
 	double x2, a, b;
-	
+
 	if (n < 0) GMT_err_pass (GMT_CHEBYSHEV_NEG_ORDER, "");
 	if (fabs (x) > 1.0) GMT_err_pass (GMT_CHEBYSHEV_BAD_DOMAIN, "");
-	
+
 	switch (n) {	/* Testing the order of the polynomial */
 		case 0:
 			*t = 1.0;
@@ -2452,19 +2569,19 @@ int GMT_chebyshev (double x, int n, double *t)
 			*t = 2.0 * x * a - b;
 			break;
 	}
-	
+
 	return (GMT_NOERROR);
 }
 
-double GMT_corrcoeff (double *x, double *y, size_t n, int mode)
+double GMT_corrcoeff (double *x, double *y, GMT_LONG n, int mode)
 {
 	/* Returns plain correlation coefficient, r.
 	 * If mode = 1 we assume mean(x) = mean(y) = 0.
 	 */
 
-	size_t i, n_use;
+	GMT_LONG i, n_use;
 	double xmean = 0.0, ymean = 0.0, dx, dy, vx, vy, vxy, r;
-	
+
 	if (mode == 0) {
 		for (i = n_use = 0; i < n; i++) {
 			if (GMT_is_dnan (x[i]) || GMT_is_dnan (y[i])) continue;
@@ -2473,10 +2590,10 @@ double GMT_corrcoeff (double *x, double *y, size_t n, int mode)
 			n_use++;
 		}
 		if (n_use == 0) return (GMT_d_NaN);
-		xmean /= (double)n_use;	
+		xmean /= (double)n_use;
 		ymean /= (double)n_use;
 	}
-	
+
 	vx = vy = vxy = 0.0;
 	for (i = n_use = 0; i < n; i++) {
 		if (GMT_is_dnan (x[i]) || GMT_is_dnan (y[i])) continue;
@@ -2486,20 +2603,20 @@ double GMT_corrcoeff (double *x, double *y, size_t n, int mode)
 		vy += dy * dy;
 		vxy += dx * dy;
 	}
-	
+
 	r = vxy / sqrt (vx * vy);
 	return (r);
 }
 
-double GMT_corrcoeff_f (float *x, float *y, size_t n, int mode)
+double GMT_corrcoeff_f (float *x, float *y, GMT_LONG n, int mode)
 {
 	/* Returns plain correlation coefficient, r.
 	 * If mode = 1 we assume mean(x) = mean(y) = 0.
 	 */
 
-	size_t i, n_use;
+	GMT_LONG i, n_use;
 	double xmean = 0.0, ymean = 0.0, dx, dy, vx, vy, vxy, r;
-	
+
 	if (mode == 0) {
 		for (i = n_use = 0; i < n; i++) {
 			if (GMT_is_fnan (x[i]) || GMT_is_fnan (y[i])) continue;
@@ -2508,10 +2625,10 @@ double GMT_corrcoeff_f (float *x, float *y, size_t n, int mode)
 			n_use++;
 		}
 		if (n_use == 0) return (GMT_d_NaN);
-		xmean /= (double)n_use;	
+		xmean /= (double)n_use;
 		ymean /= (double)n_use;
 	}
-	
+
 	vx = vy = vxy = 0.0;
 	for (i = n_use = 0; i < n; i++) {
 		if (GMT_is_fnan (x[i]) || GMT_is_fnan (y[i])) continue;
@@ -2521,7 +2638,303 @@ double GMT_corrcoeff_f (float *x, float *y, size_t n, int mode)
 		vy += dy * dy;
 		vxy += dx * dy;
 	}
-	
+
 	r = vxy / sqrt (vx * vy);
 	return (r);
 }
+
+
+double GMT_quantile (double *x, double q, GMT_LONG n)
+{
+	/* Returns the q'th (q in percent) quantile of x (assumed sorted).
+	 * q is expected to be 0 < q < 100 */
+
+	GMT_LONG i_f;
+	double p, f, df;
+
+	while (n > 1 && GMT_is_dnan (x[n-1])) n--;	/* Skip any NaNs at the end of x */
+	if (n < 1) return (GMT_d_NaN);			/* Need at least 1 point to do something */
+	if (q == 0.0) return (x[0]);			/* 0% quantile == min(x) */
+	if (q == 100.0) return (x[n-1]);		/* 100% quantile == max(x) */
+	f = (n - 1) * q / 100.0;
+	i_f = (GMT_LONG)floor (f);
+	if ((df = (f - (double)i_f)) > 0.0)		/* Must interpolate between the two neighbors */
+		p = x[i_f+1] * df + x[i_f] * (1.0 - df);
+	else						/* Exactly on a node */
+		p = x[i_f];
+
+	return (p);
+}
+
+double GMT_quantile_f (float *x, double q, GMT_LONG n)
+{
+	/* Returns the q'th (q in percent) quantile of x (assumed sorted).
+	 * q is expected to be 0 < q < 100 */
+
+	GMT_LONG i_f;
+	double p, f, df;
+
+	while (n > 1 && GMT_is_fnan (x[n-1])) n--;	/* Skip any NaNs at the end of x */
+	if (n < 1) return (GMT_d_NaN);			/* Need at least 1 point to do something */
+	if (q == 0.0) return ((double)x[0]);		/* 0% quantile == min(x) */
+	if (q == 100.0) return ((double)x[n-1]);	/* 100% quantile == max(x) */
+	f = (n - 1) * q / 100.0;
+	i_f = (GMT_LONG)floor (f);
+	if ((df = (f - (double)i_f)) > 0.0)		/* Must interpolate between the two neighbors */
+		p = (double)(x[i_f+1] * df + x[i_f] * (1.0 - df));
+	else						/* Exactly on a node */
+		p = (double)x[i_f];
+
+	return (p);
+}
+
+#define RE 0
+#define IM 1
+
+static void Cmul (double A[], double B[], double C[])
+{	/* Complex multiplication */
+	C[RE] = A[RE]*B[RE] - A[IM]*B[IM];
+	C[IM] = A[RE]*B[IM] + A[IM]*B[RE];
+}
+
+static void Cdiv (double A[], double B[], double C[])
+{	/* Complex division */
+	double denom;
+	denom = B[RE]*B[RE] + B[IM]*B[IM];
+	C[RE] = (A[RE]*B[RE] + A[IM]*B[IM])/denom;
+	C[IM] = (A[IM]*B[RE] - A[RE]*B[IM])/denom;
+}
+
+static double Cabs (double A[])
+{
+	return (hypot (A[RE], A[IM]));
+}
+
+double GMT_psi (double zz[], double p[])
+{
+/* Psi     Psi (or Digamma) function for complex arguments z.
+*
+*                 d
+*        Psi(z) = --log(Gamma(z))
+*                 dz
+*
+* zz[RE] is real and zz[IM] is imaginary component; same for p on output (only if p != NULL).
+* We also return the real component as function result.
+*/
+	static double c[15] = { 0.99999999999999709182, 57.156235665862923517, -59.597960355475491248,
+			14.136097974741747174, -0.49191381609762019978, 0.33994649984811888699e-4,
+			0.46523628927048575665e-4, -0.98374475304879564677e-4, 0.15808870322491248884e-3,
+			-0.21026444172410488319e-3, 0.21743961811521264320e-3, -0.16431810653676389022e-3,
+			0.84418223983852743293e-4, -0.26190838401581408670e-4, 0.36899182659531622704e-5};
+	double z[2], g[2], dx[2], dd[2], d[2], n[2], gg[2], f[2], x0, A[2], B[2], C[2], sx, cx, e;
+	int k;
+
+	if (zz[IM] == 0.0 && rint(zz[RE]) == zz[RE] && zz[RE] <= 0.0) {
+		if (p) { p[RE] = GMT_d_NaN; p[IM] = 0.0;}
+		return (GMT_d_NaN);	/* Singular points */
+	}
+
+	z[RE] = zz[RE];	z[IM] = zz[IM];
+	if ((x0 = z[RE]) < 0.5) {	/* reflection point */
+		z[RE] = 1.0 - z[RE];
+		z[IM] = -z[IM];
+	}
+
+	/* Lanczos approximation */
+
+	g[RE] = 607.0/128.0;	g[IM] = 0.0; /* best results when 4<=g<=5 */
+	n[RE] = d[RE] = n[IM] = d[IM] = 0.0;
+	for (k = 14; k > 0; k--) {
+		A[RE] = 1.0;	A[IM] = 0.0;
+		B[RE] = z[RE] + k - 1.0;	B[IM] = z[IM];
+		Cdiv (A, B, dx);
+		dd[RE] = c[k] * dx[RE];	dd[IM] = c[k] * dx[IM];
+		d[RE] += dd[RE];	d[IM] += dd[IM];
+		Cmul (dd, dx, B);
+		n[RE] -= B[RE];	n[IM] -= B[IM];
+	}
+	d[RE] += c[RE];
+	gg[RE] = z[RE] + g[RE] - 0.5;	gg[IM] = z[IM];
+	Cdiv (n, d, A);
+	Cdiv (g, gg, B);
+	f[RE] = log (hypot(gg[RE], gg[IM])) + A[RE] - B[RE];
+	f[IM] = atan2 (gg[IM], gg[RE])  + A[IM] - B[IM];
+	if (x0 < 0.5) {
+		C[RE] = M_PI * zz[RE];	C[IM] = M_PI * zz[IM];
+		e = exp (-2*C[IM]);	sx = sin (2*C[RE]);	cx = cos (2*C[RE]);
+		A[RE] = -e * sx;	A[IM] = e * cx + 1.0;
+		B[RE] = e * cx - 1.0;	B[IM] = e * sx;
+		Cdiv (A, B, C);
+		f[RE] -= M_PI * C[RE];	f[IM] -= M_PI * C[IM];
+	}
+	if (p) {
+		p[RE] = f[RE];
+		p[IM] = f[IM];
+	}
+	return (f[RE]);
+}
+
+#ifndef M_SQRT_PI
+#define M_SQRT_PI 1.772453850905516
+#endif
+
+#define PV_RE 0
+#define PV_IM 1
+#define QV_RE 2
+#define QV_IM 3
+void GMT_PvQv (double x, double v_ri[], double pq[], int *iter)
+{
+	/* Here, -1 <= x <= +1, v_ri is an imaginary number [r,i], and we return
+	 * the real amd imaginary parts of Pv(x) and Qv(x) in the pq array.
+	 * Based on recipe in An Atlas of Functions */
+
+	BOOLEAN p_set, q_set;
+	double M, L, K, Xn, x2, k, k1, ep, em, sx, cx, fact;
+	double a[2], v[2], vp1[2], G[2], g[2], u[2], t[2], f[2];
+	double R[2], r[2], z[2], s[2], c[2], w[2], tmp[2], X[2], A[2], B[2];
+	
+	*iter = 0;
+    	pq[PV_RE] = pq[QV_RE] = pq[PV_IM] = pq[QV_IM] = 0.0;	/* Initialize all answers to zero first */
+	p_set = q_set = FALSE;
+	if (x == -1 && v_ri[IM] == 0.0) {	/* Check special values for real nu when x = -1 */
+			/* Special Pv(-1) values */
+		if ((v_ri[RE] > 0.0 && fmod (v_ri[RE], 2.0) == 1.0) || (v_ri[RE] < 0.0 && fmod (v_ri[RE], 2.0) == 0.0)) {	/* v = 1,3,5,.. or -2, -4, -6 */
+ 			pq[PV_RE] = -1.0; p_set = TRUE;
+		}
+		else if ((v_ri[RE] >= 0.0 && fmod (v_ri[RE], 2.0) == 0.0) || (v_ri[RE] < 0.0 && fmod (v_ri[RE]+1.0, 2.0) == 0.0)) {	/* v = 0,2,4,6 or -1,-3,-5,-7 */
+ 			pq[PV_RE] = 1.0; p_set = TRUE;
+		}
+		else if (v_ri[RE] > 0.0 && ((fact = v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) < 1.0 && fact > 0.0)) { /* 0 < v < 1, 2 < v < 3, 4 < v < 5, ... */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* -inf */
+		}
+		else if (v_ri[RE] < 1.0 && ((fact = v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) < 1.0 && fact > 0.0)) {	/* -2 < v < -1, -4 < v < -3, -6 < v < -5 */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* -inf */
+		}
+		else if (v_ri[RE] > 1.0 && (v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) > 1.0) {	/* 1 < v < 2, 3 < v < 4, 5 < v < 6, .. */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* +inf */
+		}
+		else if (v_ri[RE] < 2.0 && ( v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) > 1.0) {	/* -3 < v < -2, -5 < v < -4, -7 < v < -6, .. */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* +inf */
+		}
+		/* Special Qv(-1) values */
+		if (v_ri[RE] > 0.0 && fmod (2.0 * v_ri[RE] - 1.0, 4.0) == 0.0) {	/* v = 1/2, 5/2, 9/2, ... */
+   			pq[QV_RE] = -M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] > -1.0 && fmod (2.0 * v_ri[RE] + 1.0, 4.0) == 0.0) {	/* v = -1/2, 3/2, 7/2, ... */
+   			pq[QV_RE] = M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] < -1.0 && fmod (2.0 * v_ri[RE] - 1.0, 4.0) == 0.0) {	/* v = -3/2, -7/2, -11/2, ... */
+   			pq[QV_RE] = -M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] < -2.0 && fmod (2.0 * v_ri[RE] + 1.0, 4.0) == 0.0) {	/* v = -5/2, -9/2, -13/2, ... */
+   			pq[QV_RE] = M_PI_2; q_set = TRUE;
+		}
+		else {	/* Either -inf or +inf */
+   			pq[QV_RE] = GMT_d_NaN; q_set = TRUE;
+		}
+	}
+	else if (x == +1 && v_ri[IM] == 0.0) {	/* Check special values for real nu when x = +1 */
+		pq[PV_RE] = 1.0; p_set = TRUE;
+		pq[QV_RE] = GMT_d_NaN; q_set = TRUE;
+	}
+	if (p_set && q_set) return;
+	
+	/* General case of |x| < 1 */
+
+	a[0] = a[1] = R[RE] = 1.0;	R[IM] = 0.0;
+	v[RE] = v_ri[RE];	v[IM] = v_ri[IM];
+	Cmul (v, v, z);
+	z[RE] = v[RE] - z[RE];	z[IM] = v[IM] - z[IM];
+	K = 4.0 * sqrt (Cabs(z));
+	vp1[RE] = v[RE] + 1.0;	vp1[IM] = v[IM];
+	if ((Cabs(vp1) + floor(vp1[RE])) == 0.0) {
+		a[0] = GMT_d_NaN;
+		a[1] = 0.0;
+		v[RE] = -1 - v[RE];
+		v[IM] = -v[IM];
+	}
+	z[RE] = 0.5 * M_PI * v[RE];	z[IM] = 0.5 * M_PI * v[IM];
+	ep = exp (z[IM]);	em = exp (-z[IM]);
+	sincos (z[RE], &sx, &cx);
+	s[RE] = 0.5 * sx * (em + ep); 
+	s[IM] = -0.5 * cx * (em - ep); 
+	c[RE] = 0.5 * cx * (em + ep); 
+	c[IM] = 0.5 * sx * (em - ep);
+	tmp[RE] = 0.5 + v[RE];	tmp[IM] = v[IM];
+	Cmul (tmp, tmp, w);
+	z[IM] = v[IM];
+	while (v[RE] <= 6.0) {
+		v[RE] = v[RE] + 2.0;
+		z[RE] = v[RE] - 1.0;
+		Cdiv (z, v, tmp);
+		Cmul (R,tmp,r);
+		R[RE] = r[RE];	R[IM] = r[IM];
+	}
+	z[RE] = v[RE] + 1.0;
+	tmp[RE] = 0.25;	tmp[IM] = 0.0;
+	Cdiv (tmp, z, X);
+	tmp[RE] = 0.35 + 6.1 * X[RE];	tmp[IM] = 6.1*X[IM];
+	Cmul (X, tmp, z);
+	z[RE] = 1.0 - 3.0*z[RE];	z[IM] = -3.0*z[IM];
+	Cmul (X, z, tmp);
+	G[RE] = 1.0 + 5.0 * tmp[RE];	G[IM] = 5.0 * tmp[IM];
+	z[RE] = 8.0 * X[RE];	z[IM] = 8.0 * X[IM];
+	M = sqrt(hypot(z[RE], z[IM]));
+	L = 0.5 * atan2 (z[IM], z[RE]);
+	tmp[RE] = M * cos(L);	tmp[IM] = M * sin(L);
+	Cmul (G, X, z);
+	z[RE] = 1.0 - 0.5*z[RE];	z[IM] = -0.5*z[IM];
+	Cmul (X, z, r);
+	r[RE] = 1.0 - r[RE];	r[IM] = -r[IM];
+	Cmul (R, r, z);
+	Cdiv (z, tmp, R);
+	u[RE] = g[RE] = 2.0 * x;	u[IM] = g[IM] = f[IM] = t[IM] = 0.0;
+	f[RE] = t[RE] = 1.0;
+	k = 0.5;
+	x2 = x * x;
+	Xn = 1.0 + (1e8/(1 - x2));
+	k1 = k + 1.0;
+	fact = x2 / (k1*k1 - 0.25);
+	t[RE] = (k*k - w[RE]) * fact;	t[IM] = -w[IM] * fact;
+	k += 1.0;
+	f[RE] += t[RE];	f[IM] += t[IM];
+	k1 = k + 1.0;
+	fact = u[RE] * x2 / (k1*k1 - 0.25);
+	u[RE] = (k*k - w[RE]) * fact;	u[IM] = -w[IM] * fact;
+	k += 1;
+	g[RE] += u[RE];	g[IM] += u[IM];
+	tmp[RE] = Xn * t[RE];	tmp[IM] = Xn * t[IM];
+	while (k < K || Cabs (tmp) > Cabs(f)) {
+		(*iter)++;
+		k1 = k + 1.0;
+		tmp[RE] = k*k - w[RE];	tmp[IM] = -w[IM];	fact = x2 / (k1*k1 - 0.25);
+		Cmul (t, tmp, A);
+		t[RE] = A[RE] * fact;	t[IM] = A[IM] * fact;
+		k += 1.0;
+		k1 = k + 1.0;
+		f[RE] += t[RE];	f[IM] += t[IM];
+		tmp[RE] = k*k - w[RE];	tmp[IM] = -w[IM];	fact = x2 / (k1*k1 - 0.25);
+		Cmul (u, tmp, B);
+		u[RE] = B[RE] * fact;	u[IM] = B[IM] * fact;
+		k += 1.0;
+		g[RE] += u[RE];	g[IM] += u[IM];
+		tmp[RE] = Xn * t[RE];	tmp[IM] = Xn * t[IM];
+	}
+	fact = x2 / (1.0 - x2);
+	f[RE] += t[RE] * fact;	f[IM] += t[IM] * fact;
+	g[RE] += u[RE] * fact;	g[IM] += u[IM] * fact;
+	if (!p_set) {
+		Cmul(s,R,z);
+		Cdiv(c,R,tmp);
+		Cmul (g, z, A);	Cmul (f, tmp, B);
+		pq[PV_RE] = (A[RE] + B[RE])/M_SQRT_PI;
+		pq[PV_IM] = (A[IM] + B[IM])/M_SQRT_PI;
+	}
+	if (!q_set) {
+		Cmul(c,R,z);
+		Cdiv(s,R,tmp);
+		Cmul (g, z, A);	Cmul (f, tmp, B);
+		pq[QV_RE] = a[0]*M_SQRT_PI*(A[RE] - B[RE])/2.0;
+		pq[QV_IM] = a[1]*M_SQRT_PI*(A[IM] - B[IM])/2.0;
+	}
+}
