@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_vector.c,v 1.11 2007/01/30 20:37:08 pwessel Exp $
+ *	$Id: gmt_vector.c,v 1.20 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,7 @@
 
 #define MAX_SWEEPS 50
 
-double GMT_mag3v (double *a);
-
-int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, double *z, int *nrots) {
+int	GMT_jacobi (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, int *nrots) {
 /*
  *
  * Find eigenvalues & eigenvectors of a square symmetric matrix by Jacobi's
@@ -120,7 +118,7 @@ int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, doub
  * Revised:	PW: 12-MAR-1998 for GMT 3.1
  * Revision by WHF Smith, March 03, 2000, to speed up loop indexes.
  */
-	int	p, q, pp, pq, mp1, pm, qm, nsweeps, j, jm, i, k;
+	GMT_LONG	p, q, pp, pq, mp1, pm, qm, nsweeps, j, jm, i, k;
 	double	sum, threshold, g, h, t, theta, c, s, tau;
 
 
@@ -275,7 +273,7 @@ int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, doub
 	/* Return 0 if converged; else print warning and return -1:  */
 
 	if (nsweeps == MAX_SWEEPS) {
-		fprintf (stderr, "GMT_jacobi:  Failed to converge in %d sweeps\n", nsweeps);
+		fprintf (stderr, "GMT_jacobi:  Failed to converge in %ld sweeps\n", nsweeps);
 		return(-1);
 	}
 	return(0);
@@ -490,7 +488,7 @@ double GMT_mag3v (double *a)
 void GMT_normalize3v (double *a)
 {
 	double r_length;
-	r_length = (GMT_mag3v(a));
+	r_length = GMT_mag3v(a);
 	if (r_length != 0.0) {
 		r_length = 1.0 / r_length;
 		a[0] *= r_length;
@@ -540,22 +538,26 @@ void GMT_cart_to_geo (double *alat, double *alon, double *a, int rads)
 	}
 }
 
-int GMT_fix_up_path (double **a_lon, double **a_lat, int n, BOOLEAN greenwich, double step)
-                          
-      
-                  	/* TRUE if we cross Greenwich */
-            		/* Add points when step degrees are exceeded */
+GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double step, int mode)
 {
-	
-	/* Takes pointers to a list of lon/lat pairs and adds auxiliary points if
-	 * the great circle distance between two given points exceeds
-	 * <step> spherical degree [ 1 degree].
+	/* Takes pointers to a list of <n> lon/lat pairs (in degrees) and adds
+	 * auxiliary points if the great circle distance between two given points exceeds
+	 * <step> spherical degree.
+	 * If mode=0: returns points along a great circle
+	 * If mode=1: first follows meridian, then parallel
+	 * If mode=2: first follows parallel, then meridian
+	 * Returns the new number of points (original plus auxiliary).
+	 *
+	 * The original argument "greenwich" became superfluous when the algorithm for
+	 * wrapping the inserted points was altered on 23-May-2007. This was also when
+	 * it was discovered that the calling programs sometimes defined "step" in
+	 * inches of map projection.
 	 */
-	 
-	int i, j, n_tmp, n_insert = 0, n_alloc;
+      
+	GMT_LONG i, j, n_tmp, n_step = 0, n_alloc;
 	double *lon_tmp, *lat_tmp, *old;
 	double a[3], b[3], x[3], *lon, *lat;
-	double c, d, fraction, theta, i_step;
+	double c, d, fraction, theta, minlon, maxlon;
 	
 	lon = *a_lon;
 	lat = *a_lat;
@@ -565,21 +567,82 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, BOOLEAN greenwich, d
 	lat_tmp = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
 	
 	GMT_geo_to_cart (&lat[0], &lon[0], a, TRUE);
-	lon_tmp[0] = (lon[0] >= M_PI) ? lon[0] - 2.0*M_PI : lon[0];	lat_tmp[0] = lat[0];
+	lon_tmp[0] = lon[0];	lat_tmp[0] = lat[0];
 	n_tmp = 1;
 	if (step <= 0.0) step = 1.0;
-	i_step = 1.0 / step;
 	
 	for (i = 1; i < n; i++) {
 		
 		GMT_geo_to_cart (&lat[i],&lon[i], b, TRUE);
-		
-		if ((theta = d_acos (GMT_dot3v (a, b))) == M_PI) {	/* trouble, no unique great circle */
+
+		if (mode == 1) {	/* First follow meridian, then parallel */
+			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i-1]);
+			n_step = irint (theta * R2D / step);
+			for (j = 1; j < n_step; j++) {
+				c = j / (double)n_step;
+				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
+				lat_tmp[n_tmp] = lat[i-1];
+				n_tmp++;
+				if (n_tmp == n_alloc) {
+					n_alloc <<= 1;
+					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+				}
+			}
+			theta = fabs(lat[i]-lat[i-1]);
+			n_step = irint (theta * R2D / step);
+			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
+				c = j / (double)n_step;
+				lon_tmp[n_tmp] = lon[i];
+				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
+				n_tmp++;
+				if (n_tmp == n_alloc) {
+					n_alloc <<= 1;
+					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+				}
+			}
+		}
+
+		else if (mode == 2) {	/* First follow parallel, then meridian */
+			theta = fabs(lat[i]-lat[i-1]);
+			n_step = irint (theta * R2D / step);
+			for (j = 1; j < n_step; j++) {
+				c = j / (double)n_step;
+				lon_tmp[n_tmp] = lon[i-1];
+				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
+				n_tmp++;
+				if (n_tmp == n_alloc) {
+					n_alloc <<= 1;
+					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+				}
+			}
+			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i]);
+			n_step = irint (theta * R2D / step);
+			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
+				c = j / (double)n_step;
+				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
+				lat_tmp[n_tmp] = lat[i];
+				n_tmp++;
+				if (n_tmp == n_alloc) {
+					n_alloc <<= 1;
+					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
+				}
+			}
+		}
+
+		/* Follow great circle */
+		else if ((theta = d_acos (GMT_dot3v (a, b))) == M_PI) {	/* trouble, no unique great circle */
 			if (gmtdefs.verbose) fprintf (stderr, "%s: GMT Warning: Two points in input list are antipodal - no resampling taken place!\n", GMT_program);
 		}
-		else if ((n_insert = (int)floor (theta * R2D * i_step))) {	/* Must insert n_insert points */
-			fraction = step * D2R / theta;
-			for (j = 1; j <= n_insert; j++) {
+
+		else if ((n_step = irint (theta * R2D / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+			fraction = 1.0 / (float) n_step;
+			minlon = MIN(lon[i-1],lon[i]);
+			maxlon = MAX(lon[i-1],lon[i]);
+			for (j = 1; j < n_step; j++) {
 				c = j * fraction;
 				d = 1 - c;
 				x[0] = a[0] * d + b[0] * c;
@@ -587,18 +650,22 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, BOOLEAN greenwich, d
 				x[2] = a[2] * d + b[2] * c;
 				GMT_normalize3v (x);
 				GMT_cart_to_geo (&lat_tmp[n_tmp], &lon_tmp[n_tmp], x, FALSE);
+				if (lon_tmp[n_tmp] < minlon)
+					lon_tmp[n_tmp] += TWO_PI;
+				else if (lon_tmp[n_tmp] > maxlon)
+					lon_tmp[n_tmp] -= TWO_PI;
 				n_tmp++;
 				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
 					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
 				}
 			}
 		}
-		lon_tmp[n_tmp] = (lon[i] >= M_PI) ? lon[i] - 2.0 * M_PI : lon[i];	lat_tmp[n_tmp] = lat[i];
+		lon_tmp[n_tmp] = lon[i];	lat_tmp[n_tmp] = lat[i];
 		n_tmp++;
 		if (n_tmp == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
 			lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
 		}
@@ -615,10 +682,6 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, BOOLEAN greenwich, d
 	GMT_free ((void *) old);
 	for (i = 0; i < n_tmp; i++) {
 		lon[i] *= R2D;
-		if (!greenwich && lon[i] < 0.0)
-			lon[i] += 360.0;
-		else if (greenwich && lon[i] > 180.0)
-			lon[i] -= 360.0;
 		lat[i] *= R2D;
 	}
 	*a_lon = lon;
@@ -626,7 +689,7 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, BOOLEAN greenwich, d
 	return (n_tmp);
 }		
 
-int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
+int GMT_chol_dcmp (double *a, double *d, double *cond, GMT_LONG nr, GMT_LONG n) {
 
 	/* Given a, a symmetric positive definite matrix
 	of size n, and row dimension nr, compute a lower
@@ -655,7 +718,7 @@ int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
 	
 	W H F Smith, 18 Feb 2000.
 */
-	int	i, j, k, ik, ij, kj, kk, nrp1;
+	GMT_LONG	i, j, k, ik, ij, kj, kk, nrp1;
 	double	eigmax, eigmin;
 
 	nrp1 = nr + 1;
@@ -682,7 +745,7 @@ int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
 	return (0);
 }
 
-void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly) {
+void GMT_chol_recover (double *a, double *d, GMT_LONG nr, GMT_LONG n, int nerr, int donly) {
 
 	/* Given a, a symmetric positive definite matrix of row dimension nr,
 	and size n >= abs(nerr), one uses GMT_chol_dcmp() to attempt to find
@@ -717,12 +780,12 @@ void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly)
 	W H F Smith, 18 Feb 2000
 */
 	
-	int	kbad, i, j, ii, ij, ji, nrp1;
+	GMT_LONG	kbad, i, j, ii, ij, ji, nrp1;
 	
 	kbad = abs(nerr) - 1;
 	nrp1 = nr + 1;
 	
-	for (i = 0, ii = 0; i <=kbad; i++, ii += nrp1) {
+	for (i = 0, ii = 0; i <= kbad; i++, ii += nrp1) {
 		a[ii] = d[i];
 	}
 	
@@ -736,7 +799,7 @@ void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly)
 	return;
 }
 
-void GMT_chol_solv (double *a, double *x, double *y, int nr, int n) {
+void GMT_chol_solv (double *a, double *x, double *y, GMT_LONG nr, GMT_LONG n) {
 	/* Given an n by n linear system ax = y, with a a symmetric, 
 	positive-definite matrix, y a known vector, and x an unknown
 	vector, this routine finds x, if a holds the lower-triangular
@@ -754,7 +817,7 @@ void GMT_chol_solv (double *a, double *x, double *y, int nr, int n) {
 	
 	W H F Smith, 18 Feb 2000
 */
-	int	i, j, ij, ji, ii, nrp1;
+	GMT_LONG	i, j, ij, ji, ii, nrp1;
 	
 	nrp1 = nr + 1;
 
