@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtconvert.c,v 1.47 2007/03/12 19:52:26 remko Exp $
+ *	$Id: gmtconvert.c,v 1.52 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,7 @@ struct GMTCONVERT_CTRL {
 	} E;
 	struct F {	/* -F<cols> */
 		BOOLEAN active;
-		short int *col;
+		int *col;
 	} F;
 	struct L {	/* -L */
 		BOOLEAN active;
@@ -64,8 +64,11 @@ struct GMTCONVERT_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i, j, k, pos, start, stop, n_total_read = 0, n_files = 0, n_tot_cols = 0, n_hdr_delay = 0, fno;
-	int n_out = 0, n_rev_alloc = 0, n_rev = 0, error = 0, n_seg = 0, n_in_seg = 0, n_nan, out_seg;
+	int pos, start, stop, fno, n_files = 0, n_out = 0, n_tot_cols = 0;
+	
+	GMT_LONG i, j, k, n_total_read = 0, n_hdr_delay = 0;
+	GMT_LONG n_rev_alloc = 0, n_rev = 0, n_rev_recs = 0, error = 0, n_seg = 0, n_in_seg = 0, n_nan, out_seg;
+	
 	BOOLEAN on, ascii_in_bin_out;
 
 	double	*in, *out = VNULL, *R_out = (double *)NULL;
@@ -221,13 +224,14 @@ int main (int argc, char **argv)
 	out_seg = -1;
 
 	if (Ctrl->A.active) {	/* Do paste and probably cut as well */
-		int *n_expected_fields, *n_fields, n_alloc = GMT_SMALL_CHUNK, all_set = 0;
+		int *n_expected_fields, *n_fields, all_set = 0;
+		GMT_LONG n_alloc = GMT_SMALL_CHUNK;
 		double *val;
 		FILE **fp = NULL;
 
-		fp = (FILE **) GMT_memory (VNULL, n_files, sizeof (FILE *), GMT_program);
-		n_expected_fields = (int *) GMT_memory (VNULL, n_files, sizeof (int), GMT_program);
-		n_fields = (int *) GMT_memory (VNULL, n_files, sizeof (int), GMT_program);
+		fp = (FILE **) GMT_memory (VNULL, (size_t)n_files, sizeof (FILE *), GMT_program);
+		n_expected_fields = (int *) GMT_memory (VNULL, (size_t)n_files, sizeof (int), GMT_program);
+		n_fields = (int *) GMT_memory (VNULL, (size_t)n_files, sizeof (int), GMT_program);
 
 		for (fno = 1, k = 0; fno < argc; fno++) {	/* Loop over input files, if any */
 			if (argv[fno][0] == '-') continue;
@@ -275,7 +279,7 @@ int main (int argc, char **argv)
 				all_set = 1;
 				n_tot_cols += n_expected_fields[k];
 				if (n_tot_cols > n_alloc) {
-					n_alloc += GMT_SMALL_CHUNK;
+					n_alloc <<= 1;
 					val = (double *) GMT_memory ((char *)val, (size_t)n_alloc, sizeof (double), GMT_program);
 				}
 				for (i = 0; i < n_fields[k]; i++) val[j++] = in[i];	/* Copy the input values */
@@ -325,7 +329,7 @@ int main (int argc, char **argv)
 							all_set = 1;
 							n_tot_cols += n_expected_fields[k];
 							if (n_tot_cols > n_alloc) {
-								n_alloc += GMT_SMALL_CHUNK;
+								n_alloc <<= 1;
 								val = (double *) GMT_memory ((char *)val, (size_t)n_alloc, sizeof (double), GMT_program);
 							}
 						}
@@ -360,7 +364,7 @@ int main (int argc, char **argv)
 			}
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
 					exit (EXIT_FAILURE);
 				}
 
@@ -372,12 +376,13 @@ int main (int argc, char **argv)
 				}
 
 				if (Ctrl->I.active) {	/* Collect all rows an write in reverse order */
-					if (n_rev == n_rev_alloc) {
-						n_rev_alloc += (n_out * GMT_CHUNK);
-						R_out = (double *) GMT_memory ((void *)R_out, n_rev_alloc, sizeof (double), GMT_program);
+					if (n_rev_recs == n_rev_alloc) {
+						n_rev_alloc = (n_rev_recs == 0) ? GMT_CHUNK : (n_rev_alloc << 1);
+						R_out = (double *) GMT_memory ((void *)R_out, (size_t)(n_rev_alloc * n_out), sizeof (double), GMT_program);
 					}
 					if (!Ctrl->N.active || n_nan < n_out) {
 						for (i = 0; i < n_out; i++) R_out[n_rev++] = out[i];
+						n_rev_recs++;
 					}
 				}
 				else if (!Ctrl->N.active || n_nan < n_out)
@@ -401,7 +406,7 @@ int main (int argc, char **argv)
 
 		for (k = 0; k < n_files; k++) GMT_fclose(fp[k]);
 
-		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %d records passed (input cols = %d; output cols = %d)\n", n_total_read, n_tot_cols, n_out);
+		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %ld records passed (input cols = %d; output cols = %d)\n", n_total_read, n_tot_cols, n_out);
 	}
 	else {	/* Just cutting out columns */
 		int n_expected_fields, n_fields, n_args;
@@ -478,7 +483,7 @@ int main (int argc, char **argv)
 
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 					if (n_out == 0) {
@@ -503,12 +508,13 @@ int main (int argc, char **argv)
 					}
 
 					if (Ctrl->I.active) {	/* Collect all rows an write in reverse order */
-						if (n_rev == n_rev_alloc) {
-							n_rev_alloc += (n_out * GMT_CHUNK);
-							R_out = (double *) GMT_memory ((void *)R_out, n_rev_alloc, sizeof (double), GMT_program);
+						if (n_rev_recs == n_rev_alloc) {
+							n_rev_alloc = (n_rev_recs == 0) ? GMT_CHUNK : (n_rev_alloc << 1);
+							R_out = (double *) GMT_memory ((void *)R_out, (size_t)(n_rev_alloc * n_out), sizeof (double), GMT_program);
 						}
 						if (!Ctrl->N.active || n_nan < n_out) {	/* OK to use this record */
 							for (i = 0; i < n_out; i++) R_out[n_rev++] = out[i];
+							n_rev_recs++;
 						}
 					}
 					else if (Ctrl->E.active) {
@@ -528,14 +534,12 @@ int main (int argc, char **argv)
 
 			if (fp != GMT_stdin) GMT_fclose(fp);
 		}
-		if (gmtdefs.verbose) fprintf(stderr, "%s: %d records passed (input cols = %d; output cols = %d)\n", GMT_program, n_total_read, n_expected_fields, n_out);
-		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %d segments\n", argv[0], n_seg);
+		if (gmtdefs.verbose) fprintf(stderr, "%s: %ld records passed (input cols = %d; output cols = %d)\n", GMT_program, n_total_read, n_expected_fields, n_out);
+		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %ld segments\n", argv[0], n_seg);
 	}
 
 	if (Ctrl->I.active) {	/* Time to write out rows in reverse order */
-		int n_rows;
-		n_rows = n_rev / n_out;
-		for (j = n_rows - 1; j >= 0; j--) GMT_output (fp_out, n_out, &R_out[j*n_out]);
+		for (j = n_rev_recs - 1; j >= 0; j--) GMT_output (fp_out, n_out, &R_out[j*n_out]);
 		GMT_free ((void *)R_out);
 	}
 
@@ -553,17 +557,18 @@ int main (int argc, char **argv)
 void *New_Gmtconvert_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GMTCONVERT_CTRL *C;
 	
-	C = (struct GMTCONVERT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMTCONVERT_CTRL), "New_Gmtconvert_Ctrl");
+	C = (struct GMTCONVERT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMTCONVERT_CTRL), "New_Gmtconvert_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
-	C->F.col = (short int *) GMT_memory (VNULL, GMTCONVERT_MAX_COL, sizeof (short int), GMT_program);
+	C->F.col = (int *) GMT_memory (VNULL, (size_t)GMTCONVERT_MAX_COL, sizeof (int), GMT_program);
 	
 	return ((void *)C);
 }
 
 void Free_Gmtconvert_Ctrl (struct GMTCONVERT_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.name) free ((void *)C->D.name);	
 	if (C->F.col) GMT_free ((void *)C->F.col);	
-	if (C->S.pattern) GMT_free ((void *)C->S.pattern);	
+	if (C->S.pattern) free ((void *)C->S.pattern);	
 	GMT_free ((void *)C);	
 }
