@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtselect.c,v 1.80 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: gmtselect.c,v 1.92 2008/05/06 23:53:53 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -47,6 +47,7 @@
  *		3.3.5 10-JUL-2000	Added plain -L for periodicity
  * 		3.4
  * Version:	4.1.2 06-APR-2006	No longer declares global variables
+ * Version:	4.2.1 23-APR-2007	Extended -L to -Lp to limit eligible points
  */
  
 #include "gmt.h"
@@ -78,8 +79,9 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 		BOOLEAN active;
 		char set;	/* One of f, h, i, l, c */
 	} D;
-	struct L {	/* -L[<dist>/<lfile>] */
+	struct L {	/* -L[p][<dist>/<lfile>] */
 		BOOLEAN active;
+		int mode;	/* Controls what happens beyond segment endpoints */
 		double dist;	/* Distance of influence for each line */
 		char *file;	/* Name of file with lines */
 	} L;
@@ -105,14 +107,15 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 
 int main (int argc, char **argv)
 {
-	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, n_args, err;
+	int i, j, k, fno, n_files = 0, n_args, err, n_minimum = 2;
 	int n_fields, n_expected_fields, ind, bin, last_bin = -1, n_output = -1;
 	int np[2], base = 3, wd[2], id, this_node, side, is_inside, row, col, pos;
+	
+	GMT_LONG n_read = 0, n_pass = 0;
 
-
-	BOOLEAN error = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, need_header;
+	BOOLEAN error = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, need_header, shuffle;
 	BOOLEAN cartesian = FALSE, nofile = TRUE, done = FALSE, first = TRUE, inside, plain_xy;
-	BOOLEAN greenwich = FALSE, output_header = FALSE, do_project = FALSE, binary = FALSE;
+	BOOLEAN greenwich = FALSE, output_header = FALSE, do_project = FALSE, just_copy_record = FALSE;
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, xx, yy;
 	double *in, west_border = 0.0, east_border = 0.0;
@@ -128,7 +131,7 @@ int main (int argc, char **argv)
 	struct GMT_SHORE c;
 	struct GMTSELECT_CTRL *Ctrl;
 
-	PFI near_a_line;
+	PFI near_a_line, near_a_point;
 
 	int compare_x (const void *point_1, const void *point_2);
 	void *New_Gmtselect_Ctrl (), Free_Gmtselect_Ctrl (struct GMTSELECT_CTRL *C);
@@ -183,14 +186,19 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {	/* Set line options */
 						Ctrl->L.active = TRUE;
-						for (j = 2; argv[i][j] && argv[i][j] != '/'; j++);
-							if (!argv[i][j]) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -L:  Expects -Ldist/file\n", GMT_program);
+						k = 2;
+						if (argv[i][k] == 'p') {	/* Disallow points beyond endpoints */
+							Ctrl->L.mode = 10;
+							k++;
+						}
+						for (j = k; argv[i][j] && argv[i][j] != '/'; j++);
+						if (!argv[i][j]) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -L:  Expects -L[p]dist/file\n", GMT_program);
 							error++;
 						}
 						else {
 							Ctrl->L.file = strdup (&argv[i][j+1]);
-							Ctrl->L.dist = atof (&argv[i][2]);
+							Ctrl->L.dist = atof (&argv[i][k]);
 						}
 					}
 					else {	/* Obsolete flag to process geographic data (use -f instead)  */
@@ -288,7 +296,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "gmtselect %s - Select data subsets based on multiple spatial criteria\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtselect <infiles> [-A<min_area>[/<min_level>/<max_level>]] [-C[f]<dist>/<ptfile>]\n");
-		fprintf (stderr, "\t[-D<resolution>] [-F<polygon>] [%s] [-L<dist>/<lfile>] [%s] [-I[cflrsz]\n", GMT_J_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[-D<resolution>] [-F<polygon>] [%s] [-L[p]<dist>/<lfile>] [%s] [-I[cflrsz]\n", GMT_J_OPT, GMT_H_OPT);
 		fprintf (stderr, "\t[%s] [-N<maskvalues>[o]] [%s] [-V[l]]\n\t[%s] [-Z<min>/<max>] [-%s] [%s]\n\n", GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -298,11 +306,12 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A coastline features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
 		fprintf (stderr, "\t   will be skipped [0/4] (see pscoast for details)].  Ignored unless -N is set\n");
-		fprintf (stderr, "\t-C pass locations that are within <dist> of any point in ASCII <ptfile>\n");
-		fprintf (stderr, "\t   Give 0 distance if 3rd column of ptfile has individual distances.\n");
-		fprintf (stderr, "\t   Distances are in users units [or km if -R -J are used].\n");
-		fprintf (stderr, "\t   Prepend f to use approximate (flat earth) rather than exact geodesic distances.\n");
-		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf (stderr, "\t-C pass locations that are within <dist> of any point in the ASCII <ptfile>\n");
+		fprintf (stderr, "\t   Give distance as 0 if 3rd column of <ptfile> has individual distances.\n");
+		fprintf (stderr, "\t   Distances are Cartesian and in user units [or spherical in km if -fg is used].\n");
+		fprintf (stderr, "\t   Use -Cf for approximate (flat earth) rather than exact geodesic distances.\n");
+		fprintf (stderr, "\t   If current ELLIPSOID is Sphere then great circle distances replace geodesics.\n");
+		fprintf (stderr, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s]\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t-D Choose one of the following resolutions: (Ignored unless -N is set)\n");
 		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
 		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
@@ -310,8 +319,11 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   l - low resolution [Default]\n");
 		fprintf (stderr, "\t   c - crude resolution, for tasks that need crude continent outlines only\n");
 		fprintf (stderr, "\t-L Pass locations that are within <dist> of any line in ASCII <linefile>\n");
-		fprintf (stderr, "\t   Give 0 distance if 2nd column of segment headers have individual distances.\n");
-		fprintf (stderr, "\t   Distances are in users units [or km if -R -J are used].\n");
+		fprintf (stderr, "\t   Give distance as 0 if 2nd column of segment headers have individual distances.\n");
+		fprintf (stderr, "\t   Distances are Cartesian and in user units [or spherical in km if -fg is used].\n");
+		fprintf (stderr, "\t   If current ELLIPSOID is Sphere then great circle distances replace geodesics.\n");
+		fprintf (stderr, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s]\n", GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t   Optionally, use -Lp to exclude points projecting beyond a line's endpoints.\n");
 		fprintf (stderr, "\t-F pass locations that are inside the polygons in the ASCII <polygon> file\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I Used to reverse the tests, i.e. pass locations outside the region\n");
@@ -333,8 +345,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t   Append l for long verbose, reporting every 1000 points.\n");
-		fprintf (stderr, "\t   -Z assumes the 3rd data column contains z-values and we want to keep records with\n");
-		fprintf (stderr, "\t      <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
+		fprintf (stderr, "\t-Z assumes the 3rd data column contains z-values and we want to keep records with\n");
+		fprintf (stderr, "\t   <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
@@ -376,6 +388,9 @@ int main (int argc, char **argv)
 	GMT_setmode (GMT_OUT);
 #endif
 
+	shuffle = (gmtdefs.xy_toggle[GMT_IN] != gmtdefs.xy_toggle[GMT_OUT]);	/* Must rewrite output record */
+	n_minimum = (Ctrl->Z.active) ? 3 : 2;	/* Minimum number of columns in ASCII input */
+	
 	if (!project_info.region_supplied && Ctrl->N.active) {	/* If we use coastline data or used -fg but didnt give -R we imply -Rg */
 		project_info.region_supplied = TRUE;
 		west = 0.0;	east = 360.0;	south = -90.0;	north = +90.0;
@@ -383,7 +398,7 @@ int main (int argc, char **argv)
 		GMT_io.in_col_type[1] = GMT_IS_LAT;
 	}
 	if (project_info.region_supplied) {	/* -R was set directly or indirectly; hence must set -J if not supplied */
-		if (project_info.projection < 0) {	/* -J not specifed, set one implicitly */
+		if (project_info.projection == GMT_NO_PROJ) {	/* -J not specifed, set one implicitly */
 			if (GMT_io.in_col_type[0] == GMT_IS_LON) {	/* We know we have geographic data (either via -f or -N) */
 				project_info.degree[0] = project_info.degree[1] = TRUE;
 			}
@@ -422,7 +437,7 @@ int main (int argc, char **argv)
 		np[0] = np[1] = 0;
 	}
 
-	binary = (GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
+	just_copy_record = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT] || shuffle);
 
 	/* Initiate pointer to distance calculation function */
 	if (GMT_io.in_col_type[0] & GMT_IS_GEO && !do_project) {	/* Geographic data and no -R -J conversion */
@@ -430,11 +445,13 @@ int main (int argc, char **argv)
 			GMT_distance_func = GMT_flatearth_dist_km;
 		else
 			GMT_distance_func = (PFD) ((GMT_IS_SPHERICAL) ? GMT_geodesic_dist_km : GMT_great_circle_dist_km);
-		near_a_line = (PFI) GMT_near_a_line_spherical;
+		near_a_line  = (PFI) GMT_near_a_line_spherical;
+		near_a_point = (PFI) GMT_near_a_point_spherical;
 	}
 	else {	/* Cartesian data (or lon/lat projected via -R -J) */
 		GMT_distance_func = (PFD) GMT_cartesian_dist;
-		near_a_line = (PFI) GMT_near_a_line_cartesian;
+		near_a_line  = (PFI) GMT_near_a_line_cartesian;
+		near_a_point = (PFI) GMT_near_a_point_cartesian;
 	}
 
 	if (Ctrl->C.active) { 	/* Initialize point structure used in test for proximity to points */
@@ -565,16 +582,19 @@ int main (int argc, char **argv)
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
+			if (!just_copy_record && (GMT_io.status & GMT_IO_MISMATCH)) {
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
 			n_read++;
-			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %d records, passed %d records\r", GMT_program, n_read, n_pass);
+			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %ld records, passed %ld records\r", GMT_program, n_read, n_pass);
 
-			if (Ctrl->Z.active && n_expected_fields < 3) {
-				fprintf (stderr, "%s: -Z requires a data file with at least 3 columns; this file only has %d. Exiting.\n", GMT_program, n_fields);
+			if (n_fields < n_minimum) {
+				if (Ctrl->Z.active)
+					fprintf (stderr, "%s: -Z requires a data file with at least 3 columns; this file only has %d near line %ld. Exiting.\n", GMT_program, n_fields, n_read);
+				else
+					fprintf (stderr, "%s: Data file must have at least 2 columns; this file only has %d near line %ld. Exiting.\n", GMT_program, n_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
@@ -598,18 +618,12 @@ int main (int argc, char **argv)
 			}
 			
 			if (Ctrl->C.active) {	/* Check for distance to points */
-				if (cartesian) {	/* Was given x,y data, or we projected lon,lat */
-					inside = GMT_near_a_point_cart (xx, yy, point, Ctrl->C.dist); 
-					if (inside != Ctrl->I.pass[1]) { output_header = need_header; continue;}
-				}
-				else {
-					inside = GMT_near_a_point (xx, yy, point, Ctrl->C.dist); 
-					if (inside != Ctrl->I.pass[1]) { output_header = need_header; continue;}
-				}
+				inside = near_a_point (xx, yy, point, Ctrl->C.dist); 
+				if (inside != Ctrl->I.pass[1]) { output_header = need_header; continue;}
 			}
 
 			if (Ctrl->L.active) {
-				inside = near_a_line (xx, yy, line, FALSE, NULL, NULL, NULL);
+				inside = near_a_line (xx, yy, line, Ctrl->L.mode, NULL, NULL, NULL);
 				if (inside != Ctrl->I.pass[2]) { output_header = need_header; continue;}
 			}
 
@@ -652,6 +666,7 @@ int main (int argc, char **argv)
 					while (ind < c.nb && c.bins[ind] != bin) ind++;	/* Set ind to right bin */
 					if (ind == c.nb) continue;			/* Bin not among the chosen ones */
 					last_bin = bin;
+					GMT_free_shore (&c);	/* Free previously allocated arrays */
 					if ((err = GMT_get_shore_bin (ind, &c, Ctrl->A.area, Ctrl->A.low, Ctrl->A.high))) {
 						fprintf (stderr, "%s: %s [%s resolution shoreline]\n", GMT_program, GMT_strerror(err), shore_resolution[base]);
 						exit (EXIT_FAILURE);
@@ -662,7 +677,7 @@ int main (int argc, char **argv)
 						GMT_free_polygons (p[id], np[id]);
 						if (np[id]) GMT_free ((void *)p[id]);
 						np[id] = GMT_assemble_shore (&c, wd[id],  Ctrl->A.low, TRUE, greenwich, west_border, east_border, &p[id]);
-						np[id] = GMT_prep_polygons (&p[id], np[id], greenwich, TRUE, step, -1);
+						np[id] = GMT_prep_polygons (&p[id], np[id], TRUE, step, -1);
 					}
 				}
 
@@ -714,22 +729,30 @@ int main (int argc, char **argv)
 				output_header = FALSE;
 			}
 
-			if (binary) {
+			if (just_copy_record)
+				fprintf (GMT_stdout, "%s", GMT_io.current_record);
+			else {
 				if (n_output < 0) n_output = n_expected_fields;
 				GMT_output (GMT_stdout, n_output, in);
 			}
-			else
-				fprintf (GMT_stdout, "%s", GMT_io.current_record);
 			n_pass++;
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d records, passed %d records\n", GMT_program, n_read, n_pass);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld records, passed %ld records\n", GMT_program, n_read, n_pass);
 
 	if (Ctrl->C.active) GMT_free_table (point);
 	if (Ctrl->L.active) GMT_free_table (line);
 	if (Ctrl->F.active) GMT_free_table (pol);
+	if (Ctrl->N.active) {
+		GMT_free_shore (&c);
+		GMT_shore_cleanup (&c);
+		for (id = 0; id < 2; id++) {
+			GMT_free_polygons (p[id], np[id]);
+			if (np[id]) GMT_free ((void *)p[id]);
+		}
+	}
 	
 	Free_Gmtselect_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -757,7 +780,7 @@ void *New_Gmtselect_Ctrl () {	/* Allocate and initialize a new control structure
 	int i;
 	struct GMTSELECT_CTRL *C;
 	
-	C = (struct GMTSELECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMTSELECT_CTRL), "New_Gmtselect_Ctrl");
+	C = (struct GMTSELECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMTSELECT_CTRL), "New_Gmtselect_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -772,8 +795,8 @@ void *New_Gmtselect_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Gmtselect_Ctrl (struct GMTSELECT_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	if (C->F.file) GMT_free ((void *)C->F.file);	
-	if (C->L.file) GMT_free ((void *)C->L.file);	
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->F.file) free ((void *)C->F.file);	
+	if (C->L.file) free ((void *)C->L.file);	
 	GMT_free ((void *)C);	
 }
