@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtset.c,v 1.21 2007/03/12 19:52:26 remko Exp $
+ *	$Id: gmtset.c,v 1.27 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,10 @@ int main (int argc, char **argv)
 	int i, j;
 	char *file = CNULL;
 
+	/* SPECIAL INITIALIZATION SINCE BMT_begin IS NOT USED HERE !! */
+#ifdef DEBUG
+	GMT_memtrack_init (&GMT_mem_keeper);
+#endif
 	GMT_set_home ();
 	GMT_init_fonts (&GMT_N_FONTS);
 	GMT_stdout = stdout;    GMT_stdin = stdin;
@@ -55,7 +59,7 @@ int main (int argc, char **argv)
 	GMT_program = &argv[0][i+1];	/* Name without full path */
 
 	for (i = 1, j = 0; i < argc && j == 0; i++) {
-		if (!strncmp (argv[i], "-G", 2)) {
+		if (!strncmp (argv[i], "-G", (size_t)2)) {
 			file = &argv[i][2];
 			j = i;
 		}
@@ -70,10 +74,9 @@ int main (int argc, char **argv)
 
 	GMT_setdefaults (argc, argv);
 
-	if (file)
-		GMT_savedefaults (file);
-	else
-		GMT_savedefaults (".gmtdefaults4");
+	GMT_putdefaults (file);
+
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
