@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2cpt.c,v 1.41 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grd2cpt.c,v 1.49 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,7 @@
  *		JL: 27-APR-2003, added a -R option
  *		SE: 17-SEP-2003, added a -T option
  * Version:	4
- * 
+ *
  */
 
 #include "gmt.h"
@@ -90,7 +90,9 @@ int main (int argc, char **argv)
 
 	char *grdfile = CNULL, CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
 
-	int i, j, nxy, nx, ny, nfound, ngood;
+	int j, nx, ny;
+	
+	GMT_LONG i, nxy, nfound, ngood;
 
 	float	*zdata;
 
@@ -115,7 +117,7 @@ int main (int argc, char **argv)
 
 	/* Get list of available color tables in $GMT_SHAREDIR */
 
-	GMT_getsharepath (CNULL, "GMT_CPT", ".lis", CPT_lis);
+	GMT_getsharepath ("conf", "gmt_cpt", ".conf", CPT_lis);
 	if ((fpc = fopen (CPT_lis, "r")) == NULL) {
 		fprintf (stderr, "%s: ERROR: Cannot open file %s\n", GMT_program, CPT_lis);
 		exit (EXIT_FAILURE);
@@ -271,7 +273,8 @@ int main (int argc, char **argv)
 		Ctrl->C.active = TRUE;
 		Ctrl->C.file = strdup ("rainbow");
 	}
-		
+	if (!Ctrl->E.active) Ctrl->E.levels =  11;	/* Default number of levels */
+
 	/* Open the specified master color table */
 
 	error += GMT_set_cpt_path (CPT_file, Ctrl->C.file);
@@ -284,7 +287,11 @@ int main (int argc, char **argv)
 		fprintf (stderr,"%s: GMT SYNTAX ERROR -S option:  Bad arguments\n", GMT_program);
 		error++;
 	}
-	
+	if (Ctrl->S.active && (Ctrl->T.active || Ctrl->E.active)) {
+		fprintf (stderr,"%s: GMT USAGE ERROR -S option:  Cannot be combined with -E nor -T option.\n", GMT_program);
+		error++;
+	}
+
 	if (error) exit (EXIT_FAILURE);
 
 	if (Ctrl->N.active) GMT_cpt_flags += 1;	/* bit 0 controls if BFN will be written out */
@@ -303,14 +310,14 @@ int main (int argc, char **argv)
 		}
 		nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
 		ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
-		nxy = nx * ny;
+		nxy = ((size_t)nx) * ((size_t)ny);
 
 		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
 
 		GMT_err_fail (GMT_read_grd (grdfile, &grd, zdata, w, e, s, n, GMT_pad, FALSE), grdfile);
 	}
 	else {
-		nxy = grd.nx * grd.ny;
+		nxy = ((size_t)grd.nx) * ((size_t)grd.ny);
 		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
 
 		GMT_err_fail (GMT_read_grd (grdfile, &grd, zdata, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
@@ -366,8 +373,9 @@ int main (int argc, char **argv)
 
 	/* Now the zdata are ready.  Decide how to make steps in z.  */
 	if (Ctrl->S.active) {
-		Ctrl->E.levels =  (grd.z_min < Ctrl->S.low) ? 1 : 0;
-		Ctrl->E.levels += (int)floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc) + 1;
+		/* Use predefined levels and interval */
+		Ctrl->E.levels = (grd.z_min < Ctrl->S.low) ? 1 : 0;
+		Ctrl->E.levels += (GMT_LONG)floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc) + 1;
 		if (grd.z_max > Ctrl->S.high) Ctrl->E.levels++;
 		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
 		if (grd.z_min < Ctrl->S.low) {
@@ -386,67 +394,59 @@ int main (int argc, char **argv)
 		}
 		if (j == Ctrl->E.levels-1) cdf_cpt[j].z = grd.z_max;
 	}
-	else {
-		/* Make a equaldistant color map from grd.z_min to grd.z_max */
-		if (Ctrl->T.active) {
-			double start, range;
-			
-			if ( !Ctrl->E.active ) {
-				Ctrl->E.levels =  11;
-			}
-			switch (Ctrl->T.kind) {
-				case -1:
-					start = -fabs((double)grd.z_min);
-					break;
-				case 1:
-					start = -fabs((double)grd.z_max);
-					break;
-				case -2:
-					start = -MIN(fabs((double)grd.z_min), fabs((double)grd.z_max));
-					break;
-				case 2:
-					start = -MAX(fabs((double)grd.z_min), fabs((double)grd.z_max));
-					break;
-				default:
-					start = grd.z_min;
-					break;
-			}
-			range = (Ctrl->T.kind) ? 2.0 * fabs (start) : grd.z_max - grd.z_min;
-			Ctrl->S.inc = range / (double)(Ctrl->E.levels - 1);
-			cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
-			for (i = 0; i < Ctrl->E.levels; i++) {
-				cdf_cpt[i].z = start + i * Ctrl->S.inc;
-			}
-		}
-		else {
-			/* This is completely ad-hoc.  It chooses z based on steps of 0.1 for a Gaussian CDF:  */
-			Ctrl->E.levels = 11;
-			cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
-                	/* Stupid bug fix here:  
-                	        If (mean-1.28155*sd <= grd.z_min || mean+1.28155*sd >= grd.z_max) then
-                	          reset mean and sd so they fit inside available range:  */
-                          
-			if ((mean - 1.28155*sd) <= grd.z_min || (mean + 1.28155*sd) >= grd.z_max) {
-				mean = 0.5 * (grd.z_min + grd.z_max);
-				sd = (grd.z_max - mean) / 1.5;
-				if (sd <= 0.0) {
-					fprintf (stderr, "%s:  ERROR.  Min and Max data values are equal.\n", GMT_program);
-					exit (EXIT_FAILURE);
-				}
-			}	/* End of stupid bug fix  */
 
-			cdf_cpt[0].z = grd.z_min;
-			cdf_cpt[1].z = mean - 1.28155 * sd;
-			cdf_cpt[2].z = mean - 0.84162 * sd;
-			cdf_cpt[3].z = mean - 0.52440 * sd;
-			cdf_cpt[4].z = mean - 0.25335 * sd;
-			cdf_cpt[5].z = mean;
-			cdf_cpt[6].z = mean + 0.25335 * sd;
-			cdf_cpt[7].z = mean + 0.52440 * sd;
-			cdf_cpt[8].z = mean + 0.84162 * sd;
-			cdf_cpt[9].z = mean + 1.28155 * sd;
-			cdf_cpt[10].z = grd.z_max;
+	else if (Ctrl->T.active || Ctrl->E.active) {
+		/* Make a equaldistant color map from grd.z_min to grd.z_max */
+		double start, range;
+
+		switch (Ctrl->T.kind) {
+			case -1:
+				start = -fabs((double)grd.z_min);
+				break;
+			case 1:
+				start = -fabs((double)grd.z_max);
+				break;
+			case -2:
+				start = -MIN(fabs((double)grd.z_min), fabs((double)grd.z_max));
+				break;
+			case 2:
+				start = -MAX(fabs((double)grd.z_min), fabs((double)grd.z_max));
+				break;
+			default:
+				start = grd.z_min;
+				break;
 		}
+		range = (Ctrl->T.kind) ? 2.0 * fabs (start) : grd.z_max - grd.z_min;
+		Ctrl->S.inc = range / (double)(Ctrl->E.levels - 1);
+		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
+		for (i = 0; i < Ctrl->E.levels; i++) {
+			cdf_cpt[i].z = start + i * Ctrl->S.inc;
+		}
+	}
+
+	else {
+		/* This is completely ad-hoc.  It chooses z based on steps of 0.1 for a Gaussian CDF:  */
+		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
+		if ((mean - 1.28155*sd) <= grd.z_min || (mean + 1.28155*sd) >= grd.z_max) {
+			mean = 0.5 * (grd.z_min + grd.z_max);
+			sd = (grd.z_max - mean) / 1.5;
+			if (sd <= 0.0) {
+				fprintf (stderr, "%s:  ERROR.  Min and Max data values are equal.\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+		}	/* End of stupid bug fix  */
+
+		cdf_cpt[0].z = grd.z_min;
+		cdf_cpt[1].z = mean - 1.28155 * sd;
+		cdf_cpt[2].z = mean - 0.84162 * sd;
+		cdf_cpt[3].z = mean - 0.52440 * sd;
+		cdf_cpt[4].z = mean - 0.25335 * sd;
+		cdf_cpt[5].z = mean;
+		cdf_cpt[6].z = mean + 0.25335 * sd;
+		cdf_cpt[7].z = mean + 0.52440 * sd;
+		cdf_cpt[8].z = mean + 0.84162 * sd;
+		cdf_cpt[9].z = mean + 1.28155 * sd;
+		cdf_cpt[10].z = grd.z_max;
 	}
 
 	/* Get here when we are ready to go.  cdf_cpt[].z contains the sample points.  */
@@ -495,14 +495,14 @@ int main (int argc, char **argv)
 
 void *New_Grd2cpt_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRD2CPT_CTRL *C;
-	
-	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRD2CPT_CTRL), "New_Grd2cpt_Ctrl");
-	
+
+	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD2CPT_CTRL), "New_Grd2cpt_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
 }
 
 void Free_Grd2cpt_Ctrl (struct GRD2CPT_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	GMT_free ((void *)C);	
+	if (C->C.file) free ((void *)C->C.file);
+	GMT_free ((void *)C);
 }
