@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2xyz.c,v 1.42 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grd2xyz.c,v 1.52 2008/05/08 07:41:13 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grd2xyz.c reads a grd file and prints out the x,y,z values to
+ * grd2xyz.c reads a grid file and prints out the x,y,z values to
  * standard output.
  *
  * Author:	Paul Wessel
@@ -46,11 +46,10 @@ struct GRD2XYZ_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, global = FALSE, b_only = FALSE;
-	BOOLEAN first = TRUE;
+	BOOLEAN error = FALSE, b_only = FALSE, first = TRUE;
 
-	int i, j, k, ij, nm, nx, ny, gmt_ij, n_suppressed = 0, n_out, n_files = 0;
-	int n_total = 0;
+	int i, j, k, nx, ny, n_files = 0, n_out;
+	GMT_LONG ij, nm, gmt_ij, n_suppressed = 0, n_total = 0;
 
 	float *z;
 	
@@ -124,9 +123,9 @@ int main (int argc, char **argv)
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\n\t<grdfiles> is one or more grd files to convert\n");
+		fprintf (stderr, "\n\t<grdfiles> is one or more grid files to convert\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-E Write ESRI ArcInfo ASCII interchange format.  Only one gridfile can be specified\n");
+		fprintf (stderr, "\t-E Write ESRI ArcInfo ASCII interchange format.  Only one grid file can be specified\n");
 		fprintf (stderr, "\t   Optionally append nodata value to represent NaNs [-9999]\n");
 		fprintf (stderr, "\t-H Write 1 ASCII header record [Default is no header]\n");
 		GMT_explain_option ('R');
@@ -162,17 +161,17 @@ int main (int argc, char **argv)
 	}
 
 	if (n_files == 0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify at least one input file\n", GMT_program);
+		fprintf (stderr, "%s: SYNTAX ERROR:  Must specify at least one input file\n", GMT_program);
 		error++;
 	}
 
 	if (n_files > 1 && Ctrl->E.active) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  -E can only handle one input file\n", GMT_program);
+		fprintf (stderr, "%s: SYNTAX ERROR:  -E can only handle one input file\n", GMT_program);
 		error++;
 	}
 
 	if (Ctrl->Z.active && Ctrl->E.active) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  -E is not compatible with -Z\n", GMT_program);
+		fprintf (stderr, "%s: SYNTAX ERROR:  -E is not compatible with -Z\n", GMT_program);
 		error++;
 	}
 	if (b_only && Ctrl->Z.active) GMT_io.binary[GMT_OUT] = FALSE;
@@ -180,14 +179,16 @@ int main (int argc, char **argv)
 	GMT_init_z_io (Ctrl->Z.format, Ctrl->Z.repeat, Ctrl->Z.swab, Ctrl->Z.skip, Ctrl->Z.type, &io);
 
 	if ((GMT_io.binary[GMT_OUT] || io.binary) && GMT_io.io_header[GMT_OUT]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary output data cannot have header -H\n", GMT_program);
+		fprintf (stderr, "%s: SYNTAX ERROR.  Binary output data cannot have header -H\n", GMT_program);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	if (b_only && Ctrl->Z.active) fprintf (stderr, "%s: GMT Warning.  -Z overrides -bo\n", GMT_program);
-	if (b_only && Ctrl->E.active) fprintf (stderr, "%s: GMT Warning.  -E overrides -bo\n", GMT_program);
+	if (Ctrl->Z.active && io.binary) GMT_io.binary[GMT_OUT] = TRUE;
+	
+	if (b_only && Ctrl->Z.active) fprintf (stderr, "%s: Warning.  -Z overrides -bo\n", GMT_program);
+	if (b_only && Ctrl->E.active) fprintf (stderr, "%s: Warning.  -E overrides -bo\n", GMT_program);
 
 #ifdef SET_IO_MODE
 		GMT_setmode (GMT_OUT);
@@ -202,22 +203,23 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[k]);
 
-		nm = grd.nx * grd.ny;
+		nm = ((size_t)grd.nx) * ((size_t)grd.ny);
 		n_total += nm;
 
 		if (e > w && n > s) {	/* Subset */
-			global = (fabs (grd.x_max - grd.x_min) == 360.0);
-			if (!global && (w < grd.x_min || e > grd.x_max)) error = TRUE;
-			if (s < grd.y_min || n > grd.y_max) error = TRUE;
-			if (error) {
-				fprintf (stderr, "%s: GMT ERROR: Subset exceeds data domain!\n", GMT_program);
-				exit (EXIT_FAILURE);
+			if (fabs (grd.x_max - grd.x_min) < 360.0) {
+				if (w < grd.x_min) { w = grd.x_min; error = TRUE; }
+				if (e > grd.x_min) { e = grd.x_max; error = TRUE; }
 			}
+			if (s < grd.y_min) { s = grd.y_min; error = TRUE; }
+			if (n > grd.y_max) { n = grd.y_max; error = TRUE; }
+			if (error) fprintf (stderr, "%s: Warning: Subset exceeds data domain. Subset reduced to common region.\n", GMT_program);
+			error = FALSE;
 			GMT_err_fail (GMT_adjust_loose_wesn (&w, &e, &s, &n, &grd), "");	/* Make sure w,e,s,n matches header spacing */
 			nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
 			ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
 
-			z = (float *) GMT_memory (VNULL, (size_t) (nx * ny), sizeof (float), GMT_program);
+			z = (float *) GMT_memory (VNULL, (size_t) ((size_t)nx * (size_t)ny), sizeof (float), GMT_program);
 
 			GMT_err_fail (GMT_read_grd (argv[k], &grd, z, w, e, s, n, GMT_pad, FALSE), argv[k]);
 		}
@@ -244,29 +246,35 @@ int main (int argc, char **argv)
 			}
 		}
 		else if (Ctrl->E.active) {
-			fprintf (GMT_stdout, "ncols %d\nnrows %d\n", grd.nx, grd.ny);
-			if (!grd.node_offset) {	/* Gridline format */
-				fprintf (GMT_stdout, "xllcenter ");
-				fprintf (GMT_stdout, gmtdefs.d_format, grd.x_min + 0.5 * grd.x_inc);
-				fprintf (GMT_stdout, "\nyllcenter ");
-				fprintf (GMT_stdout, gmtdefs.d_format, grd.y_min + 0.5 * grd.y_inc);
+			double slop;
+			slop = 1.0 - (grd.x_inc / grd.y_inc);
+			if (!GMT_IS_ZERO (slop)) {
+				fprintf (stderr, "%s: ERROR: x_inc must equal y_inc when writing to ESRI format\n", GMT_program);
+				exit (EXIT_FAILURE);
 			}
-			else {	/* Pixel format */
+			fprintf (GMT_stdout, "ncols %d\nnrows %d\n", grd.nx, grd.ny);
+			if (grd.node_offset) {	/* Pixel format */
 				fprintf (GMT_stdout, "xllcorner ");
 				fprintf (GMT_stdout, gmtdefs.d_format, grd.x_min);
 				fprintf (GMT_stdout, "\nyllcorner ");
 				fprintf (GMT_stdout, gmtdefs.d_format, grd.y_min);
 			}
+			else {	/* Gridline format */
+				fprintf (GMT_stdout, "xllcenter ");
+				fprintf (GMT_stdout, gmtdefs.d_format, grd.x_min);
+				fprintf (GMT_stdout, "\nyllcenter ");
+				fprintf (GMT_stdout, gmtdefs.d_format, grd.y_min);
+			}
 			fprintf (GMT_stdout, "\ncellsize ");
 			fprintf (GMT_stdout, gmtdefs.d_format, grd.x_inc);
-			fprintf (GMT_stdout, "\nnodata_value %d\n", irint (Ctrl->E.nodata));
+			fprintf (GMT_stdout, "\nnodata_value %ld\n", (GMT_LONG)irint (Ctrl->E.nodata));
 			for (j = 0; j < grd.ny; j++) {	/* Scanlines, starting in the north (ymax) */
-				ij = j * grd.nx;
+				ij = GMT_IJ (j, 0, grd.nx);
 				for (i = 0; i < grd.nx; i++, ij++) {
 					if (GMT_is_fnan (z[ij]))
-						fprintf (GMT_stdout, "%d", irint (Ctrl->E.nodata));
+						fprintf (GMT_stdout, "%ld", (GMT_LONG)irint (Ctrl->E.nodata));
 					else
-						fprintf (GMT_stdout, "%d", irint ((double)z[ij]));
+						fprintf (GMT_stdout, "%ld", (GMT_LONG)irint ((double)z[ij]));
 					if (i < (grd.nx-1)) fprintf (GMT_stdout, " ");
 				}
 				fprintf (GMT_stdout, "\n");
@@ -313,12 +321,12 @@ int main (int argc, char **argv)
 		GMT_free ((void *)z);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: %d values extracted\n", GMT_program, n_total - n_suppressed);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: %ld values extracted\n", GMT_program, n_total - n_suppressed);
 	if (n_suppressed && gmtdefs.verbose) {
 		if (Ctrl->S.reverse)
-			fprintf (stderr, "%s: %d finite values suppressed\n", GMT_program, n_suppressed);
+			fprintf (stderr, "%s: %ld finite values suppressed\n", GMT_program, n_suppressed);
 		else
-			fprintf (stderr, "%s: %d NaN values suppressed\n", GMT_program, n_suppressed);
+			fprintf (stderr, "%s: %ld NaN values suppressed\n", GMT_program, n_suppressed);
 	}
 
 	Free_Grd2xyz_Ctrl (Ctrl);	/* Deallocate control structure */
@@ -331,7 +339,7 @@ int main (int argc, char **argv)
 void *New_Grd2xyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRD2XYZ_CTRL *C;
 	
-	C = (struct GRD2XYZ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRD2XYZ_CTRL), "New_Grd2xyz_Ctrl");
+	C = (struct GRD2XYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD2XYZ_CTRL), "New_Grd2xyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
