@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: grdblend.c,v 1.52 2007/03/24 01:42:07 pwessel Exp $
+ *    $Id: grdblend.c,v 1.64 2008/05/06 04:44:28 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdblend reads any number of gridfiles that may partly overlap and
+ * grdblend reads any number of grid files that may partly overlap and
  * creates a blend of all the files given certain criteria.  Each input
  * grid is considered to have an "outer" and "inner" region.  The outer
  * region is the extent of the grid; the inner region is provided as
@@ -81,7 +81,7 @@ struct GRDBLEND_INFO {	/* Structure with info about each input grid file */
 
 int main (int argc, char **argv)
 {
-	int i, col, row, k, kk, m, n_blend, one_or_zero, err, error = 0;
+	int i, col, row, k, kk, m, n_blend, err, error = 0;
 	size_t n_fill, n_tot;
 	double wt_x, w, wt;
 	float *z, no_data_f;
@@ -166,14 +166,14 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdblend %s - Blend several partially over-lapping grdfiles onto one grid\n\n", GMT_VERSION);
+		fprintf (stderr,"grdblend %s - Blend several partially over-lapping grid files onto one grid\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdblend [<blendfile>] -G<grdfile> %s\n", GMT_I_OPT);
 		fprintf (stderr, "\t%s [-N<nodata>] [-Q] [-Z<scale>] [-V] [-W]\n\t[%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<blendfile> is an ASCII file (or stdin) with blending parameters for each input grid\n");
-		fprintf (stderr, "\t   Each record has:  grdfilename -Rw/e/s/n weight\n");
+		fprintf (stderr, "\t   Each record has:  grid filename -Rw/e/s/n weight\n");
 		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual -R.\n");
 		fprintf (stderr, "\t   Give a negative weight to invert the sense of the taper (i.e., |<weight>| outside given R.\n");
 		fprintf (stderr, "\t-G <grdfile> is the name of the final 2-D binary data set\n");
@@ -181,7 +181,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-N Set value for nodes without constraints [Default is NaN]\n");
-		fprintf (stderr, "\t-Q grdraster-compatible output without leading grd header [Default writes GMT grd file]\n");
+		fprintf (stderr, "\t-Q grdraster-compatible output without leading grd header [Default writes GMT grid file]\n");
 		fprintf (stderr, "\t   Output grid must be in native binary format (i.e., not netCDF).\n");
 		fprintf (stderr, "\t-Z Multiply z-values by this scale before writing to file [1]\n");
 		GMT_explain_option ('V');
@@ -209,7 +209,7 @@ int main (int argc, char **argv)
 	}
 	type = GMT_grdformats[S.header.type][0];
 	if (Ctrl->Q.active && (type == 'c' || type == 'n')) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  Output grdfile cannot be netCDF format\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  Output grid file cannot be netCDF format\n", GMT_program);
 		error++;
 	}
 
@@ -230,22 +230,20 @@ int main (int argc, char **argv)
 	no_data_f = (float)Ctrl->N.nodata;
 	n_blend = init_blend_job (fp, &S.header, &blend);
 
-	if (fp != GMT_stdin) fclose (fp);
+	if (fp != GMT_stdin) GMT_fclose (fp);
 
 	if (Ctrl->W.active && n_blend > 1) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Only applies when there is a single input grdfile\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Only applies when there is a single input grid file\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
 	/* Initialize header structure for output blend grid */
 
-	one_or_zero = !S.header.node_offset;
-
 	S.header.z_min = S.header.z_max = 0.0;
 
 	GMT_err_fail (GMT_grd_RI_verify (&S.header, 1), Ctrl->G.file);
 
-	n_tot = (size_t)S.header.nx * (size_t)S.header.ny;
+	n_tot = ((size_t)S.header.nx) * ((size_t)S.header.ny);
 
 	z = (float *) GMT_memory (VNULL, (size_t)S.header.nx, sizeof (float), GMT_program);	/* Memory for one output row */
 
@@ -312,7 +310,10 @@ int main (int argc, char **argv)
 	if (!Ctrl->Q.active) GMT_err_fail (GMT_update_grd_info (Ctrl->G.file, &S.header), Ctrl->G.file);
 	GMT_free ((void *)z);
 
-	for (k = 0; k < n_blend; k++) if (blend[k].open) GMT_close_grd (&blend[k].G);	/* Close all input grd files still open */
+	for (k = 0; k < n_blend; k++) if (blend[k].open) {
+		GMT_free ((void *)blend[k].z);
+		GMT_close_grd (&blend[k].G);	/* Close all input grd files still open */
+	}
 
 	if (gmtdefs.verbose) {
 		char empty[GMT_TEXT_LEN];
@@ -324,7 +325,7 @@ int main (int argc, char **argv)
 				strcpy (empty, "NaN");
 			else
 				sprintf (empty, "%g", no_data_f);
-			fprintf (stderr, "%s: %d nodes assigned values, %d set to %s\n", GMT_program, (int)n_fill, (int)(n_tot - n_fill), empty);
+			fprintf (stderr, "%s: %ld nodes assigned values, %ld set to %s\n", GMT_program, (GMT_LONG)n_fill, (GMT_LONG)(n_tot - n_fill), empty);
 		}
 	}
 
@@ -346,9 +347,10 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 
 	B = (struct GRDBLEND_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
 
-	while (fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
+	while (GMT_fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
 		if (line[0] == '#' || line[0] == '\n' || line[0] == '\r') continue;	/* Skip comment lines or blank lines */
 
+		memset ((void *)&B[n], 0, sizeof (struct GRDBLEND_INFO));	/* Initialize memory */
 		nr = sscanf (line, "%s %s %lf", B[n].file, r_in, &B[n].weight);
 		if (nr != 3) {
 			fprintf (stderr, "%s: Read error for blending parameters near row %d\n", GMT_program, n);
@@ -368,17 +370,18 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		if (n == 0) {
 			h->node_offset = B[n].G.header.node_offset;
 			one_or_zero = !h->node_offset;
+			GMT_RI_prepare (h);	/* Ensure -R -I consistency and set nx, ny */
 		}
 		if (h->node_offset != B[n].G.header.node_offset){
-			fprintf (stderr, "%s: File %s has different registration that the first file given\n", GMT_program, B[n].file);
+			fprintf (stderr, "%s: File %s has different registration than the first file given\n", GMT_program, B[n].file);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.x_inc - h->x_inc) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different x-increment (%g) that the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.x_inc, h->x_inc);
+		if (!GMT_IS_ZERO (B[n].G.header.x_inc - h->x_inc)){
+			fprintf (stderr, "%s: File %s has different x-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.x_inc, h->x_inc);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.y_inc - h->y_inc) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different y-increment (%g) that the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.y_inc, h->y_inc);
+		if (!GMT_IS_ZERO (B[n].G.header.y_inc - h->y_inc)){
+			fprintf (stderr, "%s: File %s has different y-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.y_inc, h->y_inc);
 			exit (EXIT_FAILURE);
 		}
 		if (B[n].weight < 0.0) {	/* Negative weight means invert sense of taper */
@@ -427,13 +430,13 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		GMT_close_grd (&B[n].G);
 
 		n++;
-		if (n == (int)n_alloc) {
-			n_alloc += GMT_SMALL_CHUNK;
+		if (n == (GMT_LONG)n_alloc) {
+			n_alloc <<= 1;
 			B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
 		}
 	}
 
-	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n, sizeof (struct GRDBLEND_INFO), GMT_program);
+	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, (size_t)n, sizeof (struct GRDBLEND_INFO), GMT_program);
 	*blend = B;
 
 	return (n);
@@ -448,6 +451,7 @@ void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half
 			if (B[k].open) {
 				GMT_close_grd (&B[k].G);	/* Done with this file */
 				B[k].open = FALSE;
+				GMT_free ((void *)B[k].z);
 			}
 			continue;
 		}
@@ -492,7 +496,7 @@ void decode_R (char *string, double *w, double *e, double *s, double *n) {
 void *New_Grdblend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDBLEND_CTRL *C;
 	
-	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDBLEND_CTRL), "New_Grdblend_Ctrl");
+	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDBLEND_CTRL), "New_Grdblend_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -503,6 +507,6 @@ void *New_Grdblend_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Grdblend_Ctrl (struct GRDBLEND_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
