@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcontour.c,v 1.111 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdcontour.c,v 1.131 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,14 +16,14 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdcontour reads a 2-D grd file and contours it. This algorithm handles
+ * grdcontour reads a 2-D grid file and contours it. This algorithm handles
  * cases where the old contourm failed, e.g. when a contour line passes
  * exactly through a data point. This is achieved by adding a tiny
  * number to those data points that would have a contour line going
  * through them. This is cheating, but the human eye cannot tell the
  * difference, so who cares?
  * A multitude of options exist and are explained in the usage message.
- * The 2-D grd file format is outlined in the routines read_grd/write_grd.
+ * The 2-D grid file format is outlined in the routines read_grd/write_grd.
  *
  * Author:	Paul Wessel
  * Date:	6-JAN-1991
@@ -112,10 +112,12 @@ struct SAVE {
 
 int main (int argc, char **argv)
 {
-	int i, j, k, n, nn, nm, c, n_edges, n_alloc, section, n_contours, side, id;
+	int j, k, c, n_edges, n_alloc, section, n_contours, side, id;
 	int nx, ny, n_save = 0, got, rgb[3], *edge;
+	
+	GMT_LONG nm, i, n, nn;
 
-	BOOLEAN error = FALSE, first = TRUE, interior, begin, subset = FALSE;
+	BOOLEAN error = FALSE, interior, begin, subset = FALSE;
 
 	char *grdfile = 0, line[BUFSIZ], *cont_type, *cont_do_tick;
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
@@ -191,7 +193,7 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					if (!GMT_access (&argv[i][2], R_OK)) {	/* Gave a readable file */
 						Ctrl->C.interval = 1.0;
-						Ctrl->C.cpt = (!strncmp (&argv[i][strlen(argv[i])-4], ".cpt", 4)) ? TRUE : FALSE;
+						Ctrl->C.cpt = (!strncmp (&argv[i][strlen(argv[i])-4], ".cpt", (size_t)4)) ? TRUE : FALSE;
 						Ctrl->C.file = strdup (&argv[i][2]);
 					}
 					else
@@ -199,7 +201,7 @@ int main (int argc, char **argv)
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
-					GMT_free ((void *)Ctrl->D.file);
+					free ((void *)Ctrl->D.file);
 					Ctrl->D.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
@@ -303,7 +305,7 @@ int main (int argc, char **argv)
 						/* Get the argument following a or c and up to first comma, slash (or to the end) */
 						n = k+1;
 						while (!(argv[i][n] == ',' || argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-						strncpy (txt_a, &argv[i][k], n-k);	txt_a[n-k] = '\0';
+						strncpy (txt_a, &argv[i][k], (size_t)(n-k));	txt_a[n-k] = '\0';
 						if (GMT_colorname2index (txt_a) >= 0) j = k;	/* Found a colorname; wind j back by 1 */
 						id = (argv[i][k] == 'a') ? 1 : 0;
 						if (GMT_getpen (&argv[i][j], &Ctrl->W.pen[id])) {
@@ -338,10 +340,10 @@ int main (int argc, char **argv)
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<grdfile> is 2-D netCDF grdfile to be contoured\n");
+		fprintf (stderr, "\t<grdfile> is 2-D netCDF grid file to be contoured\n");
 		fprintf (stderr, "\t-C Contours to be drawn can be specified in one of three ways:\n");
 		fprintf (stderr, "\t   1. Fixed contour interval\n");
-		fprintf (stderr, "\t   2. Name of file with contour levels in col 1 and C(ont) or A(not) in col 2\n");
+		fprintf (stderr, "\t   2. Name of file with contour levels in col 1 and C(ont) or A(nnot) in col 2\n");
 		fprintf (stderr, "\t      [and optionally an individual annotation angle in col 3.]\n");
 		fprintf (stderr, "\t   3. Name of cpt-file\n");
 		fprintf (stderr, "\t   If -T is used, only contours with upper case C or A is ticked\n");
@@ -371,7 +373,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q Do not draw contours with less than <cut> points [Draw all contours]\n");
 		GMT_explain_option ('R');
-		fprintf (stderr, "\t   [Default is extent of grdfile]\n");
+		fprintf (stderr, "\t   [Default is extent of grid]\n");
 		fprintf (stderr, "\t-S will Smooth contours by splining and resampling\n");
 		fprintf (stderr, "\t   at approximately gridsize/<smooth> intervals\n");
 		fprintf (stderr, "\t-T will embellish innermost, closed contours with ticks pointing in the downward direction\n");
@@ -443,7 +445,7 @@ int main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	if (Ctrl->D.active && Ctrl->D.file[0] == 0) {
-		fprintf (stderr, "%s: contours will be written to file contour\n", GMT_program);
+		fprintf (stderr, "%s: contours will be written to files contour_*.xyz\n", GMT_program);
 		strcpy (Ctrl->D.file,"contour");
 	}
 	GMT_z_periodic = Ctrl->Z.periodic;	/* Phase data */
@@ -452,7 +454,7 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocate memory and read data file\n", GMT_program);
 
 	if (!strcmp (grdfile,  "=")) {
-		fprintf (stderr, "%s: Piping of grdfile not supported!\n", GMT_program);
+		fprintf (stderr, "%s: Piping of grid file not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
@@ -471,20 +473,23 @@ int main (int argc, char **argv)
 		subset = TRUE;
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	/* Determine the wesn to be used to read the grdfile */
+	/* Determine the wesn to be used to read the grid file */
 
 	if (GMT_grd_setregion (&header, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No data within specified region\n", GMT_program);
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	/* Read data */
 
-	if (subset) {	/* Possibly extend the subset by 1 row/col since tiles need 4 corners to work */
+	if (subset && header.node_offset) {	/* Possibly extend the subset by 1 row/col since pixel tiles need 4 corners to work */
 		BOOLEAN global;
-		global = (GMT_io.in_col_type[0] == GMT_IS_LON && fabs (fabs (header.x_max - header.x_min) - 360.0) < GMT_CONV_LIMIT);
+		global = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (header.x_max, header.x_min));
 		if (global && data_west > project_info.w)
 			data_west -= header.x_inc;
 		else if (GMT_x_to_i (data_west, header.x_min, header.x_inc, header.xy_off, header.nx) > 0)
@@ -502,13 +507,10 @@ int main (int argc, char **argv)
 
 	nx = GMT_get_n (data_west, data_east, header.x_inc, header.node_offset);
 	ny = GMT_get_n (data_south, data_north, header.y_inc, header.node_offset);
-	nm = nx * ny;
+	nm = ((size_t)nx) * ((size_t)ny);
 	grd_original = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (grdfile, &header, grd_original, data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile);
 
-	n_edges = header.ny * (int )ceil (header.nx / 16.0);
-	edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
-
 	if (!(Ctrl->Z.scale == 1.0 && Ctrl->Z.offset == 0.0)) {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying grid by %g\n", GMT_program, Ctrl->Z.offset, Ctrl->Z.scale);
 		for (i = 0; i < nm; i++) grd_original[i] = (float)((grd_original[i] - Ctrl->Z.offset) * Ctrl->Z.scale);
@@ -518,17 +520,19 @@ int main (int argc, char **argv)
 	}
 	if (Ctrl->L.low > header.z_min) header.z_min = Ctrl->L.low;
 	if (Ctrl->L.high < header.z_max) header.z_max = Ctrl->L.high;
-
+	if (Ctrl->L.active && header.z_max < header.z_min) {	/* Specified contour range outside range of data - quit */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No contours within specified -L range\n", GMT_program);
+		GMT_free ((void *)grd_original);
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
+		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
+	}
+	
 	small = Ctrl->C.interval * 1.0e-6;
 	if (Ctrl->A.interval == 0.0) Ctrl->A.interval = Ctrl->C.interval;
 
 	small_x = 0.01 * header.x_inc;	small_y = 0.01 * header.y_inc;
 
-	GMT_plotinit (argc, argv);
-	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
-
-	GMT_map_clip_on (GMT_no_rgb, 3);
-
 	if (Ctrl->contour.annot) {	/* Wants annotated contours */
 		aval = floor (header.z_min / Ctrl->A.interval) * Ctrl->A.interval;
 		if (aval < header.z_min) aval += Ctrl->A.interval;
@@ -544,6 +548,12 @@ int main (int argc, char **argv)
 
 	if (Ctrl->C.cpt) {	/* Presumably got a cpt-file */
 		GMT_read_cpt (Ctrl->C.file);
+#ifdef GMT_CPT2	
+		if (GMT_categorical) {
+			fprintf (stderr, "%s: GMT WARNING:  Categorical data (as implied by CPT file) does not have contours.  Check plot.\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+#endif
 		n_contours = GMT_n_colors + 1;
 		if (n_contours > n_alloc) {
 			contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
@@ -592,7 +602,7 @@ int main (int argc, char **argv)
 			if (got == 3) Ctrl->contour.angle_type = 2;	/* Must set this directly if angles are provided */
 			n_contours++;
 			if (n_contours == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
 				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
 				cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_alloc, sizeof (char), GMT_program);
@@ -606,7 +616,7 @@ int main (int argc, char **argv)
 		max = ceil (header.z_max / Ctrl->C.interval) * Ctrl->C.interval; if (max > header.z_max) max -= Ctrl->C.interval;
 		for (c = irint (min/Ctrl->C.interval), n_contours = 0; c <= irint (max/Ctrl->C.interval); c++, n_contours++) {
 			if (n_contours == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
 				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
 				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -618,11 +628,28 @@ int main (int argc, char **argv)
 			cont_do_tick[n_contours] = Ctrl->T.active;
 		}
 	}
-
-	if (GMT_z_periodic && fabs (contour[n_contours-1] - contour[0] - 360.0) < GMT_SMALL) {	/* Get rid of redundant contour */
+	if (GMT_z_periodic && n_contours > 1 && fabs (contour[n_contours-1] - contour[0] - 360.0) < GMT_SMALL) {	/* Get rid of redundant contour */
 		n_contours--;
 	}
 
+	if (n_contours == 0) {	/* No contours within range of data */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No contours found\n", GMT_program);
+		GMT_plotinit (argc, argv);
+		GMT_map_basemap ();
+		GMT_plotend ();
+		GMT_free ((void *)contour);
+		GMT_free ((void *)cont_type);
+		GMT_free ((void *)cont_angle);
+		GMT_free ((void *)cont_do_tick);
+		GMT_free ((void *)grd_original);
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
+		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
+
+	}
+	
+	/* OK, now we know we have contouring to do */
+	
 	contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
 	cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
 	cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
@@ -638,6 +665,14 @@ int main (int argc, char **argv)
 	 * original grid values and subtract the current contour value. */
 	 
 	grd = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+	n_edges = header.ny * (int )ceil (header.nx / 16.0);
+	edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
+
+
+	GMT_plotinit (argc, argv);
+	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
+
+	GMT_map_clip_on (GMT_no_rgb, 3);
 
 	for (c = 0; c < n_contours; c++) {	/* For each contour value cval */
 
@@ -654,7 +689,6 @@ int main (int argc, char **argv)
 		}
 
 		section = 0;
-		first = FALSE;
 		id = (cont_type[c] == 'A' || cont_type[c] == 'a') ? 1 : 0;
 
 		Ctrl->contour.line_pen = Ctrl->W.pen[id];	/* Load current pen into contour structure */
@@ -697,7 +731,7 @@ int main (int argc, char **argv)
 							save[n_save].cval = cval;
 							n_save++;
 							if (n_save == n_alloc) {
-								n_alloc += GMT_CHUNK;
+								n_alloc <<= 1;
 								save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
 							}
 						}
@@ -719,6 +753,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->T.active && n_save) {
 		save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_save, sizeof (struct SAVE), GMT_program);
+
 		sort_and_plot_ticks (save, n_save, &header, grd_original, Ctrl->T.spacing, Ctrl->T.length, Ctrl->T.low, Ctrl->T.high, Ctrl->T.label, Ctrl->T.L_label, Ctrl->T.H_label);
 		for (i = 0; i < n_save; i++) {
 			GMT_free ((void *)save[i].x);
@@ -731,6 +766,7 @@ int main (int argc, char **argv)
 	if (Ctrl->contour.hill_label) adjust_hill_label (&Ctrl->contour, &header, grd_original);	/* Must possibly adjust label angles so that label is readable when following contours */
 	
 	GMT_contlabel_plot (&Ctrl->contour);
+	GMT_contlabel_free (&Ctrl->contour);
 
 	GMT_map_clip_off ();
 
@@ -777,7 +813,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 	for (i = 0; i < n; i++) {	/* Mark polygons that have other polygons inside them */
 		np = save[i].n;
 		for (j = 0; save[i].do_it && j < n; j++) {
-			inside = GMT_non_zero_winding (save[j].x[0], save[j].y[0], save[i].x, save[i].y, np);
+			inside = GMT_non_zero_winding (save[j].x[0], save[j].y[0], save[i].x, save[i].y, (GMT_LONG)np);
 			if (inside == 2) save[i].do_it = FALSE;
 		}
 	}
@@ -787,7 +823,6 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 	for (i = 0; i < n; i++) {
 		if (!save[i].do_it) continue;
 		np = save[i].n;
-		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
 
 		/* Get rectangular surrounding box for this polygon */
 
@@ -805,16 +840,18 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		 */
 
 		GMT_xy_to_geo (&this_lon, &this_lat, 0.5 * (xmin + xmax), ymax);
+		if (GMT_is_dnan (this_lon) || GMT_is_dnan (this_lat)) continue;
 		ix = GMT_x_to_i (this_lon, h->x_min, h->x_inc, h->xy_off, h->nx);
 		iy = GMT_y_to_j (this_lat, h->y_min, h->y_inc, h->xy_off, h->ny);
 		this_lon = GMT_i_to_x (ix, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);	/* This is that mid longitude */
 		GMT_xy_to_geo (&a, &this_lat, 0.5 * (xmin + xmax), ymin);			/* Get bottom latitude */
+		if (GMT_is_dnan (a)) continue;
 		iy_s = MIN (h->ny, GMT_y_to_j (this_lat, h->y_min, h->y_inc, h->xy_off, h->ny) + 1);	/* And its j-row */
 		done = FALSE;
 		while (!done && iy < iy_s) {
 			this_lat = GMT_j_to_y (iy, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 			GMT_geo_to_xy (this_lon, this_lat, &x0, &y0);
-			inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, np);
+			inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, (GMT_LONG)np);
 			if (inside == 2)	/* OK, this point is inside */
 				done = TRUE;
 			else
@@ -830,6 +867,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			x_mean = save[i].x[0];
 			y_mean = save[i].y[0];
 		}
+		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
 		for (j = 1, s[0] = 0.0; j < np; j++) {
 			s[j] = s[j-1] + hypot (save[i].x[j]-save[i].x[j-1], save[i].y[j]-save[i].y[j-1]);
 			if (tick_label) {
@@ -847,7 +885,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		sincos (a, &sa, &ca);
 		x0 = 0.5 * (save[i].x[0] + save[i].x[1]) + small * ca;
 		y0 = 0.5 * (save[i].y[0] + save[i].y[1]) + small * sa;
-		inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, np);
+		inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, (GMT_LONG)np);
 		sign = (inside == 2) ? 1.0 : -1.0;
 
 		n_ticks = (int)(s[np-1] / tick_gap);
@@ -947,12 +985,12 @@ void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
 			GMT_geo_to_xy (x_node, y_node, &x_node_p, &y_node_p);	/* Projected coordinates of nearest node point */
 			dx = x_node_p - C->L[k].x;
 			dy = y_node_p - C->L[k].y;
-			if (hypot (dx, dy) < GMT_CONV_LIMIT) {
+			if (GMT_IS_ZERO (hypot (dx, dy))) {
 				fprintf (stderr, "%s: Unable to adjust hill label contour orientation (node point on contour)\n", GMT_program);
 				continue;
 			}
 			dz = z[j0*h->nx+i0] - C->z;
-			if (fabs (dz) < GMT_CONV_LIMIT) {
+			if (GMT_IS_ZERO (dz)) {
 				fprintf (stderr, "%s: Unable to adjust hill label contour orientation (node value = contour value)\n", GMT_program);
 				continue;
 			}
@@ -966,7 +1004,7 @@ void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
 void *New_Grdcontour_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDCONTOUR_CTRL *C;
 	
-	C = (struct GRDCONTOUR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDCONTOUR_CTRL), "New_Grdcontour_Ctrl");
+	C = (struct GRDCONTOUR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDCONTOUR_CTRL), "New_Grdcontour_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -992,6 +1030,7 @@ void *New_Grdcontour_Ctrl () {	/* Allocate and initialize a new control structur
 }
 
 void Free_Grdcontour_Ctrl (struct GRDCONTOUR_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->D.file) free ((void *)C->D.file);	
 	GMT_free ((void *)C);	
 }
