@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcut.c,v 1.27 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdcut.c,v 1.34 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdcut.c reads a grdfile and writes a portion within it
+ * grdcut.c reads a grid file and writes a portion within it
  * to a new file.
  *
  * Author:	Walter Smith
@@ -39,6 +39,8 @@ int main (int argc, char **argv)
 	BOOLEAN	error = FALSE;
 
 	int	i, nx_old, ny_old, nx_new, ny_new, one_or_zero;
+	
+	size_t	nm;
 
 	char *grd_in, format[BUFSIZ];
 
@@ -87,13 +89,13 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdcut %s - Extract subsets from grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr,"grdcut %s - Extract subsets from grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdcut <input_grd> -G<output_grd> %s [-V] [%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<input_grd> is file to extract a subset from.\n");
-		fprintf (stderr, "\t-G specifies output grdfile\n");
+		fprintf (stderr, "\t-G specifies output grid file\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t   Obviously, the WESN you specify must be within the WESN of the input file.\n");
 		fprintf (stderr, "\t   If in doubt, run grdinfo first and check range of old file.\n");
@@ -124,7 +126,7 @@ int main (int argc, char **argv)
 	if (s_new < header.y_min || s_new > header.y_max) error = TRUE;
 	if (n_new < header.y_min || n_new > header.y_max) error = TRUE;
 
-	if (GMT_io.in_col_type[GMT_X] == GMT_IS_LON) {
+	if (GMT_io.in_col_type[GMT_X] == GMT_IS_LON) {	/* Geographic data */
 		if (w_new < header.x_min && e_new < header.x_min) {
 			header.x_min -= 360.0;
 			header.x_max -= 360.0;
@@ -133,8 +135,10 @@ int main (int argc, char **argv)
 			header.x_min += 360.0;
 			header.x_max += 360.0;
 		}
+		if (!GMT_360_RANGE (header.x_min, header.x_max) && (w_new < header.x_min || e_new > header.x_max)) error = TRUE;
 	}
-	if (w_new < header.x_min || e_new > header.x_max) error = TRUE;
+	else if (w_new < header.x_min || e_new > header.x_max)
+		error = TRUE;
 
 	if (error) {
 		fprintf (stderr, "%s: Subset exceeds data domain!\n", GMT_program);
@@ -175,11 +179,12 @@ int main (int argc, char **argv)
 	nx_new = irint ((e_new - w_new) / header.x_inc) + one_or_zero;
 	ny_new = irint ((n_new - s_new) / header.y_inc) + one_or_zero;
 
-	grd = (float *) GMT_memory (VNULL, (size_t)(nx_new * ny_new), sizeof (float), GMT_program);
+	nm = ((size_t)nx_new) * ((size_t)ny_new);
+	grd = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (grd_in, &header, grd, w_new, e_new, s_new, n_new, GMT_pad, FALSE), grd_in);
 
 	if (gmtdefs.verbose) {
-		sprintf (format, "\t%s\t%s\t%s\t%s\t%s\t%s\t%%d\t%%d\n", gmtdefs.d_format, gmtdefs.d_format,
+		sprintf (format, "\t%s\t%s\t%s\t%s\t%s\t%s\t%%ld\t%%ld\n", gmtdefs.d_format, gmtdefs.d_format,
 			gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, "%s: File spec:\tW E S N dx dy nx ny:\n", GMT_program);
 		fprintf (stderr, "%s: Old:", GMT_program);
@@ -202,7 +207,7 @@ int main (int argc, char **argv)
 void *New_Grdcut_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDCUT_CTRL *C;
 	
-	C = (struct GRDCUT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDCUT_CTRL), "New_Grdcut_Ctrl");
+	C = (struct GRDCUT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDCUT_CTRL), "New_Grdcut_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 			
@@ -210,6 +215,6 @@ void *New_Grdcut_Ctrl () {	/* Allocate and initialize a new control structure */
 }
 
 void Free_Grdcut_Ctrl (struct GRDCUT_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
