@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdedit.c,v 1.39 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdedit.c,v 1.49 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdedit reads an existing grdfile and takes command line
+ * grdedit reads an existing grid file and takes command line
  * arguments to redefine some of the grdheader parameters:
  *
  *	x_min/x_max OR x_inc (the other is recomputed)
@@ -61,8 +61,10 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE;
 
-	int i, j, k, n_expected_fields, n_fields, n_data;
-
+	int i, j, n_expected_fields, n_fields;
+	
+	GMT_LONG n_data, k;
+	
 	float *a;
 
 	double w, e, s, n, shift_amount = 0.0, f, *in;
@@ -132,7 +134,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdedit %s - Modifying the header or content of a 2-D grdfile\n\n", GMT_VERSION);
+		fprintf (stderr, "grdedit %s - Modifying the header or content of a 2-D grid file\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdedit grdfile [-A] [%s]\n", GMT_GRDEDIT);
 		fprintf (stderr, "\t[-E] [%s] [-N<xyzfile>] [%s] [-S] [-T]\n", GMT_H_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-V] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
@@ -181,7 +183,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  Must also specify -R\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->S.active && fabs (e-w-360.0) > GMT_CONV_LIMIT) {
+	if (Ctrl->S.active && !GMT_360_RANGE (w,e)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  -R longitudes must span exactly 360 degrees\n", GMT_program);
 		error++;
 	}
@@ -207,12 +209,18 @@ int main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	if (!strcmp (grdfile,  "=")) {
-		fprintf (stderr, "%s: Piping of grdfile not supported!\n", GMT_program);
+		fprintf (stderr, "%s: Piping of grid file not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
 	GMT_err_fail (GMT_read_grd_info (grdfile, &grd), grdfile);
 
+	if ((grd.type == 6 || grd.type == 20) && Ctrl->T.active) {
+		fprintf (stderr, "%s: Toggling registrations not possible for Surfer grid formats\n", GMT_program);
+		fprintf (stderr, "%s: (Use grdreformat to convert to GMT default format and work on that file)\n", GMT_program);
+		exit (EXIT_FAILURE);
+	}
+	
 	if (Ctrl->S.active && !GMT_grd_is_global (&grd)) {
 		fprintf (stderr, "%s: Shift only allowed for global grids\n", GMT_program);
 		exit (EXIT_FAILURE);
@@ -223,9 +231,15 @@ int main (int argc, char **argv)
 	/* Decode grd information given, if any */
 
 	if (Ctrl->D.active) {
+		double scale_factor, add_offset;
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Decode and change attributes in file %s\n", GMT_program, grdfile);
+		scale_factor = grd.z_scale_factor;
+		add_offset = grd.z_add_offset;
 		GMT_decode_grd_h_info (Ctrl->D.information, &grd);
-
+		if (scale_factor != grd.z_scale_factor || add_offset != grd.z_add_offset) {
+			grd.z_min = (grd.z_min - add_offset) / scale_factor * grd.z_scale_factor + grd.z_add_offset;
+			grd.z_max = (grd.z_max - add_offset) / scale_factor * grd.z_scale_factor + grd.z_add_offset;
+		}
 	}
 
 	if (Ctrl->S.active) {
@@ -257,14 +271,16 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 			n_data++;
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_data);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_data);
 				exit (EXIT_FAILURE);
 			}
+			if (GMT_y_is_outside (in[1],  grd.y_min, grd.y_max)) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[0], grd.x_min, grd.x_max)) continue;	/* Outside x-range */
 			j = GMT_y_to_j(in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 			if (j < 0 || j >= grd.ny) continue;
 			i = GMT_x_to_i(in[0], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
 			if (i < 0 || i >= grd.nx) continue;
-			k = j * grd.nx + i;
+			k = GMT_IJ (j, i, grd.nx);
 			a[k] = (float)in[2];
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -274,11 +290,11 @@ int main (int argc, char **argv)
 	}
 	else if (Ctrl->E.active) {	/* Transpose the matrix and exchange x and y info */
 		struct GRD_HEADER grd_tr;
-		int row, col, ij, ij_tr;
+		int row, col;
+		GMT_LONG ij, ij_tr;
 		float *a_tr;
 		
 		a = (float *) GMT_memory (VNULL, (size_t)(grd.nx * grd.ny), sizeof (float), GMT_program);
-		a_tr = (float *) GMT_memory (VNULL, (size_t)(grd.ny * grd.nx), sizeof (float), GMT_program);
 		GMT_err_fail (GMT_read_grd (grdfile, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
 		memcpy ((void *)&grd_tr, (void *)&grd, sizeof (struct GRD_HEADER));	/* First make a copy of header */
 		grd_tr.nx = grd.ny;	/* Then exchange x and y values */
@@ -292,17 +308,18 @@ int main (int argc, char **argv)
 		grd_tr.y_inc = grd.x_inc;
 		strcpy (grd_tr.y_units, grd.x_units);
 		/* Now transpose the matrix */
-		
+
+		a_tr = (float *) GMT_memory (VNULL, (size_t)(grd.ny * grd.nx), sizeof (float), GMT_program);
 		for (row = 0; row < grd.ny; row++) {
 			for (col = 0; col < grd.nx; col++) {
-				ij = row * grd.nx + col;
-				ij_tr = col * grd.ny + row;
+				ij = GMT_IJ (row, col, grd.nx);
+				ij_tr = GMT_IJ (col, row, grd.ny);
 				a_tr[ij_tr] = a[ij];
 			}
 		}
 		GMT_err_fail (GMT_write_grd (grdfile, &grd_tr, a_tr, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
-		GMT_free ((void *)a);
 		GMT_free ((void *)a_tr);
+		GMT_free ((void *)a);
 	}
 	else {
 		if (project_info.region_supplied) {
@@ -341,7 +358,7 @@ int main (int argc, char **argv)
 void *New_Grdedit_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDEDIT_CTRL *C;
 	
-	C = (struct GRDEDIT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDEDIT_CTRL), "New_Grdedit_Ctrl");
+	C = (struct GRDEDIT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDEDIT_CTRL), "New_Grdedit_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
@@ -349,7 +366,8 @@ void *New_Grdedit_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Grdedit_Ctrl (struct GRDEDIT_CTRL *C) {	/* Deallocate control structure */
-	if (C->D.information) GMT_free ((void *)C->D.information);	
-	if (C->N.file) GMT_free ((void *)C->N.file);	
+	if (C->D.information) free ((void *)C->D.information);	
+	if (C->N.file) free ((void *)C->N.file);	
 	GMT_free ((void *)C);	
 }
+
