@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfft.c,v 1.32 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdfft.c,v 1.37 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,7 @@
 /*
  *  grdfft.c
  *
- * A program to do various operations
- * on grdfiles in the frequency domain.
+ * A program to do various operations on grid files in the frequency domain.
  *
  * W.H.F. Smith, 7 March 1990.
  *
@@ -112,7 +111,7 @@ struct GRDFFT_CTRL {
 #define	ij_data(i,j) (2*(nx2*((j)+j_data_start)+(i)+i_data_start))
 
 
-int	narray[2];
+GMT_LONG	narray[2];
 int	i, j, k, n, nx2 = 0, ny2 = 0, ndatac, i_data_start, j_data_start;
 
 BOOLEAN	map_units = FALSE, force_narray = FALSE, suggest_narray = FALSE, leave_trend_alone = FALSE, n_user_set = FALSE;
@@ -141,8 +140,8 @@ struct F_INFO {
 struct FFT_SUGGESTION {
 	int	nx;
 	int	ny;
-	int	worksize;	/* # single-complex elements needed in work array  */
-	int	totalbytes;	/* (8*(nx*ny + worksize))  */
+	GMT_LONG	worksize;	/* # single-complex elements needed in work array  */
+	GMT_LONG	totalbytes;	/* (8*(nx*ny + worksize))  */
 	double	run_time;
 	double	rms_rel_err;
 }	fft_sug[3];	/* [0] holds fastest, [1] most accurate, [2] least storage  */
@@ -338,16 +337,16 @@ int main (int argc, char **argv)
         }
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdfft %s - Perform mathematical operations on grdfiles in the wavenumber (or frequency) domain\n\n", GMT_VERSION);
+		fprintf (stderr, "grdfft %s - Perform mathematical operations on grid files in the wavenumber (or frequency) domain\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: grdfft <in_grdfile> [-G<out_grdfile>] [-A<azimuth>] [-C<zlevel>]\n");
 		fprintf (stderr,"\t[-D[<scale>|g]] [-E[x_or_y][w]] [-F[x_or_y]<parameters>] [-I[<scale>|g]] [-L] [-M]\n");
 		fprintf (stderr,"\t[-N<stuff>] [-S<scale>] [-T<te/rl/rm/rw/ri>] [-V]\n\n");
 		
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
-		fprintf (stderr,"\tin_grdfile is the input netCDF grdfile\n");
+		fprintf (stderr,"\tin_grdfile is the input netCDF grid file\n");
 		fprintf (stderr, "\tOPTIONS:\n");
-		fprintf (stderr,"\t-G filename for output netCDF grdfile\n");
+		fprintf (stderr,"\t-G filename for output netCDF grid file\n");
 		fprintf (stderr,"\t-A<azimuth> Take azimuthal derivative along line <azimuth> degrees CW from North.\n");
 		fprintf (stderr,"\t-C<zlevel>  Continue field upward (+) or downward (-) to zlevel (meters).\n");
 		fprintf (stderr,"\t-D Differentiate, i.e., multiply by kr [ * scale].  Use -Dg to get mGal from m]\n");
@@ -365,9 +364,9 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t-N<stuff>  Choose or inquire about suitable grid dimensions for FFT.\n");
 		fprintf (stderr,"\t\t-Nf will force the FFT to use the dimensions of the data.\n");
 		fprintf (stderr,"\t\t-Nq will inQuire about more suitable dimensions.\n");
-		fprintf (stderr,"\t\t-N<nx>/<ny> will do FFT on array size <nx>/<ny> (Must be >= grdfile size).\n");
+		fprintf (stderr,"\t\t-N<nx>/<ny> will do FFT on array size <nx>/<ny> (Must be >= grid size).\n");
 		fprintf (stderr,"\t\tDefault chooses dimensions >= data which optimize speed, accuracy of FFT.\n");
-		fprintf (stderr,"\t\tIf FFT dimensions > grdfile dimensions, data are extended and tapered to zero.\n");
+		fprintf (stderr,"\t\tIf FFT dimensions > grid dimensions, data are extended and tapered to zero.\n");
 		fprintf (stderr,"\t-S multiply field by scale after inverse FFT [1.0]\n");
 		fprintf (stderr,"\t   Give -Sd to convert deflection of vertical to micro-radians\n");
 		fprintf (stderr,"\t-T Compute isostatic response.  Input file is topo load. Append elastic thickness,\n");
@@ -503,7 +502,7 @@ void read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 	/* Get dimensions as may be appropriate:  */
 	if (n_user_set) {
 		if (nx2 < h.nx || ny2 < h.ny) {
-			fprintf(stderr,"%s: Error: You specified -Nnx/ny smaller than input grdfile.  Ignored.\n", GMT_program);
+			fprintf(stderr,"%s: Error: You specified -Nnx/ny smaller than input grid.  Ignored.\n", GMT_program);
 			n_user_set = FALSE;
 		}
 	}
@@ -628,7 +627,7 @@ void remove_plane (void)
 
 void taper_edges (void)
 {
-	int	im, jm, il1, ir1, il2, ir2, jb1, jb2, jt1, jt2;
+	GMT_LONG	im, jm, il1, ir1, il2, ir2, jb1, jb2, jt1, jt2;
 	double	scale, cos_wt;
 
 	/* Note that if nx2 = h.nx+1 and ny2 = h.ny + 1, then this routine
@@ -685,35 +684,35 @@ void taper_edges (void)
 	}
 }
 
-double kx (int k)
+double kx (GMT_LONG k)
 {
 	/* Return the value of kx given k,
 		where kx = 2 pi / lambda x,
 		and k refers to the position
 		in the datac array, datac[k].  */
 
-	int	ii;
+	GMT_LONG	ii;
 
-	ii = ((int)(k/2))%nx2;
+	ii = (k/2)%nx2;
 	if (ii > nx2/2) ii -= nx2;
 	return(ii * delta_kx);
 }
 
-double ky (int k)
+double ky (GMT_LONG k)
 {
 	/* Return the value of ky given k,
 		where ky = 2 pi / lambda y,
 		and k refers to the position
 		in the datac array, datac[k].  */
 
-	int	jj;
+	GMT_LONG	jj;
 
-	jj = ((int)(k/2))/nx2;
+	jj = (k/2)/nx2;
 	if (jj > ny2/2) jj -= ny2;
 	return(jj * delta_ky);
 }
 
-double modk (int k)
+double modk (GMT_LONG k)
 {
 	/* Return the value of sqrt(kx*kx + ky*ky),
 		given k, where k is array position.  */
@@ -723,7 +722,7 @@ double modk (int k)
 
 void do_differentiate (double *par)
 {
-	int	k;
+	GMT_LONG	k;
 	double scale, fact;
 
 	/* Differentiate in frequency domain by multiplying by kr [scale optional] */
@@ -740,7 +739,7 @@ void do_differentiate (double *par)
 void do_integrate (double *par)
 {
 	/* Integrate in frequency domain by dividing by kr [scale optional] */
-	int	k;
+	GMT_LONG	k;
 	double fact, scale;
 
 	scale = (*par != 0.0) ? *par : 1.0;
@@ -754,7 +753,7 @@ void do_integrate (double *par)
 
 void do_continuation (double *zlevel)
 {
-	int	k;
+	GMT_LONG	k;
 	double tmp;
 
 	/* If z is positive, the field will be upward continued using exp[- k z].  */
@@ -768,7 +767,7 @@ void do_continuation (double *zlevel)
 
 void do_azimuthal_derivative (double *azim)
 {
-	int	k;
+	GMT_LONG	k;
 	float	tempr, tempi, fact;
 	double cos_azim, sin_azim;
 
@@ -793,7 +792,7 @@ void do_isostasy (double *par)
 	densities in kg/m**3, Te in m, etc.
 	rw, the water density, is used to set the Airy ratio and the restoring
 	force on the plate (rm - ri)*gravity if ri = rw; so use zero for topo in air.  */
-	int	k;
+	GMT_LONG	k;
 	double	airy_ratio, rigidity_d, d_over_restoring_force, mk, k2, k4, transfer_fn;
 
 	double	te;	/* Elastic thickness, SI units (m)  */
@@ -828,8 +827,8 @@ void do_isostasy (double *par)
 
 void do_filter_cos (void)
 {
-	int	k;
-	double	weight, f_wt_cos(int k);
+	GMT_LONG	k;
+	double	weight, f_wt_cos(GMT_LONG k);
 	for (k = 0; k < ndatac; k += 2) {
 		weight = f_wt_cos(k);
 		datac[k] *= (float)weight;
@@ -839,8 +838,8 @@ void do_filter_cos (void)
 
 void do_filter_exp (void)
 {
-	int	k;
-	double	weight, f_wt_exp(int k);
+	GMT_LONG	k;
+	double	weight, f_wt_exp(GMT_LONG k);
 	for (k = 0; k < ndatac; k += 2) {
 		weight = f_wt_exp(k);
 		datac[k] *= (float)weight;
@@ -848,10 +847,10 @@ void do_filter_exp (void)
 	}
 }
 
-double f_wt_cos (int k)
+double f_wt_cos (GMT_LONG k)
 {
 	double	freq, return_value = 1.0;
-	int	j;
+	GMT_LONG	j;
 	
 	/* Always return immediately when zero weight encountered.  */
 	
@@ -882,10 +881,10 @@ double f_wt_cos (int k)
 	return (return_value);
 }
 
-double f_wt_exp (int k)
+double f_wt_exp (GMT_LONG k)
 {
 	double	freq, return_value = 1.0, lo, hi;
-	int	j;
+	GMT_LONG	j;
 		
 	for (j = 0; j < 3; j++) {
 		if (!(f_info.do_this[j])) continue;	/* Only do one of x, y, or r filtering */
@@ -1013,7 +1012,7 @@ void do_spectrum (double *par, int give_wavelength)
 	 *	approximation of the integral.
 	 */
 
-	int	k, nk, nused, ifreq;
+	GMT_LONG	k, nk, nused, ifreq;
 	double	delta_k, r_delta_k, freq, *power, eps_pow;
 	PFD	get_k;
 	char	format[GMT_TEXT_LEN];
@@ -1114,21 +1113,22 @@ void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
 	 * W. H. F. Smith, 26 February 1992.
 	 *  */
 
-	int	n_factors, i, ntotal, sum2, sumnot2, nnot2;
+	int	n_factors, i, sum2, sumnot2, nnot2;
 	int	nonsymx, nonsymy, nonsym, storage;
+	GMT_LONG ntotal;
 	double	err_scale, sig_bits = FSIGNIF;
-	int get_prime_factors(int n, int *f), get_non_symmetric_f(int *f, int n);
+	int get_prime_factors(GMT_LONG n, int *f), get_non_symmetric_f(int *f, int n);
 
 	/* Find workspace needed.  First find non_symmetric factors in nx, ny  */
-	n_factors = get_prime_factors(nx, f);
+	n_factors = get_prime_factors((GMT_LONG)nx, f);
 	nonsymx = get_non_symmetric_f(f, n_factors);
-	n_factors = get_prime_factors(ny, f);
+	n_factors = get_prime_factors((GMT_LONG)ny, f);
 	nonsymy = get_non_symmetric_f(f, n_factors);
 	nonsym = MAX(nonsymx, nonsymy);
 
 	/* Now get factors of ntotal  */
-	ntotal = nx * ny;
-        n_factors = get_prime_factors(ntotal, f);
+	ntotal = ((size_t)nx) * ((size_t)ny);
+        n_factors = get_prime_factors (ntotal, f);
 	storage = MAX(nonsym, f[n_factors - 1]);
 	if (storage == 2)
 		*s = 0;
@@ -1251,7 +1251,7 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 			nx_best_t, ny_best_t, best_time, t_err, t_space);
 		fprintf(stderr,"%s:  Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
 			nx_best_e, ny_best_e, e_time, best_err, e_space);
-		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %ds\n", GMT_program,
 			nx_best_s, ny_best_s, s_time, s_err, best_space);
 	}
 	/* Fastest solution:  */
@@ -1279,7 +1279,7 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	return;
 }
 
-int get_prime_factors (int n, int *f)
+int get_prime_factors (GMT_LONG n, int *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1306,17 +1306,22 @@ int get_prime_factors (int n, int *f)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992, after D.E. Knuth, vol. II  */
 
-	int	current_factor;	/* The factor currently being tried  */
-	int	max_factor;	/* Don't try any factors bigger than this  */
+	GMT_LONG	current_factor;	/* The factor currently being tried  */
+	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
 	int	n_factors = 0;	/* Returned; one if n is prime  */
-	int	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
-	int	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
-	int	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
-	int	m;	/* Used to keep a working copy of n  */
+	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
+	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
+	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
+	GMT_LONG	m;	/* Used to keep a working copy of n  */
 
 
 	/* Initialize m and max_factor  */
+	
+#ifdef __LP64__
+	m = labs(n);
+#else
 	m = abs(n);
+#endif
 	if (m < 2) return(0);
 	max_factor = (int)floor(sqrt((double)m));
 
@@ -1399,7 +1404,7 @@ int get_prime_factors (int n, int *f)
 void *New_Grdfft_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDFFT_CTRL *C;
 	
-	C = (struct GRDFFT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDFFT_CTRL), "New_Grdfft_Ctrl");
+	C = (struct GRDFFT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDFFT_CTRL), "New_Grdfft_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
@@ -1409,6 +1414,6 @@ void *New_Grdfft_Ctrl () {	/* Allocate and initialize a new control structure */
 void Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C) {	/* Deallocate control structure */
 	if (C->operation) GMT_free ((void *)C->operation);	
 	if (C->par) GMT_free ((void *)C->par);	
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
