@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfilter.c,v 1.51 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdfilter.c,v 1.60 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdfilter.c  reads a grdfile and creates filtered grd file
+ * grdfilter.c  reads a grid file and creates filtered grid file
  *
  * user selects boxcar, gaussian, cosine arch, median, or mode filter
  * user selects distance scaling as appropriate for map work, etc.
@@ -38,6 +38,7 @@
  * Version:	4
  *		PW: 27-MAY-2004 Added extreme values filter options l, L, u, U
  *		PW: 16-NOV-2005 Added spherical filtering support & highpass option
+ *		PW: 27-APR-2007 Added -D5 to handle Mercator (img) grids
 */
 
 #define GMT_WITH_NO_PS
@@ -68,6 +69,9 @@ struct GRDFILTER_CTRL {
 	} T;
 };
 
+#define IMG2LAT(img) (2.0*atan(exp((img)*D2R))*R2D - 90.0)
+#define LAT2IMG(lat) (R2D*log(tand(0.5*((lat)+90.0))))
+
 #define GRDFILTER_WIDTH		0
 #define GRDFILTER_HALF_WIDTH	1
 #define GRDFILTER_X_SCALE	2
@@ -77,11 +81,11 @@ struct GRDFILTER_CTRL {
 #define GRDFILTER_N_FILTERS	9
 
 struct FILTER_INFO {
-	int nx;			/* The max number of filter weights in x-direction */
-	int ny;			/* The max number of filter weights in y-direction */
-	int x_half_width;	/* Number of filter nodes to either side needed at this latitude */
-	int y_half_width;	/* Number of filter nodes above/below this point (ny_f/2) */
-	int d_flag;
+	GMT_LONG nx;			/* The max number of filter weights in x-direction */
+	GMT_LONG ny;			/* The max number of filter weights in y-direction */
+	GMT_LONG x_half_width;	/* Number of filter nodes to either side needed at this latitude */
+	GMT_LONG y_half_width;	/* Number of filter nodes above/below this point (ny_f/2) */
+	GMT_LONG d_flag;
 	double dx, dy;		/* Grid spacing in original units */
 	double *x, *y;		/* Distances in original units along x and y to distance nodes */
 	double par[5];		/* [0] is filter width, [1] is 0.5*filter_width, [2] is xscale, [3] is yscale, [4] is 1/r_half for filter */
@@ -91,10 +95,12 @@ struct FILTER_INFO {
 
 int main (int argc, char **argv)
 {
-	int	nx_out, ny_out, n_in_median, n_nan = 0, i_west, i_east, j_s_pole, j_n_pole;
-	int	j_origin, i_out, j_out, half_nx, i_orig = 0;
-	int	i_in, j_in, ii, jj, i, j, ij_in, ij_out, ij_wt, effort_level;
-	int	filter_type, one_or_zero = 1, GMT_n_multiples = 0, *i_origin;
+	GMT_LONG	nx_out, ny_out, n_in_median, n_nan = 0, i_west, i_east, j_s_pole, j_n_pole;
+	GMT_LONG	j_origin, i_out, j_out, half_nx, i_orig = 0;
+	GMT_LONG	i_in, j_in, ii, jj, i, j, ij_in, ij_out, ij_wt, effort_level;
+	int	filter_type, one_or_zero = 1, GMT_n_multiples = 0;
+	
+	int *i_origin;
 
 
 	BOOLEAN	error, new_range, fast_way, slow = FALSE;
@@ -103,10 +109,10 @@ int main (int argc, char **argv)
 
 	float	*input, *output;
 
-	double	west_new, east_new, south_new, north_new;
-	double	x_scale, y_scale, x_width, y_width, y, par[5], pole_weight;
+	double	west_new, east_new, south_new, north_new, merc_range, lat_out;
+	double	x_scale = 1.0, y_scale = 1.0, x_width, y_width, y, par[5], pole_weight;
 	double	x_out, y_out, wt_sum, value, last_median, this_median, xincnew2, yincnew2;
-	double	xincold2, yincold2, y_shift = 0.0, x_fix = 0.0, y_fix = 0.0;
+	double	xincold2, yincold2, y_shift = 0.0, x_fix = 0.0, y_fix = 0.0, max_lat;
 	double	*weight, *work_array = VNULL, *x_shift = VNULL;
 
 	char	*fin = CNULL, c, filter_code[GRDFILTER_N_FILTERS] = {'b', 'c', 'g', 'm', 'p', 'l', 'L', 'u', 'U'};
@@ -197,7 +203,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdfilter %s - Filter a 2-D grdfile in the space (or time) domain\n\n", GMT_VERSION);
+		fprintf (stderr, "grdfilter %s - Filter a 2-D grid file in the space (or time) domain\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: grdfilter input_file -D<distance_flag> -F<type>[-]<filter_width>[<mode>]\n");
 		fprintf(stderr,"\t-G<output_file> [%s] [%s] [-T] [-V] [%s]\n", GMT_I_OPT, GMT_Rgeo_OPT, GMT_f_OPT);
 
@@ -208,9 +214,10 @@ int main (int argc, char **argv)
 		fprintf(stderr,"\t   -D1 grid x,y in degrees, <filter_width> in km, cartesian Distances.\n");
 		fprintf(stderr,"\t   -D2 grid x,y in degrees, <filter_width> in km, x_scaled by cos(middle y), cartesian Distances.\n");
 		fprintf(stderr,"\t   These first three options are faster; they allow weight matrix to be computed only once.\n");
-		fprintf(stderr,"\t   Next two options are slower; weights must be recomputed for each scan line.\n");
+		fprintf(stderr,"\t   Next three options are slower; weights must be recomputed for each scan line.\n");
 		fprintf(stderr,"\t   -D3 grid x,y in degrees, <filter_width> in km, x_scale varies as cos(y), cartesian Distances.\n");
 		fprintf(stderr,"\t   -D4 grid x,y in degrees, <filter_width> in km, spherical Distances.\n");
+		fprintf(stderr,"\t   -D5 grid x,y in Mercator units (-Jm1), <filter_width> in km, spherical Distances.\n");
 		fprintf(stderr,"\t-F sets the low-pass filter type and full diameter (6 sigma) filter-width.  Choose between\n");
 		fprintf(stderr,"\t   convolution-type filters which differ in how weights are assigned and geospatial\n");
 		fprintf(stderr,"\t   filters that seek to return a representative value.\n");
@@ -228,7 +235,7 @@ int main (int argc, char **argv)
 		fprintf(stderr,"\t        Append - or + to the width to instead return the smallest or largest mode.\n");
 		fprintf(stderr,"\t     u: Upper : return maximum of all points.\n");
 		fprintf(stderr,"\t     U: Upper- : return maximum of all -ve points.\n");
-		fprintf(stderr,"\t-G sets output name for filtered grdfile\n");
+		fprintf(stderr,"\t-G sets output filename for filtered grid\n");
 		fprintf(stderr, "\n\tOPTIONS:\n");
 		GMT_inc_syntax ('I', 0);
 		fprintf(stderr,"\t   The new xinc and yinc should be divisible by the old ones (new lattice is subset of old).\n");
@@ -248,8 +255,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->D.mode < 0 || Ctrl->D.mode > 4) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  Choose from the range 0-4\n", GMT_program);
+	if (Ctrl->D.mode < 0 || Ctrl->D.mode > 5) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  Choose from the range 0-5\n", GMT_program);
 		error++;
 	}
 	if (!Ctrl->F.active) {
@@ -290,7 +297,7 @@ int main (int argc, char **argv)
 	else
 		one_or_zero = (h.node_offset) ? 0 : 1;
 
-	/* Read the input grd file and close it  */
+	/* Read the input grid file and close it  */
 
 	input = (float *) GMT_memory (VNULL, (size_t)(h.nx * h.ny), sizeof(float), GMT_program);
 
@@ -329,6 +336,10 @@ int main (int argc, char **argv)
 	test_h.y_min = south_new;	test_h.y_max = north_new;	test_h.y_inc = Ctrl->I.yinc;
 	GMT_RI_prepare (&test_h);	/* Ensure -R -I consistency and set nx, ny */
 	GMT_err_fail (GMT_grd_RI_verify (&test_h, 1), Ctrl->G.file);
+	/* Copy back in case test_h were changed */
+	west_new  = test_h.x_min;	east_new  = test_h.x_max;	Ctrl->I.xinc = test_h.x_inc;
+	south_new = test_h.y_min;	north_new = test_h.y_max;	Ctrl->I.yinc = test_h.y_inc;
+	
 
 	/* We can save time by computing a weight matrix once [or once pr scanline] only
 	   if new grid spacing is a multiple of old spacing */
@@ -359,8 +370,8 @@ int main (int argc, char **argv)
 		x_fix = 0.5 * h.x_inc;
 		y_fix = 0.5 * h.y_inc;
 	}
-	full_360 = (Ctrl->D.mode && fabs (fabs (h.x_max - h.x_min) - 360.0) < GMT_CONV_LIMIT);	/* Periodic geographic grid */
-	full_180 = (Ctrl->D.mode && fabs (fabs (h.y_max - h.y_min) - 180.0) < GMT_CONV_LIMIT);	/* Full latitude range for geographic grid */
+	full_360 = (Ctrl->D.mode && GMT_360_RANGE (h.x_max, h.x_min));	/* Periodic geographic grid */
+	full_180 = (Ctrl->D.mode && GMT_180_RANGE (h.y_min, h.y_max));	/* Full latitude range for geographic grid */
 
 	/* Set up the distance scalings for lon and lat, and assign pointer to distance function  */
 
@@ -383,13 +394,22 @@ int main (int argc, char **argv)
 			y_scale = project_info.KM_PR_DEG;
 			F.radius_func = FlatEarthRadius;
 			break;
-		default:	/* Great circle distances */
+		case 4:	/* Great circle distances */
 			x_scale = 0.0;
 			y_scale = project_info.KM_PR_DEG;
 			F.radius_func = SphericalRadius;
 			if (full_360) wrap_case_x = TRUE;		/* For spherical filtering */
 			if (full_180) wrap_case_y = wrap_case_x;	/* For spherical filtering */
 			break;
+		case 5:	/* Great circle distances with Mercator coordinates */
+			x_scale = 0.0;
+			/* Get the max |lat| extent of the grid */
+			max_lat = IMG2LAT (MAX (fabs (h.y_min), fabs (h.y_max)));
+			merc_range = LAT2IMG (max_lat + (0.5 * Ctrl->F.width / project_info.KM_PR_DEG)) - LAT2IMG (max_lat);
+			y_scale = 0.5 * Ctrl->F.width / merc_range;
+			F.radius_func = SphericalRadius;
+			if (full_360) wrap_case_x = TRUE;		/* For spherical filtering */
+			break;
 	}
 			
 	switch (filter_type) {
@@ -411,14 +431,14 @@ int main (int argc, char **argv)
 	par[GRDFILTER_WIDTH] = Ctrl->F.width;
 	par[GRDFILTER_HALF_WIDTH] = 0.5 * Ctrl->F.width;
 	par[GRDFILTER_X_SCALE] = x_scale;
-	par[GRDFILTER_Y_SCALE] = y_scale;
+	par[GRDFILTER_Y_SCALE] = (Ctrl->D.mode == 5) ? project_info.KM_PR_DEG : y_scale;
 	F.d_flag = Ctrl->D.mode;
 	F.dx = h.x_inc;
 	F.dy = h.y_inc;
 	x_width = Ctrl->F.width / (h.x_inc * x_scale);
 	y_width = Ctrl->F.width / (h.y_inc * y_scale);
-	F.y_half_width = (int) (ceil(y_width) / 2.0);
-	F.x_half_width = (int) (ceil(x_width) / 2.0);
+	F.y_half_width = (GMT_LONG) (ceil(y_width) / 2.0);
+	F.x_half_width = (GMT_LONG) (ceil(x_width) / 2.0);
 	F.nx = 2 * F.x_half_width + 1;
 	F.ny = 2 * F.y_half_width + 1;
 	if (x_scale == 0.0 || F.nx < 0 || F.nx > h.nx) {	/* Safety valve when x_scale -> 0.0 */
@@ -442,7 +462,7 @@ int main (int argc, char **argv)
 	}
 
 	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: Input nx,ny = (%d %d), output nx,ny = (%d %d), filter nx,ny = (%d %d)\n", GMT_program, h.nx, h.ny, nx_out, ny_out, F.nx, F.ny);
+		fprintf(stderr,"%s: Input nx,ny = (%d %d), output nx,ny = (%ld %ld), filter nx,ny = (%ld %ld)\n", GMT_program, h.nx, h.ny, nx_out, ny_out, F.nx, F.ny);
 		fprintf(stderr,"%s: Filter type is %s.\n", GMT_program, filter_name[filter_type]);
 	}
 
@@ -479,13 +499,14 @@ int main (int argc, char **argv)
 	
 	for (j_out = 0; j_out < ny_out; j_out++) {
 
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing output line %d\r", GMT_program, j_out);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing output line %ld\r", GMT_program, j_out);
 		y_out = north_new - j_out * Ctrl->I.yinc - yincnew2;
+		lat_out = (Ctrl->D.mode == 5) ? IMG2LAT (y_out) : y_out;
 		j_origin = GMT_y_to_j (y_out, h.y_min, h.y_inc, h.xy_off, h.ny);
-		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cos (D2R * y_out);	/* Update flat-earth longitude scale */
+		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cos (D2R * lat_out);	/* Update flat-earth longitude scale */
 
 		if (Ctrl->D.mode > 2) {	/* Update max filterweight nodes to deal with at this latitude */
-			y = fabs (y_out);
+			y = fabs (lat_out);
 			if (Ctrl->D.mode == 4) y += (par[GRDFILTER_HALF_WIDTH] / par[GRDFILTER_Y_SCALE]);
 			F.x_half_width = (y < 90.0) ? MIN (F.nx / 2, irint (par[GRDFILTER_HALF_WIDTH] / (F.dx * par[GRDFILTER_Y_SCALE] * cosd (y)))) : F.nx / 2;
 		}
@@ -613,7 +634,7 @@ int main (int argc, char **argv)
 	
 	/* At last, that's it!  Output: */
 
-	if (n_nan && gmtdefs.verbose) fprintf (stderr, "%s: Unable to estimate value at %d nodes, set to NaN\n", GMT_program, n_nan);
+	if (n_nan && gmtdefs.verbose) fprintf (stderr, "%s: Unable to estimate value at %ld nodes, set to NaN\n", GMT_program, n_nan);
 	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, GMT_n_multiples);
 
 	h.nx = nx_out;
@@ -652,25 +673,34 @@ void set_weight_matrix (struct FILTER_INFO *F, double *weight, double output_lat
 	 * par[4] is the normalization distance needed for the Cosine-bell or Gaussian weight function.
 	 */
 
-	int	i, j, ij;
-	double	x, y, yc, x_jump = 0.0, normalize = 1.0, r;
+	GMT_LONG	i, j, ij;
+	double	x, y, yc, y0, x_jump = 0.0, normalize = 1.0, r, dy = 1.0, dy_half = 0.5, dy_shrink = 1.0;
 
-	yc = output_lat - y_off;
+	yc = y0 = output_lat - y_off;
+	if (F->d_flag == 5) {
+		yc = IMG2LAT (yc);	/* Recover actual latitudes */
+		dy = F->y[1] - F->y[0];
+		dy_half = 0.5 * dy;
+	}
 	for (j = -F->y_half_width; j <= F->y_half_width; j++) {
-		y = yc + ((j < 0) ? -F->y[-j] : F->y[j]);
+		y = y0 + ((j < 0) ? -F->y[-j] : F->y[j]);
 		if (F->d_flag) {	/* Must normalize based on shrinking area representation and check for going over the pole */
+			if (F->d_flag == 5) {
+				dy_shrink = (IMG2LAT (y + dy_half) - IMG2LAT (y - dy_half)) / dy;		/* Recover actual latitudes */
+				y = IMG2LAT (y);	/* Recover actual latitudes */
+			}
 			if (fabs (y) > 90.0) {	/* Must find the point across the pole */
 				y = copysign (180.0 - fabs (y), y);
 				x_jump = 180.0;
 			}
 			else
 				x_jump = 0.0;
-			normalize = cosd (y);
+			normalize = dy_shrink * cosd (y);
 		}
 		for (i = -F->x_half_width; i <= F->x_half_width; i++) {
 			x = (i < 0) ? -F->x[-i] : F->x[i];
 			ij = (j + F->y_half_width) * F->nx + i + F->x_half_width;
-			r = F->radius_func (x_off, output_lat, x + x_jump, y, par);
+			r = F->radius_func (x_off, yc, x + x_jump, y, par);
 			weight[ij] = (r > par[GRDFILTER_HALF_WIDTH]) ? -1.0 : F->weight_func (r, par);
 			if (F->d_flag) weight[ij] *= normalize;	/* Adjust for variation in area with latitude */
 		}
@@ -729,7 +759,7 @@ double GaussianWeight (double r, double par[])
 void *New_Grdfilter_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDFILTER_CTRL *C;
 	
-	C = (struct GRDFILTER_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDFILTER_CTRL), "New_Grdfilter_Ctrl");
+	C = (struct GRDFILTER_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDFILTER_CTRL), "New_Grdfilter_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.mode = -1;	
@@ -737,6 +767,6 @@ void *New_Grdfilter_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Grdfilter_Ctrl (struct GRDFILTER_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
