@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdgradient.c,v 1.32 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdgradient.c,v 1.39 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *--------------------------------------------------------------------*/
 /*
  *  grdgradient.c
- * read a grdfile and compute gradient in azim direction:
+ * read a grid file and compute gradient in azim direction:
  *
  * azim = azimuth clockwise from north in degrees.
  *
@@ -91,12 +91,14 @@ int main (int argc, char **argv)
 	
 	BOOLEAN	error = FALSE, sigma_set = FALSE, offset_set = FALSE, bad;
 	
-	int	i, j, ij, k, n, nm, nm2, mx, my, p[4], n_used = 0, pos, entry;
+	int pos, p[4], entry, mx, my;
+	
+	GMT_LONG	i, j, ij, k, n, nm, nm2, n_used = 0;
 	
 	float *data, *slp = VNULL;
 
 	double	dx_grid, dy_grid, x_factor, y_factor, dzdx, dzdy, ave_gradient;
-	double	azim, denom, max_gradient = 0.0, min_gradient = 0.0, rpi, m_pr_degree = 0.0, lat;
+	double	azim, denom, max_gradient = 0.0, min_gradient = 0.0, rpi, lat;
 	double	x_factor2 = 0.0, y_factor2 = 0.0, dzdx2 = 0.0, dzdy2 = 0.0, dzds1, dzds2;
 	double	p0 = 0.0, q0 = 0.0, p0q0_cte = 1.0, norm_z, mag, s[3], lim_x, lim_y, lim_z;
 	double	k_ads = 0.0, diffuse, spec, r_min = DBL_MAX, r_max = -DBL_MAX, scale;
@@ -209,7 +211,7 @@ int main (int argc, char **argv)
 					break;
 				case 'L':
 					Ctrl->L.active = TRUE;
-					strncpy (Ctrl->L.mode, &argv[i][2], 4);
+					strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					break;
 				case 'M':
 					Ctrl->M.active = TRUE;
@@ -246,12 +248,12 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdgradient %s - Compute directional gradients from grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr,"grdgradient %s - Compute directional gradients from grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdgradient <infile> -G<outfile> [-A<azim>[/<azim2>]] [-D[a][o][n]]\n");
 		fprintf (stderr, "[-E[s|p]<azim>/<elev[ambient/diffuse/specular/shine]>]\n");
 		fprintf (stderr, "[-L<flag>] [-M] [-N[t_or_e][<amp>[/<sigma>[/<offset>]]]] [-S<slopefile>] [-V]\n\n");
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
-		fprintf (stderr,"\t<infile> is name of input grdfile\n");
+		fprintf (stderr,"\t<infile> is name of input grid file\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A sets azimuth (0-360 CW from North (+y)) for directional derivatives\n");
 		fprintf (stderr, "\t  -A<azim>/<azim2> will compute two directions and save the one larger in magnitude.\n");
@@ -277,7 +279,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   x for periodic boundary conditions on x\n");
 		fprintf (stderr, "\t   y for periodic boundary conditions on y\n");
 		fprintf (stderr, "\t   [Default:  Natural conditions]\n");
-		fprintf (stderr, "\t-M to use map units.  In this case, dx,dy of grdfile\n");
+		fprintf (stderr, "\t-M to use map units.  In this case, dx,dy of grid\n");
 		fprintf (stderr, "\t   will be converted from degrees lon,lat into meters (Flat-earth approximation).\n");
 		fprintf (stderr, "\t   Default computes gradient in units of data/grid_distance.\n");
 		fprintf (stderr, "\t-N will normalize gradients so that max |grad| = <amp> [1.0]\n");
@@ -373,9 +375,8 @@ int main (int argc, char **argv)
 	GMT_boundcond_set (&header, &edgeinfo, GMT_pad, data);
 
 	if (Ctrl->M.active) {
-		m_pr_degree = 2.0 * M_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
-		dx_grid = m_pr_degree * header.x_inc * cosd ((header.y_max + header.y_min) / 2.0);
-		dy_grid = m_pr_degree * header.y_inc;
+		dx_grid = project_info.M_PR_DEG * header.x_inc * cosd ((header.y_max + header.y_min) / 2.0);
+		dy_grid = project_info.M_PR_DEG * header.y_inc;
 	}
 	else {
 		dx_grid = header.x_inc;
@@ -410,7 +411,7 @@ int main (int argc, char **argv)
 	for (j = k = 0; j < header.ny; j++) {
 		if (Ctrl->M.active) {
 			lat = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, 0.5 * header.node_offset, header.ny);
-			dx_grid = m_pr_degree * header.x_inc * cos (D2R * lat);
+			dx_grid = project_info.M_PR_DEG * header.x_inc * cos (D2R * lat);
 			if (dx_grid > 0.0) x_factor = -1.0 / (2.0 * dx_grid);	/* Use previous value at the poles */
 			if (Ctrl->A.active) {
 				if (Ctrl->A.two) {
@@ -424,6 +425,7 @@ int main (int argc, char **argv)
 			for (n = 0, bad = FALSE; !bad && n < 4; n++) if (GMT_is_fnan (data[ij+p[n]])) bad = TRUE;
 			if (bad) {	/* One of corners = NaN, skip */
 				data[k] = GMT_f_NaN;
+				if (Ctrl->S.active) slp[k] = GMT_f_NaN;
 				continue;
 			}
 
@@ -607,7 +609,7 @@ double specular (double nx, double ny, double nz, double *s) {
 void *New_Grdgradient_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDGRADIENT_CTRL *C;
 	
-	C = (struct GRDGRADIENT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDGRADIENT_CTRL), "New_Grdgradient_Ctrl");
+	C = (struct GRDGRADIENT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDGRADIENT_CTRL), "New_Grdgradient_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.ambient = 0.55;
@@ -619,6 +621,7 @@ void *New_Grdgradient_Ctrl () {	/* Allocate and initialize a new control structu
 }
 
 void Free_Grdgradient_Ctrl (struct GRDGRADIENT_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
+	if (C->S.file) free ((void *)C->S.file);	
 	GMT_free ((void *)C);	
 }
