@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdhisteq.c,v 1.20 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdhisteq.c,v 1.26 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * read a grdfile and find the values which divide its range
+ * read a grid file and find the values which divide its range
  * into n_cell number of quantiles.
  *
  * Author:	W.H.F. Smith
@@ -66,7 +66,7 @@ struct GRDHISTEQ_CTRL {
 
 struct	INDEXED_DATA {
 	float	x;
-	int	i;
+	GMT_LONG	i;
 };
 
 struct	CELL {
@@ -80,7 +80,7 @@ int main (int argc, char **argv)
 	
 	char *infile = CNULL;
 
-	int i;
+	GMT_LONG i;
 		
 	struct GRDHISTEQ_CTRL *Ctrl;
 
@@ -134,13 +134,13 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdhisteq %s - Histogram equalization for grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr,"grdhisteq %s - Histogram equalization for grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdhisteq <infile> -G<outfile> [-C<n_cells> -D -N[<norm>] -Q -V]\n");
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		fprintf (stderr, "\t-C<n_cells> sets how many cells (divisions) of data range to make.\n");
 		fprintf (stderr, "\t-D dump level information to stdout\n");
-		fprintf (stderr, "\t-G<outfile> will create an equalized output grdfile.\n");
-		fprintf (stderr, "\t-N use with -G to make an output grdfile with standard normal scores.\n");
+		fprintf (stderr, "\t-G<outfile> will create an equalized output grid file.\n");
+		fprintf (stderr, "\t-N use with -G to make an output grid file with standard normal scores.\n");
 		fprintf (stderr, "\t   Append <norm> to normalize the scores to <-1,+1>\n");
 		fprintf (stderr, "\t-Q to use quadratic intensity scaling.  [Default is linear]\n");
 		GMT_explain_option ('V');
@@ -162,7 +162,7 @@ int main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	if (!strcmp (infile, "=")) {
-		fprintf (stderr, "%s: Piping of input grdfile not supported!\n", GMT_program);
+		fprintf (stderr, "%s: Piping of input grid file not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
@@ -181,21 +181,22 @@ int main (int argc, char **argv)
 void do_usual (char *infile, char *outfile, int n_cells, BOOLEAN quadratic, BOOLEAN dump_intervals, int argc, char **argv)
 {
 	char format[BUFSIZ];
-	int	i, j, last_cell, n_cells_m1 = 0, nxy, nxy_0, current_cell;
-	float	*data, data_min, data_max;
+	int	last_cell, n_cells_m1 = 0, current_cell;
+	GMT_LONG	i, j, nxy, nxy_0;
+	float	*data;
 	double	delta_cell, target;
 	struct GRD_HEADER header;
 	struct	CELL *cell;
 	
 	float get_cell (float x, struct CELL *cell, int n_cells_m1, int last_cell);
 
-	sprintf (format, "%s\t%s\t%%d\n", gmtdefs.d_format, gmtdefs.d_format);
+	sprintf (format, "%s\t%s\t%%ld\n", gmtdefs.d_format, gmtdefs.d_format);
 
 	GMT_err_fail (GMT_read_grd_info (infile, &header), infile);
 
 	GMT_grd_init (&header, argc, argv, TRUE);
 
-	nxy_0 = header.nx * header.ny;
+	nxy_0 = ((GMT_LONG)header.nx) * ((GMT_LONG)header.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nxy_0, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (infile, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), infile);
 
@@ -207,8 +208,6 @@ void do_usual (char *infile, char *outfile, int n_cells, BOOLEAN quadratic, BOOL
 	nxy = nxy_0;
 	while (nxy > 0 && GMT_is_fnan (data[nxy-1])) nxy--;	/* Only deal with real numbers */
 
-	data_min = data[0];
-	data_max = data[nxy - 1];
 	last_cell = n_cells/2;
 	n_cells_m1 = n_cells - 1;
 
@@ -223,11 +222,11 @@ void do_usual (char *infile, char *outfile, int n_cells, BOOLEAN quadratic, BOOL
 		else if (quadratic) {	/* Use y = 2x - x**2 scaling  */
 
 			target = ( (double) (current_cell + 1) ) / ( (double) n_cells);
-			j = (int)floor(nxy * (1.0 - sqrt(1.0 - target)));
+			j = (GMT_LONG)floor(nxy * (1.0 - sqrt(1.0 - target)));
 		}
 		else {	/* Use simple linear scale  */
 
-			j = (int)(floor( (current_cell + 1) * delta_cell)) - 1;
+			j = (GMT_LONG)(floor( (current_cell + 1) * delta_cell)) - 1;
 		}
 
 		cell[current_cell].low = data[i];
@@ -279,11 +278,12 @@ float get_cell (float x, struct CELL *cell, int n_cells_m1, int last_cell)
 			i = (low + high) / 2;
 		}
 	} while (TRUE);
+	return (0.0);	/* Cannot get here - just used to quiet compliler */
 }
 
 void do_gaussian (char *infile, char *outfile, double norm, int argc, char **argv)
 {
-	int	i, j, nxy, nxy_0;
+	GMT_LONG	i, j, nxy, nxy_0;
 	float	*data;
 	double	dnxy;
 	struct	GRD_HEADER header;
@@ -362,13 +362,13 @@ int compare_indices (const void *point_1, const void *point_2)
 void *New_Grdhisteq_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDHISTEQ_CTRL *C;
 	
-	C = (struct GRDHISTEQ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDHISTEQ_CTRL), "New_Grdhisteq_Ctrl");
+	C = (struct GRDHISTEQ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDHISTEQ_CTRL), "New_Grdhisteq_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
 }
 
 void Free_Grdhisteq_Ctrl (struct GRDHISTEQ_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
