@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdimage.c,v 1.66 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdimage.c,v 1.79 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,11 +16,11 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdimage will read a grdfile and image the area using the PostScript
+ * grdimage will read a grid file and image the area using the PostScript
  * image command. If non-linear scaling is chosen, we first resample the data
  * onto the new grid before calling the image command.  The output image
  * will be 1-, 8-, or 24-bit depending on colors used.  As an option, grdimage
- * can read three grdfiles with red, green, blue components in the 0-255
+ * can read three grid files with red, green, blue components in the 0-255
  * range and use those colors directly (no cpt required).
  *
  * Author:	Paul Wessel
@@ -53,19 +53,18 @@ struct GRDIMAGE_CTRL {
 	struct M {	/* -M */
 		BOOLEAN active;
 	} M;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
 	struct Q {	/* -Q */
 		BOOLEAN active;
 	} Q;
-	struct S {	/* -S<radius> */
+	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
 		BOOLEAN active;
-		double radius;
+		BOOLEAN antialias;
+		int interpolant;
+		double threshold;
 	} S;
-	struct T {	/* -T[s][o[<pen>] */
-		BOOLEAN active;
-		BOOLEAN skip;
-		BOOLEAN outline;
-		struct GMT_PEN pen;
-	} T;
 };
 
 int main (int argc, char **argv)
@@ -76,26 +75,28 @@ int main (int argc, char **argv)
 
 	char *grdfile[3], *c_method[2] = {"colorimage", "colortiles",};
 
-	int i, j, k, kk, rgb[3], nm, nm2 = 0, byte, nx_f, ny_f, grid_type = 0;
-	int nx, ny, nx_proj = 0, ny_proj = 0, node, n_grids = 0, index;
+	int i, j, k, rgb[3], byte, nx_f, ny_f, grid_type = 0;
+	int nx, ny, nx_proj = 0, ny_proj = 0, n_grids = 0, index;
 	int PS_colormask_off = 0, PS_colormask, PS_interpolate;
+	
+	size_t nm, nm2 = 0, node, kk;
 
 	float *tmp1[3], *tmp2 = NULL, *map[3], *intensity = NULL;
-	
+
 	double  dx, dy, x_side, y_side, x0 = 0.0, y0 = 0.0;
 	double west, east, south, north, data_west, data_east, data_south, data_north;
 
 	struct GRD_HEADER g_head[3], r_head[3], i_head, j_head;
+	struct GMT_EDGEINFO edgeinfo;
 	struct GRDIMAGE_CTRL *Ctrl;
 
-	void GMT_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 	void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 	void *New_Grdimage_Ctrl (), Free_Grdimage_Ctrl (struct GRDIMAGE_CTRL *C);
-	
+
 	argc = GMT_begin (argc, argv);
 
 	Ctrl = (struct GRDIMAGE_CTRL *)New_Grdimage_Ctrl ();	/* Allocate and initialize a new control structure */
-	
+
 	grdfile[0] = grdfile[1] = grdfile[2] = CNULL;
 	west = east = south = north = 0.0;
 
@@ -173,6 +174,9 @@ int main (int argc, char **argv)
 				case 'M':
 					Ctrl->M.active = TRUE;
 					break;
+				case 'N':
+					Ctrl->N.active = TRUE;
+					break;
 				case '0':	/* Backwards compatibility */
 					gmtdefs.color_image = 0;
 					break;
@@ -184,19 +188,36 @@ int main (int argc, char **argv)
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
-					Ctrl->S.radius = atof (&argv[i][2]);
+					for (j = 2; j < 5 && argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case '-':
+								Ctrl->S.antialias = FALSE; break;
+							case 'n':
+								Ctrl->S.interpolant = BCR_NEARNEIGHBOR; break;
+							case 'l':
+								Ctrl->S.interpolant = BCR_BILINEAR; break;
+							case 'b':
+								Ctrl->S.interpolant = BCR_BSPLINE; break;
+							case 'c':
+								Ctrl->S.interpolant = BCR_BICUBIC; break;
+							case '/':
+								Ctrl->S.threshold = atof (&argv[i][j+1]);
+								j = 5; break;
+							default:
+								fprintf (stderr, "%s: Warning: The -S option has changed meaning. Use -S[-]b|c|l|n[/threshold] to specify interpolation mode.\n", GMT_program);
+								j = 5; break;
+						}
+					}
 					break;
 				case 'T':
-					Ctrl->T.active = TRUE;
+					fprintf (stderr, "%s: Warning: The -T option has become obsolete. Use grdview instead.\n\t-T is similar to -Sn.\n\t-Ts is similar to -Sn -Q.\n\tProcessing continues with these options. Use -E if needed.\n", GMT_program);
+					Ctrl->S.active = TRUE;
+					Ctrl->S.interpolant = BCR_NEARNEIGHBOR;
 					k = 2;
-					if (argv[i][2] == 's') Ctrl->T.skip = TRUE, k = 3;
+					if (argv[i][2] == 's') Ctrl->Q.active = TRUE, k = 3;
 					if (argv[i][k] == 'o') {	/* Want tile outline also */
-						Ctrl->T.outline = TRUE;
-						k++;
-						if (argv[i][k] && GMT_getpen (&argv[i][k], &Ctrl->T.pen)) {
-							GMT_pen_syntax ('T', " ");
-							error++;
-						}
+						fprintf (stderr, "\tThe -To option is no longer supported. Use grdview instead.\n");
+						error++;
 					}
 					break;
 
@@ -216,9 +237,9 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdimage %s - Plot grdfiles in 2-D\n\n", GMT_VERSION);
+		fprintf (stderr,"grdimage %s - Plot grid files in 2-D\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdimage <grd_z|grd_r grd_g grd_b> %s [%s] [-C<cpt_file>] [-Ei|<dpi>] [-G[f|b]<rgb>]\n", GMT_J_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-O] [-P] [-Q] [%s] [-S<radius>] [-T[s][o[<pen>]]]\n", GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-N] [-O] [-P] [-Q] [%s] [-S[-]b|c|l|n[/<threshold>]] [-T]\n", GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[%s] [-V] [%s] [%s] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -229,20 +250,26 @@ int main (int argc, char **argv)
 		GMT_explain_option ('j');
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
-		fprintf (stderr,"\t-C color palette file to convert z to rgb\n");
-		fprintf(stderr, "\t-E sets dpi for the projected grid which must be constructed\n");
-		fprintf(stderr, "\t   if -Jx or -Jm is not selected [Default gives same size as input grid]\n");
-		fprintf(stderr, "\t   Give i to do the interpolation in PostScript at device resolution.\n");
-		GMT_rgb_syntax ('G', "sets transparency color for images that otherwise would result in 1-bit images\n");
-		fprintf (stderr,"\t-I use illumination.  Append name of intensity grd file\n");
+		fprintf (stderr, "\t-C color palette file to convert z to rgb\n");
+		fprintf (stderr, "\t-E sets dpi for the projected grid which must be constructed\n");
+		fprintf (stderr, "\t   if -Jx or -Jm is not selected [Default gives same size as input grid]\n");
+		fprintf (stderr, "\t   Give i to do the interpolation in PostScript at device resolution.\n");
+		GMT_rgb_syntax ('G', "sets transparency color for images that otherwise would result in 1-bit images\n\t  ");
+		fprintf (stderr, "\t-I use illumination.  Append name of intensity grid file\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-M force monochrome image\n");
+		fprintf (stderr, "\t-N Do not clip image at the map boundary\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf(stderr, "\t-Q use PS Level 3 colormasking for nodes with z = NaN.\n");
+		fprintf (stderr, "\t-Q use PS Level 3 colormasking for nodes with z = NaN.\n");
 		GMT_explain_option ('R');
-		fprintf(stderr, "\t-S sets the search radius in projected units [Default avoids aliasing]\n");
-		GMT_pen_syntax ('T', "will image the data without interpolation by painting polygonal tiles\n\t   Append s to skip tiles for nodes with z = NaN [Default paints all tiles]\n\t   Append o[<pen>] to draw tile outline [Default uses no outline]");
+		fprintf (stderr, "\t-S Determines the interpolation mode (b = B-spline, c = bicubic, l = bilinear,\n");
+		fprintf (stderr, "\t   n = nearest-neighbor) [Default: bicubic]\n");
+		fprintf (stderr, "\t   Optionally, prepend - to switch off antialiasing [Default: on]\n");
+		fprintf (stderr, "\t   Append /<threshold> to change the minimum weight in vicinity of NaNs. A threshold of\n");
+		fprintf (stderr, "\t   1.0 requires all nodes involved in interpolation to be non-NaN; 0.5 will interpolate\n");
+		fprintf (stderr, "\t   about half way from a non-NaN to a NaN node [Default: 0.5]\n");
+		fprintf (stderr, "\t-T OBSOLETE: See man pages\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
@@ -276,6 +303,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Only one of fore/back-ground can be transparent for 1-bit images\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->S.active && (Ctrl->S.threshold < 0.0 || Ctrl->S.threshold > 1.0)) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  threshold must be in [0,1] range\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
 	/* Get color palette file */
@@ -284,7 +315,6 @@ int main (int argc, char **argv)
 	PS_interpolate = (Ctrl->E.device_dpi) ? -1 : +1;
 	PS_colormask   = (Ctrl->Q.active) ? -1 : +1;;
 
-
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocates memory and read data file\n", GMT_program);
 
 	for (i = 0; i < n_grids; i++) {
@@ -314,16 +344,17 @@ int main (int argc, char **argv)
 		south = g_head[0].y_min;
 		north = g_head[0].y_max;
 	}
-	
+
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	/* Determine the wesn to be used to read the grdfile; or exit if file is outside -R */
+	/* Determine the wesn to be used to read the grid file; or exit if file is outside -R */
 
 	if (GMT_grd_setregion (&g_head[0], &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	nx_f = g_head[0].nx;
@@ -333,14 +364,19 @@ int main (int argc, char **argv)
 
 	nx = GMT_get_n (data_west, data_east, g_head[0].x_inc, g_head[0].node_offset);
 	ny = GMT_get_n (data_south, data_north, g_head[0].y_inc, g_head[0].node_offset);
-	nm = nx * ny;
+
+	GMT_boundcond_init (&edgeinfo);
+
+	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW);
+	if (Ctrl->E.dpi > 0 || need_to_project) {
+		nm = ((size_t)(4 + nx)) * ((size_t)(4 + ny));
+		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
+	}
+	else
+		nm = ((size_t)nx) * ((size_t)ny);
 
 	for (i = 0; i < n_grids; i++) {
-#ifdef _GENPER
-		tmp1[i] = (float *) GMT_memory (VNULL, (size_t)(nm + nx), sizeof (float), GMT_program);
-#else
-		tmp1[i] = (float *) GMT_memory (VNULL, (size_t)(nm), sizeof (float), GMT_program);
-#endif
+		tmp1[i] = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 		GMT_err_fail (GMT_read_grd (grdfile[i], &g_head[i], tmp1[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile[i]);
 	}
 
@@ -365,11 +401,10 @@ int main (int argc, char **argv)
 	}
 
 	for (i = 0; i < n_grids; i++) GMT_set_proj_limits (&r_head[i], &g_head[i]);
-	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW);
 
-	if (!Ctrl->T.active && (Ctrl->E.dpi > 0 || need_to_project)) {	/* Need to resample the grd file */
+	if (Ctrl->E.dpi > 0 || need_to_project) {	/* Need to resample the grd file */
 
-		if (gmtdefs.verbose) fprintf (stderr, "%s: project grdfiles\n", GMT_program);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: project grid files\n", GMT_program);
 
 		if (Ctrl->E.dpi == 0) {	/* Use input # of nodes as # of projected nodes */
 			nx_proj = g_head[0].nx;
@@ -378,10 +413,9 @@ int main (int argc, char **argv)
 		for (i = 0; i < n_grids; i++) {
 			grid_type = (Ctrl->E.dpi > 0) ? 1 : g_head[i].node_offset;	/* Force pixel if dpi is set */
 			GMT_err_fail (GMT_grdproject_init (&r_head[i], 0.0, 0.0, nx_proj, ny_proj, Ctrl->E.dpi, grid_type), grdfile[i]);
-			GMT_init_search_radius (&Ctrl->S.radius, &r_head[i], &g_head[i], FALSE);
-			nm2 = r_head[i].nx * r_head[i].ny;
-			map[i] = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdimage");
-			GMT_grd_forward (tmp1[i], &g_head[i], map[i], &r_head[i], Ctrl->S.radius);
+			nm2 = ((size_t)r_head[i].nx) * ((size_t)r_head[i].ny);
+			map[i] = (float *) GMT_memory (VNULL, nm2, sizeof (float), "grdimage");
+			GMT_grd_project (tmp1[i], &g_head[i], map[i], &r_head[i], &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 			GMT_free ((void *)tmp1[i]);
 		}
 		if (Ctrl->I.active) {
@@ -393,8 +427,8 @@ int main (int argc, char **argv)
 				ny_proj = i_head.ny;
 			}
 			GMT_err_fail (GMT_grdproject_init (&j_head, 0.0, 0.0, nx_proj, ny_proj, Ctrl->E.dpi, grid_type), Ctrl->I.file);
-			intensity = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdimage");
-			GMT_grd_forward (tmp2, &i_head, intensity, &j_head, Ctrl->S.radius);
+			intensity = (float *) GMT_memory (VNULL, nm2, sizeof (float), "grdimage");
+			GMT_grd_project (tmp2, &i_head, intensity, &j_head, &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 			GMT_free ((void *)tmp2);
 		}
 		nm = nm2;
@@ -416,126 +450,7 @@ int main (int argc, char **argv)
 
 	GMT_plotinit (argc, argv);
 
-	GMT_map_clip_on (GMT_no_rgb, 3);
-
-	if (Ctrl->T.active) {	/* Plot image as polygonal pieces and then exit */
-		int np, index = 0, plot_n, n_use, start;
-		double *grd_x, grd_y, *xx, *yy, *xp, *yp, dx2, dy2, y_bot, y_top;
-		struct GMT_FILL *f;
-		BOOLEAN skip;
-
-		dx2 = 0.5 * g_head[0].x_inc;
-		dy2 = 0.5 * g_head[0].y_inc;
-
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling without interpolation\n", GMT_program);
-		grd_x = (double *) GMT_memory (VNULL, (size_t)g_head[0].nx, sizeof (double), GMT_program);
-
-		if (Ctrl->T.outline) GMT_setpen (&Ctrl->T.pen);
-		for (i = 0; i < g_head[0].nx; i++) {
-			grd_x[i] = GMT_i_to_x (i, g_head[0].x_min, g_head[0].x_max, g_head[0].x_inc, g_head[0].xy_off, g_head[0].nx);
-			if (GMT_io.in_col_type[0] == GMT_IS_LON) {
-				if (grd_x[i] < project_info.w && (grd_x[i] + 360.0) <= project_info.e) grd_x[i] += 360.0;
-				if (grd_x[i] > project_info.e && (grd_x[i] - 360.0) >= project_info.w) grd_x[i] -= 360.0;
-			}
-		}
-		for (j = node = 0; j < g_head[0].ny; j++) {
-			grd_y = GMT_j_to_y (j, g_head[0].y_min, g_head[0].y_max, g_head[0].y_inc, g_head[0].xy_off, g_head[0].ny);
-			y_bot = grd_y - dy2;
-			y_top = grd_y + dy2;
-			if (GMT_io.in_col_type[1] == GMT_IS_LAT) {
-				if (y_bot < -90.0) y_bot = -90.0;
-				if (y_top > +90.0) y_top = +90.0;
-			}
-			for (i = 0; i < g_head[0].nx; i++, node++) {	/* Compute rgb for each pixel */
-				if (do_rgb) {
-					for (k = 0, skip = FALSE; k < 3 && !skip; k++) {
-						if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
-							if (Ctrl->T.skip)	/* Here we want to skip this node entirely */
-								skip = TRUE;
-							else {			/* Here we simply set the color to the NaN color */
-								k = 3;
-								memcpy ((void *)rgb, (void *)GMT_bfn[GMT_NAN].rgb, 3 * sizeof (int));
-								index = -1;	/* Ensures no illumination done later */
-							}
-						}
-						else {				/* Set color, let index = 0 so illuminate test will work */
-							rgb[k] = irint (map[k][node]);	if (rgb[k] < 0) rgb[k] = 0; else if (rgb[k] > 255) rgb[k] = 255;	/* Clip */
-							index = 0;
-						}
-					}
-					if (skip) continue;
-				}
-				else {
-					if (GMT_is_fnan (map[0][node]) && Ctrl->T.skip) continue;
-					index = GMT_get_rgb_from_z (map[0][node], rgb);
-					if (GMT_cpt_skip) continue;	/* Skip this z-slice */
-				}
-				if (Ctrl->I.active && index != -1) GMT_illuminate (intensity[node], rgb);
-				if (Ctrl->M.active) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-
-				np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - dx2, grd_x[i] + dx2, y_bot, y_top);
-
-				plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
-				GMT_free ((void *)xx);
-				GMT_free ((void *)yy);
-				if (plot_n == 0) continue;	/* Outside */
-
-				if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
-
-					/* First truncate agains left border */
-
-					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
-					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
-					if (do_rgb)
-						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, Ctrl->T.outline);
-					else if (index >= 0 && (f = GMT_lut[index].fill))
-						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, Ctrl->T.outline);
-					else if (index < 0 && (f = GMT_bfn[index+3].fill))
-						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, Ctrl->T.outline);
-					else
-						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, Ctrl->T.outline);
-
-					/* Then truncate agains right border */
-
-					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
-					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
-					if (do_rgb)
-						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, Ctrl->T.outline);
-					else if (index >= 0 && (f = GMT_lut[index].fill))
-						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, Ctrl->T.outline);
-					else if (index < 0 && (f = GMT_bfn[index+3].fill))
-						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, Ctrl->T.outline);
-					else
-						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, Ctrl->T.outline);
-
-				}
-				else {
-					if (do_rgb)
-						ps_polygon (xp, yp, plot_n, rgb, Ctrl->T.outline);
-					else if (index >= 0 && (f = GMT_lut[index].fill))
-						GMT_fill (xp, yp, plot_n, f, Ctrl->T.outline);
-					else if (index < 0 && (f = GMT_bfn[index+3].fill))
-						GMT_fill (xp, yp, plot_n, f, Ctrl->T.outline);
-					else
-						ps_polygon (xp, yp, plot_n, rgb, Ctrl->T.outline);
-				}
-				GMT_free ((void *)xp);
-				GMT_free ((void *)yp);
-			}
-		}
-
-		GMT_free ((void *)grd_x);
-		for (i = 0; i < n_grids; i++) GMT_free ((void *)map[i]);
-		if (Ctrl->I.active) GMT_free ((void *)intensity);
-
-		GMT_map_clip_off();
-
-		GMT_map_basemap ();
-
-		GMT_plotend ();
-
-		GMT_end (argc, argv);
-	}
+	if (!Ctrl->N.active) GMT_map_clip_on (GMT_no_rgb, 3);
 
 	if (gmtdefs.verbose) {
 		if (GMT_cpt_pattern) fprintf (stderr, "%s: Warning: Patterns in cpt file only apply to -T\n", GMT_program);
@@ -556,7 +471,7 @@ int main (int argc, char **argv)
 	normal_x = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[0] && !resampled);
 	normal_y = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[1] && !resampled);
 	for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
-		kk = r_head[0].nx * (normal_y ? j : r_head[0].ny - j - 1);
+		kk = ((size_t)r_head[0].nx) * ((size_t)(normal_y ? j : r_head[0].ny - j - 1));
 		for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
 			node = kk + (normal_x ? i : r_head[0].nx - i - 1);
 			if (do_rgb) {
@@ -609,10 +524,11 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Creating PostScript image ", GMT_program);
 
-	if (GMT_gray) for (k = 0, GMT_b_and_w = TRUE; GMT_b_and_w && k < nm; k++) if (!(bitimage_8[k] == 0 || bitimage_8[k] == 255)) GMT_b_and_w = FALSE;
+	if (GMT_gray) for (kk = 0, GMT_b_and_w = TRUE; GMT_b_and_w && kk < nm; kk++) if (!(bitimage_8[kk] == 0 || bitimage_8[kk] == 255)) GMT_b_and_w = FALSE;
 
 	if (GMT_b_and_w) {	/* Can get away with 1 bit image */
-		int nx8, shift, byte, b_or_w, k8, nx_pixels;
+		int nx8, shift, byte, b_or_w, nx_pixels;
+		size_t k8;
 		unsigned char *bit;
 
 		if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
@@ -662,7 +578,7 @@ int main (int argc, char **argv)
 		GMT_free ((void *)bitimage_24);
 	}
 
-	GMT_map_clip_off();
+	if (!Ctrl->N.active) GMT_map_clip_off();
 
 	GMT_map_basemap ();
 
@@ -671,6 +587,7 @@ int main (int argc, char **argv)
 	Free_Grdimage_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
+	exit (EXIT_SUCCESS);
 
 	return(0);
 }
@@ -690,17 +607,15 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 	r->ny = g->ny;
 	r->node_offset = g->node_offset;
 
-#ifdef _GENPER
-	if( project_info.projection == GMT_GENPER && project_info.g_width ) {
+	if (project_info.projection == GMT_GENPER && project_info.g_width != 0.0) {
 		r->x_min = project_info.xmin;	r->x_max = project_info.xmax;
 		r->y_min = project_info.ymin;	r->y_max = project_info.ymax;
 		return;
 	}
-#endif   /* END of _GENPER */
 
 	if (GMT_IS_AZIMUTHAL) {	/* Applies only to Azimuthal projections */
-		all_lats = (fabs (g->y_max - 90.0) < GMT_CONV_LIMIT && fabs (g->y_min + 90.0) < GMT_CONV_LIMIT);
-		all_lons = (fabs (g->x_max - g->x_min - 360.0) < GMT_CONV_LIMIT);
+		all_lats = GMT_180_RANGE (g->y_max, g->y_min);
+		all_lons = GMT_360_RANGE (g->x_max, g->x_min);
 
 		if (all_lons && all_lats) {	/* Whole globe, get square box */
 			r->x_min = project_info.xmin;	r->x_max = project_info.xmax;
@@ -742,20 +657,20 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 
 void *New_Grdimage_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDIMAGE_CTRL *C;
-	
-	C = (struct GRDIMAGE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDIMAGE_CTRL), "New_Grdimage_Ctrl");
-	
+
+	C = (struct GRDIMAGE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDIMAGE_CTRL), "New_Grdimage_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
-	
-	GMT_init_pen (&C->T.pen, GMT_PENWIDTH);
+
 	C->G.f_rgb[0] = C->G.f_rgb[1] = C->G.f_rgb[2] = 0;
 	C->G.b_rgb[0] = C->G.b_rgb[1] = C->G.b_rgb[2] = 255;
-	
+	C->S.antialias = TRUE; C->S.interpolant = BCR_BICUBIC; C->S.threshold = 0.5;
+
 	return ((void *)C);
 }
 
 void Free_Grdimage_Ctrl (struct GRDIMAGE_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	if (C->I.file) GMT_free ((void *)C->I.file);	
-	GMT_free ((void *)C);	
+	if (C->C.file) free ((void *)C->C.file);
+	if (C->I.file) free ((void *)C->I.file);
+	GMT_free ((void *)C);
 }
