@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdinfo.c,v 1.45 2007/03/29 19:32:30 pwessel Exp $
+ *	$Id: grdinfo.c,v 1.55 2008/04/04 21:06:19 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdinfo reads one or more grd file and [optionally] prints out various
+ * grdinfo reads one or more grid file and [optionally] prints out various
  * statistics like mean/standard deviation and median/scale
  *
  * Author:	Paul Wessel
@@ -58,8 +58,8 @@ int main (int argc, char **argv)
 
 	int nfiles = 0, k, i, n_grds = 0, col, row;
 
-	size_t nm = 0, n_nan = 0, n, ij, i_min, i_max;
-	
+	size_t nm = 0, n_nan = 0, n = 0, ij, ij_min, ij_max;
+
 	float *a;
 
 	double x_min = DBL_MAX, y_min = DBL_MAX, z_min = DBL_MAX, x_max = -DBL_MAX, y_max = -DBL_MAX, z_max = -DBL_MAX;
@@ -67,17 +67,17 @@ int main (int argc, char **argv)
 	double mean = 0.0, median = 0.0, sum2 = 0.0, stdev = 0.0, scale = 0.0, rms = 0.0, x;
 
 	char format[BUFSIZ], text[GMT_TEXT_LEN];
-	char *type[2] = { "Normal", "Pixel"};
+	char *type[2] = { "Gridline", "Pixel"};
 
 	struct GRD_HEADER grd;
 	struct GRDINFO_CTRL *Ctrl;
 
 	void *New_Grdinfo_Ctrl (), Free_Grdinfo_Ctrl (struct GRDINFO_CTRL *C);
-	
+
 	argc = GMT_begin (argc, argv);
 
 	Ctrl = (struct GRDINFO_CTRL *)New_Grdinfo_Ctrl ();	/* Allocate and initialize a new control structure */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -144,13 +144,13 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdinfo %s - Extract information from netCDF grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr, "grdinfo %s - Extract information from netCDF grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdinfo <grdfiles> [-C] [-F] [-I[<dx>[/<dy>]]] [-L[1|2]] [-M]\n");
 		fprintf (stderr, "	[-T<dz>] [%s]\n", GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<grdfiles> may be one or more netCDF grdfiles\n");
+		fprintf (stderr, "\t<grdfiles> may be one or more netCDF grid files\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C formats report in fields on a single line using the format\n");
 		fprintf (stderr, "\t   file w e s n z0 z1 dx dy nx ny [x0 y0 x1 y1] [med scale] [mean std rms] [n_nan]\n");
@@ -207,66 +207,59 @@ int main (int argc, char **argv)
 
 		if (GMT_err_pass (GMT_read_grd_info (argv[k], &grd), argv[k])) continue;
 
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing file %s\n", GMT_program, argv[k]);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing file %s\n", GMT_program, grd.name);
 
 		n_grds++;
 
-		if (grd.z_min == grd.z_max) slow = TRUE, Ctrl->M.active = TRUE;
-
 		if (slow) {	/* Must determine the location of global min and max values */
 
 			nm = ((size_t)grd.nx) * ((size_t)grd.ny);
 			a = (float *) GMT_memory ((void *)a, nm, sizeof (float), GMT_program);
-			if (GMT_err_pass (GMT_read_grd (argv[k], &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), argv[k])) continue;
+			if (GMT_err_pass (GMT_read_grd (argv[k], &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grd.name)) continue;
 
 			z_min = DBL_MAX;	z_max = -DBL_MAX;
 			mean = median = sum2 = 0.0;
-			i_min = i_max = 0;
-			n_nan = 0;
+			ij_min = ij_max = n = 0;
 			for (ij = 0; ij < nm; ij++) {
-				if (GMT_is_fnan (a[ij])) {
-					n_nan++;
-					continue;
+				if (GMT_is_fnan (a[ij])) continue;
+				if (a[ij] < z_min) {
+					z_min = a[ij];
+					ij_min = ij;
 				}
-				if (Ctrl->M.active) {
-					if (a[ij] < z_min) {
-						z_min = a[ij];
-						i_min = ij;
-					}
-					if (a[ij] > z_max) {
-						z_max = a[ij];
-						i_max = ij;
-					}
-				}
-				if (slow) {
-					mean += a[ij];
-					sum2 += a[ij]*a[ij];
+				if (a[ij] > z_max) {
+					z_max = a[ij];
+					ij_max = ij;
 				}
+				/* Use Welford (1962) algorithm to compute mean and corrected sum of squares */
+				n++;
+				x = a[ij] - mean;
+				mean += x / n;
+				sum2 += x * (a[ij] - mean);
 			}
 
-			col = i_min % grd.nx;
-			row = i_min / grd.nx;
+			n_nan = nm - n;
+			col = ij_min % grd.nx;
+			row = ij_min / grd.nx;
 			x_min = GMT_i_to_x (col, grd.x_min, grd.x_max, grd.x_inc, grd.xy_off, grd.nx);
 			y_min = GMT_j_to_y (row, grd.y_min, grd.y_max, grd.y_inc, grd.xy_off, grd.ny);
-			col = i_max % grd.nx;
-			row = i_max / grd.nx;
+			col = ij_max % grd.nx;
+			row = ij_max / grd.nx;
 			x_max = GMT_i_to_x (col, grd.x_min, grd.x_max, grd.x_inc, grd.xy_off, grd.nx);
 			y_max = GMT_j_to_y (row, grd.y_min, grd.y_max, grd.y_inc, grd.xy_off, grd.ny);
 		}
 
 		if (Ctrl->L.norm & 1) {	/* Calculate the median and L1 scale */
 			qsort ((void *)a, nm, sizeof (float), GMT_comp_float_asc);
-			n = nm - n_nan;
 			median = (n%2) ? a[n/2] : 0.5*(a[n/2-1] + a[n/2]);
 			for (ij = 0; ij < n; ij++) a[ij] = (float)fabs (a[ij] - median);
 			qsort ((void *)a, n, sizeof (float), GMT_comp_float_asc);
 			scale = (n%2) ? 1.4826 * a[n/2] : 0.7413 * (a[n/2-1] + a[n/2]);
 		}
 		if (Ctrl->L.norm & 2) {	/* Calculate the mean, standard deviation, and rms */
-			x = (double)(nm - n_nan);
-			stdev = (rint (x) > 1.0) ? sqrt((x*sum2 - mean*mean)/(x*(x-1))) : GMT_d_NaN;
-			rms = (rint (x) > 0.0) ? sqrt (sum2 / x) : GMT_d_NaN;
-			mean = (rint (x) > 0.0) ? mean / x : GMT_d_NaN;
+			x = (double)n;
+			stdev = (n > 1) ? sqrt(sum2 / (x-1)) : GMT_d_NaN;
+			rms = (n > 0) ? sqrt (sum2 / x + mean * mean) : GMT_d_NaN;
+			mean = (n > 0) ? mean : GMT_d_NaN;
 		}
 
 		/* OK, time to report results */
@@ -281,7 +274,7 @@ int main (int argc, char **argv)
 			GMT_ascii_output_one (GMT_stdout, grd.y_inc, 2);
 			fprintf (GMT_stdout, "\n");
 		} else if (Ctrl->C.active && !Ctrl->I.active) {
-			fprintf (GMT_stdout, "%s\t", argv[k]);
+			fprintf (GMT_stdout, "%s\t", grd.name);
 			GMT_ascii_output_one (GMT_stdout, grd.x_min, 0);	fprintf (GMT_stdout, "\t");	GMT_ascii_output_one (GMT_stdout, grd.x_max, 0);	fprintf (GMT_stdout, "\t");
 			GMT_ascii_output_one (GMT_stdout, grd.y_min, 1);	fprintf (GMT_stdout, "\t");	GMT_ascii_output_one (GMT_stdout, grd.y_max, 1);	fprintf (GMT_stdout, "\t");
 			GMT_ascii_output_one (GMT_stdout, grd.z_min, 2);	fprintf (GMT_stdout, "\t");	GMT_ascii_output_one (GMT_stdout, grd.z_max, 2);	fprintf (GMT_stdout, "\t");
@@ -315,47 +308,47 @@ int main (int argc, char **argv)
 			fprintf (GMT_stdout, "\n");
 		}
 		else if (!(Ctrl->T.active || (Ctrl->I.active && Ctrl->I.status == 2))) {
-			fprintf (GMT_stdout, "%s: Title: %s\n", argv[k], grd.title);
-			fprintf (GMT_stdout, "%s: Command: %s\n", argv[k], grd.command);
-			fprintf (GMT_stdout, "%s: Remark: %s\n", argv[k], grd.remark);
-			fprintf (GMT_stdout, "%s: %s node registration used\n", argv[k], type[grd.node_offset]);
-			fprintf (GMT_stdout, "%s: grdfile format: %c%c (# %d)\n", argv[k], GMT_grdformats[grd.type][0], GMT_grdformats[grd.type][1], grd.type);
+			fprintf (GMT_stdout, "%s: Title: %s\n", grd.name, grd.title);
+			fprintf (GMT_stdout, "%s: Command: %s\n", grd.name, grd.command);
+			fprintf (GMT_stdout, "%s: Remark: %s\n", grd.name, grd.remark);
+			fprintf (GMT_stdout, "%s: %s node registration used\n", grd.name, type[grd.node_offset]);
+			fprintf (GMT_stdout, "%s: Grid file format: %c%c (# %d)\n", grd.name, (int)GMT_grdformats[grd.type][0], (int)GMT_grdformats[grd.type][1], grd.type);
 			if (Ctrl->F.active) {
 				if ((fabs (grd.x_min) < 500.0) && (fabs (grd.x_max) < 500.0) && (fabs (grd.y_min) < 500.0) && (fabs (grd.y_max) < 500.0)) {
-					fprintf (GMT_stdout, "%s: x_min: %.7f\n", argv[k], grd.x_min);
-					fprintf (GMT_stdout, "%s: x_max: %.7f\n", argv[k], grd.x_max);
-					fprintf (GMT_stdout, "%s: x_inc: %.7f\n", argv[k], grd.x_inc);
-					fprintf (GMT_stdout, "%s: name: %s\n", argv[k], grd.x_units);
-					fprintf (GMT_stdout, "%s: nx: %d\n", argv[k], grd.nx);
-					fprintf (GMT_stdout, "%s: y_min: %.7f\n", argv[k], grd.y_min);
-					fprintf (GMT_stdout, "%s: y_max: %.7f\n", argv[k], grd.y_max);
-					fprintf (GMT_stdout, "%s: y_inc: %.7f\n", argv[k], grd.y_inc);
-					fprintf (GMT_stdout, "%s: name: %s\n", argv[k], grd.y_units);
-					fprintf (GMT_stdout, "%s: ny: %d\n", argv[k], grd.ny);
+					fprintf (GMT_stdout, "%s: x_min: %.7f\n", grd.name, grd.x_min);
+					fprintf (GMT_stdout, "%s: x_max: %.7f\n", grd.name, grd.x_max);
+					fprintf (GMT_stdout, "%s: x_inc: %.7f\n", grd.name, grd.x_inc);
+					fprintf (GMT_stdout, "%s: name: %s\n", grd.name, grd.x_units);
+					fprintf (GMT_stdout, "%s: nx: %d\n", grd.name, grd.nx);
+					fprintf (GMT_stdout, "%s: y_min: %.7f\n", grd.name, grd.y_min);
+					fprintf (GMT_stdout, "%s: y_max: %.7f\n", grd.name, grd.y_max);
+					fprintf (GMT_stdout, "%s: y_inc: %.7f\n", grd.name, grd.y_inc);
+					fprintf (GMT_stdout, "%s: name: %s\n", grd.name, grd.y_units);
+					fprintf (GMT_stdout, "%s: ny: %d\n", grd.name, grd.ny);
 				}
 				else {
-					fprintf (GMT_stdout, "%s: x_min: %.2f\n", argv[k], grd.x_min);
-					fprintf (GMT_stdout, "%s: x_max: %.2f\n", argv[k], grd.x_max);
-					fprintf (GMT_stdout, "%s: x_inc: %.2f\n", argv[k], grd.x_inc);
-					fprintf (GMT_stdout, "%s: name: %s\n", argv[k], grd.x_units);
-					fprintf (GMT_stdout, "%s: nx: %d\n", argv[k], grd.nx);
-					fprintf (GMT_stdout, "%s: y_min: %.2f\n", argv[k], grd.y_min);
-					fprintf (GMT_stdout, "%s: y_max: %.2f\n", argv[k], grd.y_max);
-					fprintf (GMT_stdout, "%s: y_inc: %.2f\n", argv[k], grd.y_inc);
-					fprintf (GMT_stdout, "%s: name: %s\n", argv[k], grd.y_units);
-					fprintf (GMT_stdout, "%s: ny: %d\n", argv[k], grd.ny);
+					fprintf (GMT_stdout, "%s: x_min: %.2f\n", grd.name, grd.x_min);
+					fprintf (GMT_stdout, "%s: x_max: %.2f\n", grd.name, grd.x_max);
+					fprintf (GMT_stdout, "%s: x_inc: %.2f\n", grd.name, grd.x_inc);
+					fprintf (GMT_stdout, "%s: name: %s\n", grd.name, grd.x_units);
+					fprintf (GMT_stdout, "%s: nx: %d\n", grd.name, grd.nx);
+					fprintf (GMT_stdout, "%s: y_min: %.2f\n", grd.name, grd.y_min);
+					fprintf (GMT_stdout, "%s: y_max: %.2f\n", grd.name, grd.y_max);
+					fprintf (GMT_stdout, "%s: y_inc: %.2f\n", grd.name, grd.y_inc);
+					fprintf (GMT_stdout, "%s: name: %s\n", grd.name, grd.y_units);
+					fprintf (GMT_stdout, "%s: ny: %d\n", grd.name, grd.ny);
 				}
 			}
 			else {
 				GMT_ascii_format_one (text, grd.x_inc, GMT_io.out_col_type[0]);
 				if (isalpha ((int)text[strlen(text)-1])) text[strlen(text)-1] = '\0';	/* Chop of trailing WESN flag here */
-				fprintf (GMT_stdout, "%s: x_min: ", argv[k]);
+				fprintf (GMT_stdout, "%s: x_min: ", grd.name);
 				GMT_ascii_output_one (GMT_stdout, grd.x_min, 0);
 				fprintf (GMT_stdout, " x_max: ");
 				GMT_ascii_output_one (GMT_stdout, grd.x_max, 0);
 				fprintf (GMT_stdout, " x_inc: %s", text);
 				fprintf (GMT_stdout, " name: %s nx: %d\n", grd.x_units, grd.nx);
-				fprintf (GMT_stdout, "%s: y_min: ", argv[k]);
+				fprintf (GMT_stdout, "%s: y_min: ", grd.name);
 				GMT_ascii_output_one (GMT_stdout, grd.y_min, 1);
 				fprintf (GMT_stdout, " y_max: ");
 				GMT_ascii_output_one (GMT_stdout, grd.y_max, 1);
@@ -368,7 +361,7 @@ int main (int argc, char **argv)
 			if (Ctrl->M.active) {
 				if (z_min == -DBL_MAX) z_min = GMT_d_NaN;
 				if (z_max == +DBL_MAX) z_max = GMT_d_NaN;
-				fprintf (GMT_stdout, "%s: z_min: ", argv[k]);
+				fprintf (GMT_stdout, "%s: z_min: ", grd.name);
 				GMT_ascii_output_one (GMT_stdout, z_min, 2);
 				fprintf (GMT_stdout, " at x = ");
 				GMT_ascii_output_one (GMT_stdout, x_min, 0);
@@ -383,12 +376,12 @@ int main (int argc, char **argv)
 				fprintf (GMT_stdout, "\n");
 			}
 			else if (Ctrl->F.active) {
-				fprintf (GMT_stdout, "%s: zmin: %g\n", argv[k], grd.z_min);
-				fprintf (GMT_stdout, "%s: zmax: %g\n", argv[k], grd.z_max);
-			 	fprintf (GMT_stdout, "%s: name: %s\n", argv[k], grd.z_units);
+				fprintf (GMT_stdout, "%s: zmin: %g\n", grd.name, grd.z_min);
+				fprintf (GMT_stdout, "%s: zmax: %g\n", grd.name, grd.z_max);
+			 	fprintf (GMT_stdout, "%s: name: %s\n", grd.name, grd.z_units);
 			}
 			else {
-				fprintf (GMT_stdout, "%s: z_min: ", argv[k]);
+				fprintf (GMT_stdout, "%s: z_min: ", grd.name);
 				GMT_ascii_output_one (GMT_stdout, grd.z_min, 2);
 				fprintf (GMT_stdout, " z_max: ");
 				GMT_ascii_output_one (GMT_stdout, grd.z_max, 2);
@@ -397,20 +390,20 @@ int main (int argc, char **argv)
 
 			GMT_ascii_format_one (text, grd.z_add_offset, GMT_io.out_col_type[2]);
 			if (isalpha ((int)text[strlen(text)-1])) text[strlen(text)-1] = '\0';	/* Chop of trailing WESN flag here */
-			sprintf (format, "%s: scale_factor: %s add_offset: %%s\n", argv[k], gmtdefs.d_format);
+			sprintf (format, "%s: scale_factor: %s add_offset: %%s\n", grd.name, gmtdefs.d_format);
 			fprintf (GMT_stdout, format, grd.z_scale_factor, text);
 			if (n_nan) {
-				fprintf (GMT_stdout, "%s: ", argv[k]);
+				fprintf (GMT_stdout, "%s: ", grd.name);
 				PRINT_SIZE_T (GMT_stdout, n_nan);
 				fprintf (GMT_stdout, " nodes set to NaN\n");
 			}
 			if (Ctrl->L.norm & 1) {
-				fprintf (GMT_stdout, "%s: median: ", argv[k]);	GMT_ascii_output_one (GMT_stdout, median, 2);
+				fprintf (GMT_stdout, "%s: median: ", grd.name);	GMT_ascii_output_one (GMT_stdout, median, 2);
 				fprintf (GMT_stdout, " scale: ");		GMT_ascii_output_one (GMT_stdout, scale, 2);
 				fprintf (GMT_stdout, "\n");
 			}
 			if (Ctrl->L.norm & 2) {
-				fprintf (GMT_stdout, "%s: mean: ", argv[k]);	GMT_ascii_output_one (GMT_stdout, mean, 2);
+				fprintf (GMT_stdout, "%s: mean: ", grd.name);	GMT_ascii_output_one (GMT_stdout, mean, 2);
 				fprintf (GMT_stdout, " stdev: ");		GMT_ascii_output_one (GMT_stdout, stdev, 2);
 				fprintf (GMT_stdout, " rms: ");			GMT_ascii_output_one (GMT_stdout, rms, 2);
 				fprintf (GMT_stdout, "\n");
@@ -476,14 +469,14 @@ int main (int argc, char **argv)
 
 void *New_Grdinfo_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDINFO_CTRL *C;
-	
-	C = (struct GRDINFO_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDINFO_CTRL), "New_Grdinfo_Ctrl");
-	
+
+	C = (struct GRDINFO_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDINFO_CTRL), "New_Grdinfo_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
-		
+
 	return ((void *)C);
 }
 
 void Free_Grdinfo_Ctrl (struct GRDINFO_CTRL *C) {	/* Deallocate control structure */
-	GMT_free ((void *)C);	
+	GMT_free ((void *)C);
 }
