@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.52 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdmask.c,v 1.68 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -41,8 +41,9 @@
 #define GRDMASK_INSIDE		2
 
 struct GRDMASK_CTRL {
-	struct A {	/* -A[step] */
+	struct A {	/* -A[m|p|step] */
 		BOOLEAN active;
+		int mode;
 		double step;
 	} A;
 	struct F {	/* -F */
@@ -69,18 +70,20 @@ struct GRDMASK_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE, done, nofile = TRUE, greenwich, periodic = FALSE, resample = FALSE;
+	BOOLEAN	error = FALSE, done, nofile = TRUE, periodic = FALSE, resample = FALSE;
 
 	char line[BUFSIZ], ptr[BUFSIZ];
 
-	int	i, j, k, ij, nm, n_path, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
-	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, pos;
+	int	i, j, side, fno, n_files = 0, n_args;
+	int	di, dj, i0, j0, n_fields, n_expected_fields, pos;
 	int 	distance_flag = 0, n_pol = 0;
+	
+	GMT_LONG	k, ij, nm, n_path, n_read, n_alloc = GMT_CHUNK;
 
 	float *data;
 
-	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon;
-	double xinc2, yinc2, distance, shrink = 1.0, idx, idy, x0, y0, *in;
+	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon, step;
+	double distance, shrink = 1.0, idx, idy, x0, y0, *in;
 	double xmin1, xmin2, xmax1, xmax2, lon_w;
 
 	FILE *fp;
@@ -118,7 +121,11 @@ int main (int argc, char **argv)
 				/* Supplemental parameters */
 
 				case 'A':	/* Turn off draw_arc mode */
-					if (argv[i][2])
+					if (argv[i][2] == 'm')
+						Ctrl->A.mode = 1;
+					else if (argv[i][2] == 'p')
+						Ctrl->A.mode = 2;
+					else if (argv[i][2])
 						Ctrl->A.step = atof (&argv[i][2]);	/* Undocumented test feature */
 					else
 						Ctrl->A.active = TRUE;
@@ -169,19 +176,20 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdmask %s - Create mask grdfile from polygons or point coverage\n\n", GMT_VERSION);
+		fprintf (stderr, "grdmask %s - Create mask grid file from polygons or point coverage\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> %s\n", GMT_I_OPT);
-		fprintf (stderr, "\t%s [-A] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t%s [-A[m|p]] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
 		fprintf (stderr, "\t[%s] [-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s]\n\n", GMT_Mo_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyfiles is one or more polygon [or point] files\n");
-		fprintf (stderr, "\t-G Specify file name for output mask grd file.\n");
+		fprintf (stderr, "\t-G Specify file name for output mask grid file.\n");
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs for geographic data\n");
+		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs, i.e. connect by straight lines\n");
+		fprintf (stderr, "\t   unless m or p is appended to first follow meridian then parallel, or vice versa.\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('M');
@@ -246,12 +254,9 @@ int main (int argc, char **argv)
 	GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 
 	header.xy_off = 0.5 * header.node_offset;
-	xinc2 = header.xy_off * header.x_inc;
-	yinc2 = header.xy_off * header.y_inc;
-	one_or_zero = 1 - header.node_offset;
 	idx = 1.0 / header.x_inc;
 	idy = 1.0 / header.y_inc;
-	nm = header.nx * header.ny;
+	nm = ((size_t)header.nx) * ((size_t)header.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 	x = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
@@ -267,8 +272,10 @@ int main (int argc, char **argv)
 	}
 
 	if (distance_flag > 0) {
-		shrink = cos (0.5 * (header.y_min + header.y_max) * D2R);
-		di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
+		double width;
+		shrink = cosd (MAX(fabs(header.y_min), fabs(header.y_max)));
+		width = (shrink < GMT_CONV_LIMIT) ? DBL_MAX : ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
+		di = (width < (double)header.nx) ? (int) width : header.nx;
 		dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc));
 	}
 	else {
@@ -277,7 +284,8 @@ int main (int argc, char **argv)
 	}
 	periodic = (GMT_io.in_col_type[GMT_X] == GMT_IS_LON);	/* Dealing with geographic coordinates */
 	resample = (!Ctrl->A.active && periodic);
-	greenwich = (header.x_min <= 0.0 || header.x_max <= 0.0);
+	/* Maximum step size (in degrees) used for interpolation of line segments along great circles (when requested) */
+	step = Ctrl->A.step / project_info.x_scale / project_info.M_PR_DEG;
 	if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 
 	switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
@@ -297,9 +305,9 @@ int main (int argc, char **argv)
 	}
 
 	memset ((void *)&P, 0, sizeof (struct GMT_LINE_SEGMENT));
-	P.coord = (double **) GMT_memory (VNULL, 2, sizeof (double *), GMT_program);	/* Needed as pointers below */
-	P.min = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
-	P.max = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
+	P.coord = (double **) GMT_memory (VNULL, (size_t)2, sizeof (double *), GMT_program);	/* Needed as pointers below */
+	P.min = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
+	P.max = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
 	
 	/* Initialize all nodes to the 'outside' value */
 
@@ -348,7 +356,7 @@ int main (int argc, char **argv)
 
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
 				x[n_path] = in[GMT_X];
@@ -371,8 +379,8 @@ int main (int argc, char **argv)
 						dlon = x[n_path] - x[n_path-1];
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 						lon_sum += dlon;
-						lat_sum += y[n_path];
 					}
+					lat_sum += y[n_path];
 				}
 				else {	/* Cartesian x */
 					if (x[n_path] > xmax) xmax = x[n_path];
@@ -381,7 +389,7 @@ int main (int argc, char **argv)
 				n_path++;
 
 				if (n_path == (n_alloc-1)) {	/* n_alloc-1 since we may need 1 more to close polygon */
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(double), GMT_program);
 					y = (double *) GMT_memory ((void *)y, (size_t)n_alloc, sizeof(double), GMT_program);
 				}
@@ -431,7 +439,7 @@ int main (int argc, char **argv)
 						if (j < 0 || j >= header.ny) continue;
 						for (i = i0 - di; i <= (i0 + di); i++) {
 							if (i < 0 || i >= header.nx) continue;
-							ij = j * header.nx + i;
+							ij = GMT_IJ (j, i, header.nx);
 							x0 = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
 							y0 = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
 							distance = (GMT_distance_func) (x[k], y[k], x0, y0);
@@ -448,7 +456,6 @@ int main (int argc, char **argv)
 					x[n_path] = x[0];
 					y[n_path] = y[0];
 					dlon = x[n_path] - x[n_path-1];
-					lat_sum += y[n_path];
 					n_path++;
 					if (periodic) {
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
@@ -456,7 +463,10 @@ int main (int argc, char **argv)
 					}
 				}
 
-				if (resample) n_path = GMT_fix_up_path (&x, &y, n_path, greenwich, Ctrl->A.step);
+				if (resample) {
+					n_path = GMT_fix_up_path (&x, &y, n_path, step, Ctrl->A.mode);
+					n_alloc = n_path;	/* Since it got reset */
+				}
 
 				if (periodic) {	/* Make polygon structure so we can use spherical polygon machinery */
 					P.coord[GMT_X] = x;
@@ -464,7 +474,7 @@ int main (int argc, char **argv)
 					P.n_rows = n_path;
 					P.min[GMT_Y] = ymin;	P.max[GMT_Y] = ymax;
 					P.min[GMT_X] = xmin;	P.max[GMT_X] = xmax;
-					if (fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
+					if (GMT_360_RANGE (lon_sum, 0.0)) {	/* Contains a pole, convert to polar x,y */
 						P.pole = (lat_sum < 0.0) ? -1 : +1;	/* S or N pole */
 					}
 					else
@@ -505,7 +515,8 @@ int main (int argc, char **argv)
 
 						/* Here, point is inside or on edge, we must assign value */
 
-						data[j*header.nx+i] = (float)Ctrl->N.mask[side];
+						ij = GMT_IJ (j, i, header.nx);
+						data[ij] = (float)Ctrl->N.mask[side];
 					}
 					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %d scanning row %5.5d\r", GMT_program, n_pol, j);
 				}
@@ -533,7 +544,7 @@ int main (int argc, char **argv)
 void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDMASK_CTRL *C;
 	
-	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
+	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->A.step = gmtdefs.line_step;
@@ -542,6 +553,6 @@ void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
