@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdpaste.c,v 1.18 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdpaste.c,v 1.23 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdpaste.c reads two grdfiles and writes a new file with
+ * grdpaste.c reads two grid files and writes a new file with
  * the first two pasted together along their common edge.
  *
  * Author:	Walter Smith
@@ -44,6 +44,8 @@ int main (int argc, char **argv)
 	char *grd_a, *grd_b, format[BUFSIZ];
 
 	int	i, way, one_or_zero, n_in = 0;
+	
+	size_t nm;
 
 	float	*c;
 
@@ -99,7 +101,7 @@ int main (int argc, char **argv)
 	}
 
 	if (GMT_give_synopsis_and_exit || argc == 1) {
-		fprintf (stderr,"grdpaste %s - Join two grdfiles along common edge\n\n", GMT_VERSION);
+		fprintf (stderr,"grdpaste %s - Join two grid files along common edge\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdpaste <file_a> <file_b> -G<outfile> [-V]\n\n");
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -208,7 +210,7 @@ int main (int argc, char **argv)
 
 	/* Now we can do it  */
 
-	sprintf (format, "%%s: \t%s\t%s\t%s\t%s\t%s\t%s\t%%d\t%%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+	sprintf (format, "%%s: \t%s\t%s\t%s\t%s\t%s\t%s\t%%ld\t%%ld\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 	if (gmtdefs.verbose) {
 		fprintf (stderr, "File spec:\tW E S N dx dy nx ny:\n");
 		fprintf (stderr, format, grd_a, ha.x_min, ha.x_max, ha.y_min, ha.y_max, ha.x_inc, ha.y_inc, ha.nx, ha.ny);
@@ -216,7 +218,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, format, Ctrl->G.file, hc.x_min, hc.x_max, hc.y_min, hc.y_max, hc.x_inc, hc.y_inc, hc.nx, hc.ny);
 	}
 
-	c = (float *) GMT_memory (VNULL, (size_t)(hc.nx*hc.ny), sizeof (float), GMT_program);
+	nm = ((size_t)hc.nx) * ((size_t)hc.ny);
+	c = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 	hc.node_offset = ha.node_offset;
 	hc.z_scale_factor = ha.z_scale_factor;
 	hc.z_add_offset = ha.z_add_offset;
@@ -263,7 +266,7 @@ int main (int argc, char **argv)
 void *New_Grdpaste_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDPASTE_CTRL *C;
 	
-	C = (struct GRDPASTE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDPASTE_CTRL), "New_Grdpaste_Ctrl");
+	C = (struct GRDPASTE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDPASTE_CTRL), "New_Grdpaste_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 			
@@ -271,6 +274,6 @@ void *New_Grdpaste_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Grdpaste_Ctrl (struct GRDPASTE_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
