@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdproject.c,v 1.34 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdproject.c,v 1.43 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdproject reads a geographical grdfile and evaluates the grid at new grid positions
+ * grdproject reads a geographical grid file and evaluates the grid at new grid positions
  * specified by the map projection and new dx/dy values using a weighted average of all
  * points within the search radius. Optionally, grdproject may perform the inverse
  * transformation, going from rectangular coordinates to geographical.
@@ -64,11 +64,11 @@ struct GRDPROJECT_CTRL {
 		BOOLEAN active;
 		int nx, ny;
 	} N;
-	struct S {	/* -S<radius> */
+	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
 		BOOLEAN active;
-		BOOLEAN bilinear;
-		BOOLEAN careful;
-		double radius;
+		BOOLEAN antialias;
+		int interpolant;
+		double threshold;
 	} S;
 };
 
@@ -78,9 +78,9 @@ int main (int argc, char **argv)
 
 	char *infile, format[BUFSIZ], unit_name[GRD_UNIT_LEN], scale_unit_name[GRD_UNIT_LEN];
 
-	int i, unit = 0;
+	int i, j, unit = 0;
 	
-	size_t nm;
+	GMT_LONG nm;
 	
 	float *geo, *rect;
 	
@@ -159,14 +159,26 @@ int main (int argc, char **argv)
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
-					if (argv[i][2] == '+')
-						Ctrl->S.careful = TRUE;
-					else if (argv[i][2] == '-') {
-						Ctrl->S.careful = TRUE;
-						Ctrl->S.bilinear = TRUE;
+					for (j = 2; j < 5 && argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case '-':
+								Ctrl->S.antialias = FALSE; break;
+							case 'n':
+								Ctrl->S.interpolant = BCR_NEARNEIGHBOR; break;
+							case 'l':
+								Ctrl->S.interpolant = BCR_BILINEAR; break;
+							case 'b':
+								Ctrl->S.interpolant = BCR_BSPLINE; break;
+							case 'c':
+								Ctrl->S.interpolant = BCR_BICUBIC; break;
+							case '/':
+								Ctrl->S.threshold = atof (&argv[i][j+1]);
+								j = 5; break;
+							default:
+								fprintf (stderr, "%s: Warning: The -S option has changed meaning. Use -S[-]b|c|l|n[/threshold] to specify interpolation mode.\n", GMT_program);
+								j = 5; break;
+						}
 					}
-					else
-						Ctrl->S.radius = atof (&argv[i][2]);
 					break;
 				default:
 					error = TRUE;
@@ -182,15 +194,16 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdproject %s - Project geographical grid to/from rectangular grid\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdproject <in_grdfile> %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
+		fprintf (stderr, "usage: grdproject <in_grdfile> -G<out_grdfile> %s\n", GMT_J_OPT);
 		fprintf (stderr, "\t[-A[k|m|n|i|c|p]] [-C[<dx/dy>]] [-D%s] [-E<dpi>] [-F]\n", GMT_inc_OPT);
-		fprintf (stderr, "\t[-G<out_grdfile>] [-I] [-Mc|i|m] [-N<nx/ny>] [-S<radius>] [-V]\n\n");
+		fprintf (stderr, "\t[-I] [-Mc|i|m] [-N<nx/ny>] [%s]\n", GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-S[-]b|c|l|n[/<threshold>]] [-V]\n\n");
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<in_grdfile> is data set to be transformed\n");
+		fprintf (stderr, "\t-G name of output grid\n");
 		GMT_explain_option ('J');
-		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A force projected values to be in actual meters [Default uses the given map scale]\n");
 		fprintf (stderr, "\t   Specify another unit by appending k (km), m (miles), n (nautical miles), i (inch), c (cm), or p (points)\n");
@@ -199,14 +212,19 @@ int main (int argc, char **argv)
 		GMT_inc_syntax ('D', 0);
 		fprintf (stderr, "\t-E sets dpi for output grid\n");
 		fprintf (stderr, "\t-F toggle between pixel and grid registration  [Default is same as input]\n");
-		fprintf (stderr, "\t-G name of output grid\n");
 		fprintf (stderr, "\t-I Inverse transformation from rectangular to geographical\n");
 		fprintf (stderr, "\t-M Temporarily reset MEASURE_UNIT to be c (cm), i (inch), m (meter), or p (point)\n");
 		fprintf (stderr, "\t   Cannot be used if -A is set.\n");
 		fprintf (stderr, "\t-N sets the number of nodes for the new grid\n");
 		fprintf (stderr, "\t   Only one of -D, -E, and -N can be specified!\n");
 		fprintf (stderr, "\t   If none are specified, nx,ny of the input file is used\n");
-		fprintf (stderr, "\t-S sets the search radius in projected units [Default avoids aliasing]\n");
+		GMT_explain_option ('R');
+		fprintf (stderr, "\t-S Determines the interpolation mode (b = B-spline, c = bicubic, l = bilinear,\n");
+		fprintf (stderr, "\t   n = nearest-neighbor) [Default: bicubic]\n");
+		fprintf (stderr, "\t   Optionally, prepend - to switch off antialiasing [Default: on]\n");
+		fprintf (stderr, "\t   Append /<threshold> to change the minimum weight in vicinity of NaNs. A threshold of\n");
+		fprintf (stderr, "\t   1.0 requires all nodes involved in interpolation to be non-NaN; 0.5 will interpolate\n");
+		fprintf (stderr, "\t   about half way from a non-NaN to a NaN node [Default: 0.5]\n");
 		GMT_explain_option ('V');
 
 		exit (EXIT_FAILURE);
@@ -220,14 +238,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Must specify output file\n", GMT_program);
 		error++;
 	}
-	if (!project_info.region_supplied) {
+	/*if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
-	}
-	if (Ctrl->S.radius < 0.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Max search radius, if specified, must be positive\n", GMT_program);
-		error++;
-	}
+	}*/
 	if ((Ctrl->M.active + Ctrl->A.active) == 2) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Can specify only one of -A and -M\n", GMT_program);
 		error++;
@@ -248,6 +262,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option.  Must specify positive dpi\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->S.active && (Ctrl->S.threshold < 0.0 || Ctrl->S.threshold > 1.0)) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  threshold must be in [0,1] range\n", GMT_program);
+		error++;
+	}
 
 	if (error) exit (EXIT_FAILURE);
 
@@ -257,6 +275,77 @@ int main (int argc, char **argv)
 	unit = GMT_check_scalingopt ('A', Ctrl->A.unit, scale_unit_name);
 	GMT_init_scales (unit, &fwd_scale, &inv_scale, &inch_to_unit, &unit_to_inch, unit_name);
 
+	if (!project_info.region_supplied) {
+		double ww, ee, ss, nn;
+		char opt_R[BUFSIZ];
+		struct GRD_HEADER head;
+		GMT_err_fail (GMT_read_grd_info (infile, &head), infile);
+		w = head.x_min;		e = head.x_max;
+		s = head.y_min;		n = head.y_max;
+		if (!Ctrl->I.active) {
+			sprintf (opt_R, "-R%.12f/%.12f/%.12f/%.12f", w, e, s, n);
+			GMT_parse_R_option (opt_R, &ww, &ee, &ss, &nn);
+			GMT_err_fail (GMT_map_setup (w, e, s, n), "");
+		}
+		else {			/* Do inverse transformation */
+			double x_c, y_c, lon_t, lat_t;
+			/* Obtain a first crude extimation of the good -R */
+			x_c = (w + e) / 2.; 		/* mid point of projected coords */
+			y_c = (s + n) / 2.; 
+			if (y_c > 0)
+				GMT_parse_common_options ("-R-180/180/0/80", &ww, &ee, &ss, &nn);
+			else
+				GMT_parse_R_option ("-R-180/180/-80/0", &ww, &ee, &ss, &nn);
+			if (shift_xy) {
+				x_c -= Ctrl->C.easting;
+				x_c -= Ctrl->C.northing;
+			}
+			/* Convert from 1:1 scale */ 
+			if (unit) {
+				x_c *= fwd_scale;
+				y_c *= fwd_scale;
+			}
+			x_c *= project_info.x_scale;
+			y_c *= project_info.y_scale;
+
+			if (Ctrl->C.active) {	/* Then correct so lower left corner is (0,0) */
+				x_c += project_info.x0;
+				y_c += project_info.y0;
+			}
+			GMT_err_fail (GMT_map_setup (ww, ee, ss, nn), "");
+			GMT_xy_to_geo (&lon_t, &lat_t, x_c, y_c);
+			sprintf (opt_R, "-R%.12f/%.12f/%.12f/%.12f", lon_t-1, lon_t+1, lat_t-1, lat_t+1);
+			if (gmtdefs.verbose) fprintf (stderr, "First opt_R\t %s\t%g\t%g\n", opt_R, x_c, y_c);
+			GMT_parse_common_options (opt_R, &ww, &ee, &ss, &nn);
+			GMT_err_fail (GMT_map_setup (ww, ee, ss, nn), "");
+
+			/* Finaly obtain the good limits */
+			if (shift_xy) {
+				w -= Ctrl->C.easting;	e -= Ctrl->C.easting;
+				s -= Ctrl->C.northing;	n -= Ctrl->C.northing;
+			}
+			if (unit) {
+				w *= fwd_scale;		e *= fwd_scale;
+				s *= fwd_scale;		n *= fwd_scale;
+			}
+			w *= project_info.x_scale;	e *= project_info.x_scale;
+			s *= project_info.y_scale;	n *= project_info.y_scale;
+
+			if (Ctrl->C.active) {
+				w += project_info.x0;	e += project_info.x0;
+				s += project_info.y0;	n += project_info.y0;
+			}
+
+			GMT_xy_to_geo (&ww, &ss, w, s);		/* SW corner */
+			GMT_xy_to_geo (&ee, &nn, e, n);		/* NE corner */
+			sprintf (opt_R, "-R%.12f/%.12f/%.12f/%.12fr", ww, ss, ee, nn);
+			if (gmtdefs.verbose) fprintf (stderr, "Second opt_R\t %s\n", opt_R);
+			GMT_parse_common_options (opt_R, &ww, &ee, &ss, &nn);
+			w = ww;		e = ee;
+			s = ss;		n = nn;
+		}
+	}
+
 	GMT_err_fail (GMT_map_setup (w, e, s, n), "");
 
 	xmin = (Ctrl->C.active) ? project_info.xmin - project_info.x0 : project_info.xmin;
@@ -264,7 +353,7 @@ int main (int argc, char **argv)
 	ymin = (Ctrl->C.active) ? project_info.ymin - project_info.y0 : project_info.ymin;
 	ymax = (Ctrl->C.active) ? project_info.ymax - project_info.y0 : project_info.ymax;
 	if (Ctrl->A.active) {	/* Convert to chosen units */
-		strncpy (unit_name, scale_unit_name, GRD_UNIT_LEN);
+		strncpy (unit_name, scale_unit_name, (size_t)GRD_UNIT_LEN);
 		xmin /= project_info.x_scale;
 		xmax /= project_info.x_scale;
 		ymin /= project_info.y_scale;
@@ -302,15 +391,11 @@ int main (int argc, char **argv)
 
 		GMT_err_fail (GMT_read_grd_info (infile, &r_head), infile);
 
-		if (Ctrl->S.careful) {
-			GMT_boundcond_init (&edgeinfo);
-			nm = ((size_t)(4 + r_head.nx)) * ((size_t)(4 + r_head.ny));
-			GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
-		}
-		else
-			nm = ((size_t)r_head.nx) * ((size_t)r_head.ny);
+		GMT_boundcond_init (&edgeinfo);
+		nm = ((GMT_LONG)(4 + r_head.nx)) * ((GMT_LONG)(4 + r_head.ny));
+		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 
-		rect = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+		rect = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		GMT_err_fail (GMT_read_grd (infile, &r_head, rect, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), infile);
 		offset = r_head.node_offset;		/* Same as input */
 		if (Ctrl->F.active) offset = !offset;	/* Toggle */
@@ -321,7 +406,7 @@ int main (int argc, char **argv)
 		GMT_err_fail (GMT_grdproject_init (&g_head, Ctrl->D.xinc, Ctrl->D.yinc, Ctrl->N.nx, Ctrl->N.ny, Ctrl->E.dpi, offset), Ctrl->G.file);
 		geo = (float *) GMT_memory (VNULL, (size_t)(g_head.nx * g_head.ny), sizeof (float), GMT_program);
 		if (gmtdefs.verbose) {
-			fprintf (stderr, "%s:  Transform ", GMT_program);
+			fprintf (stderr, "%s: Transform ", GMT_program);
 			fprintf (stderr, format, g_head.x_min, g_head.x_max, g_head.y_min, g_head.y_max);
 			fprintf (stderr, " <-- ");
 			fprintf (stderr, format, xmin, xmax, ymin, ymax);
@@ -343,20 +428,17 @@ int main (int argc, char **argv)
 				r_head.x_max *= inv_scale;
 				r_head.y_min *= inv_scale;
 				r_head.y_max *= inv_scale;
-				Ctrl->S.radius *= inv_scale;
 			}
 			r_head.x_min *= project_info.x_scale;
 			r_head.x_max *= project_info.x_scale;
 			r_head.y_min *= project_info.y_scale;
 			r_head.y_max *= project_info.y_scale;
-			Ctrl->S.radius *= project_info.x_scale;
 		}
 		else if (gmtdefs.measure_unit != GMT_INCH) {	/* Convert from inch to whatever */
 			r_head.x_min *= unit_to_inch;
 			r_head.x_max *= unit_to_inch;
 			r_head.y_min *= unit_to_inch;
 			r_head.y_max *= unit_to_inch;
-			Ctrl->S.radius *= unit_to_inch;
 		}
 		if (Ctrl->C.active) {	/* Then correct so lower left corner is (0,0) */
 			r_head.x_min += project_info.x0;
@@ -367,31 +449,20 @@ int main (int argc, char **argv)
 		r_head.x_inc = GMT_get_inc (r_head.x_min, r_head.x_max, r_head.nx, r_head.node_offset);
 		r_head.y_inc = GMT_get_inc (r_head.y_min, r_head.y_max, r_head.ny, r_head.node_offset);
 
-		/* rect xy values and Ctrl->S.radius are here in GMT projected inches */
-
-		GMT_init_search_radius (&Ctrl->S.radius, &r_head, &g_head, TRUE);
-		if (Ctrl->S.careful)
-			GMT_grd_project (rect, &r_head, geo, &g_head, &edgeinfo, Ctrl->S.bilinear, TRUE);
-		else
-			GMT_grd_inverse (geo, &g_head, rect, &r_head, Ctrl->S.radius);
-
-		if (Ctrl->S.careful) GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
+		GMT_grd_project (rect, &r_head, geo, &g_head, &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, TRUE);
 
+		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
 		GMT_err_fail (GMT_write_grd (Ctrl->G.file, &g_head, geo, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 	}
 	else {	/* Forward projection from geographical to rectangular grid */
 
 		GMT_err_fail (GMT_read_grd_info (infile, &g_head), infile);
 
-		if (Ctrl->S.careful) {
-			GMT_boundcond_init (&edgeinfo);
-			nm = ((size_t)(4 + g_head.nx)) * ((size_t)(4 + g_head.ny));
-			GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
-		}
-		else
-			nm = ((size_t)g_head.nx) * ((size_t)g_head.ny);
+		GMT_boundcond_init (&edgeinfo);
+		nm = ((GMT_LONG)(4 + g_head.nx)) * ((GMT_LONG)(4 + g_head.ny));
+		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 
-		geo = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+		geo = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		GMT_err_fail (GMT_read_grd (infile, &g_head, geo, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), infile);
 
 		r_head.x_min = project_info.xmin;	r_head.x_max = project_info.xmax;
@@ -400,16 +471,13 @@ int main (int argc, char **argv)
 			if (unit) {	/* Undo the 1:1 unit used */
 				Ctrl->D.xinc *= inv_scale;
 				Ctrl->D.yinc *= inv_scale;
-				Ctrl->S.radius *= inv_scale;
 			}
 			Ctrl->D.xinc *= project_info.x_scale;
 			Ctrl->D.yinc *= project_info.y_scale;
-			Ctrl->S.radius *= project_info.x_scale;
 		}
 		else if (gmtdefs.measure_unit != GMT_INCH) {	/* Convert from inch to whatever */
 			Ctrl->D.xinc *= unit_to_inch;
 			Ctrl->D.yinc *= unit_to_inch;
-			Ctrl->S.radius *= unit_to_inch;
 		}
 		if (set_n) {
 			Ctrl->N.nx = g_head.nx;
@@ -417,7 +485,7 @@ int main (int argc, char **argv)
 		}
 
 		if (gmtdefs.verbose) {
-			fprintf (stderr, "%s:  Transform ", GMT_program);
+			fprintf (stderr, "%s: Transform ", GMT_program);
 			fprintf (stderr, format, g_head.x_min, g_head.x_max, g_head.y_min, g_head.y_max);
 			fprintf (stderr, " --> ");
 			fprintf (stderr, format, xmin, xmax, ymin, ymax);
@@ -428,12 +496,9 @@ int main (int argc, char **argv)
 		if (Ctrl->F.active) offset = !offset;	/* Toggle */
 
 		GMT_err_fail (GMT_grdproject_init (&r_head, Ctrl->D.xinc, Ctrl->D.yinc, Ctrl->N.nx, Ctrl->N.ny, Ctrl->E.dpi, offset), Ctrl->G.file);
-		rect = (float *) GMT_memory (VNULL, (size_t)(r_head.nx * r_head.ny), sizeof (float), GMT_program);
-		GMT_init_search_radius (&Ctrl->S.radius, &r_head, &g_head, FALSE);
-		if (Ctrl->S.careful)
-			GMT_grd_project (geo, &g_head, rect, &r_head, &edgeinfo, Ctrl->S.bilinear, FALSE);
-		else
-			GMT_grd_forward (geo, &g_head, rect, &r_head, Ctrl->S.radius);
+		nm = ((GMT_LONG)r_head.nx) * ((GMT_LONG)r_head.ny);
+		rect = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+		GMT_grd_project (geo, &g_head, rect, &r_head, &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 
 		/* Modify output rect header if -A, -C, -M have been set */
 
@@ -473,8 +538,7 @@ int main (int argc, char **argv)
 
 		/* rect xy values are here in GMT projected units chosen by user */
 
-		if (Ctrl->S.careful) GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
-
+		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
 		GMT_err_fail (GMT_write_grd (Ctrl->G.file, &r_head, rect, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 	}
 
@@ -491,14 +555,15 @@ int main (int argc, char **argv)
 void *New_Grdproject_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDPROJECT_CTRL *C;
 	
-	C = (struct GRDPROJECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDPROJECT_CTRL), "New_Grdproject_Ctrl");
+	C = (struct GRDPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDPROJECT_CTRL), "New_Grdproject_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->S.antialias = TRUE; C->S.interpolant = BCR_BICUBIC; C->S.threshold = 0.5;
 		
 	return ((void *)C);
 }
 
 void Free_Grdproject_Ctrl (struct GRDPROJECT_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
