@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdreformat.c,v 1.35 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdreformat.c,v 1.43 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,10 +16,10 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdreformat.c reads a grd file in one format and outputs it in another
+ * grdreformat.c reads a grid file in one format and outputs it in another
  *
  * Author:	Paul Wessel
- * Date:	3-JAN-1991-2007
+ * Date:	3-JAN-1991-2008
  * Version:	4
  */
 
@@ -89,13 +89,13 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
-		fprintf (stderr, "grdreformat %s - Converting between different grdfile formats\n\n", GMT_VERSION);
+		fprintf (stderr, "grdreformat %s - Converting between different grid file formats\n\n", GMT_VERSION);
 		fprintf( stderr, "usage: grdreformat ingrdfile[=id[/scale/offset]] outgrdfile[=id[/scale/offset]] [-N]\n\t[%s] [%s] [-V]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\tingrdfile is the grd file to convert\n");
-		fprintf (stderr, "\toutgrdfile is the new converted grd file\n");
+		fprintf (stderr, "\tingrdfile is the grid file to convert\n");
+		fprintf (stderr, "\toutgrdfile is the new converted grid file\n");
 		fprintf( stderr, "\tscale and offset, if given, will multiply data by scale and add offset.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-N  Do NOT write the header (native grids only - ignored otherwise).\n");
@@ -104,8 +104,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('f');
 		GMT_explain_option ('V');
 
-		fprintf (stderr, "\n	The following grdfile formats are supported:\n\n");
-		GMT_getsharepath (CNULL, "gmtformats", ".d", line);
+		fprintf (stderr, "\n	The following grid file formats are supported:\n\n");
+		GMT_getsharepath ("conf", "gmt_formats", ".conf", line);
 		if ((fp = fopen (line, "r")) == NULL) {
 			fprintf (stderr, "%s: GMT ERROR: Cannot read file %s\n", GMT_program, line);
 			exit (EXIT_FAILURE);
@@ -129,7 +129,7 @@ int main (int argc, char **argv)
 	GMT_err_fail (GMT_grd_get_format (file[0], &grd, TRUE), file[0]);
 	type[0] = grd.type;
 	strcpy (fname[0], grd.name);
-	GMT_err_fail (GMT_grd_get_format (file[1], &grd, TRUE), file[1]);
+	GMT_err_fail (GMT_grd_get_format (file[1], &grd, FALSE), file[1]);
 	type[1] = grd.type;
 	strcpy (fname[1], grd.name);
 
@@ -188,7 +188,7 @@ int main (int argc, char **argv)
 void *New_Grdreformat_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDREFORMAT_CTRL *C;
 	
-	C = (struct GRDREFORMAT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDREFORMAT_CTRL), "New_Grdreformat_Ctrl");
+	C = (struct GRDREFORMAT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDREFORMAT_CTRL), "New_Grdreformat_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
