@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdsample.c,v 1.46 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdsample.c,v 1.53 2008/05/01 18:36:32 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,10 +16,10 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdsample reads a grdfile and evaluates the grid at new grid positions
+ * grdsample reads a grid file and evaluates the grid at new grid positions
  * specified by new dx/dy values using a 2-D Taylor expansion of order 3.
  * In order to evaluate derivatives along the edges of the surface, I assume 
- * natural bi-cubic spline conditions, i.e. both the second and third normal 
+ * natural bicubic spline conditions, i.e. both the second and third normal 
  * derivatives are zero, and that the dxdy derivative in the corners are zero, too.
  *
  * Author:	Paul Wessel
@@ -47,9 +47,10 @@ struct GRDSAMPLE_CTRL {
 		BOOLEAN active;
 		char mode[4];
 	} L;
-	struct Q {	/* -Q[<value>] */
+	struct Q {	/* -Q[b|c|l|n][[/]<threshold>] */
 		BOOLEAN active;
-		double value;
+		int interpolant;
+		double threshold;
 	} Q;
 	struct T {	/* -T */
 		BOOLEAN active;
@@ -58,13 +59,13 @@ struct GRDSAMPLE_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, node = FALSE;
+	BOOLEAN error = FALSE;
 
 	char *infile = CNULL, format[BUFSIZ];
 
-	int i, j, ii = 0, jj = 0, mx = 0;
+	int i, j, ii = 0, jj = 0;
 
-	size_t ij, k, nm;
+	GMT_LONG ij, nm;
 	
 	float *a, *b;
 
@@ -123,11 +124,31 @@ int main (int argc, char **argv)
 					break;
 				case 'L':
 					Ctrl->L.active = TRUE;
-					strncpy (Ctrl->L.mode, &argv[i][2], 4);
+					strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					break;
 				case 'Q':
 					Ctrl->Q.active = TRUE;
-					Ctrl->Q.value = (argv[i][2]) ? atof (&argv[i][2]) : 1.0;
+					Ctrl->Q.interpolant = BCR_BILINEAR;
+					for (j = 2; j < 5 && argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case 'n':
+								Ctrl->Q.interpolant = BCR_NEARNEIGHBOR; break;
+							case 'l':
+								Ctrl->Q.interpolant = BCR_BILINEAR; break;
+							case 'b':
+								Ctrl->Q.interpolant = BCR_BSPLINE; break;
+							case 'c':
+								Ctrl->Q.interpolant = BCR_BICUBIC; break;
+							case '/':
+							default:
+								Ctrl->Q.threshold = atof (&argv[i][j]);
+								if (j == 2 && Ctrl->Q.threshold < GMT_SMALL) {
+									Ctrl->Q.interpolant = BCR_NEARNEIGHBOR;
+									fprintf (stderr, "%s: Warning: Option -Q0 depreciated. Use -Qn instead.\n", GMT_program);
+								}
+								j = 5; break;
+						}
+					}
 					break;
 				case 'T':	/* Convert from pixel file <-> gridfile */
 					Ctrl->T.active = TRUE;
@@ -150,7 +171,7 @@ int main (int argc, char **argv)
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<old_grdfile> is data set to be resampled\n");
-		fprintf (stderr, "\t-G sets the name of the interpolated output grdfile\n");
+		fprintf (stderr, "\t-G sets the name of the interpolated output grid file\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-F Force pixel registration  [Default is same as input]\n");
 		GMT_inc_syntax ('I', 0);
@@ -160,12 +181,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   or one or both of\n");
 		fprintf (stderr, "\t   x for periodic boundary conditions on x\n");
 		fprintf (stderr, "\t   y for periodic boundary conditions on y\n");
-		fprintf (stderr, "\t-Q Quick mode, use bilinear rather than bicubic interpolation.\n");
-		fprintf (stderr, "\t   Optionally, append <value> in the 0 < value <= 1 range.\n");
-		fprintf (stderr, "\t   [Default = 1 requires all 4 nodes to be non-NaN.], <value> = 0.5\n");
-		fprintf (stderr, "\t   will interpolate about 1/2 way from a non-NaN to a NaN node, while\n");
-		fprintf (stderr, "\t   0.1 will go about 90%% of the way, etc. -Q0 will return the value of the \n");
-		fprintf (stderr, "\t   nearest node instead of interpolating.\n");
+		fprintf (stderr, "\t-Q Quick mode, use bilinear rather than bicubic [Default] interpolation.\n");
+		fprintf (stderr, "\t   Alternatively, select interpolation mode by adding b = B-spline, c = bicubic,\n");
+		fprintf (stderr, "\t   l = bilinear, or n = nearest-neighbor.\n");
+		fprintf (stderr, "\t   Optionally, append <threshold> in the range [0,1]. [Default = 1 requires all\n");
+		fprintf (stderr, "\t   4 or 16 nodes to be non-NaN.], <threshold> = 0.5 will interpolate about 1/2 way\n");
+		fprintf (stderr, "\t   from a non-NaN to a NaN node, while 0.1 will go about 90%% of the way, etc.\n");
+		fprintf (stderr, "\t   -Q0 will return the value of the nearest node instead of interpolating (Same as -Qn).\n");
 		fprintf (stderr, "\t-R specifies a subregion [Default is old region]\n");
 		fprintf (stderr, "\t-T Toggles between grid registration and pixel registration\n");
 		GMT_explain_option ('V');
@@ -174,7 +196,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (Ctrl->Q.active && (Ctrl->Q.value < 0.0 || Ctrl->Q.value > 1.0)) {
+	if (Ctrl->Q.active && (Ctrl->Q.threshold < 0.0 || Ctrl->Q.threshold > 1.0)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  threshold must be in [0,1] range\n", GMT_program);
 		error++;
 	}
@@ -228,15 +250,16 @@ int main (int argc, char **argv)
 
 	GMT_boundcond_param_prep (&grd_a, &edgeinfo);
 
-	a = (float *) GMT_memory (VNULL, (size_t)((grd_a.nx + 4) * (grd_a.ny + 4)), sizeof(float), GMT_program);
+	nm = ((size_t)grd_a.nx + 4) * ((size_t)grd_a.ny + 4);	/* Reserve space for padding */
+	a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 
 	if (project_info.region_supplied) {
 		if (!edgeinfo.nxp && (grd_b.x_min < grd_a.x_min || grd_b.x_max > grd_a.x_max)) {
-			fprintf (stderr, "%s:  Selected region exceeds the X-boundaries of the grdfile!\n", GMT_program);
+			fprintf (stderr, "%s:  Selected region exceeds the X-boundaries of the grid file!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
 		else if (!edgeinfo.nyp && (grd_b.y_min < grd_a.y_min || grd_b.y_max > grd_a.y_max)) {
-			fprintf (stderr, "%s:  Selected region exceeds the Y-boundaries of the grdfile!\n", GMT_program);
+			fprintf (stderr, "%s:  Selected region exceeds the Y-boundaries of the grid file!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
 	}
@@ -249,7 +272,7 @@ int main (int argc, char **argv)
 	GMT_err_fail (GMT_grd_RI_verify (&grd_b, 1), Ctrl->G.file);
 
 	nm = ((size_t)grd_b.nx) * ((size_t)grd_b.ny);
-	b = (float *) GMT_memory (VNULL, nm, sizeof(float), GMT_program);
+	b = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 
 	sprintf (format, "%%s: New grid (%s/%s/%s/%s) nx = %%d ny = %%d dx = %s dy = %s node_offset = %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 	if (gmtdefs.verbose) fprintf (stderr, format, GMT_program, grd_b.x_min, grd_b.x_max, grd_b.y_min, grd_b.y_max, grd_b.nx, grd_b.ny, grd_b.x_inc, grd_b.y_inc, grd_b.node_offset);
@@ -260,7 +283,7 @@ int main (int argc, char **argv)
 
 	/* Initialize bcr structure:  */
 
-	GMT_bcr_init (&grd_a, GMT_pad, Ctrl->Q.active, Ctrl->Q.value, &bcr);
+	GMT_bcr_init (&grd_a, GMT_pad, Ctrl->Q.interpolant, Ctrl->Q.threshold, &bcr);
 
 	/* Set boundary conditions  */
 
@@ -279,11 +302,6 @@ int main (int argc, char **argv)
 			lon[i] += grd_a.x_inc * edgeinfo.nxp;
 	}
 
-	if (Ctrl->Q.value == 0.0) {
-		node = TRUE;
-		mx = grd_a.nx + 4;
-	}
-	
 	for (j = ij = 0; j < grd_b.ny; j++) {
 		lat = GMT_j_to_y (j, grd_b.y_min, grd_b.y_max, grd_b.y_inc, grd_b.xy_off, grd_b.ny);
 		if (!edgeinfo.nyp)
@@ -293,15 +311,7 @@ int main (int argc, char **argv)
 		else if (lat < grd_a.y_min)
 			lat += grd_a.y_inc * edgeinfo.nyp;
 		for (i = 0; i < grd_b.nx; i++, ij++) {
-			if (node) {
-				ii = GMT_x_to_i (lon[i], grd_a.x_min, grd_a.x_inc, grd_a.xy_off, grd_a.nx);
-				jj = GMT_y_to_j (lat, grd_a.y_min, grd_a.y_inc, grd_a.xy_off, grd_a.ny);
-				k = ((size_t)(j+GMT_pad[3])) * ((size_t)mx) + ((size_t)i) + ((size_t)GMT_pad[0]);
-
-				b[ij] = a[k];
-			}
-			else
-				b[ij] = (float)GMT_get_bcr_z (&grd_a, lon[i], lat, a, &edgeinfo, &bcr);
+			b[ij] = (float)GMT_get_bcr_z (&grd_a, lon[i], lat, a, &edgeinfo, &bcr);
 		}
 	}
 
@@ -322,14 +332,14 @@ int main (int argc, char **argv)
 void *New_Grdsample_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDSAMPLE_CTRL *C;
 	
-	C = (struct GRDSAMPLE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDSAMPLE_CTRL), "New_Grdsample_Ctrl");
+	C = (struct GRDSAMPLE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDSAMPLE_CTRL), "New_Grdsample_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
-	C->Q.value = 1.0;
+	C->Q.interpolant = BCR_BICUBIC; C->Q.threshold = 1.0;
 	return ((void *)C);
 }
 
 void Free_Grdsample_Ctrl (struct GRDSAMPLE_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
