@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdtrend.c,v 1.23 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdtrend.c,v 1.30 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 /* grdtrend <input.grd> -N[r]<n_model> [-T<trend.grd>] [-V]
 	[-W<weight.grd] [-D<differences.grd]
 
-Reads a grdfile and fits a trend surface.  Trend surface
+Reads a grid file and fits a trend surface.  Trend surface
 is defined by:
 
 m1 +m2*x + m3*y + m4*xy + m5*x*x + m6*y*y + m7*x*x*x
@@ -33,7 +33,7 @@ n_model = 4 fits a bilinear surface,
 n_model = 6 fits a biquadratic,
 n_model = 10 fits a bicubic surface.
 
-The user may write out grdfiles of the fitted surface
+The user may write out grid files of the fitted surface
 [-T<trend.grd>] and / or of the residuals (input data
 minus fitted trend) [-D<differences.grd] and / or of
 the weights used in iterative fitting [-W<weight.grd].
@@ -59,8 +59,8 @@ Remarks:
 
 We adopt a translation and scaling of the x,y coordinates.
 We choose x,y such that they are in [-1,1] over the range
-of the grdfile.  If the problem is unweighted, all input
-values are filled (no "holes" or NaNs in the input grdfile),
+of the grid file.  If the problem is unweighted, all input
+values are filled (no "holes" or NaNs in the input grid file),
 and n_model <= 4 (bilinear or simpler), then the normal
 equations matrix (G'G in Menke notation) is diagonal under
 this change of coordinates, and the solution is trivial.
@@ -111,9 +111,11 @@ int main(int argc, char **argv)
 
 	char	*i_filename = NULL, format[BUFSIZ];
 
-	int	i, j, k, ierror = 0, iterations, nxy;
+	int	j, k, ierror = 0, iterations;
+	
+	GMT_LONG	i, nxy;
 
-	float	*data;		/* Pointer for array from input grdfile  */
+	float	*data;		/* Pointer for array from input grid file  */
 	float	*trend;		/* Pointer for array containing fitted surface  */
 	float	*resid;		/* Pointer for array containing residual surface  */
 	float	*weight = VNULL;/* Pointer for array containing data weights  */
@@ -131,12 +133,11 @@ int main(int argc, char **argv)
 
 	void	gauss(double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag);		/* QR solution of the Normal equations  */
 	void	set_up_vals(double *val, int nval, double vmin, double vmax, double dv, int pixel_reg);		/* Store x[i], y[j] once for all to save time  */
-	void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
 	void	grd_trivial_model(float *data, int nx, int ny, double *xval, double *yval, double *gtd, int n_model);	/* Fit trivial models.  See Remarks above.  */
 	void	compute_trend(float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff);	/* Find trend from a model  */
-	void	compute_resid(float *data, float *trend, float *resid, int nxy);	/* Find residuals from a trend  */
-	void	compute_chisq(float *resid, float *weight, int nxy, double *chisq, double scale);	/* Find Chi-Squared from weighted residuals  */
-	void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale);	/* Find weights from residuals  */
+	void	compute_resid(float *data, float *trend, float *resid, GMT_LONG nxy);	/* Find residuals from a trend  */
+	void	compute_chisq(float *resid, float *weight, GMT_LONG nxy, double *chisq, double scale);	/* Find Chi-Squared from weighted residuals  */
+	void	compute_robust_weight(float *resid, float *weight, GMT_LONG nxy, double *scale);	/* Find weights from residuals  */
 	void	write_model_parameters(double *gtd, int n_model);	/* Do reports if gmtdefs.verbose == TRUE  */
 	void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted);		/* Fill normal equations matrices  */
 	void	*New_Grdtrend_Ctrl (), Free_Grdtrend_Ctrl (struct GRDTREND_CTRL *C);
@@ -212,13 +213,13 @@ int main(int argc, char **argv)
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr,"\t<input.grd> is name of grdfile to fit trend to.\n");
+		fprintf (stderr,"\t<input.grd> is name of grid file to fit trend to.\n");
 		fprintf (stderr,"\t-N # model parameters to fit; integer in [1,10].  Append r for robust fit.\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
-		fprintf (stderr,"\t-D Supply filename to write grdfile of differences (input - trend).\n");
-		fprintf (stderr,"\t-T Supply filename to write grdfile of trend.\n");
+		fprintf (stderr,"\t-D Supply filename to write grid file of differences (input - trend).\n");
+		fprintf (stderr,"\t-T Supply filename to write grid file of trend.\n");
 		GMT_explain_option ('V');
-		fprintf (stderr,"\t-W Supply filename if you want to [read and] write grdfile of weights.\n");
+		fprintf (stderr,"\t-W Supply filename if you want to [read and] write grid file of weights.\n");
 		fprintf (stderr,"\t   If <weight.grd> can be read at run, and if robust = FALSE, weighted problem will be solved.\n");
 		fprintf (stderr,"\t   If robust = TRUE, weights used for robust fit will be written to <weight.grd>.\n");
 		GMT_explain_option ('.');
@@ -245,7 +246,7 @@ int main(int argc, char **argv)
 	GMT_err_fail (GMT_read_grd_info (i_filename, &head_d), i_filename);
 
 	GMT_grd_init (&head_d, argc, argv, TRUE);
-	nxy = head_d.nx * head_d.ny;
+	nxy = ((GMT_LONG)head_d.nx) * ((GMT_LONG)head_d.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nxy, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (i_filename, &head_d, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), i_filename);
 
@@ -372,6 +373,7 @@ int main(int argc, char **argv)
 	GMT_free ((void *)pstuff);
 	GMT_free ((void *)gtd);
 	GMT_free ((void *)gtg);
+	GMT_free ((void *)old);
 	GMT_free ((void *)yval);
 	GMT_free ((void *)xval);
 	GMT_free ((void *)resid);
@@ -430,7 +432,9 @@ void load_pstuff (double *pstuff, int n_model, double x, double y, int newx, int
 
 void compute_trend (float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff)
 {
-	int	i, j, k, ij;
+	int	i, j, k;
+	GMT_LONG	ij;
+	void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
 
 	for (ij = 0, j = 0; j < ny; j++) {
 		for (i = 0; i < nx; i++, ij++) {
@@ -443,9 +447,9 @@ void compute_trend (float *trend, int nx, int ny, double *xval, double *yval, do
 	}
 }
 
-void compute_resid (float *data, float *trend, float *resid, int nxy)
+void compute_resid (float *data, float *trend, float *resid, GMT_LONG nxy)
 {
-	int	i;
+	GMT_LONG	i;
 
 	for (i = 0; i < nxy; i++) {
 		if (GMT_is_fnan (data[i])) {
@@ -463,9 +467,10 @@ void grd_trivial_model (float *data, int nx, int ny, double *xval, double *yval,
 	/* Routine to fit up elementary polynomial model of grd data, 
 	model = gtd[0] + gtd[1]*x + gtd[2]*y + gtd[3] * x * y,
 	where x,y are normalized to range [-1,1] and there are no
-	NaNs in grdfile, and problem is unweighted least squares.  */
+	NaNs in grid file, and problem is unweighted least squares.  */
 
-	int	i, j, ij;
+	int	i, j;
+	GMT_LONG	ij;
 	double	x2, y2, sumx2 = 0.0, sumy2 = 0.0, sumx2y2 = 0.0;
 
 	/* First zero the model parameters to use for sums:  */
@@ -498,9 +503,9 @@ void grd_trivial_model (float *data, int nx, int ny, double *xval, double *yval,
 	return;
 }
 
-void compute_chisq (float *resid, float *weight, int nxy, double *chisq, double scale)
+void compute_chisq (float *resid, float *weight, GMT_LONG nxy, double *chisq, double scale)
 {
-	int	i;
+	GMT_LONG	i;
 	double	tmp;
 
 	*chisq = 0.0;
@@ -521,9 +526,9 @@ void compute_chisq (float *resid, float *weight, int nxy, double *chisq, double
 	return;
 }
 
-void compute_robust_weight (float *resid, float *weight, int nxy, double *scale)
+void compute_robust_weight (float *resid, float *weight, GMT_LONG nxy, double *scale)
 {
-	int	i, j, j2;
+	GMT_LONG	i, j, j2;
 	double	r, mad;
 
 	for (i = j = 0; i < nxy; i++) {
@@ -587,13 +592,15 @@ void load_gtg_and_gtd (float *data, int nx, int ny, double *xval, double *yval,
 {
 	/* Routine to load the matrix G'G (gtg) and vector G'd (gtd)
 	for the normal equations.  Routine uses indices i,j to refer
-	to the grdfile of data, and k,l to refer to the k_row, l_col
+	to the grid file of data, and k,l to refer to the k_row, l_col
 	of the normal equations matrix.  We need sums of [weighted]
 	data and model functions in gtg and gtd.  We save time by
 	loading only lower triangular part of gtg and then filling
 	by symmetry after i,j loop.  */
 
-	int	i, j, ij, k, l, n_used;
+	int	i, j, k, l, n_used;
+	GMT_LONG	ij;
+	void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
 
 /*	First zero things out to start:  */
 
@@ -798,7 +805,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
 void *New_Grdtrend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDTREND_CTRL *C;
 	
-	C = (struct GRDTREND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDTREND_CTRL), "New_Grdtrend_Ctrl");
+	C = (struct GRDTREND_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDTREND_CTRL), "New_Grdtrend_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -806,8 +813,8 @@ void *New_Grdtrend_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Grdtrend_Ctrl (struct GRDTREND_CTRL *C) {	/* Deallocate control structure */
-	if (C->D.file) GMT_free ((void *)C->D.file);	
-	if (C->T.file) GMT_free ((void *)C->T.file);	
-	if (C->W.file) GMT_free ((void *)C->W.file);	
+	if (C->D.file) free ((void *)C->D.file);	
+	if (C->T.file) free ((void *)C->T.file);	
+	if (C->W.file) free ((void *)C->W.file);	
 	GMT_free ((void *)C);	
 }
