@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvector.c,v 1.41 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdvector.c,v 1.48 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
-   grdvector reads 2 grdfiles that contains the 2 components of a vector
+   grdvector reads 2 grid files that contains the 2 components of a vector
    field (cartesian or polar) and plots vectors at the grid positions.
    This is basically a short-hand for using grd2xyz | psxy -SV and is
    more convenient for such plots on a grid.
@@ -78,8 +78,10 @@ struct GRDVECTOR_CTRL {
 
 int main (int argc, char **argv)
 {
-	int	i, j, n = 0, nm, nx, ny, ij, i0, j0, di, dj;
+	int	i, j, n = 0, nx, ny, i0, j0, di, dj;
 
+	GMT_LONG ij, nm;
+	
 	BOOLEAN shrink_properties = FALSE, error = FALSE;
 
 	char *file[2], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT];
@@ -157,7 +159,16 @@ int main (int argc, char **argv)
 					break;
 				case 'Q':
 					Ctrl->Q.active = TRUE;
-					if (argv[i][2] && argv[i][3] != 'n') {
+					for (j = 2; argv[i][j] && argv[i][j] != 'n'; j++);
+					if (argv[i][j]) {	/* Normalize option used */
+						Ctrl->Q.norm = GMT_convert_units (&argv[i][j+1], GMT_INCH);
+						if (Ctrl->Q.norm <= 0.0) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -Qn option:  No reference length given\n", GMT_program);
+							error++;
+						}
+						argv[i][j] = '\0';	/* Temporarily chop of the n<norm> string */
+					}
+					if (argv[i][2] && argv[i][3] != 'n') {	/* We specified the three parameters */
 						if (sscanf (&argv[i][2], "%[^/]/%[^/]/%s", txt_a, txt_b, txt_c) != 3) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -Q option:  Could not decode arrowwidth/headlength/headwidth\n", GMT_program);
 							error++;
@@ -168,14 +179,7 @@ int main (int argc, char **argv)
 							Ctrl->Q.width  = GMT_convert_units (txt_c, GMT_INCH);
 						}
 					}
-					for (j = 2; argv[i][j] && argv[i][j] != 'n'; j++);
-					if (argv[i][j]) {	/* Normalize option used */
-						Ctrl->Q.norm = atof (&argv[i][j+1]);
-						if (Ctrl->Q.norm <= 0.0) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -Qn option:  No reference length given\n", GMT_program);
-							error++;
-						}
-					}
+					if (Ctrl->Q.norm > 0.0) argv[i][j] = 'n';	/* Restore the n<norm> string */
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
@@ -219,17 +223,17 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grdvector %s - Plot vector fields from grdfiles\n\n", GMT_VERSION);
+		fprintf (stderr, "grdvector %s - Plot vector fields from grid files\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdvector compx.grd compy.grd %s %s [-A]\n", GMT_J_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[%s] [-C<cpt>] [-E] [-G<fill>] [-I<dx/dy>] [-K] [-N] [-O] [-P] [-Q<params>] [-S[l]<scale>[<unit>]]\n", GMT_B_OPT);
 		fprintf (stderr, "\t[-T] [%s] [-V] [-W<pen>] [%s] [%s] [-Z] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\tcompx & compy are grdfiles with the 2 vector components.\n");
+		fprintf (stderr, "\tcompx & compy are grid files with the 2 vector components.\n");
 		GMT_explain_option ('j');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A means grdfiles have polar (r, theta) components [Default is Cartesian]\n");
+		fprintf (stderr, "\t-A means grids have polar (r, theta) components [Default is Cartesian]\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign colors based on vector length\n");
 		fprintf (stderr, "\t-E cEnter vectors on grid nodes [Default draws from grid node]\n");
@@ -242,7 +246,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Q Select vector plot [Default is stick-plot].\n");
 		fprintf (stderr, "\t   Optionally, specify vector parameters\n");
 		fprintf (stderr, "\t   <params> are arrowwidth/headlength/headwidth [Default is 0.03i/0.12i/0.09i]\n");
-		fprintf (stderr, "\t   Append n<size> which will cause vectors shorter than <size> to be\n");
+		fprintf (stderr, "\t   Append n<size>[unit] which will cause vectors shorter than <size> to be\n");
 		fprintf (stderr, "\t     scaled down\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S sets scale for vector length in data units per %s [1]\n", GMT_unit_names[gmtdefs.measure_unit]);
@@ -295,7 +299,7 @@ int main (int argc, char **argv)
 		error++;
 	}
 	if (n != 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify two input grdfiles\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify two input grid files\n", GMT_program);
 		error++;
 	}
 
@@ -304,7 +308,7 @@ int main (int argc, char **argv)
 	if (Ctrl->C.active) GMT_read_cpt (Ctrl->C.file);
 
 	if (!(strcmp (file[0], "=") || strcmp (file[1], "="))) {
-		fprintf (stderr, "%s: Piping of grdfiles not supported!\n", GMT_program);
+		fprintf (stderr, "%s: Piping of grid files not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
@@ -328,20 +332,21 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	/* Determine the wesn to be used to read the grdfile */
+	/* Determine the wesn to be used to read the grid file */
 
 	if (GMT_grd_setregion (&h[0], &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	/* Read data */
 
 	nx = GMT_get_n (data_west, data_east, h[0].x_inc, h[0].node_offset);
 	ny = GMT_get_n (data_south, data_north, h[0].y_inc, h[0].node_offset);
-	nm = nx * ny;
+	nm = ((GMT_LONG)nx) * ((GMT_LONG)ny);
 	r = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	theta = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (file[0], &h[0], r, data_west, data_east, data_south, data_north, GMT_pad, FALSE), file[0]);
@@ -399,7 +404,7 @@ int main (int argc, char **argv)
 		y = GMT_j_to_y (j, h[0].y_min, h[0].y_max, h[0].y_inc, h[0].xy_off, h[0].ny);
 		for (i = i0; i < h[1].nx; i += di) {
 
-			ij = j * h[0].nx + i;
+			ij = GMT_IJ (j, i, h[0].nx);
 			if (GMT_is_fnan (r[ij]) || GMT_is_fnan (theta[ij])) continue;	/* Cannot plot NaN-vectors */
 
 			value = r[ij];
@@ -483,7 +488,7 @@ int main (int argc, char **argv)
 void *New_Grdvector_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVECTOR_CTRL *C;
 	
-	C = (struct GRDVECTOR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVECTOR_CTRL), "New_Grdvector_Ctrl");
+	C = (struct GRDVECTOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDVECTOR_CTRL), "New_Grdvector_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	GMT_init_fill (&C->G.fill, -1, -1, -1);
@@ -496,6 +501,6 @@ void *New_Grdvector_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Grdvector_Ctrl (struct GRDVECTOR_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
