@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdview.c,v 1.68 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: grdview.c,v 1.93 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
  * contour lines. This allows the user to specify that the contours
  * should be smoothed before plotting. This will make the resulting
  * image smoother, especially if nx and ny are relatively small.
- * As an option, a drape grdfile can be specified.  Then, the colors
+ * As an option, a drape grid file can be specified.  Then, the colors
  * are calculated from that file while the topo file is used for shape.
  * Alternatively, give three drape files (red, green, blue components)
  * to bypass the z -> rgb via the cpt file.
@@ -66,9 +66,9 @@ struct GRDVIEW_CTRL {
 	} I;
 	struct L {	/* -L<flag> */
 		BOOLEAN active;
-		BOOLEAN bilinear;
+		BOOLEAN interpolant;
 		char mode[4];
-		double value;
+		double threshold;
 	} L;
 	struct N {	/* -N<level>[/<color>] */
 		BOOLEAN active;
@@ -77,7 +77,7 @@ struct GRDVIEW_CTRL {
 		double level;
 	} N;
 	struct Q {	/* -Q<type>[g] */
-		BOOLEAN active;
+		BOOLEAN active, special;
 		BOOLEAN outline;
 		BOOLEAN mask;
 		BOOLEAN monochrome;
@@ -98,7 +98,7 @@ struct GRDVIEW_CTRL {
 	struct W {	/* -W[+]<type><pen> */
 		BOOLEAN active;
 		BOOLEAN contour;
-		struct GMT_PEN pen[2];
+		struct GMT_PEN pen[3];
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
@@ -129,11 +129,13 @@ int main (int argc, char **argv)
 	char *topofile;
 	char *c_method[2] = {"colorimage", "colortiles",};
 
-	int 	i, j, ij, n_edges, k, k1, n, max, i_bin, j_bin, i_bin_old, j_bin_old;
-	int	side, way, nm, nm2, nx_f, ny_f, nx, ny, n_drape = 0, nm_use;
-	int	bin, two, mx, my, sw, se, nw, ne, id, n4, nk, c, rgb[3], i_start, i_stop;
+	int 	j, n_edges, max, i_bin, j_bin, i_bin_old, j_bin_old;
+	int	side, way, nx_f, ny_f, nx, ny;
+	int	two, sw, se, nw, ne, id, n4, nk, c, rgb[3], i_start, i_stop;
 	int	j_start, j_stop, i_inc, j_inc, q_set = 0, n_commas, ii, jj, n_out;
-	int	PS_colormask_off = 0, PS_colormask,*edge, bin_inc[4], ij_inc[4];
+	int	PS_colormask_off = 0, PS_colormask,*edge;
+
+	GMT_LONG bin, i, ij, k, k1, n, nm, nm2, nm_use, mx, my, n_drape = 0, bin_inc[4], ij_inc[4];
 
 	float *grd[3], *zgrd, *intensity = NULL, *topo;
 
@@ -150,16 +152,16 @@ int main (int argc, char **argv)
 	struct GRDVIEW_BIN *binij = NULL;
 	struct GRDVIEW_CTRL *Ctrl;
 
-	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, int *n);
-	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, int *n);
-	double get_z_ave (double v[], double next_up, int n);
-	void add_node (double x[], double y[], double z[], double v[], int *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], int ij, int bin);
-	void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity);
-	double get_intensity (float *intensity, int k, int nx);
-	struct GRDVIEW_CONT *get_cont_struct (int bin, struct GRDVIEW_BIN *binij, double value);
+	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
+	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
+	double get_z_ave (double v[], double next_up, GMT_LONG n);
+	void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin);
+	void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity);
+	double get_intensity (float *intensity, GMT_LONG k, int nx);
+	struct GRDVIEW_CONT *get_cont_struct (GMT_LONG bin, struct GRDVIEW_BIN *binij, double value);
 	struct GRDVIEW_POINT *get_point (double x, double y);
 	void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level);
-	int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[]);
+	int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[]);
 	int quick_idist(int x1, int y1, int x2, int y2);
 	int get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2);
 	BOOLEAN GMT_rgb_is_nan_rgb (int rgb[]);
@@ -232,10 +234,10 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						Ctrl->L.active = TRUE;
-						strncpy (Ctrl->L.mode, &argv[i][2], 4);
+						strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					}
 					else
-						Ctrl->L.bilinear = TRUE;
+						Ctrl->L.interpolant = BCR_BILINEAR;
 					break;
 				case 'N':
 					if (argv[i][2]) {
@@ -262,7 +264,7 @@ int main (int argc, char **argv)
 						case 'm':	/* Mesh plot */
 							Ctrl->Q.mode = GRDVIEW_MESH;
 							if (argv[i][3] == '/' && GMT_getfill (&argv[i][4], &Ctrl->Q.fill)) {
-								fprintf (stderr, "%s: GMT SYNTAX ERROR -Qm option: To give mesh color, use -Qm<color>\n", GMT_program);
+								fprintf (stderr, "%s: GMT SYNTAX ERROR -Qm option: To give mesh color, use -Qm/<color>\n", GMT_program);
 								error = TRUE;
 							}
 							break;
@@ -270,6 +272,8 @@ int main (int argc, char **argv)
 							Ctrl->Q.mode = GRDVIEW_SURF;
 							if (argv[i][3] == 'm') Ctrl->Q.outline = TRUE;
 							break;
+						case 't':	/* texture image */
+							Ctrl->Q.special = TRUE;
 						case 'i':	/* image w/ clipmask */
 						case 'I':	/* Backward compatibility, gives -Qi */
 							Ctrl->Q.mode = GRDVIEW_IMAGE;
@@ -277,6 +281,7 @@ int main (int argc, char **argv)
 							break;
 						case 'c':	/* image w/ colormask */
 							if (argv[i][3] && isdigit ((int)argv[i][3])) Ctrl->Q.dpi = atoi (&argv[i][3]);
+							Ctrl->Q.mode = GRDVIEW_IMAGE;
 							Ctrl->Q.mask = TRUE;
 							break;
 						default:
@@ -303,31 +308,31 @@ int main (int argc, char **argv)
 						}
 					}
 					break;
-				case 'W':	/* Contour and/or mesh pens */
+				case 'W':	/* Contour, mesh, or facade pens */
 					Ctrl->W.active = TRUE;
-					j = (argv[i][2] == 'm' || argv[i][2] == 'c') ? 3 : 2;
-                                        if (j == 2) {   /* Set both */
+					j = (argv[i][2] == 'm' || argv[i][2] == 'c' || argv[i][2] == 'f') ? 3 : 2;
+					if (j == 2) {   /* Set all */
 						if (GMT_getpen (&argv[i][2], &Ctrl->W.pen[0])) {
 							GMT_pen_syntax ('W', " ");
 							error++;
 						}
 						else
-							Ctrl->W.pen[1] = Ctrl->W.pen[0];
+							Ctrl->W.pen[2] = Ctrl->W.pen[1] = Ctrl->W.pen[0];
 						Ctrl->W.contour = TRUE;
 					}
 					else {	/* Here , j = 3.  BUT: First check that the m or c is not part of a color name instead */
 						char txt_a[GMT_LONG_TEXT];
 						n = j+1;
 						while (!(argv[i][n] == ',' || argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-						strncpy (txt_a, &argv[i][2], n-2);	txt_a[n-2] = '\0';
+						strncpy (txt_a, &argv[i][2], (size_t)(n-2));	txt_a[n-2] = '\0';
 						if (GMT_colorname2index (txt_a) >= 0) j = 2;	/* Found a colorname; reset j to 2 */
-						id = (argv[i][2] == 'm') ? 1 : 0;
+						id = (argv[i][2] == 'f') ? 2 : ((argv[i][2] == 'm') ? 1 : 0);
 						if (GMT_getpen (&argv[i][j], &Ctrl->W.pen[id])) {
 							GMT_pen_syntax ('W', " ");
 							error++;
 						}
 						if (j == 2) {	/* We discovered it wasnt -Wc or -Wm after all so copy pen */
-							Ctrl->W.pen[1] = Ctrl->W.pen[0];
+							Ctrl->W.pen[2] = Ctrl->W.pen[1] = Ctrl->W.pen[0];
 							Ctrl->W.contour = TRUE;
 						}
 						else if (id == 0)
@@ -375,7 +380,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-G <drapefile> rather than <topofile> is the data set to color-code.\n");
 		fprintf (stderr, "\t   Use <topofile> as the relief and \'drape\' the image on top.\n");
 		fprintf (stderr, "\t   Note that -Jz and -N always refers to the <topofile>\n");
-		fprintf (stderr, "\t   Alternatively, give three grdfiles with the red, green, and blue components in 0-255 range.\n");
+		fprintf (stderr, "\t   Alternatively, give three grid files with the red, green, and blue components in 0-255 range.\n");
 		fprintf (stderr, "\t   If so you must also choose -Qi.\n");
 		fprintf (stderr, "\t-I gives name of intensity file and selects illumination\n");
 		fprintf (stderr, "\t-L sets boundary conditions when resampling the grid.  <flags> can be either\n");
@@ -391,7 +396,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q sets plot reQuest. Choose one of the following:\n");
-		fprintf (stderr, "\t   -Qm for Mesh plot [Default].  Append color for mesh paint [%d/%d/%d]\n", Ctrl->Q.fill.rgb[0], Ctrl->Q.fill.rgb[1], Ctrl->Q.fill.rgb[2]);
+		fprintf (stderr, "\t   -Qm for Mesh plot [Default].  Append /color for mesh paint [%d/%d/%d]\n", Ctrl->Q.fill.rgb[0], Ctrl->Q.fill.rgb[1], Ctrl->Q.fill.rgb[2]);
 		fprintf (stderr, "\t   -Qs[m] for colored or shaded Surface.  Append m to draw meshlines on the surface.\n");
 		fprintf (stderr, "\t   -Qi for scanline converting polygons to rasterimage.  Append effective dpi [100].\n");
 		fprintf (stderr, "\t   -Qc. As -Qi but use PS Level 3 colormasking for nodes with z = NaN.  Append effective dpi [100].\n");
@@ -399,17 +404,21 @@ int main (int argc, char **argv)
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S will smooth contours first (see grdcontour for <smooth> value info).\n");
 		GMT_pen_syntax ('T', "will image the data without interpolation by painting polygonal tiles\n\t   Append s to skip tiles for nodes with z = NaN [Default paints all tiles]\n\t   Append o[<pen>] to draw tile outline [Default uses no outline]");
+		fprintf (stderr, "\t   Cannot be used with -Jz|Z as it produces a flat image.\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		GMT_pen_syntax ('W', "sets pen attributes for contours in form <type><pen> [Default is no contours]");
-		fprintf (stderr, "\t   <type> can be c for contours or m for mesh\n");
+		GMT_pen_syntax ('W', "sets pen attributes for various features in form <type><pen>");
+		fprintf (stderr, "\t   <type> can be c for contours, m for mesh, and f for facade\n");
 		fprintf (stderr, "\t   c draw scontours on top of surface or mesh.  [Default is no contours]\n");
 		fprintf (stderr, "\t     Optionally append pen attributes [width = %gp, color = (%d/%d/%d), solid line].\n", 
 			Ctrl->W.pen[0].width, Ctrl->W.pen[0].rgb[0], Ctrl->W.pen[0].rgb[1], Ctrl->W.pen[0].rgb[2]);
 		fprintf (stderr, "\t   m sets attributes for mesh lines [[width = %gp, color = (%d/%d/%d), solid line].\n", 
 			Ctrl->W.pen[1].width, Ctrl->W.pen[1].rgb[0], Ctrl->W.pen[1].rgb[1], Ctrl->W.pen[1].rgb[2]);
 		fprintf (stderr, "\t     Requires -Qm or -Qsm to take effect.\n");
+		fprintf (stderr, "\t   f sets attributes for facade outline [[width = %gp, color = (%d/%d/%d), solid line].\n", 
+			Ctrl->W.pen[2].width, Ctrl->W.pen[2].rgb[0], Ctrl->W.pen[2].rgb[1], Ctrl->W.pen[2].rgb[2]);
+		fprintf (stderr, "\t     Requires -N to take effect.\n");
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
 		GMT_explain_option ('c');
@@ -454,6 +463,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Qi option:  Must specify positive dpi\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->T.active && project_info.JZ_set) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Cannot specify -JZ|z\n", GMT_program);
+		error++;
+	}
 	if (Ctrl->S.value < 0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  smooth value must be positive\n", GMT_program);
 		error++;
@@ -470,7 +483,12 @@ int main (int argc, char **argv)
 		GMT_read_cpt (Ctrl->C.file);
 		if (GMT_b_and_w) Ctrl->Q.monochrome = TRUE;
 	}
-
+#ifdef GMT_CPT2	
+	if (GMT_categorical && Ctrl->W.active) {
+		fprintf (stderr, "%s: GMT WARNING -W:  Categorical data (as implied by CPT file) does not have contours.  Check plot.\n", GMT_program);
+		exit (EXIT_FAILURE);
+	}
+#endif	
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	get_contours = ( (Ctrl->Q.mode == GRDVIEW_MESH && Ctrl->W.contour) || (Ctrl->Q.mode == GRDVIEW_SURF && GMT_n_colors > 1) );
@@ -513,7 +531,7 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	/* Determine the wesn to be used to read the grdfile */
+	/* Determine the wesn to be used to read the grid file */
 
 	if (GMT_grd_setregion (&t_head, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
@@ -527,13 +545,14 @@ int main (int argc, char **argv)
 			GMT_map_basemap ();
 		GMT_plotend ();
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	/* Read data */
 
 	if (subset) {	/* Possibly extend the subset by 1 row/col since tiles need 4 corners to work */
 		BOOLEAN global;
-		global = (GMT_io.in_col_type[0] == GMT_IS_LON && fabs (fabs (t_head.x_max - t_head.x_min) - 360.0) < GMT_CONV_LIMIT);
+		global = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (t_head.x_max, t_head.x_min));
 		if (global && data_west > project_info.w)
 			data_west -= t_head.x_inc;
 		else if (GMT_x_to_i (data_west, t_head.x_min, t_head.x_inc, t_head.xy_off, t_head.nx) > 0)
@@ -569,8 +588,8 @@ int main (int argc, char **argv)
 
 	xval = (double *) GMT_memory (VNULL, (size_t)t_head.nx, sizeof (double), GMT_program);
 	yval = (double *) GMT_memory (VNULL, (size_t)t_head.ny, sizeof (double), GMT_program);
-	for (i = 0; i < t_head.nx; i++) xval[i] = t_head.x_min + i * t_head.x_inc + delx;
-	for (j = 0; j < t_head.ny; j++) yval[j] = t_head.y_max - j * t_head.y_inc - dely;
+	for (i = 0; i < t_head.nx; i++) xval[i] = GMT_i_to_x (i, t_head.x_min, t_head.x_max, t_head.x_inc, t_head.xy_off, t_head.nx);
+	for (j = 0; j < t_head.ny; j++) yval[j] = GMT_j_to_y (j, t_head.y_min, t_head.y_max, t_head.y_inc, t_head.xy_off, t_head.ny);
 
 	GMT_boundcond_param_prep (&t_head, &edgeinfo);
 
@@ -704,13 +723,13 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: Intensity file has improper dimensions!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		if (drape_resample) GMT_bcr_init (&i_head, GMT_pad, Ctrl->L.bilinear, Ctrl->L.value, &i_bcr);
+		if (drape_resample) GMT_bcr_init (&i_head, GMT_pad, Ctrl->L.interpolant, Ctrl->L.threshold, &i_bcr);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
 	}
 
 	if (two) {	/* Initialize bcr stuff */
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = two;
-		GMT_bcr_init (&t_head, GMT_pad, Ctrl->L.bilinear, Ctrl->L.value, &t_bcr);
+		GMT_bcr_init (&t_head, GMT_pad, Ctrl->L.interpolant, Ctrl->L.threshold, &t_bcr);
 
 		/* Set boundary conditions  */
 
@@ -726,7 +745,7 @@ int main (int argc, char **argv)
 		for (i = 0; i < t_head.nx - 1; i++, bin++, ij++) {
 			for (jj = n_out = 0; jj < 2; jj++) {	/* Loop over the 4 nodes making up one tile */
 				for (ii = 0; ii < 2; ii++) {
-					bin = (j + jj) * t_head.nx + (i + ii);
+					bin = GMT_IJ (j + jj, i + ii, t_head.nx);
 					ij = (two) ? ((j + jj) + 2) * mx + (i + ii) + two : bin;
 					if (GMT_is_fnan (topo[ij])) no_nans = FALSE;
 					if (GMT_map_outside (xval[i+ii], yval[j+jj])) n_out++;
@@ -760,48 +779,49 @@ int main (int argc, char **argv)
 	yy = (double *) GMT_memory (VNULL, (size_t)max, sizeof(double), GMT_program);
 	if (Ctrl->N.active) {
 		ps_comment ("Plot the plane at desired level");
+		GMT_setpen (&Ctrl->W.pen[2]);
 		if (!z_project.draw[0])	{	/* Southern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_min + dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[2])	{	/* Northern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_max - dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[3])	{	/* Western side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_min + delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[1])	{	/* Eastern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_max - delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
 			}
 		}
 
@@ -829,9 +849,10 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (Ctrl->T.active) {	/* Plot image as polygonal pieces */
-		int p;
-		double *xx, *yy, xpos, ypos;
+	if (Ctrl->T.active) {	/* Plot image as polygonal pieces. Here, -JZ is not set */
+		double *xx, *yy;
+		struct GMT_FILL fill;
+		GMT_init_fill (&fill, -1, -1, -1);	/* Initialize fill structure */
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling without interpolation\n", GMT_program);
 
@@ -839,28 +860,23 @@ int main (int argc, char **argv)
 		for (j = k = 0; j < header.ny; j++) {
 			for (i = 0; i < header.nx; i++, k++) {	/* Compute rgb for each pixel */
 				if (GMT_is_fnan (topo[k]) && Ctrl->T.skip) continue;
-				GMT_get_rgb_from_z (topo[k], rgb);
-				if (Ctrl->I.active) GMT_illuminate (intensity[k], rgb);
-
+				GMT_get_rgb_from_z (topo[k], fill.rgb);
+				if (Ctrl->I.active) GMT_illuminate (intensity[k], fill.rgb);
 				n = GMT_graticule_path (&xx, &yy, 1, xval[i] - dx2, xval[i] + dx2, yval[j] - dy2, yval[j] + dy2);
-
-				/* Do map projection and plot */
-
-				for (p = 0; p < n; p++) {
-					GMT_geoz_to_xy (xx[p], yy[p], topo[k], &xpos, &ypos);
-					xx[p] = xpos;	yy[p] = ypos;
-				}
-				ps_polygon (xx, yy, n, rgb, Ctrl->T.outline);
+				GMT_fill_polygon (xx, yy, 0.0, n, &fill, Ctrl->T.outline);
 				GMT_free ((void *)xx);
 				GMT_free ((void *)yy);
 			}
 		}
+		GMT_free ((void *) xval);
+		GMT_free ((void *) yval);
 	}
+	
 	if (Ctrl->Q.mode == GRDVIEW_IMAGE) {	/* compute image */
-		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, *ix, *iy;
+		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, last_i, last_j, *ix, *iy;
 		int *top_jp = VNULL, *bottom_jp = VNULL, p;
 		BOOLEAN done;
-		double xp, yp, sum_w, w, sum_i, x_width, y_width, value, i_dpi_i;
+		double xp, yp, sum_w, w, sum_i, x_width, y_width, value;
 		double sum_r, sum_g, sum_b, intval = 0.0, y_drape, *x_drape;
 		float *int_drape = VNULL;
 		unsigned char *bitimage_24 = (unsigned char *)NULL, *bitimage_8 = (unsigned char *)NULL;
@@ -876,7 +892,7 @@ int main (int argc, char **argv)
 		y_width = z_project.ymax - z_project.ymin;
 		nx_i = irint (x_width * Ctrl->Q.dpi);	/* Size of image in pixels */
 		ny_i = irint (y_width * Ctrl->Q.dpi);
-		i_dpi_i = 1.0 / Ctrl->Q.dpi;
+		last_i = nx_i - 1;	last_j = ny_i - 1;
 		if (drape_resample) {
 			nm_drape = d_head[0].nx * d_head[0].ny;
 			ix = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
@@ -893,8 +909,9 @@ int main (int argc, char **argv)
 					}
 					else {
 						GMT_geoz_to_xy (x_drape[i], y_drape, value, &xp, &yp);
-						ix[bin] = (int)floor ((xp - z_project.xmin) * Ctrl->Q.dpi);
-						iy[bin] = (int)floor ((yp - z_project.ymin) * Ctrl->Q.dpi);
+						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
+						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 					if (Ctrl->I.active) int_drape[bin] = (float)GMT_get_bcr_z(&i_head, x_drape[i], y_drape, intensity, &edgeinfo, &i_bcr);
 				}
@@ -916,8 +933,9 @@ int main (int argc, char **argv)
 					}
 					else {
 						GMT_geoz_to_xy (xval[i], yval[j], (double)topo[ij], &xp, &yp);
-						ix[bin] = (int)floor ((xp - z_project.xmin) * Ctrl->Q.dpi);
-						iy[bin] = (int)floor ((yp - z_project.ymin) * Ctrl->Q.dpi);
+						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
+						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 				}
 			}
@@ -964,7 +982,7 @@ int main (int argc, char **argv)
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Start rasterization\n", GMT_program);
 		for (j = j_start; j != j_stop; j += j_inc) {
 
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.4d\r", GMT_program, j);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.6d\r", GMT_program, j);
 
 			for (i = i_start; i != i_stop; i += i_inc) {
 				bin = j * header.nx + i;
@@ -1029,6 +1047,7 @@ int main (int argc, char **argv)
 							rgb[2] = irint (sum_b * sum_w);
 							if (Ctrl->I.active) intval = sum_i * sum_w;
 						}
+						if (Ctrl->Q.special) GMT_get_rgb_from_z (zgrd[bin], rgb);
 						if (Ctrl->I.active && bad < 4) GMT_illuminate (intval, rgb);
 						kk = layers * ((ny_i-jp-1) * nx_i + ip) + PS_colormask_off;
 						if (Ctrl->Q.monochrome) /* GMT_YIQ transformation */
@@ -1065,7 +1084,7 @@ int main (int argc, char **argv)
 					y_imask[nk-k] = y_imask[nk-k1] = z_project.ymin + bottom_jp[ip] * y_pixel_size;
 				}
 			}
-			ps_clipon (x_imask, y_imask, 4 * nx_i, GMT_no_rgb, 3);
+			ps_clipon (x_imask, y_imask, (GMT_LONG)(4 * nx_i), GMT_no_rgb, 3);
 			GMT_free ((void *)x_imask);
 			GMT_free ((void *)y_imask);
 		}
@@ -1110,7 +1129,7 @@ int main (int argc, char **argv)
 				GMT_geoz_to_xy (x_right, y_bottom, (double)(topo[ij+ij_inc[1]]), &xx[1], &yy[1]);
 				GMT_geoz_to_xy (x_right, y_top, (double)(topo[ij+ij_inc[2]]), &xx[2], &yy[2]);
 				GMT_geoz_to_xy (x_left, y_top, (double)(topo[ij+ij_inc[3]]), &xx[3], &yy[3]);
-				ps_patch (xx, yy, 4, Ctrl->Q.fill.rgb, TRUE);
+				ps_patch (xx, yy, (GMT_LONG)4, Ctrl->Q.fill.rgb, TRUE);
 				if (Ctrl->W.contour) {
 					pen_not_set = TRUE;
 					if (binij[bin].first_cont == NULL) continue;
@@ -1125,7 +1144,7 @@ int main (int argc, char **argv)
 							GMT_setpen (&Ctrl->W.pen[0]);
 							pen_not_set = FALSE;
 						}
-						ps_line (xx, yy, k, 3, FALSE, TRUE);
+						ps_line (xx, yy, (GMT_LONG)k, 3, FALSE, TRUE);
 					}
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 				}
@@ -1162,8 +1181,8 @@ int main (int argc, char **argv)
 					X_vert[0] = X_vert[3] = x_left;	X_vert[1] = X_vert[2] = x_right;
 					Y_vert[0] = Y_vert[1] = y_bottom;	Y_vert[2] = Y_vert[3] = y_top;
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], 0.0, &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, 4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0);
+					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
 					continue;
 				}
 
@@ -1427,7 +1446,7 @@ int main (int argc, char **argv)
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 					if (Ctrl->Q.outline) {
 						for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
-						ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+						ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
 					}
 				}
 				else {	/* No Contours, paint tile (any of the corner values will give correct color */
@@ -1441,8 +1460,8 @@ int main (int argc, char **argv)
 					/* Now paint the polygon piece */
 
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, 4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
 				}
 			}
 		}
@@ -1460,7 +1479,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->N.facade) {	/* Cover the two front sides */
 		ps_comment ("Painting the frontal facade");
-		GMT_setpen (&Ctrl->W.pen[0]);
+		GMT_setpen (&Ctrl->W.pen[2]);
 		if (!z_project.draw[0])	{	/* Southern side */
 			for (i = n = 0, ij = sw; i < header.nx; i++, ij++) {
 				if (GMT_is_fnan (topo[ij])) continue;
@@ -1547,7 +1566,7 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-struct GRDVIEW_CONT *get_cont_struct (int bin, struct GRDVIEW_BIN *binij, double value)
+struct GRDVIEW_CONT *get_cont_struct (GMT_LONG bin, struct GRDVIEW_BIN *binij, double value)
 {
 	struct GRDVIEW_CONT *cont, *new_cont;
 
@@ -1577,7 +1596,8 @@ struct GRDVIEW_POINT *get_point (double x, double y)
 
 void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level)
 {
-	int i, j, ij;
+	int i, j;
+	GMT_LONG ij;
 	double xtmp, ytmp, tmp, delx, dely;
 
 	delx = (header->node_offset) ? 0.5 * header->x_inc :0.0;
@@ -1587,7 +1607,7 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 	z_project.ymax = z_project.ymin;	/* Reset from whatever it was */
 
 	for (j = 0; j < header->ny; j++) {
-		ij = (j + two) * header->nx + two;
+		ij = GMT_IJ (j + two, two, header->nx);
 		for (i = 0; i < header->nx; i++, ij++) {
 			if (GMT_is_fnan (topo[ij])) continue;
 			GMT_geoz_to_xy (header->x_min + i * header->x_inc, header->y_max - j * header->y_inc, (double)topo[ij], &xtmp, &ytmp);
@@ -1617,13 +1637,13 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 	}
 }
 
-double get_intensity (float *intensity, int k, int nx)
+double get_intensity (float *intensity, GMT_LONG k, int nx)
 {
 	/* Finds the average intensity for this polygon */
 	return (0.25 * (intensity[k] + intensity[k+1] + intensity[k-nx] + intensity[k-nx+1]));
 }
 
-int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[])
+int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[])
 {
 	int i, what;
 	double x[6], y[6];
@@ -1633,7 +1653,7 @@ int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[])
 		y[i] = iy[bin+bin_inc[i]];
 	}
 	x[4] = x[0];	y[4] = y[0];
-	what = GMT_non_zero_winding ((double)ip, (double)jp, x, y, 5);
+	what = GMT_non_zero_winding ((double)ip, (double)jp, x, y, (GMT_LONG)5);
 	return (what);
 }
 
@@ -1661,7 +1681,7 @@ int get_side (double x, double y, double x_left, double y_bottom, double xinc, d
 	return (side);
 }
 
-void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, int *n) {
+void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
 	int k;
 	for (k = 0; k < ncont; k++, (*n)++) {
 		x[*n] = xcont[k];
@@ -1671,7 +1691,7 @@ void copy_points_fw (double x[], double y[], double z[], double v[], double xcon
 	}
 }
 
-void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, int *n) {
+void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
 	int k;
 	for (k = ncont - 1; k >= 0; k--, (*n)++) {
 		x[*n] = xcont[k];
@@ -1681,15 +1701,15 @@ void copy_points_bw (double x[], double y[], double z[], double v[], double xcon
 	}
 }
 
-double get_z_ave (double v[], double next_up, int n) {
-	int k;
+double get_z_ave (double v[], double next_up, GMT_LONG n) {
+	GMT_LONG k;
 	double z_ave;
 
 	for (k = 0, z_ave = 0.0; k < n; k++) z_ave += MIN (v[k], next_up);
 	return (z_ave / n);
 }
 
-void add_node (double x[], double y[], double z[], double v[], int *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], int ij, int bin) {
+void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin) {
 	/* Adds a corner node to list of points and increments counter */
 	x[*k] = X_vert[node];
 	y[*k] = Y_vert[node];
@@ -1698,7 +1718,7 @@ void add_node (double x[], double y[], double z[], double v[], int *k, int node,
 	(*k)++;
 }
 
-void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity) {
+void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity) {
 	int index, rgb[3];
 	struct GMT_FILL *f;
 
@@ -1727,7 +1747,7 @@ BOOLEAN GMT_rgb_is_nan_rgb (int rgb[])
 void *New_Grdview_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVIEW_CTRL *C;
 	
-	C = (struct GRDVIEW_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVIEW_CTRL), "New_Grdview_Ctrl");
+	C = (struct GRDVIEW_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDVIEW_CTRL), "New_Grdview_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -1735,14 +1755,18 @@ void *New_Grdview_Ctrl () {	/* Allocate and initialize a new control structure *
 	GMT_init_pen (&C->T.pen, GMT_PENWIDTH);
 	GMT_init_pen (&C->W.pen[0], 3.0 * GMT_PENWIDTH);	/* Contour pen */
 	GMT_init_pen (&C->W.pen[1], GMT_PENWIDTH);	/* Mesh pen */
+	GMT_init_pen (&C->W.pen[2], 3.0 * GMT_PENWIDTH);	/* Facade pen */
 	C->Q.dpi = 100;
 	GMT_init_fill (&C->Q.fill, 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0]);
-	C->L.value = 1.0;
+	C->L.interpolant = BCR_BICUBIC; C->L.threshold = 1.0;
 	C->S.value = 1;
 	return ((void *)C);
 }
 
 void Free_Grdview_Ctrl (struct GRDVIEW_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+	int i;
+	if (C->C.file) free ((void *)C->C.file);	
+	for (i = 0; i < 3; i++) if (C->G.file[i]) free ((void *)C->G.file[i]);	
+	if (C->I.file) free ((void *)C->I.file);	
 	GMT_free ((void *)C);	
 }
