@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.33 2007/03/12 19:52:26 remko Exp $
+ *	$Id: grdvolume.c,v 1.41 2008/04/25 04:00:41 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdvolume reads a 2d binary gridded grdfile, and calculates the volume
+ * grdvolume reads a 2d binary gridded grid file, and calculates the volume
  * under the surface using exact integration of the bilinear interpolating
  * surface.  As an option, the user may supply a contour value; then the
  * volume is only integrated inside the chosen contour.
@@ -63,12 +63,11 @@ int main (int argc, char **argv)
 	int i, j, n = 0, c, k, pos, neg, nc, n_contours, mode = 0, nz = 0; 
 
 	size_t ij, nm, nx, ny;
-	
-	long ij_inc[4];
+	GMT_LONG ij_inc[4];
 	
 	float *f;
 
-	double take_out, west, east, south, north, dv, da, cval = 0.0, cellsize, fact, m_pr_deg;
+	double take_out, west, east, south, north, dv, da, cval = 0.0, cellsize, fact, dist_pr_deg;
 	double *area, *vol, *height, this_base, small;
 
 	struct GRDVOLUME_CTRL *Ctrl;
@@ -100,6 +99,7 @@ int main (int argc, char **argv)
 					if (argv[i][2] == 'L' || argv[i][2] == 'l') full = TRUE;
 				case 'R':
 				case ':':
+				case 'f':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -150,7 +150,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdvolume %s - Calculating volume under a surface within a contour\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T]\n\t[%s] [-V] [-Z<fact>[/<shift>]]\n", GMT_Rgeo_OPT);
+		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T]\n\t[%s] [-V] [-Z<fact>[/<shift>]] [%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -166,12 +166,13 @@ int main (int argc, char **argv)
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t   Append l for listing of all results (when contour search is selected)\n");
 		fprintf (stderr, "\t-Z Subtract <shift> and then multiply data by <fact> before processing [1/0]\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
 	if (!grdfile) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grd file\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grid file\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->C.active && !(n == 1 || n == 3)) {
@@ -204,7 +205,7 @@ int main (int argc, char **argv)
 	ny = (size_t)GMT_get_n (south, north, grd.y_inc, grd.node_offset);
 	nm = nx * ny;
 
-	f = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+	f = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 
 	GMT_err_fail (GMT_read_grd (grdfile, &grd, f, west, east, south, north, GMT_pad, FALSE), grdfile);
 
@@ -212,9 +213,9 @@ int main (int argc, char **argv)
 	mode = (Ctrl->T.active) ? 1 : 0;
 	cellsize = grd.x_inc * grd.y_inc;
 	if (Ctrl->S.active) {
-		m_pr_deg = TWO_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
-		if (Ctrl->S.unit == 'k') m_pr_deg *= 0.001;	/* Use km instead */
-		cellsize *= m_pr_deg * m_pr_deg;
+		dist_pr_deg = project_info.M_PR_DEG;
+		if (Ctrl->S.unit == 'k') dist_pr_deg *= 0.001;	/* Use km instead */
+		cellsize *= dist_pr_deg * dist_pr_deg;
 	}
 
 	n_contours = (Ctrl->C.active) ? irint ((Ctrl->C.high - Ctrl->C.low) / Ctrl->C.inc) + 1 : 1;
@@ -386,10 +387,10 @@ int main (int argc, char **argv)
 	GMT_free ((void *)vol);
 	GMT_free ((void *)height);
 
-	GMT_end (argc, argv);
-
 	Free_Grdvolume_Ctrl (Ctrl);	/* Deallocate control structure */
 
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
@@ -618,7 +619,7 @@ double median3 (double x[])
 void *New_Grdvolume_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVOLUME_CTRL *C;
 	
-	C = (struct GRDVOLUME_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVOLUME_CTRL), "New_Grdvolume_Ctrl");
+	C = (struct GRDVOLUME_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDVOLUME_CTRL), "New_Grdvolume_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->L.value = GMT_d_NaN;
