@@ -1,6 +1,6 @@
-/* $Id: img2mercgrd.c,v 1.19 2007/03/24 01:42:07 pwessel Exp $
+/* $Id: img2mercgrd.c,v 1.28 2008/03/24 08:58:32 guru Exp $
  *
- * Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ * Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  * See COPYING file for copying and redistribution conditions.
  *
  * img2mercgrd.c
@@ -48,46 +48,82 @@
  **WHFS 27 Nov 1998 fixed bug so that -T0 gives correct results
  **  PW 18 Oct 1999 Use WORDS_BIGENDIAN macro set by configure.
  *   PW 12 Apr 2006 Replaced -x -y with -W -D
- *   PW 28 Nov 2006 Added -C for setting origin to main img origin
+ *   PW 28 Nov 2006 Added -C for setting origin to main img origin (0,0)
  *
  */
 
 
 #include "gmt_imgsubs.h"
 
+struct IMG2MERCGRD_CTRL {
+	char *file;	/* Input file name */
+	struct C {	/* -C */
+		BOOLEAN active;
+	} C;
+	struct D {	/* -D[<minlat>/<maxlat>] */
+		BOOLEAN active;
+		double min, max;
+	} D;
+	struct G {	/* -G<output_grdfile> */
+		BOOLEAN active;
+		char *file;
+	} G;
+	struct N {	/* -N<ave> */
+		BOOLEAN active;
+		int value;
+	} N;
+	struct S {	/* -S<scale> 	(use 0.1 for grav,curv,north,east files.) */
+		BOOLEAN active;
+		double value;
+	} S;
+	struct T {	/* -T<type> */
+		BOOLEAN active;
+		int value;
+	} T;
+	struct W {	/* -W<maxlon> */
+		BOOLEAN active;
+		double value;
+	} W;
+	struct m {	/* -m<minutes> */
+		BOOLEAN active;
+		double value;
+	} m;
+};
+
 int main (int argc, char **argv)
 {
-	BOOLEAN absolute = FALSE;
-	int	i, ij, navgsq, error, irarg = -1, pad[4], tempint;
-	int	navg = 1;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
-	int	output_type = -1;	/* force user to choose something between 0 and 3  */
+	int	i, navgsq, error = FALSE, irarg = -1, tempint, n_files = 0;
+	int	navg;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
 	int	iout, jout, iinstart, iinstop, jinstart, jinstop, k, kk, ion, jin, jj, iin, ii, kstart;
-	double	scale_factor = 1.0;	/* use 0.1 for grav,curv,north,east files.  */
+	size_t nm;
+	GMT_LONG ij;
 	double	west, east, south, north, toplat, botlat, dx, rnavgsq, csum, dsum, left, bottom;
 	struct GRD_HEADER h;
 	struct GMT_IMG_COORD imgcoord;
 	float	*a, empty_val;
 	short int *row;
 	int	*ix;
-	char	*infilename, *outfile, infile[320];
-	FILE	*fp;
+	char	infile[BUFSIZ];
+	FILE	*fp = NULL;
 
 	struct GMT_IMG_RANGE imgrange = { GMT_IMG_MAXLON, GMT_IMG_MINLAT, GMT_IMG_MAXLAT, GMT_IMG_MPIXEL };
 
+	struct IMG2MERCGRD_CTRL *Ctrl;
+	
+	void *New_Img2mercgrd_Ctrl (), Free_Img2mercgrd_Ctrl (struct IMG2MERCGRD_CTRL *C);
+
 	argc = GMT_begin (argc, argv);
 	
+	Ctrl = (struct IMG2MERCGRD_CTRL *)New_Img2mercgrd_Ctrl ();	/* Allocate and initialize a new control structure */
+
 	GMT_grd_init (&h, argc, argv, FALSE);
 	
 	empty_val = GMT_f_NaN;
 
-	error = FALSE;
 	west = east = south = north = 0.0;
-	fp = NULL;
-	outfile = CNULL;
-	infilename = CNULL;
 
-	pad[3] = pad[2] = pad[1] = pad[0] = 0;
 	GMT_io.in_col_type[0] = GMT_IS_LON;	GMT_io.in_col_type[1] = GMT_IS_LAT;
+	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -104,49 +140,56 @@ int main (int argc, char **argv)
 				/* Supplemental parameters:  */
 
 				case 'C':
-					absolute = TRUE;
+					Ctrl->C.active = TRUE;
+					break;
+				case 'D':
+					Ctrl->D.active = TRUE;
+					if (argv[i][2] && (sscanf(&argv[i][2], "%lf/%lf", &Ctrl->D.min, &Ctrl->D.max)) != 2) {
+						error++;
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -D: Failed to decode <minlat>/<maxlat>.\n", GMT_program);
+					}
+					else {
+						Ctrl->D.min = GMT_IMG_MINLAT_80;
+						Ctrl->D.max = GMT_IMG_MAXLAT_80;
+					}
 					break;
 				case 'G':
-					outfile = &argv[i][2];
+					Ctrl->G.active = TRUE;
+					Ctrl->G.file = strdup (&argv[i][2]);
 					break;
 				case 'N':
-					if ((sscanf(&argv[i][2], "%d", &navg)) != 1 || navg < 1) {
+					Ctrl->N.active = TRUE;
+					if ((sscanf(&argv[i][2], "%d", &Ctrl->N.value)) != 1 || Ctrl->N.value < 1) {
 						error++;
 						fprintf(stderr,"%s:  GMT SYNTAX ERROR -N requires an integer > 1.\n", GMT_program);
 					}
 					break;
 				case 'S':
-					if ((sscanf(&argv[i][2], "%lf", &scale_factor)) != 1) {
+					Ctrl->S.active = TRUE;
+					if ((sscanf(&argv[i][2], "%lf", &Ctrl->S.value)) != 1) {
 						error++;
 						fprintf(stderr,"%s:  GMT SYNTAX ERROR -S requires a scale factor.\n", GMT_program);
 					}
 					break;				
 				case 'T':
-					if ((sscanf(&argv[i][2], "%d", &output_type)) != 1 || output_type < 0 || output_type > 3) {
+					Ctrl->T.active = TRUE;
+					if ((sscanf(&argv[i][2], "%d", &Ctrl->T.value)) != 1) {
 						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -T requires an output type 0,1,2 or 3.\n", GMT_program);
-					}
-					break;
-				case 'm':
-					if ((sscanf(&argv[i][2], "%lf", &imgrange.mpixel)) != 1 || imgrange.mpixel <= 0.0) {
-						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -m requires a positive value.\n", GMT_program);
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -T requires an output type 0-3.\n", GMT_program);
 					}
 					break;
 				case 'W':
-					if ((sscanf(&argv[i][2], "%lf", &imgrange.maxlon)) != 1 || imgrange.maxlon < 360.0) {
+					Ctrl->W.active = TRUE;
+					if ((sscanf(&argv[i][2], "%lf", &Ctrl->W.value)) != 1) {
 						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -W requires a number >= 360.0.\n", GMT_program);
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -W requires a longitude >= 360.0.\n", GMT_program);
 					}
 					break;
-				case 'D':
-					if (argv[i][2] && (sscanf(&argv[i][2], "%lf/%lf", &imgrange.minlat, &imgrange.maxlat)) != 2) {
+				case 'm':
+					Ctrl->m.active = TRUE;
+					if ((sscanf(&argv[i][2], "%lf", &Ctrl->m.value)) != 1) {
 						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -D: Failed to decode <minlat>/<maxlat>.\n", GMT_program);
-					}
-					else {
-						imgrange.minlat = GMT_IMG_MINLAT_80;
-						imgrange.maxlat = GMT_IMG_MAXLAT_80;
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -m requires a positive value.\n", GMT_program);
 					}
 					break;
 				default:
@@ -155,17 +198,22 @@ int main (int argc, char **argv)
 					break;
 			}
 		}
+		else if (n_files == 0) {
+			Ctrl->file = strdup (argv[i]);
+			n_files++;
+		}
 		else {
-			infilename = argv[i];
+			fprintf(stderr,"%s:  GMT SYNTAX ERROR: More than one world image file name given.\n", GMT_program);
+			error++;
 		}
 	}
 
 	if (GMT_give_synopsis_and_exit || argc == 1) {
-		fprintf(stderr,"img2mercgrd %s - Extract a grd file from an img file, preserving Mercator projection.\n\n", GMT_VERSION);
+		fprintf(stderr,"img2mercgrd %s - Extract a grid file from an img file, preserving Mercator projection.\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:  img2mercgrd <world_image_filename> %s -G<grdfile> -T<type>\n", GMT_Rgeo_OPT);
 		fprintf(stderr,"\t\t[-C] [-D[<minlat>/<maxlat>]] [-N<navg>] [-S<scale>] [-V] [-W<maxlon>] [-m<minutes>]\n\n");
 
-		if (GMT_give_synopsis_and_exit) exit (-1);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
 		fprintf (stderr, "\tREQUIRED ARGUMENTS:\n");
 		fprintf(stderr,"\t<world_image_filename> gives location of img file.\n");
@@ -180,7 +228,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\tOPTIONAL ARGUMENTS:\n");
 		fprintf(stderr,"\t-C Refer Mercator coordinates to img source origin [Default sets lower left to 0,0].\n");
 		fprintf(stderr,"\t-N<navg> will ouput averages of input in navg by navg squares.  [no averaging]\n");
-		fprintf(stderr,"\t-S<scale> will multiply img integer values by scale for output [%g]\n", scale_factor);
+		fprintf(stderr,"\t-S<scale> will multiply img integer values by scale for output [1]\n");
 		GMT_explain_option ('V');
 		
 		fprintf (stderr, "\n\tOPTIONAL ARGUMENTS WHICH DEPEND ON THE img FILE VERSION:\n");
@@ -188,39 +236,58 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D[<minlon>/<maxlon>] input img file bottom and top latitudes. [%.3f/%.3f]\n", GMT_IMG_MINLAT, GMT_IMG_MAXLAT);
 		fprintf (stderr, "\t   If no latitudes are given it is taken to mean %.3f/%.3f\n", GMT_IMG_MINLAT_80, GMT_IMG_MAXLAT_80);
 		fprintf (stderr, "\t-W<maxlon> input img file runs from 0 to <maxlon> longitude. [360.0]\n");
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (irarg < 0 || west >= east || south >= north) {
 		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Must specify -R with west < east and south < north.\n", GMT_program);
 		error++;
 	}
-	if (outfile == NULL) {
-		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Must specify output grdfile name with -G.\n", GMT_program);
+	if (!Ctrl->G.active || Ctrl->G.file == NULL) {
+		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Must specify output grid file name with -G.\n", GMT_program);
 		error++;
 	}
-	if (infilename == NULL) {
+	if (Ctrl->file == NULL) {
 		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Must specify input imgfile name.\n", GMT_program);
 		error++;
 	}
-	if (output_type < 0 || output_type > 3) {
+	if (Ctrl->D.active && (Ctrl->D.min <= -90 || Ctrl->D.max >= 90.0 || Ctrl->D.max <= Ctrl->D.min)) {
+		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Min/max latitudes are invalid\n", GMT_program);
+		error++;
+	}
+	if (Ctrl->T.value < 0 || Ctrl->T.value > 3) {
 		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Must specify output type in the range 0-3 with -T.\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->W.active && Ctrl->W.value < 360.0) {
+		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Requires a maximum longitude >= 360.0 with -W.\n", GMT_program);
+		error++;
+	}
+	if (Ctrl->m.active && Ctrl->m.value <= 0.0) {
+		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Requires a positive value with -m.\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
+	if (Ctrl->W.active) imgrange.maxlon = Ctrl->W.value;
+	if (Ctrl->m.active) imgrange.mpixel = Ctrl->m.value;
+	if (Ctrl->D.active) {
+		imgrange.minlat = Ctrl->D.min;
+		imgrange.maxlat = Ctrl->D.max;
+	}
+	
 	if (GMT_img_setup_coord ( &imgrange, &imgcoord) ) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Error in img coordinate specification [-m -W or -D]\n", GMT_program);
 		error++;
 	}
-	else if (navg && (imgcoord.nx360%navg != 0 || imgcoord.nyrow%navg != 0) ) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Bad choice of navg in -N.  Must divide %d and %d\n", GMT_program,
+	else if (Ctrl->N.value && (imgcoord.nx360%Ctrl->N.value != 0 || imgcoord.nyrow%Ctrl->N.value != 0) ) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Bad choice of navg in -N.  Must divide %ld and %ld\n", GMT_program,
 			imgcoord.nx360, imgcoord.nyrow);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_getdatapath (infilename, infile);
+	GMT_getdatapath (Ctrl->file, infile);
 	if ((fp = fopen(infile, "rb")) == NULL) {
 		fprintf(stderr, "%s: GMT SYNTAX ERROR:  Cannot open %s for binary read.\n", GMT_program, infile);
 		error++;
@@ -228,13 +295,14 @@ int main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 	
 	GMT_io.out_col_type[0] = GMT_io.out_col_type[1] = GMT_IS_FLOAT;		/* Since output is no longer lon/lat */
+	navg = Ctrl->N.value;
 	
 	/* Expected edges of input image based on coordinate initialization (might not exactly match user spec):  */
 	toplat = GMT_img_ypix_to_lat (0.0, &imgcoord);
 	botlat = GMT_img_ypix_to_lat ( (double)imgcoord.nyrow, &imgcoord);
 	dx = 1.0 / ( (double)imgcoord.nx360 / 360.0);
 	if (gmtdefs.verbose)
-		fprintf (stderr, "%s expects %s to be %d by %d pixels spanning 0/%5.1f/%.8g/%.8g\n",
+		fprintf (stderr, "%s expects %s to be %ld by %ld pixels spanning 0/%5.1f/%.8g/%.8g\n",
 			GMT_program, infile, imgcoord.nxcol, imgcoord.nyrow, dx*imgcoord.nxcol, botlat, toplat);
 
 	if (toplat < north) {
@@ -279,13 +347,18 @@ int main (int argc, char **argv)
 	rnavgsq = 1.0 / navgsq;
 
 	/* Set up header with Mercatorized dimensions assuming -Jm1  */
-	if (absolute) {
+	if (Ctrl->C.active) {
+		int equator;
+		equator = irint (GMT_img_lat_to_ypix (0.0, &imgcoord));
 		h.x_min = iinstart * navg * dx;
 		h.x_max = h.x_min + h.nx * navg * dx;
-		h.y_max = (imgcoord.nyrow - jinstart) * navg * dx;
+		h.y_max = (imgcoord.nyrow - jinstart - equator) * navg * dx;
 		h.y_min = h.y_max - h.ny * navg * dx;
-		left = 0.0;
-		bottom = imgrange.minlat;
+		left = bottom = 0.0;
+		if (h.x_max > 360.0) {
+			h.x_max -= 360.0;
+			h.x_min -= 360.0;
+		}
 	}
 	else {
 		h.x_min = 0.0;
@@ -302,7 +375,7 @@ int main (int argc, char **argv)
 	h.node_offset = 1;
 	h.z_scale_factor = 1.0;
 	h.z_add_offset = 0.0;
-	if (output_type < 3)
+	if (Ctrl->T.value < 3)
 		strcpy (h.z_units, "meters, mGal, Eotvos, or micro-radians, depending on img file and -S.");
 	else
 		strcpy (h.z_units, "T/F, one or more constraints fell in this pixel.");
@@ -311,13 +384,14 @@ int main (int argc, char **argv)
 	h.z_min = DBL_MAX;
 	h.z_max = -DBL_MAX;
 
-	GMT_err_fail (GMT_grd_RI_verify (&h, 1), outfile);
+	GMT_err_fail (GMT_grd_RI_verify (&h, 1), Ctrl->G.file);
 
 	/* Now malloc some space for float grd array, integer pixel index, and short integer data buffer.  */
 
 	row = (short int *) GMT_memory (VNULL, (size_t)(navg * imgcoord.nxcol), sizeof (short int), GMT_program);
 	ix = (int *) GMT_memory (VNULL, (size_t)(navgsq * h.nx), sizeof (int), GMT_program);
-	a = (float *) GMT_memory (VNULL, (size_t)(h.nx * h.ny), sizeof (float), GMT_program);
+	nm = ((size_t)h.nx) * ((size_t)h.ny);
+	a = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
 	/* Load ix with the index to the correct column, for each output desired.  This helps for Greenwich, 
 		also faster averaging of the file, etc.  Note for averaging each n by n block is looped in turn. */
@@ -345,7 +419,7 @@ int main (int argc, char **argv)
 
 	/* Now before beginning data loop, fseek if needed.  */
 	if (jinstart > 0 && jinstart < imgcoord.nyrow) {
-		fseek (fp, 2 * imgcoord.nxcol * jinstart, SEEK_SET);
+		fseek (fp, (long)(2 * imgcoord.nxcol * jinstart), SEEK_SET);
 	}
 	
 	/* Now loop over output points, reading and handling data as needed */
@@ -373,7 +447,7 @@ int main (int argc, char **argv)
 					dsum = 0.0;
 					for (k = 0, kk = kstart; k < navgsq; k++, kk++) {
 						tempint = (int)row[ix[kk]];
-						if (output_type) {
+						if (Ctrl->T.value) {
 							if ( ( (abs(tempint))%2) != 0) {
 								csum += 1.0;
 								tempint--;
@@ -386,7 +460,7 @@ int main (int argc, char **argv)
 				}
 				else {
 					tempint = (int)row[ix[iout]];
-					if (output_type && abs(tempint)%2 != 0) {
+					if (Ctrl->T.value && abs(tempint)%2 != 0) {
 						csum = 1.0;
 						tempint--;
 					}
@@ -395,9 +469,9 @@ int main (int argc, char **argv)
 					dsum = (double) tempint;
 				}
 				
-				dsum *= scale_factor;
+				if (Ctrl->S.active) dsum *= Ctrl->S.value;
 				
-				switch (output_type) {
+				switch (Ctrl->T.value) {
 					case 0:
 					case 1:
 						a[ij] = (float) dsum;
@@ -411,7 +485,7 @@ int main (int argc, char **argv)
 				}
 				
 
-				if (output_type != 2 || csum >= 0.5) {
+				if (Ctrl->T.value != 2 || csum >= 0.5) {
 					if (h.z_min > a[ij]) h.z_min = a[ij];
 					if (h.z_max < a[ij]) h.z_max = a[ij];
 				}
@@ -421,15 +495,38 @@ int main (int argc, char **argv)
 	
 	fclose (fp);
 	if (gmtdefs.verbose)
-		fprintf(stderr,"Created %d by %d Mercatorized .grd file.  Min, Max values are %.8g  %.8g\n", h.nx, h.ny, h.z_min, h.z_max);
+		fprintf(stderr,"Created %d by %d Mercatorized grid file.  Min, Max values are %.8g  %.8g\n", h.nx, h.ny, h.z_min, h.z_max);
 
-	GMT_err_fail (GMT_write_grd (outfile, &h, a, 0.0, 0.0, 0.0, 0.0, pad, FALSE), outfile);
+	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &h, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	GMT_free((void *)a);
 	GMT_free((void *)ix);
 	GMT_free((void *)row);
 	
-        GMT_end (argc, argv);
+	Free_Img2mercgrd_Ctrl (Ctrl);	/* Deallocate control structure */
+
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
+
+void *New_Img2mercgrd_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct IMG2MERCGRD_CTRL *C;
+	
+	C = (struct IMG2MERCGRD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct IMG2MERCGRD_CTRL), "New_Img2mercgrd_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->D.min = GMT_IMG_MINLAT;
+	C->D.max = GMT_IMG_MAXLAT;
+	C->N.value = 1;		/* No averaging */
+	C->T.value = -1;	/* Force user to change this */
+	C->m.value = GMT_IMG_MPIXEL;
+	
+	return ((void *)C);
+}
+
+void Free_Img2mercgrd_Ctrl (struct IMG2MERCGRD_CTRL *C) {	/* Deallocate control structure */
+	if (C->file) free ((void *)C->file);	
+	if (C->G.file) free ((void *)C->G.file);	
+	GMT_free ((void *)C);	
+}
