@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: makecpt.c,v 1.36 2007/03/12 19:52:26 remko Exp $
+ *	$Id: makecpt.c,v 1.45 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -83,7 +83,7 @@ int main(int argc, char **argv)
 	
 	/* Get list of available color tables in $GMT_SHAREDIR */
 
-	GMT_getsharepath (CNULL, "GMT_CPT", ".lis", CPT_lis);
+	GMT_getsharepath ("conf", "gmt_cpt", ".conf", CPT_lis);
 	if ((fpc = fopen (CPT_lis, "r")) == NULL) {
 		fprintf (stderr, "%s: ERROR: Cannot open file %s\n", GMT_program, CPT_lis);
 		exit (EXIT_FAILURE);
@@ -197,11 +197,11 @@ int main(int argc, char **argv)
 		z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
 		nz = 0;
 		while (GMT_fgets (buffer, BUFSIZ, fpl)) {
-			if (buffer[0] == '#') continue;
+			if (GMT_is_a_blank_line (buffer)) continue;	/* Skip blank lines or # comments */
 			z[nz] = atof (buffer);
 			nz++;
 			if (nz == n_alloc) {
-				n_alloc += GMT_SMALL_CHUNK;
+				n_alloc <<= 1;
 				z = (double *) GMT_memory ((void *)z, (size_t)n_alloc, sizeof(double), GMT_program);
 			}
 		}
@@ -232,7 +232,13 @@ int main(int argc, char **argv)
 	else {	/* Just copy what was in the cpt file */
 		nz = GMT_n_colors + 1;
 		z = (double *) GMT_memory (VNULL, (size_t)nz, sizeof(double), GMT_program);
-		for (i = 0; i < nz-1; i++) z[i] = GMT_lut[i].z_low;
+		if (Ctrl->I.active) {
+			/* Reverse the intervals (only relavant for non-equidistant color maps) */
+			for (i = 0; i < nz-1; i++) z[i] = GMT_lut[0].z_low + GMT_lut[GMT_n_colors-1].z_high - GMT_lut[GMT_n_colors-1-i].z_high;
+		}
+		else {
+			for (i = 0; i < nz-1; i++) z[i] = GMT_lut[i].z_low;
+		}
 		z[i] = GMT_lut[i-1].z_high;
 	}
 
@@ -258,14 +264,14 @@ int main(int argc, char **argv)
 void *New_Makecpt_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct MAKECPT_CTRL *C;
 	
-	C = (struct MAKECPT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct MAKECPT_CTRL), "New_Makecpt_Ctrl");
+	C = (struct MAKECPT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct MAKECPT_CTRL), "New_Makecpt_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
 }
 
 void Free_Makecpt_Ctrl (struct MAKECPT_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
-	if (C->T.file) GMT_free ((void *)C->T.file);	
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->T.file) free ((void *)C->T.file);	
 	GMT_free ((void *)C);	
 }
