@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mapproject.c,v 1.81 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: mapproject.c,v 1.92 2008/04/21 21:49:19 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -66,7 +66,7 @@ struct MAPPROJECT_CTRL {	/* All control options for this program (except common
 	} F;
 	struct G {	/* -G<lon0>/<lat0>[/e|E|k|K|m|M|n|N|c|C|d|D] */
 		BOOLEAN active;
-		int mode;		/* 1 = distance to fixed point, 2 = cumulative distances */
+		int mode;		/* 1 = distance to fixed point, 2 = cumulative distances, 3 = incremental distances, 4 = 2nd point in cols 3/4 */
 		BOOLEAN geodesic;	/* TRUE if we want geodesic distances [Default is great circle distances] */
 		double lon, lat;	/* Fixed point of reference */
 		char unit;
@@ -97,9 +97,12 @@ struct MAPPROJECT_CTRL {	/* All control options for this program (except common
 
 int main (int argc, char **argv)
 {
-	int i, j, k, fno, n = 0, n_read = 0, n_files = 0, x, y, n_args, unit = 0, n_slash;
+	int k, fno, n_files = 0, x, y, n_args, unit = 0, n_slash;
 	int n_fields, n_expected_fields, proj_type = 0, *n_output, save[2] = {0,0}, two, n_out;
 	int fmt[2], pos;
+	
+	GMT_LONG i, j, n = 0, n_read = 0;
+	
 
 	BOOLEAN error = FALSE;
 	BOOLEAN nofile = TRUE, done = FALSE, first = TRUE;
@@ -107,7 +110,7 @@ int main (int argc, char **argv)
 	BOOLEAN geodetic_calc = FALSE, greenwich = FALSE;
 	BOOLEAN datum_conv_only = FALSE, double_whammy = FALSE, shift_xy = FALSE;
 
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, x_in = 0.0, y_in = 0.0, d, s;
+	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, x_in = 0.0, y_in = 0.0, d = 0.0, s = 0.0;
 	double xmin, xmax, ymin, ymax, *in, *out, fwd_scale, inv_scale, xtmp, ytmp;
 	double x_in_min, x_in_max, y_in_min, y_in_max, inch_to_unit, unit_to_inch;
 	double x_out_min, x_out_max, y_out_min, y_out_max, u_scale, d_scale;
@@ -218,6 +221,14 @@ int main (int argc, char **argv)
 						error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &Ctrl->G.lat), txt_b);
 						if (n_slash == 2) Ctrl->G.unit = c;
 					}
+					else if (argv[i][2] == '+') {				/* Got -G+[units] */
+						Ctrl->G.mode = 4;
+						Ctrl->G.unit = argv[i][3];
+					}
+					else if (argv[i][2] == '-') {				/* Got -G-[units] */
+						Ctrl->G.mode = 3;
+						Ctrl->G.unit = argv[i][3];
+					}
 					else {				/* Got -G[units] */
 						Ctrl->G.mode = 2;
 						Ctrl->G.unit = argv[i][2];
@@ -286,7 +297,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "mapproject %s - Forward and Inverse map transformations and geodesy\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: mapproject <infiles> %s %s [-C[<dx/dy>]]\n", GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[-Ab|B|f|F<lon0/lat0>] [-Dc|i|m|p] [-E[<datum>]] [-F[k|m|n|i|c|p]] [-G[<lon0/lat0>/][<unit>]\n");
+		fprintf (stderr, "\t[-Ab|B|f|F<lon0/lat0>] [-Dc|i|m|p] [-E[<datum>]] [-F[k|m|n|i|c|p]] [-G[<lon0/lat0>/][+|-][<unit>]\n");
 		fprintf (stderr, "\t[%s] [-I] [-L<line.xy>[/<unit>]] [%s] [-Q[e|d]] [-S] [-T[h]<from>[/<to>]\n", GMT_H_OPT, GMT_M_OPT);
 		fprintf (stderr, "\t[-V[l]] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
 
@@ -295,6 +306,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\tinfiles (in ASCII or binary) has 2 or more columns.  If no file(s) is given, standard input is read.\n");
 		GMT_explain_option ('J');
 		GMT_explain_option ('R');
+		fprintf (stderr, "\t   If UTM and -C are used then -R is optional (automatically set to match UTM zone)\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A Calculate azimuths from specified point to each input data point with -Af.\n");
 		fprintf (stderr, "\t   Use -Ab to calculate backazimuths from data to specified point.\n");
@@ -310,7 +322,9 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   If <datum> = - or not given we assume WGS-84.\n");
 		fprintf (stderr, "\t-F force projected values to be in actual meters [Default uses the given plot scale]\n");
 		fprintf (stderr, "\t   Specify unit by appending k (km), m (miles), n (nautical miles), i (inch), c (cm), or p (points)\n");
-		fprintf (stderr, "\t-G Calculate distances to specified point OR culumlative distances along track (if point not given).\n");
+		fprintf (stderr, "\t-G Calculate distances to specified point OR cumulative distances along track (if point not given).\n");
+		fprintf (stderr, "\t   Use -G+[<unit>] to get provide <lon0> <lat0> from two extra input columns.\n");
+		fprintf (stderr, "\t   Use -G-[<unit>] to get distance increments rather than cumulate distances along track\n");
 		fprintf (stderr, "\t   Specify unit as m(e)ter, (k)m, (m)ile, (n)autical mile, (d)egree, or (c)artesian in user units.\n");
 		fprintf (stderr, "\t   Unit C means Cartesian distances after first projecting the input coordinates (-R, -J).\n");
 		fprintf (stderr, "\t   Units E, K, M, N, D mean geodesic distance using current ellipsoid [lower case is spherical].\n");
@@ -362,7 +376,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "-----------------------------------------------------------------------------------------\n");
 		for (i = 0; i < GMT_N_DATUMS; i++) {
 			(!strcmp (gmtdefs.datum[i].name, "WGS 1984")) ? fprintf (stderr, "->") : fprintf (stderr, "  ");
-			fprintf (stderr, "%3d %-34s %-23s %5.0f %5.0f %5.0f %s\n", i, gmtdefs.datum[i].name, gmtdefs.datum[i].ellipsoid, gmtdefs.datum[i].xyz[0], gmtdefs.datum[i].xyz[1], gmtdefs.datum[i].xyz[2], gmtdefs.datum[i].region);
+			fprintf (stderr, "%3ld %-34s %-23s %5.0f %5.0f %5.0f %s\n", i, gmtdefs.datum[i].name, gmtdefs.datum[i].ellipsoid, gmtdefs.datum[i].xyz[0], gmtdefs.datum[i].xyz[1], gmtdefs.datum[i].xyz[2], gmtdefs.datum[i].region);
 		}
 		fprintf (stderr, "-----------------------------------------------------------------------------------------\n");
 	}
@@ -378,10 +392,24 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR: -A, -G, and -L cannot work with -I\n", GMT_program);
 		error++;
 	}
-	if (project_info.projection < 0 && (Ctrl->G.mode || Ctrl->L.active) && proj_type == 2) {	/* Must have -J */
+	if (project_info.projection == GMT_NO_PROJ && (Ctrl->G.mode || Ctrl->L.active) && proj_type == 2) {	/* Must have -J */
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -J option with selected form of -G or -L\n", GMT_program);
 		error++;
 	}
+	if (!project_info.region_supplied && project_info.projection == GMT_UTM && Ctrl->C.active) {	/* Set default UTM region from zone info */
+		if (!project_info.utm_zoney && project_info.utm_hemisphere == 0) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  -Ju need zone specification with latitude or hemisphere selection\n", GMT_program);
+			error++;
+		}
+		if (GMT_UTMzone_to_wesn (project_info.utm_zonex, project_info.utm_zoney, project_info.utm_hemisphere, &west, &east, &south, &north)) {
+			error++;
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Bad UTM zone\n", GMT_program);
+		}
+		else if (gmtdefs.verbose)
+			fprintf (stderr, "%s: UTM zone used to generate region %g/%g/%g/%g\n", GMT_program, west, east, south, north);
+		
+		project_info.region_supplied = TRUE;
+	}
 	if (!project_info.region_supplied && !(geodetic_calc || Ctrl->T.active || Ctrl->E.active)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -423,7 +451,7 @@ int main (int argc, char **argv)
 	if (Ctrl->E.active) GMT_ECEF_init (&Ctrl->E.datum);
 	if (Ctrl->F.active) unit = GMT_check_scalingopt ('F', Ctrl->F.unit, scale_unit_name);
 
-	if (Ctrl->T.active && project_info.projection && project_info.region_supplied) {	/* Do datum shift & project coordinates */
+	if (Ctrl->T.active && project_info.projection != GMT_LINEAR && project_info.region_supplied) {	/* Do datum shift & project coordinates */
 		double_whammy = TRUE;
 		if (Ctrl->I.active) {	/* Need to set the ellipsoid to that of the old datum */
 			if (GMT_datum.from.ellipsoid_id < 0) {
@@ -457,7 +485,7 @@ int main (int argc, char **argv)
 
 	pure_ascii = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
 
-	if (project_info.projection < 0) {	/* Supply dummy linear proj */
+	if (project_info.projection == GMT_NO_PROJ) {	/* Supply dummy linear proj */
 		GMT_parse_J_option ("x1d");
 		if (!project_info.region_supplied) {
 			west = 0.0;	east = 360.0;
@@ -480,7 +508,7 @@ int main (int argc, char **argv)
 		ymin = (Ctrl->C.active) ? project_info.ymin - project_info.y0 : project_info.ymin;
 		ymax = (Ctrl->C.active) ? project_info.ymax - project_info.y0 : project_info.ymax;
 		if (Ctrl->F.active) {	/* Convert to GMT inches */
-			strncpy (unit_name, scale_unit_name, GMT_TEXT_LEN);
+			strncpy (unit_name, scale_unit_name, (size_t)GMT_TEXT_LEN);
 			xmin /= project_info.x_scale;
 			xmax /= project_info.x_scale;
 			ymin /= project_info.y_scale;
@@ -563,7 +591,7 @@ int main (int argc, char **argv)
 		Ctrl->C.easting *= u_scale;
 		Ctrl->C.northing *= u_scale;
 	}
-	if (Ctrl->G.mode == 2 && proj_type == 2) {	/* Must project the fixed point here */
+	if (Ctrl->G.mode >= 2 && proj_type == 2) {	/* Must project the fixed point here */
 		GMT_geo_to_xy (Ctrl->G.lon, Ctrl->G.lat, &xtmp, &ytmp);
 		if (Ctrl->C.active) {	/* Change origin from lower left to projection center */
 			xtmp -= project_info.x0;
@@ -634,7 +662,7 @@ int main (int argc, char **argv)
 				if (GMT_io.status & GMT_IO_EOF) continue;
 
 				if (GMT_io.status & GMT_IO_MISMATCH && n_fields < 2) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 					continue;
 				}
 				if (!out) out = (double *) GMT_memory (VNULL, (size_t)n_expected_fields, sizeof (double), GMT_program);
@@ -717,7 +745,7 @@ int main (int argc, char **argv)
 					GMT_output (GMT_stdout, *n_output, out);
 				}
 				n++;
-				if (long_verbose && (n%1000) == 0) fprintf (stderr, "%s: Projected %d points\r", GMT_program, n);
+				if (long_verbose && (n%1000) == 0) fprintf (stderr, "%s: Projected %ld points\r", GMT_program, n);
 			}
 		}
 		else {		/* Do forward transformation */
@@ -732,7 +760,7 @@ int main (int argc, char **argv)
 				if (GMT_io.status & GMT_IO_EOF) continue;
 
 				if (GMT_io.status & GMT_IO_MISMATCH && n_fields < 2) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 					continue;
 				}
 				if (!out) out = (double *) GMT_memory (VNULL, (size_t)n_expected_fields+3, sizeof (double), GMT_program);
@@ -784,7 +812,15 @@ int main (int argc, char **argv)
 				}
 				if (geodetic_calc) {	/* Get either distances or azimuths */
 					if (Ctrl->G.mode) {	/* Cumulative distances along track */
-						if (Ctrl->G.mode == 2 && line_start)
+						if (Ctrl->G.mode == 4) {
+							if (proj_type == 2) {	/* Calculate Cartesian distances using projected units */
+								GMT_geo_to_xy (in[2], in[3], &xtmp, &ytmp);
+								s = hypot (xtmp - out[0], ytmp - out[1]);
+							}
+							else
+								s = d_scale * (*distance_func) (in[0], in[1], in[2], in[3]);
+						}
+						else if (Ctrl->G.mode >= 2 && line_start)
 							s = d = 0.0;
 						else if (proj_type == 2)	/* Calculate Cartesian distances using projected units */
 							s = hypot (Ctrl->G.lon - out[0], Ctrl->G.lat - out[1]);
@@ -792,9 +828,12 @@ int main (int argc, char **argv)
 							s = hypot (Ctrl->G.lon - in[0], Ctrl->G.lat - in[1]);
 						else				/* Great circle distances */
 							s = d_scale * (*distance_func) (Ctrl->G.lon, Ctrl->G.lat, in[0], in[1]);
-						if (Ctrl->G.mode == 2) {
+						if (Ctrl->G.mode >= 2) {
 							line_start = FALSE;
-							d += s;
+							if (Ctrl->G.mode >= 3)	/* Increments */
+								d = s;
+							else
+								d += s;		/* Cumulative */
 							if (proj_type == 2) {	/* Calculate distances using projected units */
 								Ctrl->G.lon = out[0];
 								Ctrl->G.lat = out[1];
@@ -894,14 +933,14 @@ int main (int argc, char **argv)
 					}
 				}
 				n++;
-				if (long_verbose && (n%1000) == 0) fprintf (stderr, "%s: Projected %d points\r", GMT_program, n);
+				if (long_verbose && (n%1000) == 0) fprintf (stderr, "%s: Projected %ld points\r", GMT_program, n);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
 	if (gmtdefs.verbose && n_read > 0) {
-		fprintf (stderr, "%s: Projected %d points\n", GMT_program, n);
+		fprintf (stderr, "%s: Projected %ld points\n", GMT_program, n);
 		sprintf (format, "%%s: Input extreme values:  Xmin: %s Xmax: %s Ymin: %s Ymax %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, x_in_min, x_in_max, y_in_min, y_in_max);
 		if (!geodetic_calc) {
@@ -909,22 +948,22 @@ int main (int argc, char **argv)
 			fprintf (stderr, format, GMT_program, x_out_min, x_out_max, y_out_min, y_out_max);
 			if (Ctrl->I.active) {
 				if (Ctrl->E.active)
-					fprintf (stderr, "%s: Mapped %d ECEF coordinates [m] to (lon,lat,h)\n", GMT_program, n);
+					fprintf (stderr, "%s: Mapped %ld ECEF coordinates [m] to (lon,lat,h)\n", GMT_program, n);
 				else
-					fprintf (stderr, "%s: Mapped %d x-y pairs [%s] to lon-lat\n", GMT_program, n, unit_name);
+					fprintf (stderr, "%s: Mapped %ld x-y pairs [%s] to lon-lat\n", GMT_program, n, unit_name);
 			}
 			else if (Ctrl->T.active && GMT_datum.h_given)
-				fprintf (stderr, "%s: Datum-converted %d (lon,lat,h) triplets\n", GMT_program, n);
+				fprintf (stderr, "%s: Datum-converted %ld (lon,lat,h) triplets\n", GMT_program, n);
 			else if (Ctrl->T.active)
-				fprintf (stderr, "%s: Datum-converted %d (lon,lat) pairs\n", GMT_program, n);
+				fprintf (stderr, "%s: Datum-converted %ld (lon,lat) pairs\n", GMT_program, n);
 			else if (Ctrl->E.active)
-				fprintf (stderr, "%s: Mapped %d (lon,lat,h) triplets to ECEF coordinates [m]\n", GMT_program, n);
+				fprintf (stderr, "%s: Mapped %ld (lon,lat,h) triplets to ECEF coordinates [m]\n", GMT_program, n);
 			else if (GMT_IS_MAPPING)
-				fprintf (stderr, "%s: Mapped %d lon-lat pairs to x-y [%s]\n", GMT_program, n, unit_name);
+				fprintf (stderr, "%s: Mapped %ld lon-lat pairs to x-y [%s]\n", GMT_program, n, unit_name);
 			else
-				fprintf (stderr, "%s: Mapped %d data pairs to x-y [%s]\n", GMT_program, n, unit_name);
+				fprintf (stderr, "%s: Mapped %ld data pairs to x-y [%s]\n", GMT_program, n, unit_name);
 		}
-		if (Ctrl->S.active && n != n_read) fprintf (stderr, "%s: %d fell outside region\n", GMT_program, n_read - n);
+		if (Ctrl->S.active && n != n_read) fprintf (stderr, "%s: %ld fell outside region\n", GMT_program, n_read - n);
 	}
 
 	GMT_free ((void *)out);
@@ -941,7 +980,7 @@ int main (int argc, char **argv)
 void *New_Mapproject_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct MAPPROJECT_CTRL *C;
 	
-	C = (struct MAPPROJECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct MAPPROJECT_CTRL), "New_Mapproject_Ctrl");
+	C = (struct MAPPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct MAPPROJECT_CTRL), "New_Mapproject_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -951,6 +990,6 @@ void *New_Mapproject_Ctrl () {	/* Allocate and initialize a new control structur
 }
 
 void Free_Mapproject_Ctrl (struct MAPPROJECT_CTRL *C) {	/* Deallocate control structure */
-	if (C->L.file) GMT_free ((void *)C->L.file);	
+	if (C->L.file) free ((void *)C->L.file);	
 	GMT_free ((void *)C);	
 }
