@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: pscoupe.c,v 1.32 2007/03/24 01:42:07 pwessel Exp $
+ *    $Id: pscoupe.c,v 1.36 2008/03/24 08:58:32 guru Exp $
  *
- *    Copyright (c) 1996-2007 by G. Patau
+ *    Copyright (c) 1996-2008 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
@@ -43,7 +43,7 @@ int main (int argc, char **argv)
 
     int     i, j, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
     int     n_args;
-    int     n_rec = 0;
+    GMT_LONG     n_rec = 0;
 
     BOOLEAN    outline = FALSE, p_outline = FALSE, t_outline = FALSE;
     BOOLEAN    error = FALSE, nofile = TRUE, polygon = FALSE;
@@ -639,10 +639,10 @@ int main (int argc, char **argv)
             ps_comment (line);
         }
         if (GMT_io.io_header[GMT_IN]) 
-            for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+            for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
         
 
-        while (fgets (line, BUFSIZ, fp)) {
+        while (GMT_fgets (line, BUFSIZ, fp)) {
             n_rec++;
             if (read_cmt) {
                 sscanf (line, "%s %s %s %s %s %s %s %s %s %s %s %s %s %[^\n]\n",
@@ -1014,11 +1014,11 @@ Definition of scalar moment.
             }
 
         }
-        if (fp != stdin) fclose (fp);
+        if (fp != stdin) GMT_fclose (fp);
     }
     
     if (gmtdefs.verbose) 
-        fprintf (stderr, "pscoupe: Number of records read: %i\n", n_rec);
+        fprintf (stderr, "pscoupe: Number of records read: %li\n", n_rec);
 
 
     if (skip_if_outside) GMT_map_clip_off ();
