@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: pspolar.c,v 1.25 2007/03/24 01:42:07 pwessel Exp $ 
+ *    $Id: pspolar.c,v 1.29 2008/03/24 08:58:32 guru Exp $ 
  *
- *    Copyright (c) 1996-2007 by G. Patau
+ *    Copyright (c) 1996-2008 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
@@ -36,7 +36,7 @@
 
 #define POINTSIZE 0.005
 
-int main (int argc,char **argv)
+int main (int argc, char **argv)
 {
     int     i, symbol = 0, n, n_files = 0, fno;
     int     n_args;
@@ -218,7 +218,7 @@ int main (int argc,char **argv)
                          if(strchr(argv[i], 'V')) {
                              vecS = TRUE;
                              strcpy(txt_a,strchr(argv[i], 'V'));
-                             if(strncmp(txt_a,"VG",2) == 0 || strncmp(txt_a,"VL",2) == 0 || strlen(txt_a) == 1) {
+                             if(strncmp(txt_a,"VG",(size_t)2) == 0 || strncmp(txt_a,"VL",(size_t)2) == 0 || strlen(txt_a) == 1) {
                                  v_width = 0.03; h_length = 0.12; h_width = 0.1; shape = gmtdefs.vector_shape;
                                  if (!gmtdefs.measure_unit) {
                                      v_width = 0.075; h_length = 0.3; h_width = 0.25; shape = gmtdefs.vector_shape;
@@ -404,7 +404,7 @@ int main (int argc,char **argv)
             sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, 512, fp);
+        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, 512, fp);
         
         GMT_world_map = TRUE;
         
@@ -426,7 +426,7 @@ int main (int argc,char **argv)
         GMT_setpen (&fpen);
         ps_circle (plot_x0, plot_y0, ech, ffill.rgb, outline_F);
 
-        while (fgets (line, BUFSIZ, fp)) {
+        while (GMT_fgets (line, BUFSIZ, fp)) {
             switch (hypo) {
                 case 0 :
                     if(!plot_polS) {
@@ -625,7 +625,7 @@ int main (int argc,char **argv)
                 }
             }
         }
-        if (fp != stdin) fclose (fp);
+        if (fp != stdin) GMT_fclose (fp);
     }
     
     if (skip_if_outside) GMT_map_clip_off ();
