@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: psvelo.c,v 1.27 2007/03/24 01:42:07 pwessel Exp $
+ *    $Id: psvelo.c,v 1.31 2008/03/24 08:58:32 guru Exp $
  *
- *    Copyright (c) 1996-2007 by G. Patau
+ *    Copyright (c) 1996-2008 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
@@ -49,34 +49,34 @@ int main (int argc, char **argv)
 {
 	int 	i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
 	int	n_args;
-        int     n_rec = 0;
+	GMT_LONG     n_rec = 0;
 
-        BOOLEAN	outline = FALSE;
+	BOOLEAN	outline = FALSE;
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE;
-        BOOLEAN shade_uncert = FALSE;
-        BOOLEAN rescale_sigma = FALSE;
+	BOOLEAN shade_uncert = FALSE;
+	BOOLEAN rescale_sigma = FALSE;
 	BOOLEAN done, no_size_needed, greenwich;
 	BOOLEAN read_ellipse = FALSE, read_rotated_ellipse = FALSE;
-        BOOLEAN des_ellipse = TRUE, des_arrow = TRUE;
-        BOOLEAN read_anisotropy = FALSE;
-        BOOLEAN read_wedge = FALSE;
-        BOOLEAN read_cross = FALSE;
+	BOOLEAN des_ellipse = TRUE, des_arrow = TRUE;
+	BOOLEAN read_anisotropy = FALSE;
+	BOOLEAN read_wedge = FALSE;
+	BOOLEAN read_cross = FALSE;
 	BOOLEAN old_GMT_world_map, skip_if_outside = TRUE;
 
 	double xy[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0;
 	double plot_x, plot_y, scale = 0.0;
-        double vxy[2], plot_vx, plot_vy;
-        double eps1 = 0.0, eps2 = 0.0, spin = 0.0, spinsig = 0.0, theta = 0.0;
+	double vxy[2], plot_vx, plot_vy;
+	double eps1 = 0.0, eps2 = 0.0, spin = 0.0, spinsig = 0.0, theta = 0.0;
 	double direction, small_axis, great_axis; 
-        double sigma_x, sigma_y, corr_xy; 
-        double confidence = 0., sigma_scale = 1.0, conrad = 1.0;
+	double sigma_x, sigma_y, corr_xy; 
+	double confidence = 0., sigma_scale = 1.0, conrad = 1.0;
 	double v_width = 0.01, h_length = 0.12, h_width = 0.03, vector_shape = 0.4;
-        double wedge_amp = 1.e7;
+	double wedge_amp = 1.e7;
 	double t11 = 1.0, t12 = 0.0, t21 = 0.0, t22 = 1.0;
-        double hl,hw,vw, fontsize = 0.0;
+	double hl,hw,vw, fontsize = 0.0;
 
-        char station_name[20];
-        char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], txt_c[GMT_TEXT_LEN];
+	char station_name[20];
+	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], txt_c[GMT_TEXT_LEN];
 	char line[BUFSIZ], symbol_type, col[12][GMT_TEXT_LEN];
 
 	FILE *fp = NULL;
@@ -85,7 +85,7 @@ int main (int argc, char **argv)
 	struct GMT_PEN pen;
 	struct GMT_FILL fill, efill; /* efill is for uncertainty wedge */
 
-        int justify;
+	int justify;
 
 	argc = GMT_begin (argc, argv);
 
@@ -307,12 +307,12 @@ int main (int argc, char **argv)
 			sprintf (line, "File: %s", argv[fno]);
 			ps_comment (line);
 		}
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, 512, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, 512, fp);
 
 		if (gmtdefs.verbose && (read_ellipse || read_rotated_ellipse))	 
 		  fprintf (stderr, "psvelo: 2-D confidence interval and scaling factor %f %f\n",confidence, conrad);
 
-		while (fgets (line, 512, fp)) {
+		while (GMT_fgets (line, 512, fp)) {
                         n_rec++;
                         if (read_ellipse || read_rotated_ellipse) {
 			  sscanf (line, "%s %s %s %s %s %s %s %[^\n]\n", 
@@ -445,7 +445,7 @@ int main (int argc, char **argv)
                                         trace_cross(xy[0],xy[1],eps1,eps2,theta,scale,v_width,h_length,h_width,vector_shape,outline,pen);
                                         break;
                                 case WEDGE:
-                                        sprintf (line, "begin wedge number %i",n_rec);
+                                        sprintf (line, "begin wedge number %li",n_rec);
 					ps_comment (line);
                                         GMT_geo_to_xy(xy[0], xy[1], &plot_x, &plot_y);
                                         get_trans (xy[0], xy[1], &t11, &t12, &t21, &t22);
@@ -455,11 +455,11 @@ int main (int argc, char **argv)
                                         break;
 			}
 		}
-	if (fp != stdin) fclose (fp);
+	if (fp != stdin) GMT_fclose (fp);
 	}
 
 	if (gmtdefs.verbose) 
-			fprintf (stderr, "psvelo: Number of records read: %i\n", n_rec);
+			fprintf (stderr, "psvelo: Number of records read: %li\n", n_rec);
 
 	if (gmtdefs.verbose && rescale_sigma) 
 			fprintf (stderr, "psvelo: Rescaling uncertainties by a factor of %f\n", sigma_scale);
