@@ -1,5 +1,5 @@
-/*	$Id: utilmeca.c,v 1.7 2007/02/02 16:23:20 pwessel Exp $
- *    Copyright (c) 1996-2007 by G. Patau
+/*	$Id: utilmeca.c,v 1.12 2008/03/24 08:58:32 guru Exp $
+ *    Copyright (c) 1996-2008 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  */
@@ -77,7 +77,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
     double str, radius, increment;
     double si, co;
 
-    int lineout = 1, i, npoints;
+	int lineout = 1, i;
+	GMT_LONG npoints;
 
     struct AXIS N_axis;
 
@@ -103,7 +104,7 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         i = -1;
         increment = 1;
         str = meca.NP1.str;
-        while(str <= meca.NP1.str + 180.) {
+        while(str <= meca.NP1.str + 180. + EPSIL) {
             i++;
             radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -114,7 +115,7 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(fabs(fault + 1.) < EPSIL) {
             /* normal fault, close first compressing part */
             str = meca.NP1.str + 180.; 
-            while(str >= meca.NP1.str) {
+            while(str >= meca.NP1.str - EPSIL) {
                 i++;
                 sincosd (str, &si, &co);
                 x[i] = x0 + si * radius_size;
@@ -122,12 +123,12 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
                 str -= increment;
             }
             npoints = i + 1;
-            ps_polygon(x, y, npoints, rgb, outline);
+            ps_polygon (x, y, npoints, rgb, outline);
             i = -1;
         }
         /* second nodal plane part */
         str = meca.NP2.str;
-        while(str <= meca.NP2.str + 180.) {
+        while(str <= meca.NP2.str + 180. + EPSIL) {
             i++;
             radius = proj_radius(meca.NP2.str, meca.NP2.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -143,7 +144,7 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         else {
             /* normal fault, close second compressing part */
             str = meca.NP2.str + 180.;
-            while(str >= meca.NP2.str) {
+            while(str >= meca.NP2.str - EPSIL) {
                 i++;
                 sincosd (str, &si, &co);
                 x[i] = x0 + si * radius_size;
@@ -174,7 +175,9 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         /* second compressing part */
         i = 0;
         str = meca.NP1.str + 180.;
-        while(increment == 1 ? str <= meca.NP1.str + 270. : str >= meca.NP1.str + 90.) {
+        while(increment == 1 ? 
+	      str <= meca.NP1.str + 270. + EPSIL : 
+	      str >= meca.NP1.str + 90. - EPSIL) {
             sincosd (str, &si, &co);
             x[i] = x0 + si * radius_size;
             y[i] = y0 + co * radius_size;
@@ -194,7 +197,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(meca.NP1.str > N_axis.str)
             meca.NP1.str -= 360.;
         str = meca.NP1.str;
-        while(fabs(90. - meca.NP1.dip) < EPSIL ? str <= meca.NP1.str : str <= N_axis.str) {
+        while(fabs(90. - meca.NP1.dip) < EPSIL ? 
+	      str <= meca.NP1.str + EPSIL : str <= N_axis.str + EPSIL) {
             i++;
             radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -211,7 +215,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(fault * (meca.NP2.str - N_axis.str) < -EPSIL)
             meca.NP2.str += fault * 360.;
         str = fabs(90. - meca.NP2.dip) < EPSIL ? meca.NP2.str : N_axis.str;
-        while(increment == 1. ? str <= meca.NP2.str : str >= meca.NP2.str) {
+        while(increment == 1. ? 
+	      str <= meca.NP2.str + EPSIL : str >= meca.NP2.str - EPSIL) {
             i++;
             radius = proj_radius(meca.NP2.str - (1. + fault) * 90., meca.NP2.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -227,7 +232,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(increment * (meca.NP1.str - meca.NP2.str) < - EPSIL)
             meca.NP1.str += increment * 360.;
         str = meca.NP2.str;
-        while(increment == 1. ? str <= meca.NP1.str : str >= meca.NP1.str) {
+        while(increment == 1. ? 
+	      str <= meca.NP1.str + EPSIL : str >= meca.NP1.str - EPSIL) {
             i++;
             sincosd (str, &si, &co);
             x[i] = x0 + si * radius_size;
@@ -245,7 +251,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
             meca.NP1.str += 360.;
         increment = -1.;
         str = meca.NP1.str;
-        while(fabs(90. - meca.NP1.dip) < EPSIL ? str >= meca.NP1.str : str >= N_axis.str) {
+        while(fabs(90. - meca.NP1.dip) < EPSIL ? 
+	      str >= meca.NP1.str -EPSIL : str >= N_axis.str - EPSIL) {
             i++;
             radius = proj_radius(meca.NP1.str - 180., meca.NP1.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -260,7 +267,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(fault * (N_axis.str - meca.NP2.str) < - EPSIL)
             meca.NP2.str -= fault * 360.; 
         str = fabs(90. - meca.NP2.dip) < EPSIL ? meca.NP2.str : N_axis.str;
-        while(increment == 1. ? str <= meca.NP2.str : str >= meca.NP2.str) {
+        while(increment == 1. ? 
+	      str <= meca.NP2.str + EPSIL : str >= meca.NP2.str - EPSIL) {
             i++;
             radius = proj_radius(meca.NP2.str - (1. - fault) * 90., meca.NP2.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -276,7 +284,8 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
         if(increment * (meca.NP1.str - meca.NP2.str) < - EPSIL)
             meca.NP1.str += increment * 360.;
         str = meca.NP2.str;
-        while(increment == 1. ? str <= meca.NP1.str : str >= meca.NP1.str) {
+        while(increment == 1. ? 
+	      str <= meca.NP1.str + EPSIL : str >= meca.NP1.str - EPSIL) {
             i++;
             sincosd (str, &si, &co);
             x[i] = x0 + si * radius_size;
@@ -297,7 +306,8 @@ double ps_meca(double x0,double y0,st_me meca,double size)
 
 {
 
-    int i, npoints;
+	int i;
+	GMT_LONG npoints;
 
     double proj_radius();
 
@@ -325,7 +335,7 @@ double ps_meca(double x0,double y0,st_me meca,double size)
         i = -1;
         increment = 1;
         str = meca.NP1.str;
-        while(str <= meca.NP1.str + 180.) {
+        while(str <= meca.NP1.str + 180. + EPSIL) {
             i++;
             radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;   
             sincosd (str, &si, &co);
@@ -339,7 +349,7 @@ double ps_meca(double x0,double y0,st_me meca,double size)
         i = -1;
         increment = 1;
         str = meca.NP2.str;
-        while(str <= meca.NP2.str + 180.) {
+        while(str <= meca.NP2.str + 180. + EPSIL) {
             i++;
             radius = proj_radius(meca.NP2.str, meca.NP2.dip, str) * radius_size;
             sincosd (str, &si, &co);
@@ -359,7 +369,8 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
 
 {
 
-    int i, npoints;
+	int i;
+	GMT_LONG npoints;
 
     double proj_radius();
 
@@ -388,7 +399,7 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
                i = -1;
                increment = 1;
                str = meca.NP1.str;
-               while(str <= meca.NP1.str + 180.) {
+               while(str <= meca.NP1.str + 180. + EPSIL) {
                    i++;
                    radius = proj_radius(meca.NP1.str, meca.NP1.dip, str) * radius_size;
                    sincosd (str, &si, &co);
@@ -404,7 +415,7 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
                i = -1;
                increment = 1;
                str = meca.NP2.str;
-               while(str <= meca.NP2.str + 180.) {
+               while(str <= meca.NP2.str + 180. + EPSIL) {
                    i++;
                    radius = proj_radius(meca.NP2.str, meca.NP2.dip, str) * radius_size;
                    sincosd (str, &si, &co);
@@ -474,7 +485,7 @@ double datan2(double y,double x)
         if(fabs(y) < EPSIL) {
 /*
             fprintf(stderr, "undetermined form 0. / 0.");
-            exit();
+            exit(EXIT_FAILURE);
 */
             arctg = 0.;
         }
@@ -736,7 +747,7 @@ double proj_radius(double str1,double dip1,double str)
         printf("\nFor ps_mechanism r == 1 for str = str1");
         printf("\n            else r == 0. is used.");
 */
-        r = (str == str1 || str == str1 + 180) ? 1. : 0.;
+        r = (fabs(str - str1) < EPSIL || fabs(str - str1 - 180) < EPSIL) ? 1. : 0.;
     }
     else {
         dip = atan(tand(dip1) * sind(str - str1));
@@ -762,13 +773,13 @@ Genevieve Patau, 18 mars 1999
 #define ROTATE(a,i,j,k,l) g=a[i][j];h=a[k][l];a[i][j]=g-s*(h+g*tau);\
         a[k][l]=h+s*(g-h*tau);
 
-void jacobi(float **a, int n, float d[], float **v, int *nrot)
+void jacobi(float **a, GMT_LONG n, float d[], float **v, int *nrot)
 {
-        int j,iq,ip,i;
+        GMT_LONG j,iq,ip,i;
         float tresh,theta,tau,t,sm,s,h,g,c,*b,*z;
 
-        b=vector(1,n);
-        z=vector(1,n);
+        b=vector((size_t)1,n);
+        z=vector((size_t)1,n);
         for (ip=1;ip<=n;ip++) {
                 for (iq=1;iq<=n;iq++) v[ip][iq]=0.0;
                 v[ip][ip]=1.0;
@@ -785,8 +796,8 @@ void jacobi(float **a, int n, float d[], float **v, int *nrot)
                                 sm += (float)fabs((double)a[ip][iq]);
                 }
                 if (sm == 0.0) {
-                        free_vector(z,1,n);
-                        free_vector(b,1,n);
+                        free_vector(z,(size_t)1,n);
+                        free_vector(b,(size_t)1,n);
                         return;
                 }
                 if (i < 4)
@@ -869,11 +880,11 @@ void momten2axe(struct M_TENSOR mt,struct AXIS *T,struct AXIS *N,struct AXIS *P)
     a[1][0]=(float)mt.f[3]; a[1][1]=(float)mt.f[1]; a[1][2]=(float)mt.f[5];
     a[2][0]=(float)mt.f[4]; a[2][1]=(float)mt.f[5]; a[2][2]=(float)mt.f[2];
 
-    d=vector(1,NP);
-    r=vector(1,NP);
-    v=matrix(1,NP,1,NP);
-    e=convert_matrix(&a[0][0],1,num,1,num);
-    jacobi(e,num,d,v,&nrot);
+    d=vector((GMT_LONG)1,(GMT_LONG)NP);
+    r=vector((GMT_LONG)1,(GMT_LONG)NP);
+    v=matrix((GMT_LONG)1,(GMT_LONG)NP,(GMT_LONG)1,(GMT_LONG)NP);
+    e=convert_matrix(&a[0][0],(GMT_LONG)1,(GMT_LONG)num,(GMT_LONG)1,(GMT_LONG)num);
+    jacobi(e,(GMT_LONG)num,d,v,&nrot);
 
 /* sort eigenvalues */
     max = -10000.;
@@ -904,7 +915,7 @@ double ps_tensor(double x0,double y0,double size,struct AXIS T,struct AXIS N,str
 {
     int d, b = 1, m;
     int i, ii, n = 0, j = 1, j2 = 0, j3 = 0;
-    int npoints;
+    GMT_LONG npoints;
     int lineout = 1;
     int rgb1[3], rgb2[3];
     int big_iso = 0;
