@@ -1,5 +1,5 @@
-/*	$Id: utilstrain.c,v 1.5 2007/02/02 16:25:38 pwessel Exp $
- *    Copyright (c) 1996-2007 by G. Patau
+/*	$Id: utilstrain.c,v 1.8 2008/03/24 08:58:32 guru Exp $
+ *    Copyright (c) 1996-2008 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  */
@@ -384,7 +384,7 @@ void paint_wedge (double x0, double y0, double spin, double spinsig, double ssca
 
 #define NPOINTS 1000
 
-     int    npoints = NPOINTS;
+     GMT_LONG    npoints = NPOINTS;
      int  i;
 
      /* relative to center of ellipse */
