@@ -1,7 +1,7 @@
 /*
- *	$Id: grdread.c,v 1.6 2007/03/14 16:56:56 pwessel Exp $
+ *	$Id: grdread.c,v 1.10 2008/04/29 21:05:53 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -25,11 +25,11 @@
  *		11/12/98 P Wessel, ANSI-C and calls GMT_begin()
  *		10/07/99 P Wessel, Did not set x,y if [x,y,z,d] was used
  *		10/20/03 P Wessel, longer path names [R Mueller]
+ *		4/22/08 P Wessel, Now works with either Matlab or Octave
  */
  
 #include "gmt.h"
 #include "mex.h"
-#include "matrix.h"
 
 int grdread (double z_8[], double info[], char *filein, struct GRD_HEADER *grd)
 {
@@ -85,7 +85,7 @@ int grdread (double z_8[], double info[], char *filein, struct GRD_HEADER *grd)
 void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
 {
 	struct GRD_HEADER grd;
-	double *z_8, *info = (double *)NULL, *x, *y, off;
+	double *z_8, *info = (double *)NULL, *x = NULL, *y = NULL, off;
 	char *filein, *argv = "grdread-mex";
 	int error, ns, ssz, pz, i;
  
@@ -148,7 +148,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
  
 	/* Call grdread to get the contents of the file */
  
-	if (error = grdread (z_8, info, filein, &grd)) {
+	if ((error = grdread (z_8, info, filein, &grd))) {
 		if (error == 1)
 			mexErrMsgTxt ("grdread: failure to allocate memory\n");
 		else
@@ -162,4 +162,3 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
 	mxFree (filein);
 	return;
 }
-
