@@ -1,7 +1,7 @@
 /*
- *	$Id: grdwrite.c,v 1.6 2007/03/14 16:56:56 pwessel Exp $
+ *	$Id: grdwrite.c,v 1.10 2008/04/29 21:05:53 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -24,12 +24,17 @@
  *		10/06/98 P Wessel, upgrade to GMT 3.1 function calls
  *		11/12/98 P Wessel, ANSI-C and calls GMT_begin()
  *		10/20/03 P Wessel, longer path names [R Mueller]
+ *		4/22/08 P Wessel, Now works with either Matlab or Octave
 */
  
 #include "gmt.h"
 #include "mex.h"
-#include "matrix.h"
 
+#ifdef GMT_MATLAB
+#define MEX_PROG "Matlab"
+#else
+#define MEX_PROG "Octave"
+#endif
 
 int grdwrite (double z_8[], double x[], double y[], double info[], char *fileout, char *title, int nx, int ny, int pix)
 {
@@ -82,7 +87,7 @@ int grdwrite (double z_8[], double x[], double y[], double info[], char *fileout
 		grd.node_offset = pix;
 	}
 		
-	strcpy (grd.remark, "File written from Matlab with grdwrite");
+	sprintf (grd.remark, "File written from %s with grdwrite", MEX_PROG);
 	
 	/*  Allocate memory */
 
@@ -114,7 +119,7 @@ int grdwrite (double z_8[], double x[], double y[], double info[], char *fileout
 
 void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
 {
-	double *z_8, *info = (double *)NULL, *x, *y;
+	double *z_8, *info = (double *)NULL, *x = NULL, *y = NULL;
 	char *fileout, title[80], *argv = "grdwrite-mex";
 	int error, ns, ssz, nx, ny, k, pix = 0;	/* If no info we assume gridline reg */
 
@@ -174,7 +179,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
 
 	/* Do the actual computations in a subroutine */
  
-	if (error = grdwrite (z_8, x, y, info, fileout, title, nx, ny, pix)) {
+	if ((error = grdwrite (z_8, x, y, info, fileout, title, nx, ny, pix))) {
 		if (error == 1)
 			mexErrMsgTxt ("grdwrite: failure to allocate memory\n");
 		else if (error == 2)
