@@ -1,7 +1,7 @@
 /*-------------------------------------------------------------------------
- *	$Id: mgd77.h,v 1.77 2007/04/02 15:25:57 remko Exp $
+ *	$Id: mgd77.h,v 1.90 2008/04/22 03:34:38 mtchandl Exp $
  * 
- *    Copyright (c) 2005-2007 by P. Wessel
+ *    Copyright (c) 2005-2008 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *
  *  File:	mgd77.h
@@ -103,9 +103,15 @@
 #define MGD77_IGF_1967		3
 #define MGD77_IGF_1980		4
 
+#define TWT_PDR_WRAP	10.0						/* The 10 second PDR wrap-around we see in SIO cruises */
+#define TWT_PDR_WRAP_TRIGGER	0.5 * TWT_PDR_WRAP	/* Any jump in TWT that exceeds this triggers a wrap */
+
 #define GMT_IMG_MINLAT -72.0059773539
 #define GMT_IMG_MAXLAT +72.0059773539
 
+#define MGD77_BAD_HEADER_RECNO		-1
+#define MGD77_BAD_HEADER_ITEM		-2
+
 /* Return error numbers */
 
 #define MGD77_NO_ERROR			0
@@ -131,6 +137,12 @@
 #define MGD77_BAD_ARG			20
 #define MGD77_BAD_IGRFDATE		21
 
+/* For mgd77+ files */
+
+#define NCPOS_TIME	0
+#define NCPOS_LAT	3
+#define NCPOS_LON	4
+
 /* We will use bit flags to keep track of which data column we are referring to.
  * field 0 is rightmost bit (1), field 1 is the next bit (2), field 2 is 4 and
  * so on for powers of 2.  We add these bit powers together to get the bit patterns
@@ -173,95 +185,6 @@ typedef char* Text;	/* Used to indicate character strings */
  * MGD77_write_header will do exactly what they say.
  */
 
-struct MGD77_HEADER_PARAMS {		/* See MGD-77 Documentation from NGDC for details */
-/* START OF MGD77_HEADER_PARAMS */
-	/* Sequence No 01: */
-	char	Record_Type;
-	char	Survey_Identifier[9];
-	char	Format_Acronym[6];
-	char	Data_Center_File_Number[9];
-	char	Paramaters_Surveyed_Code[6];
-	char	File_Creation_Year[5];
-	char	File_Creation_Month[3];
-	char	File_Creation_Day[3];
-	char	Source_Institution[40];
-	/* Sequence No 02: */
-	char	Country[19];
-	char	Platform_Name[22];
-	char	Platform_Type_Code;
-	char	Platform_Type[7];
-	char	Chief_Scientist[33];
-	/* Sequence No 03: */
-	char	Project_Cruise_Leg[59];
-	char	Funding[21];
-	/* Sequence No 04: */
-	char	Survey_Departure_Year[5];
-	char	Survey_Departure_Month[3];
-	char	Survey_Departure_Day[3];
-	char	Port_of_Departure[33];
-	char	Survey_Arrival_Year[5];
-	char	Survey_Arrival_Month[3];
-	char	Survey_Arrival_Day[3];
-	char	Port_of_Arrival[31];
-	/* Sequence No 05: */
-	char	Navigation_Instrumentation[41];
-	char	Geodetic_Datum_Position_Determination_Method[39];
-	/* Sequence No 06: */
-	char	Bathymetry_Instrumentation[41];
-	char	Bathymetry_Add_Forms_of_Data[39];
-	/* Sequence No 07: */
-	char	Magnetics_Instrumentation[41];
-	char	Magnetics_Add_Forms_of_Data[39];
-	/* Sequence No 08: */
-	char	Gravity_Instrumentation[41];
-	char	Gravity_Add_Forms_of_Data[39];
-	/* Sequence No 09: */
-	char	Seismic_Instrumentation[41];
-	char	Seismic_Add_Forms_of_Data[39];
-	/* Sequence No 10: (Format_Description is split across 10 and 11 */
-	char	Format_Type;
-	char	Format_Description[95];
-	/* Sequence No 11: */
-	char	Topmost_Latitude[4];
-	char	Bottommost_Latitude[4];
-	char	Leftmost_Longitude[5];
-	char	Rightmost_Longitude[5];
-	/* Sequence No 12: */
-	char	Bathymetry_Digitizing_Rate[4];
-	char	Bathymetry_Sampling_Rate[13];
-	char	Bathymetry_Assumed_Sound_Velocity[6];
-	char	Bathymetry_Datum_Code[3];
-	char	Bathymetry_Interpolation_Scheme[57];
-	/* Sequence No 13: */
-	char	Magnetics_Digitizing_Rate[4];
-	char	Magnetics_Sampling_Rate[3];
-	char	Magnetics_Sensor_Tow_Distance[5];
-	char	Magnetics_Sensor_Depth[6];
-	char	Magnetics_Sensor_Separation[4];
-	char	Magnetics_Ref_Field_Code[3];
-	char	Magnetics_Ref_Field[13];
-	char	Magnetics_Method_Applying_Res_Field[48];
-	/* Sequence No 14: */
-	char	Gravity_Digitizing_Rate[4];
-	char	Gravity_Sampling_Rate[3];
-	char	Gravity_Theoretical_Formula_Code;
-	char	Gravity_Theoretical_Formula[18];
-	char	Gravity_Reference_System_Code;
-	char	Gravity_Reference_System[17];
-	char	Gravity_Corrections_Applied[39];
-	/* Sequence No 15: */
-	char	Gravity_Departure_Base_Station[8];
-	char	Gravity_Departure_Base_Station_Name[34];
-	char	Gravity_Arrival_Base_Station[8];
-	char	Gravity_Arrival_Base_Station_Name[32];
-	/* Sequence No 16+17: */
-	char	Number_of_Ten_Degree_Identifiers[3];
-	char	Ten_Degree_Identifier[151];
-	/* Sequence No 18-24: */
-	char	Additional_Documentation[7][79];
-/* END OF MGD77_HEADER_PARAMS */
-};
-
 #define MGD77_COL_ABBREV_LEN	16
 #define MGD77_COL_NAME_LEN	64
 #define MGD77_COL_UNIT_LEN	64
@@ -305,7 +228,8 @@ struct MGD77_HEADER {
 	struct MGD77_META meta;				/* Holds some meta-data derived directly from data records */
 	char *author;					/* Name of author of last creation/modification */
 	char *history;					/* History of creation/modifications */
-	int n_records;					/* Number of MGD77 data records found */
+	char *E77;					/* Statement of E77 information encoded */
+	GMT_LONG n_records;					/* Number of MGD77 data records found */
 	int n_fields;					/* Number of columns returned */
 	int errors[3];					/* Number of total errors, (warnings, errors) found when reading this header */
 	BOOLEAN no_time;				/* TRUE for those few cruises that have no time values */
@@ -324,6 +248,7 @@ struct MGD77_DATA_RECORD {	/* See MGD77 Documentation from NGDC for details */
 	double time;				/* Time using current GMT absolute time conventions (J2000 UTC) */
 	char word[MGD77_N_STRING_FIELDS][10];	/* The 3 text strings in MGD77 records */
 	unsigned int bit_pattern;		/* Bit pattern indicating which of the 27 fields are present in current record */
+	BOOLEAN keep_nav;				/* Set to false when navigation is bad */
 };
 
 struct MGD77_DATASET {	/* Info for an entire MGD77+ data set */
@@ -388,11 +313,12 @@ struct MGD77_CONTROL {
 	int verbose_dest;				/* 1 = to stdout, 2 = to stderr */
 	int nc_id;					/* netCDF ID for current open file (MGD77+ only) */
 	int nc_recid;					/* netCDF ID for dimension of records (time) */
-	int rec_no;					/* Current record to read/write for record-based i/o */
+	GMT_LONG rec_no;					/* Current record to read/write for record-based i/o */
 	int format;					/* 0 if any file format, 1 if MGD77, and 2 if netCDF, 3 if ascii table */
 	/* Format-related issues */
 	int time_format;				/* Either GMT_IS_ABSTIME or GMT_IS_RELTIME */
 	/* Data use information */
+	BOOLEAN original;				/* TRUE means we want original not revised header attributes */
 	BOOLEAN Want_Header_Item[MGD77_N_HEADER_ITEMS];	/* TRUE means print this header item if dump is selected */
 	BOOLEAN use_flags[MGD77_N_SETS];		/* TRUE means programs will use error bitflags (if present) when returning data */
 	BOOLEAN use_corrections[MGD77_N_SETS];		/* TRUE means we will apply correction factors (if present) when reading data */
@@ -439,9 +365,11 @@ struct MGD77_CORRTABLE {
 
 /* Primary user functions */
 
-extern void MGD77_Init (struct MGD77_CONTROL *F, BOOLEAN remove_blanks);						/* Initialize the MGD77 machinery */
+extern void MGD77_Init (struct MGD77_CONTROL *F);						/* Initialize the MGD77 machinery */
 extern void MGD77_Reset (struct MGD77_CONTROL *F);									/* Reset after finishing a file */
+extern void MGD77_end (struct MGD77_CONTROL *F);				/* Free up MGD77-related variables */
 extern int MGD77_Path_Expand (struct MGD77_CONTROL *F, char **argv, int argc, char ***list);				/* Returns the full list of IDs */
+extern void MGD77_Path_Free (int n, char **list);	/* Free the list of IDs */
 extern void MGD77_Cruise_Explain (void);										/* Explains how to specify IDs */
 extern int MGD77_Get_Path (char *track_path, char *track, struct MGD77_CONTROL *F);					/* Returns full path to cruise */
 extern int MGD77_Open_File (char *leg, struct MGD77_CONTROL *F, int rw);						/* Opens a MGD77[+] file */
@@ -462,18 +390,19 @@ extern void MGD77_List_Header_Items (struct MGD77_CONTROL *F);
 extern int MGD77_Select_Header_Item (struct MGD77_CONTROL *F, char *item);
 extern int MGD77_Get_Set (char *abbrev);										/* Returns 0 if abbrev is in the MGD77 set, else 1 */
 extern void MGD77_Fatal_Error (int error);										/* Print message for this error and exit */
-extern BOOLEAN MGD77_Pass_Record (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, int rec);				/* Tests if a record passes all specified logical & exact tests */
-extern void MGD77_Apply_Bitflags (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, int rec, BOOLEAN apply_bits[]);	/* Replaces values whose flags are ON with NaNs */
+extern BOOLEAN MGD77_Pass_Record (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, GMT_LONG rec);				/* Tests if a record passes all specified logical & exact tests */
+extern void MGD77_Apply_Bitflags (struct MGD77_CONTROL *F, struct MGD77_DATASET *S, GMT_LONG rec, BOOLEAN apply_bits[]);	/* Replaces values whose flags are ON with NaNs */
 extern void MGD77_Set_Unit (char *dist, double *scale, int way);							/* Convert appended distance unit to a numerical scale to give meters */
 extern void MGD77_nc_status (int status);										/* Checks for netCDF errors and aborts with error message */
-extern void MGD77_Process_Ignore (char code, char format);								/* Process the ignre-format option */
+extern void MGD77_Process_Ignore (char code, char *format);								/* Process the ignre-format option */
 extern void MGD77_Ignore_Format (int format);										/* Dissallow some formats for consideration */
 extern struct MGD77_DATASET *MGD77_Create_Dataset ();									/* Create an empty data set structure */
 extern void MGD77_Prep_Header_cdf (struct MGD77_CONTROL *F, struct MGD77_DATASET *S);					/* Prepare header before we write */
 extern void MGD77_Dump_Header_Params (struct MGD77_CONTROL *F, struct MGD77_HEADER_PARAMS *P);				/* Dump of header items, one per line */
 extern void MGD77_Verify_Header (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, FILE *ufp);				/* Verify content of header per MGD77 docs */
 extern void MGD77_Verify_Prep (struct MGD77_CONTROL *F, struct MGD77_DATASET *D);
-extern void MGD77_Verify_Prep_m77 (struct MGD77_CONTROL *F, struct MGD77_META *C, struct MGD77_DATA_RECORD *D, int nrec);
+extern void MGD77_Verify_Prep_m77 (struct MGD77_CONTROL *F, struct MGD77_META *C, struct MGD77_DATA_RECORD *D, GMT_LONG nrec);
+extern int MGD77_Remove_E77 (struct MGD77_CONTROL *F);
 
 /* Secondary user functions */
 
@@ -484,10 +413,11 @@ extern int MGD77_Write_Data_Record_m77 (struct MGD77_CONTROL *F, struct MGD77_DA
 
 /* These are only for developers */
 
-extern int MGD77_dbl_are_constant (double x[], int n, double limits[]);
-extern int MGD77_txt_are_constant (char *txt, int n, int width);
-extern int MGD77_do_scale_offset_before_write (double new[], const double x[], int n, double scale, double offset, int type);
+extern BOOLEAN MGD77_dbl_are_constant (double x[], GMT_LONG n, double limits[]);
+extern BOOLEAN MGD77_txt_are_constant (char *txt, GMT_LONG n, int width);
+extern int MGD77_do_scale_offset_before_write (double new[], const double x[], GMT_LONG n, double scale, double offset, int type);
 extern void MGD77_select_high_resolution ();
+extern void MGD77_free_plain_mgd77 (struct MGD77_HEADER *H);
 
 /* User functions for direct use of Carter corrections */
 
@@ -508,7 +438,7 @@ extern void MGD77_IGF_text (FILE *fp, int version);
 extern int MGD77_carter_init (struct MGD77_CARTER *C);
 extern int MGD77_carter_get_bin (double lon, double lat, int *bin);
 extern int MGD77_carter_get_zone (int bin, struct MGD77_CARTER *C, int *zone);
-extern double *MGD77_Distances (double x[], double y[], int n, int dist_flag);
+extern double *MGD77_Distances (double x[], double y[], GMT_LONG n, int dist_flag);
 
 /* Global variables used by MGD77 programs */
 
@@ -521,6 +451,8 @@ extern int MGD77_pos[MGD77_N_DATA_EXTENDED];
 
 void MGD77_Parse_Corrtable (struct MGD77_CONTROL *F, char *tablefile, char **cruises, int n_cruises, struct MGD77_CORRTABLE ***CORR);
 void MGD77_Init_Correction (struct MGD77_CORRTABLE *CORR, double **value);
-double MGD77_Correction (struct MGD77_CORRECTION *C, double **value, double *aux, int rec);
+double MGD77_Correction (struct MGD77_CORRECTION *C, double **value, double *aux, GMT_LONG rec);
+
+#include "mgd77_functions.h"	/* These were created by mgd77netcdfhelper.sh */
 
 #endif	/* _MGD77_H */
