@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77list.c,v 1.76 2007/03/12 21:45:19 pwessel Exp $
+ *	$Id: mgd77list.c,v 1.88 2008/04/17 01:24:29 guru Exp $
  *
- *    Copyright (c) 2004-2007 by P. Wessel
+ *    Copyright (c) 2004-2008 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -27,6 +27,7 @@
  * Version:	1.0 Based somewhat on the old gmtlist.c
  *		31-MAR-2006: Changed -X to -L to avoid GMT collision
  *		23-MAY-2006: Added -Q for limits on speed/azimuths
+ *		21-FEB-2008: Added -Ga|b<rec> for limits on rec range
  *
  *
  */
@@ -44,9 +45,9 @@
 
 struct AUXLIST {
 	char name[MGD77_COL_ABBREV_LEN];
-	int type;
-	int text;
-	int requested;
+	GMT_LONG type;
+	GMT_LONG text;
+	GMT_LONG requested;
 	char header[GMT_TEXT_LEN];
 };
 #define AUX_YR	0
@@ -65,9 +66,9 @@ struct AUXLIST {
 #define AUX_GR	13
 #define AUX_ID	14
 struct AUX {
-	int type;
-	int text;
-	int pos;
+	GMT_LONG type;
+	GMT_LONG text;
+	GMT_LONG pos;
 };
 
 #define ADJ_CT	0
@@ -77,16 +78,18 @@ struct AUX {
 
 int main (int argc, char **argv)
 {
-	int i, c, id, k, kx, pos, rec, prevrec, n_out = 0, argno, code, n_cruises = 0, n_paths;
+	int i, c, id, k, kx, pos, argno, code, n_cruises = 0, n_paths;
 	int t_col, x_col, y_col, z_col, m_col = 0, f_col = 0, g_col = 0, m1_col = 0, m2_col = 0, ms_col = 0, twt_col = 0;
 	int n_sub, n_out_columns, n_cols_to_process, n_aux, select_option, adj_code[4];
 	int time_column, lon_column, lat_column, dist_flag = 2, GF_version = MGD77_NOT_SET;
 	
+	GMT_LONG rec, prevrec, n_out = 0, start_rec = 0, stop_rec;
+	
 	BOOLEAN error = FALSE, exact = FALSE, string_output = FALSE, limit_institution = FALSE, need_depth = FALSE;
-	BOOLEAN negative_depth = FALSE,  negative_msd = FALSE, need_distances, need_time, limit_on_time = FALSE;
+	BOOLEAN negative_depth = FALSE,  negative_msd = FALSE, need_distances, need_time, limit_on_time = FALSE, limit_on_recs = FALSE;
 	BOOLEAN limit_on_dist = FALSE, limit_on_wesn = FALSE, need_lonlat = FALSE, first_cruise = TRUE, need_twt = FALSE;
-	BOOLEAN need_date, need_sound = FALSE, apply_corrections = FALSE, apply_bits[MGD77_N_SETS], might_need_bits;
-	BOOLEAN limit_on_azimuth = FALSE, limit_on_velocity = FALSE, lonlat_not_NaN, first_warning = TRUE;
+	BOOLEAN need_date, need_sound = FALSE, apply_corrections = FALSE, PDR_wrap;
+	BOOLEAN limit_on_azimuth = FALSE, limit_on_velocity = FALSE, lonlat_not_NaN, first_warning = TRUE, has_prev_twt = FALSE;
 	
 	char *start_date = CNULL, *stop_date = CNULL, f_setting[BUFSIZ], fx_setting[BUFSIZ], **list;
 	char *tvalue[MGD77_MAX_COLS], institution[3], s_unit[2], d_unit[2], *aux_tvalue[N_AUX], *correction_table = CNULL;
@@ -94,7 +97,7 @@ int main (int argc, char **argv)
 	double west, east, south, north, start_time, stop_time, start_dist, IGRF[7], correction, min_vel = 0.0, max_vel = DBL_MAX;
 	double stop_dist, dist_scale, vel_scale, ds, dt, cumulative_dist, aux_dvalue[N_AUX], *out = VNULL;
 	double sound_speed = 0.0, i_sound_speed = 0.0, n_days, date = 0.0, g, m, z, v, twt, *dvalue[MGD77_MAX_COLS];
-	double min_az = 0.0, max_az = 360.0;
+	double min_az = 0.0, max_az = 360.0, prev_twt = 0, d_twt, twt_pdrwrap_corr;
 	
 	struct MGD77_CONTROL M;
 	struct MGD77_DATASET *D;
@@ -122,20 +125,21 @@ int main (int argc, char **argv)
 	
 	PFD GMT_azimuth_func;
 	
-	int separate_aux_columns (struct MGD77_CONTROL *F, char *fx_setting, struct AUX *aux, struct AUXLIST *A);
+	GMT_LONG separate_aux_columns (struct MGD77_CONTROL *F, char *fx_setting, struct AUX *aux, struct AUXLIST *A);
 
 	argc = GMT_begin (argc, argv);		/* Initialize GMT Machinery */
 	
 	/* Initialize MGD77 output order and other parameters*/
 	
-	MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	west = 0.0;	east = 360.0;	south = -90.0;	north = 90.0;
 	start_time = start_dist = 0.0;	/* Default covers all times and distances */
 	stop_time = stop_dist = DBL_MAX;
+	stop_rec = INT_MAX;
 	aux_dvalue[AUX_WT] = 1.0;	/* Default weight */
 	aux_dvalue[AUX_RT] = 5.0;	/* Default record type */
-	memset ((void *)adj_code, 0, 4*sizeof (int));
-	memset ((void *)institution, 0, 3);
+	memset ((void *)adj_code, 0, 4*sizeof (GMT_LONG));
+	memset ((void *)institution, 0, (size_t)3);
 	d_unit[0] = 'k';	d_unit[1] = '\0';	/* Default is -Ndk */
 	s_unit[0] = 'e';	s_unit[1] = '\0';	/* Default is -Nse */
 	strcpy (f_setting, MGD77_ALL);	/* Default is a full MGD77 record */
@@ -278,16 +282,20 @@ int main (int argc, char **argv)
 				}
 				break;
 	
-			case 'G':	/* Specify normal gravity equation directly */
-				GF_version = atoi (&argv[i][2]);
-				if (GF_version < MGD77_IGF_HEISKANEN || GF_version > MGD77_IGF_1980) {
-					fprintf(stderr, "%s: ERROR -G: Gravity formula code must be 1-4\n", GMT_program);
-					error++;
+			case 'G':		/* Assign start/stop records for sub-section */
+				limit_on_recs = TRUE;
+				if (argv[i][2] == 'a') {		/* Start rec */
+					start_rec = atoi (&argv[i][3]);
 				}
+				else if (argv[i][2] == 'b')	 {	/* Stop rec */
+					stop_rec = atoi (&argv[i][3]);
+				}
+				else
+					error = TRUE;
 				break;
 	
 			case 'I':
-				MGD77_Process_Ignore (argv[i][1], argv[i][2]);
+				MGD77_Process_Ignore (argv[i][1], &argv[i][2]);
 				break;
 
 			case 'L':	/* Crossover correction table */
@@ -531,7 +539,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-E Only records that exactly matches the requested geophysical information in -F will be used.\n");
 		fprintf (stderr, "\t   [Default will output all record that matches at least one column]\n");
 		fprintf (stderr, "\t-H Write one header record with column names\n");
-		fprintf (stderr, "\t-I Ignore certain data file formats from consideration. Append a|c|t to ignore\n");
+		fprintf (stderr, "\t-I Ignore certain data file formats from consideration. Append combination of act to ignore\n");
 		fprintf (stderr, "\t   (a) MGD77 ASCII, (c) MGD77+ netCDF, or (t) plain table files. [Default ignores none]\n");
 		fprintf (stderr, "\t-L Subtract systematic corrections from the data. If no correction file is given,\n");
 		fprintf (stderr, "\t   the default file mgd77_corrections.d in $MGD77_HOME is assumed.\n");
@@ -543,9 +551,9 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   n Nautical units (nautical miles, knots)\n");
 		fprintf (stderr, "\t   [Default is -Ndk -Nse]\n");
 		fprintf (stderr, "\t-Q Only return data whose azimuth (-Qa) or velocity (-Qv) fall inside specified range:\n");
-		fprintf (stderr, "\t   -Qa<min_az>/<max_az>, where <min_az> < <max_az> [all azimuths, i.e. 0/360]");
-		fprintf (stderr, "\t   -Qv<min_vel>[/<max_vel>], where <max_vel> is optional [all velocities, i.e., 0/infinity]");
-		fprintf (stderr, "\t      Velocities are given in m/s unless changed by -Ns");
+		fprintf (stderr, "\t   -Qa<min_az>/<max_az>, where <min_az> < <max_az> [all azimuths, i.e. 0/360]\n");
+		fprintf (stderr, "\t   -Qv<min_vel>[/<max_vel>], where <max_vel> is optional [all velocities, i.e., 0/infinity]\n");
+		fprintf (stderr, "\t      Velocities are given in m/s unless changed by -Ns\n");
 		fprintf (stderr, "\t-R Only return data inside the specified region [0/360/-90/90]\n");
 		fprintf (stderr, "\t-Sa<dist> lists from dist  (in m; append k, m, or n) [Start of the cruise]\n");
 		fprintf (stderr, "\t-Sb<dist> lists up to dist (in m; append k, m, or n) [End of the cruise]\n");
@@ -574,7 +582,7 @@ int main (int argc, char **argv)
 	if (MGD77_Get_Column ("depth", &M) == MGD77_NOT_SET) negative_depth = FALSE;	/* Just so we don't accidently access dvalue[z_col] further down in the loop */
 	if (MGD77_Get_Column ("msd", &M) == MGD77_NOT_SET) negative_msd = FALSE;	/* Just so we don't accidently access dvalue[m_col] further down in the loop */
 	n_aux = separate_aux_columns (&M, fx_setting, aux, auxlist);				/* Determine which auxillary columns are requested (if any) */
-	aux_tvalue[AUX_ID] = (char *)GMT_memory (VNULL, GMT_TEXT_LEN, sizeof (char), GMT_program);	/* Just in case */
+	aux_tvalue[AUX_ID] = (char *)GMT_memory (VNULL, (size_t)GMT_TEXT_LEN, sizeof (char), GMT_program);	/* Just in case */
 	
 	/* Most auxillary columns depend on values in the data columns.  If the user did not specify the required data columns
 	 * then we must append them to make sure we have access to the values we need to calculate the auxillary values.
@@ -680,7 +688,7 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Now processing cruise %s\n", GMT_program, list[argno]);
 		
-		if (limit_institution && strncmp (M.NGDC_id, institution, 2)) {	/* Institution does not match specification */
+		if (limit_institution && strncmp (M.NGDC_id, institution, (size_t)2)) {	/* Institution does not match specification */
 			MGD77_Close_File (&M);
 			continue;
 		}
@@ -808,16 +816,13 @@ int main (int argc, char **argv)
 
 		if (apply_corrections) MGD77_Init_Correction (CORR[argno], dvalue);	/* Initialize origins if needed */
 		
-		apply_bits[MGD77_M77_SET] = (M.n_bit_tests == 0 && D->flags[MGD77_M77_SET] && M.use_flags[MGD77_M77_SET]); /* Will need to consider the bitflags for MGD77 set */
-		apply_bits[MGD77_CDF_SET] = (M.n_bit_tests == 0 && D->flags[MGD77_CDF_SET] && M.use_flags[MGD77_CDF_SET]); /* Will need to consider the bitflags for MGD77 set */
-		might_need_bits = (apply_bits[MGD77_M77_SET] || apply_bits[MGD77_CDF_SET]);	/* If FALSE we skip bit-checks entirely */
-
+		has_prev_twt = PDR_wrap = FALSE;
+		twt_pdrwrap_corr = 0.0;
+		
 		/* Start processing records  */
 		
 		for (rec = 0, prevrec = -1; rec < D->H.n_records; rec++) {
 		
-			if (might_need_bits) MGD77_Apply_Bitflags (&M, D, rec, apply_bits);	/* Possibly replace values with NaNs, according to the bitflags (if any) */
-
 			/* Compute accumulated distance along track (Great circles or Flat Earth) */
 		
 			if (need_distances) {
@@ -838,7 +843,7 @@ int main (int argc, char **argv)
 					}
 				}
 				if (auxlist[AUX_SP].requested) {
-					if (rec == 0) {	/* Initialize various counters */
+					if (rec == 0 || prevrec < 0) {	/* Initialize various counters */
 						dt = dvalue[t_col][1] - dvalue[t_col][0];
 						if (auxlist[AUX_SP].requested) aux_dvalue[AUX_SP] = (GMT_is_dnan (dt) || dt == 0.0) ? GMT_d_NaN : vel_scale * ds / dt;
 					}
@@ -850,8 +855,9 @@ int main (int argc, char **argv)
 				if (lonlat_not_NaN) prevrec = rec;	/* This was a record with OK lon,lat; make it the previous point for distance calculations */
 			}
 			
-			/* Check if time or distance falls outside specified range */
+			/* Check if rec no, time or distance falls outside specified ranges */
 		
+			if (limit_on_recs && (rec < start_rec || rec > stop_rec)) continue;
 			if (limit_on_dist && (cumulative_dist < start_dist || cumulative_dist > stop_dist)) continue;
 			if (limit_on_time && (dvalue[t_col][rec] < start_time || dvalue[t_col][rec] > stop_time)) continue;
 		
@@ -925,8 +931,10 @@ int main (int argc, char **argv)
 						aux_dvalue[AUX_CT] = z - dvalue[z_col][rec];
 					}
 				}
-				else
-					aux_dvalue[AUX_CT] = MGD77_carter_correction (dvalue[x_col][rec], dvalue[y_col][rec], 1000.0 * dvalue[twt_col][rec], &Carter);
+				else {
+					twt = 1000.0 * dvalue[twt_col][rec];
+					aux_dvalue[AUX_CT] = MGD77_carter_correction (dvalue[x_col][rec], dvalue[y_col][rec], twt, &Carter);
+				}
 				if (negative_depth) aux_dvalue[AUX_CT] = -aux_dvalue[AUX_CT];
 			}
 
@@ -937,8 +945,21 @@ int main (int argc, char **argv)
 				if (adj_code[ADJ_DP] & 2 && GMT_is_dnan (z))	/* Try uncorr. depth */
 					z = dvalue[twt_col][rec] * i_sound_speed;
 				if (adj_code[ADJ_DP] & 4 && GMT_is_dnan (z)) {	/* Try Carter depth */
-					twt = dvalue[twt_col][rec] * sound_speed;	/* Factor of 2 dealt with earlier */
-					MGD77_carter_depth_from_xytwt (dvalue[x_col][rec], dvalue[y_col][rec], dvalue[twt_col][rec], &Carter, &z);
+					twt = dvalue[twt_col][rec];
+					if (!GMT_is_dnan (twt)) {	/* OK, valid twt */
+						if (has_prev_twt) {	/* OK, may look at change in twt */
+							d_twt = twt - prev_twt;
+							if (fabs (d_twt) > TWT_PDR_WRAP_TRIGGER) {
+								twt_pdrwrap_corr += copysign (TWT_PDR_WRAP, -d_twt);
+								if (!PDR_wrap) fprintf (stderr, "%s: PDR travel time wrap detected for cruise %s\n", GMT_program, list[argno]);
+								PDR_wrap = TRUE;
+							}
+						}
+						has_prev_twt = TRUE;
+						prev_twt = twt;
+					}
+					twt += twt_pdrwrap_corr;
+					MGD77_carter_depth_from_xytwt (dvalue[x_col][rec], dvalue[y_col][rec], 1000.0 * twt, &Carter, &z);
 				}
 				dvalue[z_col][rec] = z;
 			}
@@ -1029,14 +1050,17 @@ int main (int argc, char **argv)
 	
 	if (!string_output) GMT_free ((void *)out);
 	
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Returned %d output records from %d cruises\n", GMT_program, n_out, n_cruises);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Returned %ld output records from %d cruises\n", GMT_program, n_out, n_cruises);
 	
+	MGD77_Path_Free (n_paths, list);
+	MGD77_end (&M);
+
 	exit (EXIT_SUCCESS);
 }
 
-int separate_aux_columns (struct MGD77_CONTROL *F, char *fx_setting, struct AUX *aux, struct AUXLIST *auxlist)
+GMT_LONG separate_aux_columns (struct MGD77_CONTROL *F, char *fx_setting, struct AUX *aux, struct AUXLIST *auxlist)
 {
-	int i, j, k, this_aux, n_aux;
+	GMT_LONG i, j, k, this_aux, n_aux;
 	
 	fx_setting[0] = '\0';
 	for (i = k = n_aux = 0; i < F->n_out_columns; i++) {
