@@ -1,8 +1,8 @@
 /* -------------------------------------------------------------------
- *	$Id: mgd77magref.h,v 1.6 2007/01/30 20:37:09 pwessel Exp $	
+ *	$Id: mgd77magref.h,v 1.10 2008/03/27 06:50:54 guru Exp $	
  *      See COPYING file for copying and redistribution conditions.
  *
- *    Copyright (c) 2005-2007 by P. Wessel and M. T. Chandler
+ *    Copyright (c) 2005-2008 by P. Wessel and M. T. Chandler
  *	File:	mgd77magref.h
  *
  *	MGD77 Magnetic Reference Fields for mgd77sniffer
@@ -20,6 +20,7 @@
  *
  * ------------------------------------------------------------------*/
 /*		  Field,		Code,	Start,		End */
+		{"Unused",      0,      9999,       9999},
 		{"AWC 70",		1,		1965,		1970},
 		{"AWC 75",		2,		1967,		1974},
 		{"IGRF-65",		3,		1965,		1970},
@@ -32,4 +33,6 @@
 		{"POGO 0869",	10,		1965,		1968},
 		{"IGRF-80",		11,		1980,		1985},
 		{"IGRF-85",		12,		1985,		1990},
-		{"IGRF-90",		13,		1990,		1995}
+		{"IGRF-90",		13,		1990,		1995},
+		{"IGRF-95",		14,		1995,		2000},
+		{"Other",       88,     9999,       9999}
