@@ -1,8 +1,8 @@
 /* -------------------------------------------------------------------
- *	$Id: mgd77sniffer.h,v 1.20 2007/01/31 01:40:43 mtchandl Exp $	
+ *	$Id: mgd77sniffer.h,v 1.31 2008/04/08 05:24:16 mtchandl Exp $	
  *      See COPYING file for copying and redistribution conditions.
  *
- *    Copyright (c) 2004-2007 by P. Wessel and M. T. Chandler
+ *    Copyright (c) 2004-2008 by P. Wessel and M. T. Chandler
  *	File:	mgd77sniffer.h
  *
  *	Include file for mgd77sniffer
@@ -18,6 +18,8 @@
  
  #include "gmt.h"
  #include "mgd77.h"
+ #include "mgd77_e77.h"
+ #include "mgd77_rls_coeffs.h"
  
 /* Constants */
 #define MGD77_N_DATA_FIELDS         27
@@ -38,6 +40,20 @@
 #define MGD77_MIN_RLS_PTS           100
 #define MGD77_N_MAG_RF              13
 #define MGD77_MAX_SEARCH            50
+#define MGD77_POS_BIT               (1 << 0)
+#define MGD77_ZERO_BIT              (1 << 1)
+#define MGD77_NEG_BIT               (1 << 2)
+
+/* RLS statistic array cell names */
+#define MGD77_RLS_SLOPE             0
+#define MGD77_RLS_ICEPT             1
+#define MGD77_RLS_STD               2
+#define MGD77_RLS_SXX               3
+#define MGD77_RLS_CORR              4
+#define MGD77_RLS_SIG               5
+#define MGD77_RLS_RMS               6
+#define MGD77_RLS_SUMX2             7
+#define MGD77_N_STATS               8
 
 /* LMS Limits - Obtained from 4616 bathy and 1657 gravity cruises */
 #define MGD77_MIN_DEPTH_SLOPE       0.900404              /* Q2.5 */
@@ -60,32 +76,6 @@
 #define SUMMARY_WARN         7
 #define MGD77_N_WARN_TYPES   8
 
-/* E77 Error classes */
-#define E77_NAV                0
-#define E77_VALUE              1
-#define E77_SLOPE              2
-#define E77_GRID               3
-#define N_ERROR_CLASSES        4
-#define N_DEFAULT_TYPES      MGD77_N_NUMBER_FIELDS
-
-/* E77 Nav Error Types */
-#define NAV_TIME_OOR         1          /* A */
-#define NAV_TIME_DECR        2          /* B */
-#define NAV_HISPD            4          /* C */
-#define NAV_ON_LAND          8          /* D */
-#define NAV_LAT_UNDEF       16          /* E */
-#define NAV_LON_UNDEF       32          /* F */
-#define N_NAV_TYPES          6
-
-/* E77 Header Errata Codes */
-#define E77_HDR_SCALE        1
-#define E77_HDR_OFFSET       2
-#define E77_HDR_BCC          3
-#define E77_HDR_PRECISION    4
-#define E77_HDR_FLAGRANGE    5
-#define E77_HDR_ANOM_FAA     6
-#define E77_HDR_ANOM_MAG     7
-
 /* MGD77 date constants */
 #define AUX_YR	0
 #define AUX_MO	1
@@ -97,8 +87,8 @@
 struct BAD_SECTION {	/* To flag a range of records as bad for given field */
 	char abbrev[8];	/* Field name */
 	int col;	/* Column number */
-	int start;	/* First record to flag */
-	int stop;	/* Last record to flag */
+	GMT_LONG start;	/* First record to flag */
+	GMT_LONG stop;	/* Last record to flag */
 };
 #define MAX_BAD_SECTIONS	100
 
@@ -120,6 +110,8 @@ struct MGD77_SNIFFER_DEFAULTS {
 	double maxTimeGrad;  /* maximum +/- time derivative     */
 	double maxSpaceGrad; /* maximum +/- space derivative    */
 	double maxDiff;      /* maximum +/- value change        */
+	double binmin;       /* minimum possible MGD77 value    */
+	double binmax;       /* maximum possible MGD77 value    */
 	double maxArea;      /* maximum grid offset area        */
 };
 
@@ -136,10 +128,11 @@ struct MGD77_MAG_RF {
 
 /* Local functions */
 void read_grid (struct MGD77_GRID_INFO *info, float **grid, double w, double e, double s, double n, BOOLEAN bilinear, double threshold);
-int sample_grid (struct MGD77_GRID_INFO *info, struct MGD77_DATA_RECORD *D, double **g, float *grid, int n_grid, int n);
-void regress_ls (double *x, double *y, int n, double *stat, int col, double S_xx);
-void regress_rls (double *x, double *y, int nvalues, double *stat, int col, double S_xx);
-void regress_lms (double *x, double *y, int nvalues, double *stat, int gridField);
-void regresslms_sub (double *x, double *y, double angle0, double angle1, int nvalues, int n_angle, double *stat, int gridField);
-double lms (double *x, int n);
-double median (double *x, int n);
+int sample_grid (struct MGD77_GRID_INFO *info, struct MGD77_DATA_RECORD *D, double **g, float *grid, GMT_LONG n_grid, GMT_LONG n);
+void regress_ls (double *x, double *y, GMT_LONG n, double *stat, int col);
+void regress_rls (double *x, double *y, GMT_LONG nvalues, double *stat, int col);
+void regress_lms (double *x, double *y, GMT_LONG nvalues, double *stat, int gridField);
+void regresslms_sub (double *x, double *y, double angle0, double angle1, GMT_LONG nvalues, int n_angle, double *stat, int gridField);
+GMT_LONG decimate (double *x, double *y, GMT_LONG nclean, double min, double max, double delta, double **dec_new, double **dec_orig, GMT_LONG *extreme, char *fieldTest);
+double lms (double *x, GMT_LONG n);
+double median (double *x, GMT_LONG n);
