@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77track.c,v 1.49 2007/03/26 01:54:58 pwessel Exp $
+ *	$Id: mgd77track.c,v 1.60 2008/04/17 01:24:29 guru Exp $
  *
- *    Copyright (c) 2004-2007 by P. Wessel
+ *    Copyright (c) 2004-2008 by P. Wessel
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -51,9 +51,9 @@ struct MGD77TRACK_MARKER {
 
 int main (int argc, char **argv)
 {
-	int i, j, rec, first_rec, last_rec, argno, n_cruises = 0;
-	int n_id = 0, mrk = 0, annot_name = 0, dist_flag = 2, n_paths;
-	int this_julian = 0, last_julian, n_alloc_c = GMT_SMALL_CHUNK;
+	GMT_LONG rec, first_rec, last_rec, n_alloc_c = GMT_SMALL_CHUNK;
+	int i, j, n_id = 0, mrk = 0, annot_name = 0, dist_flag = 2, n_paths;
+	int this_julian = 0, last_julian, argno, n_cruises = 0;
 	
 	BOOLEAN error = FALSE, first, both = FALSE, no_clip = FALSE;
 	BOOLEAN  annot_tick = FALSE, draw_tick = FALSE;
@@ -77,7 +77,8 @@ int main (int argc, char **argv)
 
 	int get_annotinfo (char *args, struct MGD77TRACK_ANNOT *info);
 	void annot_legname (double x, double y, double lon, double lat, double angle, char *text, double size);
-	double heading (int rec, double *lon, double *lat, int n_records);
+	double heading (GMT_LONG rec, double *lon, double *lat, GMT_LONG n_records);
+	BOOLEAN bad_coordinates (double lon, double lat);
 
 	argc = GMT_begin (argc, argv);		/* Initialize GMT Machinery */
 	
@@ -103,7 +104,7 @@ int main (int argc, char **argv)
 
 	/* Initialize MGD77 output order and other parameters*/
 	
-	MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	west = 0.0;	east = 360.0;	south = -90.0;	north = 90.0;
 	start_time = start_dist = 0.0;
 	stop_time = stop_dist = DBL_MAX;
@@ -157,8 +158,12 @@ int main (int argc, char **argv)
 					error = TRUE;
 				break;
 
+			case 'F':	/* Do NOT apply bitflags */
+				M.use_flags[MGD77_M77_SET] = M.use_flags[MGD77_CDF_SET] = FALSE;
+				break;
+				
 			case 'I':
-				MGD77_Process_Ignore (argv[i][1], argv[i][2]);
+				MGD77_Process_Ignore (argv[i][1], &argv[i][2]);
 				break;
 			case 'L':
 				error += get_annotinfo (&argv[i][2], &info);
@@ -258,7 +263,7 @@ int main (int argc, char **argv)
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
 		fprintf (stderr, "mgd77track %s - Plot track-line map of MGD77 cruises\n\n", MGD77_VERSION);
 		fprintf (stderr, "usage: mgd77track cruise(s) %s %s [-A[c][size]] [%s]\n", GMT_Rgeo_OPT, GMT_J_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-Cf|g|e] [-Da<startdate>] [-Db<stopdate>] [-I<code>] [-K] [-L<trackticks>] [-N] [-O] [-P] [-Sa<startdist>[unit]]\n");
+		fprintf (stderr, "\t[-Cf|g|e] [-Da<startdate>] [-Db<stopdate>] [-F] [-I<code>] [-K] [-L<trackticks>] [-N] [-O] [-P] [-Sa<startdist>[unit]]\n");
 		fprintf (stderr, "\t[-Sb<stopdist>[unit]] [-TT|t|d<ms,mc,mfs,mf,mfc>] [%s] [-V] [-W<pen>] [%s]\n", GMT_U_OPT, GMT_X_OPT);
 		fprintf (stderr, "\t[%s] [%s]\n\n", GMT_Y_OPT, GMT_c_OPT);
       
@@ -277,7 +282,8 @@ int main (int argc, char **argv)
 		fprintf (stderr,"	   e Ellipsoidal (geodesic) using current ellipsoid\n");
 		fprintf (stderr, "	-Da plots from <startdate> (given as yyyy-mm-ddT[hh:mm:ss]) [Start of cruise]\n");
 		fprintf (stderr, "	-Db plots up to <stopdate> (given as yyyy-mm-ddT[hh:mm:ss]) [End of cruise]\n");
-		fprintf (stderr, "	-I Ignore certain data file formats from consideration. Append a|c|t to ignore\n");
+		fprintf(stderr,"	-F Do NOT apply bitflags to MGD77+ cruises [Default applies error flags stored in the file]\n");
+		fprintf(stderr,"	-I Ignore certain data file formats from consideration. Append combination of act to ignore\n");
 		fprintf (stderr, "	   (a) MGD77 ASCII, (c) MGD77+ netCDF, or (t) plain table files. [Default ignores none]\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "	-L puts time/distance log marks on the track. E.g. a500ka24ht6h means (a)nnotate\n");
@@ -364,10 +370,10 @@ int main (int argc, char **argv)
 		track_time = (double*)D.values[0];
 		lon = (double*)D.values[1];
 		lat = (double*)D.values[2];
-		GMT_err_fail (GMT_distances (lon, lat, D.H.n_records, 1.0, dist_flag, track_dist), "");	/* Work internally in meters */
-		for (rec = 0; rec < D.H.n_records && track_time[rec] < start_time && track_dist[rec] < start_dist; rec++);	/* Find first record of interest */
+		GMT_err_fail (GMT_distances (lon, lat, D.H.n_records, 1.0, dist_flag, &track_dist), "");	/* Work internally in meters */
+		for (rec = 0; rec < D.H.n_records && bad_coordinates(lon[rec], lat[rec]) && track_time[rec] < start_time && track_dist[rec] < start_dist; rec++);	/* Find first record of interest */
 		first_rec = rec;
-		for (rec = D.H.n_records - 1; rec && track_time[rec] > stop_time && track_dist[rec] > stop_dist; rec--);	/* Find last record of interest */
+		for (rec = D.H.n_records - 1; rec && track_time[rec] > stop_time && bad_coordinates(lon[rec], lat[rec]) && track_dist[rec] > stop_dist; rec--);	/* Find last record of interest */
 		last_rec = rec;
 		if (gmtdefs.verbose) fprintf (stderr, "mgd77track: Plotting %s [%s]\n", list[argno], D.H.mgd77->Survey_Identifier);
 		sprintf (comment, "Tracking %s", list[argno]);
@@ -380,7 +386,7 @@ int main (int argc, char **argv)
 
 		first = TRUE;
 		for (rec = first_rec; rec <= last_rec; rec++) {
-			if (GMT_outside (lon[rec], lat[rec])) {
+			if (bad_coordinates(lon[rec], lat[rec]) || GMT_outside (lon[rec], lat[rec])) {
 				first = TRUE;
 				continue;
 			}
@@ -398,7 +404,7 @@ int main (int argc, char **argv)
 						strcpy (cruise_id[n_id].text, name);
 						n_id++;
 						if (n_id == n_alloc_c) {
-							n_alloc_c += GMT_SMALL_CHUNK;
+							n_alloc_c <<= 1;
 							cruise_id = (struct MGD77TRACK_LEG_ANNOT *) GMT_memory ((void *)cruise_id, (size_t)n_alloc_c, sizeof (struct MGD77TRACK_LEG_ANNOT), "mgd77track");
 						}
 					}
@@ -502,12 +508,15 @@ int main (int argc, char **argv)
 	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Plotted %d cruises\n", GMT_program, n_cruises);
 
+	MGD77_Path_Free (n_paths, list);
+	MGD77_end (&M);
+	
 	exit (EXIT_SUCCESS);
 }
 
-double heading (int rec, double *lon, double *lat, int n_records)
+double heading (GMT_LONG rec, double *lon, double *lat, GMT_LONG n_records)
 {
-	int i1, i2, j;
+	GMT_LONG i1, i2, j;
 	double angle, x1, x0, y1, y0;
 	double sum_x2 = 0.0, sum_xy = 0.0, sum_y2 = 0.0, dx, dy;
 	
@@ -529,7 +538,7 @@ double heading (int rec, double *lon, double *lat, int n_records)
 	else if (sum_x2 < GMT_CONV_LIMIT)	/* Line is vertical */
 		angle = 90.0;
 	else
-		angle = (fabs(sum_xy) < GMT_CONV_LIMIT) ? 90.0 : d_atan2 (sum_xy, sum_x2) * R2D;
+		angle = (GMT_IS_ZERO (sum_xy)) ? 90.0 : d_atan2 (sum_xy, sum_x2) * R2D;
 	if (angle > 90.0)
 		angle -= 180;
 	else if (angle < -90.0)
@@ -549,7 +558,7 @@ int get_annotinfo (char *args, struct MGD77TRACK_ANNOT *info)
 	i1 = 0;
 	while (args[i1]) {
 		flag1 = 'a';
-		if (isalpha (args[i1])) {
+		if (isalpha ((int)args[i1])) {
 			flag1 = args[i1];
 			if (isupper(flag1)) flag1 = tolower (flag1);
 			i1++;
@@ -626,3 +635,7 @@ void annot_legname (double x, double y, double lon, double lat, double angle, ch
 	GMT_smart_justify (just, angle, GMT_u2u[GMT_PT][GMT_INCH] * 0.15 * size, GMT_u2u[GMT_PT][GMT_INCH] * 0.15 * size, &x, &y);
 	ps_text (x, y, size, text, angle, just, 0);
 }
+
+BOOLEAN bad_coordinates (double lon, double lat) {
+	return (GMT_is_dnan (lon) || GMT_is_dnan (lat));
+}
\ No newline at end of file
