@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: dat2gmt.c,v 1.11 2007/03/12 19:52:27 remko Exp $
+ *	$Id: dat2gmt.c,v 1.16 2008/05/12 20:31:30 guru Exp $
  *
- *    Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -20,11 +20,12 @@
 #include "gmt.h"
 #include "gmt_mgg.h"
 
-#define MEM_SIZE 50000
+#define MEM_SIZE 65536
 
 int main (int argc, char **argv)
 {
-	int rec, n_records, leg_year, time, mag, top, yr, mo, dy, hr, mi, sc, n_alloc = MEM_SIZE, conv;
+	int leg_year, n_records;
+	int rec, time, mag, top, yr, mo, dy, hr, mi, sc, n_alloc = MEM_SIZE, conv;
 	double lat, lon, grv;
 	char line[BUFSIZ], agency[10], stime[20], sgrv[10], smag[10], stop[10];
 	FILE *fpi, *fpo;
@@ -33,7 +34,7 @@ int main (int argc, char **argv)
 
 	argc = GMT_begin (argc, argv);
 	
-	if (argc != 3 || ((fpi = GMT_fopen(argv[1], "r")) == NULL) || ((fpo = fopen(argv[2], "wb")) == NULL)) {
+	if (argc != 3 || ((fpi = fopen(argv[1], "r")) == NULL) || ((fpo = fopen(argv[2], "wb")) == NULL)) {
 		fprintf(stderr, "usage: dat2gmt asciifile file.gmt\n");
 		exit (EXIT_FAILURE);
 	}
@@ -61,7 +62,7 @@ int main (int argc, char **argv)
 		record[rec].gmt[2] = (conv > 0) ? top : GMTMGG_NODATA;
 		rec++;
 		if (rec == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			record = (struct GMTMGG_REC *) GMT_memory((void *)record, (size_t)n_alloc, sizeof (struct GMTMGG_REC), "dat2gmt");
 		}
 	}
