@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2dat.c,v 1.12 2007/03/12 19:52:27 remko Exp $
+ *	$Id: gmt2dat.c,v 1.16 2008/05/12 20:31:30 guru Exp $
  *
- *    Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -21,7 +21,8 @@
 
 int main (int argc, char **argv)
 {
-	int rec, n_records, leg_year, yr, mo, dy, hr, mi, sc;
+	int n_records, leg_year;
+	int rec, yr, mo, dy, hr, mi, sc;
 	double lat, lon;
 	char agency[10];
 	FILE *fp;
@@ -30,7 +31,7 @@ int main (int argc, char **argv)
 
 	argc = GMT_begin (argc, argv);
 
-	if (argc != 2 || (fp = GMT_fopen(argv[1], "rb")) == NULL) {
+	if (argc != 2 || (fp = fopen(argv[1], "rb")) == NULL) {
 		fprintf(stderr, "usage: gmt2dat file.gmt > asciifile\n");
 		exit (EXIT_FAILURE);
 	}
