@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtinfo.c,v 1.1.1.1 2000/12/28 01:23:45 gmt Exp $
+ *	$Id: gmtinfo.c,v 1.15 2008/04/28 21:27:37 guru Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -18,19 +18,21 @@
 #include "gmt.h"
 #include "gmt_mgg.h"
 
-struct GMTMGG_TIME *gmt;
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int leg_year, n_records, rec, n_grv, n_mag, n_top;
+	int leg_year, n_records;
+	int rec, n_grv, n_mag, n_top;
 	int begin_day, begin_mo, begin_yr, end_day, end_mo, end_yr;
 	int hour, minute, second, nfiles=0, i, verbose = FALSE;
-	double xmin1, xmax1, xmin2, xmax2, ymin, ymax, xmin, xmax, lat, lon, old_x;
+	double xmin1, xmax1, xmin2, xmax2, ymin, ymax, xmin, xmax, lat, lon, old_x = 0.0;
 	char agency[10], greenwich, gmtfile[100];
 	FILE *fp;
 	struct GMTMGG_REC record;
+	struct GMTMGG_TIME *gmt;
 	
-	gmtmggpath_init();
+	argc = GMT_begin (argc, argv);
+
+	gmtmggpath_init(GMT_SHAREDIR);
 	
 	for (i = 1; i < argc; i++) if (!strcmp(argv[i], "-v")) verbose = TRUE;
 	
@@ -52,15 +54,15 @@ main (int argc, char **argv)
 	
 		if (fread((void *)(&leg_year), (size_t)4, (size_t)1, fp) != 1) {
 			fprintf(stderr,"gmtinfo: Error while reading first year\n");
-			exit(EXIT_FAILURE);
+			exit (EXIT_FAILURE);
 		}
 		if (fread((void *)(&n_records), (size_t)4, (size_t)1, fp) != 1) {
 			fprintf(stderr,"gmtinfo: Error while reading no of records\n");
-			exit(EXIT_FAILURE);
+			exit (EXIT_FAILURE);
 		}
 		if (fread((void *)agency, (size_t)10, (size_t)1, fp) != 1) {
 			fprintf(stderr,"gmtinfo: Error while reading info-header\n");
-			exit(EXIT_FAILURE);
+			exit (EXIT_FAILURE);
 		}
 	
 		gmt = gmtmgg_init (leg_year);	/* Initialize gmt_structure */
@@ -76,7 +78,7 @@ main (int argc, char **argv)
 		for (rec = 0; rec < n_records; rec++) {
 			if (fread((void *)(&record), (size_t)18, (size_t)1, fp) != 1) {
 				fprintf(stderr,"gmtinfo: Error reading data record no %d\n",rec);
-				exit(EXIT_FAILURE);
+				exit (EXIT_FAILURE);
 			}
 			if (rec == 0)
 				gmtmgg_date(record.time,&begin_yr,&begin_mo,&begin_day,&hour,&minute,&second,gmt);
@@ -114,15 +116,19 @@ main (int argc, char **argv)
 			xmax = MAX(xmax1,xmax2);
 		}
 		if (xmin > xmax) xmin -= 360.0;
-		printf("Info: %s Year: %d N_recs: %d n_g: %d n_m: %d n_t: %d W: %.5lf E: %.5lf S: %.5lf N: %.5lf Begins: %d %d %d Ends: %d %d %d\n",
+		printf("Info: %s Year: %d N_recs: %d n_g: %d n_m: %d n_t: %d W: %.5f E: %.5f S: %.5f N: %.5f Begins: %d %d %d Ends: %d %d %d\n",
 			agency, leg_year, n_records, n_grv, n_mag, n_top, xmin, xmax, ymin, ymax,
 			begin_day, begin_mo, begin_yr, end_day, end_mo, end_yr);
 	}
 	
 	if (nfiles == 0) {
 		fprintf(stderr, "usage: gmtinfo leg(s)\n");
-		exit(EXIT_FAILURE);
+		exit (EXIT_FAILURE);
 	}
+
+	gmtmgg_end ();
+
+	GMT_end (argc, argv);
 	
 	exit (EXIT_SUCCESS);
 }
