@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtlist.c,v 1.22 2007/03/14 23:11:11 pwessel Exp $
+ *	$Id: gmtlist.c,v 1.27 2008/05/12 20:31:30 guru Exp $
  *
- *    Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -44,7 +44,9 @@
 
 int main (int argc, char **argv)
 {
-	int leg_year, rec, i, j, n_records, argno, id, no[10], nval = 0;
+	int leg_year, n_records;
+	
+	int rec, i, j, argno, id, no[10], nval = 0;
 	int start_time = 0, stop_time = 2000000000, dlon, last_lon = 0;
 	int mon1, day1 = 0, year1, hour1, min1, time, last_lat = 0, mon2, day2 = 0, year2, hour2, min2;
 	int yy, mm, dd, hh, mi, ss, jd, dt, last_time = 0, n_cruises = 0;
@@ -251,7 +253,7 @@ int main (int argc, char **argv)
 	if (!geodetic) greenwich = TRUE;
 	
 	if (correct) {	/* Read correction table */
-		if ((fpc = GMT_fopen(corrfile, "rb")) == NULL) {
+		if ((fpc = fopen(corrfile, "rb")) == NULL) {
 			fprintf(stderr, "Could not read correction file %s\n", corrfile);
 			exit (EXIT_FAILURE);
 		}
@@ -328,7 +330,7 @@ int main (int argc, char **argv)
    			fprintf (stderr, "gmtlist : Cannot find leg %s\n", argv[argno]);
      			continue;
   		}
-		if ((fp = GMT_fopen (gmtfile, "rb")) == NULL) {
+		if ((fp = fopen (gmtfile, "rb")) == NULL) {
 			fprintf (stderr,"gmtlist: Could not open %s\n", gmtfile);
 			continue;
 		}
@@ -546,6 +548,7 @@ int main (int argc, char **argv)
 		GMT_free ((void *)gmt);
 	}
 
+	gmtmgg_end ();
 	GMT_end (argc, argv);
 	
 	exit (EXIT_SUCCESS);
