@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtpath.c,v 1.1.1.1 2000/12/28 01:23:45 gmt Exp $
+ *	$Id: gmtpath.c,v 1.12 2008/04/28 21:27:37 guru Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -15,12 +15,14 @@
 #include "gmt.h"
 #include "gmt_mgg.h"
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	char path[BUFSIZ];
 	int i, error = FALSE;
+
+	argc = GMT_begin (argc, argv);
 	
-	gmtmggpath_init();
+	gmtmggpath_init(GMT_SHAREDIR);
 
 	for (i = 1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -37,5 +39,8 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	gmtmgg_end ();
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
