@@ -1,6 +1,6 @@
 /*
- *	$Id: gmtstitch.c,v 1.23 2007/03/12 19:52:27 remko Exp $
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	$Id: gmtstitch.c,v 1.29 2008/03/24 08:58:33 guru Exp $
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  */
 /* gmtstitch will combine pieces of coastlines or similar segments into a
  * continuous line, polygon, or group of lines/polygons so that the jump
@@ -17,19 +17,19 @@
 #define END_B	1
 
 struct BUDDY {
-	int id;
-	int orig_id;
-	int end_order;
+	GMT_LONG id;
+	GMT_LONG orig_id;
+	GMT_LONG end_order;
 	double dist;
 };
 
 struct LINK {
-	int id;
-	int orig_id;
-	int group;
-	int pos;
-	int n;
-	int used;
+	GMT_LONG id;
+	GMT_LONG orig_id;
+	GMT_LONG group;
+	GMT_LONG pos;
+	GMT_LONG n;
+	GMT_LONG used;
 	double x_end[2];
 	double y_end[2];
 	struct BUDDY buddy[2];
@@ -39,11 +39,11 @@ struct LINK {
 int main (int argc, char **argv)
 {
 	struct LINK *seg;
-	int nearest_end[2][2], ii, end;
-	int i, j, k, np, distance_flag = 0, n_files = 0, ns, n_args, id, pos, start_id, done, end_order;
-	int n_new, n, chain = 0, n_islands = 0, n_trouble = 0, n_closed = 0, id2, L, G, error = 0;
-	int n_alloc = GMT_SMALL_CHUNK, n_id_alloc = GMT_CHUNK, out_seg, match = 0, n_steps;	
-	BOOLEAN individual_file = FALSE, first;
+	GMT_LONG nearest_end[2][2], ii, end;
+	GMT_LONG i, j, k, np, distance_flag = 0, n_files = 0, ns, n_args, id, pos, start_id, done, end_order;
+	GMT_LONG n_new, n, chain = 0, n_islands = 0, n_trouble = 0, n_closed = 0, id2, L, G, error = 0;
+	GMT_LONG n_alloc = GMT_SMALL_CHUNK, n_id_alloc = GMT_CHUNK, out_seg, match = 0, n_steps;	
+	BOOLEAN individual_file = FALSE, first, *skip;
 	struct GMT_DATASET *D;
 	double dd[2][2], p_dummy_x, p_dummy_y, p_last_x, p_last_y, p_first_x, p_first_y, distance, cutoff = 0.0;
 	char format[BUFSIZ], filename[BUFSIZ];
@@ -51,7 +51,7 @@ int main (int argc, char **argv)
 #ifdef DEBUG
 	FILE *fp3;
 #endif
-	void Write_This_Segment (FILE *fp, struct GMT_LINE_SEGMENT *line, int start, int end);
+	void Write_This_Segment (FILE *fp, struct GMT_LINE_SEGMENT *line, GMT_LONG start, GMT_LONG end);
 	
 	argc = GMT_begin (argc, argv);
 
@@ -147,7 +147,7 @@ int main (int argc, char **argv)
 
 	/* Now we are ready to take on some input values */
 
-	if (individual_file && !format) strcpy (format, "gmtstitch_segment_%d.d");
+	if (individual_file && !format) strcpy (format, "gmtstitch_segment_%ld.d");
 	n_args = (argc > 1) ? argc : 2;
 
 	switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
@@ -167,8 +167,8 @@ int main (int argc, char **argv)
 
 	/* Allocate memory and read in all the files; each file can have many lines (-M) */
 	
-	D = (struct GMT_DATASET *) GMT_memory (VNULL, 1, sizeof (struct GMT_DATASET), GMT_program);
-	D->table = (struct GMT_TABLE **) GMT_memory (VNULL, n_alloc, sizeof (struct GMT_TABLE *), GMT_program);
+	D = (struct GMT_DATASET *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_DATASET), GMT_program);
+	D->table = (struct GMT_TABLE **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct GMT_TABLE *), GMT_program);
 	for (k = 1, i = 0; k < argc; k++) {
 		if (argv[k][0] == '-') continue;
 
@@ -177,13 +177,13 @@ int main (int argc, char **argv)
 		GMT_import_table ((void *)argv[k], GMT_IS_FILE, &D->table[i], 0.0, FALSE, FALSE, TRUE);
 		i++;
 		if (i == n_alloc) {
-			n_alloc += GMT_SMALL_CHUNK;
-			D->table = (struct GMT_TABLE **) GMT_memory ((void *)D->table, n_alloc, sizeof(struct GMT_TABLE *), GMT_program);
+			n_alloc <<= 1;
+			D->table = (struct GMT_TABLE **) GMT_memory ((void *)D->table, (size_t)n_alloc, sizeof(struct GMT_TABLE *), GMT_program);
 		}
 	}
-	if (i < n_alloc) D->table = (struct GMT_TABLE **) GMT_memory ((void *)D->table, i, sizeof(struct GMT_TABLE *), GMT_program);
+	if (i < n_alloc) D->table = (struct GMT_TABLE **) GMT_memory ((void *)D->table, (size_t)i, sizeof(struct GMT_TABLE *), GMT_program);
 	D->n_tables = i;
-	seg = (struct LINK *) GMT_memory (VNULL, n_id_alloc, sizeof(struct LINK), GMT_program);
+	seg = (struct LINK *) GMT_memory (VNULL, (size_t)n_id_alloc, sizeof(struct LINK), GMT_program);
 	id = pos = ns = out_seg = 0;
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Check for closed polygons\n", GMT_program);
 	
@@ -206,10 +206,10 @@ int main (int argc, char **argv)
 				else
 					fp = GMT_stdout;
 #if DEBUG
-				sprintf (GMT_io.segment_header, "> Segment %d\n", out_seg);
+				sprintf (GMT_io.segment_header, "> Segment %ld\n", out_seg);
 #endif
 				if (!individual_file) GMT_write_segmentheader (fp, D->table[k]->segment[j]->n_columns);
-				Write_This_Segment (fp, D->table[k]->segment[j], 0, np-1);
+				Write_This_Segment (fp, D->table[k]->segment[j], (GMT_LONG)0, np-1);
 				if (individual_file) GMT_fclose (fp);
 				n_islands++;
 				out_seg++;
@@ -230,8 +230,8 @@ int main (int argc, char **argv)
 			seg[id].buddy[0].dist = seg[id].buddy[1].dist = DBL_MAX;
 			id++;
 			if (id == n_id_alloc) {
-				n_id_alloc += GMT_CHUNK;
-				seg = (struct LINK *) GMT_memory ((void *)seg, n_id_alloc, sizeof(struct LINK), GMT_program);
+				n_id_alloc <<= 1;
+				seg = (struct LINK *) GMT_memory ((void *)seg, (size_t)n_id_alloc, sizeof(struct LINK), GMT_program);
 			}
 		}
 	}
@@ -241,15 +241,18 @@ int main (int argc, char **argv)
 		exit (EXIT_SUCCESS);
 	}
 	
-	if (ns < n_id_alloc) seg = (struct LINK *) GMT_memory ((void *)seg, ns, sizeof(struct LINK), GMT_program);
+	if (ns < n_id_alloc) seg = (struct LINK *) GMT_memory ((void *)seg, (size_t)ns, sizeof(struct LINK), GMT_program);
+	skip = (BOOLEAN *) GMT_memory (VNULL, (size_t)ns, sizeof(BOOLEAN), GMT_program);
 	
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Found %d closed polygons\n", GMT_program, n_islands);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Found %ld closed polygons\n", GMT_program, n_islands);
 	
 	/* The algorithm will be confused if there are identical duplicates of segments - thus we check */
 	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Check for duplicate lines\n", GMT_program);
 	for (i = 0; i < ns; i++) {
+		if (skip[i]) continue;	/* Skip segment that has been determined to be a duplicate segment */
 		for (j = i + 1; j < ns; j++) {
+			if (skip[j]) continue;	/* Skip segment that has been determined to be a duplicate segment */
 			if ((seg[i].x_end[0] == seg[j].x_end[0] && seg[i].y_end[0] == seg[j].y_end[0]) ||
 			    (seg[i].x_end[0] == seg[j].x_end[1] && seg[i].y_end[0] == seg[j].y_end[1]) ||
 			    (seg[i].x_end[1] == seg[j].x_end[0] && seg[i].y_end[1] == seg[j].y_end[0]) ||
@@ -261,14 +264,25 @@ int main (int argc, char **argv)
 					}
 					match = (match == seg[i].n) ? 1 : 0;
 					if (match) {
-						fprintf (stderr, "%s: Segments %d and %d are duplicates - eliminate before using gmtstich", GMT_program, i, j);
-						exit (EXIT_FAILURE);
+						fprintf (stderr, "%s: Segments %ld and %ld are duplicates - Segment %ld will be ignored\n", GMT_program, i, j, j);
+						skip[j] = TRUE;
 					}
 				}
 			}
 		}
 	}
 	
+	/* Eliminate the duplicate segments from consideration */
+	
+	for (i = j = 0; i < ns; i++) {
+		if (skip[i]) continue;
+		if (i > j) seg[j] = seg[i];
+		j++;
+	}
+	if (j < ns && gmtdefs.verbose) fprintf (stderr, "%s: %ld duplicate segment removed\n", GMT_program, ns - j);
+	ns = j;
+	GMT_free ((void *)skip);
+	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Calculate and rank end point separations\n", GMT_program);
 	
 	/* We determine the distance from each segments two endpoints to the two endpoints on every other
@@ -277,7 +291,6 @@ int main (int argc, char **argv)
 	 */
 	 
 	for (i = 0; i < ns; i++) {
-
 		for (j = i; j < ns; j++) {
 			/* nearest_end indicates which end is closest to this end */
 			if (i == j) {	/* Store offset between the endpoints of a single segment (should be 0 if closed) */
@@ -316,7 +329,7 @@ int main (int argc, char **argv)
 
 #ifdef DEBUG	
 	fp3 = fopen ("link.dat", "w");
-	for (i = 0; i < ns; i++) fprintf (fp3, "%d\t%d\t%d\t%d\t%g\t%d\t%d\t%g\n", i, seg[i].orig_id, seg[i].buddy[0].orig_id, seg[i].buddy[0].end_order, seg[i].buddy[0].dist, seg[i].buddy[1].orig_id, seg[i].buddy[1].end_order, seg[i].buddy[1].dist);
+	for (i = 0; i < ns; i++) fprintf (fp3, "%ld\t%ld\t%ld\t%ld\t%g\t%ld\t%ld\t%g\n", i, seg[i].orig_id, seg[i].buddy[0].orig_id, seg[i].buddy[0].end_order, seg[i].buddy[0].dist, seg[i].buddy[1].orig_id, seg[i].buddy[1].end_order, seg[i].buddy[1].dist);
 	fclose (fp3);
 #endif
 	start_id = done = 0;
@@ -333,12 +346,12 @@ int main (int argc, char **argv)
 		id = start_id;
 		end_order = n_steps = 0;
 #if DEBUG
-		if (gmtdefs.verbose) fprintf (stderr, "%d\n", seg[id].orig_id);
+		if (gmtdefs.verbose) fprintf (stderr, "%ld\n", seg[id].orig_id);
 #endif
 		while (!done && seg[id].buddy[end_order].dist <= cutoff && !seg[seg[id].buddy[end_order].id].used) {
 			id2 = seg[id].buddy[end_order].id;
 #if DEBUG
-			if (gmtdefs.verbose) fprintf (stderr, "%d\n", seg[id2].orig_id);
+			if (gmtdefs.verbose) fprintf (stderr, "%ld\n", seg[id2].orig_id);
 #endif
 			if (id2 == start_id)	/* Closed polygon, start here */
 				done = TRUE;
@@ -359,7 +372,7 @@ int main (int argc, char **argv)
 		
 		start_id = id;
 		
-		memset (GMT_io.segment_header, 0, BUFSIZ);
+		memset (GMT_io.segment_header, 0, (size_t)BUFSIZ);
 		if (individual_file) {
 			sprintf (filename, format, out_seg);
 			if ((fp = GMT_fopen (filename, GMT_io.w_mode)) == NULL ) {
@@ -370,7 +383,7 @@ int main (int argc, char **argv)
 		else {
 			fp = GMT_stdout;
 #if DEBUG
-			sprintf (GMT_io.segment_header, "> Segment %d\n", out_seg);
+			sprintf (GMT_io.segment_header, "> Segment %ld\n", out_seg);
 #endif
 			GMT_write_segmentheader (fp, D->table[seg[id].group]->segment[seg[id].pos]->n_columns);
 		}
@@ -408,7 +421,7 @@ int main (int argc, char **argv)
 					j = 0;
 					n = np;
 				}
-				Write_This_Segment (fp, D->table[G]->segment[L], np-1-j, 0);
+				Write_This_Segment (fp, D->table[G]->segment[L], np-1-j, (GMT_LONG)0);
 				p_last_x = D->table[G]->segment[L]->coord[0][0];
 				p_last_y = D->table[G]->segment[L]->coord[1][0];
 				if (first) p_first_x = D->table[G]->segment[L]->coord[0][np-1], p_first_y = D->table[G]->segment[L]->coord[1][np-1];
@@ -429,7 +442,7 @@ int main (int argc, char **argv)
 			k++;
 		} while (!done);
 		if (individual_file) GMT_fclose (fp);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Segment %d made from %d pieces\n", GMT_program, out_seg, k);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Segment %ld made from %ld pieces\n", GMT_program, out_seg, k);
 		
 		if (p_first_x == p_last_x && p_first_y == p_last_y) n_closed++;
 		
@@ -442,10 +455,10 @@ int main (int argc, char **argv)
 		done = (start_id == ns);	/* No more unused segments */
 	}
 
-	fprintf (stderr, "%s: Segments in: %d Segments out: %d\n", GMT_program, ns + n_islands, chain + n_islands);
-	if (n_trouble) fprintf (stderr, "%s: %d trouble spots\n", GMT_program, n_trouble);
-	if (n_closed) fprintf (stderr, "%s: %d new closed segments\n", GMT_program, n_closed);
-	if (n_islands) fprintf (stderr, "%s: %d were already closed\n", GMT_program, n_islands);
+	fprintf (stderr, "%s: Segments in: %ld Segments out: %ld\n", GMT_program, ns + n_islands, chain + n_islands);
+	if (n_trouble) fprintf (stderr, "%s: %ld trouble spots\n", GMT_program, n_trouble);
+	if (n_closed) fprintf (stderr, "%s: %ld new closed segments\n", GMT_program, n_closed);
+	if (n_islands) fprintf (stderr, "%s: %ld were already closed\n", GMT_program, n_islands);
 	
 	GMT_free ((void *)seg);
 	GMT_free_dataset (D);
@@ -453,9 +466,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void Write_This_Segment (FILE *fp, struct GMT_LINE_SEGMENT *line, int start, int end)
+void Write_This_Segment (FILE *fp, struct GMT_LINE_SEGMENT *line, GMT_LONG start, GMT_LONG end)
 {
-	int i, j, inc, done = FALSE;
+	GMT_LONG i, j, inc, done = FALSE;
 	static double out[BUFSIZ];
 	
 	inc = (start < end) ? +1 : -1;
