@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: nc2xy.c,v 1.13 2007/03/12 19:52:27 remko Exp $
+ *	$Id: nc2xy.c,v 1.21 2008/04/26 19:13:40 guru Exp $
  *
- *	Copyright (c) 2006-2007 by R. Scharroo
+ *	Copyright (c) 2006-2008 by R. Scharroo
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -31,15 +31,13 @@
 int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, suppress = FALSE, reverse = FALSE, skip;
-
-	int i, k, ncid, varid[NVAR], dimid = 0, ndims, n_files = 0, n_out;
-	int n_total = 0, n_suppressed = 0;
-
-	size_t len, j;
-
+	int ncid, varid[NVAR], dimid = 0, ndims;
+	GMT_LONG i, k, n_files = 0, n_out;
+	int n_dimid = -1;
+	size_t n_total = 0, j, n_suppressed = 0;
 	double w, e, s, n, out[NVAR], add_offset[NVAR], scale_factor[NVAR], missing_value[NVAR];
-
-	char varnm[NVAR][BUFSIZ];
+	char varnm[NVAR][GMT_TEXT_LEN], long_name[GMT_LONG_TEXT], units[GMT_LONG_TEXT];
+	struct GMT_TIME_SYSTEM time_system;
 
 	argc = GMT_begin (argc, argv);
 
@@ -54,6 +52,7 @@ int main (int argc, char **argv)
 			switch (argv[i][1]) {
 				/* Common parameters */
 
+				case 'f':
 				case 'V':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
@@ -80,7 +79,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "nc2xy %s - Converting netCDF column file(s) to ASCII xy data\n\n", GMT_VERSION);
-		fprintf( stderr, "usage: nc2xy <files> [-F<var1>/<var2>/...] [-S[r]] [-V] > xyfile\n");
+		fprintf( stderr, "usage: nc2xy <files> [-F<var1>/<var2>/...] [-S[r]] [-V] [%s] > xyfile\n", GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -90,6 +89,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-S Suppress records with NaN values [Default prints all nodes]\n");
 		fprintf (stderr, "\t   Append r to reverse the suppression (only output records with NaNs)\n");
 		GMT_explain_option ('V');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -130,41 +130,63 @@ int main (int argc, char **argv)
 			varid[1] = 1;
 		}
 
-		/* Get further variable attributes */
+		/* Get further variable sizes and attributes */
 
-		n_total = -1;
 		for (i = 0; i < n_out; i++) {
+			/* Check that variable is 1-dimensional */
 			GMT_err_fail (nc_inq_varndims (ncid, varid[i], &ndims), argv[k]);
 			if (ndims != 1) {
-				fprintf (stderr, "%s: Variable %s in not 1-dimensional\n", GMT_program, varnm[i]);
+				fprintf (stderr, "%s: Variable %s is not 1-dimensional\n", GMT_program, varnm[i]);
 				exit (EXIT_FAILURE);
 			}
+			/* Check that dimension is same as the others */
 			GMT_err_fail (nc_inq_vardimid (ncid, varid[i], &dimid), argv[k]);
-			GMT_err_fail (nc_inq_dimlen (ncid, dimid, &len), argv[k]);
-			if (n_total < 0)
-				n_total = len;
-			else if (n_total != (int)len) {
-				fprintf (stderr, "%s: Variable %s has different length than others\n", GMT_program, varnm[i]);
+			if (n_dimid < 0) {
+				n_dimid = dimid;
+				GMT_err_fail (nc_inq_dimlen (ncid, dimid, &n_total), argv[k]);
+			}
+			else if (n_dimid != dimid) {
+				fprintf (stderr, "%s: Variable %s has different dimension than others\n", GMT_program, varnm[i]);
 				exit (EXIT_FAILURE);
 			}
+			/* Check all attributes */
 			if (nc_get_att_double (ncid, varid[i], "scale_factor", &scale_factor[i])) scale_factor[i] = 1.0;
 			if (nc_get_att_double (ncid, varid[i], "add_offset", &add_offset[i])) add_offset[i] = 0.0;
 			if (nc_get_att_double (ncid, varid[i], "_FillValue", &missing_value[i]) &&
 			    nc_get_att_double (ncid, varid[i], "missing_value", &missing_value[i])) missing_value[i] = GMT_d_NaN;
+			if (GMT_nc_get_att_text (ncid, varid[i], "long_name", long_name, (size_t)GMT_LONG_TEXT)) long_name[0] = 0;
+			if (GMT_nc_get_att_text (ncid, varid[i], "units", units, (size_t)GMT_LONG_TEXT)) units[0] = 0;
+			GMT_str_tolower (long_name); GMT_str_tolower (units);
+
+			/* Scan for geographical or time units */
+			if (!strcmp (long_name, "longitude") || strstr (units, "degrees_e"))
+				GMT_io.in_col_type[i] = GMT_IS_LON;
+			else if (!strcmp (long_name, "latitude") || strstr (units, "degrees_n"))
+				GMT_io.in_col_type[i] = GMT_IS_LAT;
+			else if (!strcmp (long_name, "time") || !strcmp (varnm[i], "time")) {
+				GMT_io.in_col_type[i] = GMT_IS_RELTIME;
+				memcpy ((void *)&time_system, (void *)&gmtdefs.time_system, sizeof (struct GMT_TIME_SYSTEM));
+				if (GMT_get_time_system (units, &time_system) || GMT_init_time_system_structure (&time_system))
+					fprintf (stderr, "%s: Warning: Time units [%s] in NetCDF file not recognised, defaulting to gmtdefaults.\n", GMT_program, units);
+				/* Determine scale between data and internal time system, as well as the offset (in internal units) */
+				scale_factor[i] = time_system.scale * gmtdefs.time_system.i_scale;
+				add_offset[i] = (time_system.rata_die - gmtdefs.time_system.rata_die) + (time_system.epoch_t0 - gmtdefs.time_system.epoch_t0);
+				add_offset[i] *= GMT_DAY2SEC_F * gmtdefs.time_system.i_scale;
+			}
+			else if (GMT_io.in_col_type[i] == GMT_IS_UNKNOWN)
+				GMT_io.in_col_type[i] = GMT_IS_FLOAT;
 		}
 
 		/* Load data record by record and scale as required */
 
-		for (j = 0; j < (size_t)n_total; j++) {
+		for (j = 0; j < n_total; j++) {
 			skip = FALSE;
 			for (i = 0; i < n_out; i++) {
 				GMT_err_fail (nc_get_var1_double (ncid, varid[i], &j, &out[i]), argv[k]);
 				if (out[i] == missing_value[i])
 					out[i] = GMT_d_NaN;
-				else {
-					out[i] *= scale_factor[i];
-					out[i] += add_offset[i];
-				}
+				else
+					out[i] = out[i] * scale_factor[i] + add_offset[i];
 				if (suppress && GMT_is_dnan (out[i])) {
 					skip = TRUE;
 					continue;
@@ -177,12 +199,12 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: %d values extracted\n", GMT_program, n_total - n_suppressed);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: %ld values extracted\n", GMT_program, (GMT_LONG)(n_total - n_suppressed));
 	if (n_suppressed && gmtdefs.verbose) {
 		if (reverse)
-			fprintf (stderr, "%s: %d finite values suppressed\n", GMT_program, n_suppressed);
+			fprintf (stderr, "%s: %ld finite values suppressed\n", GMT_program, (GMT_LONG)n_suppressed);
 		else
-			fprintf (stderr, "%s: %d NaN values suppressed\n", GMT_program, n_suppressed);
+			fprintf (stderr, "%s: %ld NaN values suppressed\n", GMT_program, (GMT_LONG)n_suppressed);
 	}
 
 	GMT_end (argc, argv);
