@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: nearneighbor.c,v 1.47 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: nearneighbor.c,v 1.54 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -70,7 +70,7 @@ struct NEARNEIGHBOR_CTRL {	/* All control options for this program (except commo
 
 struct NEARNEIGHBOR_NODE {	/* Structure with point id and distance pairs for all sectors */
 	float *distance;	/* Distance of nearest datapoint to this node per sector */
-	int *datum;		/* Point id of this data point */
+	GMT_LONG *datum;		/* Point id of this data point */
 };
 
 struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
@@ -79,8 +79,10 @@ struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
 
 int main (int argc, char **argv)
 {
-	int i, j, k, ij, i0, j0, *di, dj, sector, n, n_alloc = 5 * GMT_CHUNK, n_fields, nx_2;
-	int n_set, n_filled, n_almost, n_none, n_files = 0, n_args, fno, one_or_zero, n_expected_fields, distance_flag = 0;
+	GMT_LONG k, ij, n, n_alloc = GMT_CHUNK, n_set, n_filled, n_almost, n_none;
+	
+	int i, j, i0, j0, *di, dj, sector, n_fields;
+	int n_files = 0, n_args, fno, n_expected_fields, distance_flag = 0;
 	int max_di, actual_max_di, ii, jj, n_req, x_wrap, y_wrap;
 
 	BOOLEAN error = FALSE, done = FALSE, nofile = TRUE;
@@ -102,7 +104,8 @@ int main (int argc, char **argv)
 	struct NEARNEIGHBOR_POINT  *point;
 	struct NEARNEIGHBOR_CTRL *Ctrl;
 
-	void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id);
+	void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, GMT_LONG id);
+	void free_node (struct NEARNEIGHBOR_NODE *node);
 	void *New_Nearneighbor_Ctrl (), Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -156,7 +159,7 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						Ctrl->L.active = TRUE;
-						strncpy (Ctrl->L.mode, &argv[i][2], 4);
+						strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					}
 					else {
 						GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -277,7 +280,6 @@ int main (int argc, char **argv)
         	n_files = 1;
         n_args = (argc > 1) ? argc : 2;
         
-	one_or_zero = !header.node_offset;
 	header.xy_off = 0.5 * header.node_offset;
 	xinc2 = header.xy_off * header.x_inc;
 	yinc2 = header.xy_off * header.y_inc;
@@ -286,8 +288,7 @@ int main (int argc, char **argv)
 
 	GMT_boundcond_param_prep (&header, &edgeinfo);
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid dimensions are nx = %d, ny = %d\n", GMT_program,
-		header.nx, header.ny);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid dimensions are nx = %d, ny = %d\n", GMT_program, header.nx, header.ny);
 
 	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
@@ -339,7 +340,6 @@ int main (int argc, char **argv)
 	y_top = header.y_max + dj * header.y_inc;	y_bottom = header.y_min - dj * header.y_inc;
 	x_width = header.x_max - header.x_min;		y_width = header.y_max - header.y_min;
 	half_x_width = 0.5 * x_width;			half_y_width = 0.5 * y_width;
-	nx_2 = edgeinfo.nxp / 2;
 	n = 0;
 	replicate_x = (edgeinfo.nxp && !header.node_offset);	/* Gridline registration has duplicate column */
 	replicate_y = (edgeinfo.nyp && !header.node_offset);	/* Gridline registration has duplicate row */
@@ -368,7 +368,7 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
@@ -403,7 +403,7 @@ int main (int argc, char **argv)
 
 					if (distance > Ctrl->S.radius) continue;	/* Data constraint is too far from this node */
 
-					k = jj * header.nx + ii;
+					k = GMT_IJ (jj, ii, header.nx);
 					dx = in[0] - x0[ii];	dy = in[1] - y0[jj];
 
 					/* Check for wrap-around in x or y.  This should only occur if the
@@ -440,15 +440,15 @@ int main (int argc, char **argv)
 				}
 			}
 			n++;
-			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10ld\r", GMT_program, n);
 			if (n == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10d\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10ld\n", GMT_program, n);
 
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 	grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
@@ -473,6 +473,7 @@ int main (int argc, char **argv)
 			if (n_filled < Ctrl->N.min_sectors) { 	/* Not minimum set of neighbors in all sectors, set to empty and goto next node */
 				n_almost++;
 				grd[ij] = (float)Ctrl->E.value;
+				free_node (grid_node[ij]);
 				continue;
 			}
 
@@ -490,17 +491,18 @@ int main (int argc, char **argv)
 				}
 			}
 			grd[ij] = (float)(grd_sum / weight_sum);
+			free_node (grid_node[ij]);
 		}
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\r", GMT_program, j);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10d\r", GMT_program, j);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\n", GMT_program, j);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10d\n", GMT_program, j);
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	if (gmtdefs.verbose) {
 		sprintf (line, "%s)\n", gmtdefs.d_format);
-		fprintf (stderr, "%s: %d nodes were assigned an average value\n", GMT_program, n_set);
-		fprintf (stderr, "%s: %d nodes failed sector criteria and %d nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
+		fprintf (stderr, "%s: %ld nodes were assigned an average value\n", GMT_program, n_set);
+		fprintf (stderr, "%s: %ld nodes failed sector criteria and %ld nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
 		(GMT_is_dnan (Ctrl->E.value)) ? fprintf (stderr, "NaN)\n") : fprintf (stderr, line, Ctrl->E.value);
 	}
 
@@ -521,18 +523,20 @@ int main (int argc, char **argv)
 
 struct NEARNEIGHBOR_NODE *add_new_node(int n)
 {
-	struct NEARNEIGHBOR_NODE *new, *add_new_node(int n);
-
+	struct NEARNEIGHBOR_NODE *new;
+	
 	new = (struct NEARNEIGHBOR_NODE *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_NODE), GMT_program);
 	new->distance = (float *) GMT_memory (VNULL, (size_t)n, sizeof (float), GMT_program);
-	new->datum = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), GMT_program);
+	new->datum = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), GMT_program);
 	while (n > 0) new->datum[--n] = -1;
 
 	return (new);
 }
 
-void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id)
+void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, GMT_LONG id)
 {
+	struct NEARNEIGHBOR_NODE *add_new_node(int n);
+
 	/* Allocates node space if not already used and updates the value if closer to node */
 
 	if (!(*node)) *node = add_new_node (n_sector);
@@ -542,10 +546,20 @@ void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, dou
 	}
 }
 
+void free_node (struct NEARNEIGHBOR_NODE *node)
+{
+	/* Frees allocated node space */
+
+	if (!node) return;	/* Nothing to do */
+	GMT_free ((void *)node->distance);
+	GMT_free ((void *)node->datum);
+	GMT_free ((void *)node);
+}
+
 void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct NEARNEIGHBOR_CTRL *C;
 
-	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
+	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.sectors = 4;
@@ -554,6 +568,6 @@ void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control struct
 }
 
 void Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);
+	if (C->G.file) free ((void *)C->G.file);
 	GMT_free ((void *)C);
 }
