@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psbasemap.c,v 1.39 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psbasemap.c,v 1.43 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -197,7 +197,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
 		error++;
 	}
-
+	if (Ctrl->L.active && ! GMT_IS_MAPPING) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  -L applies to geographical data only\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -213,7 +216,8 @@ int main (int argc, char **argv)
 
 	if (Ctrl->G.active) {
 		double *x, *y;
-		int np, donut;
+		GMT_LONG np;
+		BOOLEAN donut;
 		np = GMT_map_clip_path (&x, &y, &donut);
 		GMT_fill (x, y, (1 + donut) * np, &Ctrl->G.fill, FALSE);
 		GMT_free ((void *)x);
@@ -242,7 +246,7 @@ int main (int argc, char **argv)
 void *New_Psbasemap_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSBASEMAP_CTRL *C;
 	
-	C = (struct PSBASEMAP_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSBASEMAP_CTRL), "New_Psbasemap_Ctrl");
+	C = (struct PSBASEMAP_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSBASEMAP_CTRL), "New_Psbasemap_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
