@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pscoast.c,v 1.70 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pscoast.c,v 1.81 2008/04/05 23:22:59 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@
  * Optionally, the user may choose to issue clip paths rather than paint the
  * polygons.  That way one may clip subsequent images to be visible only
  * inside or outside the coastline.
- * 
+ *
  *
  * Author:	Paul Wessel
  * Date:	24-JUN-2000
@@ -80,7 +80,7 @@ struct PSCOAST_CTRL {
 	} G;
 	struct I {	/* -I<feature>[/<pen>] */
 		BOOLEAN active;
-		BOOLEAN use[GMT_N_RLEVELS];
+		int use[GMT_N_RLEVELS];
 		struct GMT_PEN pen[GMT_N_RLEVELS];
 	} I;
 	struct L {	/* -L */
@@ -89,7 +89,7 @@ struct PSCOAST_CTRL {
 	} L;
 	struct N {	/* -N<feature>[/<pen>] */
 		BOOLEAN active;
-		BOOLEAN use[GMT_N_BLEVELS];
+		int use[GMT_N_BLEVELS];
 		struct GMT_PEN pen[GMT_N_BLEVELS];
 	} N;
 	struct Q {	/* -Q */
@@ -122,10 +122,12 @@ struct PSCOAST_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i, n, np, ind, bin, base, anti_bin = -1, level, flag, np_new, k, last_k, err;
+	int i, np, ind, bin, base, anti_bin = -1, level, flag, np_new, k, last_k, err;
 	int level_to_be_painted = 0, direction, start_direction, stop_direction;
 	int n_blevels = 0, n_rlevels = 0, bin_trouble, lowest_level = 1, n_pieces = 0;
 
+	GMT_LONG n;
+	
 	BOOLEAN	error = FALSE, shift = FALSE, need_coast_base, recursive, fill_ocean, fill_land;
 	BOOLEAN	greenwich = FALSE, possibly_donut_hell = FALSE, fill_in_use = FALSE;
 	BOOLEAN clobber_background, paint_polygons = FALSE, donut, dumping = FALSE;
@@ -153,8 +155,6 @@ int main (int argc, char **argv)
 
 	Ctrl = (struct PSCOAST_CTRL *) New_Pscoast_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
 
-	step = gmtdefs.line_step;
-	
 	/* Check and interpret the command line arguments */
 
 	for (i = 1; i < argc; i++) {
@@ -234,7 +234,7 @@ int main (int argc, char **argv)
 					}
 					else	/* No pen specified, use default */
 						GMT_init_pen (&pen, GMT_PENWIDTH);
-						
+
 					switch (argv[i][2]) {
 						case 'a':
 							for (k = 0; k < GMT_N_BLEVELS; k++) Ctrl->N.use[k] = TRUE;
@@ -536,7 +536,7 @@ int main (int argc, char **argv)
 		east -= 360.0;
 	}
 
-	if (Ctrl->Q.active && project_info.projection == -1) {	/* Fake area and linear projection */
+	if (Ctrl->Q.active && project_info.projection == GMT_NO_PROJ) {	/* Fake area and linear projection */
 		GMT_parse_J_option ("x1d");
 		east = north = 1.0;
 	}
@@ -581,7 +581,7 @@ int main (int argc, char **argv)
 			GMT_ps.clip_on = TRUE;	/* Signal that this program initiates clipping that wil outlive this process */
 
 		GMT_plotinit (argc, argv);
-		 
+
         	if (Ctrl->Q.active) {  /* Just undo previous clip-path */
               		ps_clipoff ();
 
@@ -590,8 +590,11 @@ int main (int argc, char **argv)
 			GMT_plotend ();
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
+			
+			Free_Pscoast_Ctrl (Ctrl);	/* Deallocate control structure */
 
 			GMT_end (argc, argv);
+			exit (EXIT_SUCCESS);
 		}
 
 		if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
@@ -641,20 +644,20 @@ int main (int argc, char **argv)
 			ps_line (&xtmp[n], &ytmp[n], n, -1, TRUE, FALSE);
 			GMT_fill (xtmp, ytmp, n, &Ctrl->S.fill, -9);
 		}
-		else 
+		else
 			GMT_fill (xtmp, ytmp, n, &Ctrl->S.fill, FALSE);
 		GMT_free ((void *)xtmp);
 		GMT_free ((void *)ytmp);
-		fill_ocean = FALSE; 
+		fill_ocean = FALSE;
 	}
 	if (clipping) {
 		start_direction = (Ctrl->G.clip) ? 1 : -1;
-		stop_direction  = (Ctrl->G.clip) ? 1 : -1; 
+		stop_direction  = (Ctrl->G.clip) ? 1 : -1;
 		level_to_be_painted = (Ctrl->G.clip) ? 1 : 0;
 	}
 	else if (recursive) {
 		start_direction = (fill_land) ? 1 : -1;
-		stop_direction  = (fill_land) ? 1 : -1; 
+		stop_direction  = (fill_land) ? 1 : -1;
 		level_to_be_painted = (fill_land) ? 1 : 0;
 	}
 	else {
@@ -663,13 +666,11 @@ int main (int argc, char **argv)
 		lowest_level = (fill_ocean) ? 0 : 1;
 	}
 
-	if (west < 0.0 && east > 0.0 && project_info.projection > 5) greenwich = TRUE;
+	if (west < 0.0 && east > 0.0 && !GMT_IS_LINEAR) greenwich = TRUE;
 	if ((360.0 - fabs (project_info.e - project_info.w) ) < c.bsize) {
 		GMT_world_map = TRUE;
-		if (project_info.projection == GMT_GNOMONIC) GMT_world_map = FALSE;
-#ifdef _GENPER
-		if (project_info.projection == GMT_GENPER) GMT_world_map = FALSE;
-#endif /* _GENPER */
+		if (project_info.projection == GMT_GNOMONIC || project_info.projection == GMT_GENPER) GMT_world_map = FALSE;
+		if (GMT_IS_AZIMUTHAL) GMT_world_map = FALSE;
 	}
 	if (GMT_world_map && greenwich)
 		edge = project_info.central_meridian;
@@ -679,13 +680,11 @@ int main (int argc, char **argv)
 		if (edge > 180.0) edge = 180.0;
 	}
 
-	/* west_border = project_info.w;	east_border = project_info.e; */
 	if (project_info.w < 0.0 && project_info.e <= 0.0) {	/* Temporarily shift boundaries */
 		project_info.w += 360.0;
 		project_info.e += 360.0;
 		if (project_info.central_meridian < 0.0) project_info.central_meridian += 360.0;
 	}
-	/* west_border = project_info.w;	east_border = project_info.e; */
 	west_border = floor (project_info.w / c.bsize) * c.bsize;
 	east_border = ceil (project_info.e / c.bsize) * c.bsize;
 
@@ -693,6 +692,9 @@ int main (int argc, char **argv)
 
 	if (Ctrl->Z.active) project_info.z_level = Ctrl->Z.level;
 
+	/* Maximum step size (in degrees) used for interpolation of line segments along great circles */
+	step = gmtdefs.line_step / project_info.x_scale / project_info.M_PR_DEG;
+
 	for (ind = 0; need_coast_base && ind < c.nb; ind++) {	/* Loop over necessary bins only */
 
 		bin = c.bins[ind];
@@ -724,7 +726,7 @@ int main (int argc, char **argv)
 			bin_y[2] = bin_y[3] = c.lat_sw + c.bsize;
 			GMT_geo_to_xy (c.lon_sw + 0.5 * c.bsize, c.lat_sw + 0.5 * c.bsize, &x_c, &y_c);
 			dist = hypot (x_c - x_0, y_c - y_0);
-			donut_hell = GMT_non_zero_winding (anti_lon, anti_lat, bin_x, bin_y, 5);
+			donut_hell = GMT_non_zero_winding (anti_lon, anti_lat, bin_x, bin_y, (GMT_LONG)5);
 			if (!donut_hell) donut_hell = (dist > 0.8 * project_info.r);
 		}
 
@@ -732,12 +734,11 @@ int main (int argc, char **argv)
 
 			/* Assemble one or more segments into polygons */
 
-	/*		if ((np = GMT_assemble_shore (&c, direction, TRUE, shift, edge, &p)) == 0) continue; */
 			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, TRUE, shift, west_border, east_border, &p)) == 0) continue;
 
 			/* Get clipped polygons in x,y inches that can be plotted */
 
-			np_new = GMT_prep_polygons (&p, np, greenwich, donut_hell, step, bin_trouble);
+			np_new = GMT_prep_polygons (&p, np, donut_hell, step, bin_trouble);
 
 			if (clipping) {
 				for (k = level_to_be_painted; k < GMT_MAX_GSHHS_LEVEL - 1; k++) {
@@ -789,7 +790,6 @@ int main (int argc, char **argv)
 		}
 
 		if (Ctrl->W.active && c.ns) {	/* Draw or dump shorelines, no need to assemble polygons */
-			/* if ((np = GMT_assemble_shore (&c, direction, FALSE, shift, edge, &p)) == 0) continue; */
 			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, FALSE, shift, west_border, east_border, &p)) == 0) continue;
 
 			for (i = 0; i < np; i++) {
@@ -803,7 +803,7 @@ int main (int argc, char **argv)
 					}
 				}
 				else {
-					if (donut_hell) p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, greenwich, step);
+					if (donut_hell) p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, step, 0);
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
 					if (GMT_n_plot) GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);
 				}
@@ -840,7 +840,10 @@ int main (int argc, char **argv)
 				shift = ((left - edge) <= 180.0 && (right - edge) > 180.0);
 			}
 
-			if ((np = GMT_assemble_br (&r, shift, edge, &p)) == 0) continue;
+			if ((np = GMT_assemble_br (&r, shift, edge, &p)) == 0) {
+				GMT_free_br (&r);
+				continue;
+			}
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
@@ -868,10 +871,7 @@ int main (int argc, char **argv)
 			/* Free up memory */
 
 			GMT_free_br (&r);
-			for (k = 0; k < np; k++) {
-				GMT_free ((void *)p[k].lon);
-				GMT_free ((void *)p[k].lat);
-			}
+			GMT_free_polygons (p, np);
 			GMT_free ((void *)p);
 		}
 		GMT_br_cleanup (&r);
@@ -902,7 +902,10 @@ int main (int argc, char **argv)
 				shift = ((left - edge) <= 180.0 && (right - edge) > 180.0);
 			}
 
-			if ((np = GMT_assemble_br (&b, shift, edge, &p)) == 0) continue;
+			if ((np = GMT_assemble_br (&b, shift, edge, &p)) == 0) {
+				GMT_free_br (&b);
+				continue;
+			}
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
@@ -915,7 +918,7 @@ int main (int argc, char **argv)
 					}
 				}
 				else {
-					p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, greenwich, step);
+					p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, step, 0);
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
 					if (!GMT_n_plot) continue;
 
@@ -931,10 +934,7 @@ int main (int argc, char **argv)
 			/* Free up memory */
 
 			GMT_free_br (&b);
-			for (k = 0; k < np; k++) {
-				GMT_free ((void *)p[k].lon);
-				GMT_free ((void *)p[k].lat);
-			}
+			GMT_free_polygons (p, np);
 			GMT_free ((void *)p);
 		}
 		GMT_br_cleanup (&b);
@@ -968,7 +968,7 @@ void recursive_painting (int k0, int np, struct GMT_GSHHS_POL p[], int level, in
 	/* To avoid pointing where no paint should go we assemble all the swiss cheese polygons by starting
 	 * with the lowest level (0) and looking for polygons inside polygons of the same level.  We do this
 	 * using a recursive algorithm */
-	 
+
 	int k, flag;
 	char text[GMT_TEXT_LEN];
 	BOOLEAN add_this_polygon_to_path (int k0, struct GMT_GSHHS_POL *p, int level, int k);
@@ -989,7 +989,7 @@ void recursive_painting (int k0, int np, struct GMT_GSHHS_POL p[], int level, in
 		}
 
 		if (p[k].level == start_level) {	/* Done nesting, time to paint the assembled swiss cheese polygon */
-			ps_clipon (p[k].lon, p[k].lat, 0, GMT_no_rgb, 4);	/* Dummy call with no points just to end current path */
+			ps_clipon (p[k].lon, p[k].lat, (GMT_LONG)0, GMT_no_rgb, 4);	/* Dummy call with no points just to end current path */
 			GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].level], -1);
 		}
 	}
@@ -1033,11 +1033,11 @@ BOOLEAN add_this_polygon_to_path (int k0, struct GMT_GSHHS_POL *p, int level, in
 void *New_Pscoast_Ctrl () {	/* Allocate and initialize a new control structure */
 	int k;
 	struct PSCOAST_CTRL *C;
-	
-	C = (struct PSCOAST_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSCOAST_CTRL), "New_Pscoast_Ctrl");
-	
+
+	C = (struct PSCOAST_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSCOAST_CTRL), "New_Pscoast_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
-	
+
 	C->A.high = GMT_MAX_GSHHS_LEVEL;			/* Include all GSHHS levels */
 	C->D.set = 'l';						/* Low-resolution coastline data */
 	C->E.azimuth = 180.0;
@@ -1053,10 +1053,10 @@ void *New_Pscoast_Ctrl () {	/* Allocate and initialize a new control structure *
 #ifdef DEBUG
 	C->debug.bin = -1;
 #endif
-	
+
 	return ((void *)C);
 }
 
 void Free_Pscoast_Ctrl (struct PSCOAST_CTRL *C) {	/* Deallocate control structure */
-	GMT_free ((void *)C);	
+	GMT_free ((void *)C);
 }
