@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pscontour.c,v 1.81 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pscontour.c,v 1.101 2008/03/27 07:28:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -104,24 +104,31 @@ struct PSCONTOUR_CHAIN {
 
 int main(int argc, char **argv)
 {
-	int n, np, nx, i, ij, k, k2, k3, node1, node2, c, rgb[3], PSCONTOUR_SUM;
-	int n_alloc, section = 0, *ind, *vert, *cind, n_fields, n_contours;
+	int *ind, nx, i, k, k2, k3, node1, node2, c, rgb[3], PSCONTOUR_SUM;
+	int n_alloc, section = 0, *vert, *cind, n_fields, n_contours = 0;
 	int add, close, bad, n_expected_fields, low, high, last_entry, last_exit;
 
-	BOOLEAN error = FALSE, more, skip = FALSE;
+	GMT_LONG ij, n, np;
+	
+	BOOLEAN error = FALSE, more, skip = FALSE, advance = FALSE;
 
 	double xx[3], yy[3], zz[3], xout[5], yout[5], *in, west, east, south, north;
 	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, current_contour = -DBL_MAX, xyz[2][3];
 
 	char line[BUFSIZ], cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT];
+#ifdef TRIANGLE_D
+	char *algorithm = "Shewchuk";
+#else
+	char *algorithm = "Watson";
+#endif
 
 	FILE *fp = NULL, *fp_d = NULL;
 
-	struct PSCONTOUR *cont;
+	struct PSCONTOUR *cont = NULL;
 	struct PSCONTOUR_CTRL *Ctrl;
 
 	int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
-	void paint_it (double x[], double y[], int n, double z);
+	void paint_it (double x[], double y[], GMT_LONG n, double z);
 	void *New_Pscontour_Ctrl (), Free_Pscontour_Ctrl (struct PSCONTOUR_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -180,7 +187,7 @@ int main(int argc, char **argv)
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
-					GMT_free ((void *)Ctrl->D.file);
+					free ((void *)Ctrl->D.file);
 					Ctrl->D.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
@@ -238,11 +245,11 @@ int main(int argc, char **argv)
 	}
 
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
-		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation\n\n", GMT_VERSION);
+		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation [%s]\n\n", GMT_VERSION, algorithm);
 		fprintf (stderr,"usage: pscontour <xyzfile> -C<cpt_file> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
 		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [-E<az>/<el>]\n", GMT_B_OPT);
 		fprintf (stderr, "\t[%s] [%s] [-I] [%s] [-K] [-L<pen>] [%s] [-N] [-O]\n", GMT_CONTG, GMT_H_OPT, GMT_Jz_OPT, GMT_Mo_OPT);
- 		fprintf (stderr, "\t[-P] [S-] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
+ 		fprintf (stderr, "\t[-P] [-S] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
 		fprintf (stderr, "\t[%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -351,7 +358,12 @@ int main(int argc, char **argv)
 		fprintf (stderr, "%s: -I option requires constant color between contours!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-
+#ifdef GMT_CPT2	
+	if (GMT_categorical) {
+		fprintf (stderr, "%s: GMT WARNING:  Categorical data (as implied by CPT file) does not have contours.  Check plot.\n", GMT_program);
+		exit (EXIT_FAILURE);
+	}
+#endif
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
 	GMT_plotinit (argc, argv);
@@ -382,7 +394,7 @@ int main(int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
@@ -409,11 +421,18 @@ int main(int argc, char **argv)
 			n++;
 
 			if (n == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof (double), GMT_program);
 				y = (double *) GMT_memory ((void *)y, (size_t)n_alloc, sizeof (double), GMT_program);
 				z = (double *) GMT_memory ((void *)z, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
+			if (n == INT_MAX) {
+				fprintf (stderr, "%s: ERROR: Cannot triangulate more than %d points\n", GMT_program, INT_MAX);
+				GMT_free ((void *)x);
+				GMT_free ((void *)y);
+				GMT_free ((void *)z);
+				exit (EXIT_FAILURE);
+			}	
 		}
 
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -430,9 +449,9 @@ int main(int argc, char **argv)
 
 	for (i = 0; i < n; i++) GMT_geo_to_xy (x[i], y[i], &x[i], &y[i]);
 
-	if (fp_d) {
-		n_alloc = 3 * GMT_CHUNK;
-		ind = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), GMT_program);
+	if (Ctrl->T.active) {	/* Read precalculated triangulation indices */
+		n_alloc = GMT_CHUNK;
+		ind = (int *) GMT_memory (VNULL, (size_t)(3*n_alloc), sizeof (int), GMT_program);
 
 		ij = np = 0;
 
@@ -442,12 +461,24 @@ int main(int argc, char **argv)
 			more = (GMT_fgets (line, BUFSIZ, fp_d) != CNULL);
 
 		while (more) {
-			if (!GMT_io.binary[GMT_IN] && sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]) != 3) continue;
-			ij += 3;
-			np++;
-			if (ij == n_alloc) {
-				n_alloc += (3 * GMT_CHUNK);
-				ind = (int *) GMT_memory ((void *)ind, (size_t)n_alloc, sizeof (int), GMT_program);
+			if (!GMT_io.binary[GMT_IN]) {	/* Ascii input, must be more careful */
+				advance = FALSE;
+				if (!GMT_is_a_blank_line (line)) {	/* Got a record to process */
+					i = sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]);
+				 	if (i != 3) {	/* Mangled record? */
+						fprintf (stderr, "%s: Error reading triangulation indices near line %ld\n", GMT_program, np);
+						exit (EXIT_FAILURE);
+					}
+					advance = TRUE;
+				}
+			}
+			if (advance) {	/* All is well if we get here */
+				ij += 3;
+				np++;
+			}
+			if (np == n_alloc) {
+				n_alloc <<= 1;
+				ind = (int *) GMT_memory ((void *)ind, (size_t)(3*n_alloc), sizeof (int), GMT_program);
 			}
 			if (GMT_io.binary[GMT_IN])	/* Binary input */
 				more = (GMT_fread ((void *)&ind[ij], sizeof (int), (size_t)3, fp_d) == 3);
@@ -459,7 +490,7 @@ int main(int argc, char **argv)
 	}
 	else	/* Do Delaunay triangulation */
 
-		np = GMT_delaunay (x, y, n, &ind);
+		np = GMT_delaunay (x, y, (int)n, &ind);
 
 	if (Ctrl->L.active) {	/* Draw triangular mesh */
 
@@ -472,28 +503,29 @@ int main(int argc, char **argv)
 			xx[2] = x[ind[k]];	yy[2] = y[ind[k++]];
 
 			if (project_info.three_D) for (c = 0; c < 3; c++) GMT_xy_do_z_to_xy (xx[c], yy[c], project_info.z_level, &xx[c], &yy[c]);
-			ps_line (xx, yy, 3, 3, 1, FALSE);
+			ps_line (xx, yy, (GMT_LONG)3, 3, 1, FALSE);
 		}
 	}
 
 	/* Get PSCONTOUR structs */
 
-	n_contours = GMT_n_colors + 1;
-	cont = (struct PSCONTOUR *) GMT_memory (VNULL, (size_t)n_contours, sizeof (struct PSCONTOUR), GMT_program);
+	if (Ctrl->W.active) {
+		n_contours = GMT_n_colors + 1;
+		cont = (struct PSCONTOUR *) GMT_memory (VNULL, (size_t)n_contours, sizeof (struct PSCONTOUR), GMT_program);
 
-	for (i = 0; i < GMT_n_colors; i++) {
-		cont[i].val = GMT_lut[i].z_low;
-		cont[i].type = (GMT_lut[i].annot && !Ctrl->A.off) ? 'A' : 'C';
-		cont[i].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		for (i = 0; i < GMT_n_colors; i++) {
+			cont[i].val = GMT_lut[i].z_low;
+			cont[i].type = (GMT_lut[i].annot && !Ctrl->A.off) ? 'A' : 'C';
+			cont[i].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		}
+		cont[GMT_n_colors].val = GMT_lut[GMT_n_colors-1].z_high;
+		cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].annot & 2) && !Ctrl->A.off) ? 'A' : 'C';
+		cont[GMT_n_colors].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		for (i = 0; i < n_contours; i++) {
+			cont[i].n_alloc = GMT_SMALL_CHUNK;
+			cont[i].L = (struct PSCONTOUR_LINE *) GMT_memory (VNULL, (size_t)GMT_SMALL_CHUNK, sizeof (struct PSCONTOUR_LINE), GMT_program);
+		}
 	}
-	cont[GMT_n_colors].val = GMT_lut[GMT_n_colors-1].z_high;
-	cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].annot & 2) && !Ctrl->A.off) ? 'A' : 'C';
-	cont[GMT_n_colors].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
-	for (i = 0; i < n_contours; i++) {
-		cont[i].n_alloc = GMT_SMALL_CHUNK;
-		cont[i].L = (struct PSCONTOUR_LINE *) GMT_memory (VNULL, (size_t)GMT_SMALL_CHUNK, sizeof (struct PSCONTOUR_LINE), GMT_program);
-	}
-
 	Ctrl->contour.line_pen = Ctrl->W.pen;
 
 	for (i = ij = 0; i < np; i++, ij += 3) {	/* For all triangles */
@@ -515,7 +547,7 @@ int main(int argc, char **argv)
 					zzz += zz[k];
 					kzz++;
 				}
-				if (kzz) paint_it (xx, yy, 3, zzz / kzz);
+				if (kzz) paint_it (xx, yy, (GMT_LONG)3, zzz / kzz);
 			}
 			else {	/* Must paint all those slices separately */
 
@@ -626,7 +658,7 @@ int main(int argc, char **argv)
 				cont[c].L[n].y1 = yc[k2++];
 				n++;
 				if (n >= cont[c].n_alloc) {
-					cont[c].n_alloc += GMT_SMALL_CHUNK;
+					cont[c].n_alloc <<= 1;
 					cont[c].L = (struct PSCONTOUR_LINE *) GMT_memory ((void *)cont[c].L, (size_t)cont[c].n_alloc, sizeof (struct PSCONTOUR_LINE), GMT_program);
 				}
 				cont[c].nl = n;
@@ -730,7 +762,7 @@ int main(int argc, char **argv)
 					p = p->next;
 					GMT_free ((void *)q);
 					if (n == n_alloc) {
-						n_alloc += GMT_SMALL_CHUNK;
+						n_alloc <<= 1;
 						xp = (double *) GMT_memory ((void *)xp, (size_t)n_alloc, sizeof (double), GMT_program);
 						yp = (double *) GMT_memory ((void *)yp, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
@@ -772,12 +804,13 @@ int main(int argc, char **argv)
 				GMT_free ((void *)xp);
 				GMT_free ((void *)yp);
 			}
-
+			GMT_free ((void *)head_c);
 		}
 		GMT_contlabel_plot (&Ctrl->contour);
+		GMT_contlabel_free (&Ctrl->contour);
 	}
 
-        if (!Ctrl->N.active) GMT_map_clip_off ();
+    if (!Ctrl->N.active) GMT_map_clip_off ();
 
 	GMT_map_basemap ();
 
@@ -788,7 +821,23 @@ int main(int argc, char **argv)
 	GMT_free ((void *)x);
 	GMT_free ((void *)y);
 	GMT_free ((void *)z);
-	GMT_free ((void *)ind);
+	if (Ctrl->W.active) GMT_free ((void *)cont);
+	if (Ctrl->T.active)
+		GMT_free ((void *)ind);	/* Allocated above by GMT_memory */
+	else {
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	/* Shewchuck's function allocated the memory separately */
+		GMT_memtrack_off (GMT_mem_keeper);
+#endif
+#endif
+		GMT_free ((void *) ind);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+		GMT_memtrack_on (GMT_mem_keeper);
+#endif
+#endif
+	}
 
 	Free_Pscontour_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -864,16 +913,24 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 		if (j%2) j--;	/* Contour went through a single vertex only, skip this */
 	}
 
-	*xc = xout;
-	*yc = yout;
-	*zc = zout;
-	*v = vout;
-	*cindex = cind;
-
-	return (j/2);
+	nx = j / 2;	/* Since j might have changed */
+	if (nx) {
+		*xc = xout;
+		*yc = yout;
+		*zc = zout;
+		*v = vout;
+		*cindex = cind;
+	} else {
+		GMT_free ((void *)xout);
+		GMT_free ((void *)yout);
+		GMT_free ((void *)zout);
+		GMT_free ((void *)vout);
+		GMT_free ((void *)cind);
+	}
+	return (nx);
 }
 
-void paint_it (double x[], double y[], int n, double z) {
+void paint_it (double x[], double y[], GMT_LONG n, double z) {
 	int index, k, rgb[3];
 	struct GMT_FILL *f;
 
@@ -887,16 +944,16 @@ void paint_it (double x[], double y[], int n, double z) {
 	if (project_info.three_D)
 		for (k = 0; k < n; k++) GMT_xy_do_z_to_xy (x[k], y[k], project_info.z_level, &x[k], &y[k]);
 	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {
-		GMT_fill (x, y, n, f, FALSE);	/* Contours drawn separately later if desired */
+		GMT_fill (x, y, (GMT_LONG)n, f, FALSE);	/* Contours drawn separately later if desired */
 	}
 	else
-		ps_patch (x, y, n, rgb, FALSE);	/* Contours drawn separately later if desired */
+		ps_patch (x, y, (GMT_LONG)n, rgb, FALSE);	/* Contours drawn separately later if desired */
 }
 
 void *New_Pscontour_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSCONTOUR_CTRL *C;
 	
-	C = (struct PSCONTOUR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSCONTOUR_CTRL), "New_Pscontour_Ctrl");
+	C = (struct PSCONTOUR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSCONTOUR_CTRL), "New_Pscontour_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	GMT_contlabel_init (&C->contour, 1);
@@ -910,5 +967,8 @@ void *New_Pscontour_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Pscontour_Ctrl (struct PSCONTOUR_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);	
+	if (C->D.file) free ((void *)C->D.file);	
+	if (C->T.file) free ((void *)C->T.file);	
 	GMT_free ((void *)C);	
 }
