@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pshistogram.c,v 1.52 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pshistogram.c,v 1.62 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
  *		     Now uses two passes to get accurate mean and stdev
  * 27-JAN-2005: Added -T option to select column of interrest
  * Version:	4.1.2 Free of global variables, and now uses double precision
+ *		4.2.1 -C is now -F (-C will still work) and -C<cpt> paints bars
  */
 
 #include "gmt.h"
@@ -38,21 +39,24 @@ struct PSHISTOGRAM_CTRL {
 	struct A {	/* -A */
 		BOOLEAN active;
 	} A;
-	struct C {	/* -C */
+	struct C {	/* -C<cpt> */
 		BOOLEAN active;
+		char *file;
 	} C;
 	struct E {	/* -Eazimuth/elevation */
 		BOOLEAN active;
 		double azimuth, elevation;
 	} E;
+	struct F {	/* -F */
+		BOOLEAN active;
+	} F;
 	struct G {	/* -Gfill */
 		BOOLEAN active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -I[o] */
 		BOOLEAN active;
-		BOOLEAN dump;
-		BOOLEAN dumpall;
+		int mode;
 	} I;
 	struct L {	/* -L<pen> */
 		BOOLEAN active;
@@ -89,7 +93,7 @@ struct PSHISTOGRAM_INFO {	/* Control structure for pshistogram */
 	double	yy0, yy1;
 	int	*boxh;
 	int	n_boxes;
-	int	n_counted;
+	GMT_LONG	n_counted;
 	double	box_width;
 	double	west, east, south, north;
 	int	center_box, cumulative;
@@ -100,9 +104,11 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, automatic = FALSE;
 
-	char string[BUFSIZ], text[BUFSIZ], buffer[BUFSIZ], format[BUFSIZ];
+	char buffer[BUFSIZ], format[BUFSIZ];
 	
-	int i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields, pos, n;
+	GMT_LONG i, n_alloc = GMT_CHUNK, n;
+	
+	int n_expected_fields, n_fields;
 	
 	double	*data, stats[6];
 
@@ -113,9 +119,9 @@ int main (int argc, char **argv)
 	struct PSHISTOGRAM_INFO F;
 	struct PSHISTOGRAM_CTRL *Ctrl;
 	
-	int fill_boxes (struct PSHISTOGRAM_INFO *F, double *x, int n);
-	int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill);
-	int get_loc_scl (double *x, int n, double *stats);
+	int fill_boxes (struct PSHISTOGRAM_INFO *F, double *x, GMT_LONG n);
+	int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt);
+	int get_loc_scl (double *x, GMT_LONG n, double *stats);
 	void *New_Pshistogram_Ctrl (), Free_Pshistogram_Ctrl (struct PSHISTOGRAM_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -137,6 +143,7 @@ int main (int argc, char **argv)
 				case 'K':
 				case 'O':
 				case 'P':
+				case 'R':
 				case 'U':
 				case 'V':
 				case 'X':
@@ -156,12 +163,22 @@ int main (int argc, char **argv)
 					Ctrl->A.active = TRUE;
 					break;
 				case 'C':
-					Ctrl->C.active = TRUE;
+					if (argv[i][2]) {	/* Gave an argument */
+						Ctrl->C.file = strdup (&argv[i][2]);
+						Ctrl->C.active = TRUE;
+					}
+					else {/* Backwards compatible with old -C (now -F) */
+						Ctrl->F.active = TRUE;
+						fprintf (stderr, "%s: Warning: -C for centering is obsolete, please use -F\n", GMT_program);
+					}
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
+				case 'F':
+					Ctrl->F.active = TRUE;
+					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
 					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
@@ -178,25 +195,12 @@ int main (int argc, char **argv)
 					break;
 				case 'I':
 					Ctrl->I.active = TRUE;
-					if (argv[i][2] == 'o') Ctrl->I.dump = TRUE;
-					if (argv[i][2] == 'O') Ctrl->I.dumpall = TRUE;
+					if (argv[i][2] == 'o') Ctrl->I.mode = 1;
+					if (argv[i][2] == 'O') Ctrl->I.mode = 2;
 					break;
 				case 'Q':
 					Ctrl->Q.active = TRUE;
 					break;
-				case 'R':
-					project_info.region_supplied = TRUE;
-					strcpy (string, &argv[i][2]);
-					pos = 0;
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.west);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.east);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.south);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.north);
-					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
 					break;
@@ -228,7 +232,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"pshistogram %s - Calculate and plot histograms\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: pshistogram [file] %s -W<width> [%s] [-C] [-Eaz/el] [-G<fill>]\n", GMT_Jx_OPT, GMT_B_OPT);
+		fprintf (stderr, "usage: pshistogram [file] %s -W<width> [%s] [-C<cpt>] [-Eaz/el] [-F] [-G<fill>]\n", GMT_Jx_OPT, GMT_B_OPT);
 		fprintf (stderr, "\t[%s] [-I[o|O]] [-K] [-L<pen>] [-O] [-P] [-Q] [%s]\n", GMT_Ho_OPT, GMT_Rx_OPT);
 		fprintf (stderr, "\t[-S] [-T<col>] [%s] [-V] [%s] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT);
 		fprintf (stderr, "\t[-Z[type]] [%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_bi_OPT, GMT_f_OPT);
@@ -242,8 +246,9 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-A will plot horizontal bars [Default is vertical]\n");
-		fprintf (stderr, "\t-C will center the bins\n");
+		fprintf (stderr, "\t-C Use cpt-file to assign fill to bars based on the mid x-value\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-F will center the bins\n");
 		GMT_fill_syntax ('G', "Select color/pattern for columns.");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I will inquire about min/max x and y.  No plotting is done\n");
@@ -278,7 +283,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (!Ctrl->I.active && (project_info.projection < 0 || project_info.projection > 9)) {
+	if (!Ctrl->I.active && !GMT_IS_LINEAR) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -J option:  Only linear projection supported.\n", GMT_program);
 		error++;
 	}
@@ -296,8 +301,12 @@ int main (int argc, char **argv)
 	}
 	/* Now must specify either fill color with -G or outline pen with -L; JLL */
  
-	if (!(Ctrl->I.active || Ctrl->G.active || Ctrl->L.active)) {
-		fprintf (stderr, "%s: Must specify either fill color (-G), outline pen attributes (-L), or both.\n", GMT_program);
+	if (!(Ctrl->C.active || Ctrl->I.active || Ctrl->G.active || Ctrl->L.active)) {
+		fprintf (stderr, "%s: Must specify either fill (-G) or lookup colors (-C), outline pen attributes (-L), or both.\n", GMT_program);
+		error++;
+	}
+	if (Ctrl->C.active && Ctrl->G.active) {
+		fprintf (stderr, "%s: Cannot specify both fill (-G) and lookup colors (-C).\n", GMT_program);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
@@ -331,10 +340,12 @@ int main (int argc, char **argv)
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	F.cumulative = Ctrl->Q.active;
-	F.center_box = Ctrl->C.active;
+	F.center_box = Ctrl->F.active;
 	if (Ctrl->I.active) gmtdefs.verbose = TRUE;
 	if (!Ctrl->I.active && !project_info.region_supplied) automatic = TRUE;
 
+	if (Ctrl->C.active)  GMT_read_cpt (Ctrl->C.file);
+
 	if (fp_in == NULL) {
 		fp_in = GMT_stdin;
 #ifdef SET_IO_MODE
@@ -353,18 +364,18 @@ int main (int argc, char **argv)
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, 2, n);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, 2, n);
 			exit (EXIT_FAILURE);
 		}
 		if (Ctrl->T.col >= n_fields) {
-			fprintf (stderr, "%s: Expected at least %d fields but only found %d near line %d\n", GMT_program, Ctrl->T.col+1, n_fields, n);
+			fprintf (stderr, "%s: Expected at least %d fields but only found %d near line %ld\n", GMT_program, Ctrl->T.col+1, n_fields, n);
 			exit (EXIT_FAILURE);
 		}
 		data[n] = in[Ctrl->T.col];
 		if (!GMT_is_dnan (data[n])) n++;
 
 		if (n == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			data = (double *) GMT_memory ((void *)data, (size_t) n_alloc, sizeof (double), GMT_program);
 		}
 		x_min = MIN (x_min, data[n-1]);
@@ -376,7 +387,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr,"%s: %d points read\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr,"%s: %ld points read\n", GMT_program, n);
 
 	data = (double *) GMT_memory ((void *)data, (size_t)n , sizeof (double), GMT_program);
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
@@ -418,12 +429,12 @@ int main (int argc, char **argv)
 	}
 
 	if (Ctrl->I.active) {	/* Only info requested, quit before plotting */
-		if (Ctrl->I.dump||Ctrl->I.dumpall) {
+		if (Ctrl->I.mode) {
 			int ibox;
 			double xx, yy;
 			sprintf (format, "%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
 			for (ibox = 0; ibox < F.n_boxes; ibox++) {
-				if (Ctrl->I.dump&&(F.boxh[ibox] == 0)) continue;
+				if (Ctrl->I.mode == 1 && F.boxh[ibox] == 0) continue;
 				xx = F.west + ibox * F.box_width;
 				if (F.center_box) xx -= (0.5 * F.box_width);
 				if (F.hist_type == PSHISTOGRAM_LOG_COUNTS)
@@ -495,7 +506,7 @@ int main (int argc, char **argv)
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 
 	GMT_map_clip_on (GMT_no_rgb, 3);
-	if ( plot_boxes (&F, Ctrl->S.active, Ctrl->A.active, Ctrl->L.active, &Ctrl->L.pen, &Ctrl->G.fill) ) {
+	if ( plot_boxes (&F, Ctrl->S.active, Ctrl->A.active, Ctrl->L.active, &Ctrl->L.pen, &Ctrl->G.fill, Ctrl->C.active) ) {
 		fprintf (stderr, "%s: Fatal error during box plotting.\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
@@ -517,10 +528,10 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
+int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, GMT_LONG n) {
 
 	double	add_half = 0.0;
-	int	b0, b1, i, ibox, count_sum;
+	GMT_LONG	b0, b1, i, ibox, count_sum;
 
 	F->n_boxes = (int)ceil( (F->east - F->west) / F->box_width);
 
@@ -601,10 +612,11 @@ int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
 	return (0);
 }
 
-int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill)
+int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt)
 {
-	int	i, ibox, first = TRUE;
-	double	x[4], y[4], xx, yy, *px, *py;
+	int	i, ibox, first = TRUE, index, rgb[3];
+	double	x[4], y[4], xx, yy, xval, *px, *py;
+	struct GMT_FILL *f;
 
 	if (draw_outline) GMT_setpen (pen);
 
@@ -624,6 +636,7 @@ int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, B
 			x[1] = x[0] + F->box_width;
 			if (x[0] < F->west) x[0] = F->west;
 			if (x[1] > F->east) x[1] = F->east;
+			xval = 0.5 * (x[0] + x[1]);	/* Used for cpt lookup */
 			x[2] = x[1];
 			x[3] = x[0];
 			y[0] = y[1] = F->south;
@@ -655,8 +668,15 @@ int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, B
 				ps_plot (px[3], py[3], PSL_PEN_DRAW);
 				ps_plot (px[2], py[2], PSL_PEN_DRAW);
 			}
+			else if (cpt) {
+				index = GMT_get_rgb_from_z (xval, rgb);
+				if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill)))	/* Pattern */
+					GMT_fill (px, py, (GMT_LONG)4, f, draw_outline);
+				else
+					ps_patch (px, py, (GMT_LONG)4, rgb, draw_outline);
+			}
 			else
-				GMT_fill (px, py, 4, fill, draw_outline);
+				GMT_fill (px, py, (GMT_LONG)4, fill, draw_outline);
 		}
 	}
 
@@ -665,11 +685,12 @@ int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, B
 	return (0);
 }
 
-int get_loc_scl (double *data, int n, double *stats)
+int get_loc_scl (double *data, GMT_LONG n, double *stats)
 {
 	/* Returns stats[] = L2, L1, LMS location, L2, L1, LMS scale  */
 
-	int	i, j, n_multiples;
+	GMT_LONG	i, j;
+	int n_multiples;
 	double	dx;
 
 	if (n < 3) return (-1);
@@ -682,16 +703,16 @@ int get_loc_scl (double *data, int n, double *stats)
 
 	/* Get mode */
 
-	GMT_mode (data, (size_t)n, (size_t)j, 0, 0, &n_multiples, &stats[2]);
+	GMT_mode (data, n, j, 0, 0, &n_multiples, &stats[2]);
 	if (n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, n_multiples);
 
 	/* Get MAD for L1 */
 
-	GMT_getmad (data, (size_t)n, stats[1], &stats[4]);
+	GMT_getmad (data, n, stats[1], &stats[4]);
 
 	/* Get LMSscale for mode */
 
-	GMT_getmad (data, (size_t)n, stats[2], &stats[5]);
+	GMT_getmad (data, n, stats[2], &stats[5]);
 
 	/* Calculate mean and stdev in two passes to minimize risk of overflow */
 
@@ -710,7 +731,7 @@ int get_loc_scl (double *data, int n, double *stats)
 void *New_Pshistogram_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSHISTOGRAM_CTRL *C;
 	
-	C = (struct PSHISTOGRAM_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSHISTOGRAM_CTRL), "New_Pshistogram_Ctrl");
+	C = (struct PSHISTOGRAM_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSHISTOGRAM_CTRL), "New_Pshistogram_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -722,5 +743,6 @@ void *New_Pshistogram_Ctrl () {	/* Allocate and initialize a new control structu
 }
 
 void Free_Pshistogram_Ctrl (struct PSHISTOGRAM_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
