@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psimage.c,v 1.35 2007/03/12 19:52:26 remko Exp $
+ *	$Id: psimage.c,v 1.42 2008/04/15 15:44:44 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -68,7 +68,7 @@ int main (int argc, char **argv)
 {
 	int i, j, n, justify, PS_interpolate;
 
-	BOOLEAN error = FALSE, invert;
+	BOOLEAN error = FALSE;
 
 	double x, y;
 
@@ -267,7 +267,6 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	invert = !Ctrl->I.active;	/* Opposite sense than expected to match -Gfill behavior */
 	PS_interpolate = (Ctrl->W.interpolate) ? -1 : +1;
 	
 	picture = ps_load_image (file, &header);
@@ -286,7 +285,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->E.dpi > 0.0) Ctrl->W.width = (double) header.width / Ctrl->E.dpi;
 	if (Ctrl->W.height == 0.0) Ctrl->W.height = header.height * Ctrl->W.width / header.width;
-	justify = GMT_just_decode (Ctrl->C.justify, -1, -1);
+	justify = GMT_just_decode (Ctrl->C.justify, 12);
 	Ctrl->C.x -= 0.5 * ((justify-1)%4) * Ctrl->W.width;
 	Ctrl->C.y -= 0.5 * (justify/4) * Ctrl->W.height;
 
@@ -296,9 +295,10 @@ int main (int argc, char **argv)
 		for (i = 0; i < Ctrl->N.nx; i++) {
 			x = Ctrl->C.x + i * Ctrl->W.width;
 			if (header.depth == 0)
-				ps_epsimage (x, y, Ctrl->W.width, Ctrl->W.height, picture, header.length, header.width, header.height, header.xorigin, header.yorigin);
+				ps_epsimage (x, y, Ctrl->W.width, Ctrl->W.height, picture, (GMT_LONG)header.length, header.width, header.height, (GMT_LONG)header.xorigin, (GMT_LONG)header.yorigin);
 			else if (header.depth == 1)
-				ps_bitimage (x, y, Ctrl->W.width, Ctrl->W.height, picture, header.width, header.height, Ctrl->I.active, Ctrl->G.f_rgb, Ctrl->G.b_rgb);
+				/* Invert is opposite from what is expected. This is to match the behaviour of -Gp */
+				ps_bitimage (x, y, Ctrl->W.width, Ctrl->W.height, picture, header.width, header.height, !Ctrl->I.active, Ctrl->G.f_rgb, Ctrl->G.b_rgb);
 			else
 				GMT_color_image (x, y, Ctrl->W.width, Ctrl->W.height, picture, header.width, header.height, PS_interpolate * header.depth);
 		}
@@ -322,7 +322,7 @@ int main (int argc, char **argv)
 void *New_Psimage_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSIMAGE_CTRL *C;
 	
-	C = (struct PSIMAGE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSIMAGE_CTRL), "New_Psimage_Ctrl");
+	C = (struct PSIMAGE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSIMAGE_CTRL), "New_Psimage_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	GMT_init_pen (&C->F.pen, GMT_PENWIDTH);
