@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pslegend.c,v 1.84 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pslegend.c,v 1.99 2008/04/15 15:44:43 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -71,10 +71,10 @@ int main (int argc, char **argv)
 	char *del = "rm -f", *null = "/dev/null", *escape = "\\", quote = '\'';
 #endif
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT], txt_e[GMT_LONG_TEXT];
-	char txt_f[GMT_LONG_TEXT], key[GMT_LONG_TEXT], *f = CNULL, sub[GMT_LONG_TEXT], cmd[BUFSIZ];
+	char txt_f[GMT_LONG_TEXT], key[GMT_LONG_TEXT], *f = CNULL, sub[GMT_LONG_TEXT];
 	char symbol[GMT_LONG_TEXT], text[BUFSIZ], *u = CNULL, image[BUFSIZ], xx[GMT_LONG_TEXT], yy[GMT_LONG_TEXT];
 	char size[GMT_LONG_TEXT], angle[GMT_LONG_TEXT], mapscale[GMT_LONG_TEXT], font[GMT_LONG_TEXT], lspace[GMT_LONG_TEXT];
-	char tw[GMT_LONG_TEXT], jj[GMT_LONG_TEXT], line[BUFSIZ], vpen[GMT_LONG_TEXT], script[GMT_LONG_TEXT];
+	char tw[GMT_LONG_TEXT], jj[GMT_LONG_TEXT], line[BUFSIZ], vpen[GMT_LONG_TEXT], script[GMT_LONG_TEXT], tmptxt[GMT_LONG_TEXT];
 	char sarg[GMT_LONG_TEXT], sparg[BUFSIZ], *jarg = CNULL, txtcolor[GMT_LONG_TEXT], psxy[GMT_LONG_TEXT], pstext[GMT_LONG_TEXT];
 	char bar_cpt[GMT_LONG_TEXT], bar_gap[GMT_LONG_TEXT], bar_height[GMT_LONG_TEXT], bar_opts[BUFSIZ], mode;
 	
@@ -88,14 +88,13 @@ int main (int argc, char **argv)
 	
 	FILE *fp = NULL, *fpo;
 	
-	void GMT_replace_symbol (char *symbol);
 	void *New_Pslegend_Ctrl (), Free_Pslegend_Ctrl (struct PSLEGEND_CTRL *C);
 	
 	/* Because pslegend uses system calls we must first make a copy of any arguments that GMT_begin will remove, such
 	 * as +gmtdefaults and --PAR=value
 	 */
 	 
-	memset ((void *)sparg, 0, BUFSIZ);
+	memset ((void *)sparg, 0, (size_t)BUFSIZ);
 	for (i = 1, k = 0; i < argc; i++) {
 		if (argv[i][0] == '+' || (argv[i][0] == '-' && argv[i][1] == '-')) {
 			if (k) strcat (sparg, " ");
@@ -230,6 +229,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-F Draw border around the legend (using FRAME_PEN) [Default is no border]\n");
 		GMT_fill_syntax ('G', "Set the fill for the legend box [Default is no fill].");
 		GMT_explain_option ('K');
+		fprintf (stderr, "\t-L Sets the linespacing factor in units of the current annotation font size [1.1]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-S Dump legend script to stdout, or optionally to file <script>.\n");
@@ -272,15 +272,21 @@ int main (int argc, char **argv)
 		}
 	}
 	else {
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Generate legend plot\n", GMT_program);
-		sprintf (script, "GMT%d.bat", getpid ());
+		if (GMT_TMPDIR)
+			sprintf (script, "%s%cpslegend_%d.bat", GMT_TMPDIR, DIR_DELIM, (int)getpid ());
+		else
+			sprintf (script, "GMT%d.bat", (int)getpid ());
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Generate temporary legend script %s\n", GMT_program, script);
 		fpo = fopen (script, "w");
 	}
+#ifdef WIN32
+	fprintf (fpo, "@ECHO OFF\n");
+#endif
 	if (!fp) fp = GMT_stdin;
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	justify = GMT_just_decode (Ctrl->D.justify, -1, -1);
+	justify = GMT_just_decode (Ctrl->D.justify, 12);
 	
 	if (!Ctrl->D.cartesian) {
 		GMT_geo_to_xy (Ctrl->D.lon, Ctrl->D.lat, &x, &y);
@@ -297,7 +303,7 @@ int main (int argc, char **argv)
 	Ctrl->D.lon -= 0.5 * ((justify-1)%4) * Ctrl->D.width;
 	Ctrl->D.lat -= 0.5 * (justify/4) * Ctrl->D.height;
 
-	/* Initialize psxy and pstext call strings with appropriate values */
+	/* Initialize psxy and pstext call strings and tmptxt file name with appropriate values */
 	
 	if (GMT_ps.absolute) {	/* Must pass the -Xa -Ya settings to every psxy and pstext call */
 		mode = 'a';
@@ -309,25 +315,24 @@ int main (int argc, char **argv)
 		sprintf (pstext, "pstext -R -JX -O -K %s", sparg);
 		sprintf (psxy, "psxy -R -JX -O -K %s", sparg);
 	}
+	if (GMT_TMPDIR)
+		sprintf (tmptxt, "%s%cpslegend_%d.txt", GMT_TMPDIR, DIR_DELIM, (int)getpid ());
+	else
+		sprintf (tmptxt, "pslegend_%d.txt", (int)getpid ());
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Use temporary input file %s\n", GMT_program, tmptxt);
 	
 	/* First draw legend frame box.  Note -JX%gi/-%gi which means y is positive down from the top of the box */
 
 	if (Ctrl->F.active || Ctrl->G.active)
-		sprintf (cmd, "psbasemap -R0/%g/0/%g -JX%gi/-%gi -X%c%gi -Y%c%gi -K %s", Ctrl->D.width, Ctrl->D.height, Ctrl->D.width, Ctrl->D.height, mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
+		fprintf (fpo, "psbasemap -R0/%g/0/%g -JX%gi/-%gi -X%c%gi -Y%c%gi -K %s", Ctrl->D.width, Ctrl->D.height, Ctrl->D.width, Ctrl->D.height, mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
 	else
-		sprintf (cmd, "psxy -R0/%g/0/%g -JX%gi/-%gi %s -X%c%gi -Y%c%gi -K %s", Ctrl->D.width, Ctrl->D.height, Ctrl->D.width, Ctrl->D.height, null, mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
-	if (GMT_ps.portrait) strcat (cmd, " -P");
-	if (Ctrl->F.active) strcat (cmd, " -B0");
-	if (GMT_ps.overlay) strcat (cmd, " -O");
-	if (Ctrl->G.active) {
-		sprintf (sub, " %s", f);
-		strcat (cmd, sub);
-	}
-	if (u) {
-		sprintf (sub, " -U\"%s\"", u);
-		strcat (cmd, sub);
-	}
-	fprintf (fpo, "%s\n", cmd);
+		fprintf (fpo, "psxy -R0/%g/0/%g -JX%gi/-%gi %s -X%c%gi -Y%c%gi -K %s", Ctrl->D.width, Ctrl->D.height, Ctrl->D.width, Ctrl->D.height, null, mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
+	if (GMT_ps.portrait) fprintf (fpo, " -P");
+	if (Ctrl->F.active) fprintf (fpo, " -B0");
+	if (GMT_ps.overlay) fprintf (fpo, " -O");
+	if (Ctrl->G.active) fprintf (fpo, " %s", f);
+	if (u) fprintf (fpo, " -U\"%s\"", u);
+	fprintf (fpo, "\n");
 
 	x0 = Ctrl->C.dx;
 	y0 = Ctrl->C.dy;
@@ -338,17 +343,23 @@ int main (int argc, char **argv)
 
 	while (GMT_fgets (line, BUFSIZ, fp)) {
 
-		if (line[0] == '\0' || line[0] == '#') continue;	/* Skip blank lines or # comments */
+		if (GMT_is_a_blank_line (line)) continue;	/* Skip blank lines or # comments */
 
 		switch (line[0]) {
 			case 'C':	/* Color change */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				sscanf (&line[2], "%[^\n]", txtcolor);
-				if (txtcolor[0] == '-') sprintf (pstext, "pstext -R -JX -O -K");
-				else sprintf (pstext, "pstext -R -JX -O -K -G%s", txtcolor);
+				if (GMT_ps.absolute) {	/* Must pass the -Xa -Ya settings to every psxy and pstext call */
+					if (txtcolor[0] == '-') sprintf (pstext, "pstext -R -JX -O -K -X%c%gi -Y%c%gi %s", mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
+					else sprintf (pstext, "pstext -R -JX -O -K -G%s -X%c%gi -Y%c%gi %s", txtcolor, mode, Ctrl->D.lon, mode, Ctrl->D.lat, sparg);
+				}
+				else {
+					if (txtcolor[0] == '-') sprintf (pstext, "pstext -R -JX -O -K %s", sparg);
+					else sprintf (pstext, "pstext -R -JX -O -K -G%s %s", txtcolor, sparg);
+				}
 				break;
 			case 'B':	/* Color scale Bar */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s %s %s %[^\n]", bar_cpt, bar_gap, bar_height, bar_opts);
@@ -358,7 +369,7 @@ int main (int argc, char **argv)
 				y0 += L + half_line_spacing;
 				break;
 			case 'H':	/* Header record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s %s %[^\n]", size, font, text);
@@ -369,12 +380,12 @@ int main (int argc, char **argv)
 				break;
 
 			case 'I':	/* Image record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s %s %s", image, size, key);
 				(void) ps_load_image (image, &header);
-				justify = GMT_just_decode (key, -1, -1);
+				justify = GMT_just_decode (key, 12);
 				x_off = (justify%4 == 1) ? x0 : ((justify%4 == 3) ? Ctrl->D.width - Ctrl->C.dx : 0.5 * Ctrl->D.width);
 				L = GMT_convert_units (size, GMT_INCH) * ((double)header.height / (double)header.width);
 				fprintf (fpo, "psimage -O -K %s -W%s -C%gi/%gi/%s\n", image, size, x_off, Ctrl->D.height-y0, key);
@@ -382,20 +393,20 @@ int main (int argc, char **argv)
 				break;
 
 			case 'L':	/* Label record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s %s %s %[^\n]", size, font, key, text);
 				if (size[0] == '-') sprintf (size, "%g", gmtdefs.label_font_size);
 				if (font[0] == '-') sprintf (font, "%d", gmtdefs.label_font);
 				y0 += Ctrl->L.spacing * atoi (size) / 72.0;
-				justify = GMT_just_decode (key, -1, 0);
+				justify = GMT_just_decode (key, 0);
 				x_off = (justify%4 == 1) ? x0 : ((justify%4 == 3) ? Ctrl->D.width - Ctrl->C.dx : 0.5 * Ctrl->D.width);
 				fprintf (fpo, "echo %c%g %g %s 0 %s B%s %s%c | %s\n", quote, x_off, y0, size, font, key, text, quote, pstext);
 				break;
 
 			case 'M':	/* Map scale record M lon0|- lat0 length[:label:just] f|p [-R -J] */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				n_scan = sscanf (&line[2], "%s %s %s %s %s %s", txt_a, txt_b, txt_c, txt_d, txt_e, txt_f);
@@ -435,16 +446,13 @@ int main (int argc, char **argv)
 				break;
 
 			case 'S':	/* Symbol record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				symbol_number++;
 				sscanf (&line[2], "%s %s %s %s %s %s %[^\n]", txt_a, symbol, size, txt_c, txt_d, txt_b, text);
 				off_ss = GMT_convert_units (txt_a, GMT_INCH);
 				off_tt = GMT_convert_units (txt_b, GMT_INCH);
 				if (n_columns == 1 || symbol_number%n_columns == 1) y0 += half_line_spacing;
-				if ((txt_c[0] == 'P' || txt_c[0] == 'p') && symbol[0] != 'k') {	/* Must use kustom equivalent symbol */
-					GMT_replace_symbol (symbol);	/* E.g., convert c0.5i to kcircle/0.5i */
-				}
 				x_off = x0 + (Ctrl->D.width / n_columns) * ((symbol_number - 1) % n_columns);
 				if (symbol[0] == 'f') {	/* Front is different, must plot as a line segment */
 					i = 0;
@@ -453,26 +461,19 @@ int main (int argc, char **argv)
 						fprintf (stderr, "%s: ERROR: -Sf option must have a tick length\n", GMT_program);
 						exit (EXIT_FAILURE);
 					}
-					i++;
+					size[i] = '\0';	/* Temporarily truncate */
 					x = 0.5*GMT_convert_units (size, gmtdefs.measure_unit);
-					fprintf (fpo, "echo %g %g > pslegend_tmp.txt\necho %g %g >> pslegend_tmp.txt\n", x_off + off_ss-x, y0, x_off + off_ss+x, y0);
-					sprintf (cmd, "%s -S%s%s pslegend_tmp.txt", psxy, symbol, &size[i]);
-					if (txt_c[0] != '-') {
-						sprintf (sub, " -G%s", txt_c);
-						strcat (cmd, sub);
-					}
-					if (txt_d[0] != '-') {
-						sprintf (sub, " -W%s", txt_d);
-						strcat (cmd, sub);
-					}
-					fprintf (fpo, "%s\n", cmd);
-					fprintf (fpo, "%s pslegend_tmp.txt\n", del);
+					size[i] = '/';	/* Undo truncation */
+					i++;
+					fprintf (fpo, "echo %g %g > %s\necho %g %g >> %s\n", x_off + off_ss-x, y0, tmptxt, x_off + off_ss+x, y0, tmptxt);
+					fprintf (fpo, "%s -S%s%s %s", psxy, symbol, &size[i], tmptxt);
+					if (txt_c[0] != '-') fprintf (fpo, " -G%s", txt_c);
+					if (txt_d[0] != '-') fprintf (fpo, " -W%s", txt_d);
+					fprintf (fpo, "\n%s %s\n", del, tmptxt);
 				}
 				else {	/* Regular symbols */
 					if (symbol[0] == 'k')
 						sprintf (sub, "%s/%s", symbol, size);
-				/*	else if (symbol[0] == 'l')
-						sprintf (sub, "%s", symbol); */
 					else
 						sprintf (sub, "%s%s", symbol, size);
 					if (symbol[0] == 'E' || symbol[0] == 'e') {	/* Ellipse needs more arguments */
@@ -482,12 +483,14 @@ int main (int argc, char **argv)
 					else if (symbol[0] == 'V' || symbol[0] == 'v') {	/* Vector also need more args */
 						i = 0;
 						while (size[i] != '/' && size[i]) i++;
-						if (size[i] != '/') {
+						if (size[i] != '/') {	/* The necessary arguments not supplied! */
 							sprintf (sub, "vb");
 							exit (EXIT_FAILURE);
 						}
-						else
-							sprintf (sub, "%sb%s", symbol, &size[++i]);
+						else {
+							size[i++] = '\0';	/* So GMT_convert_units won'c complain */
+							sprintf (sub, "%sb%s", symbol, &size[i]);
+						}
 						x = GMT_convert_units (size, gmtdefs.measure_unit);
 						sprintf (sarg, "%g %g 0 %g", x_off + off_ss, y0, x);
 					}
@@ -497,35 +500,29 @@ int main (int argc, char **argv)
 					}
 					else
 						sprintf (sarg, "%g %g", x_off + off_ss, y0);
-					sprintf (cmd, "echo %s | %s -S%s", sarg, psxy, sub);
-					if (txt_c[0] != '-') {
-						sprintf (sub, " -G%s", txt_c);
-						strcat (cmd, sub);
-					}
-					if (txt_d[0] != '-') {
-						sprintf (sub, " -W%s", txt_d);
-						strcat (cmd, sub);
-					}
-					fprintf (fpo, "%s\n", cmd);
+					fprintf (fpo, "echo %s | %s -S%s", sarg, psxy, sub);
+					if (txt_c[0] != '-') fprintf (fpo, " -G%s", txt_c);
+					if (txt_d[0] != '-') fprintf (fpo, " -W%s", txt_d);
+					fprintf (fpo, "\n");
 				}
 				fprintf (fpo, "echo %c%g %g %g 0 %d LM %s%c | %s\n", quote, x_off + off_tt, y0, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, quote, pstext);
 				if (n_columns == 1 || symbol_number%n_columns == 0) y0 += half_line_spacing;
 				break;
 
 			case 'D':	/* Delimiter record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s %s", txt_a, txt_b);
 				L = GMT_convert_units (txt_a, GMT_INCH);
 				y0 += quarter_line_spacing;
-				fprintf (fpo, "echo %g %g > pslegend_tmp.txt\necho %g %g >> pslegend_tmp.txt\n", L, y0, Ctrl->D.width - L, y0);
-				fprintf (fpo, "%s -W%s pslegend_tmp.txt\n%s pslegend_tmp.txt\n", psxy, txt_b, del);
+				fprintf (fpo, "echo %g %g > %s\necho %g %g >> %s\n", L, y0, tmptxt, Ctrl->D.width - L, y0, tmptxt);
+				fprintf (fpo, "%s -W%s %s\n%s %s\n", psxy, txt_b, tmptxt, del, tmptxt);
 				y0 += quarter_line_spacing;
 				break;
 
 			case 'G':	/* Gap record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s", txt_a);
@@ -533,7 +530,7 @@ int main (int argc, char **argv)
 				y0 += L;
 				break;
 			case 'N':	/* n_columns record */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				sscanf (&line[2], "%s", txt_a);
@@ -542,17 +539,17 @@ int main (int argc, char **argv)
 				break;
 
 			case 'V':	/* Vertical line from here to next V */
-				if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+				if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 				flush_paragraph = FALSE;
 				if (n_columns > 1 && symbol_number > 0 && symbol_number%n_columns != 0) y0 += half_line_spacing, symbol_number = 0;	/* Did not end with full column */
 				if (draw_vertical_line) {	/* Second time, now draw line */
-					fprintf (fpo, "echo # vertical lines > pslegend_tmp.txt\n");
+					fprintf (fpo, "echo # vertical lines > %s\n", tmptxt);
 					for (i = 1; i < n_columns; i++) {
 						x_off = i * Ctrl->D.width / n_columns;
-						fprintf (fpo, "echo %s> bar %d >> pslegend_tmp.txt\n", escape, i);
-						fprintf (fpo, "echo %g %g >> pslegend_tmp.txt\necho %g %g >> pslegend_tmp.txt\n", x_off, y_start+V-quarter_line_spacing, x_off, y0-V+quarter_line_spacing);
+						fprintf (fpo, "echo %s> bar %d >> %s\n", escape, i, tmptxt);
+						fprintf (fpo, "echo %g %g >> %s\necho %g %g >> %s\n", x_off, y_start+V-quarter_line_spacing, tmptxt, x_off, y0-V+quarter_line_spacing, tmptxt);
 					}
-					fprintf (fpo, "%s -W%s -H -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", psxy, vpen, del);
+					fprintf (fpo, "%s -W%s -H -M %s\n%s %s\n", psxy, vpen, tmptxt, del, tmptxt);
 					draw_vertical_line = FALSE;
 				}
 				else {
@@ -579,13 +576,13 @@ int main (int argc, char **argv)
 				if (n == 0 || lspace[0] == '-') sprintf (lspace, "%gi", one_line_spacing);
 				if (n == 0 || tw[0] == '-') sprintf (tw, "%gi", Ctrl->D.width - 2.0 * Ctrl->C.dx);
 				if (n == 0 || jj[0] == '-') sprintf (jj, "j");
-				fprintf (fpo, "echo %s> %s %s %s %s %s %s %s %s %s > pslegend_tmp.txt\n", escape, xx, yy, size, angle, font, key, lspace, tw, jj);
+				fprintf (fpo, "echo %s> %s %s %s %s %s %s %s %s %s > %s\n", escape, xx, yy, size, angle, font, key, lspace, tw, jj, tmptxt);
 				flush_paragraph = TRUE;
 				break;
 
 			case 'T':	/* paragraph text record */
 				sscanf (&line[2], "%[^\n]", text);
-				fprintf (fpo, "echo %c%s%c >> pslegend_tmp.txt\n", quote, text, quote);
+				fprintf (fpo, "echo %c%s%c >> %s\n", quote, text, quote, tmptxt);
 				break;
 
 			default:
@@ -596,21 +593,21 @@ int main (int argc, char **argv)
 	}
 	if (fp != GMT_stdin) GMT_fclose (fp);
 
-	if (flush_paragraph) fprintf (fpo, "%s -M pslegend_tmp.txt\n%s pslegend_tmp.txt\n", pstext, del);
+	if (flush_paragraph) fprintf (fpo, "%s -M %s\n%s %s\n", pstext, tmptxt, del, tmptxt);
 	/* Revert to original region (-R), projection and size (-J) and position (-X, -Y) */
-	sprintf (cmd, "psxy %s -R%g/%g/%g/%g %s -X%c%gi -Y%c%gi -O %s", null, west, east, south, north, jarg, mode, -Ctrl->D.lon+GMT_ps.x_origin, mode, -Ctrl->D.lat+GMT_ps.y_origin, sparg);
-	if (!GMT_ps.last_page) strcat (cmd, " -K");
-	fprintf (fpo, "%s\n", cmd);
+	fprintf (fpo, "psxy %s -R%g/%g/%g/%g %s -X%c%gi -Y%c%gi -O %s", null, west, east, south, north, jarg, mode, -Ctrl->D.lon+GMT_ps.x_origin, mode, -Ctrl->D.lat+GMT_ps.y_origin, sparg);
+	if (!GMT_ps.last_page) fprintf (fpo, " -K");
+	fprintf (fpo, "\n");
 
 	if (!Ctrl->S.active) {	/* Add auto-delete command at the end of the script and then execute it */
 		fprintf (fpo, "%s %s\n", del, script);
 		if (fpo != stdout) fclose (fpo);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Executing legend script\n", GMT_program);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Executing and removing legend script\n", GMT_program);
 #ifdef WIN32
 		system (script);
 #else
-		sprintf (cmd, "sh %s", script);
-		system (cmd);
+		sprintf (sub, "sh %s", script);
+		system (sub);
 #endif
 	}
 	else if (fpo != stdout)
@@ -625,49 +622,10 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void GMT_replace_symbol (char *symbol)
-{
-	char new[BUFSIZ];
-
-	switch (symbol[0]) {
-		case 'a':	/* Star */
-			sprintf (new, "kstar");
-			break;
-		case 'c':	/* Circle */
-			sprintf (new, "kcircle");
-			break;
-		case 'd':	/* Diamond */
-			sprintf (new, "kdiamond");
-			break;
-		case 'g':	/* Octagon */
-			sprintf (new, "koctagon");
-			break;
-		case 'h':	/* Hexagon */
-			sprintf (new, "khexagon");
-			break;
-		case 'i':	/* Inverted triangle */
-			sprintf (new, "kinvtriangle");
-			break;
-		case 'n':	/* Pentagon */
-			sprintf (new, "kpentagon");
-			break;
-		case 's':	/* Square */
-			sprintf (new, "ksquare");
-			break;
-		case 't':	/* Triangle */
-			sprintf (new, "ktriangle");
-			break;
-		default:	/* None of the above */
-			strcpy (new, symbol);
-			break;
-	}
-	strcpy (symbol, new);
-}
-
 void *New_Pslegend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSLEGEND_CTRL *C;
 	
-	C = (struct PSLEGEND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSLEGEND_CTRL), "New_Pslegend_Ctrl");
+	C = (struct PSLEGEND_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSLEGEND_CTRL), "New_Pslegend_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -678,6 +636,6 @@ void *New_Pslegend_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Pslegend_Ctrl (struct PSLEGEND_CTRL *C) {	/* Deallocate control structure */
-	if (C->S.file) GMT_free ((void *)C->S.file);	
+	if (C->S.file) free ((void *)C->S.file);	
 	GMT_free ((void *)C);	
 }
