@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.53 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psmask.c,v 1.61 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -40,8 +40,6 @@
 #include "gmt.h"
 #include "pslib.h"
 
-#define PSMASK_IJ(i,j) ((i) + (j) * h->nx)
-
 struct PSMASK_CTRL {
 	struct C {	/* -C */
 		BOOLEAN active;
@@ -86,12 +84,15 @@ struct PSMASK_INFO {
 
 int main (int argc, char **argv)
 {
-	int i, j, ij, n, nm, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
-	int section, n_alloc, n_read, n_fields, one_or_zero, distance_flag = 0;
+	GMT_LONG ij, n, nm;
+	
+	int i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
+	int section, n_fields, distance_flag = 0;
 
-	BOOLEAN error = FALSE, first = TRUE, use_poly, node_only;
+	GMT_LONG n_read, n_alloc;
+	BOOLEAN error = FALSE, first = TRUE, node_only;
 
-	char dfile[BUFSIZ], line[BUFSIZ], *grd;
+	char line[BUFSIZ], *grd;
 
 	double *in, distance, x0, y0, x1, y1, shrink = 1.0;
 	double xinc2, yinc2, *x, *y;
@@ -102,10 +103,10 @@ int main (int argc, char **argv)
 	struct PSMASK_INFO info;
 	struct PSMASK_CTRL *Ctrl;
 
-	void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag);
-	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int nn, int id, char *file);
-	void shrink_clip_contours (double *x, double *y, int n, double w, double e);
-	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
+	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag);
+	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file);
+	void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e);
+	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max);
 	void *New_Psmask_Ctrl (), Free_Psmask_Ctrl (struct PSMASK_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -152,7 +153,7 @@ int main (int argc, char **argv)
 					break;
 				case 'D':	/* Dump the polygons to files */
 					Ctrl->D.active = TRUE;
-					GMT_free ((void *)Ctrl->D.file);
+					free ((void *)Ctrl->D.file);
 					Ctrl->D.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
@@ -292,7 +293,6 @@ int main (int argc, char **argv)
 		h.xy_off = 0.5 * h.node_offset;
 		xinc2 = 0.5 * h.x_inc;
 		yinc2 = 0.5 * h.y_inc;
-		one_or_zero = 1 - h.node_offset;
 		GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
 	}
 
@@ -375,7 +375,7 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 			n_read++;
@@ -414,7 +414,7 @@ int main (int argc, char **argv)
 		}
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d data points\n", GMT_program, n_read);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld data points\n", GMT_program, n_read);
 
 		if (Ctrl->N.active) for (i = 0; i < nm; i++) grd[i] = 1 - grd[i];	/* Reverse sense of test */
 
@@ -440,32 +440,31 @@ int main (int argc, char **argv)
 			first = TRUE;
 			while ((n = clip_contours (&info, grd, &h, xinc2, yinc2, edge, first, &x, &y, &n_alloc)) > 0) {
 				shrink_clip_contours (x, y, n, h.x_min, h.x_max);
-				if (Ctrl->D.active) dump_clip_contours (&info, x, y, n, section, dfile);
+				if (Ctrl->D.active) dump_clip_contours (&info, x, y, n, section, Ctrl->D.file);
 				draw_clip_contours (x, y, n, Ctrl->G.fill.rgb, section, first);
 				first = FALSE;
 				section++;
 			}
 
-			draw_clip_contours (x, y, 0, Ctrl->G.fill.rgb, section, 2);	/* Activate clip-path */
+			draw_clip_contours (x, y, (GMT_LONG)0, Ctrl->G.fill.rgb, section, 2);	/* Activate clip-path */
 
 			GMT_free ((void *)edge);
 			GMT_free ((void *)x);
 			GMT_free ((void *)y);
 		}
 		else {	/* Just paint tiles */
-			int start, plot_n, n_use, np;
+			int start;
+			GMT_LONG n_use, np, plot_n;
 			double *grd_x, grd_y, y_bot, y_top, *xx, *yy, *xp, *yp;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling...\n", GMT_program);
 			grd_x = (double *) GMT_memory (VNULL, (size_t)h.nx, sizeof (double), GMT_program);
 			for (i = 0; i < h.nx; i++) grd_x[i] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
 
-			use_poly = (Ctrl->G.fill.use_pattern || project_info.three_D);
-
 			for (j = 0; j < h.ny; j++) {
 				grd_y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 				y_bot = grd_y - yinc2;
 				y_top = grd_y + yinc2;
-				ij = j * h.nx + 1;
+				ij = GMT_IJ (j, 1, h.nx);
 				for (i = 0; i < h.nx; i++, ij++) {
 					if (((int)grd[ij]) == 0) continue;
 
@@ -524,9 +523,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag)
+void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag)
 {
-	int i;
+	GMT_LONG i;
 	double x, y;
 	char comment[GMT_TEXT_LEN];
 
@@ -551,9 +550,9 @@ void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int
 	}
 }
 
-void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int nn, int id, char *file)
+void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file)
 {
-	int i;
+	GMT_LONG i;
 	double out[2];
 	char fname[BUFSIZ];
 	FILE *fp;
@@ -589,7 +588,7 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, int n
 	GMT_fclose (fp);
 }
 
-int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max)
+int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max)
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
@@ -597,9 +596,10 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 	 */
 	 
 	static int i0, j0, side;
-	int i, j, ij, n = 0, n_edges, edge_word, edge_bit;
+	GMT_LONG ij;
+	int i, j, n = 0, n_edges, edge_word, edge_bit;
 	BOOLEAN go_on = TRUE;
-	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
+	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max);
 	 
 	 
 	n_edges = h->ny * (int) ceil (h->nx / 16.0);
@@ -621,7 +621,7 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 
 	if (side == 4) {
 		for (j = j0; go_on && j < h->ny; j++) {
-			ij = i0 + j * h->nx;
+			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
 				edge_word = ij / 32 + info->offset;
 				edge_bit = ij % 32;
@@ -645,7 +645,7 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 	}
 	if (n == 0 && side == 5) {
 		for (j = j0; go_on && j < h->ny; j++) {
-			ij = i0 + j * h->nx;
+			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {
 				edge_word = ij / 32 + info->offset;
 				edge_bit = ij % 32;
@@ -666,17 +666,18 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 	return (n);
 }
 
-int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max)
+int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max)
 {
-	int n = 1, k, k0, ij, ij0, n_cuts, kk_opposite, first_k, more;
+	int n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
 	int edge_word, edge_bit, m;
+	GMT_LONG ij, ij0;
 	double xk[4], yk[4], x0, y0;
 
 	m = *max - 2;
 	
 	more = TRUE;
 	do {
-		ij = i + j * h->nx;
+		ij = GMT_IJ (j, i, h->nx);
 		x0 = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 		y0 = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 		n_cuts = 0;
@@ -690,7 +691,7 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 
 			/* Skip edge already has been used */
 
-			ij0 = PSMASK_IJ (i + info->i_off[k], j + info->j_off[k]);
+			ij0 = GMT_IJ (j + info->j_off[k], i + info->i_off[k], h->nx);
 			edge_word = ij0 / 32 + info->k_off[k] * info->offset;
 			edge_bit = ij0 % 32;
 			if (edge[edge_word] & info->bit[edge_bit]) continue;
@@ -726,8 +727,8 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 		}
 
 		if (n > m) {	/* Must try to allocate more memory */
-			*max += GMT_CHUNK;
-			m += GMT_CHUNK;
+			*max = (*max == 0) ? GMT_CHUNK : ((*max) << 1);
+			m = (m == 0) ? GMT_CHUNK : (m << 1);
 			*xx = (double *) GMT_memory ((void *)*xx, (size_t)(*max), sizeof (double), "trace_clip_contours");
 			*yy = (double *) GMT_memory ((void *)*yy, (size_t)(*max), sizeof (double), "trace_clip_contours");
 		}
@@ -770,7 +771,7 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 			n++;
 		}
 		if (more) {	/* Mark this edge as used */
-			ij0 = PSMASK_IJ (i + info->i_off[kk], j + info->j_off[kk]);
+			ij0 = GMT_IJ (j + info->j_off[kk], i + info->i_off[kk], h->nx);
 			edge_word = ij0 / 32 + info->k_off[kk] * info->offset;
 			edge_bit = ij0 % 32;
 			edge[edge_word] |= info->bit[edge_bit];
@@ -786,10 +787,10 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 	return (n);
 }
 
-void shrink_clip_contours (double *x, double *y, int n, double w, double e)
+void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e)
 {
 	/* Moves outside points to boundary */
-	int i;
+	GMT_LONG i;
 
 	for (i = 0; i < n; i++) {
 		if (x[i] < w)
@@ -806,7 +807,7 @@ void shrink_clip_contours (double *x, double *y, int n, double w, double e)
 void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSMASK_CTRL *C;
 	
-	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
+	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -817,5 +818,6 @@ void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 }
 
 void Free_Psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.file) free ((void *)C->D.file);	
 	GMT_free ((void *)C);	
 }
