@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.54 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psrose.c,v 1.67 2008/05/13 12:31:31 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -52,6 +52,9 @@ struct PSROSE_CTRL {	/* All control options for this program (except common args
 		BOOLEAN active;
 		double azimuth, elevation;
 	} E;
+	struct F {	/* -F */
+		BOOLEAN active;
+	} F;
 	struct G {	/* -G<fill> */
 		BOOLEAN active;
 		struct GMT_FILL fill;
@@ -59,6 +62,13 @@ struct PSROSE_CTRL {	/* All control options for this program (except common args
 	struct I {	/* -I */
 		BOOLEAN active;
 	} I;
+	struct L {	/* -L */
+		BOOLEAN active;
+		char w[GMT_LONG_TEXT];
+		char e[GMT_LONG_TEXT];
+		char s[GMT_LONG_TEXT];
+		char n[GMT_LONG_TEXT];
+	} L;
 	struct M {	/* -M<params> */
 		BOOLEAN active;
 		double width, length, thickness;
@@ -90,24 +100,26 @@ int main (int argc, char **argv)
 	BOOLEAN error = FALSE, find_mean = FALSE, half_only = FALSE, windrose = TRUE;
 	BOOLEAN automatic = FALSE, sector_plot = FALSE;
 
-	char text[BUFSIZ], lw[10], le[10], ln[10], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"};
+	char text[BUFSIZ], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"};
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT];
 
-	int i, n_bins, bin, n_annot, n = 0, n_alloc = GMT_CHUNK;
+	int n_bins, bin, n_annot;
 	int status, n_alpha, n_modes, n_fields, n_expected_fields;
+	
+	GMT_LONG i, n = 0, n_alloc = GMT_CHUNK;
 
 	double max = 0.0, radius, az, x_origin, y_origin, tmp, one_or_two = 1.0, s, c, half_bin_width;
 	double angle1, angle2, angle, x, y, mean_theta, mean_radius, xr = 0.0, yr = 0.0;
 	double x1, x2, y1, y2, total = 0.0, total_arc, off, max_radius, az_offset, *in;
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, asize, lsize, this_az;
-	double *xx, *yy, *sum, *azimuth, *length, *mode_direction, *mode_length;
+	double *xx, *yy, *sum, *azimuth, *length, *mode_direction, *mode_length, dim[3];
 
 	FILE *fp = NULL, *fpm = NULL;
 
 	struct PSROSE_CTRL *Ctrl;
+	struct GMT_FILL f;
 
 	void plot3d (double x, double y, int pen);
-	void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOOLEAN outline);
 	void arc3d (double x, double y, double r, double a1, double a2, int status);
 	void *New_Psrose_Ctrl (), Free_Psrose_Ctrl (struct PSROSE_CTRL *C);
 
@@ -117,7 +129,6 @@ int main (int argc, char **argv)
 
 	asize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 	lsize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
-	strcpy (lw, "WEST");	strcpy (le, "EAST");	strcpy (ln, "NORTH");
 	mode_direction = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
 	mode_length = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
 
@@ -167,6 +178,9 @@ int main (int argc, char **argv)
 					Ctrl->E.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
+				case 'F':		/* Disable scalebar plotting */
+					Ctrl->F.active = TRUE;
+					break;
 				case 'G':		/* Set Gray shade */
 					Ctrl->G.active = TRUE;
 					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
@@ -177,6 +191,19 @@ int main (int argc, char **argv)
 				case 'I':		/* Compute statistics only - no plot */
 					Ctrl->I.active = TRUE;
 					break;
+				case 'L':		/* Overwride default labeling */
+					Ctrl->L.active = TRUE;
+					if (argv[i][2]) {
+						n = sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%s", Ctrl->L.w, Ctrl->L.e, Ctrl->L.s, Ctrl->L.n);
+						if (n != 4) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -L option.  Correct syntax:\n", GMT_program);
+							fprintf (stderr, "\t-L<westlabel/eastlabel/southlabel/<northlabel>>\n");
+							error = TRUE;
+						}
+					}
+					else
+						Ctrl->L.w[0] = Ctrl->L.e[0] = Ctrl->L.s[0] = Ctrl->L.n[0] = '\0';
+					break;
 				case 'M':		/* Get arrow parameters */
 					Ctrl->M.active = TRUE;
 					if (argv[i][2]) {
@@ -232,8 +259,8 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"psrose %s - Polar histogram (rose diagram) plotter\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psrose <infile> [-A<sector_angle>[r]] [%s]\n", GMT_B_OPT);
-		fprintf (stderr, "\t[-C[<modes>]] [-D] [-E<azim>/<elev>] [-G<fill>] [%s] [-I] [-K] [-M<parameters>]\n", GMT_H_OPT);
-		fprintf (stderr, "\t[-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-S<scale>[n]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_t_OPT);
+		fprintf (stderr, "\t[-C[<modes>]] [-D] [-E<azim>/<elev>] [-G<fill>] [%s] [-I] [-K] [-L[<wlab/elab/slab/nlab>]]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-M<parameters>] [-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-S<scale>[n]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_t_OPT);
 		fprintf (stderr, "\t[-W<pen>] [%s] [%s] [-Z<scale>] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_bi_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -247,10 +274,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-C plot vectors listed in the <modes> file.  If no file, use mean direction\n");
 		fprintf (stderr, "\t-D will center the sectors\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		GMT_fill_syntax ('G', "Specifies color for diagram [Default is no fill]");
+		fprintf (stderr, "\t-F Do not draw the scale length bar [Default plots scale in lower right corner]\n");
+		GMT_fill_syntax ('G', "Specifies color for diagram [Default is no fill].");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I for inquire.  Only compute statistics - no plot is created\n");
 		GMT_explain_option ('K');
+		fprintf (stderr, "\t-L Override default labels [Default is WEST/EAST/SOUTH/NORTH for full circle and 90W/90E/-/0 for half-circle]\n");
+		fprintf (stderr, "\t   If no argument is given then labels will be disabled.  Give - to disable an individual label\n");
 		fprintf (stderr, "\t-M Append tailwidth/headlength/headwidth/r/g/b to set arrow attributes [0.03i/0.12i/0.1i/0/0/0\n");
 		fprintf (stderr, "\t-N normalizes rose plots for area, i.e. takes sqrt(r) before plotting [FALSE]\n");
 		fprintf (stderr, "\t   Only applicable if normalization has been specified with -S<radius>n\n");
@@ -285,7 +315,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  radius must be nonzero\n", GMT_program);
 		error++;
 	}
-	if (fabs (Ctrl->Z.scale) < GMT_CONV_LIMIT) {
+	if (GMT_IS_ZERO (Ctrl->Z.scale)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Z option:  factor must be nonzero\n", GMT_program);
 		error++;
 	}
@@ -338,7 +368,7 @@ int main (int argc, char **argv)
 	}
 
 	max_radius = east;
-	half_only = (fabs (south + 90.0) < GMT_CONV_LIMIT);
+	half_only = GMT_IS_ZERO (south + 90.0);
 	if (Ctrl->A.rose) windrose = FALSE;
 	sector_plot = (Ctrl->A.inc > 0.0);
 	if (sector_plot) windrose = FALSE;	/* Draw rose diagram instead of sector diagram */
@@ -374,7 +404,7 @@ int main (int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
@@ -403,7 +433,7 @@ int main (int argc, char **argv)
 
 		n++;
 		if (n == n_alloc) {	/* Get more memory */
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			azimuth = (double *) GMT_memory ((void *)azimuth, (size_t)n_alloc, sizeof (double), GMT_program);
 			length = (double *) GMT_memory ((void *)length, (size_t)n_alloc, sizeof (double), GMT_program);
 		}
@@ -446,7 +476,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->I.active || gmtdefs.verbose) {
 		if (file) strcpy (text, file); else strcpy (text, "<stdin>");
-		sprintf (format, "%%s: Info for %%s: n = %%d rmax = %s mean r/az = (%s/%s) totlength = %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "%%s: Info for %%s: n = %%ld rmax = %s mean r/az = (%s/%s) totlength = %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, text, n, max, mean_radius, mean_theta, total);
 		if (Ctrl->I.active) {
 			GMT_free ((void *)sum);
@@ -459,20 +489,20 @@ int main (int argc, char **argv)
 	}
 
 	if (automatic) {
-		if (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) {
+		if (GMT_IS_ZERO (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval)) {
 			tmp = pow (10.0, floor (d_log10 (max)));
 			if ((max / tmp) < 3.0) tmp *= 0.5;
 		}
 		else
 			tmp = frame_info.axis[0].item[GMT_ANNOT_UPPER].interval;
 		max_radius = ceil (max / tmp) * tmp;
-		if (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT || frame_info.axis[0].item[GMT_GRID_UPPER].interval < GMT_CONV_LIMIT) {	/* Tickmarks not set */
+		if (GMT_IS_ZERO (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval) || GMT_IS_ZERO (frame_info.axis[0].item[GMT_GRID_UPPER].interval)) {	/* Tickmarks not set */
 			frame_info.axis[0].item[GMT_ANNOT_UPPER].interval = frame_info.axis[0].item[GMT_GRID_UPPER].interval = tmp;
 			frame_info.plot = TRUE;
 		}
 	}
 
-	if (frame_info.plot && frame_info.axis[1].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) frame_info.axis[1].item[GMT_ANNOT_UPPER].interval = frame_info.axis[1].item[GMT_GRID_UPPER].interval = 30.0;
+	if (frame_info.plot && GMT_IS_ZERO (frame_info.axis[1].item[GMT_ANNOT_UPPER].interval)) frame_info.axis[1].item[GMT_ANNOT_UPPER].interval = frame_info.axis[1].item[GMT_GRID_UPPER].interval = 30.0;
 
 	/* Ready to plot */
 
@@ -504,9 +534,10 @@ int main (int argc, char **argv)
 
 		for (bin = 0; bin < n_bins; bin++) {
 			az = bin * Ctrl->A.inc - az_offset + half_bin_width;
-			angle1 = (90.0 - az - Ctrl->A.inc);
-			angle2 = angle1 + Ctrl->A.inc;
-			pie3d (0.0, 0.0, sum[bin] * Ctrl->S.scale, angle1, angle2, Ctrl->G.fill.rgb, FALSE);
+			dim[1] = (90.0 - az - Ctrl->A.inc);
+			dim[2] = dim[1] + Ctrl->A.inc;
+			dim[0] = sum[bin] * Ctrl->S.scale;
+			GMT_pie (0.0, 0.0, project_info.z_level, dim, &(Ctrl->G.fill), FALSE);
 		}
 	}
 	else if (Ctrl->A.rose) {	/* Draw rose diagram */
@@ -560,10 +591,11 @@ int main (int argc, char **argv)
 			n_modes = 0;
 			n_alloc = 1;
 			while (GMT_fgets (text, BUFSIZ, fpm)) {
+				if (GMT_is_a_blank_line (text)) continue;	/* Skip blank lines or # comments */
 				sscanf (text, "%lf %lf", &mode_direction[n_modes], &mode_length[n_modes]);
 				n_modes++;
 				if (n_modes == n_alloc) {
-					n_alloc += GMT_SMALL_CHUNK;
+					n_alloc <<= 1;
 					mode_direction = (double *) GMT_memory ((void *)mode_direction, (size_t)n_alloc, sizeof (double), "psrose");
 					mode_length = (double *) GMT_memory ((void *)mode_length, (size_t)n_alloc, sizeof (double), "psrose");
 				}
@@ -579,11 +611,21 @@ int main (int argc, char **argv)
 			sincosd (angle, &s, &c);
 			xr = Ctrl->S.scale * mode_length[i] * c;
 			yr = Ctrl->S.scale * mode_length[i] * s;
-			GMT_vector3D (0.0, 0.0, xr, yr, project_info.z_level, Ctrl->M.thickness, Ctrl->M.length, Ctrl->M.width, gmtdefs.vector_shape, Ctrl->M.rgb, TRUE);
+			GMT_init_fill (&f, Ctrl->M.rgb[0], Ctrl->M.rgb[1], Ctrl->M.rgb[2]);       /* Initialize fill structure */
+			GMT_vector (0.0, 0.0, xr, yr, project_info.z_level, Ctrl->M.thickness, Ctrl->M.length, Ctrl->M.width, gmtdefs.vector_shape, &f, TRUE);
 		}
 
 	}
 
+	if (Ctrl->L.active) {	/* Deactivate those with - */
+		if (Ctrl->L.w[0] == '-' && Ctrl->L.w[1] == '\0') Ctrl->L.w[0] = '\0';
+		if (Ctrl->L.e[0] == '-' && Ctrl->L.e[1] == '\0') Ctrl->L.e[0] = '\0';
+		if (Ctrl->L.s[0] == '-' && Ctrl->L.s[1] == '\0') Ctrl->L.s[0] = '\0';
+		if (Ctrl->L.n[0] == '-' && Ctrl->L.n[1] == '\0') Ctrl->L.n[0] = '\0';
+	}
+	else {	/* Use default labels */
+		strcpy (Ctrl->L.w, "WEST");	strcpy (Ctrl->L.e, "EAST");	strcpy (Ctrl->L.s, "SOUTH");	strcpy (Ctrl->L.n, "NORTH");
+	}
 	if (frame_info.plot) {	/* Draw grid lines etc */
 		GMT_setpen (&gmtdefs.grid_pen[0]);
 		off = max_radius * Ctrl->S.scale;
@@ -610,15 +652,17 @@ int main (int argc, char **argv)
 			int which, no_degree;
 			which = (gmtdefs.degree_format >= 100) ? gmt_ring : gmt_degree;
 			no_degree = (gmtdefs.degree_format >= 1000);	/* No, we don't want the degree symbol at all */
-			if (no_degree) {
-				sprintf (lw, "90W");
-				sprintf (le, "90E");
-				sprintf (ln, "0");
-			}
-			else {
-				sprintf (lw, "90%cW", gmtdefs.encoding.code[which]);
-				sprintf (le, "90%cE", gmtdefs.encoding.code[which]);
-				sprintf (ln, "0%c", gmtdefs.encoding.code[which]);
+			if (!Ctrl->L.active) {	/* Use default labels */
+				if (no_degree) {
+					sprintf (Ctrl->L.w, "90W");
+					sprintf (Ctrl->L.e, "90E");
+					sprintf (Ctrl->L.n, "0");
+				}
+				else {
+					sprintf (Ctrl->L.w, "90%cW", (int)gmtdefs.encoding.code[which]);
+					sprintf (Ctrl->L.e, "90%cE", (int)gmtdefs.encoding.code[which]);
+					sprintf (Ctrl->L.n, "0%c",   (int)gmtdefs.encoding.code[which]);
+				}
 			}
 			y = -(3.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * asize);
 			if (frame_info.axis[0].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[0].label, 0.0, 10, 0);
@@ -635,26 +679,28 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			GMT_text3D (0.0, -off - 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, "SOUTH", 0.0, 10, 0);
-			plot3d (off, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 2);
-			plot3d (off, -off, 3);
-			plot3d (off, gmtdefs.tick_length - off, 2);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, gmtdefs.tick_length - off, -2);
+			GMT_text3D (0.0, -off - 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.s, 0.0, 10, 0);
+			if (!Ctrl->F.active) {	/* Draw scale bar */
+				plot3d (off, -off, 3);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 2);
+				plot3d (off, -off, 3);
+				plot3d (off, gmtdefs.tick_length - off, 2);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 3);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, gmtdefs.tick_length - off, -2);
+				if (frame_info.axis[0].label[0]) {
+					strcat (format, " %s");
+					sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval, frame_info.axis[0].label);
+				}
+				else
+					sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
+				GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
+			}
 			y = -(off + 5.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * lsize + GMT_font[gmtdefs.label_font].height * lsize);
 			if (frame_info.axis[1].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[1].label, 0.0, 10, 0);
-			if (frame_info.axis[0].label[0]) {
-				strcat (format, " %s");
-				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval, frame_info.axis[0].label);
-			}
-			else
-				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
-			GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 		}
-		GMT_text3D (off + 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, le, 0.0, 5, 0);
-		GMT_text3D (-off - 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, lw, 0.0, 7, 0);
-		GMT_text3D (0.0, off + 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, ln, 0.0, 2, 0);
+		GMT_text3D (off + 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.e, 0.0, 5, 0);
+		GMT_text3D (-off - 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.w, 0.0, 7, 0);
+		GMT_text3D (0.0, off + 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.n, 0.0, 2, 0);
 		ps_setpaint (gmtdefs.background_rgb);
 	}
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin + y_origin, 0.0);
@@ -667,6 +713,8 @@ int main (int argc, char **argv)
 	GMT_free ((void *)yy);
 	GMT_free ((void *)azimuth);
 	GMT_free ((void *)length);
+	GMT_free ((void *)mode_length);
+	GMT_free ((void *)mode_direction);
 
 	Free_Psrose_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -677,27 +725,13 @@ int main (int argc, char **argv)
 
 void arc3d (double x, double y, double r, double a1, double a2, int status)
 {
-	int i, n;
-	double da, a, *xx, *yy, s, c;
-
 	if (project_info.three_D) {	/* Must project and draw line segments */
-		n = (int)ceil (a2 - a1);
+		int n;
+		double *xx, *yy;
 
-		xx = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
-		yy = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
-
-		a1 *= D2R;
-		a2 *= D2R;
-		da = (a2 - a1) / n;
-		for (i = 0; i <= n; i++) {
-			a = (i == n) ? a2 : a1 + da * i;
-			sincos (a, &s, &c);
-			xx[i] = x + r * c;
-			yy[i] = y + r * s;
-		}
-		n++;
-		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
-		ps_line (xx, yy, n, status, FALSE, TRUE);
+		n = GMT_get_arc (x, y, r, a1, a2, &xx, &yy);
+		GMT_2D_to_3D (xx, yy, project_info.z_level, (GMT_LONG)n);
+		ps_line (xx, yy, (GMT_LONG)n, status, FALSE, TRUE);
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
@@ -711,41 +745,10 @@ void plot3d (double x, double y, int pen)
 	ps_plot (x, y, pen);
 }
 
-void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOOLEAN outline)
-{
-	int i, n;
-	double da, a, *xx, *yy, s, c;
-
-	if (project_info.three_D) {	/* Must project and draw line segments */
-		n = (int)ceil (a2 - a1);
-
-		xx = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
-		yy = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
-
-		a1 *= D2R;
-		a2 *= D2R;
-		da = (a2 - a1) / n;
-		for (i = 0; i <= n; i++) {
-			a = (i == n) ? a2 : a1 + da * i;
-			sincos (a, &s, &c);
-			xx[i] = x + rad * c;
-			yy[i] = y + rad * s;
-		}
-		xx[i] = x;	yy[i] = y;
-		n += 2;
-		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
-		ps_polygon (xx, yy, n, rgb, outline);
-		GMT_free ((void *)xx);
-		GMT_free ((void *)yy);
-	}
-	else
-		ps_pie (x, y, rad, a1, a2, rgb, outline);
-}
-
 void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSROSE_CTRL *C;
 	
-	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
+	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -768,6 +771,6 @@ void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
 }
 
 void Free_Psrose_Ctrl (struct PSROSE_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
