@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psscale.c,v 1.76 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psscale.c,v 1.89 2008/04/15 15:44:44 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
  *		      10-NOV-2003. PW: Enhanced -E to set back or foreground only
  *		      08-MAR-2006. PW: Enhanced -I to set low/high intensity
  *		      23-MAY-2006. PW: Added -Q to handle logarithmic scales
+ *		      04-OCT-2007. PW: Can now handle -B1p for log scales
  * Version:	4
  *
  */
@@ -96,6 +97,7 @@ int main(int argc, char **argv)
 	int i, j, n;
 
 	double max_intens[2], dz, *z_width, x_origin = 0.0, y_origin = 0.0;
+	double start_val, stop_val;
 
 	FILE *fp = NULL;
 
@@ -237,13 +239,14 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   By default all color changes are annotated (but see -B).  To use a subset,\n");
 		fprintf (stderr, "\t   add an extra column to the cpt-file with a L, U, or B\n");
 		fprintf (stderr, "\t   to annotate Lower, Upper, or Both color segment boundaries\n");
+		fprintf (stderr, "\t   If a categorical CPT file is given the -Li is set automatically\n");
 		fprintf (stderr, "\t-E add sidebar triangles for back- and foreground colors.\n");
 		fprintf (stderr, "\t   Specify b(ackground) or f(oreground) to get one only [Default is both]\n");
 		fprintf (stderr, "\t   Optionally, append triangle height [Default scales with barwidth]\n");
 		fprintf (stderr, "\t-I add illumination for +-<max_intens> or <low_i> to <high_i> [-1.0/1.0]\n");
 		fprintf (stderr, "\t   Alternatively, specify <lower>/<upper> intensity values\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "\t-L For equal-sized color rectangles. -B interval, if set, is ignored\n");
+		fprintf (stderr, "\t-L For equal-sized color rectangles. -B interval cannot be used.\n");
 		fprintf (stderr, "\t   Append i to annotate the interval range instead of lower/upper.\n");
 		fprintf (stderr, "\t   If <gap> is appended, we separate each rectangle by <gap> units and center each\n");
 		fprintf (stderr, "\t   lower (z0) annotation on the rectangle.  Ignored if not a discrete cpt table.\n");
@@ -283,10 +286,13 @@ int main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option: dpi must be > 0\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->L.active && B_set) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option: Cannot be used with -B option.\n", GMT_program);
+		error++;
+	}
 	      
 	if (error) exit (EXIT_FAILURE);
 
-	if (Ctrl->L.active && B_set) B_set = FALSE;	/* -L overrides intervals set with -B */
 
 	if (gmtdefs.verbose) {
 		if (Ctrl->C.file)
@@ -296,6 +302,14 @@ int main(int argc, char **argv)
 	}
 
 	GMT_read_cpt (Ctrl->C.file);
+
+#ifdef GMT_CPT2	
+	if (GMT_categorical) {
+		Ctrl->L.active = Ctrl->L.interval = TRUE;
+		if (gmtdefs.verbose) fprintf (stderr, "%s: CPT is for categorical data.", GMT_program);
+	}
+#endif
+	
 	if (gmtdefs.verbose) fprintf (stderr, "  CPT range from %g to %g\n",
 		GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high);
 
@@ -354,9 +368,18 @@ int main(int argc, char **argv)
 	/* Because psscale uses -D to position things we need to make some
 	 * changes so that BoundingBox and others are set ~correctly */
 
-	sprintf (text, "X%gi/%gi", Ctrl->D.length, Ctrl->D.width);
+	if (Ctrl->Q.active && B_set) {
+		sprintf (text, "X%gil/%gi", Ctrl->D.length, Ctrl->D.width);
+		start_val = pow (10.0, GMT_lut[0].z_low);
+		stop_val  = pow (10.0, GMT_lut[GMT_n_colors-1].z_high);
+	}
+	else {
+		sprintf (text, "X%gi/%gi", Ctrl->D.length, Ctrl->D.width);
+		start_val = GMT_lut[0].z_low;
+		stop_val  = GMT_lut[GMT_n_colors-1].z_high;
+	}
 	GMT_parse_J_option (text);	/* Fake linear projection */
-	GMT_err_fail (GMT_map_setup (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, Ctrl->D.width), "");
+	GMT_err_fail (GMT_map_setup (start_val, stop_val, 0.0, Ctrl->D.width), "");
 
 	/* We must do any origin translation manually in psscale */
 
@@ -395,7 +418,7 @@ int main(int argc, char **argv)
 
 	if (GMT_ps.unix_time) {	/* Extra shift/unshift since psscale does things differently */
 		ps_transrotate (-GMT_ps.x_origin, -GMT_ps.y_origin, 0.0);
-		GMT_timestamp (GMT_ps.unix_time_pos[0], GMT_ps.unix_time_pos[1], GMT_ps.unix_time_label);
+		GMT_timestamp (GMT_ps.unix_time_pos[0], GMT_ps.unix_time_pos[1], GMT_ps.unix_time_just, GMT_ps.unix_time_label);
 		ps_transrotate (GMT_ps.x_origin, GMT_ps.y_origin, 0.0);
 	}
 
@@ -418,13 +441,13 @@ int main(int argc, char **argv)
 
 void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dpi, BOOLEAN flip_anot, BOOLEAN B_set, BOOLEAN equi, BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, int extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome)
 {
-	int i, ii, id, j, ndec = -1, dec, p_val, depth, rgb[3], rrggbb[3];
+	int i, ii, id, j, nb, ndec = -1, dec, p_val, depth, rgb[3], rrggbb[3];
 	int nx = 0, ny = 0, nm, barmem, k, justify, l_justify, this_just, use_labels = 0, Label_justify;
 	BOOLEAN reverse, all = TRUE, use_image, center = FALSE, const_width = TRUE, do_annot;
 	char format[GMT_LONG_TEXT], text[GMT_LONG_TEXT], test[GMT_LONG_TEXT], unit[GMT_LONG_TEXT];
 	unsigned char *bar = 0, *tmp = 0;
 	double off, annot_off, label_off, len, len2, size, x0, x1, dx, xx, dir, y_base;
-	double z, xleft, xright, inc_i, inc_j;
+	double z, xleft, xright, inc_i, inc_j, start_val, stop_val;
 	double get_z (double x, double *width, int n);
 	double xp[4], yp[4];
 	struct GMT_FILL *f;
@@ -436,7 +459,13 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 
 	/* Find max decimals needed */
 
+	start_val = GMT_lut[0].z_low;
+	stop_val  = GMT_lut[GMT_n_colors-1].z_high;
 	if (B_set) {
+		if (logscl) {	/* Must set original start/stop values for axis */
+			start_val = pow (10.0, GMT_lut[0].z_low);
+			stop_val  = pow (10.0, GMT_lut[GMT_n_colors-1].z_high);
+		}
 		ndec = GMT_get_format (frame_info.axis[0].item[0].interval, frame_info.axis[0].unit, frame_info.axis[0].prefix, format);
 	}
 	else {
@@ -560,17 +589,17 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				x1 += z_width[ii];
 				if ((f = GMT_lut[ii].fill)) {	/* Using pattern fills */
 					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
-					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, center, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+					GMT_fill (xp, yp, (GMT_LONG)4, f, center);	/* Contours drawn separately below if desired */
 				}
 				else if (intens) {
-					ii = (GMT_gray || monochrome) ? 1 : 3;
-					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*ii, sizeof (char), GMT_program);
-					for (j = 0, k = i*ii; j < ny*ii; k+=(nx-1)*ii) {
+					nb = (GMT_gray || monochrome) ? 1 : 3;
+					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*nb, sizeof (char), GMT_program);
+					for (j = 0, k = i*nb; j < ny*nb; k+=(nx-1)*nb) {
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 					}
-					GMT_color_image (x0 + gap, 0.0, z_width[i] - 2*gap, width, tmp, 1, ny, depth);
+					GMT_color_image (x0 + gap, 0.0, z_width[ii] - 2*gap, width, tmp, 1, ny, depth);
 					GMT_free ((void *)tmp);
 				}
 				else {
@@ -583,31 +612,31 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			}
 		}
 
-		if (extend & 1) {	/* Add color triangles for background color  */
+		if (extend & (reverse + 1)) {	/* Add color triangle on left side */
 			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_FGD : GMT_BGD;
 			if ((f = GMT_bfn[id].fill))
-				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, 1);
 		}
-		if (extend & 2) {	/* Add color triangles for foreground color  */
+		if (extend & (2 - reverse)) {	/* Add color triangle on right side */
 			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_BGD : GMT_FGD;
 			if ((f = GMT_bfn[id].fill))
-				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, 1);
 		}
 
 		if (gap == 0.0) {
@@ -618,9 +647,9 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 		}
 		if (B_set) {	/* Used -B */
 			if (flip_anot)
-				GMT_xy_axis (xleft, width, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[0], FALSE, frame_info.side[0]-1);
+				GMT_xy_axis (xleft, width, length, start_val, stop_val, &frame_info.axis[0], FALSE, frame_info.side[0]-1);
 			else
-				GMT_xy_axis (xleft, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[0], TRUE, frame_info.side[0]-1);
+				GMT_xy_axis (xleft, 0.0, length, start_val, stop_val, &frame_info.axis[0], TRUE, frame_info.side[0]-1);
 			dx =  GMT_get_map_interval (0, GMT_GRID_UPPER); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 		}
@@ -662,10 +691,10 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 						sprintf (text, format, GMT_lut[i].z_low, GMT_lut[i].z_high);
 					else if (logscl) {
 						p_val = irint (GMT_lut[i].z_low);
-						if (fabs (GMT_lut[i].z_low - (double)p_val) > GMT_CONV_LIMIT)
-							do_annot = FALSE;
+						if (GMT_IS_ZERO (GMT_lut[i].z_low - (double)p_val))
+							sprintf (text, "10@+%d@+", (int)irint (GMT_lut[i].z_low));
 						else
-							sprintf (text, "10@+%d@+", irint (GMT_lut[i].z_low));
+							do_annot = FALSE;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_low);
@@ -685,10 +714,10 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 					do_annot = TRUE;
 					if (logscl) {
 						p_val = irint (GMT_lut[i].z_high);
-						if (fabs (GMT_lut[i].z_high - (double)p_val) > GMT_CONV_LIMIT)
-							do_annot = FALSE;
-						else
+						if (GMT_IS_ZERO (GMT_lut[i].z_high - (double)p_val))
 							sprintf (text, "10@+%d@+", p_val);
+						else
+							do_annot = FALSE;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_high);
@@ -716,24 +745,25 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			x0 = x1 = 0.0;
 			yp[0] = yp[1] = 0.0;	yp[2] = yp[3] = width;
 			for (i = 0; i < GMT_n_colors; i++) {
-				x1 += z_width[i];
-				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
+				ii = (reverse) ? GMT_n_colors - i - 1 : i;
+				x1 += z_width[ii];
+				if ((f = GMT_lut[ii].fill)) {	/* Using pattern fills */
 					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
-					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, center, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+					GMT_fill (xp, yp, (GMT_LONG)4, f, center);	/* Contours drawn separately below if desired */
 				}
 				else if (intens) {
-					ii = (GMT_gray || monochrome) ? 1 : 3;
-					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*ii, sizeof (char), GMT_program);
-					for (j = 0, k = i*ii; j < ny*ii; k+=(nx-1)*ii) {
+					nb = (GMT_gray || monochrome) ? 1 : 3;
+					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*nb, sizeof (char), GMT_program);
+					for (j = 0, k = i*nb; j < ny*nb; k+=(nx-1)*nb) {
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 					}
-					GMT_color_image (x0 + gap, 0.0, z_width[i] - 2*gap, width, tmp, 1, ny, depth);
+					GMT_color_image (x0 + gap, 0.0, z_width[ii] - 2*gap, width, tmp, 1, ny, depth);
 					GMT_free ((void *)tmp);
 				}
 				else {
-					memcpy ((void *)rgb, (void *)GMT_lut[i].rgb_low, (size_t)(3 * sizeof (int)));
+					memcpy ((void *)rgb, (void *)GMT_lut[ii].rgb_low, (size_t)(3 * sizeof (int)));
 					if (intens) GMT_illuminate (max_intens[1], rgb);
 					if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
 					ps_rect (x0 + gap, 0.0, x1 - gap, width, rgb, center);
@@ -759,31 +789,31 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 		annot_off = -(((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0] + off);
 		if (logscl) annot_off = -(((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0]);
 		label_off = annot_off - 0.7 * gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH] - 1.5 * gmtdefs.annot_offset[0];
-		if (extend & 1) {	/* Add color triangles for background */
+		if (extend & (reverse + 1)) {	/* Add color triangles at bottom */
 			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_FGD : GMT_BGD;
 			if ((f = GMT_bfn[id].fill))
-				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, 1);
 		}
-		if (extend & 2) {	/* Add color triangles for foreground */
+		if (extend & (2 - reverse)) {	/* Add color triangles at top */
 			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_BGD : GMT_FGD;
 			if ((f = GMT_bfn[id].fill))
-				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, 1);
 		}
 		if (gap == 0.0) {
 			ps_segment (xleft, 0.0, xleft + length, 0.0);
@@ -801,9 +831,9 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			d_swap (project_info.x0, project_info.y0);
 			for (i = 0; i < 5; i++) frame_info.axis[1].item[i].parent = 1;
 			if (flip_anot)
-				GMT_xy_axis (-width, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[1], TRUE, frame_info.side[0]-1);
+				GMT_xy_axis (-width, 0.0, length, start_val, stop_val, &frame_info.axis[1], TRUE, frame_info.side[0]-1);
 			else
-				GMT_xy_axis (0.0, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, &frame_info.axis[1], FALSE, frame_info.side[0]-1);
+				GMT_xy_axis (0.0, 0.0, length, start_val, stop_val, &frame_info.axis[1], FALSE, frame_info.side[0]-1);
 			ps_rotatetrans (0.0, 0.0, 90.0);
 		}
 		else {
@@ -839,10 +869,10 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 						sprintf (text, format, GMT_lut[i].z_low, GMT_lut[i].z_high);
 					else if (logscl) {
 						p_val = irint (GMT_lut[i].z_low);
-						if (fabs (GMT_lut[i].z_low - (double)p_val) > GMT_CONV_LIMIT)
-							do_annot = FALSE;
-						else
+						if (GMT_IS_ZERO (GMT_lut[i].z_low - (double)p_val))
 							sprintf (text, "10@+%d@+", p_val);
+						else
+							do_annot = FALSE;
 						this_just = l_justify;
 					}
 					else
@@ -863,10 +893,10 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 					do_annot = TRUE;
 					if (logscl) {
 						p_val = irint (GMT_lut[i].z_high);
-						if (fabs (GMT_lut[i].z_high - (double)p_val) > GMT_CONV_LIMIT)
-							do_annot = FALSE;
-						else
+						if (GMT_IS_ZERO (GMT_lut[i].z_high - (double)p_val))
 							sprintf (text, "10@+%d@+", p_val);
+						else
+							do_annot = FALSE;
 						this_just = l_justify;
 					}
 					else
@@ -901,7 +931,7 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 		ps_rotatetrans (-width, 0.0, -90.0);
 
 	}
-	if (use_image) GMT_free ((void *)bar);
+	if (use_image || intens) GMT_free ((void *)bar);
 }
 
 double get_z (double x, double *width, int n)
@@ -924,7 +954,7 @@ void fix_format (char *unit, char *format)
 
 	if (unit && unit[0]) {	/* Must append the unit string */
 		if (!strchr (unit, '%'))	/* No percent signs */
-			strncpy (text, unit, GMT_TEXT_LEN);
+			strncpy (text, unit, (size_t)GMT_TEXT_LEN);
 		else {
 			for (i = j = 0; i < (int)strlen (unit); i++) {
 				text[j++] = unit[i];
@@ -943,7 +973,7 @@ void fix_format (char *unit, char *format)
 void *New_Psscale_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSSCALE_CTRL *C;
 	
-	C = (struct PSSCALE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSSCALE_CTRL), "New_Psscale_Ctrl");
+	C = (struct PSSCALE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSSCALE_CTRL), "New_Psscale_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.dpi = 300;
@@ -954,5 +984,7 @@ void *New_Psscale_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Psscale_Ctrl (struct PSSCALE_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) free ((void *)C->C.file);
+	if (C->Z.file) free ((void *)C->Z.file);
 	GMT_free ((void *)C);	
 }
