@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pstext.c,v 1.65 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pstext.c,v 1.80 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
  * Version:	3.1 based on old 3.0
  * Version:	3.3: PW: Introducing paragraph mode -M
  * Version:	3.3.4: PW: Input x,y can be in dd:mm[:ss] format
- * Version:	4.1.x
+ * Version:	4.2.x: Now can hande @_, @:, @; everywhere and @;colorname; is OK
  *
  */
 
@@ -103,11 +103,14 @@ struct PSTEXT_INFO {
 
 int main(int argc, char **argv)
 {
-	int i, j, k, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
-	int ix, iy, *use_font, n_processed = 0, n_read = 0, n_paragraphs = 0, pos;
+	int i, j, k, last_font, fno, n_files = 0, n_args, nscan;
+	int ix, iy, *use_font, n_processed = 0, n_paragraphs = 0, pos;
+	
+	GMT_LONG n_words = 0, n_read = 0;
+	
 	size_t n_alloc = 0;
 
-	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x, plot_y;
+	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x = 0.0, plot_y = 0.0;
 	double z_level = 0.0, tmp, dx, dy, xx[2], yy[2];
 
 	BOOLEAN error = FALSE, nofile = TRUE, master_record = FALSE;
@@ -123,7 +126,7 @@ int main(int argc, char **argv)
 	struct PSTEXT_INFO T;
 	struct PSTEXT_CTRL *Ctrl;
 
-	void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *info);
+	void GMT_putwords (double x, double y, char **text, GMT_LONG n_words, struct PSTEXT_INFO *info);
 	void *New_Pstext_Ctrl (), Free_Pstext_Ctrl (struct PSTEXT_CTRL *C);
 	void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C);
 	
@@ -167,10 +170,14 @@ int main(int argc, char **argv)
 					break;
 				case 'C':
 					Ctrl->C.active = TRUE;
-					k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					Ctrl->C.dx = GMT_convert_units (txt_a, GMT_INCH);
-					Ctrl->C.dy = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->C.dx;
-					Ctrl->C.percent = (strchr (argv[i], '%')) ? TRUE : FALSE;
+					if (argv[i][2]) {	/* Replace default settings with user settings */
+						Ctrl->C.percent = (strchr (argv[i], '%')) ? TRUE : FALSE;
+						k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
+						for (j = 0; txt_a[j]; j++) if (txt_a[j] == '%') txt_a[j] = '\0';	/* Remove % signs before processing values */
+						for (j = 0; k == 2 && txt_b[j]; j++) if (txt_b[j] == '%') txt_b[j] = '\0';
+						Ctrl->C.dx = GMT_convert_units (txt_a, GMT_INCH);
+						Ctrl->C.dy = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->C.dx;
+					}
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
@@ -185,9 +192,9 @@ int main(int argc, char **argv)
 						}
 						argv[i][j] = 0;
 					}
-					sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
+					j = sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
 					Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
-					Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
+					Ctrl->D.dy = (j == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->D.dx;
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
@@ -224,12 +231,12 @@ int main(int argc, char **argv)
 						Ctrl->W.outline = TRUE;
 						Ctrl->W.mode = argv[i][j];
 						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->W.pen)) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, Ctrl->W.mode);
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, (int)Ctrl->W.mode);
 							error++;
 						}
 						if (j > 3) {	/* Gave fill */
 							char txt_a[GMT_LONG_TEXT];
-							strncpy (txt_a, &argv[i][2], j-3);
+							strncpy (txt_a, &argv[i][2], (size_t)(j-3));
 							txt_a[j-3] = '\0';
 							if (GMT_getfill (txt_a, &Ctrl->W.fill)) {
 								GMT_fill_syntax ('W', " ");
@@ -245,7 +252,7 @@ int main(int argc, char **argv)
 							int n;
 							n = 2;
 							while (!(argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-							strncpy (txt_a, &argv[i][2], n-2);	txt_a[n-2] = '\0';
+							strncpy (txt_a, &argv[i][2], (size_t)(n-2));	txt_a[n-2] = '\0';
 							if (GMT_colorname2index (txt_a) >= 0) {	/* Found a colorname; thus this was actually a fill statement instead */
 								if (GMT_getfill (txt_a, &Ctrl->W.fill)) {
 									GMT_fill_syntax ('W', " ");
@@ -257,7 +264,7 @@ int main(int argc, char **argv)
 								Ctrl->W.outline = TRUE;	/* Want box outline */
 								Ctrl->W.mode = argv[i][j];
 								if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->W.pen)) {
-									fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, Ctrl->W.mode);
+									fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, (int)Ctrl->W.mode);
 									error++;
 								}
 								argv[i][j] = 0;
@@ -294,7 +301,7 @@ int main(int argc, char **argv)
 	if (argc == 1 || Ctrl->L.active || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pstext <txtfile> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
-		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n", GMT_B_OPT);
+		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>[/<dy>][v[<pen>]] [-Eaz/el] [-G<color>]\n", GMT_B_OPT);
 		fprintf (stderr, "\t[%s] [-K] [-L] [%s] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_Mo_OPT, GMT_U_OPT);
 		fprintf (stderr, "\t[-V] [-W[<fill>,][o|O|c|C[<pen>]]] [%s] [%s]\n", GMT_X_OPT, GMT_X_OPT);
 		fprintf (stderr, "\t[-Z<zlevel>] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT);
@@ -304,16 +311,15 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\tBuilt-in escape sequences:\n");
 		fprintf (stderr, "\t   @~ toggles between current font and Symbol font\n");
 		fprintf (stderr, "\t   @%%<no>%% switches to font number <no>; @%%%% resets font\n");
+		fprintf (stderr, "\t   @:<size>: switches font size; @:: resets font size\n");
+		fprintf (stderr, "\t   @;<color>; switches font color; @;; resets font color\n");
 		fprintf (stderr, "\t   @+ toggles between normal and superscript mode\n");
 		fprintf (stderr, "\t   @- toggles between normal and subscript mode\n");
 		fprintf (stderr, "\t   @# toggles between normal and Small Caps mode\n");
+		fprintf (stderr, "\t   @_ toggles between normal and underlined text\n");
 		fprintf (stderr, "\t   @!<char1><char2> makes one composite character\n");
 		fprintf (stderr, "\t   @@ prints the @ sign itself\n");
-		fprintf (stderr, "\t   @e, @o, @a, @E, @O, @A give the accented Scandinavian characters\n");
-		fprintf (stderr, "\t   In addition, paragraph text (-M) also understands these sequences:\n");
-		fprintf (stderr, "\t   @:<size>: switches font size; @:: resets font size\n");
-		fprintf (stderr, "\t   @;<r/g/b>; switches font color; @;; resets font color\n");
-		fprintf (stderr, "\t   @_ toggles between normal and underlined text\n");
+		fprintf (stderr, "\t   Use @a, @c, @e, @n, @o, @s, @u, @A, @C @E, @N, @O, @U for accented European characters\n");
 		fprintf (stderr, "\t(See manual page for more information)\n");
 
 		if (Ctrl->L.active) {	/* List fonts */
@@ -334,7 +340,7 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   if -W has been set.  Append units {c,i,m,p} or %% of fontsize [15%%]\n");
 		fprintf (stderr, "\t-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
 		fprintf (stderr, "\t   Use -Dj to move text origin away from point (direction determined by text's justification)\n");
-		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point\n");
+		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point.  If <add_y> is not given it equal <add_x>\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
 		fprintf (stderr, "\t-G set the color (red/green/blue (0-255)) for solid text [%d/%d/%d]\n", gmtdefs.basemap_frame_rgb[0], gmtdefs.basemap_frame_rgb[1], gmtdefs.basemap_frame_rgb[2]);
 		GMT_explain_option ('H');
@@ -342,7 +348,7 @@ int main(int argc, char **argv)
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
 		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
-		fprintf (stderr, "\t   parjust is one of (GMTMANSECTION)eft, (c)enter, (r)ight, or (j)ustified.\n");
+		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		fprintf (stderr, "\t-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -465,8 +471,10 @@ int main(int argc, char **argv)
 
 		while (GMT_fgets (line, BUFSIZ, fp)) {
 
-			if (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag[GMT_IN] != '#')) continue;	/* Skip blank lines or # comments */
+			if (GMT_is_a_blank_line (line)) continue;	/* Skip blank lines or # comments */
 
+			GMT_enforce_rgb_triplets (line, BUFSIZ);	/* If @; is used, make sure the color information passed on to ps_text is in r/b/g format */
+			
 			if (do_paragraphs) {	/* Paragraph mode */
 				if (line[0] == GMT_io.EOF_flag[GMT_IN]) {
 
@@ -479,12 +487,12 @@ int main(int argc, char **argv)
 
 					nscan = sscanf (&line[2], "%s %s %lf %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
 					if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
-						fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Record %ld had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 
 					if (nscan != 9) {
-						fprintf (stderr, "%s: Record %d had incomplete paragraph information, must exit)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Record %ld had incomplete paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 					GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
@@ -499,9 +507,9 @@ int main(int argc, char **argv)
 						T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
 					}
 					T.text_justify = (int)pjust;
-					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
+					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, 12);
 					if (T.block_justify == -99) {
-						fprintf (stderr, "%s: Record %d had bad justification info (set to LB)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Record %ld had bad justification info (set to LB)\n", GMT_program, n_read);
 						T.block_justify = 1;
 					}
 					if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
@@ -509,7 +517,7 @@ int main(int argc, char **argv)
 					else {	/* Gave a font name */
 						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
 						if (T.paragraph_font == GMT_N_FONTS) {
-							fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+							fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 							T.paragraph_font = 0;
 						}
 					}
@@ -524,7 +532,7 @@ int main(int argc, char **argv)
 				else {	/* Text block record */
 					if (skip_text_records) continue;	/* Skip all records for this paragraph */
 					if (!master_record) {
-						fprintf (stderr, "%s: Text record line %d not preceeded by paragraph information, must exit)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Text record line %ld not preceeded by paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 
@@ -534,7 +542,7 @@ int main(int argc, char **argv)
 						word[n_words] = (char *) GMT_memory (VNULL, (size_t)(1), sizeof (char), GMT_program);
 						n_words++;
 						if ((size_t)n_words == n_alloc) {
-							n_alloc += GMT_CHUNK;
+							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
 					}
@@ -544,7 +552,7 @@ int main(int argc, char **argv)
 						strcpy (word[n_words], p);
 						n_words++;
 						if ((size_t)n_words == n_alloc) {
-							n_alloc += GMT_CHUNK;
+							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
 					}
@@ -556,11 +564,11 @@ int main(int argc, char **argv)
 			else {	/* Old-style pstext input */
 				nscan = sscanf (line, "%s %s %lf %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
 				if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
-					fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+					fprintf (stderr, "%s: Record %ld had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 					exit (EXIT_FAILURE);
 				}
 				if (nscan != 7) {
-					fprintf (stderr, "%s: Record %d is incomplete (skipped)\n", GMT_program, n_read);
+					fprintf (stderr, "%s: Record %ld is incomplete (skipped)\n", GMT_program, n_read);
 					continue;
 				}
 				n_read++;
@@ -571,9 +579,9 @@ int main(int argc, char **argv)
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 
-				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
+				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, 12);
 				if (T.block_justify == -99) {
-					fprintf (stderr, "%s: Record %d had bad justification info (skipped)\n", GMT_program, n_read);
+					fprintf (stderr, "%s: Record %ld had bad justification info (skipped)\n", GMT_program, n_read);
 					continue;
 				}
 				if (Ctrl->A.active) {
@@ -594,7 +602,7 @@ int main(int argc, char **argv)
 				else {	/* Gave a font name */
 					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
 					if (T.paragraph_font == GMT_N_FONTS) {
-						fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+						fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 						T.paragraph_font = 0;
 					}
 				}
@@ -631,7 +639,7 @@ int main(int argc, char **argv)
 				}
 				if (T.boxflag & 32) {	/* Draw line from original point to shifted location */
 					GMT_setpen (&T.vecpen);
-					if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, 2);
+					if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, (GMT_LONG)2);
 					ps_segment (xx[0], yy[0], xx[1], yy[1]);
 				}
 			}
@@ -673,7 +681,7 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *T) {
+void GMT_putwords (double x, double y, char **text, GMT_LONG n_words, struct PSTEXT_INFO *T) {
 	char *boxpen_texture = CNULL, *vecpen_texture = CNULL;
 	int boxpen_width, boxpen_offset, boxpen_rgb[3];
 	int vecpen_width, vecpen_offset, vecpen_rgb[3];
@@ -740,7 +748,7 @@ void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C) {
 void *New_Pstext_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSTEXT_CTRL *C;
 	
-	C = (struct PSTEXT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSTEXT_CTRL), "New_Pstext_Ctrl");
+	C = (struct PSTEXT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSTEXT_CTRL), "New_Pstext_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
