@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pswiggle.c,v 1.49 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pswiggle.c,v 1.54 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -91,7 +91,9 @@ int main (int argc, char **argv)
 
 	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 
-	int i, j, n, k, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields, wantx, wanty;
+	GMT_LONG i, j, n, n_alloc = GMT_CHUNK;
+
+	int k, n_files = 0, fno, n_args, n_expected_fields, n_fields, wantx, wanty;
 
 	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2;
 	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az;
@@ -101,7 +103,7 @@ int main (int argc, char **argv)
 	struct PSWIGGLE_CTRL *Ctrl;
 
 	void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units);
-	void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track);
+	void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track);
 	void *New_Pswiggle_Ctrl (), Free_Pswiggle_Ctrl (struct PSWIGGLE_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -396,7 +398,7 @@ int main (int argc, char **argv)
 			n = 0;
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 					exit (EXIT_FAILURE);
 				}
 
@@ -405,7 +407,7 @@ int main (int argc, char **argv)
 				z[n] = in[2] - Ctrl->C.value;
 				n++;
 				if (n >= n_alloc) {
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					lon = (double *) GMT_memory ((void *)lon, (size_t)n_alloc, sizeof (double), GMT_program);
 					lat = (double *) GMT_memory ((void *)lat, (size_t)n_alloc, sizeof (double), GMT_program);
 					z = (double *) GMT_memory ((void *)z, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -495,10 +497,10 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
+void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
 {
 	double dx, dy, len, az = 0.0, s, c, x_inc, y_inc;
-	int n = 0, i;
+	GMT_LONG n = 0, i;
 	static char *name[2] = {"Positive", "Negative"}, comment[GMT_LONG_TEXT];
 
 	if (fixed) {
@@ -591,7 +593,7 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 	GMT_xyz_to_xy (x0, y0 - dy, 0.0, &xx[1], &yy[1]);
 	GMT_xyz_to_xy (x0, y0 + dy, 0.0, &xx[2], &yy[2]);
 	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 + dy, 0.0, &xx[3], &yy[3]);
-	ps_line (xx, yy, 4, 3, FALSE, TRUE);
+	ps_line (xx, yy, (GMT_LONG)4, 3, FALSE, TRUE);
 	off = ((gmtdefs.map_scale_height > 0.0) ? gmtdefs.tick_length : 0.0) + gmtdefs.annot_offset[0];
 	GMT_text3D (x0 + off, y0, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt, 0.0, 5, 0);
 }
@@ -599,7 +601,7 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 void *New_Pswiggle_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSWIGGLE_CTRL *C;
 	
-	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSWIGGLE_CTRL), "New_Pswiggle_Ctrl");
+	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSWIGGLE_CTRL), "New_Pswiggle_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.value = DBL_MAX;
@@ -613,6 +615,6 @@ void *New_Pswiggle_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Pswiggle_Ctrl (struct PSWIGGLE_CTRL *C) {	/* Deallocate control structure */
-	if (C->S.label) GMT_free ((void *)C->S.label);	
+	if (C->S.label) free ((void *)C->S.label);	
 	GMT_free ((void *)C);	
 }
