@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxyz.c,v 1.112 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psxyz.c,v 1.131 2008/05/06 04:03:12 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,10 @@ struct PSXYZ_CTRL {
 		BOOLEAN active;
 		char *file;
 	} C;
+	struct D {	/* -D<dx>/<dy>[/<dz>] */
+		BOOLEAN active;
+		double dx, dy, dz;
+	} D;
 	struct E {	/* -E<azimuth/elevation> */
 		BOOLEAN active;
 		double azimuth, elevation;
@@ -66,6 +70,7 @@ struct PSXYZ_CTRL {
 	} S;
 	struct W {	/* -W<pen> */
 		BOOLEAN active;
+		int mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
 		struct GMT_PEN pen;
 	} W;
 	struct Z {	/* -Z<z_level> */
@@ -82,6 +87,7 @@ struct PSXYZ_DATA1 {
 	double x, y, z, value, dist;
 	struct GMT_FILL f;
 	struct GMT_PEN p;
+	BOOLEAN outline;
 };
 
 struct PSXYZ_DATA2 {
@@ -91,16 +97,18 @@ struct PSXYZ_DATA2 {
 int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, done, penset_OK = TRUE;
-	BOOLEAN	get_rgb = FALSE, read_symbol, clip_set = FALSE, fill_active;
-	BOOLEAN default_outline = FALSE, default_polygon = FALSE, outline_active;
+	BOOLEAN	get_rgb = FALSE, read_symbol, clip_set = FALSE, fill_active, bar_clip;
+	BOOLEAN default_outline = FALSE, outline_active;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT];
 
-	int 	i, j, n, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
-	int	n_alloc = GMT_CHUNK, n_args, three, four, five, n_required = 3;
-	int	n_total_read = 0, n_expected = 0, this_outline, change;
+	GMT_LONG	i, n, n_alloc = GMT_CHUNK, n_total_read = 0;
+	
+	int 	j, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
+	int	n_args, three, four, five, n_required = 3;
+	int	n_expected = 0, this_outline, change;
 
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0;
+	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, dim[3], DX, DY;
 	double lux[3], tmp, *in, x_1, x_2, y_1, y_2, xxx, yyy, font_size;
 	double x2, y2, v_w, h_l, h_w, dx, dy, xx[2], yy[2], s, c, size, *xp, *yp;
 
@@ -115,9 +123,9 @@ int main (int argc, char **argv)
 
 	void column3D (double x, double y, double z, double base, double x_size, double y_size, int *rgb, int outline);
 	void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, int outline);
-	void barx3D (double x, double y, double z, double base, double size, int *rgb, int outline);
-	void bary3D (double x, double y, double z, double base, double size, int *rgb, int outline);
-	void sort_on_distance (struct PSXYZ_DATA1 *data, int n);
+	void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
+	void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
+	void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n);
 	void *New_Psxyz_Ctrl (), Free_Psxyz_Ctrl (struct PSXYZ_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -178,6 +186,18 @@ int main (int argc, char **argv)
 					Ctrl->C.file = strdup (&argv[i][2]);
 					Ctrl->C.active = TRUE;
 					break;
+				case 'D':
+					if ((n = sscanf (&argv[i][2], "%[^/]/%[^/]/%s", txt_a, txt_b, txt_c)) < 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give x and y [and z] offsets\n", GMT_program);
+						error++;
+					}
+					else {
+						Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
+						Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
+						if (n == 3) Ctrl->D.dz = GMT_convert_units (txt_c, GMT_INCH);
+						Ctrl->D.active = TRUE;
+					}
+					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
@@ -207,7 +227,10 @@ int main (int argc, char **argv)
 					break;
 				case 'W':		/* Set line attributes */
 					Ctrl->W.active = TRUE;
-					if (argv[i][2] && GMT_getpen (&argv[i][2], &Ctrl->W.pen)) {
+					j = 2;
+					if (argv[i][j] == '-') {Ctrl->W.mode = 1; j++;}
+					if (argv[i][j] == '+') {Ctrl->W.mode = 2; j++;}
+					if (argv[i][j] && GMT_getpen (&argv[i][j], &Ctrl->W.pen)) {
 						GMT_pen_syntax ('W', " ");
 						error++;
 					}
@@ -232,8 +255,8 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"psxyz %s - Plot lines, polygons, and symbols in 3-D\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: psxyz <xyzfiles> %s %s [%s]\n", GMT_J_OPT, GMT_Rgeoz_OPT, GMT_B_OPT);
-		fprintf(stderr, "\t[%s] [-C<cpt>] [-E<az/el>] [-G<fill>] [%s] [-K] [-L] [%s] [-N] [-O]\n", GMT_Jz_OPT, GMT_Ho_OPT, GMT_Mo_OPT);
-		fprintf(stderr, "\t[-P] [-Q] [-S<symbol><size>[/size_y]] [%s] [-V] [-W[<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
+		fprintf(stderr, "\t[%s] [-C<cpt>] [-D<dx>/<dy>[/<dz>]] [-E<az/el>] [-G<fill>] [%s] [-K] [-L] [%s] [-N] [-O]\n", GMT_Jz_OPT, GMT_Ho_OPT, GMT_Mo_OPT);
+		fprintf(stderr, "\t[-P] [-Q] [-S<symbol><size>[/size_y]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
 		fprintf(stderr, "\t[%s] [%s] [%s] [%s] [%s]\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -245,8 +268,9 @@ int main (int argc, char **argv)
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on t-value in 4th column (requires -S)\n");
 		fprintf (stderr, "\t   Without -S, psxyz expects -M lines/polygons and looks for -Z<tval> strings in each multiheader\n");
+		fprintf (stderr, "\t-D Offset symbol or line positions by <dx>/<dy>[/<dy>] [no offset].\n");
 		fprintf (stderr, "\t-E set user viewpoint azimuth and elevation [180/90].\n");
-		GMT_fill_syntax ('G', "Specify color (for symbols/polygons) or pattern (for polygons) [Default is no fill]");
+		GMT_fill_syntax ('G', "Specify color or pattern [Default is no fill].");
 		fprintf (stderr, "\t   If -G is specified but not -S, psxyz draws a filled polygon.\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('Z');
@@ -307,8 +331,9 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Wedges: the start and stop directions of pie wedge must be in input columns 4 and 5.\n");
 		fprintf (stderr, "\t     If -SW rather than -Sw is selected, psxy will expect two azimuths instead.\n");
 		GMT_explain_option ('V');
-		GMT_pen_syntax ('W', "sets pen attributes");
+		GMT_pen_syntax ('W', "sets pen attributes:");
 		fprintf (stderr, "\t   Implicitly draws symbol outline with this pen.\n");
+		fprintf (stderr, "\t   The leading + applies cpt color (-C) to both symbol fill and pen; - applies it to pen only.\n");
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
 		GMT_explain_option (':');
@@ -334,10 +359,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->G.fill.use_pattern && !(S.symbol == GMT_SYMBOL_LINE || S.symbol == GMT_SYMBOL_CUSTOM)) {	/* fill-pattern only for polygons */
-		error++;
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: Fill-pattern only used with polygons\n", GMT_program);
-	}
 	if (Ctrl->C.active && S.symbol == 0 && !GMT_io.multi_segments[GMT_IN]) {
 		error++;
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Must also specify a symbol (see -S) or give polygon file with -M\n", GMT_program);
@@ -350,6 +371,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have symbol information\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->W.active && Ctrl->W.mode && !Ctrl->C.active) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -W option +|-<pen> requires the -C option\n", GMT_program);
+		error++;
+	}
 
 	if (error) exit (EXIT_FAILURE);
 
@@ -364,7 +389,6 @@ int main (int argc, char **argv)
 	default_pen = current_pen = Ctrl->W.pen;
 	default_fill = current_fill = Ctrl->G.fill;
 	default_outline = Ctrl->W.active;
-	default_polygon = polygon;
 
 	if (get_rgb) n_cols_start++;
 
@@ -399,13 +423,14 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
+	lux[0] = fabs (z_project.sin_az * z_project.cos_el);
+	lux[1] = fabs (z_project.cos_az * z_project.cos_el);
+	lux[2] = fabs (z_project.sin_el);
+	tmp = MAX (lux[0], MAX (lux[1], lux[2]));
+	for (i = 0; i < 3; i++) lux[i] = (lux[i] / tmp) - 0.5;
+	
 	if ((Ctrl->C.active || current_fill.rgb[0]) >= 0 && (S.symbol == GMT_SYMBOL_COLUMN || S.symbol == GMT_SYMBOL_CUBE)) {	/* Modify the color for each facet */
-		lux[0] = fabs (z_project.sin_az * z_project.cos_el);
-		lux[1] = fabs (z_project.cos_az * z_project.cos_el);
-		lux[2] = fabs (z_project.sin_el);
-		tmp = MAX (lux[0], MAX (lux[1], lux[2]));
 		for (i = 0; i < 3; i++) {
-			lux[i] = (lux[i] / tmp) - 0.5;
 			rgb[3*i] = current_fill.rgb[0];
 			rgb[3*i+1] = current_fill.rgb[1];
 			rgb[3*i+2] = current_fill.rgb[2];
@@ -435,9 +460,20 @@ int main (int argc, char **argv)
 		ps_setfont (S.font_no);		/* Set the required font */
 	}
 	if (S.convert_angles && GMT_IS_MAPPING) S.convert_angles = 2;
+	if ((S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR) && S.v_just == 3) {
+		/* Reading 2nd coordinate so must set column types */
+		GMT_io.in_col_type[three] = GMT_io.in_col_type[GMT_X];
+		GMT_io.in_col_type[four] = GMT_io.in_col_type[GMT_Y];
+	}
 	fill_active = Ctrl->G.active;	/* Make copies because we will change the values */
 	outline_active =  Ctrl->W.active;
 	if (S.symbol > 0 && !outline_active && !fill_active && !get_rgb) outline_active = TRUE;		/* If no fill nor outline for symbols then turn outline on */
+	bar_clip = ((S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && !Ctrl->N.active) ? TRUE : FALSE;
+			/* Bars get clipped instead of skipped */
+	if (Ctrl->D.active) {
+		GMT_xyz_to_xy (Ctrl->D.dx, Ctrl->D.dy, Ctrl->D.dz, &DX, &DY);
+		ps_transrotate (DX, DY, 0.0);	/* Shift plot a bit */
+	}
 
 	done = FALSE;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
@@ -489,17 +525,11 @@ int main (int argc, char **argv)
 				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 	
 				if (GMT_io.status & GMT_IO_MISMATCH && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
 
-				/* S.size_x = S.size_y = 0.0; */
-				if (S.read_size) {
-					S.size_x = in[three] * GMT_u2u[S.u][GMT_INCH];
-					S.size_y = (S.symbol == GMT_SYMBOL_COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
-				}
-
 				if (read_symbol) {
 					i = strlen (GMT_io.current_record) - 1;
 					GMT_io.current_record[i--] = 0;	/* Chop off the \n */
@@ -511,10 +541,15 @@ int main (int argc, char **argv)
 					GMT_parse_symbol_option (Ctrl->S.arg, &S, 1, FALSE);
 					n_required = n_cols_start + S.n_required;
 				} */
+				if (S.read_size) {
+					S.size_x = in[three] * GMT_u2u[S.u][GMT_INCH];
+					S.size_y = (S.symbol == GMT_SYMBOL_COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
+				}
 
 				data1[n].flag = 0;
 				data1[n].lon = in[0];
 				data1[n].lat = in[1];
+				data1[n].string = NULL;
 				if (S.read_vector) {
 					data1[n].x_size = (float)in[three];	/* direction */
 					data1[n].y_size = (float)(in[four] * GMT_u2u[S.u][GMT_INCH]);	/* length */
@@ -555,17 +590,17 @@ int main (int argc, char **argv)
 				else if (S.symbol == GMT_SYMBOL_PIE) {
 					data1[n].x_size = (float)S.size_x;
 					if (S.convert_angles == 1) {	/* Got azimuths instead */
-						data1[n].y_size = (float)(D2R * (90.0 - in[three+S.read_size]));	/* Start direction in radians */
-						data1[n].z_size = (float)(D2R * (90.0 - in[four+S.read_size]));		/* Stop direction in radians */
+						data1[n].y_size = (float)(90.0 - in[three+S.read_size]);	/* Start direction in degrees */
+						data1[n].z_size = (float)(90.0 - in[four+S.read_size]);		/* Stop direction in degrees */
 					}
 					else {
-						data1[n].y_size = (float)(D2R * in[three+S.read_size]);			/* Start direction in radians */
-						data1[n].z_size = (float)(D2R * in[four+S.read_size]);			/* Stop direction in radians */
+						data1[n].y_size = (float)in[three+S.read_size];			/* Start direction in degrees */
+						data1[n].z_size = (float)in[four+S.read_size];			/* Stop direction in degrees */
 					}
 				}
 				else if (S.symbol == GMT_SYMBOL_RECT) {
-					data1[n].x_size = (float)(0.5 * in[three+S.read_size]);	/* x-dim */
-					data1[n].y_size = (float)(0.5 * in[four+S.read_size]);	/* y-dim */
+					data1[n].x_size = (float)(in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH]);	/* x-dim */
+					data1[n].y_size = (float)(in[four+S.read_size] * GMT_u2u[S.u][GMT_INCH]);	/* y-dim */
 				}
 				else {
 					data1[n].x_size = (float)S.size_x;
@@ -576,19 +611,34 @@ int main (int argc, char **argv)
 				/* Skip zero-size symbols */
 		
 				if (S.symbol != GMT_SYMBOL_POINT && S.symbol < GMT_SYMBOL_ELLIPSE && S.size_x <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (S.read_vector && S.v_just < 3 && data1[n].y_size <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: vector length <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: vector length <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (S.symbol == GMT_SYMBOL_ELLIPSE && (data1[n].y_size <= 0.0 || data1[n].z_size <= 0.0)) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: ellipse axes <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: ellipse axes <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 
-				if (!Ctrl->N.active) {
+				if (bar_clip) {
+					if (S.symbol == GMT_SYMBOL_BARX) {
+						if (in[GMT_X] < west)
+							in[GMT_X] = west;
+						else if (in[GMT_X] > east)
+							in[GMT_X] = east;
+					}
+					else if (S.symbol == GMT_SYMBOL_BARY) {
+						if (in[GMT_Y] < south)
+							in[GMT_Y] = south;
+						else if (in[GMT_Y] > north)
+							in[GMT_Y] = north;
+					}
+					if (in[2] < project_info.z_bottom || in[2] > project_info.z_top) continue;
+				}
+				else if (!Ctrl->N.active) {
 					if (in[2] < project_info.z_bottom || in[2] > project_info.z_top) continue;
 					GMT_map_outside (in[0], in[1]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
@@ -625,12 +675,18 @@ int main (int argc, char **argv)
 					strcpy (data1[n].string, S.string);
 				}
 				data1[n].symbol = S.symbol;
+				if (Ctrl->W.mode) {
+					memcpy ((void *)Ctrl->W.pen.rgb, (void *)current_fill.rgb, 3*sizeof (int));
+					current_pen = Ctrl->W.pen;
+				}
+				if (Ctrl->W.mode & 1) memcpy ((void *)current_fill.rgb, (void *)GMT_no_rgb, 3*sizeof (int));
 				data1[n].f = current_fill;
 				data1[n].p = current_pen;
+				data1[n].outline = outline_active;
 
 				n++;
 				if (n == n_alloc) {
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					data1 = (struct PSXYZ_DATA1 *) GMT_memory ((void *)data1, (size_t)n_alloc, sizeof (struct PSXYZ_DATA1), GMT_program);
 				}
 				if (read_symbol) n_expected = BUFSIZ;
@@ -656,34 +712,34 @@ int main (int argc, char **argv)
 		
 				if (GMT_cpt_skip) continue;	/* Skip this because cpt says so */
 
-				if (outline_active) GMT_setpen (&data1[i].p);
+				if (data1[i].outline) GMT_setpen (&data1[i].p);
 		
 				switch (data1[i].symbol) {
 					case GMT_SYMBOL_NONE:
 						break;
 					case GMT_SYMBOL_STAR:
 						size = (S.equal_area) ? 1.67289326141 * data1[i].x_size : data1[i].x_size;
-						GMT_star3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_star (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_BARX:
 						if (data1[i].flag & 4) {
-							GMT_geo_to_xy (data1[i].x, data1[i].y-data1[i].x_size, &x_1, &y_1);
-							GMT_geo_to_xy (data1[i].x, data1[i].y+data1[i].x_size, &x_2, &y_2);
+							GMT_geo_to_xy (data1[i].x, data1[i].y-0.5*data1[i].x_size, &x_1, &y_1);
+							GMT_geo_to_xy (data1[i].x, data1[i].y+0.5*data1[i].x_size, &x_2, &y_2);
 							xxx = 0.5 * hypot (x_1 - x_2, y_1 - y_2);
-							barx3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, data1[i].f.rgb, outline_active);
+							barx3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, &data1[i].f, data1[i].outline);
 						}
 						else
-							barx3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, data1[i].f.rgb, outline_active);
+							barx3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_BARY:
 						if (data1[i].flag & 4) {
-							GMT_geo_to_xy (data1[i].x-data1[i].x_size, data1[i].y, &x_1, &y_1);
-							GMT_geo_to_xy (data1[i].x+data1[i].x_size, data1[i].y, &x_2, &y_2);
+							GMT_geo_to_xy (data1[i].x-0.5*data1[i].x_size, data1[i].y, &x_1, &y_1);
+							GMT_geo_to_xy (data1[i].x+0.5*data1[i].x_size, data1[i].y, &x_2, &y_2);
 							xxx = 0.5 * hypot (x_1 - x_2, y_1 - y_2);
-							bary3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, data1[i].f.rgb, outline_active);
+							bary3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, &data1[i].f, data1[i].outline);
 						}
 						else
-							bary3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, data1[i].f.rgb, outline_active);
+							bary3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_COLUMN:
 						if (data1[i].flag & 4) {
@@ -693,75 +749,83 @@ int main (int argc, char **argv)
 							GMT_geo_to_xy (data1[i].x, data1[i].y-data1[i].y_size, &x_1, &y_1);
 							GMT_geo_to_xy (data1[i].x, data1[i].y+data1[i].y_size, &x_2, &y_2);
 							yyy = 0.5 * hypot (x_1 - x_2, y_1 - y_2);
-							column3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, yyy, rgb, outline_active);
+							column3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, xxx, yyy, rgb, data1[i].outline);
 						}
 						else
-							column3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, (double)data1[i].y_size, rgb, outline_active);
+							column3D (data1[i].x, data1[i].y, data1[i].z, data1[i].z_size, (double)data1[i].x_size, (double)data1[i].y_size, rgb, data1[i].outline);
 						break;
 					case GMT_SYMBOL_CROSS:
-						GMT_cross3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size);
+						dim[0] = (double)data1[i].x_size;
+						GMT_cross (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
 						break;
 					case GMT_SYMBOL_POINT:
-						GMT_cross3D (data1[i].x, data1[i].y, data1[i].z, GMT_POINT_SIZE);
+						dim[0] = GMT_POINT_SIZE;
+						GMT_cross (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
 						break;
 					case GMT_SYMBOL_CIRCLE:
-						GMT_circle3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, data1[i].f.rgb, outline_active);
+						size = (double)data1[i].x_size;
+						GMT_circle (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_SQUARE:
 						size = (S.equal_area) ? 1.25331413732 * data1[i].x_size : data1[i].x_size;
-						GMT_square3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_square (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_HEXAGON:
 						size = (S.equal_area) ? 1.09963611079 * data1[i].x_size : data1[i].x_size;
-						GMT_hexagon3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_hexagon (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_PENTAGON:
 						size = (S.equal_area) ? 1.14948092619 * data1[i].x_size : data1[i].x_size;
-						GMT_pentagon3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_pentagon (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_OCTAGON:
 						size = (S.equal_area) ? 1.05390736526 * data1[i].x_size : data1[i].x_size;
-						GMT_octagon3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_octagon (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_TRIANGLE:
 						size = (S.equal_area) ? 1.55512030156 * data1[i].x_size : data1[i].x_size;
-						GMT_triangle3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_triangle (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_ITRIANGLE:
 						size = (S.equal_area) ? 1.55512030156 * data1[i].x_size : data1[i].x_size;
-						GMT_itriangle3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_itriangle (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_DIAMOND:
 						size = (S.equal_area) ? 1.25331413732 * data1[i].x_size : data1[i].x_size;
-						GMT_diamond3D (data1[i].x, data1[i].y, data1[i].z, size, data1[i].f.rgb, outline_active);
+						GMT_diamond (data1[i].x, data1[i].y, data1[i].z, &size, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_CUBE:
-						cube3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, (double)data1[i].y_size, rgb, outline_active);
+						cube3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, (double)data1[i].y_size, rgb, data1[i].outline);
 						break;
 					case GMT_SYMBOL_ELLIPSE:
 						if (data1[i].flag == 2)
-							GMT_plot_ellipse (data1[i].lon, data1[i].lat, data1[i].z, data1[i].y_size, data1[i].z_size, data1[i].x_size, current_fill, outline_active);
-						else
-							GMT_ellipse3D (data1[i].x, data1[i].y, data1[i].z, data1[i].x_size, data1[i].y_size, data1[i].z_size, data1[i].f.rgb, outline_active);
+							GMT_plot_ellipse (data1[i].lon, data1[i].lat, data1[i].z, data1[i].y_size, data1[i].z_size, data1[i].x_size, current_fill, data1[i].outline);
+						else {
+							dim[0] = data1[i].x_size; dim[1] = data1[i].y_size; dim[2] = data1[i].z_size;
+							GMT_ellipse (data1[i].x, data1[i].y, data1[i].z, dim, &data1[i].f, data1[i].outline);
+						}
 						break;
 					case GMT_SYMBOL_ROTATERECT:
 						if (data1[i].flag == 2)
-							GMT_plot_rectangle (data1[i].lon, data1[i].lat, data1[i].z, data1[i].y_size, data1[i].z_size, data1[i].x_size, current_fill, outline_active);
-						else
-							GMT_rotrect3D (data1[i].x, data1[i].y, data1[i].z, data1[i].x_size, data1[i].y_size, data1[i].z_size, data1[i].f.rgb, outline_active);
+							GMT_plot_rectangle (data1[i].lon, data1[i].lat, data1[i].z, data1[i].y_size, data1[i].z_size, data1[i].x_size, current_fill, data1[i].outline);
+						else {
+							dim[0] = data1[i].x_size; dim[1] = data1[i].y_size; dim[2] = data1[i].z_size;
+							GMT_rotrect (data1[i].x, data1[i].y, data1[i].z, dim, &data1[i].f, data1[i].outline);
+						}
 						break;
 					case GMT_SYMBOL_TEXT:
-						font_size = 2.0 * data1[i].x_size * 72.0;	/* To get points */
-						if (outline_active && fill_active) {
+						font_size = data1[i].x_size * 72.0;	/* To get points */
+						if (data1[i].outline && fill_active) {
 							ps_setpaint (data1[i].f.rgb);
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, FALSE);
-							ps_setpaint (current_pen.rgb);
+							ps_setpaint (data1[i].p.rgb);
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, TRUE);
 						}
 						else if (fill_active)
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, FALSE);
 						else
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, TRUE);
+						GMT_free ((void *)data1[i].string);
 						break;
 					case GMT_SYMBOL_VECTOR:
 						if (data1[i].flag & 2) {
@@ -772,9 +836,9 @@ int main (int argc, char **argv)
 							data1[i].x_size = (float)(90.0 - data1[i].x_size);
 						}
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[3], in[4], &x2, &y2);
+							GMT_geo_to_xy (in[three], in[four], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -788,8 +852,8 @@ int main (int argc, char **argv)
 								x2 -= dx;		y2 -= dy;
 							}
 						}
-						this_outline = (S.v_double_heads) ? outline_active + 8 : outline_active;
-						GMT_vector3D (data1[i].x, data1[i].y, x2, y2, data1[i].z, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, data1[i].f.rgb, this_outline);
+						this_outline = (S.v_double_heads) ? data1[i].outline + 8 : data1[i].outline;
+						GMT_vector (data1[i].x, data1[i].y, x2, y2, data1[i].z, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &data1[i].f, this_outline);
 						break;
 					case GMT_SYMBOL_VECTOR2:
 						if (data1[i].flag & 2) {
@@ -800,9 +864,9 @@ int main (int argc, char **argv)
 							data1[i].x_size = (float)(90.0 - data1[i].x_size);
 						}
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[3], in[4], &x2, &y2);
+							GMT_geo_to_xy (in[three], in[four], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -820,12 +884,12 @@ int main (int argc, char **argv)
 							v_w = S.v_width * data1[i].y_size * data1[i].z_size;
 							h_l = S.h_length * data1[i].y_size * data1[i].z_size;
 							h_w = S.h_width * data1[i].y_size * data1[i].z_size;
-							this_outline = (S.v_double_heads) ? outline_active + 8 : outline_active;
-							GMT_vector3D (data1[i].x, data1[i].y, x2, y2, data1[i].z, v_w, h_l, h_w, gmtdefs.vector_shape, data1[i].f.rgb, this_outline);
+							this_outline = (S.v_double_heads) ? data1[i].outline + 8 : data1[i].outline;
+							GMT_vector (data1[i].x, data1[i].y, x2, y2, data1[i].z, v_w, h_l, h_w, gmtdefs.vector_shape, &data1[i].f, this_outline);
 						}
 						else {	/* Leave as specified */
-							this_outline = (S.v_double_heads) ? outline_active + 8 : outline_active;
-							GMT_vector3D (data1[i].x, data1[i].y, x2, y2, data1[i].z, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, data1[i].f.rgb, this_outline);
+							this_outline = (S.v_double_heads) ? data1[i].outline + 8 : data1[i].outline;
+							GMT_vector (data1[i].x, data1[i].y, x2, y2, data1[i].z, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &data1[i].f, this_outline);
 						}
 						break;
 					case GMT_SYMBOL_PIE:
@@ -835,28 +899,30 @@ int main (int argc, char **argv)
 							GMT_azim_to_angle (data1[i].lon, data1[i].lat, 0.1, (double)(R2D*data1[i].z_size), &tmp);
 							data1[i].z_size = (float)(D2R*tmp);
 						}
-						GMT_pie3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, (double)data1[i].y_size, (double)data1[i].z_size, data1[i].f.rgb, outline_active);
+						dim[0] = 0.5 * data1[i].x_size; dim[1] = data1[i].y_size; dim[2] = data1[i].z_size;
+						GMT_pie (data1[i].x, data1[i].y, data1[i].z, dim, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_XDASH:
-						GMT_xyz_to_xy (data1[i].x-(double)data1[i].x_size, data1[i].y, data1[i].z, &xx[0], &yy[0]);
-						GMT_xyz_to_xy (data1[i].x+(double)data1[i].x_size, data1[i].y, data1[i].z, &xx[1], &yy[1]);
+						GMT_xyz_to_xy (data1[i].x-0.5*(double)data1[i].x_size, data1[i].y, data1[i].z, &xx[0], &yy[0]);
+						GMT_xyz_to_xy (data1[i].x+0.5*(double)data1[i].x_size, data1[i].y, data1[i].z, &xx[1], &yy[1]);
 						ps_segment (xx[0], yy[0], xx[1], yy[1]);
 						break;
 					case GMT_SYMBOL_YDASH:
-						GMT_xyz_to_xy (data1[i].x, data1[i].y-(double)data1[i].x_size, data1[i].z, &xx[0], &yy[0]);
-						GMT_xyz_to_xy (data1[i].x, data1[i].y+(double)data1[i].x_size, data1[i].z, &xx[1], &yy[1]);
+						GMT_xyz_to_xy (data1[i].x, data1[i].y-0.5*(double)data1[i].x_size, data1[i].z, &xx[0], &yy[0]);
+						GMT_xyz_to_xy (data1[i].x, data1[i].y+0.5*(double)data1[i].x_size, data1[i].z, &xx[1], &yy[1]);
 						ps_segment (xx[0], yy[0], xx[1], yy[1]);
 						break;
 					case GMT_SYMBOL_ZDASH:
-						GMT_xyz_to_xy (data1[i].x, data1[i].y, data1[i].z-(double)data1[i].x_size, &xx[0], &yy[0]);
-						GMT_xyz_to_xy (data1[i].x, data1[i].y, data1[i].z+(double)data1[i].x_size, &xx[1], &yy[1]);
+						GMT_xyz_to_xy (data1[i].x, data1[i].y, data1[i].z-0.5*(double)data1[i].x_size, &xx[0], &yy[0]);
+						GMT_xyz_to_xy (data1[i].x, data1[i].y, data1[i].z+0.5*(double)data1[i].x_size, &xx[1], &yy[1]);
 						ps_segment (xx[0], yy[0], xx[1], yy[1]);
 						break;
 					case GMT_SYMBOL_RECT:
-						GMT_rect3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, (double)data1[i].y_size, data1[i].f.rgb, outline_active);
+						dim[0] = data1[i].x_size; dim[1] = data1[i].y_size;
+						GMT_rect (data1[i].x-0.5*dim[0], data1[i].y-0.5*dim[1], data1[i].z, dim, &data1[i].f, data1[i].outline);
 						break;
 					case GMT_SYMBOL_CUSTOM:
-						GMT_draw_custom_symbol (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, S.custom, &current_pen, &current_fill, outline_active);
+						GMT_draw_custom_symbol (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, S.custom, &current_pen, &current_fill, data1[i].outline);
 						break;
 				}
 			}
@@ -890,7 +956,7 @@ int main (int argc, char **argv)
 				n = 0;
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
@@ -898,7 +964,7 @@ int main (int argc, char **argv)
 					data2[n].z = in[2];
 					n++;
 					if (n == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						data2 = (struct PSXYZ_DATA2 *) GMT_memory ((void *)data2, (size_t)n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
 					}
 					n_fields = GMT_input (fp, &n_expected, &in);
@@ -949,7 +1015,12 @@ int main (int argc, char **argv)
 
 	}
 
-	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) GMT_contlabel_plot (&S.G);
+	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
+		GMT_contlabel_plot (&S.G);
+		GMT_contlabel_free (&S.G);
+	}
+
+	if (Ctrl->D.active) ps_transrotate (-DX, -DY, 0.0);	/* Shift plot a bit */
 
 	if (clip_set) GMT_map_clip_off ();
 
@@ -1004,7 +1075,7 @@ void column3D (double x, double y, double z, double base, double x_size, double
 		}
 		k = z_project.face[i] / 2;
 		for (j = 0; j < 4; j++) GMT_xyz_to_xy (xp[j], yp[j], zp[j], &plot_x[j], &plot_y[j]);
-		ps_patch (plot_x, plot_y, 4, &rgb[3*k], outline);
+		ps_patch (plot_x, plot_y, (GMT_LONG)4, &rgb[3*k], outline);
 	}
 }
 
@@ -1042,36 +1113,38 @@ void cube3D (double x, double y, double z, double x_size, double y_size, int *rg
 		}
 		k = z_project.face[i] / 2;
 		for (j = 0; j < 4; j++) GMT_xyz_to_xy (xp[j], yp[j], zp[j], &plot_x[j], &plot_y[j]);
-		ps_patch (plot_x, plot_y, 4, &rgb[3*k], outline);
+		ps_patch (plot_x, plot_y, (GMT_LONG)4, &rgb[3*k], outline);
 	}
 }
 
-void bary3D (double x, double y, double z, double base, double size, int rgb[], int outline)
+void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline)
 {
 	int i;
 	double xp[4], yp[4], plot_x[4], plot_y[4];
 
+	size *= 0.5;
 	xp[0] = xp[3] = x - size;	xp[1] = xp[2] = x + size;
 	yp[0] = yp[1] = base;	yp[2] = yp[3] = y;
 	for (i = 0; i < 4; i++) GMT_xyz_to_xy (xp[i], yp[i], z, &plot_x[i], &plot_y[i]);
-	ps_patch (plot_x, plot_y, 4, rgb, outline);
+	GMT_fill (plot_x, plot_y, (GMT_LONG)4, f, outline);
 }
 
-void barx3D (double x, double y, double z, double base, double size, int rgb[], int outline)
+void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline)
 {
 	int i;
 	double xp[4], yp[4], plot_x[4], plot_y[4];
 
+	size *= 0.5;
 	yp[0] = yp[3] = y - size;	yp[1] = yp[2] = y + size;
 	xp[0] = xp[1] = base;	xp[2] = xp[3] = x;
 	for (i = 0; i < 4; i++) GMT_xyz_to_xy (xp[i], yp[i], z, &plot_x[i], &plot_y[i]);
-	ps_patch (plot_x, plot_y, 4, rgb, outline);
+	GMT_fill (plot_x, plot_y, (GMT_LONG)4, f, outline);
 }
 
-void sort_on_distance (struct PSXYZ_DATA1 *data, int n)
+void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n)
 {
 	/* This function sorts the data array such that points farthest away are plotted first */
-	int i;
+	GMT_LONG i;
 
 	int dist_compare(const void *point_1, const void *point_2);
 	double dx, dy, x0, y0, x, y, dr, a, b, c;
@@ -1124,7 +1197,7 @@ int dist_compare (const void *point_1, const void *point_2)
 void *New_Psxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSXYZ_CTRL *C;
 	
-	C = (struct PSXYZ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSXYZ_CTRL), "New_Psxyz_Ctrl");
+	C = (struct PSXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSXYZ_CTRL), "New_Psxyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -1137,6 +1210,6 @@ void *New_Psxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 }
 
 void Free_Psxyz_Ctrl (struct PSXYZ_CTRL *C) {	/* Deallocate control structure */
-	if (C->C.file) GMT_free ((void *)C->C.file);	
+	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
