@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: sample1d.c,v 1.33 2007/03/12 19:52:26 remko Exp $
+ *	$Id: sample1d.c,v 1.42 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,8 +56,9 @@ struct SAMPLE1D_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i, j, k, n, m, n_alloc = 0, m_alloc, result, n_col = 0, rows = 1, n_fields, n_expected_fields;
-	int n_files = 0, fno, n_args, n_req, m_supplied = 0;
+	int result, n_col = 0, n_fields, n_expected_fields, n_files = 0, fno, n_args, n_req;
+	
+	GMT_LONG i, j, k, n, m, n_alloc = 0, m_alloc, rows = 1, m_supplied = 0;
 
 	BOOLEAN error = FALSE, *nan_flag = VNULL, nofile = TRUE, done = FALSE;
 
@@ -109,6 +110,10 @@ int main (int argc, char **argv)
 						case 'C':
 							Ctrl->F.mode = 2;
 							break;
+						case 'n':
+						case 'N':
+							Ctrl->F.mode = 3;
+							break;
 						default:	/* Use GMT defaults */
 							break;
 					}
@@ -161,7 +166,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"sample1d %s - Resampling of 1-D data sets\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c] [%s] [-I<t_inc>] [%s] [-N<knotfile>]\n", GMT_H_OPT, GMT_M_OPT);
+		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c|n] [%s] [-I<t_inc>] [%s] [-N<knotfile>]\n", GMT_H_OPT, GMT_M_OPT);
 		fprintf (stderr, "\t[-S<xstart>] [-T<time_col>] [-V] [%s] [%s]\n\n", GMT_b_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -173,6 +178,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   l Linear interpolation\n");
 		fprintf (stderr, "\t   a Akima spline interpolation\n");
 		fprintf (stderr, "\t   c Cubic spline interpolation\n");
+		fprintf (stderr, "\t   n No interpolation (nearest point)\n");
 		fprintf (stderr, "\t   [Default is -F%c]\n", type[Ctrl->F.mode]);
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I <x_inc> sets equidistant grid interval [x1 - x0]\n");
@@ -242,20 +248,26 @@ int main (int argc, char **argv)
 		t_supplied_out = (double *) GMT_memory (VNULL, (size_t)m_alloc, sizeof (double), GMT_program);
 		m = 0;
 		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fpt);
-		while (GMT_fgets (line, BUFSIZ, fpt)) {
-			if (line[0] == '#' || line[0] == GMT_io.EOF_flag[GMT_IN]) continue;
-			sscanf (line, "%lf", &t_supplied_out[m]);
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
+		n_fields = GMT_input (fpt, &n_expected_fields, &in);
+		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+			while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {
+				n_fields = GMT_input (fpt, &n_expected_fields, &in);
+			}
+			if (GMT_io.status & GMT_IO_EOF) continue;	/* At EOF */
+			t_supplied_out[m] = in[0];
 			m++;
 			if (m == m_alloc) {	/* Get more memory */
-				m_alloc += GMT_CHUNK;
+				m_alloc <<= 1;
 				t_supplied_out = (double *) GMT_memory ((void *)t_supplied_out, (size_t)m_alloc, sizeof (double), GMT_program);
 			}
+			n_fields = GMT_input (fpt, &n_expected_fields, &in);
 		}
 		GMT_fclose (fpt);
 		m_supplied = m;
 		t_supplied_out = (double *) GMT_memory ((void *)t_supplied_out, (size_t)m_supplied, sizeof (double), GMT_program);
 		t_out = (double *) GMT_memory (VNULL, (size_t)m_supplied, sizeof (double), GMT_program);
-                if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d knots from file\n", GMT_program, m_supplied);
+                if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld knots from file\n", GMT_program, m_supplied);
 	}
 
 	if (n_files > 0)
@@ -306,7 +318,7 @@ int main (int argc, char **argv)
 
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 					exit (EXIT_FAILURE);
 				}
 
@@ -335,7 +347,7 @@ int main (int argc, char **argv)
 				n++;
 
 				if (n == n_alloc) {	/* Get more memory */
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					for (j = 0; j < n_col; j++) col[j] = (double *) GMT_memory ((void *)col[j], (size_t)n_alloc, sizeof (double), GMT_program);
 				}
 
@@ -419,7 +431,7 @@ int main (int argc, char **argv)
 					result = GMT_intpol (col[Ctrl->T.col], col[j], n, m, t_out, out[j], Ctrl->F.mode);
 
 				if (result != 0) {
-					fprintf (stderr, "%s: Error from GMT_intpol near row %d!\n", GMT_program, rows+result+1);
+					fprintf (stderr, "%s: Error from GMT_intpol near row %ld!\n", GMT_program, rows+result+1);
 					exit (EXIT_FAILURE);
 				}
 			}
@@ -443,7 +455,10 @@ int main (int argc, char **argv)
 	GMT_free ((void *)t_out);
 	GMT_free ((void *)out);
 	GMT_free ((void *)col);
-
+	if (dout) GMT_free ((void *)dout);
+	if (nan_flag) GMT_free ((void *)nan_flag);
+	if (Ctrl->N.active) GMT_free ((void *)t_supplied_out);
+	
 	Free_Sample1d_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
@@ -454,7 +469,7 @@ int main (int argc, char **argv)
 void *New_Sample1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SAMPLE1D_CTRL *C;
 	
-	C = (struct SAMPLE1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SAMPLE1D_CTRL), "New_Sample1d_Ctrl");
+	C = (struct SAMPLE1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SAMPLE1D_CTRL), "New_Sample1d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->F.mode = gmtdefs.interpolant;
@@ -463,6 +478,6 @@ void *New_Sample1d_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Sample1d_Ctrl (struct SAMPLE1D_CTRL *C) {	/* Deallocate control structure */
-	if (C->N.file) GMT_free ((void *)C->N.file);	
+	if (C->N.file) free ((void *)C->N.file);	
 	GMT_free ((void *)C);	
 }
