@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegy.c,v 1.21 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pssegy.c,v 1.27 2008/05/12 22:35:47 guru Exp $
  *
- *    Copyright (c) 1999-2007 by T. Henstock
+ *    Copyright (c) 1999-2008 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegy program to plot segy files in postscript with variable trace spacing option
@@ -53,7 +53,7 @@ int main (int argc, char **argv)
 	int i, nm;
 	int ix, iy, n_traces=10000, n_samp=0, n_sampr=0, shade[3]={0,0,0}, trans[3]={-1,-1,-1}, n_tracelist=0;
 	int check, polarity=1;
-	BOOLEAN plot_it = FALSE;
+	int plot_it = FALSE;
 
 	float bias = 0.0, scale = 1.0, clip = 0.0, deviation = 0.0;
 	float toffset=0.0;
@@ -164,7 +164,7 @@ int main (int argc, char **argv)
 				case 'T':
 					strcpy (input, &argv[i][2]);
 					if ((trace_file = strlen(input))){
-						if ((fpt = GMT_fopen (input, "r")) == NULL){
+						if ((fpt = fopen (input, "r")) == NULL){
 							fprintf(stderr, "%s: Cannot find trace list file %s\n", GMT_program, input);
 							error++;
 						}
@@ -185,7 +185,7 @@ int main (int argc, char **argv)
 					break;
 			}
 		}
-		else if ((fpi = GMT_fopen (argv[i], "rb")) == NULL) {
+		else if ((fpi = fopen (argv[i], "rb")) == NULL) {
 			fprintf (stderr, "%s: Cannot find segy file %s\n", GMT_program,argv[i]);
 			exit (EXIT_FAILURE);
 		}
@@ -255,7 +255,7 @@ int main (int argc, char **argv)
 		error++;
 		fprintf(stderr,"%s: SYNTAX ERROR: Must specify a positive deviation\n",GMT_program);
 	}
-	if (project_info.projection >2){
+	if (!GMT_IS_LINEAR){
 		fprintf(stderr,"%s: WARNING: you asked for a non-rectangular projection. \n It will probably still work, but be prepared for problems\n",GMT_program);
 	}
 
@@ -269,7 +269,7 @@ int main (int argc, char **argv)
 	if (fpi == NULL) fpi = stdin;
 
 	if (trace_file){ /* must read in file of desired trace locations */
-		tracelist = (double *) GMT_memory (CNULL, GMT_CHUNK, sizeof(double), "pssegy");
+		tracelist = (double *) GMT_memory (CNULL, (size_t)GMT_CHUNK, sizeof(double), "pssegy");
 		n_tracelist = GMT_CHUNK;
 		ix=0;
 		while ((fscanf(fpt, "%lf", &test)) != EOF){
@@ -277,7 +277,7 @@ int main (int argc, char **argv)
 			ix++;
 			if(ix == n_tracelist){ /* need more memory in array */
 				n_tracelist += GMT_CHUNK;
-				tracelist = (double *) GMT_memory ((char *)tracelist, n_tracelist, sizeof(double), "pssegy");
+				tracelist = (double *) GMT_memory ((char *)tracelist, (size_t)n_tracelist, sizeof(double), "pssegy");
 			}
 		}
 		n_tracelist = ix;
@@ -332,7 +332,7 @@ use a few of these*/
 
 	if (!n_sampr){ /* no number of samples still - a problem! */
 		fprintf(stderr, "%s: Error, number of samples per trace unknown\n", GMT_program);
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 	if(gmtdefs.verbose) 
@@ -350,11 +350,11 @@ use a few of these*/
 
 	if (!dy){ /* still no sample interval at this point is a problem! */
 		fprintf(stderr, "%s: Error, no sample interval in reel header\n", GMT_program);
-		exit(-1);
+		exit(EXIT_FAILURE);
 	}
 
 
-	bitmap = (unsigned char *) GMT_memory (NULL, nm, sizeof (unsigned char), "pssegy");
+	bitmap = (unsigned char *) GMT_memory (NULL, (size_t)nm, sizeof (unsigned char), "pssegy");
 
 	ix=0;
 	while ((ix<n_traces) && (header = get_segy_header(fpi))){   /* read traces one by one */
