@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegyz.c,v 1.27 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: pssegyz.c,v 1.32 2008/05/12 22:35:47 guru Exp $
  *
- *    Copyright (c) 1999-2007 by T. Henstock
+ *    Copyright (c) 1999-2008 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegyz program to plot segy files in 3d in postscript with variable trace spacing option
@@ -233,7 +233,7 @@ int main (int argc, char **argv)
 					break;
 			}
 			}
-		else if ((fpi = GMT_fopen (argv[i], "rb")) == NULL) {
+		else if ((fpi = fopen (argv[i], "rb")) == NULL) {
 			fprintf (stderr, "%s: Cannot find segy file %s\n",  GMT_program, argv[i]);
 			exit (EXIT_FAILURE);
 		}
@@ -299,7 +299,7 @@ int main (int argc, char **argv)
  		error++;
  		fprintf(stderr,"%s: SYNTAX ERROR: Must specify a positive deviation\n",GMT_program);
  	}
- 	if (project_info.projection >2){
+ 	if (!GMT_IS_LINEAR){
  		fprintf(stderr,"%s: WARNING: you asked for a non-rectangular projection. \n It will probably still work, but be prepared for problems\n",GMT_program);
  	}
   	if (!project_info.z_bottom && !project_info.z_top){ /* no z range in the -R parameter */
@@ -378,7 +378,7 @@ use a few of these*/
 	}
 
 	if(gmtdefs.verbose) 
-		fprintf(stderr, "%s: Number of samples is %d\n", GMT_program, n_samp);
+		fprintf(stderr, "%s: Number of samples is %dl\n", GMT_program, n_samp);
 
 	if(binhead.dsfc != 5) fprintf(stderr, "%s: WARNING data not in IEEE format\n", GMT_program);
 
@@ -396,7 +396,7 @@ use a few of these*/
 	}
 
 
-	bitmap = (unsigned char *) GMT_memory (NULL, nm, sizeof (char), "pssegyz");
+	bitmap = (unsigned char *) GMT_memory (NULL, (size_t)nm, sizeof (char), "pssegyz");
 
 	ix=0;
 	while ((ix<n_traces) && (header = get_segy_header(fpi))){   /* read traces one by one */
