@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: spectrum1d.c,v 1.34 2007/03/12 19:52:26 remko Exp $
+ *	$Id: spectrum1d.c,v 1.39 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -96,7 +96,7 @@ struct SPECTRUM1D_CTRL {
 
 struct SPECTRUM1D_INFO {	/* Control structure for spectrum1d */
 	BOOLEAN	y_given;
-	int	n_spec, window, window_2;
+	GMT_LONG	n_spec, window, window_2;
 	float	*datac;
 	double	dt, x_variance, y_variance, d_n_windows, y_pow;
 	struct	SPEC {
@@ -115,7 +115,8 @@ int main (int argc, char **argv)
 	
 	char	buffer[BUFSIZ];
 
-	int	i, k, n_alloc, n_expected_fields, n_fields, j, n_outputs, window_test = 2, n_data;
+	int	i, k, j, n_expected_fields, n_fields, n_outputs, window_test = 2;
+	GMT_LONG	n_alloc, n_data;
 
 	float	*x, *y = NULL;
 
@@ -127,7 +128,7 @@ int main (int argc, char **argv)
 	struct SPECTRUM1D_CTRL *Ctrl;
 
 	void alloc_arrays (struct SPECTRUM1D_INFO *C);
-	void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data);
+	void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, GMT_LONG n_data);
 	void write_output (struct SPECTRUM1D_INFO *C, char *cols, int n_outputs, BOOLEAN write_wavelength, char *namestem);
 	void free_space (struct SPECTRUM1D_INFO *C);
 	void *New_Spectrum1d_Ctrl (), Free_Spectrum1d_Ctrl (struct SPECTRUM1D_CTRL *C);
@@ -176,13 +177,9 @@ int main (int argc, char **argv)
 				case 'N':
 					Ctrl->N.active = TRUE;
 					if (argv[i][2]) {
-						GMT_free ((void *)Ctrl->N.name);
+						free ((void *)Ctrl->N.name);
 						Ctrl->N.name = strdup (&argv[i][2]);
 					}
-					else {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -N option: Append a name stem\n", GMT_program);
-						error = TRUE;
-					}
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
@@ -309,7 +306,7 @@ int main (int argc, char **argv)
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_data);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_data);
 			exit (EXIT_FAILURE);
 		}
 
@@ -318,7 +315,7 @@ int main (int argc, char **argv)
                 n_data++;
 
                 if (n_data == n_alloc) {
-                        n_alloc += GMT_CHUNK;
+                        n_alloc <<= 1;
 			x = (float *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(float), GMT_program);
 			if (C.y_given) y = (float *)GMT_memory ((void *)y, (size_t)n_alloc, sizeof(float), GMT_program);
                 }
@@ -326,7 +323,7 @@ int main (int argc, char **argv)
         if (fp != GMT_stdin) GMT_fclose(fp);
 	x = (float *) GMT_memory ((void *)x, (size_t)n_data, sizeof(float), GMT_program);
 	if (C.y_given) y = (float *)GMT_memory ((void *)y, (size_t)n_data, sizeof(float), GMT_program);
-	if (gmtdefs.verbose) fprintf(stderr,"Read %d data points.\n", n_data);
+	if (gmtdefs.verbose) fprintf(stderr,"Read %ld data points.\n", n_data);
 
 	compute_spectra (&C, x, y, n_data);
 
@@ -353,10 +350,10 @@ void alloc_arrays (struct SPECTRUM1D_INFO *C)
 	C->datac = (float *) GMT_memory (VNULL, (size_t)C->window_2, sizeof(float), GMT_program);
 }
 
-void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
+void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, GMT_LONG n_data)
 {
-	int	n_windows, w, i, t_start, t_stop, t, f;
-	int	narray;
+	GMT_LONG	n_windows, w, i, t_start, t_stop, t, f;
+	GMT_LONG	narray;
 	float	work = 0.0;
 	double	dw, spec_scale, x_varp, y_varp = 1.0, one_on_nw, co_quad;
 	double	xreal, ximag, yreal, yimag, xpower, ypower, co_spec, quad_spec;
@@ -381,7 +378,7 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 	dw = (n_windows > 1) ? (double)(n_data - C->window) / (double)(n_windows - 1) : 1.0;
 
 	for (w = 0; w < n_windows; w++) {
-		t_start = (int)floor (0.5 + w * dw);
+		t_start = (GMT_LONG)floor (0.5 + w * dw);
 		t_stop = t_start + C->window;
 		if (C->y_given) {
 			for (t = t_start, i = 0; t < t_stop; t++, i+=2) {
@@ -440,7 +437,7 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 
 		if (gmtdefs.verbose) {
 			C->y_pow = (C->y_given) ? C->y_variance/y_varp : 0.0;
-			fprintf(stderr,"Window %d from %d to %d\n", w, t_start, t_stop);
+			fprintf(stderr,"Window %ld from %ld to %ld\n", w, t_start, t_stop);
 			sprintf(format, "X var:  %s  X pow:  %s  ratio:  %s  Y var:  %s  Y pow:  %s  ratio:  %s\n",
 				gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 			fprintf(stderr, format, C->x_variance, x_varp, (C->x_variance/x_varp), C->y_variance, y_varp, C->y_pow);
@@ -471,7 +468,7 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 
 void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN write_wavelength, char *namestem)
 {
-	int	i, j;
+	GMT_LONG	i, j;
 	double	delta_f, eps_pow, out[3], *f_or_w;
 	char	fname[GMT_LONG_TEXT];
 	FILE	*fpout;
@@ -627,7 +624,7 @@ void free_space (struct SPECTRUM1D_INFO *C)
 
 void detrend_and_hanning (struct SPECTRUM1D_INFO *C)
 {
-	int	i, t;
+	GMT_LONG	i, t;
 	double	sumx, sumtx, sumy, sumty, sumt2, x_slope, x_mean, y_slope, y_mean;
 	double	t_factor, h_period, h_scale, hc, hw, tt;
 	sumx = 0.0;
@@ -693,7 +690,7 @@ void detrend_and_hanning (struct SPECTRUM1D_INFO *C)
 void *New_Spectrum1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SPECTRUM1D_CTRL *C;
 	
-	C = (struct SPECTRUM1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SPECTRUM1D_CTRL), "New_Spectrum1d_Ctrl");
+	C = (struct SPECTRUM1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPECTRUM1D_CTRL), "New_Spectrum1d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.inc = 1.0;
@@ -710,6 +707,6 @@ void *New_Spectrum1d_Ctrl () {	/* Allocate and initialize a new control structur
 }
 
 void Free_Spectrum1d_Ctrl (struct SPECTRUM1D_CTRL *C) {	/* Deallocate control structure */
-	if (C->N.name) GMT_free ((void *)C->N.name);	
+	if (C->N.name) free ((void *)C->N.name);	
 	GMT_free ((void *)C);	
 }
