@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: splitxyz.c,v 1.38 2007/03/12 19:52:26 remko Exp $
+ *	$Id: splitxyz.c,v 1.43 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -78,8 +78,8 @@ struct  DATA    {
 
 int main(int argc, char **argv)
 {
-	int     i, j, k, ndata, n_alloc, nprofiles, begin, end, n_expected_fields, n_outputs = 0, n_fields;
-	int	n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES];
+	GMT_LONG     i, ndata, n_alloc, nprofiles, begin, end;
+	int	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_fields;
 
 	BOOLEAN	error = FALSE, ok, hilow = FALSE, z_selected = FALSE;
 
@@ -93,7 +93,7 @@ int main(int argc, char **argv)
 
 	FILE    *fp = NULL, *fpout;
 
-	void	filterz(struct DATA *data, int ndata, double z_filter, double *fwork, int hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, int begin, int end, double xy_filter, double *fwork, int hilow);
+	void	filterz(struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, int hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, int hilow);
 	void *New_Splitxyz_Ctrl (), Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -355,13 +355,13 @@ int main(int argc, char **argv)
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, i);
 			exit (EXIT_FAILURE);
 
 		}
 		i++;
                 if (ndata == n_alloc) {
-                        n_alloc += GMT_CHUNK;
+                        n_alloc <<= 1;
                         data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
                 }
 		if (!Ctrl->S.active) {
@@ -486,14 +486,14 @@ int main(int argc, char **argv)
 					/* If (Ctrl->N.name) we write separate files, else GMT_stdout with > marks:  */
 
 					if (Ctrl->N.name) {
-						sprintf(filename, "%s.profile%d", Ctrl->N.name, nprofiles);
+						sprintf(filename, "%s.profile%ld", Ctrl->N.name, nprofiles);
 						if ( (fpout = GMT_fopen(filename, GMT_io.w_mode)) == NULL) {
 							fprintf(stderr,"%s:  Cannot create %s\n", GMT_program, argv[i]);
 							exit (EXIT_FAILURE);
 						}
 					}
 					else
-						fprintf (GMT_stdout, "> Start profile # %d\n", nprofiles);
+						fprintf (GMT_stdout, "> Start profile # %ld\n", nprofiles);
 
 					for (i = begin; i < end; i++) {
 						for (j = 0; j < n_outputs; j++) {	/* Remember to convert CCW angles back to azimuths */
@@ -504,7 +504,7 @@ int main(int argc, char **argv)
 
 					if (fpout != GMT_stdout) {
 						GMT_fclose (fpout);
-						if (gmtdefs.verbose) fprintf (stderr,"%s: Wrote %d points to file %s\n", GMT_program, end-begin, filename);
+						if (gmtdefs.verbose) fprintf (stderr,"%s: Wrote %ld points to file %s\n", GMT_program, end-begin, filename);
 					}
 				}
 			}
@@ -514,7 +514,7 @@ int main(int argc, char **argv)
 
 	/* Get here when all profiles have been found and written.  */
 
-	if (gmtdefs.verbose) fprintf(stderr,"%s:  Split %d data into %d files.\n", GMT_program, ndata, nprofiles);
+	if (gmtdefs.verbose) fprintf(stderr,"%s:  Split %ld data into %ld files.\n", GMT_program, ndata, nprofiles);
 	GMT_free ((void *)data);
 
 	Free_Splitxyz_Ctrl (Ctrl);	/* Deallocate control structure */
@@ -525,11 +525,11 @@ int main(int argc, char **argv)
 }
 
 
-void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int hilow)
+void filterz (struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, int hilow)
 {
 
 	double	half_width, sum, dt, tmp;
-	int	i, j, k, istart, istop;
+	GMT_LONG	i, j, k, istart, istop;
 
 	half_width = 0.5 * z_filter;
 	sum = 0.0;
@@ -552,7 +552,7 @@ void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[2] * fwork[k]);
 		}
@@ -570,7 +570,7 @@ void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int
 
 void filterxy_setup (double *fwork)
 {
-	int	i;
+	GMT_LONG	i;
 	double	tmp, sum = 0.0;
 
 	tmp = M_PI / SPLITXYZ_F_RES;
@@ -584,9 +584,9 @@ void filterxy_setup (double *fwork)
 	return;
 }
 
-void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *fwork, int hilow)
+void filterxy (struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, int hilow)
 {
-	int	i, j, k, istart, istop;
+	GMT_LONG	i, j, k, istart, istop;
 	double	half_width, dt, sum;
 
 	half_width = 0.5 * fabs(xy_filter);
@@ -601,7 +601,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[0] * fwork[k]);
 		}
@@ -624,7 +624,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[1] * fwork[k]);
 		}
@@ -644,7 +644,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 void *New_Splitxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SPLITXYZ_CTRL *C;
 	
-	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SPLITXYZ_CTRL), "New_Splitxyz_Ctrl");
+	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPLITXYZ_CTRL), "New_Splitxyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
         C->A.azimuth = 90.0;
@@ -654,5 +654,6 @@ void *New_Splitxyz_Ctrl () {	/* Allocate and initialize a new control structure
 }
 
 void Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C) {	/* Deallocate control structure */
+	if (C->N.name) free ((void *)C->N.name);	
 	GMT_free ((void *)C);	
 }
