@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: backtracker.c,v 1.31 2007/03/12 19:52:27 remko Exp $
+ *	$Id: backtracker.c,v 1.36 2008/03/24 08:58:33 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -66,20 +66,20 @@ int main (int argc, char **argv)
 {
 	struct EULER *p;		/* Pointer to array of stage poles */
 
-	int n_points;			/* Number of data points read */
-	int n_chunk;			/* Total length or array returned by libeuler functions */
-	int n_track;			/* Number of points in a track segment */
-	int n_stages;			/* Number of stage poles */
-	int n_segments;			/* Number of path segments written out */
-	int n_skipped = 0;		/* Number of points skipped because t < 0 */
+	GMT_LONG n_points;			/* Number of data points read */
+	GMT_LONG n_chunk;			/* Total length or array returned by libeuler functions */
+	GMT_LONG n_track;			/* Number of points in a track segment */
+	GMT_LONG n_stages;			/* Number of stage poles */
+	GMT_LONG n_segments;			/* Number of path segments written out */
+	GMT_LONG n_skipped = 0;		/* Number of points skipped because t < 0 */
 	int track_limit = 0;		/* Flag for limiting the output track extent [no limit] */
 	int n_args;
 	int n_files = 0, fno;
 	int n_fields, n_expected_fields;
-	int n_read = 0;
+	GMT_LONG n_read = 0;
 	int n_out;
 
-	int i, j, k;			/* Misc. counters */
+	GMT_LONG i, j, k;			/* Misc. counters */
 
 	BOOLEAN error = FALSE;		/* Set to TRUE if arguments are inconsistent */
 	BOOLEAN flowline = FALSE;	/* TRUE means we want flowlines, FALSE we want hotspot tracks */
@@ -122,7 +122,7 @@ int main (int argc, char **argv)
 	argc = GMT_begin (argc, argv);
 
 #ifdef DEBUG
-	fprintf (stderr, "DEBUG mode\n");
+	if (gmtdefs.verbose) fprintf (stderr, "DEBUG mode\n");
 #endif
 	GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
 	GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
@@ -374,7 +374,7 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 			if (fix_all) in[2] = t_fix;
@@ -395,7 +395,7 @@ int main (int argc, char **argv)
 				age = in[2];
 
 			if (age > upper_age) {	/* Points older than oldest stage cannot be used */
-				fprintf (stderr, "%s: Seamount at line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, in[2], upper_age);
+				fprintf (stderr, "%s: Seamount at line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, in[2], upper_age);
 				n_skipped++;
 				continue;
 			}
@@ -404,7 +404,7 @@ int main (int argc, char **argv)
 
 			if (make_path) {	/* Asked for paths, now write out several multiple segment tracks */
 				if (seg_files) {
-					sprintf (buffer, "%s.%d", file_stem, n_points);
+					sprintf (buffer, "%s.%ld", file_stem, n_points);
 					if ((fpo = GMT_fopen (buffer, GMT_io.w_mode)) == NULL) {
 						fprintf (stderr, "%s: Error - cannot create file %s\n", GMT_program, buffer);
 						exit (EXIT_FAILURE);
@@ -449,15 +449,18 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) {
 		if (make_path)
-			fprintf (stderr, "%s: %d segments written\n", GMT_program, n_points);
+			fprintf (stderr, "%s: %ld segments written\n", GMT_program, n_points);
 		else
-			fprintf (stderr, "%s: %d points projected\n", GMT_program, n_points);
+			fprintf (stderr, "%s: %ld points projected\n", GMT_program, n_points);
 	}
 
-	if (gmtdefs.verbose && n_skipped) fprintf (stderr, "%s: %d points skipped because age < 0\n", GMT_program, n_skipped);
+	if (gmtdefs.verbose && n_skipped) fprintf (stderr, "%s: %ld points skipped because age < 0\n", GMT_program, n_skipped);
 
 	/* Clean up and exit */
 
+	GMT_free ((void *)p);
+	if (out) GMT_free ((void *)out);
+	
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
