@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdrotater.c,v 1.20 2007/03/12 19:52:27 remko Exp $
+ *	$Id: grdrotater.c,v 1.26 2008/03/24 08:58:33 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *   Contact info: www.soest.hawaii.edu/pwessel
  *--------------------------------------------------------------------*/
 /*
- * grdrotater will read a grdfile, apply a finite rotation to the grid
+ * grdrotater will read a grid file, apply a finite rotation to the grid
  * coordinates, and then interpolate the old grid at the new coordinates.
  *
  * Author:	Paul Wessel
@@ -29,10 +29,10 @@
 
 int main (int argc, char **argv)
 {
-	int i, j, k, ij, ii, jj, nx, ny, mx = 0, my, ix, iy;
-	int iii, jjj, ii_o, jj_o;
+	GMT_LONG i, j, k, ij, ii, jj, nx, ny, mx = 0, my, ix, iy;
+	GMT_LONG iii, jjj, ii_o, jj_o;
 
-	BOOLEAN error = FALSE, bilinear = FALSE, user_polygon_given = FALSE;
+	BOOLEAN error = FALSE, interpolant = BCR_BICUBIC, user_polygon_given = FALSE;
 	BOOLEAN inside, node = FALSE, outline = TRUE, skip_grid = FALSE;
 	
 	float *original_grid = VNULL, *rotated_grid;
@@ -95,8 +95,24 @@ int main (int argc, char **argv)
 					outline = FALSE;
 					break;
 				case 'Q':
-					bilinear = TRUE;
-					threshold = (argv[i][2]) ? atof (&argv[i][2]) : 1.0;
+					interpolant = BCR_BILINEAR;
+					for (j = 2; j < 5 && argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case 'n':
+								interpolant = BCR_NEARNEIGHBOR; break;
+							case 'l':
+								interpolant = BCR_BILINEAR; break;
+							case 'b':
+								interpolant = BCR_BSPLINE; break;
+							case 'c':
+								interpolant = BCR_BICUBIC; break;
+							case '/':
+							default:
+								threshold = atof (&argv[i][j]);
+								if (j == 2 && threshold < GMT_SMALL) interpolant = BCR_NEARNEIGHBOR;
+								j = 5; break;
+						}
+					}
 					break;
 				case 'S':
 					skip_grid = TRUE;
@@ -115,7 +131,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdrotater %s - Finite rotation reconstruction of geographic grid\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdrotater <grdfile> -T<plon>/<plat>/<pomega> -G<rotgrid> [-F<polygonfile>]\n");
-		fprintf (stderr, "\t[%s] [%s] [-N] [-Q[<value>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-N] [-Q[b|c|l|n][[/]<threshold>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
 		fprintf (stderr, "\t[%s] > projpol\n\n", GMT_b_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -131,11 +147,12 @@ int main (int argc, char **argv)
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Do not output the rotated polygon or grid outline\n");
 		fprintf (stderr, "\t-Q Quick mode, use bilinear rather than bicubic [Default] interpolation.\n");
-		fprintf (stderr, "\t   Optionally, append <value> in the 0 <= value <= 1 range.\n");
-		fprintf (stderr, "\t   [Default = 1 requires all 4 nodes to be non-NaN.], <value> = 0.5\n");
-		fprintf (stderr, "\t   will interpolate about 1/2 way from a non-NaN to a NaN node, while\n");
-		fprintf (stderr, "\t   0.1 will go about 90%% of the way, etc. -Q0 will return the value of the \n");
-		fprintf (stderr, "\t   nearest node instead of interpolating.\n");
+		fprintf (stderr, "\t   Alternatively, select interpolation mode by adding b = B-spline, c = bicubic,\n");
+		fprintf (stderr, "\t   l = bilinear, or n = nearest-neighbor.\n");
+		fprintf (stderr, "\t   Optionally, append <threshold> in the range [0,1]. [Default = 1 requires all\n");
+		fprintf (stderr, "\t   4 or 16 nodes to be non-NaN.], <threshold> = 0.5 will interpolate about 1/2 way\n");
+		fprintf (stderr, "\t   from a non-NaN to a NaN node, while 0.1 will go about 90%% of the way, etc.\n");
+		fprintf (stderr, "\t   -Q0 will return the value of the nearest node instead of interpolating (Same as -Qn).\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S Do NOT rotate the grid - just produce the rotated outline\n");
 		GMT_explain_option ('V');
@@ -219,13 +236,13 @@ int main (int argc, char **argv)
 		GMT_err_fail (GMT_read_grd (grdfile, &g_head, original_grid, data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile);
 
 		GMT_boundcond_init (&edgeinfo);
-		if (fabs (fabs (g_head.x_max - g_head.x_min) - 360.0) < GMT_CONV_LIMIT) GMT_boundcond_parse (&edgeinfo, "g");
+		if (GMT_360_RANGE (g_head.x_max, g_head.x_min)) GMT_boundcond_parse (&edgeinfo, "g");
 	
 		GMT_boundcond_param_prep (&g_head, &edgeinfo);
 
 		/* Initialize bcr structure:  */
 
-		GMT_bcr_init (&g_head, GMT_pad, bilinear, threshold, &bcr);
+		GMT_bcr_init (&g_head, GMT_pad, interpolant, threshold, &bcr);
 		if (threshold == 0.0) node = TRUE;
 
 		/* Set boundary conditions  */
@@ -423,28 +440,28 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	 * Note that the path is the same for pixel or grid-registered grids.
 	 */
 
-	int np = 0, add, i, j;
+	GMT_LONG np = 0, add, i, j;
 	struct GMT_TABLE *pol;
 	
-	pol = (struct GMT_TABLE *)GMT_memory (VNULL, 1, sizeof (struct GMT_TABLE), GMT_program);
-	pol->segment = (struct GMT_LINE_SEGMENT **) GMT_memory (VNULL, 1, sizeof (struct GMT_LINE_SEGMENT *), GMT_program);
-	pol->segment[0] = (struct GMT_LINE_SEGMENT *) GMT_memory (VNULL, 1, sizeof (struct GMT_LINE_SEGMENT), GMT_program);
-	pol->segment[0]->coord = (double **)GMT_memory (VNULL, 2, sizeof (double *), GMT_program);
-	pol->segment[0]->min = (double *)GMT_memory (VNULL, 2, sizeof (double), GMT_program);
-	pol->segment[0]->max = (double *)GMT_memory (VNULL, 2, sizeof (double), GMT_program);
+	pol = (struct GMT_TABLE *)GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_TABLE), GMT_program);
+	pol->segment = (struct GMT_LINE_SEGMENT **) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_LINE_SEGMENT *), GMT_program);
+	pol->segment[0] = (struct GMT_LINE_SEGMENT *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_LINE_SEGMENT), GMT_program);
+	pol->segment[0]->coord = (double **)GMT_memory (VNULL, (size_t)2, sizeof (double *), GMT_program);
+	pol->segment[0]->min = (double *)GMT_memory (VNULL, (size_t)2, sizeof (double), GMT_program);
+	pol->segment[0]->max = (double *)GMT_memory (VNULL,(size_t) 2, sizeof (double), GMT_program);
 		
 	/* Add south border w->e */
 	if (h->y_min == -90.0) {	/* If at the S pole we just add it twice for end longitudes */
 		add = 2;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
 		pol->segment[0]->coord[GMT_X][0] = h->x_min;	pol->segment[0]->coord[GMT_X][1] = h->x_max;
 		pol->segment[0]->coord[GMT_Y][0] = pol->segment[0]->coord[GMT_Y][1] = h->y_min;
 	}
 	else {				/* Loop along south border from west to east */
 		add = h->nx - !h->node_offset;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
 		for (i = 0; i < add; i++) {
 			pol->segment[0]->coord[GMT_X][i] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
 			pol->segment[0]->coord[GMT_Y][i] = h->y_min;
@@ -453,8 +470,8 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	np += add;
 	/* Add east border s->n */
 	add = h->ny - !h->node_offset;
-	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 	for (j = 0; j < add; j++) {	/* Loop along east border from south to north */
 		pol->segment[0]->coord[GMT_X][np+j] = h->x_max;
 		pol->segment[0]->coord[GMT_Y][np+j] = GMT_j_to_y (h->ny - 1 - j, h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
@@ -463,15 +480,15 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	/* Add north border e->w */
 	if (h->y_max == 90.0) {	/* If at the N pole we just add it twice for end longitudes */
 		add = 2;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 		pol->segment[0]->coord[GMT_X][np] = h->x_max;	pol->segment[0]->coord[GMT_X][np+1] = h->x_min;
 		pol->segment[0]->coord[GMT_Y][np] = pol->segment[0]->coord[GMT_Y][np+1] = h->y_max;
 	}
 	else {			/* Loop along north border from east to west */
 		add = h->nx - !h->node_offset;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 		for (i = 0; i < add; i++) {
 			pol->segment[0]->coord[GMT_X][np+i] = GMT_i_to_x (h->nx - 1 - i, h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
 			pol->segment[0]->coord[GMT_Y][np+i] = h->y_max;
@@ -480,8 +497,8 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	np += add;
 	/* Add west border n->s */
 	add = h->ny - !h->node_offset;
-	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np + 1, sizeof (double), GMT_program);
-	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np + 1, sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np + 1), sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np + 1), sizeof (double), GMT_program);
 	for (j = 0; j < add; j++) {	/* Loop along west border from north to south */
 		pol->segment[0]->coord[GMT_X][np+j] = h->x_min;
 		pol->segment[0]->coord[GMT_Y][np+j] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
