@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: hotspotter.c,v 1.28 2007/03/12 19:52:27 remko Exp $
+ *	$Id: hotspotter.c,v 1.35 2008/04/25 04:00:41 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -110,32 +110,32 @@
 int main (int argc, char **argv)
 {
 
-	int n_smts;		/* Number of seamounts read */
-	int n_stages;		/* Number of stage rotations (poles) */
-	int n_chunk;		/* Number of path values returned by libspotter functions */
-	int n_track;		/* Number of points along a single flowline */
-	int node_x_width;	/* Number of x-nodes covered by the seamount in question (y-dependent) */
-	int node_y_width;	/* Number of y-nodes covered by the seamount */
-	int n_args;		/* Command line argument counter */
+	GMT_LONG n_smts;		/* Number of seamounts read */
+	GMT_LONG n_stages;		/* Number of stage rotations (poles) */
+	GMT_LONG n_chunk;		/* Number of path values returned by libspotter functions */
+	GMT_LONG n_track;		/* Number of points along a single flowline */
+	GMT_LONG node_x_width;	/* Number of x-nodes covered by the seamount in question (y-dependent) */
+	GMT_LONG node_y_width;	/* Number of y-nodes covered by the seamount */
+	GMT_LONG n_args;		/* Command line argument counter */
 	int n_files = 0;	/* Number of input files on the command line */
-	int node;		/* The current node index */
+	GMT_LONG node;		/* The current node index */
 	int one_or_zero;	/* Will be 1 or 0 depending on grid format (-F) */
 	int n_fields, n_expected_fields;
-	int n_read = 0;		/* Number of records read */
+	GMT_LONG n_read = 0;		/* Number of records read */
 
 	size_t n_nodes;		/* Number of nodes in the output CVA grid */
 
 	/* Misc. counters */
 
-	int i, j, kx, ky, m, ii, jj, i0, j0, k0;
+	GMT_LONG i, j, kx, ky, m, ii, jj, i0, j0, k0;
 	int dummy[4], fno;
 
 	BOOLEAN	error = FALSE;		/* TRUE when arguments are wrong */
 	BOOLEAN nofile = TRUE;		/* TRUE when data are to be read from stdin */
 	BOOLEAN truncate_ages = FALSE;	/* Truncate all seamounts ages > upper_age */
 	BOOLEAN finite = FALSE;		/* TRUE if stage pole file contains finite rotation poles instead */
+	BOOLEAN normalize = FALSE;	/* TRUE if we want CVA in percents */
 	BOOLEAN done;
-	BOOLEAN resize = FALSE;
 
 	float *CVA;			/* The CVA surface we seek to calculate */
 
@@ -227,7 +227,7 @@ int main (int argc, char **argv)
 					upper_age = atof (&argv[i][2]);
 					break;
 				case 'S':
-					resize = TRUE;
+					normalize = TRUE;
 					break;
 				case 'T':
 					truncate_ages = TRUE;
@@ -245,7 +245,8 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Create CVA image from seamount locations\n\n", GMT_program, SPOTTER_VERSION);
 		fprintf (stderr, "usage: %s [<files>] -E<stage_file> -G<CVAgrid> %s\n", GMT_program, GMT_Id_OPT);
-		fprintf (stderr, "\t%s [-C] [-D<factor>] [-F] [%s] [-N<upper_age>] [-T] [-V] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_H_OPT, GMT_t_OPT, GMT_bi_OPT);
+		fprintf (stderr, "\t%s [-C] [-D<factor>] [-F] [%s] [-N<upper_age>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[-S] [-T] [-V] [%s] [%s]\n\n", GMT_t_OPT, GMT_bi_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -259,6 +260,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D Scale affecting distance between points along flowline [0.5]\n");
 		fprintf (stderr, "\t-F Force pixel-registration [Default is gridline registration]\n");
 		fprintf (stderr, "\t-N sets upper age in m.y. for seamounts whose plate age is NaN [180]\n");
+		fprintf (stderr, "\t-S Normalize CVA grid to percentages of the CVA maximum\n");
 		fprintf (stderr, "\t-T truncate all ages to max age in stage pole model [Default extrapolates]\n");
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
@@ -399,7 +401,7 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 
@@ -409,10 +411,6 @@ int main (int argc, char **argv)
 			y_smt = in[1] * D2R;
 			z_smt = in[2];
 			r_smt = in[3];
-			if (resize) {
-				z_smt = 1.0;
-				/* r_smt = 2.0 * yinc_r * EQ_RAD; */
-			}
 			if (GMT_is_dnan (in[4])) {	/* Age is NaN, assign value */
 				t_smt = upper_age;
 			}
@@ -423,7 +421,7 @@ int main (int argc, char **argv)
 						t_smt = upper_age;
 					}
 					else {
-						fprintf (stderr, "%s: Seamounts near line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
+						fprintf (stderr, "%s: Seamounts near line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
 						continue;
 					}
 				}
@@ -433,11 +431,11 @@ int main (int argc, char **argv)
 
 			r_smt /= EQ_RAD;				/* Converts radius in km to radians */
 			norm = -4.5 / (r_smt * r_smt);			/* Gaussian normalization */
-			node_y_width = (int)ceil (i_yinc_r * r_smt);	/* y-node coverage */
+			node_y_width = (GMT_LONG)ceil (i_yinc_r * r_smt);	/* y-node coverage */
 
 			/* STEP 2: Calculate this seamount's flowline */
 
-			n_chunk = spotter_forthtrack (&x_smt, &y_smt, &t_smt, 1, p, n_stages, sampling_int_in_km, 0.0, FALSE, NULL, &c);
+			n_chunk = spotter_forthtrack (&x_smt, &y_smt, &t_smt, (GMT_LONG)1, p, n_stages, sampling_int_in_km, 0.0, FALSE, NULL, &c);
 
 			/* STEP 3: Convolve this flowline with seamount shape and add to CVA grid */
 
@@ -464,15 +462,15 @@ int main (int argc, char **argv)
 
 				/* OK, this point is within our region, get node index */
 
-				i = (int) floor (((c[kx] - west)  * i_xinc_r) + offset);
-				j = (int) floor (((north - c[ky]) * i_yinc_r) + offset);
+				i = (GMT_LONG) floor (((c[kx] - west)  * i_xinc_r) + offset);
+				j = (GMT_LONG) floor (((north - c[ky]) * i_yinc_r) + offset);
 				node = j * CVA_map.nx + i;
 
 				if (!processed_node[node]) {	/* Have not added to the CVA at this node yet */
 
 					/* Shape is z_smt * exp (r^2 * norm) */
 
-					node_x_width = (int) ceil (r_smt * ilatfactor[j]);
+					node_x_width = (GMT_LONG) ceil (r_smt * ilatfactor[j]);
 					dx = c[kx] - xpos[i];
 					dy = c[ky] - ypos[j];
 
@@ -503,7 +501,7 @@ int main (int argc, char **argv)
 
 			n_smts++;	/* Go to next seamount */
 
-			if (gmtdefs.verbose && !(n_smts%100)) fprintf (stderr, "%s: Processed %5d seamounts\r", GMT_program, n_smts);
+			if (gmtdefs.verbose && !(n_smts%100)) fprintf (stderr, "%s: Processed %5ld seamounts\r", GMT_program, n_smts);
 		}
 
 		if (fp != stdin) GMT_fclose (fp);
@@ -511,8 +509,27 @@ int main (int argc, char **argv)
 
 	/* OK, Done processing, time to write out */
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed %5d seamounts\n", GMT_program, n_smts);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed %5ld seamounts\n", GMT_program, n_smts);
 
+	if (normalize) {	/* Convert CVA values to percent of CVA maximum */
+		size_t k;
+		double scale;
+		
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Normalize CVS grid to percentages of max CVA\n", GMT_program);
+		CVA_map.z_min = +DBL_MAX;
+		CVA_map.z_max = -DBL_MAX;
+		for (k = 0; k < n_nodes; k++) {
+			if (GMT_is_fnan (CVA[k])) continue;
+			if (CVA[k] < CVA_map.z_min) CVA_map.z_min = CVA[k];
+			if (CVA[k] > CVA_map.z_max) CVA_map.z_max = CVA[k];
+		}
+		if (gmtdefs.verbose) fprintf (stderr, "%s: CVA min/max: %g %g -> ", GMT_program, CVA_map.z_min, CVA_map.z_max);
+		scale = 100.0 / CVA_map.z_max;
+		for (k = 0; k < n_nodes; k++) CVA[k] *= (float)scale;
+		CVA_map.z_min *= scale;
+		CVA_map.z_max *= scale;
+		if (gmtdefs.verbose) fprintf (stderr, "%g %g\n", CVA_map.z_min, CVA_map.z_max);
+	}
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Write CVA grid %s\n", GMT_program, CVA_file);
 
 	dummy[0] = dummy[1] = dummy[2] = dummy[3] = 0;	/* No grid boundary padding today */
@@ -527,6 +544,7 @@ int main (int argc, char **argv)
 	GMT_free ((void *)ilatfactor);
 	GMT_free ((void *)xpos);
 	GMT_free ((void *)ypos);
+	GMT_free ((void *)p);
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done\n", GMT_program);
 
