@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: libspotter.c,v 1.38 2007/03/05 21:47:11 pwessel Exp $
+ *	$Id: libspotter.c,v 1.44 2008/05/12 22:35:47 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -53,13 +53,13 @@ void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w);
 void matrix_transpose (double At[3][3], double A[3][3]);
 void matrix_mult (double a[3][3], double b[3][3], double c[3][3]);
 void make_rot_matrix_sub (double E[3], double w, double R[3][3]);
-void reverse_rotation_order (struct EULER *p, int n);
-void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[], int n, BOOLEAN stages, BOOLEAN convert);
+void reverse_rotation_order (struct EULER *p, GMT_LONG n);
+void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[], GMT_LONG n, BOOLEAN stages, BOOLEAN convert);
 void set_I_matrix (double R[3][3]);
-BOOLEAN must_do_track (int sideA[], int sideB[]);
-void set_inout_sides (double x, double y, double wesn[], int sideXY[2]);
+BOOLEAN must_do_track (GMT_LONG sideA[], GMT_LONG sideB[]);
+void set_inout_sides (double x, double y, double wesn[], GMT_LONG sideXY[2]);
 
-void spotter_finite_to_fwstages (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates);
+void spotter_finite_to_fwstages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates);
 
 int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in, BOOLEAN finite_out, double *t_max, BOOLEAN verbose)
 {
@@ -72,10 +72,10 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	FILE *fp;
 	struct EULER *e;
 	char  buffer[BUFSIZ];
-	int n, nf, i = 0, j, k, n_alloc = GMT_SMALL_CHUNK;
+	GMT_LONG n, nf, i = 0, j, k, n_alloc = GMT_SMALL_CHUNK;
 	double last_t, K[8];
 
-	e = (struct EULER *) GMT_memory (VNULL, n_alloc, sizeof (struct EULER), "libspotter");
+	e = (struct EULER *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct EULER), "libspotter");
 
 	if ((fp = GMT_fopen (file, "r")) == NULL) {
 		fprintf (stderr, "libspotter: ERROR: Cannot open stage pole file: %s\n", file);
@@ -85,7 +85,7 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	if (flowline) finite_out = TRUE;	/* Override so we get finite poles for conversion to forward stage poles at the end */
 
 	last_t = (finite_in) ? 0.0 : DBL_MAX;
-	while (fgets (buffer, 512, fp) != NULL) { /* Expects lon lat t0 t1 ccw-angle */
+	while (GMT_fgets (buffer, 512, fp) != NULL) { /* Expects lon lat t0 t1 ccw-angle */
 		if (buffer[0] == '#' || buffer[0] == '\n') continue;
 
 		if (finite_in) {	/* The record formats is: lon lat t0 [t1] omega [covar] */
@@ -114,7 +114,7 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 
 
 		if (e[i].t_stop >= e[i].t_start) {
-			fprintf (stderr, "libspotter: ERROR: Stage rotation %d has start time younger than stop time\n", i);
+			fprintf (stderr, "libspotter: ERROR: Stage rotation %ld has start time younger than stop time\n", i);
 			GMT_exit (EXIT_FAILURE);
 		}
 		e[i].duration = e[i].t_start - e[i].t_stop;
@@ -140,11 +140,11 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 		e[i].lat_r = e[i].lat * D2R;
 		i++;
 		if (i == n_alloc) {
-			n_alloc += GMT_SMALL_CHUNK;
-			e = (struct EULER *) GMT_memory ((void *)e, n_alloc, sizeof (struct EULER), "libspotter");
+			n_alloc <<= 1;
+			e = (struct EULER *) GMT_memory ((void *)e, (size_t)n_alloc, sizeof (struct EULER), "libspotter");
 		}
 	}
-	fclose (fp);
+	GMT_fclose (fp);
 
 	n = i;
 
@@ -152,7 +152,7 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	if (!finite_in && finite_out) spotter_stages_to_finite (e, n, TRUE, TRUE);	/* Convert backward stage poles to finite poles */
 
 
-	e = (struct EULER *) GMT_memory ((void *)e, n, sizeof (struct EULER), "libspotter");
+	e = (struct EULER *) GMT_memory ((void *)e, (size_t)n, sizeof (struct EULER), "libspotter");
 
 	if (flowline) {	/* Get the forward stage poles from the total reconstruction poles */
 		spotter_finite_to_fwstages (e, n, TRUE, TRUE);
@@ -178,7 +178,7 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
  *	age t_zero.  For t_zero = 0 this means the hotspot
  */
 
-int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EULER p[], int ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c)
+int spotter_backtrack (double xp[], double yp[], double tp[], GMT_LONG np, struct EULER p[], GMT_LONG ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c)
 /* xp, yp;	Points, in RADIANS */
 /* tp;		Age of feature in m.y. */
 /* np;		# of points */
@@ -190,7 +190,7 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 /* wesn:	if do_time >= 10, only to track within the given box */
 /* **c;		Pointer to return track vector */
 {
-	int i, j = 0, k, kk = 0, start_k = 0, nd = 1, nn, n_alloc = 2 * GMT_CHUNK, sideA[2], sideB[2];
+	GMT_LONG i, j = 0, k, kk = 0, start_k = 0, nd = 1, nn, n_alloc = 2 * GMT_CHUNK, sideA[2], sideB[2];
 	BOOLEAN path, bend, go = FALSE, box_check;
 	double t, tt = 0.0, dt, d_lon, tlon, dd = 0.0, i_km = 0.0, xnew, xx, yy, next_x, next_y;
 	double s_lat, c_lat, s_lon, c_lon, cc, ss, cs, i_nd, *track = VNULL;
@@ -207,7 +207,7 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 	}
 
 	if (path) {
-		track = (double *) GMT_memory (VNULL, n_alloc, sizeof (double), "libspotter");
+		track = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "libspotter");
 		i_km = EQ_RAD / d_km;
 	}
 
@@ -218,7 +218,7 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 		if (path) {
 			start_k = kk++;
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 		}
@@ -266,25 +266,25 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 			}
 			if (path) {
 				if (!bend) {
-					nd = (int) ceil ((fabs (d_lon) * c_lat) * i_km);
+					nd = (GMT_LONG) ceil ((fabs (d_lon) * c_lat) * i_km);
 					i_nd = 1.0 / nd;
 					dd = d_lon * i_nd;
 					tt = dt * i_nd;
 				}
 				track[kk++] = xp[i];
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 				track[kk++] = yp[i];
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 				if (do_time) {
 					track[kk++] = (do_time == 2) ? (double)(ns - j) : t;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 				}
@@ -301,18 +301,18 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 					if (xx >= TWO_PI) xx -= TWO_PI;
 					track[kk++] = xx;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 					track[kk++] = yy;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 					if (do_time) {
 						track[kk++] = (do_time == 2) ? (double)(ns - j) : t - k * tt;
 						if (kk == n_alloc) {
-							n_alloc += BIG_CHUNK;
+							n_alloc <<= 1;
 							track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 						}
 					}
@@ -325,18 +325,18 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 		if (path) {
 			track[kk++] = xp[i];
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 			track[kk++] = yp[i];
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 			if (do_time) {
 				track[kk++] = (do_time == 2) ? (double)(ns - j) : t;
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 			}
@@ -352,7 +352,7 @@ int spotter_backtrack (double xp[], double yp[], double tp[], int np, struct EUL
 	return (np);
 }
 
-void set_inout_sides (double x, double y, double wesn[], int sideXY[2]) {
+void set_inout_sides (double x, double y, double wesn[], GMT_LONG sideXY[2]) {
 	/* Given the rectangular region in wesn, return -1, 0, +1 for
 	 * x and y if the point is left/below (-1) in (0), or right/above (+1).
 	 * 
@@ -374,14 +374,19 @@ void set_inout_sides (double x, double y, double wesn[], int sideXY[2]) {
 		sideXY[0] = 0;
 }
 
-BOOLEAN must_do_track (int sideA[], int sideB[]) {
-	int dx, dy;
+BOOLEAN must_do_track (GMT_LONG sideA[], GMT_LONG sideB[]) {
+	GMT_LONG dx, dy;
 	/* First check if any of the two points are inside the box */
 	if (sideA[0] == 0 && sideA[1] == 0) return (TRUE);
 	if (sideB[0] == 0 && sideB[1] == 0) return (TRUE);
 	/* Now check if the two points may cut a corner */
+#ifdef __LP64__
+	dx = labs (sideA[0] - sideB[0]);
+	dy = labs (sideA[1] - sideB[1]);
+#else
 	dx = abs (sideA[0] - sideB[0]);
 	dy = abs (sideA[1] - sideB[1]);
+#endif
 	if (dx && dy) return (TRUE);
 	if (dx == 2 || dy == 2) return (TRUE);	/* COuld cut across the box */
 	return (FALSE);
@@ -392,7 +397,7 @@ BOOLEAN must_do_track (int sideA[], int sideB[]) {
  *	seamount of age tp.  For t_zero = 0 this means from the hotspot.
  */
 
-int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EULER p[], int ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c)
+int spotter_forthtrack (double xp[], double yp[], double tp[], GMT_LONG np, struct EULER p[], GMT_LONG ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c)
 /* xp, yp;	Points, in RADIANS */
 /* tp;		Age of feature in m.y. */
 /* np;		# of points */
@@ -404,7 +409,7 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 /* wesn:	if do_time >= 10, only to track within the given box */
 /* c;		Pointer to return track vector */
 {
-	int i, j = 0, k, kk = 0, start_k = 0, nd = 1, nn, n_alloc = 2 * GMT_CHUNK, sideA[2], sideB[2];
+	GMT_LONG i, j = 0, k, kk = 0, start_k = 0, nd = 1, nn, n_alloc = BIG_CHUNK, sideA[2], sideB[2];
 	BOOLEAN path, bend, go = FALSE, box_check;
 	double t, tt = 0.0, dt, d_lon, tlon, dd = 0.0, i_km = 0.0, xnew, xx, yy, *track = VNULL;
 	double s_lat, c_lat, s_lon, c_lon, cc, ss, cs, i_nd, next_x, next_y;
@@ -421,7 +426,7 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 	}
 
 	if (path) {
-		track = (double *) GMT_memory (VNULL, n_alloc, sizeof (double), "libspotter");
+		track = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "libspotter");
 		i_km = EQ_RAD / d_km;
 	}
 
@@ -432,7 +437,7 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 		if (path) {
 			start_k = kk++;
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 		}
@@ -480,25 +485,25 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 			}
 			if (path) {
 				if (!bend) {
-					nd = (int) ceil ((fabs (d_lon) * c_lat) * i_km);
+					nd = (GMT_LONG) ceil ((fabs (d_lon) * c_lat) * i_km);
 					i_nd = 1.0 / nd;
 					dd = d_lon * i_nd;
 					tt = dt * i_nd;
 				}
 				track[kk++] = xp[i];
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 				track[kk++] = yp[i];
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 				if (do_time) {
 					track[kk++] = (do_time == 2) ? (double)(ns - j) : t;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 				}
@@ -514,18 +519,18 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 					if (xx >= TWO_PI) xx -= TWO_PI;
 					track[kk++] = xx;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 					track[kk++] = yy;
 					if (kk == n_alloc) {
-						n_alloc += BIG_CHUNK;
+						n_alloc <<= 1;
 						track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 					}
 					if (do_time) {
 						track[kk++] = (do_time == 2) ? (double)(ns - j) : t + k * tt;
 						if (kk == n_alloc) {
-							n_alloc += BIG_CHUNK;
+							n_alloc <<= 1;
 							track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 						}
 					}
@@ -539,18 +544,18 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
 		if (path) {
 			track[kk++] = xp[i];
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 			track[kk++] = yp[i];
 			if (kk == n_alloc) {
-				n_alloc += BIG_CHUNK;
+				n_alloc <<= 1;
 				track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 			}
 			if (do_time) {
 				track[kk++] = (do_time == 2) ? (double)(ns - j) : t;
 				if (kk == n_alloc) {
-					n_alloc += BIG_CHUNK;
+					n_alloc <<= 1;
 					track = (double *) GMT_memory ((void *)track, (size_t)n_alloc, sizeof (double), "libspotter");
 				}
 			}
@@ -571,7 +576,7 @@ int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EU
  * Based partly on Cox and Hart, 1986
  */
 
-void spotter_finite_to_fwstages (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates)
+void spotter_finite_to_fwstages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates)
 {
 	/* Convert finite rotations to forward stage rotations for flowlines */
 	/* p[]		: Array of structure elements with rotation parameters
@@ -580,7 +585,7 @@ void spotter_finite_to_fwstages (struct EULER p[], int n, BOOLEAN finite_rates,
 	 * stage_rates	: TRUE if stage rotations should be returned in degree/my [else we return opening angle]
 	 */
 	 
-	int i;
+	GMT_LONG i;
 	double *elon, *elat, *ew, t_old;
 	double R_young[3][3], R_old[3][3], R_stage[3][3];
 
@@ -619,7 +624,7 @@ void spotter_finite_to_fwstages (struct EULER p[], int n, BOOLEAN finite_rates,
 	reverse_rotation_order (p, n);	/* Flip order since stages go from oldest to youngest */
 }
 
-void spotter_finite_to_stages (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates)
+void spotter_finite_to_stages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates)
 {
 	/* Convert finite rotations to backwards stage rotations for backtracking */
 	/* p[]		: Array of structure elements with rotation parameters
@@ -628,7 +633,7 @@ void spotter_finite_to_stages (struct EULER p[], int n, BOOLEAN finite_rates, BO
 	 * stage_rates	: TRUE if stage rotations should be returned in degree/my [else we return opening angle]
 	 */
 	 
-	int i;
+	GMT_LONG i;
 	double *elon, *elat, *ew, t_old;
 	double R_young[3][3], R_old[3][3], R_stage[3][3];
 
@@ -663,7 +668,7 @@ void spotter_finite_to_stages (struct EULER p[], int n, BOOLEAN finite_rates, BO
 	reverse_rotation_order (p, n);	/* Flip order since stages go from oldest to youngest */
 }
 
-void spotter_stages_to_finite (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates)
+void spotter_stages_to_finite (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates)
 {
 	/* Convert stage rotations to finite rotations */
 	/* p[]		: Array of structure elements with rotation parameters
@@ -672,7 +677,7 @@ void spotter_stages_to_finite (struct EULER p[], int n, BOOLEAN finite_rates, BO
 	 * stage_rates	: TRUE if stage rotations given in degree/my [else we have opening angle]
 	 */
 
-	int i;
+	GMT_LONG i;
 	double *elon, *elat, *ew;
 	double R_young[3][3], R_old[3][3], R_stage[3][3];
 
@@ -704,7 +709,7 @@ void spotter_stages_to_finite (struct EULER p[], int n, BOOLEAN finite_rates, BO
 	GMT_free ((void *)ew);
 }
 
-void spotter_add_rotations (struct EULER a[], int n_a, struct EULER b[], int n_b, struct EULER *c[], int *n_c)
+void spotter_add_rotations (struct EULER a[], GMT_LONG n_a, struct EULER b[], GMT_LONG n_b, struct EULER *c[], GMT_LONG *n_c)
 {
 	/* Takes two finite rotation models and adds them together.
 	 * We do this by first converting both to stage poles.  We then
@@ -719,14 +724,18 @@ void spotter_add_rotations (struct EULER a[], int n_a, struct EULER b[], int n_b
 
 	struct EULER *a2, *b2, *c2;
 	double *t, t_min, t_max, Ra[3][3], Rb[3][3], Rab[3][3], lon, lat, w, sign_a, sign_b;
-	int i, j, k, n_k = 0;
+	GMT_LONG i, j, k, n_k = 0;
 	BOOLEAN a_ok = TRUE, b_ok = TRUE;
 
 	sign_a = (n_a > 0) ? +1.0 : -1.0;
 	sign_b = (n_b > 0) ? +1.0 : -1.0;
+#ifdef __LP64__
+	n_a = labs (n_a);
+	n_b = labs (n_b);
+#else
 	n_a = abs (n_a);
 	n_b = abs (n_b);
-
+#endif
 	/* Allocate more than we need, must likely */
 
 	t = (double *) GMT_memory (VNULL, (size_t)(n_a + n_b), sizeof (double), "libspotter");
@@ -820,11 +829,11 @@ void spotter_add_rotations (struct EULER a[], int n_a, struct EULER b[], int n_b
 	*c = c2;
 }
 
-double spotter_t2w (struct EULER a[], int n, double t)
+double spotter_t2w (struct EULER a[], GMT_LONG n, double t)
 {
 	/* Take time, return cumulative omega */
 
-	int i;
+	GMT_LONG i;
 	double w = 0.0;
 
 	i = n - 1;
@@ -988,9 +997,9 @@ void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w)
 	}
 }
 
-void reverse_rotation_order (struct EULER *p, int n)
+void reverse_rotation_order (struct EULER *p, GMT_LONG n)
 {	/* Simply shuffles the array from 1:n to n:1 */
-	int i, j;
+	GMT_LONG i, j;
 	struct EULER p_tmp;
 
 	for (i = 0; i < n/2; i++) {
@@ -1003,11 +1012,11 @@ void reverse_rotation_order (struct EULER *p, int n)
 	}
 }
 
-void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[], int n, BOOLEAN stages, BOOLEAN convert)
+void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[], GMT_LONG n, BOOLEAN stages, BOOLEAN convert)
 {	/* Reload the EULER structure from the lon, lat, w arrays.
 	 * stages is TRUE if we are loading stage rotations (FALSE is finite poles).
 	 * convert is TRUE if we must change angles to rates or vice versa */
-	int i;
+	GMT_LONG i;
 
 	for (i = 0; i < n; i++) {
 		p[i].lon = lon[i];
@@ -1030,7 +1039,7 @@ void set_I_matrix (double R[3][3])
 	R[0][0] = R[1][1] = R[2][2] = 1.0;
 }
 
-int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, int np, char flag, double out[])
+int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT_LONG np, char flag, double out[])
 {
 	/* Given time and rotation parameters, calculate uncertainty in the
 	 * reconstructed point in the form of a confidence ellipse.  To follow
@@ -1042,14 +1051,15 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, int
 	 * that rotation, the covariance matrix is R * cov(r) * R^t.
 	 */
 
-	int matrix_dim = 3, i, j, k, kk = 3, nrots;
+	GMT_LONG matrix_dim = 3L;
+	int i, j, k, kk = 3, nrots;
 	double R[3][3], x[3], y[3], M[3][3], MtRt[3][3], Rt[3][3], RM[3][3], cov[3][3], tmp[3][3], C[9];
 	double z_unit_vector[3], EigenValue[3], EigenVector[9], work1[3], work2[3], x_in_plane[3], y_in_plane[3];
 	double x_comp, y_comp, w;
 
 	/* Find the unique rotation in question */
 
-	for (i = 0, k = -1; k < 0 && i < np; i++) if (fabs (p[i].t_start - t) < GMT_CONV_LIMIT) k = i;
+	for (i = 0, k = -1; k < 0 && i < np; i++) if (GMT_IS_ZERO (p[i].t_start - t)) k = i;
 	if (k == -1) return (1);	/* Did not match finite rotation time */
 
 	/* Generate R, the rotation matrix.  This is actually R^t since w is -ve */
