@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: originator.c,v 1.34 2007/04/02 15:25:57 remko Exp $
+ *	$Id: originator.c,v 1.42 2008/05/12 22:35:47 guru Exp $
  *
- *   Copyright (c) 2000-2007 by P. Wessel
+ *   Copyright (c) 2000-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -106,14 +106,16 @@ struct HOTSPOT {
 
 int main (int argc, char **argv)
 {
-	int i, j, k, n, kk, ns, nh, nc, np, x, y, n_max_spots, fno, n_files = 0, n_args;
-	int n_fields, n_expected_fields, n_read, n_skipped = 0, n_best_hs = 1, n_out = 0, tdz_output = 0;
+	int x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
+	int n_fields, n_expected_fields, n_best_hs = 1, tdz_output = 0, n_out = 0;
+	GMT_LONG  i, j, k, n, kk, ns, nh, nc, np, n_read, n_skipped = 0;
 
 	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better;
-	BOOLEAN finite = FALSE;		/* TRUE if stage pole file contains finite rotation poles instead */
+	BOOLEAN finite = FALSE, degree = FALSE;		/* fitite is TRUE if stage pole file contains finite rotation poles instead */
 
 	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, plon, plat, out[5];
-	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt = 0.0, A[3], H[3], N[3], X[3], dlon;
+	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt = 0.0, A[3], H[3], N[3], X[3];
+	double dlon, fix_r, fix_t;
 
 	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN];
 
@@ -168,13 +170,21 @@ int main (int argc, char **argv)
 				case 'L':
 					n_out = 3;
 					switch (argv[i][2]) {
+						case 'L':
+							degree = TRUE;
 						case 'l':
 							tdz_output = 3;
 							n_out = 5;
 							break;
 						case 'w':
+						case 'W':
 							tdz_output = 2;
 							break;
+						case 'T':
+							degree = TRUE;
+						case 't':
+							tdz_output = 1;
+							break;
 						default:
 							tdz_output = 1;
 							break;
@@ -183,6 +193,10 @@ int main (int argc, char **argv)
 				case 'N':
 					upper_age = atof (&argv[i][2]);
 					break;
+				case 'Q':
+					sscanf (&argv[i][2], "%lg/%lg", &fix_r, &fix_t);
+					n_input = 3;
+					break;
 				case 'S':
 					n_best_hs = atoi (&argv[i][2]);
 					break;
@@ -206,18 +220,18 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 5;
-        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 5) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 5 columns\n", GMT_program);
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_input;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_input) {
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_input);
 		error++;
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Associate seamounts with hotspot point sources\n\n", GMT_program, GMT_VERSION);
 		fprintf (stderr, "usage: %s [<xyfiles>] -E<euler_file> -F<hotspot_file> [-C] [-D<d_km>]\n", GMT_program);
-		fprintf (stderr, "	[-H] [-L] [-N<upper_age>] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [%s]\n\n", GMT_t_OPT);
+		fprintf (stderr, "	[-H] [-L[flag]] [-N<upper_age>] [-Qr/t] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [%s]\n\n", GMT_t_OPT);
 
-		if (GMT_give_synopsis_and_exit) exit (-1);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyfiles is one or more seamount (x,y,z,r,t) files\n");
 		fprintf (stderr, "\t-E Specify file name for Euler poles.\n");
@@ -226,10 +240,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-C The file given with -E contains finite rotation poles [Default is stage poles]\n");
 		fprintf (stderr, "\t-D set sampling interval in km along tracks [5].\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-L Output (time, dist, z) triplets for nearest hotspot only (ignores -S).\n");
+		fprintf (stderr, "\t-L Output information for closest approach for nearest hotspot only (ignores -S).\n");
+		fprintf (stderr, "\t   -Lt gives (omega, dist, z) [Default].\n");
 		fprintf (stderr, "\t   -Lw gives (omega, dist, z).\n");
 		fprintf (stderr, "\t   -Ll gives (lon, lat, time, dist, z).\n");
+		fprintf (stderr, "\t   dist is in km; use upper case T,W,L to get dist in spherical degrees.\n");
 		fprintf (stderr, "\t-N set age (in m.y.) for seafloor where age == NaN [180].\n");
+		fprintf (stderr, "\t-Q input files has (x,y,z) only. Append constant r/t to use.\n");
 		fprintf (stderr, "\t-S Report the <n_hs> closest hotSpots [1].\n");
 		fprintf (stderr, "\t-T Truncate seamount ages exceeding the upper age set with -N [no truncation] \n");
 		GMT_explain_option ('V');
@@ -263,14 +280,14 @@ int main (int argc, char **argv)
 
 	nh = hotspot_init (hsfile, &hotspot);
 	if (n_best_hs <= 0 || n_best_hs > nh) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %d\n", GMT_program, nh);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %ld\n", GMT_program, nh);
 		exit (EXIT_FAILURE);
 	}
 	n_max_spots = MIN (n_best_hs, nh);
 
 	ns = spotter_init (efile, &p, TRUE, finite, FALSE, &upper_age, gmtdefs.verbose);
 
-	hot = (struct HOTSPOT *) GMT_memory (VNULL, nh, sizeof (struct HOTSPOT), GMT_program);
+	hot = (struct HOTSPOT *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT), GMT_program);
 
 	x = (gmtdefs.xy_toggle[1]) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y on output */
 
@@ -281,14 +298,14 @@ int main (int argc, char **argv)
 	n_args = (argc > 1) ? argc : 2;
 
 	done = FALSE;
-	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 5;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : n_input;
 	n = 0;
 	if (!tdz_output) n_out = n_expected_fields;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 
 		if (!nofile && argv[fno][0] == '-') continue;
 		if (nofile) {
-			fp = stdin;
+			fp = GMT_stdin;
 			done = TRUE;
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -318,9 +335,13 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
+			if (n_input == 3) {	/* set constant r,t values */
+				in[3] = fix_r;
+				in[4] = fix_t;
+			}
 			if (GMT_is_dnan (in[4])) {	/* Age is NaN, assign value */
 				t_smt = upper_age;
 			}
@@ -331,7 +352,7 @@ int main (int argc, char **argv)
 						t_smt = upper_age;
 					}
 					else {
-						fprintf (stderr, "%s: Seamounts near line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
+						fprintf (stderr, "%s: Seamounts near line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
 						continue;
 					}
 				}
@@ -345,17 +366,18 @@ int main (int argc, char **argv)
 			z_smt = in[2];
 			r_smt = in[3];
 
-			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5ld\r", GMT_program, n);
 
-			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, 1, p, ns, d_km, 0.0, TRUE, NULL, &c);
+			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, (GMT_LONG)1, p, ns, d_km, 0.0, TRUE, NULL, &c);
 
-			np = (int) c[0];
+			np = (GMT_LONG) c[0];
 
 			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT));
 
 			for (kk = 0, k = 1; kk < np; kk++, k += 3) {	/* For this seamounts track */
 				for (j = 0; j < nh; j++) {	/* For all hotspots */
-					dist = GMT_great_circle_dist (hot[j].lon, hot[j].lat, R2D * c[k], R2D * c[k+1]) * KM_PR_DEG;
+					dist = GMT_great_circle_dist (hot[j].lon, hot[j].lat, R2D * c[k], R2D * c[k+1]);
+					if (!degree) dist *= KM_PR_DEG;
 					if (dist < hot[j].np_dist) {
 						hot[j].np_dist = dist;
 						hot[j].nearest = kk;	/* Index of nearest point on the flowline */
@@ -427,7 +449,7 @@ int main (int argc, char **argv)
 				if (hot[j].stage == 0) hot[j].stage++;
 			}
 
-			if (nh > 1) qsort ((void *)hot, nh, sizeof(struct HOTSPOT), comp_hs);
+			if (nh > 1) qsort ((void *)hot, (size_t)nh, sizeof(struct HOTSPOT), comp_hs);
 
 			if (hot[0].np_dist < max_dist) {
 				if (tdz_output == 1) {	/* Want time, dist, z output */
@@ -465,10 +487,10 @@ int main (int argc, char **argv)
 			n++;
 		}
 
-		if (fp != stdin) fclose (fp);
+		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5d\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5ld\n", GMT_program, n);
 
 	GMT_free ((void *)hotspot);
 	GMT_free ((void *)hot);
@@ -505,18 +527,18 @@ int hotspot_init (char *file, struct HOTSPOT **p)
 
 	e = (struct HOTSPOT *) GMT_memory (VNULL, n_alloc, sizeof (struct HOTSPOT), GMT_program);
 
-	while (fgets (buffer, 512, fp) != NULL) {
+	while (GMT_fgets (buffer, 512, fp) != NULL) {
 		if (buffer[0] == '#' || buffer[0] == '\n') continue;
 		sscanf (buffer, "%lf %lf %s %s", &e[i].lon, &e[i].lat, e[i].abbrev, e[i].name);
 		e[i].id = i;
 		e[i].np_dist = 1.0e100;
 		i++;
 		if ((size_t)i == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			e = (struct HOTSPOT *) GMT_memory ((void *)e, n_alloc, sizeof (struct HOTSPOT), GMT_program);
 		}
 	}
-	fclose (fp);
+	GMT_fclose (fp);
 	e = (struct HOTSPOT *) GMT_memory ((void *)e, (size_t)i, sizeof (struct HOTSPOT), GMT_program);
 	*p = e;
 
