@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: rotconverter.c,v 1.21 2007/03/12 19:52:27 remko Exp $
+ *	$Id: rotconverter.c,v 1.24 2008/03/24 08:58:33 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -69,9 +69,9 @@ int main (int argc, char **argv)
 	struct EULER *p;		/* Pointer to array of stage poles */
 	struct EULER *a, *b;		/* Pointer to arrays of stage poles */
 
-	int i, j, n_p, n_a = 1, n_b;	/* Misc. counters */
-	int last_sign, n_slash;
-	int rate_or_rad = 0;		/* Default unit is degrees/My */
+	GMT_LONG i, j, n_p, n_a = 1, n_b;	/* Misc. counters */
+	GMT_LONG last_sign, n_slash;
+	GMT_LONG rate_or_rad = 0;		/* Default unit is degrees/My */
 
 	BOOLEAN error = FALSE;		/* Set to TRUE if arguments are inconsistent */
 	BOOLEAN geodetic = TRUE;	/* Want 0-360 range */
@@ -203,9 +203,9 @@ int main (int argc, char **argv)
 
 	last_sign = +1;
 	for (i = 1; i < argc; i++) {
-		if (!strncmp (argv[i], "-D", 2)) continue;
-		if (!strncmp (argv[i], "-V", 2)) continue;
-		if (!strncmp (argv[i], "-F", 2)) continue;
+		if (!strncmp (argv[i], "-D", (size_t)2)) continue;
+		if (!strncmp (argv[i], "-V", (size_t)2)) continue;
+		if (!strncmp (argv[i], "-F", (size_t)2)) continue;
 
 		if (argv[i][0] == '-') {
 			last_sign = -1;
