@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: spotter.h,v 1.13 2007/01/30 20:37:09 pwessel Exp $
+ *	$Id: spotter.h,v 1.17 2008/03/24 08:58:33 guru Exp $
  *
- *   Copyright (c) 1999-2007 by P. Wessel
+ *   Copyright (c) 1999-2008 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #define SPOTTER_VERSION "1.1"
 #define EQ_RAD 6371.0087714
 #define KM_PR_DEG (EQ_RAD * M_PI / 180.0)
-#define BIG_CHUNK 50000
+#define BIG_CHUNK 65536
 #define T_2_PA	250.0
 #define PA_2_T  (1.0 / T_2_PA)
 #define SQRT_CHI2 2.44774689322	/* This is sqrt (Chi^2) for 95% and 2 degrees of freedom */
@@ -46,21 +46,21 @@ struct EULER {	/* Structure with info on each Euler (stage) pole */
 };
 
 struct FLOWLINE {			/* Structure with the nearest nodes for a single flowline */
-	int n;				/* number of points in this flowline */
-	int ij;				/* Node in bathymetry grid where this flowline originated */
-	int *node;			/* Nodes in CVA grid covered by this flowline */
+	GMT_LONG n;				/* number of points in this flowline */
+	GMT_LONG ij;				/* Node in bathymetry grid where this flowline originated */
+	GMT_LONG *node;			/* Nodes in CVA grid covered by this flowline */
 	unsigned short *PA;		/* Predicted Ages along flowline (t = PI/250, to nearest 0.004 My) */
 };
 
 /* ANSI-C Function prototypes (see libspotter.c for details): */
 
 EXTERN_MSC int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in, BOOLEAN finite_out, double *t_max, BOOLEAN verbose);
-EXTERN_MSC int spotter_backtrack  (double xp[], double yp[], double tp[], int np, struct EULER p[], int ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c);
-EXTERN_MSC int spotter_forthtrack (double xp[], double yp[], double tp[], int np, struct EULER p[], int ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c);
-EXTERN_MSC void spotter_finite_to_stages (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates);
-EXTERN_MSC void spotter_stages_to_finite (struct EULER p[], int n, BOOLEAN finite_rates, BOOLEAN stage_rates);
-EXTERN_MSC void spotter_add_rotations (struct EULER a[], int n_a, struct EULER b[], int n_b, struct EULER *c[], int *n_c);
-EXTERN_MSC double spotter_t2w (struct EULER a[], int n, double t);
-EXTERN_MSC int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, int np, char conf, double out[]);
+EXTERN_MSC int spotter_backtrack  (double xp[], double yp[], double tp[], GMT_LONG np, struct EULER p[], GMT_LONG ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c);
+EXTERN_MSC int spotter_forthtrack (double xp[], double yp[], double tp[], GMT_LONG np, struct EULER p[], GMT_LONG ns, double d_km, double t_zero, BOOLEAN do_time, double wesn[], double **c);
+EXTERN_MSC void spotter_finite_to_stages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates);
+EXTERN_MSC void spotter_stages_to_finite (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates);
+EXTERN_MSC void spotter_add_rotations (struct EULER a[], GMT_LONG n_a, struct EULER b[], GMT_LONG n_b, struct EULER *c[], GMT_LONG *n_c);
+EXTERN_MSC double spotter_t2w (struct EULER a[], GMT_LONG n, double t);
+EXTERN_MSC int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT_LONG np, char conf, double out[]);
 EXTERN_MSC void matrix_vect_mult (double a[3][3], double b[3], double c[3]);
 EXTERN_MSC void make_rot_matrix (double lonp, double latp, double w, double R[3][3]);
