@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: surface.c,v 1.43 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: surface.c,v 1.54 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -60,10 +60,10 @@
  *	work; simple testing today suggests it makes no difference, but I will
  *	leave it in.
  *
- * V 4.2 -- 4 June, 1991-2000:	P. Wessel upgraded to GMT v2.0 grdfile i/o.
+ * V 4.2 -- 4 June, 1991-2000:	P. Wessel upgraded to GMT v2.0 grid file i/o.
  *	Added feature to constrain solution to be within lower and/or
  *	upper bounds.  Bounds can be min/max input data value, another
- *	value outside the input data range, or be provided by a grdfile
+ *	value outside the input data range, or be provided by a grid file
  *	whose values all are outside the input data range.
  *
  * V 4.3 -- 26 February, 1992:	W. H. F. Smith added option to suggest better
@@ -79,6 +79,7 @@
  * Version:	4
  * Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
  * Version:	4.1.2 PW: Minimized (but not eliminated) global variables
+ * 64-bit Compliant: Yes
  *
  *
  */
@@ -130,13 +131,13 @@ struct SURFACE_CTRL {
 	} Z;
 };
 
-#define SURFACE_OUTSIDE 2000000000	/* Index number indicating data is outside usable area */
+#define SURFACE_OUTSIDE LONG_MAX	/* Index number indicating data is outside usable area */
 
 struct SURFACE_DATA {	/* Data point and index to node it currently constrains  */
 	float x;
 	float y;
 	float z;
-	int index;
+	GMT_LONG index;
 };
 
 struct SURFACE_BRIGGS {		/* Coefficients in Taylor series for Laplacian(z) a la I. C. Briggs (1974)  */
@@ -144,7 +145,7 @@ struct SURFACE_BRIGGS {		/* Coefficients in Taylor series for Laplacian(z) a la
 };
 
 struct SURFACE_GLOBAL {		/* Things needed inside compare function must be global for now */
-	int block_ny;			/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG block_ny;			/* Number of nodes in y-dir for a given grid factor */
 	double grid_xinc, grid_yinc;	/* size of each grid cell for a given grid factor */
 	double x_min, y_min;		/* Lower left corner of grid */
 } GMT_Surface_Global;
@@ -155,27 +156,29 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 					 * I means just interpolate from larger grid */
 	char format[BUFSIZ];
 	char *low_file, *high_file;	/* Pointers to grids with low and high limits, if selected */
-	int n_alloc;
-	int npoints;			/* Number of data points */
-	int nx;				/* Number of nodes in x-dir. */
-	int ny;				/* Number of nodes in y-dir. (Final grid) */
-	int mx;
-	int my;
-	int ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
-	int block_nx;			/* Number of nodes in x-dir for a given grid factor */
-	int block_ny;			/* Number of nodes in y-dir for a given grid factor */
-	int max_iterations;		/* Max iter per call to iterate */
-	int total_iterations;
 	int grid, old_grid;		/* Node spacings  */
-	int grid_east;
 	int n_fact;			/* Number of factors in common (ny-1, nx-1) */
 	int factors[32];		/* Array of common factors */
 	int long_verbose;
-	int n_empty;			/* No of unconstrained nodes at initialization  */
 	int set_low;			/* 0 unconstrained,1 = by min data value, 2 = by user value */
 	int set_high;			/* 0 unconstrained,1 = by max data value, 2 = by user value */
-	int constrained;		/* TRUE if set_low or set_high is TRUE */
-	int offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	size_t n_alloc;
+	GMT_LONG npoints;			/* Number of data points */
+	GMT_LONG ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
+	GMT_LONG n_empty;			/* No of unconstrained nodes at initialization  */
+	GMT_LONG nx;				/* Number of nodes in x-dir. */
+	GMT_LONG ny;				/* Number of nodes in y-dir. (Final grid) */
+	GMT_LONG nxny;			/* Total number of grid nodes without boundaries  */
+	GMT_LONG mx;
+	GMT_LONG my;
+	GMT_LONG mxmy;			/* Total number of grid nodes with boundaries  */
+	GMT_LONG block_nx;			/* Number of nodes in x-dir for a given grid factor */
+	GMT_LONG block_ny;			/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG max_iterations;		/* Max iter per call to iterate */
+	GMT_LONG total_iterations;
+	GMT_LONG grid_east;
+	GMT_LONG offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	BOOLEAN constrained;		/* TRUE if set_low or set_high is TRUE */
 	float *lower, *upper;		/* arrays for minmax values, if set */
 	float *u;			/* Pointer to grid array */
 	double low_limit, high_limit;	/* Constrains on range of solution */
@@ -210,8 +213,8 @@ struct SURFACE_SUGGESTION {	/* Used to find top ten list of faster grid dimensio
 };
 
 int compare_points (const void *point_1v, const void *point_2v);
-int gcd_euclid (int a, int b);	/* Finds the greatest common divisor  */
-int get_prime_factors (int n, int *f);
+GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b);	/* Finds the greatest common divisor  */
+int get_prime_factors (GMT_LONG n, int *f);
 int iterate (struct SURFACE_INFO *C, int mode);
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform);
 
@@ -395,8 +398,9 @@ int main (int argc, char **argv)
 
 		fprintf (stderr, "\tsurface will read from standard input or a single <xyz-file>.\n\n");
 		fprintf (stderr, "\tRequired arguments to surface:\n");
-		fprintf (stderr, "\t-G sets output grd File name\n");
+		fprintf (stderr, "\t-G sets output grid file name\n");
 		GMT_inc_syntax ('I', 0);
+		fprintf (stderr, "\t\tNote that only gridline registration can be used.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A<aspect_ratio>  = 1.0  by default which gives an isotropic solution.\n");
@@ -411,7 +415,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t\t-Ll<limit> specifies lower limit; forces solution to be >= <limit>.\n");
 		fprintf (stderr, "\t\t-Lu<limit> specifies upper limit; forces solution to be <= <limit>.\n");
 		fprintf (stderr, "\t\t<limit> can be any number, or the letter d for min (or max) input data value,\n");
-		fprintf (stderr, "\t\tor the filename of a grdfile with bounding values.  [Default solution unconstrained].\n");
+		fprintf (stderr, "\t\tor the filename of a grid with bounding values.  [Default solution unconstrained].\n");
 		fprintf (stderr, "\t\tExample:  -Ll0 gives a non-negative solution.\n");
 		fprintf (stderr, "\t-N sets max <n_iterations> in each cycle; default = 250.\n");
 		fprintf (stderr, "\t-S sets <search_radius> to initialize grid; default = 0 will skip this step.\n");
@@ -490,10 +494,12 @@ int main (int argc, char **argv)
 
 	C.relax_old = 1.0 - C.relax_new;
 
-	C.nx = C.h.nx;
-	C.ny = C.h.ny;
+	C.nx = (GMT_LONG)C.h.nx;
+	C.ny = (GMT_LONG)C.h.ny;
+	C.nxny = C.nx * C.ny;
 	C.mx = C.nx + 4;
 	C.my = C.ny + 4;
+	C.mxmy = C.mx * C.my;
 	C.r_xinc = 1.0 / C.h.x_inc;
 	C.r_yinc = 1.0 / C.h.y_inc;
 	GMT_Surface_Global.x_min = C.h.x_min;
@@ -503,11 +509,11 @@ int main (int argc, char **argv)
 	C.grid = gcd_euclid (C.nx-1, C.ny-1);
 
 	if (gmtdefs.verbose || Ctrl->Q.active) {
-		sprintf (C.format, "W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (C.format, "%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, C.format, C.h.x_min, C.h.x_max, C.h.y_min, C.h.y_max, C.nx-1, C.ny-1);
 	}
 	if (C.grid == 1 && gmtdefs.verbose) fprintf (stderr, "%s:  WARNING:  Your grid dimensions are mutually prime.\n", GMT_program);
-	if ((C.grid == 1 && gmtdefs.verbose) || Ctrl->Q.active) suggest_sizes_for_surface (C.factors, C.nx-1, C.ny-1);
+	if ((C.grid == 1 && gmtdefs.verbose) || Ctrl->Q.active) suggest_sizes_for_surface (C.factors, (int)C.nx-1, (int)C.ny-1);
 	if (Ctrl->Q.active) exit (EXIT_SUCCESS);
 
 	/* New idea: set grid = 1, read data, setting index.  Then throw
@@ -531,7 +537,7 @@ int main (int argc, char **argv)
 	/* Set up factors and reset grid to first value  */
 
 	C.grid = gcd_euclid (C.nx-1, C.ny-1);
-	C.n_fact = get_prime_factors (C.grid, C.factors);
+	C.n_fact = get_prime_factors ((GMT_LONG)C.grid, C.factors);
 	set_grid_parameters (&C);
 	while (C.block_nx < 4 || C.block_ny < 4) {
 		smart_divide (&C);
@@ -544,12 +550,12 @@ int main (int argc, char **argv)
 	/* Allocate more space  */
 
 	C.briggs = (struct SURFACE_BRIGGS *) GMT_memory (VNULL, (size_t)C.npoints, sizeof(struct SURFACE_BRIGGS), GMT_program);
-	C.iu = (char *) GMT_memory (VNULL, (size_t)(C.mx * C.my), sizeof(char), GMT_program);
-	C.u = (float *) GMT_memory (VNULL, (size_t)(C.mx * C.my), sizeof(float), GMT_program);
+	C.iu = (char *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(char), GMT_program);
+	C.u = (float *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(float), GMT_program);
 
 	if (C.radius > 0) initialize_grid (&C); /* Fill in nodes with a weighted avg in a search radius  */
 
-	if (gmtdefs.verbose) fprintf (stderr, "Grid\tMode\tIteration\tMax Change\tConv Limit\tTotal Iterations\n");
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid\tMode\tIteration\tMax Change\tConv Limit\tTotal Iterations\n", GMT_program);
 
 	set_coefficients (&C);
 
@@ -628,14 +634,14 @@ void set_coefficients (struct SURFACE_INFO *C)
 
 	C->ij_sw_corner = 2 * C->my + 2;			/*  Corners of array of actual data  */
 	C->ij_se_corner = C->ij_sw_corner + (C->nx - 1) * C->my;
-	C->ij_nw_corner = C->ij_sw_corner + (C->ny - 1);
-	C->ij_ne_corner = C->ij_se_corner + (C->ny - 1);
+	C->ij_nw_corner = C->ij_sw_corner + C->ny - 1;
+	C->ij_ne_corner = C->ij_se_corner + C->ny - 1;
 
 }
 
 void set_offset (struct SURFACE_INFO *C)
 {
-	int	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
+	GMT_LONG	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
 	int	i, j, kase;
 
 	add_w[0] = -C->my; add_w[1] = add_w[2] = add_w[3] = add_w[4] = -C->grid_east;
@@ -674,8 +680,8 @@ void fill_in_forecast (struct SURFACE_INFO *C) {
 	   after grid is divided.   
 	 */
 
-	int i, j, ii, jj, index_0, index_1, index_2, index_3;
-	int index_new;
+	GMT_LONG i, j, ii, jj, index_0, index_1, index_2, index_3;
+	GMT_LONG index_new;
 	char *iu;
 	double delta_x, delta_y, a0, a1, a2, a3;
 	double old_size;
@@ -754,7 +760,7 @@ int compare_points (const void *point_1v, const void *point_2v)
 		    Sorts on index first, then on radius to node corresponding to index, so that index
 		    goes from low to high, and so does radius.
 		*/
-	int block_i, block_j, index_1, index_2;
+	GMT_LONG block_i, block_j, index_1, index_2;
 	double x0, y0, dist_1, dist_2;
 	struct SURFACE_DATA *point_1, *point_2;
 	
@@ -794,11 +800,11 @@ void set_index (struct SURFACE_INFO *C) {
 		/* recomputes data[k].index for new value of grid,
 		   sorts data on index and radii, and throws away
 		   data which are now outside the usable limits. */
-	int i, j, k, k_skipped = 0;
+	GMT_LONG i, j, k, k_skipped = 0;
 
 	for (k = 0; k < C->npoints; k++) {
-		i = (int)floor(((C->data[k].x-C->h.x_min)*C->r_grid_xinc) + 0.5);
-		j = (int)floor(((C->data[k].y-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		i = (GMT_LONG)floor(((C->data[k].x-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		j = (GMT_LONG)floor(((C->data[k].y-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (i < 0 || i >= C->block_nx || j < 0 || j >= C->block_ny) {
 			C->data[k].index = SURFACE_OUTSIDE;
 			k_skipped++;
@@ -814,7 +820,7 @@ void set_index (struct SURFACE_INFO *C) {
 }
 
 void find_nearest_point (struct SURFACE_INFO *C) {
-	int i, j, ij_v2, k, last_index, block_i, block_j, iu_index, briggs_index;
+	GMT_LONG i, j, ij_v2, k, last_index, block_i, block_j, iu_index, briggs_index;
 	double x0, y0, dx, dy, xys, xy1, btemp;
 	double b0, b1, b2, b3, b4, b5;
 	float z_at_node, *u;
@@ -912,15 +918,13 @@ void initialize_grid (struct SURFACE_INFO *C)
 	 * For the initial gridsize, compute weighted averages of data inside the search radius
 	 * and assign the values to u[i,j] where i,j are multiples of gridsize.
 	 */
-	int	irad, jrad, i, j, imin, imax, jmin, jmax, index_1, index_2, k, ki, kj, k_index;
-	char *iu;
+	GMT_LONG	irad, jrad, i, j, imin, imax, jmin, jmax, index_1, index_2, k, ki, kj, k_index;
 	double	r, rfact, sum_w, sum_zw, weight, x0, y0;
 	float *u;
 	u = C->u;
-	iu = C->iu;
 
-	 irad = (int)ceil(C->radius/C->grid_xinc);
-	 jrad = (int)ceil(C->radius/C->grid_yinc);
+	 irad = (GMT_LONG)ceil(C->radius/C->grid_xinc);
+	 jrad = (GMT_LONG)ceil(C->radius/C->grid_yinc);
 	 rfact = -4.5/(C->radius*C->radius);
 	 for (i = 0; i < C->block_nx; i ++ ) {
 	 	x0 = C->h.x_min + i*C->grid_xinc;
@@ -970,10 +974,10 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 	 * For the initial gridsize, load constrained nodes with weighted avg of their data;
 	 * and then do something with the unconstrained ones.
 	 */
-	 int	k, k_index, u_index, block_i, block_j;
-	 char *iu;
-	 double	sum_w, sum_zw, weight, x0, y0, dx, dy, dx_scale, dy_scale;
-	 float *u;
+	GMT_LONG	k, k_index, u_index, block_i, block_j;
+	char *iu;
+	double	sum_w, sum_zw, weight, x0, y0, dx, dy, dx_scale, dy_scale;
+	float *u;
 	u = C->u;
 	iu = C->iu;
 
@@ -1013,7 +1017,8 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 
 void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 {
-	int	i, j, k, kmax = 0, kmin = 0, n_fields, n_expected_fields;
+	GMT_LONG	i, j, k, kmax = 0, kmin = 0;
+	int n_fields, n_expected_fields;
 	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
 	char	buffer[BUFSIZ];
 
@@ -1029,15 +1034,15 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, k);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, k);
 			exit (EXIT_FAILURE);
 		}
 
 		if (GMT_is_dnan (in[2])) continue;
 
-		i = (int)floor(((in[0]-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		i = (GMT_LONG)floor(((in[0]-C->h.x_min)*C->r_grid_xinc) + 0.5);
 		if (i < 0 || i >= C->block_nx) continue;
-		j = (int)floor(((in[1]-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		j = (GMT_LONG)floor(((in[1]-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (j < 0 || j >= C->block_ny) continue;
 
 		C->data[k].index = i * C->block_ny + j;
@@ -1048,8 +1053,8 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 		if (zmax < in[2]) zmax = in[2], kmax = k;
 		k++;
 		C->z_mean += in[2];
-		if (k == C->n_alloc) {
-			C->n_alloc += GMT_CHUNK;
+		if (k == (GMT_LONG)C->n_alloc) {
+			C->n_alloc <<= 1;
 			C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
 		}
 	}
@@ -1065,7 +1070,7 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 
 	C->z_mean /= k;
 	if (gmtdefs.verbose) {
-		sprintf(C->format, "%s %s %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf(C->format, "%s: %s %s %s\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, "%s: Minimum value of your dataset x,y,z at: ", GMT_program);
 		fprintf (stderr, C->format, (double)C->data[kmin].x, (double)C->data[kmin].y, (double)C->data[kmin].z);
 		fprintf (stderr, "%s: Maximum value of your dataset x,y,z at: ", GMT_program);
@@ -1090,7 +1095,7 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 
 void write_output (struct SURFACE_INFO *C, char *grdfile)
 {	/* Uses v.2.0 netCDF grd format - hence need to transpose original grid to be GMT compatible.  This will be rewritten, eventually */
-	int	index, i, j, k;
+	GMT_LONG	index, i, j, k;
 	float *u, *v2;
 	u = C->u;
 
@@ -1098,7 +1103,7 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 		
 	strcpy (C->h.title, GMT_program);
 
-	v2 = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+	v2 = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 	index = C->ij_sw_corner;
 	for (i = 0; i < C->nx; i++, index += C->my) {
 		for (j = 0; j < C->ny; j++) {
@@ -1118,9 +1123,9 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 int iterate (struct SURFACE_INFO *C, int mode)
 {
 
-	int	i, j, k, ij, kase, briggs_index, ij_v2;
-	int	x_case, y_case, x_w_case, x_e_case, y_s_case, y_n_case;
-	int	iteration_count = 0;
+	GMT_LONG	i, j, k, ij, kase, briggs_index, ij_v2;
+	GMT_LONG	x_case, y_case, x_w_case, x_e_case, y_s_case, y_n_case;
+	GMT_LONG	iteration_count = 0;
 	char *iu;
 
 	double	current_limit = C->converge_limit / C->grid;
@@ -1134,7 +1139,7 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	double	y_0_const = 4 * C->l_epsilon * (1.0 - C->boundary_tension) / y_denom;
 	double	y_1_const = (C->boundary_tension - 2 * C->l_epsilon * (1.0 - C->boundary_tension) ) / y_denom;
 
-	sprintf(C->format,"%%4d\t%%c\t%%8d\t%s\t%s\t%%10d\n", gmtdefs.d_format, gmtdefs.d_format);
+	sprintf(C->format,"%s: %%4ld\t%%c\t%%8ld\t%s\t%s\t%%10ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format);
 
 	u = C->u;
 	iu = C->iu;
@@ -1148,8 +1153,6 @@ int iterate (struct SURFACE_INFO *C, int mode)
 		/* First set d2[]/dn2 = 0 along edges:  */
 		/* New experiment : (1-T)d2[]/dn2 + Td[]/dn = 0  */
 
-
-
 		for (i = 0; i < C->nx; i += C->grid) {
 			/* set d2[]/dy2 = 0 on south side:  */
 			ij = C->ij_sw_corner + i * C->my;
@@ -1358,12 +1361,12 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	if (gmtdefs.verbose && !C->long_verbose) fprintf (stderr, C->format,
 		C->grid, C->mode_type[mode], iteration_count, max_change, current_limit, C->total_iterations);
 
-	return(iteration_count);
+	return (iteration_count);
 }
 
 void check_errors (struct SURFACE_INFO *C) {
 
-	int	i, j, k, ij, n_nodes, move_over[12];
+	GMT_LONG	i, j, k, ij, move_over[12];
 	char *iu;	/* move_over = C->offset[kase][12], but grid = 1 so move_over is easy  */
 
 	double	x0, y0, dx, dy, mean_error, mean_squared_error, z_est, z_err, curvature, c;
@@ -1492,7 +1495,6 @@ void check_errors (struct SURFACE_INFO *C) {
 	 mean_squared_error = sqrt( mean_squared_error / C->npoints);
 	 
 	 curvature = 0.0;
-	 n_nodes = C->nx * C->ny;
 	 
 	 for (i = 0; i < C->nx; i++) {
 	 	for (j = 0; j < C->ny; j++) {
@@ -1503,15 +1505,15 @@ void check_errors (struct SURFACE_INFO *C) {
 		}
 	}
 
-	 fprintf (stderr, "Fit info: N data points  N nodes\tmean error\trms error\tcurvature\n");
-	 sprintf (C->format,"\t%%8d\t%%8d\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-	 fprintf (stderr, C->format, C->npoints, n_nodes, mean_error, mean_squared_error, curvature);
+	 fprintf (stderr, "%s: Fit info: N data points  N nodes\tmean error\trms error\tcurvature\n", GMT_program);
+	 sprintf (C->format,"%s:\t%%8ld\t%%8ld\t%s\t%s\t%s\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+	 fprintf (stderr, C->format, C->npoints, C->nxny, mean_error, mean_squared_error, curvature);
  }
 
 void remove_planar_trend (struct SURFACE_INFO *C)
 {
 
-	int	i;
+	GMT_LONG	i;
 	double	a, b, c, d, xx, yy, zz;
 	double	sx, sy, sz, sxx, sxy, sxz, syy, syz;
 
@@ -1559,7 +1561,7 @@ void remove_planar_trend (struct SURFACE_INFO *C)
 
 void replace_planar_trend (struct SURFACE_INFO *C)
 {
-	int	i, j, ij;
+	GMT_LONG	i, j, ij;
 	float *u;
 	u = C->u;
 
@@ -1584,7 +1586,7 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 		coefficients, eliminating calls to temp file.
 	*/
 
-	int	last_index, n_outside, k;
+	GMT_LONG	last_index, n_outside, k;
 
 	/* Sort the data  */
 
@@ -1610,14 +1612,14 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 	C->npoints -= n_outside;
 	C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->npoints, sizeof(struct SURFACE_DATA), GMT_program);
 	if (gmtdefs.verbose && (n_outside)) {
-		fprintf (stderr, "%s: %d unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
-		fprintf (stderr, "\tYou should have pre-processed the data with block-mean, -median, or -mode.\n");
+		fprintf (stderr, "%s: %ld unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
+		fprintf (stderr, "%s: You should have pre-processed the data with block-mean, -median, or -mode.\n", GMT_program);
 	}
 }
 
 void rescale_z_values (struct SURFACE_INFO *C)
 {
-	int	i;
+	GMT_LONG	i;
 	double	ssz = 0.0;
 
 	for (i = 0; i < C->npoints; i++) ssz += (C->data[i].z * C->data[i].z);
@@ -1640,16 +1642,16 @@ void rescale_z_values (struct SURFACE_INFO *C)
 
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 {
-	int i, j, ij;
+	GMT_LONG i, j, ij;
 	double yy;
 	struct GRD_HEADER hdr;
 
 	/* Load lower/upper limits, verify range, deplane, and rescale */
 
 	if (C->set_low > 0) {
-		C->lower = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+		C->lower = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 		if (C->set_low < 3)
-			for (i = 0; i < C->nx * C->ny; i++) C->lower[i] = (float)C->low_limit;
+			for (i = 0; i < C->nxny; i++) C->lower[i] = (float)C->low_limit;
 		else {
 			GMT_err_fail (GMT_read_grd_info (C->low_file, &hdr), C->low_file);
 			if (hdr.nx != C->nx || hdr.ny != C->ny) {
@@ -1671,9 +1673,9 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		C->constrained = TRUE;
 	}
 	if (C->set_high > 0) {
-		C->upper = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+		C->upper = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 		if (C->set_high < 3)
-			for (i = 0; i < C->nx * C->ny; i++) C->upper[i] = (float)C->high_limit;
+			for (i = 0; i < C->nxny; i++) C->upper[i] = (float)C->high_limit;
 		else {
 			GMT_err_fail (GMT_read_grd_info (C->high_file, &hdr), C->high_file);
 			if (hdr.nx != C->nx || hdr.ny != C->ny) {
@@ -1723,7 +1725,7 @@ double guess_surface_time (int factors[], int nx, int ny)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
-	int	gcd;		/* Current value of the gcd  */
+	GMT_LONG	gcd;		/* Current value of the gcd  */
 	int	nxg, nyg;	/* Current value of the grid dimensions  */
 	int	nfactors = 0;	/* Number of prime factors of current gcd  */
 	int	factor;		/* Currently used factor  */
@@ -1733,7 +1735,7 @@ double guess_surface_time (int factors[], int nx, int ny)
 	double	length;		/* Current propagation distance.  */
 
 
-	gcd = gcd_euclid(nx, ny);
+	gcd = gcd_euclid((GMT_LONG)nx, (GMT_LONG)ny);
 	if (gcd > 1) {
 		nfactors = get_prime_factors(gcd, factors);
 		nxg = nx/gcd;
@@ -1848,7 +1850,7 @@ int compare_sugs (const void *point_1, const void *point_2)
 		return(0);
 }
 
-int get_prime_factors (int n, int *f)
+int get_prime_factors (GMT_LONG n, int *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1875,17 +1877,21 @@ int get_prime_factors (int n, int *f)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992, after D.E. Knuth, vol. II  */
 
-	int	current_factor;	/* The factor currently being tried  */
-	int	max_factor;	/* Don't try any factors bigger than this  */
+	GMT_LONG	current_factor;	/* The factor currently being tried  */
+	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
 	int	n_factors = 0;	/* Returned; one if n is prime  */
-	int	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
-	int	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
-	int	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
-	int	m;	/* Used to keep a working copy of n  */
+	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
+	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
+	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
+	GMT_LONG	m;	/* Used to keep a working copy of n  */
 
 
 	/* Initialize m and max_factor  */
+#ifdef __LP64__
+	m = labs(n);
+#else
 	m = abs(n);
+#endif
 	if (m < 2) return(0);
 	max_factor = (int)floor(sqrt((double)m));
 
@@ -1968,7 +1974,7 @@ int get_prime_factors (int n, int *f)
 
 #define IABS(i)	(((i) < 0) ? -(i) : (i))
 
-int gcd_euclid (int a, int b)
+GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b)
 {
 	/* Returns the greatest common divisor of u and v by Euclid's method.
 	 * I have experimented also with Stein's method, which involves only
@@ -2019,7 +2025,7 @@ void load_parameters (struct SURFACE_INFO *C, struct SURFACE_CTRL *Ctrl)
 void *New_Surface_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SURFACE_CTRL *C;
 	
-	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SURFACE_CTRL), "New_Surface_Ctrl");
+	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SURFACE_CTRL), "New_Surface_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = 250;
@@ -2030,6 +2036,8 @@ void *New_Surface_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Surface_Ctrl (struct SURFACE_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
+	if (C->L.low) free ((void *)C->L.low);	
+	if (C->L.high) free ((void *)C->L.high);	
 	GMT_free ((void *)C);	
 }
