@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: trend2d.c,v 1.35 2007/03/12 19:52:26 remko Exp $
+ *	$Id: trend2d.c,v 1.40 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -103,13 +103,14 @@ struct	TREND2D_DATA {
 
 int main(int argc, char **argv)
 {
-	int	i, j, k, n_data, n_outputs, n_model, n_model_max, np, significant, rank, n_req;
+	int	i, j, k, n_outputs, n_model, significant, rank, n_req, np;
+	GMT_LONG	n_data;
 
 	BOOLEAN	error = FALSE, weighted_output = FALSE;
 
 	double	*gtg, *v, *gtd, *lambda, *workb, *workz, *c_model, *o_model, *w_model, *work;	/* Arrays  */
 	double	xmin, xmax, ymin, ymax, c_chisq, o_chisq, w_chisq, scale = 1.0, prob;
-	double	get_chisq(struct TREND2D_DATA *data, int n_data, int n_model);
+	double	get_chisq(struct TREND2D_DATA *data, GMT_LONG n_data, int n_model);
 
 	char format[BUFSIZ];
 
@@ -118,16 +119,16 @@ int main(int argc, char **argv)
 	struct	TREND2D_DATA *data;
 	struct TREND2D_CTRL *Ctrl;
 
-	void read_data(struct TREND2D_DATA **data, int *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp);
-	void write_output(struct TREND2D_DATA *data, int n_data, char *output_choice, int n_outputs);
-	void transform_x(struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax);
-	void untransform_x(struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax);
-	void recompute_weights(struct TREND2D_DATA *data, int n_data, double *work, double *scale);
-	void allocate_array_space(int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
+	void read_data(struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp);
+	void write_output(struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs);
+	void transform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
+	void untransform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
+	void recompute_weights(struct TREND2D_DATA *data, GMT_LONG n_data, double *work, double *scale);
+	void allocate_array_space(GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
 	void free_the_memory(double *gtg, double *v, double *gtd, double *lambda, double *workb, double *workz, double *c_model, double *o_model, double *w_model, struct TREND2D_DATA *data, double *work);
-	void calc_m_and_r(struct TREND2D_DATA *data, int n_data, double *model, int n_model, double *grow);
+	void calc_m_and_r(struct TREND2D_DATA *data, GMT_LONG n_data, double *model, int n_model, double *grow);
 	void move_model_a_to_b(double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b);
-	void load_gtg_and_gtd(struct TREND2D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp);
+	void load_gtg_and_gtd(struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, int mp);
 	void solve_system(double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
 	void *New_Trend2d_Ctrl (), Free_Trend2d_Ctrl (struct TREND2D_CTRL *C);
 	
@@ -136,7 +137,6 @@ int main(int argc, char **argv)
 	Ctrl = (struct TREND2D_CTRL *)New_Trend2d_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	n_outputs = 0;
-	n_model_max = 0;
 	sprintf (format, "%s\t", gmtdefs.d_format);
 
 	for (i = 1; i < argc; i++) {
@@ -287,7 +287,7 @@ int main(int argc, char **argv)
 #endif
 	project_info.w = 0;	project_info.e = 360.0;	/* For -L not to cause trouble in GMT_input */
 	np = Ctrl->N.value;	/* Row dimension for matrices gtg and v  */
-	allocate_array_space(np, &gtg, &v, &gtd, &lambda, &workb, &workz, &c_model, &o_model, &w_model);
+	allocate_array_space((GMT_LONG)np, &gtg, &v, &gtd, &lambda, &workb, &workz, &c_model, &o_model, &w_model);
 
 	read_data(&data, &n_data, &xmin, &xmax, &ymin, &ymax, Ctrl->W.active, &work, fp);
 
@@ -306,11 +306,11 @@ int main(int argc, char **argv)
 	transform_x(data, n_data, xmin, xmax, ymin, ymax);	/* Set domain to [-1, 1] or [-pi, pi]  */
 
 	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s:  Read %d data with X values from %.8g to %.8g\n", GMT_program, n_data, xmin, xmax);
+		fprintf(stderr,"%s:  Read %ld data with X values from %.8g to %.8g\n", GMT_program, n_data, xmin, xmax);
 		fprintf(stderr,"N_model\tRank\tChi_Squared\tSignificance\n");
 	}
 
-	sprintf (format, "%%d\t%%d\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
+	sprintf (format, "%%ld\t%%ld\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
 
 	if (Ctrl->I.active) {
 		n_model = 1;
@@ -433,10 +433,11 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp)
+void read_data (struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp)
 {
 
-	int	i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	GMT_LONG	i, n_alloc = GMT_CHUNK;
+	int n_expected_fields, n_fields;
 	char	buffer[BUFSIZ];
 	double	*in;
 
@@ -455,7 +456,7 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 			exit (EXIT_FAILURE);
 		}
 
@@ -479,9 +480,14 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 		i++;
 
 		if (i == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			*data = (struct TREND2D_DATA *) GMT_memory ((void *)*data, (size_t)n_alloc, sizeof(struct TREND2D_DATA), GMT_program);
 		}
+		if (i == INT_MAX) {
+			fprintf (stderr, "%s: ERROR: Cannot process more than %d data points\n", GMT_program, INT_MAX);
+			GMT_free ((void *)data);
+			exit (EXIT_FAILURE);
+		}
 	}
 	if (fp != GMT_stdin) GMT_fclose(fp);
 	*data = (struct TREND2D_DATA *) GMT_memory ((void *)*data, (size_t)i, sizeof(struct TREND2D_DATA), GMT_program);
@@ -489,7 +495,7 @@ void read_data (struct TREND2D_DATA **data, int *n_data, double *xmin, double *x
 	*n_data = i;
 }
 
-void allocate_array_space (int np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
+void allocate_array_space (GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model)
 {
 	*gtg = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
 	*v = (double *) GMT_memory (VNULL, (size_t)(np*np), sizeof(double), GMT_program);
@@ -502,9 +508,10 @@ void allocate_array_space (int np, double **gtg, double **v, double **gtd, doubl
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
 
-void write_output (struct TREND2D_DATA *data, int n_data, char *output_choice, int n_outputs)
+void write_output (struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs)
 {
-	int	i, j;
+	GMT_LONG	i;
+	int j;
 	double out[6];
 
 	for (i = 0; i < n_data; i++) {
@@ -549,9 +556,9 @@ void free_the_memory (double *gtg, double *v, double *gtd, double *lambda, doubl
 	GMT_free ((void *)gtg);
 }
 
-void transform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax)
+void transform_x (struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offsetx, scalex;
 	double	offsety, scaley;
 
@@ -566,9 +573,9 @@ void transform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xma
 	}
 }
 
-void untransform_x (struct TREND2D_DATA *data, int n_data, double xmin, double xmax, double ymin, double ymax)
+void untransform_x (struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax)
 {
-	int	i;
+	GMT_LONG	i;
 	double	offsetx, scalex;
 	double	offsety, scaley;
 
@@ -583,9 +590,9 @@ void untransform_x (struct TREND2D_DATA *data, int n_data, double xmin, double x
 	}
 }
 
-double get_chisq (struct TREND2D_DATA *data, int n_data, int n_model)
+double get_chisq (struct TREND2D_DATA *data, GMT_LONG n_data, int n_model)
 {
-	int	i, nu;
+	GMT_LONG	i, nu;
 	double	chi = 0.0;
 
 
@@ -602,9 +609,9 @@ double get_chisq (struct TREND2D_DATA *data, int n_data, int n_model)
 	return(chi);
 }
 
-void recompute_weights (struct TREND2D_DATA *data, int n_data, double *work, double *scale)
+void recompute_weights (struct TREND2D_DATA *data, GMT_LONG n_data, double *work, double *scale)
 {
-	int	i;
+	GMT_LONG	i;
 	double	k, ksq, rr;
 
 	/* First find median { fabs(data[].r) },
@@ -686,12 +693,13 @@ void load_g_row (double x, double y, int n, double *gr)
 	}
 }
 
-void calc_m_and_r (struct TREND2D_DATA *data, int n_data, double *model, int n_model, double *grow)
+void calc_m_and_r (struct TREND2D_DATA *data, GMT_LONG n_data, double *model, int n_model, double *grow)
 {
 	/*	model[n_model] holds solved coefficients of m_type model.
 		grow[n_model] is a vector for a row of G matrix.  */
 
-	int	i, j;
+	GMT_LONG	i;
+	int j;
 	for (i = 0; i < n_data; i++) {
 		load_g_row(data[i].x, data[i].y, n_model, grow);
 		data[i].m = 0.0;
@@ -711,13 +719,14 @@ void move_model_a_to_b (double *model_a, double *model_b, int n_model, double *c
 	*chisq_b = *chisq_a;
 }
 
-void load_gtg_and_gtd (struct TREND2D_DATA *data, int n_data, double *gtg, double *gtd, double *grow, int n_model, int mp)
+void load_gtg_and_gtd (struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, int mp)
       	    	      
       			  
    			    	/* mp is row dimension of gtg  */
 {
 
-	int	i, j, k;
+	GMT_LONG	i;
+	int j, k;
 	double	wz;
 
 	/* First zero the contents for summing:  */
@@ -755,7 +764,8 @@ void load_gtg_and_gtd (struct TREND2D_DATA *data, int n_data, double *gtg, doubl
 void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
 {
 
-	int	i, j, k, rank = 0, n, m, nrots;
+	int	i, j, k, rank = 0, nrots;
+	GMT_LONG n, m;
 	double	c_test, temp_inverse_ij;
 
 	if (n_model == 1) {
@@ -787,7 +797,7 @@ void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp,
 void *New_Trend2d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct TREND2D_CTRL *C;
 	
-	C = (struct TREND2D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TREND2D_CTRL), "New_Trend2d_Ctrl");
+	C = (struct TREND2D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct TREND2D_CTRL), "New_Trend2d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->C.value = 1.0e06;		/* Condition number for matrix solution  */	
