@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: triangulate.c,v 1.39 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: triangulate.c,v 1.53 2008/03/27 07:28:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  * using the method by Watson, D. F., ACORD: Automatic contouring of raw data,
  * Computers & Geosciences, 8, 97-101, 1982.  Optionally, the output may take
  * the form of (1) a multi-segment file with the vertex coordinates needed to
- * draw the triangles, or (2) a grdfile based on gridding the plane estimates
+ * draw the triangles, or (2) a grid file based on gridding the plane estimates
  * PS. Instead of Watson's method you may choose to link with the triangulate
  * routine written by Jonathan Shewchuk.  See the file TRIANGLE.HOWTO for
  * details.  That function is far faster than Watson's method.
@@ -67,8 +67,10 @@ struct TRIANGULATE_EDGE {
 
 int main (int argc, char **argv)
 {
-	int i, j, ij, ij1, ij2, ij3, np, k, fno, n = 0, n_alloc = 0, n_files = 0, n_edge;
-	int n_expected_fields, n_args, *link, n_fields, compare_edge(const void *p1, const void *p2);
+	int *link, compare_edge(const void *p1, const void *p2);
+	GMT_LONG ij, ij1, ij2, ij3, np, n_alloc, n = 0;
+	int i, j, k, fno, n_files = 0, n_edge;
+	int n_expected_fields, n_args, n_fields;
 	int i_min, i_max, j_min, j_max, p;
 
 	BOOLEAN error = FALSE, map_them = FALSE;
@@ -81,6 +83,11 @@ int main (int argc, char **argv)
 	float *grd;
 
 	char line[BUFSIZ];
+#ifdef TRIANGLE_D
+	char *algorithm = "Shewchuk";
+#else
+	char *algorithm = "Watson";
+#endif
 
 	FILE *fp = NULL;
 
@@ -163,7 +170,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "triangulate %s - Optimal (Delaunay) triangulation of Cartesian xyz-data\n\n", GMT_VERSION);
+		fprintf (stderr, "triangulate %s - Optimal (Delaunay) triangulation of Cartesian xyz-data [%s]\n\n", GMT_VERSION, algorithm);
 		fprintf (stderr, "usage: triangulate <infiles> [-Dx|y] [-E<empty>] [-F] [-G<grdfile>]\n");
 		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n", GMT_H_OPT, GMT_I_OPT, GMT_J_OPT, GMT_M_OPT);
 		fprintf (stderr, "\t[%s] [-V] [%s] [%s]\n\t[%s]\n\n", GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
@@ -175,7 +182,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Dx or -Dy takes derivative in that direction (only with -G) [Default is z value].\n");
 		fprintf (stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
 		fprintf (stderr, "\t-F Force pixel registration (only with -G) [Default is gridline registration]\n");
-		fprintf (stderr, "\t-G Grid data. Give name of output gridfile and specify -R -I\n");
+		fprintf (stderr, "\t-G Grid data. Give name of output grid file and specify -R -I\n");
 		GMT_explain_option ('H');
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('J');   
@@ -241,7 +248,7 @@ int main (int argc, char **argv)
 		GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 	}
 
-	if (project_info.region_supplied && project_info.projection > 0) { /* Gave -R -J */
+	if (project_info.region_supplied && project_info.projection != GMT_NO_PROJ) { /* Gave -R -J */
 		map_them = TRUE;
 		GMT_err_fail (GMT_map_setup (header.x_min, header.x_max, header.y_min, header.y_max), "");
 	}
@@ -283,7 +290,7 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
@@ -293,11 +300,18 @@ int main (int argc, char **argv)
 			n++;
 
 			if (n == n_alloc) {	/* Get more memory */
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
 				yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 				if (do_grid) zz = (double *) GMT_memory ((void *)zz, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
+			if (n == INT_MAX) {
+				fprintf (stderr, "%s: ERROR: Cannot triangulate more than %d points\n", GMT_program, INT_MAX);
+				GMT_free ((void *)xx);
+				GMT_free ((void *)yy);
+				if (do_grid) GMT_free ((void *)zz);
+				exit (EXIT_FAILURE);
+			}
 		}
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -317,7 +331,7 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Do Delaunay optimal triangulation on projected coordinates\n", GMT_program);
 
-		np = GMT_delaunay (xxp, yyp, n, &link);
+		np = GMT_delaunay (xxp, yyp, (int)n, &link);
 
 		GMT_free ((void *)xxp);
 		GMT_free ((void *)yyp);
@@ -325,10 +339,10 @@ int main (int argc, char **argv)
 	else {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Do Delaunay optimal triangulation on given coordinates\n", GMT_program);
 
-		np = GMT_delaunay (xx, yy, n, &link);
+		np = GMT_delaunay (xx, yy, (int)n, &link);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: %d Delaunay triangles found\n", GMT_program, np);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: %ld Delaunay triangles found\n", GMT_program, np);
 
 	if (do_grid) {
 
@@ -382,7 +396,7 @@ int main (int argc, char **argv)
 				for (i = i_min; i <= i_max; i++, p++) {
 					xp = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
 
-					if (!GMT_non_zero_winding (xp, yp, vx, vy, 4)) continue;	/* Outside */
+					if (!GMT_non_zero_winding (xp, yp, vx, vy, (GMT_LONG)4)) continue;	/* Outside */
 
 					if (Ctrl->D.dir == GMT_X)
 						grd[p] = (float)a;
@@ -394,6 +408,8 @@ int main (int argc, char **argv)
 			}
 		}
 		GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
+		GMT_free ((void *)zz);
+		GMT_free ((void *)grd);
 	}
 	if (GMT_io.multi_segments[GMT_OUT]) {	/* Must find unique edges to output only once */
 		n_edge = 3 * np;
@@ -430,7 +446,18 @@ int main (int argc, char **argv)
 
 	GMT_free ((void *) xx);
 	GMT_free ((void *) yy);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	/* Shewchuck's function allocated the memory separately */
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
+#endif
 	GMT_free ((void *) link);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
+#endif
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
 
@@ -464,7 +491,7 @@ int compare_edge (const void *p1, const void *p2)
 void *New_Triangulate_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct TRIANGULATE_CTRL *C;
 	
-	C = (struct TRIANGULATE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TRIANGULATE_CTRL), "New_Triangulate_Ctrl");
+	C = (struct TRIANGULATE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct TRIANGULATE_CTRL), "New_Triangulate_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.dir = -1;	/* No derivatives */
@@ -472,6 +499,6 @@ void *New_Triangulate_Ctrl () {	/* Allocate and initialize a new control structu
 }
 
 void Free_Triangulate_Ctrl (struct TRIANGULATE_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
