@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys.c,v 1.64 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: x2sys.c,v 1.85 2008/04/17 02:49:32 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -64,12 +64,13 @@ char *X2SYS_HOME;
 char *X2SYS_program;
 
 char *x2sys_xover_format = "%9.5lf %9.5lf %10.1lf %10.1lf %9.2lf %9.2lf %9.2lf %8.1lf %8.1lf %8.1lf %5.1lf %5.1lf\n";
-char *x2sys_xover_header = "%s %d %s %d\n";
-char *x2sys_header = "> %s %d %s %d\n";
+char *x2sys_xover_header = "%s %ld %s %ld\n";
+char *x2sys_header = "> %s %ld %s %ld\n";
 struct MGD77_CONTROL M;
 
 void x2sys_set_home (void);
 int x2sys_record_length (struct X2SYS_INFO *s);
+int get_first_year (double t);
 
 #define MAX_DATA_PATHS 32
 char *x2sys_datadir[MAX_DATA_PATHS];	/* Directories where track data may live */
@@ -138,7 +139,8 @@ void x2sys_skip_header (FILE *fp, struct X2SYS_INFO *s)
 
 int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_IO *G)
 {
-	int j, k, i, n_read = 0, pos;
+	int j, k, i, n_read = 0;
+	int pos;
 	BOOLEAN error = FALSE;
 	char line[BUFSIZ], buffer[GMT_TEXT_LEN], p[BUFSIZ], c;
 	unsigned char u;
@@ -163,7 +165,7 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_
 					}
 					GMT_chop (line);	/* Remove trailing CR or LF */
 				}
-				strncpy (buffer, &line[s->info[j].start_col], s->info[j].n_cols);
+				strncpy (buffer, &line[s->info[j].start_col], (size_t)s->info[j].n_cols);
 				buffer[s->info[j].n_cols] = 0;
 				GMT_scanf (buffer, G->in_col_type[j], &data[j]);
 				break;
@@ -239,19 +241,20 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_
 	return ((error || n_read != s->n_fields) ? -1 : 0);
 }
  
-int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
+int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec)
 {
 	/* Reads the entire contents of the file given and returns the
 	 * number of data records.  The data matrix is return in the
 	 * pointer data.
 	 */
 
-	int i, j;
+	GMT_LONG i, j;
 	size_t n_alloc;
 	FILE *fp;
 	double **z, *rec;
 	char path[BUFSIZ];
 
+	strcpy (s->path, fname);
  	if (n_x2sys_paths) {
   		if (x2sys_get_data_path (path, fname, s->suffix)) {
    			fprintf (stderr, "x2sys_read_file : Cannot find track %s\n", fname);
@@ -261,6 +264,7 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
    			fprintf (stderr, "x2sys_read_file : Cannot open file %s\n", path);
      			return (-1);
   		}
+		strcpy (s->path, path);
 	}
 	else if ((fp = fopen (fname, G->r_mode)) == NULL) {
 		fprintf (stderr, "x2sys_read_file: Could not open %s\n", fname);
@@ -269,12 +273,12 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 
 	n_alloc = GMT_CHUNK;
 
-	rec = (double *) GMT_memory (VNULL, s->n_fields, sizeof (double), "x2sys_read_file");
-	z = (double **) GMT_memory (VNULL, s->n_fields, sizeof (double *), "x2sys_read_file");
+	rec = (double *) GMT_memory (VNULL, (size_t)s->n_fields, sizeof (double), "x2sys_read_file");
+	z = (double **) GMT_memory (VNULL, (size_t)s->n_fields, sizeof (double *), "x2sys_read_file");
 	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory (VNULL, n_alloc, sizeof (double), "x2sys_read_file");
-	p->ms_rec = (int *) GMT_memory (VNULL, n_alloc, sizeof (int), "x2sys_read_file");
+	p->ms_rec = (GMT_LONG *) GMT_memory (VNULL, n_alloc, sizeof (GMT_LONG), "x2sys_read_file");
 	x2sys_skip_header (fp, s);
-	if (s->multi_segment) p->n_segments = -1;	/* So that first increment sets it to 0 */
+	p->n_segments = (s->multi_segment) ? -1 : 0;	/* So that first increment sets it to 0 */
 
 	j = 0;
 	while (!x2sys_read_record (fp, rec, s, G)) {	/* Gets the next data record */
@@ -282,32 +286,34 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 		if (s->multi_segment && s->ms_next) p->n_segments++;
 		p->ms_rec[j] = p->n_segments;
 		j++;
-		if (j == (int)n_alloc) {	/* Get more */
-			n_alloc += GMT_CHUNK;
+		if (j == (GMT_LONG)n_alloc) {	/* Get more */
+			n_alloc <<= 1;
 			for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], n_alloc, sizeof (double), "x2sys_read_file");
-			p->ms_rec = (int *) GMT_memory ((void *)p->ms_rec, n_alloc, sizeof (int), "x2sys_read_file");
+			p->ms_rec = (GMT_LONG *) GMT_memory ((void *)p->ms_rec, n_alloc, sizeof (GMT_LONG), "x2sys_read_file");
 		}
 	}
 
 	fclose (fp);
+	GMT_free ((void *)rec);
 	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)j, sizeof (double), "x2sys_read_file");
-	p->ms_rec = (int *) GMT_memory ((void *)p->ms_rec, (size_t)j, sizeof (int), "x2sys_read_file");
+	p->ms_rec = (GMT_LONG *) GMT_memory ((void *)p->ms_rec, (size_t)j, sizeof (GMT_LONG), "x2sys_read_file");
 
 	*data = z;
 
 	p->n_rows = j;
 	p->year = 0;
-	strncpy (p->name, fname, 32);
+	strncpy (p->name, fname, (size_t)32);
+	*n_rec = p->n_rows;
 
-	return (j);
+	return (X2SYS_NOERROR);
 }
 
-int x2sys_initialize (char *fname, struct GMT_IO *G,  struct X2SYS_INFO *init)
+int x2sys_initialize (char *fname, struct GMT_IO *G,  struct X2SYS_INFO **I)
 {
 	/* Reads the format definition file and sets all information variables */
 
 	int i = 0, c;
-	size_t n_alloc = 10;
+	size_t n_alloc = GMT_TINY_CHUNK;
 	FILE *fp;
 	struct X2SYS_INFO *X;
 	char line[BUFSIZ], cardcol[80], yes_no;
@@ -332,23 +338,23 @@ int x2sys_initialize (char *fname, struct GMT_IO *G,  struct X2SYS_INFO *init)
 		X->read_file = (PFI) x2sys_read_ncfile;
 		X->geographic = TRUE;
 		X->geodetic = 0;
-		MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+		MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	}
 	else if (!strcmp (fname, "mgd77")) {
 		X->read_file = (PFI) x2sys_read_mgd77file;
 		X->geographic = TRUE;
 		X->geodetic = 0;
-		MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+		MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	}
 	else
 		X->read_file = (PFI) x2sys_read_file;
 	while (fgets (line, BUFSIZ, fp)) {
 		if (line[0] == '\0') continue;
 		if (line[0] == '#') {
-			if (!strncmp (line, "#SKIP ", 6)) X->skip = atoi (&line[6]);
-			if (!strncmp (line, "#BINARY ", 7)) X->ascii_in = FALSE;
-			if (!strncmp (line, "#GEO ", 3)) X->geographic = TRUE;
-			if (!strncmp (line, "#MULTISEG ", 9)) {
+			if (!strncmp (line, "#SKIP ", (size_t)6)) X->skip = atoi (&line[6]);
+			if (!strncmp (line, "#BINARY ", (size_t)7)) X->ascii_in = FALSE;
+			if (!strncmp (line, "#GEO ", (size_t)3)) X->geographic = TRUE;
+			if (!strncmp (line, "#MULTISEG ", (size_t)9)) {
 				X->multi_segment = TRUE;
 				sscanf (line, "%*s %c", &X->ms_flag);
 			}
@@ -371,14 +377,14 @@ int x2sys_initialize (char *fname, struct GMT_IO *G,  struct X2SYS_INFO *init)
 		if (!strcmp (X->info[i].name, "t") || !strcmp (X->info[i].name, "time")) X->t_col = i;
 		i++;
 		if (i == (int)n_alloc) {
-			n_alloc += 10;
+			n_alloc <<= 1;
 			X->info = (struct X2SYS_DATA_INFO *) GMT_memory ((void *)X->info, n_alloc, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
 		}
 
 	}
 	fclose (fp);
 
-	if (i < (int)n_alloc) X->info = (struct X2SYS_DATA_INFO *) GMT_memory ((void *)X->info, i, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
+	if (i < (int)n_alloc) X->info = (struct X2SYS_DATA_INFO *) GMT_memory ((void *)X->info, (size_t)i, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
 	X->n_fields = X->n_out_columns = i;
 
 	if (!X->ascii_in) {	/* Binary mode needed */
@@ -396,10 +402,22 @@ int x2sys_initialize (char *fname, struct GMT_IO *G,  struct X2SYS_INFO *init)
 	X->n_data_cols = x2sys_n_data_cols (X);
 	X->rec_size = (8 + X->n_data_cols) * sizeof (double);
 
-	init = X;
+	*I = X;
 	return (X2SYS_NOERROR);
 }
 
+void x2sys_end (struct X2SYS_INFO *X)
+{	/* Free allcoated memory */
+	int id;
+	if (X2SYS_HOME) GMT_free ((void *)X2SYS_HOME);
+	if (!X) return;
+	if (X->out_order) GMT_free ((void *)X->out_order);
+	if (X->use_column) GMT_free ((void *)X->use_column);
+	x2sys_free_info (X);
+	for (id = 0; id < n_x2sys_paths; id++) GMT_free  ((void *)x2sys_datadir[id]);
+	MGD77_end (&M);
+}
+
 int x2sys_record_length (struct X2SYS_INFO *s)
 {
 	int i, rec_length = 0;
@@ -432,12 +450,11 @@ int x2sys_n_data_cols (struct X2SYS_INFO *s)
 {
 	int i, n = 0;
 
-	for (i = 0; i < s->n_out_columns; i++) {
+	for (i = 0; i < s->n_out_columns; i++) {	/* Loop over all possible fields in this data set */
 		if (i == s->x_col) continue;
 		if (i == s->y_col) continue;
 		if (i == s->t_col) continue;
-		if (!s->use_column[i]) continue;
-		n++;
+		n++;	/* Only count data columns */
 	}
 
 	return (n);
@@ -447,14 +464,15 @@ int x2sys_pick_fields (char *string, struct X2SYS_INFO *s)
 {
 	/* Scan the -Fstring and select which columns to use and which order
 	 * they should appear on output.  Default is all columns and the same
-	 * order as on input
+	 * order as on input.  Once this is set you can loop through i = 0:n_out_columns
+	 * and use out_order[i] to get the original column number.
 	 */
 
 	char line[BUFSIZ], p[BUFSIZ];
 	int i = 0, j, pos = 0;
 
-	strncpy (s->fflags, string, BUFSIZ);
-	strncpy (line, string, BUFSIZ);	/* Make copy for later use */
+	strncpy (s->fflags, string, (size_t)BUFSIZ);
+	strncpy (line, string, (size_t)BUFSIZ);	/* Make copy for later use */
 	memset ((void *)s->use_column, 0, (size_t)(s->n_fields * sizeof (int)));
 
 	s->x_col = s->y_col = s->t_col = -1;	/* Need to reset this to match data order */
@@ -488,11 +506,11 @@ void x2sys_set_home (void)
 	if (X2SYS_HOME) return;	/* Already set elsewhere */
 
 	if ((this = getenv ("X2SYS_HOME")) != CNULL) {	/* Set user's default path */
-		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 1), 1, "x2sys_set_home");
+		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 1), (size_t)1, "x2sys_set_home");
 		strcpy (X2SYS_HOME, this);
 	}
 	else {
-		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (GMT_SHAREDIR) + 7), 1, "x2sys_set_home");
+		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (GMT_SHAREDIR) + 7), (size_t)1, "x2sys_set_home");
 		sprintf (X2SYS_HOME, "%s%cx2sys", GMT_SHAREDIR, DIR_DELIM);
 	}
 }
@@ -511,9 +529,9 @@ void x2sys_free_data (double **data, int n)
 	GMT_free ((void *)data);
 }
 
-double *x2sys_dummytimes (int n)
+double *x2sys_dummytimes (GMT_LONG n)
 {
-	int i;
+	GMT_LONG i;
 	double *t;
 
 	/* Make monotonically increasing dummy time sequence */
@@ -525,7 +543,7 @@ double *x2sys_dummytimes (int n)
 	return (t);
 }
 
-int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec)
+int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec)
 {
 	/* Reads the entire contents of the file given and returns the
 	 * number of data records.  The data matrix is return in the
@@ -533,7 +551,8 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 	 * MGG format from old Lamont by Wessel and Smith.
 	 */
 
-	int i, j;
+	int year, n_records;
+	GMT_LONG i, j;
 	char gmtfile[BUFSIZ], name[80];
 	FILE *fp;
 	double **z;
@@ -547,22 +566,24 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 		s->flags |= 1;
 	}
 
-	strncpy (name, fname, 80);
+	strncpy (name, fname, (size_t)80);
 	if (strstr (fname, ".gmt"))	/* Name includes .gmt suffix */
 		name[strlen(fname)-4] = 0;
 
   	if (gmtmggpath_func (gmtfile, name)) return (GMT_GRDIO_FILE_NOT_FOUND);
-
+	strcpy (s->path, gmtfile);
 	if ((fp = fopen (gmtfile, "rb")) == NULL) return (GMT_GRDIO_OPEN_FAILED);
 
-	if (fread ((void *)&(p->year), sizeof (int), (size_t)1, fp) != 1) {
+	if (fread ((void *)&year, sizeof (int), (size_t)1, fp) != 1) {
 		fprintf (stderr, "x2sys_read_gmtfile: Could not read leg year from %s\n", gmtfile);
 		return (-1);
 	}
-	if (fread ((void *)&(p->n_rows), sizeof (int), (size_t)1, fp) != 1) {
+	p->year = year;
+	if (fread ((void *)&n_records, sizeof (int), (size_t)1, fp) != 1) {
 		fprintf (stderr, "x2sys_read_gmtfile: Could not read n_records from %s\n", gmtfile);
 		return (GMT_GRDIO_READ_FAILED);
 	}
+	p->n_rows = n_records;
 	memset ((void *)p->name, 0, (size_t)32);
 
 	if (fread ((void *)p->name, (size_t)10, sizeof (char), fp) != 1) {
@@ -576,7 +597,7 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 	for (j = 0; j < p->n_rows; j++) {
 
 		if (fread ((void *)&record, (size_t)18, (size_t)1, fp) != 1) {
-			fprintf (stderr, "x2sys_read_gmtfile: Could not read record %d from %s\n", j, gmtfile);
+			fprintf (stderr, "x2sys_read_gmtfile: Could not read record %ld from %s\n", j, gmtfile);
 			return (GMT_GRDIO_READ_FAILED);
 		}
 
@@ -600,9 +621,10 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 	return (X2SYS_NOERROR);
 }
 
-int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec)
+int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec)
 {
-	int i, j, col[MGD77_N_DATA_EXTENDED], n_alloc = GMT_CHUNK;
+	GMT_LONG i, j;
+	int col[MGD77_N_DATA_EXTENDED], n_alloc = GMT_CHUNK;
 	char path[BUFSIZ], *tvals[MGD77_N_STRING_FIELDS];
 	double **z, dvals[MGD77_N_DATA_EXTENDED];
 	struct MGD77_HEADER H;
@@ -610,7 +632,7 @@ int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, str
 	double NaN;
 
 	GMT_make_dnan(NaN);
-	MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 
   	if (n_x2sys_paths) {
   		if (x2sys_get_data_path (path, fname, s->suffix)) return (GMT_GRDIO_FILE_NOT_FOUND);
@@ -618,33 +640,38 @@ int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, str
 	}
 	else if (MGD77_Open_File (fname, &M, 0))
 		return (GMT_GRDIO_FILE_NOT_FOUND);
-
+	strcpy (s->path, M.path);
+	
 	if (MGD77_Read_Header_Record (fname, &M, &H)) {
 		fprintf (stderr, "%s: Error reading header sequence for cruise %s\n", X2SYS_program, fname);
 		return (GMT_GRDIO_READ_FAILED);
 	}
 
-	for (i = 0; i < MGD77_N_STRING_FIELDS; i++) tvals[i] = (char *) GMT_memory (VNULL, 9, sizeof (char), "x2sys_read_mgd77file");
+	for (i = 0; i < MGD77_N_STRING_FIELDS; i++) tvals[i] = (char *) GMT_memory (VNULL, (size_t)9, sizeof (char), "x2sys_read_mgd77file");
 	z = (double **) GMT_memory (VNULL, (size_t)s->n_fields, sizeof (double *), "x2sys_read_mgd77file");
 	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
 	for (i = 0; i < s->n_out_columns; i++) {
 		col[i] = MGD77_Get_Column (s->info[s->out_order[i]].name, &M);
 	}
 
+	p->year = 0;
 	j = 0;
 	while (!MGD77_Read_Data_Record (&M, &H, dvals, tvals)) {		/* While able to read a data record */
 		GMT_lon_range_adjust (s->geodetic, &dvals[MGD77_LONGITUDE]);
 		for (i = 0; i < s->n_out_columns; i++) z[i][j] = dvals[col[i]];
-
+		if (p->year == 0 && !GMT_is_fnan (dvals[0])) p->year = get_first_year (dvals[0]);
 		j++;
 		if (j == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
 		}
 	}
 	MGD77_Close_File (&M);
+	MGD77_end (&M);
+	MGD77_free_plain_mgd77 (&H);
+	GMT_free ((void *)H.mgd77);
 
-	strncpy (p->name, fname, 32);
+	strncpy (p->name, fname, (size_t)32);
 	p->n_rows = j;
 	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)p->n_rows, sizeof (double), "x2sys_read_mgd77file");
 
@@ -658,14 +685,26 @@ int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, str
 	return (X2SYS_NOERROR);
 }
 
-int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec)
+int get_first_year (double t)
+{
+	/* obtain yyyy/mm/dd and return year */
+	GMT_cal_rd rd;
+	double s;
+	struct GMT_gcal CAL;
+	GMT_dt2rdc (t, &rd, &s);
+	GMT_gcal_from_rd (rd, &CAL);
+	return (CAL.year);
+}
+
+int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec)
 {
 	int i;
+	char path[BUFSIZ];
 	double **z;
 	struct MGD77_DATASET *S;
 	struct MGD77_CONTROL M;
 
-	MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	M.format  = MGD77_FORMAT_CDF;		/* Set input file's format to netCDF */
 	for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (M.format == i) ? TRUE : FALSE;	/* Only allow the specified input format */
 
@@ -674,7 +713,14 @@ int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct
 	
 	S = MGD77_Create_Dataset ();	/* Get data structure w/header */
 
-	if (MGD77_Open_File (fname, &M, 0)) return (GMT_GRDIO_FILE_NOT_FOUND);
+  	if (n_x2sys_paths) {
+  		if (x2sys_get_data_path (path, fname, s->suffix)) return (GMT_GRDIO_FILE_NOT_FOUND);
+		if (MGD77_Open_File (path, &M, 0)) return (GMT_GRDIO_OPEN_FAILED);
+	}
+	else if (MGD77_Open_File (fname, &M, 0))
+		return (GMT_GRDIO_FILE_NOT_FOUND);
+	strcpy (s->path, M.path);
+
 	if (MGD77_Read_Header_Record (fname, &M, &S->H)) {	/* Returns info on all columns */
 		fprintf (stderr, "x2sys_read_nc77file: Error reading header sequence for cruise %s\n", fname);
      		return (GMT_GRDIO_READ_FAILED);
@@ -689,12 +735,14 @@ int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct
 	z = (double **) GMT_memory (VNULL, (size_t)M.n_out_columns, sizeof (double *), "x2sys_read_nc77file");
 	for (i = 0; i < M.n_out_columns; i++) z[i] = (double *) S->values[i];
 
-	strncpy (p->name, fname, 32);
+	strncpy (p->name, fname, (size_t)32);
 	p->n_rows = S->H.n_records;
 	p->ms_rec = NULL;
 	p->n_segments = 0;
+	p->year = S->H.meta.Departure[0];
 	for (i = 0; i < MGD77_N_SETS; i++) if (S->flags[i]) GMT_free ((void *)S->flags[i]);
 	GMT_free ((void *)S->H.mgd77);
+	MGD77_end (&M);
 
 	*data = z;
 	*n_rec = p->n_rows;
@@ -719,11 +767,11 @@ int x2sys_read_list (char *file, char ***list, int *nf)
 	FILE *fp;
 
 	if ((fp = x2sys_fopen (file, "r")) == NULL) {
-  		fprintf (stderr, "x2sys_initialize : Cannot find track list file %s in either current or X2SYS_HOME directories\n", line);
+  		fprintf (stderr, "x2sys_read_list : Cannot find track list file %s in either current or X2SYS_HOME directories\n", line);
 		return (GMT_GRDIO_FILE_NOT_FOUND);
 	}
 	
-	p = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), "x2sys_read_list");
+	p = (char **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char *), "x2sys_read_list");
 
 	while (fgets (line, BUFSIZ, fp)) {
 		GMT_chop (line);	/* Remove trailing CR or LF */
@@ -732,13 +780,13 @@ int x2sys_read_list (char *file, char ***list, int *nf)
 		strcpy (p[n], name);
 		n++;
 		if (n == n_alloc) {
-			n_alloc += GMT_CHUNK;
-			p = (char **) GMT_memory ((void *)p, n_alloc, sizeof (char *), "x2sys_read_list");
+			n_alloc <<= 1;
+			p = (char **) GMT_memory ((void *)p, (size_t)n_alloc, sizeof (char *), "x2sys_read_list");
 		}
 	}
 	fclose (fp);
 
-	p = (char **) GMT_memory ((void *)p, n, sizeof (char *), "x2sys_read_list");
+	p = (char **) GMT_memory ((void *)p, (size_t)n, sizeof (char *), "x2sys_read_list");
 
 	*list = p;
 	*nf = n;
@@ -746,14 +794,15 @@ int x2sys_read_list (char *file, char ***list, int *nf)
 	return (X2SYS_NOERROR);
 }
 
-int x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, struct GMT_IO *G)
+int x2sys_set_system (char *TAG, struct X2SYS_INFO **S, struct X2SYS_BIX *B, struct GMT_IO *G)
 {
 	char tag_file[BUFSIZ], line[BUFSIZ], p[BUFSIZ], sfile[BUFSIZ], suffix[16];
 	int geodetic = 0, pos = 0, n;
 	double dist;
 	BOOLEAN geographic = FALSE;
 	FILE *fp;
-
+	struct X2SYS_INFO *s;
+	
 	if (!TAG) return (X2SYS_TAG_NOT_SET);
 	
 	x2sys_set_home ();
@@ -773,8 +822,6 @@ int x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, str
 	while (fgets (line, BUFSIZ, fp) && line[0] == '#');	/* Skip comment records */
 	GMT_chop (line);	/* Remove trailing CR or LF */
 
-	GMT->R.processed = FALSE;	/* In case -R has been processed before */
-	
 	while ((GMT_strtok (line, " \t", &pos, p))) {	/* Process the -D -I -R -G -W arguments from the header */
 		if (p[0] == '-') {
 			switch (p[1]) {
@@ -831,7 +878,7 @@ int x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, str
 	}
 	x2sys_err_pass (x2sys_fclose (tag_file, fp), tag_file);
 	
-	x2sys_err_pass (x2sys_initialize (sfile, G, *s), sfile);	/* Initialize X2SYS and info structure */
+	x2sys_err_pass (x2sys_initialize (sfile, G, &s), sfile);	/* Initialize X2SYS and info structure */
 
 	if (geographic) {
 		if (geodetic == 0 && (B->x_min < 0 || B->x_max < 0)) {
@@ -842,21 +889,22 @@ int x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, str
 			fprintf (stderr, "%s: Your -R and -G settings are contradicting each other!\n", X2SYS_program);
 			return (X2SYS_CONFLICTING_ARGS);
 		}
-		(*s)->geographic = TRUE;
-		(*s)->geodetic = geodetic;	/* Override setting */
-		if (fabs (fabs (B->x_max - B->x_min) - 360.0) <= GMT_CONV_LIMIT) B->periodic = 1;
+		s->geographic = TRUE;
+		s->geodetic = geodetic;	/* Override setting */
+		if (GMT_360_RANGE (B->x_max, B->x_min)) B->periodic = 1;
 	}
 	if (GMT_io.multi_segments[GMT_IN]) {	/* Files have multiple segments; make sure this is also set in s */
-		(*s)->multi_segment = TRUE;
-		(*s)->ms_flag = GMT_io.EOF_flag[GMT_IN];
+		s->multi_segment = TRUE;
+		s->ms_flag = GMT_io.EOF_flag[GMT_IN];
 	}
 	if (suffix[0])
-		strcpy ((*s)->suffix, suffix);
+		strcpy (s->suffix, suffix);
 	else
-		strcpy ((*s)->suffix, sfile);
+		strcpy (s->suffix, sfile);
 		
 	x2sys_path_init (TAG);		/* Prepare directory paths to data */
 	
+	*S = s;
 	return (X2SYS_NOERROR);
 }
 
@@ -906,6 +954,9 @@ int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode, int *ID)
 
 	if ((ftrack = fopen (track_path, "r")) == NULL) return (GMT_GRDIO_FILE_NOT_FOUND);
 
+#ifdef DEBUG
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
 	if (mode == 1)
 		B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
 	else
@@ -934,13 +985,16 @@ int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode, int *ID)
 	GMT_fclose (ftrack);
 	last_id++;
 	if (mode == 1) B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory ((void *)B->head, (size_t)last_id, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
 
 	*ID = last_id;
 	
 	return (X2SYS_NOERROR);
 }
 
-int x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B)
+int x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B, BOOLEAN swap)
 {
 	char index_file[BUFSIZ], index_path[BUFSIZ];
 	FILE *fbin;
@@ -953,20 +1007,33 @@ int x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B)
 		fprintf (stderr,"%s: Could not open %s\n", X2SYS_program, index_path);
 		return (GMT_GRDIO_OPEN_FAILED);
 	}
-
+#ifdef DEBUG
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
 	B->base = (struct X2SYS_BIX_DATABASE *) GMT_memory (VNULL, (size_t)B->nm_bin, sizeof (struct X2SYS_BIX_DATABASE), X2SYS_program);
 
 	while ((fread ((void *)(&index), sizeof (int), (size_t)1, fbin)) == 1) {
 		fread ((void *)(&no_of_tracks), sizeof (int), (size_t)1, fbin);
+		if (swap) {
+			index = GMT_swab4 (index);
+			no_of_tracks = GMT_swab4 (no_of_tracks);
+		}
 		B->base[index].first_track = B->base[index].last_track = x2sys_bix_make_track (0, 0);
 		for (i = 0; i < no_of_tracks; i++) {
 			fread ((void *)(&id), sizeof (int), (size_t)1, fbin);
 			fread ((void *)(&flag), sizeof (int), (size_t)1, fbin);
+			if (swap) {
+				id = GMT_swab4 (id);
+				flag = GMT_swab4 (flag);
+			}
 			B->base[index].last_track->next_track = x2sys_bix_make_track (id, flag);
 			B->base[index].last_track = B->base[index].last_track->next_track;
 			B->base[index].n_tracks++;
 		}
 	}
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
 	GMT_fclose (fbin);
 	return (X2SYS_NOERROR);
 }
@@ -991,7 +1058,7 @@ int x2sys_bix_get_ij (double x, double y, int *i, int *j, struct X2SYS_BIX *B, i
 	}
 	index = (*j) * B->nx_bin + (*i);
 	if (index < 0 || index >= B->nm_bin) {
-		fprintf (stderr, "x2sys_binlist: Index (%d) outside range implied by -R -I! [0-%d>\n", index, B->nm_bin);
+		fprintf (stderr, "x2sys_binlist: Index (%d) outside range implied by -R -I! [0-%ld>\n", index, B->nm_bin);
 		return (X2SYS_BIX_BAD_IJ);
 	}
 
@@ -1130,5 +1197,5 @@ void x2sys_err_fail (int err, char *file)
 		fprintf (stderr, "%s: %s [%s]\n", X2SYS_program, x2sys_strerror(err), file);
 	else
 		fprintf (stderr, "%s: %s\n", X2SYS_program, x2sys_strerror(err));
-	exit (EXIT_FAILURE);
+	GMT_exit (EXIT_FAILURE);
 }
