@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys.h,v 1.25 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: x2sys.h,v 1.33 2008/04/12 01:38:49 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -139,6 +139,7 @@ struct X2SYS_INFO {
 	char ms_flag;			/* Multi-segment header flag */
 	char suffix[16];		/* Suffix for these data files */
 	char fflags[BUFSIZ];		/* Text copy of selected columns */
+	char path[BUFSIZ];		/* Full path to current data file */
 	struct X2SYS_DATA_INFO *info;	/* Array of info for each data field */
 };
 
@@ -160,9 +161,9 @@ struct X2SYS_DATA_INFO {
 struct X2SYS_FILE_INFO {
 	/* Information for a particular data file */
 	int year;		/* Starting year for this leg */
-	int n_rows;		/* Number of rows */
-	int n_segments;		/* Number of segments in this file */
-	int *ms_rec;		/* Pointer to array with start record for each segment */
+	GMT_LONG n_rows;		/* Number of rows */
+	GMT_LONG n_segments;		/* Number of segments in this file */
+	GMT_LONG *ms_rec;		/* Pointer to array with start record for each segment */
 	char name[32];		/* Name of cruise or agency */
 };
 
@@ -178,7 +179,7 @@ struct X2SYS_BIX {
 	double dist_gap;	/* We have a data-gap if two records differ by this amount in distance [if there is no time column] */
 	int nx_bin;		/* Number of x bins */
 	int ny_bin;		/* Number of y bins */
-	int nm_bin;		/* Total number of bins */
+	GMT_LONG nm_bin;		/* Total number of bins */
 	int periodic;		/* 1 if x is periodic */
 	unsigned int *binflag;	/* The bin array */
 	struct X2SYS_BIX_DATABASE *base;
@@ -222,15 +223,15 @@ extern int x2sys_access (char *fname, int mode);
 extern void x2sys_path (char *fname, char *path);
 
 extern int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_IO *G);
-extern int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G);
-extern int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec);
-extern int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec);
-extern int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, int *n_rec);
+extern int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec);
+extern int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec);
+extern int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec);
+extern int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G, GMT_LONG *n_rec);
 extern int x2sys_xover_output (FILE *fp, int n, double out[]);
 extern int x2sys_n_data_cols (struct X2SYS_INFO *s);
 extern int x2sys_read_list (char *file, char ***list, int *n);
 
-extern double *x2sys_dummytimes (int n);
+extern double *x2sys_dummytimes (GMT_LONG n);
 
 extern void x2sys_skip_header (FILE *fp, struct X2SYS_INFO *s);
 extern int x2sys_fclose (char *fname, FILE *fp);
@@ -238,14 +239,15 @@ extern void x2sys_free_info (struct X2SYS_INFO *s);
 extern void x2sys_free_data (double **data, int n);
 extern int x2sys_pick_fields (char *string, struct X2SYS_INFO *s);
 
-extern int x2sys_initialize (char *fname, struct GMT_IO *G, struct X2SYS_INFO *I);
+extern int x2sys_initialize (char *fname, struct GMT_IO *G, struct X2SYS_INFO **I);
+extern void x2sys_end (struct X2SYS_INFO *X);
 
 extern int x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, struct GMT_IO *G);
 extern void x2sys_bix_init (struct X2SYS_BIX *B, BOOLEAN alloc);
 extern struct X2SYS_BIX_TRACK_INFO *x2sys_bix_make_entry (char *name, int id_no, int flag);
 extern struct X2SYS_BIX_TRACK *x2sys_bix_make_track (int id, int flag);
 extern int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode, int *ID);
-extern int x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B);
+extern int x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B, BOOLEAN swap);
 extern int x2sys_bix_get_ij (double x, double y, int *i, int *j, struct X2SYS_BIX *B, int *ID);
 
 extern void x2sys_path_init (char *TAG);
