@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_datalist.c,v 1.23 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: x2sys_datalist.c,v 1.27 2008/03/24 08:58:33 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,8 @@ int main (int argc, char **argv)
 {
 	char *sfile, *def = "x2sys", *fflags = CNULL, *TAG = CNULL;
 
-	int i, j, k, n_data_col_out = 0, bad;
+	GMT_LONG i, j, k, bad;
+	int n_data_col_out = 0;
 
 	BOOLEAN error = FALSE, suppress = FALSE, t_given;
 
@@ -171,10 +172,8 @@ int main (int argc, char **argv)
 			}
 			else {	/* use the specified formats */
 				for (k = 0; k < s->n_out_columns; k++) {
-					if (s->out_order[i] == s->t_col && s->info[s->out_order[i]].format[0] == '-')
-						GMT_ascii_output_one (GMT_stdout, GMT_dt_from_usert (data[k][j]), i);
-					else if (s->info[s->out_order[i]].format[0] == '-')
-						GMT_ascii_output_one (GMT_stdout, data[k][j], i);
+					if (s->info[s->out_order[i]].format[0] == '-')
+						GMT_ascii_output_one (GMT_stdout, data[k][j], k);
 					else
 						fprintf (GMT_stdout, s->info[s->out_order[i]].format, data[k][j]);
 					(k == (s->n_out_columns - 1)) ? fprintf (GMT_stdout, "\n") : fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
