@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_put.c,v 1.20 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: x2sys_put.c,v 1.29 2008/05/12 22:35:47 guru Exp $
  *
- *      Copyright (c) 1999-2007 by P. Wessel
+ *      Copyright (c) 1999-2008 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *      Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
-/* x2sys_put will read one track bin file (.tbf) (from x2sys_putlist)
+/* x2sys_put will read one track bin file (.tbf) (from x2sys_binlist)
  * and update the track index database. The tbf file has the index
  * numbers of all the x by x degree boxes traversed by each data set.
  * The bin dimension and extent are determined by the tbf header.
@@ -45,7 +45,7 @@ int main (int argc, char **argv)
 	char track_file[BUFSIZ], index_file[BUFSIZ], old_track_file[BUFSIZ], old_index_file[BUFSIZ];
 	char track_path[BUFSIZ], index_path[BUFSIZ], old_track_path[BUFSIZ], old_index_path[BUFSIZ];
 
-	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, found_it;
+	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, found_it, swap = FALSE;
 
 	FILE *fp = NULL, *fbin, *ftrack;
 
@@ -73,6 +73,9 @@ int main (int argc, char **argv)
 				case 'T':
 					TAG = &argv[i][2];
 					break;
+				case 'S':
+					swap = TRUE;	/* Swap option for index.b reading */
+					break;
 				default:
 					error = TRUE;
 					fprintf (stderr, "%s: Unrecognized option -%c\n", GMT_program, argv[i][1]);
@@ -87,7 +90,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || error || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "x2sys_put %s - Update track index database from track bin file\n\n", X2SYS_VERSION);
-		fprintf (stderr, "usage: x2sys_put [<info.tbf>]  -T<TAG> [-V]\n\n");
+		fprintf (stderr, "usage: x2sys_put [<info.tbf>]  -T<TAG> [-S] [-V]\n\n");
 		fprintf (stderr, "	<info.tbf> is one track bin file from x2sys_binlist [Default reads stdin]\n");
 		fprintf (stderr, "	-T <TAG> is the system tag for this compilation\n");
 
@@ -95,6 +98,7 @@ int main (int argc, char **argv)
 
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr,"	-D will remove the listed tracks  [Default will add to database]\n");
+		fprintf (stderr, "	-S Byte swap binary records during read [no swapping]\n");
 		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
@@ -114,17 +118,6 @@ int main (int argc, char **argv)
 	for (i = max_flag = 0, bit = 1; i < s->n_fields; i++, bit <<= 1) max_flag |= bit;
 
 	x2sys_bix_init (&B, FALSE);
-	x2sys_free_info (s);
-
-	sprintf (track_file, "%s_tracks.d", TAG);
-	sprintf (index_file, "%s_index.b",  TAG);
-	x2sys_path (track_file, track_path);
-	x2sys_path (index_file, index_path);
-
-	if ((ftrack = GMT_fopen (track_path, "r")) == NULL) {
-		fprintf (stderr, "%s: Could not find %s\n", GMT_program, track_file);
-		exit (EXIT_FAILURE);
-	}
 
 	/* Read existing track-information from <ID>_tracks.d file */
 
@@ -133,10 +126,13 @@ int main (int argc, char **argv)
 
 	/* Read geographical track-info from <ID>_index.b file */
 
-	x2sys_err_fail (x2sys_bix_read_index (TAG, &B), "");
+	x2sys_err_fail (x2sys_bix_read_index (TAG, &B, swap), "");
 
 	/* Ok, now we can start reading new info */
 
+#ifdef DEBUG
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
 	GMT_fgets (line, BUFSIZ, fp);
 	while (line[0] == '>') {	/* Next segment */
 		sscanf (line, "> %s", track);
@@ -196,9 +192,17 @@ int main (int argc, char **argv)
 		}
 	}
 	if (fp != GMT_stdin) GMT_fclose (fp);
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
 
 	/* Done, now we must rewrite the <ID>_index.b and <ID>_tracks.d files */
 
+	sprintf (track_file, "%s_tracks.d", TAG);
+	sprintf (index_file, "%s_index.b",  TAG);
+	x2sys_path (track_file, track_path);
+	x2sys_path (index_file, index_path);
+
 	sprintf (old_track_file, "%s_old", track_file);
 	sprintf (old_index_file, "%s_old", index_file);
 	x2sys_path (old_track_file, old_track_path);
@@ -225,8 +229,8 @@ int main (int argc, char **argv)
 	for (this_info = B.head->next_info; this_info; this_info = this_info->next_info)
 		fprintf (ftrack,"%s %d %d\n",this_info->trackname, this_info->track_id, this_info->flag);
 
-	GMT_fclose (ftrack);
-	chmod (track_file, S_RDONLY);
+	fclose (ftrack);
+	chmod (track_file, (mode_t)S_RDONLY);
 
 	for (bin = 0; bin < B.nm_bin; bin++) {
 		if (B.base[bin].n_tracks == 0) continue;
@@ -238,12 +242,13 @@ int main (int argc, char **argv)
 			fwrite ((void *)(&this_track->track_flag), (size_t)4, (size_t)1, fbin);
 		}
 	}
-	GMT_fclose (fbin);
-	chmod (index_file, S_RDONLY);
-	GMT_free ((void *)B.base);
+	fclose (fbin);
+	chmod (index_file, (mode_t)S_RDONLY);
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s completed successfully\n", GMT_program);
 
+	x2sys_end (s);
+
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
@@ -259,7 +264,7 @@ int x2sys_bix_remove_track (int track_id, struct X2SYS_BIX *B)
 	for (bin = 0; bin < B->nm_bin; bin++) {
 		if (B->base[bin].n_tracks == 0) continue;	/* No tracks crossed this bin */
 
-		for (track = B->base[bin].first_track; track->next_track && track->next_track->track_id == track_id; track = track->next_track);	/* Finds the track or end-of-list */
+		for (track = B->base[bin].first_track; track->next_track && track->next_track->track_id != track_id; track = track->next_track);	/* Finds the track or end-of-list */
 
 		if (track->next_track) {	/* Ok, found it. Lets remove it from the list */
 			skip_track = track->next_track;
