@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: xyz2grd.c,v 1.67 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: xyz2grd.c,v 1.79 2008/05/15 22:00:36 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -77,10 +77,13 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, do_grid = FALSE, count = FALSE, average = TRUE;
 	BOOLEAN b_only = FALSE, done = FALSE, nofile = TRUE;
-
-	int i, gmt_ij, ii, jj, z = 2, nm = 0, n_read = 0, n_filled = 0, n_used = 0, fno, n_args, *flag = VNULL;
-	int ij = -1;	/* Will be incremented to 0 or set first time around */
-	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_expected_fields, n_fields, n_confused = 0, n_files = 0, high_low = 0;
+	
+	int i, ii, jj , z = 2, fno, n_args, *flag = VNULL;
+	int n_expected_fields, n_fields, n_files = 0, high_low = 0;
+	
+	GMT_LONG gmt_ij, nm = 0, n_read = 0, n_filled = 0, n_used = 0;
+	GMT_LONG ij = -1;	/* Will be incremented to 0 or set first time around */
+	GMT_LONG n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 
 	double *in, d_value, wesn[4];
 
@@ -187,7 +190,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "xyz2grd %s - Converting [xy]z data to a GMT grdfile\n\n", GMT_VERSION);
+		fprintf (stderr, "xyz2grd %s - Converting [xy]z data to a GMT grid file\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: xyz2grd [<[xy]zfile(s)>] -G<grdfile> %s %s\n", GMT_I_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-A[n|z|u|l]] [%s] [E[<nodata>]] [-F] [%s]\n", GMT_GRDEDIT, GMT_Ho_OPT);
 		fprintf (stderr, "\t[-N<nodata>] [-S[<zfile]] [-V] [-Z[<flags>]] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
@@ -195,7 +198,7 @@ int main (int argc, char **argv)
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyzfile(s) is a 1- or 3-column file [or standard input] with [x,y,]z values\n");
-		fprintf (stderr, "\t-G to name the output grdfile.\n");
+		fprintf (stderr, "\t-G to name the output grid file.\n");
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -213,7 +216,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-N set value for nodes without input xyz triplet [Default is NaN]\n");
 		fprintf (stderr, "\t   Z-table entries that equal <nodata> are replaced by NaN\n");
 		fprintf (stderr, "\t-S Requires -Z and will swap the byte-order of the input data and write\n");
-		fprintf (stderr, "\t   the result to <zfile> (or stdout if no file given).  No grdfile created!\n");
+		fprintf (stderr, "\t   the result to <zfile> (or stdout if no file given).  No grid file created!\n");
 		fprintf (stderr, "\t   For this option, only one input file (or stdin) is allowed.\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-Z sets exact specification of incoming 1-column z-table\n");
@@ -276,7 +279,7 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (!Ctrl->S.active && !Ctrl->G.file) {
+	if (!Ctrl->S.active && !(Ctrl->G.active || Ctrl->G.file)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
@@ -417,7 +420,7 @@ int main (int argc, char **argv)
 			while ((io.read_item) (fp, &d_value)) {
 				ij++;
 				if (ij == io.n_expected) {
-					fprintf (stderr, "%s: More than %d records, only %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+					fprintf (stderr, "%s: More than %ld records, only %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 					exit (EXIT_FAILURE);
 				}
 				(io.get_gmt_ij) (&io, ij, &gmt_ij);
@@ -428,16 +431,17 @@ int main (int argc, char **argv)
 			}
 			ij++;
 			if (ij != io.n_expected) {	/* Input amount doesn't match expectations */
-				fprintf (stderr, "%s: Found %d records, but %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+				fprintf (stderr, "%s: Found %ld records, but %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 				exit (EXIT_FAILURE);
 			}
 
 			GMT_check_z_io (&io, a);	/* This fills in missing periodic row or column */
 		}
 		else if (Ctrl->E.active) {	/* Read an ESRI Arc Interchange grid format in ASCII */
-			int n_left, j;
+			GMT_LONG n_left, j;
 			float value;
 			
+			grd.node_offset = 0;
 			GMT_fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %d", &grd.nx) != 1) {
 				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
@@ -453,11 +457,13 @@ int main (int argc, char **argv)
 				fprintf (stderr, "%s: Error decoding xll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
+			if (!strncmp (line, "xllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
 			GMT_fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.y_min) != 1) {
 				fprintf (stderr, "%s: Error decoding yll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
+			if (!strncmp (line, "yllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
 			GMT_fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.x_inc) != 1) {
 				fprintf (stderr, "%s: Error decoding cellsize record\n", GMT_program);
@@ -471,8 +477,6 @@ int main (int argc, char **argv)
 				}
 			}
 			grd.y_inc = grd.x_inc;
-			grd.node_offset = 0;
-			if (!strcmp (line, "xllcorner")) grd.node_offset = 1;	/* Pixel grid */
 			grd.xy_off = 0.5 * grd.node_offset;
 			grd.x_max = grd.x_min + (grd.nx - 1 + grd.node_offset) * grd.x_inc;
 			grd.y_max = grd.y_min + (grd.ny - 1 + grd.node_offset) * grd.y_inc;
@@ -493,7 +497,7 @@ int main (int argc, char **argv)
 				n_left--;
 			}
 			if (n_left) {
-				fprintf (stderr, "%s: Expected %d points, found only %d\n", GMT_program, nm, nm - n_left);
+				fprintf (stderr, "%s: Expected %ld points, found only %ld\n", GMT_program, nm, nm - n_left);
 				exit (EXIT_FAILURE);
 			}
 		}
@@ -502,7 +506,7 @@ int main (int argc, char **argv)
 			while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
 				n_read++;
@@ -518,7 +522,7 @@ int main (int argc, char **argv)
 				jj = GMT_y_to_j (in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 				if (jj == -1) jj++, n_confused++;
 				if (jj == grd.ny) jj--, n_confused++;
-				ij = jj * grd.nx + ii;
+				ij = GMT_IJ (jj, ii, grd.nx);
 				if (high_low) {	/* Always come here if looking for extreme values */
 					if (flag[ij]) {	/* Already assigned the first value */
 						if ((high_low == -1 && (in[z] < (double)a[ij])) || (high_low == +1 && (in[z] > (double)a[ij]))) a[ij] = (float)in[z];
@@ -539,8 +543,8 @@ int main (int argc, char **argv)
 	}
 
 	if (!(Ctrl->Z.active || Ctrl->E.active)) {	/* xyz data could have resulted in duplicates */
-		if (GMT_io.in_col_type[0] == GMT_IS_LON && (fabs (fabs (grd.x_max - grd.x_min) - 360.0) < GMT_CONV_LIMIT) && !grd.node_offset) {	/* Make sure longitudes got replicated */
-			int j, ij_west, ij_east;
+		if (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (grd.x_max, grd.x_min) && !grd.node_offset) {	/* Make sure longitudes got replicated */
+			GMT_LONG j, ij_west, ij_east;
 
 			for (j = 0; j < grd.ny; j++) {	/* For each row, look at west and east bin */
 				ij_west = j * grd.nx;
@@ -589,22 +593,22 @@ int main (int argc, char **argv)
 				n_stuffed++;
 			}
 		}
-		GMT_free ((void *)flag);
 		
 		if (gmtdefs.verbose) {
 			sprintf (line, "%s\n", gmtdefs.d_format);
-			fprintf (stderr, "%s:  n_read: %d  n_used: %d  n_filled: %d  n_empty: %d set to ", GMT_program,
+			fprintf (stderr, "%s:  n_read: %ld  n_used: %ld  n_filled: %ld  n_empty: %ld set to ", GMT_program,
 				n_read, n_used, n_filled, n_empty);
 			(GMT_is_dnan (Ctrl->N.value)) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.value);
-			if (n_bad) fprintf (stderr, "%s: %d records unreadable\n", GMT_program, n_bad);
-			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %d nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
-			if (n_confused) fprintf (stderr, "%s: Warning - %d values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
+			if (n_bad) fprintf (stderr, "%s: %ld records unreadable\n", GMT_program, n_bad);
+			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %ld nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
+			if (n_confused) fprintf (stderr, "%s: Warning - %ld values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
 		}
 	}
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	GMT_free ((void *)a);
+	if (!Ctrl->E.active) GMT_free ((void *)flag);
 
 	Free_Xyz2grd_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -616,7 +620,7 @@ int main (int argc, char **argv)
 void *New_Xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct XYZ2GRD_CTRL *C;
 	
-	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, 1, sizeof (struct XYZ2GRD_CTRL), "New_Xyz2grd_Ctrl");
+	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct XYZ2GRD_CTRL), "New_Xyz2grd_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = GMT_d_NaN;
@@ -626,7 +630,8 @@ void *New_Xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Xyz2grd_Ctrl (struct XYZ2GRD_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
-	if (C->S.file) GMT_free ((void *)C->S.file);	
+	if (C->D.information) free ((void *)C->D.information);	
+	if (C->G.file) free ((void *)C->G.file);	
+	if (C->S.file) free ((void *)C->S.file);	
 	GMT_free ((void *)C);	
 }
